@@ -1,6 +1,6 @@
 /*
  **********************************************************************
- *   Copyright (C) 1997-2006, International Business Machines
+ *   Copyright (C) 1997-2003, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
 *
@@ -88,8 +88,11 @@ static void U_CALLCONV
 deleteLocale(void *obj) {
     delete (Locale *) obj;
 }
+U_CDECL_END
 
-static UBool U_CALLCONV locale_cleanup(void)
+
+UBool
+locale_cleanup(void)
 {
     U_NAMESPACE_USE
 
@@ -112,7 +115,6 @@ static UBool U_CALLCONV locale_cleanup(void)
 
     return TRUE;
 }
-U_CDECL_END
 
 U_NAMESPACE_BEGIN
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(Locale)
@@ -124,7 +126,6 @@ void locale_set_default_internal(const char *id)
 {
     U_NAMESPACE_USE
     UErrorCode   status = U_ZERO_ERROR;
-    UBool canonicalize = FALSE;
 
     // If given a NULL string for the locale id, grab the default
     //   name from the system.
@@ -134,7 +135,6 @@ void locale_set_default_internal(const char *id)
         umtx_lock(NULL);
         id = uprv_getDefaultLocaleID();
         umtx_unlock(NULL);
-        canonicalize = TRUE; // always canonicalize host ID
     }
 
     // put the locale id into a canonical form,
@@ -144,11 +144,7 @@ void locale_set_default_internal(const char *id)
     status = U_ZERO_ERROR;
     char localeNameBuf[512];
 
-    if (canonicalize) {
-        uloc_canonicalize(id, localeNameBuf, sizeof(localeNameBuf)-1, &status);
-    } else {
-        uloc_getName(id, localeNameBuf, sizeof(localeNameBuf)-1, &status);
-    }
+    uloc_getName(id, localeNameBuf, sizeof(localeNameBuf)-1, &status);
     localeNameBuf[sizeof(localeNameBuf)-1] = 0;  // Force null termination in event of
                                                  //   a long name filling the buffer.
                                                  //   (long names are truncated.)
@@ -160,7 +156,7 @@ void locale_set_default_internal(const char *id)
     umtx_unlock(NULL);
     if (hashTableNeedsInit) {
         status = U_ZERO_ERROR;
-        UHashtable *tHashTable = uhash_open(uhash_hashChars, uhash_compareChars, NULL, &status);
+        UHashtable *tHashTable = uhash_open(uhash_hashChars, uhash_compareChars, &status);
         if (U_FAILURE(status)) {
             return;
         }
@@ -168,7 +164,6 @@ void locale_set_default_internal(const char *id)
         umtx_lock(NULL);
         if (gDefaultLocalesHashT == NULL) {
             gDefaultLocalesHashT = tHashTable;
-            ucln_common_registerCleanup(UCLN_COMMON_LOCALE, locale_cleanup);
             umtx_unlock(NULL);
         } else {
             umtx_unlock(NULL);
@@ -193,7 +188,7 @@ void locale_set_default_internal(const char *id)
             // No way to report errors from here.
             return;
         }
-        newDefault->init(localeNameBuf, FALSE);
+        newDefault->init(localeNameBuf);
 
         // Add newly created Locale to the hash table of default Locales
         const char *key = newDefault->getName();
@@ -261,7 +256,7 @@ Locale::~Locale()
 Locale::Locale()
     : UObject(), fullName(fullNameBuffer), baseName(NULL)
 {
-    init(NULL, FALSE);
+    init(NULL);
 }
 
 /*
@@ -284,7 +279,7 @@ Locale::Locale( const   char * newLanguage,
 {
     if( (newLanguage==NULL) && (newCountry == NULL) && (newVariant == NULL) )
     {
-        init(NULL, FALSE); /* shortcut */
+        init(NULL); /* shortcut */
     }
     else
     {
@@ -346,7 +341,7 @@ Locale::Locale( const   char * newLanguage,
             size += 1;  // at least: _v
         }
 
-        if ( newKeywords != NULL)
+        if ( newKeywords != NULL) 
         {
             ksize = (int32_t)uprv_strlen(newKeywords);
             size += ksize + 1;
@@ -357,7 +352,7 @@ Locale::Locale( const   char * newLanguage,
 
         /*if the whole string is longer than our internal limit, we need
         to go to the heap for temporary buffers*/
-        if (size >= ULOC_FULLNAME_CAPACITY)
+        if (size > ULOC_FULLNAME_CAPACITY)
         {
             togo_heap = (char *)uprv_malloc(sizeof(char)*(size+1));
             togo = togo_heap;
@@ -399,22 +394,14 @@ Locale::Locale( const   char * newLanguage,
 
         if ( ksize != 0)
         {
-            if (uprv_strchr(newKeywords, '=')) {
-                *p++ = '@'; /* keyword parsing */
-            }
-            else {
-                *p++ = '_'; /* Variant parsing with a script */
-                if ( vsize == 0) {
-                    *p++ = '_'; /* No country found */
-                }
-            }
+            *p++ = '@';
             uprv_strcpy(p, newKeywords);
             p += ksize;
         }
 
         // Parse it, because for example 'language' might really be a complete
         // string.
-        init(togo, FALSE);
+        init(togo);
 
         if (togo_heap) {
             uprv_free(togo_heap);
@@ -445,6 +432,11 @@ Locale &Locale::operator=(const Locale &other)
         fullName = fullNameBuffer;
     }
 
+    if(baseName && baseName != baseNameBuffer) {
+        uprv_free(baseName);
+        baseName = NULL;
+    }
+
     /* Allocate the full name if necessary */
     if(other.fullName != other.fullNameBuffer) {
         fullName = (char *)uprv_malloc(sizeof(char)*(uprv_strlen(other.fullName)+1));
@@ -452,19 +444,13 @@ Locale &Locale::operator=(const Locale &other)
     /* Copy the full name */
     uprv_strcpy(fullName, other.fullName);
 
-    /* baseName is the cached result of getBaseName.  if 'other' has a
-       baseName and it fits in baseNameBuffer, then copy it. otherwise set
-       it to NULL, and let the user lazy-create it (in getBaseName) if they
-       want it. */
-    if(baseName && baseName != baseNameBuffer) {
-        uprv_free(baseName);
+    if(other.baseName) {
+        if(other.baseName != other.baseNameBuffer) {
+            baseName = (char *)uprv_malloc(sizeof(char)*(uprv_strlen(other.fullName)+1));
+        }
+        uprv_strcpy(baseName, other.baseName);
     }
-    baseName = NULL;
 
-    if(other.baseName == other.baseNameBuffer) {
-        uprv_strcpy(baseNameBuffer, other.baseNameBuffer);
-        baseName = baseNameBuffer;
-    }
 
     /* Copy the language and country fields */
     uprv_strcpy(language, other.language);
@@ -489,7 +475,7 @@ Locale::operator==( const   Locale& other) const
 }
 
 /*This function initializes a Locale from a C locale ID*/
-Locale& Locale::init(const char* localeID, UBool canonicalize)
+Locale& Locale::init(const char* localeID)
 {
     fIsBogus = FALSE;
     /* Free our current storage */
@@ -525,10 +511,7 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
 
         // "canonicalize" the locale ID to ICU/Java format
         err = U_ZERO_ERROR;
-        length = canonicalize ?
-            uloc_canonicalize(localeID, fullName, sizeof(fullNameBuffer), &err) :
-            uloc_getName(localeID, fullName, sizeof(fullNameBuffer), &err);
-
+        length = uloc_getName(localeID, fullName, sizeof(fullNameBuffer), &err);
         if(err == U_BUFFER_OVERFLOW_ERROR || length >= (int32_t)sizeof(fullNameBuffer)) {
             /*Go to heap for the fullName if necessary*/
             fullName = (char *)uprv_malloc(sizeof(char)*(length + 1));
@@ -537,9 +520,7 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
                 break; // error: out of memory
             }
             err = U_ZERO_ERROR;
-            length = canonicalize ?
-                uloc_canonicalize(localeID, fullName, length+1, &err) :
-                uloc_getName(localeID, fullName, length+1, &err);
+            length = uloc_getName(localeID, fullName, length + 1, &err);
         }
         if(U_FAILURE(err) || err == U_STRING_NOT_TERMINATED_WARNING) {
             /* should never occur */
@@ -548,22 +529,16 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
 
         variantBegin = length;
 
-        /* after uloc_getName/canonicalize() we know that only '_' are separators */
+        /* after uloc_getName() we know that only '_' are separators */
         separator = field[0] = fullName;
         fieldIdx = 1;
         while ((separator = uprv_strchr(field[fieldIdx-1], SEP_CHAR)) && fieldIdx < (int32_t)(sizeof(field)/sizeof(field[0]))-1) {
             field[fieldIdx] = separator + 1;
-            fieldLen[fieldIdx-1] = (int32_t)(separator - field[fieldIdx-1]);
+            fieldLen[fieldIdx-1] = separator - field[fieldIdx-1];
             fieldIdx++;
         }
-        // variant may contain @foo or .foo POSIX cruft; remove it
-        separator = uprv_strchr(field[fieldIdx-1], '@');
-        char* sep2 = uprv_strchr(field[fieldIdx-1], '.');
-        if (separator!=NULL || sep2!=NULL) {
-            if (separator==NULL || (sep2!=NULL && separator > sep2)) {
-                separator = sep2;
-            }
-            fieldLen[fieldIdx-1] = (int32_t)(separator - field[fieldIdx-1]);
+        if((separator = uprv_strchr(field[fieldIdx-1], '@'))) {
+            fieldLen[fieldIdx-1] = separator - field[fieldIdx-1];
         } else {
             fieldLen[fieldIdx-1] = length - (int32_t)(field[fieldIdx-1] - fullName);
         }
@@ -575,7 +550,7 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
             break; // error: one of the fields is too long
         }
 
-        variantField = 2; /* Usually the 2nd one, except when a script is used. */
+        variantField = 0;
         if (fieldLen[0] > 0) {
             /* We have a language */
             uprv_memcpy(language, fullName, fieldLen[0]);
@@ -596,6 +571,7 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
             /* We have a country and no script */
             uprv_memcpy(country, field[1], fieldLen[1]);
             country[fieldLen[1]] = 0;
+            variantField = 2;
         }
         if (variantField > 0 && fieldLen[variantField] > 0) {
             /* We have a variant */
@@ -604,7 +580,7 @@ Locale& Locale::init(const char* localeID, UBool canonicalize)
 
         // successful end of init()
         return *this;
-    } while(0); /*loop doesn't iterate*/
+    } while(0);
 
     // when an error occurs, then set this object to "bogus" (there is no UErrorCode here)
     setToBogus();
@@ -634,29 +610,25 @@ Locale::setToBogus() {
     fIsBogus = TRUE;
 }
 
-const Locale& U_EXPORT2
+const Locale&
 Locale::getDefault()
 {
-    const Locale *retLocale;
     umtx_lock(NULL);
-    retLocale = gDefaultLocale;
+    UBool needInit = (gDefaultLocale == NULL);
     umtx_unlock(NULL);
-    if (retLocale == NULL) {
-        locale_set_default_internal(NULL);
+    if (needInit) {
         umtx_lock(NULL);
-        // Need a mutex  in case some other thread set a new
-        // default inbetween when we set and when we get the new default.  For
-        // processors with weak memory coherency, we might not otherwise see all
-        // of the newly created new default locale.
-        retLocale = gDefaultLocale;
+        /* uprv_getDefaultLocaleID is not thread safe, so we surround it with a mutex */
+        const char *cLocale = uprv_getDefaultLocaleID();
         umtx_unlock(NULL);
+        locale_set_default_internal(cLocale);
     }
-    return *retLocale;
+    return *gDefaultLocale;
 }
 
 
 
-void U_EXPORT2
+void
 Locale::setDefault( const   Locale&     newLocale,
                             UErrorCode&  status)
 {
@@ -671,12 +643,12 @@ Locale::setDefault( const   Locale&     newLocale,
     locale_set_default_internal(localeID);
 }
 
-Locale U_EXPORT2
+Locale
 Locale::createFromName (const char *name)
 {
     if (name) {
         Locale l("");
-        l.init(name, FALSE);
+        l.init(name);
         return l;
     }
     else {
@@ -684,12 +656,6 @@ Locale::createFromName (const char *name)
     }
 }
 
-Locale U_EXPORT2
-Locale::createCanonical(const char* name) {
-    Locale loc("");
-    loc.init(name, TRUE);
-    return loc;
-}
 
 const char *
 Locale::getISO3Language() const
@@ -745,7 +711,7 @@ Locale::getDisplayLanguage(const Locale &displayLocale,
     length=uloc_getDisplayLanguage(fullName, displayLocale.fullName,
                                    buffer, result.getCapacity(),
                                    &errorCode);
-    result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+    result.releaseBuffer(length);
 
     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
         buffer=result.getBuffer(length);
@@ -757,7 +723,11 @@ Locale::getDisplayLanguage(const Locale &displayLocale,
         length=uloc_getDisplayLanguage(fullName, displayLocale.fullName,
                                        buffer, result.getCapacity(),
                                        &errorCode);
-        result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+        result.releaseBuffer(length);
+    }
+
+    if(U_FAILURE(errorCode)) {
+        result.truncate(0);
     }
 
     return result;
@@ -785,7 +755,7 @@ Locale::getDisplayScript(const Locale &displayLocale,
     length=uloc_getDisplayScript(fullName, displayLocale.fullName,
                                   buffer, result.getCapacity(),
                                   &errorCode);
-    result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+    result.releaseBuffer(length);
 
     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
         buffer=result.getBuffer(length);
@@ -797,7 +767,11 @@ Locale::getDisplayScript(const Locale &displayLocale,
         length=uloc_getDisplayScript(fullName, displayLocale.fullName,
                                       buffer, result.getCapacity(),
                                       &errorCode);
-        result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+        result.releaseBuffer(length);
+    }
+
+    if(U_FAILURE(errorCode)) {
+        result.truncate(0);
     }
 
     return result;
@@ -825,7 +799,7 @@ Locale::getDisplayCountry(const Locale &displayLocale,
     length=uloc_getDisplayCountry(fullName, displayLocale.fullName,
                                   buffer, result.getCapacity(),
                                   &errorCode);
-    result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+    result.releaseBuffer(length);
 
     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
         buffer=result.getBuffer(length);
@@ -837,7 +811,11 @@ Locale::getDisplayCountry(const Locale &displayLocale,
         length=uloc_getDisplayCountry(fullName, displayLocale.fullName,
                                       buffer, result.getCapacity(),
                                       &errorCode);
-        result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+        result.releaseBuffer(length);
+    }
+
+    if(U_FAILURE(errorCode)) {
+        result.truncate(0);
     }
 
     return result;
@@ -865,7 +843,7 @@ Locale::getDisplayVariant(const Locale &displayLocale,
     length=uloc_getDisplayVariant(fullName, displayLocale.fullName,
                                   buffer, result.getCapacity(),
                                   &errorCode);
-    result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+    result.releaseBuffer(length);
 
     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
         buffer=result.getBuffer(length);
@@ -877,7 +855,11 @@ Locale::getDisplayVariant(const Locale &displayLocale,
         length=uloc_getDisplayVariant(fullName, displayLocale.fullName,
                                       buffer, result.getCapacity(),
                                       &errorCode);
-        result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+        result.releaseBuffer(length);
+    }
+
+    if(U_FAILURE(errorCode)) {
+        result.truncate(0);
     }
 
     return result;
@@ -905,7 +887,7 @@ Locale::getDisplayName(const Locale &displayLocale,
     length=uloc_getDisplayName(fullName, displayLocale.fullName,
                                buffer, result.getCapacity(),
                                &errorCode);
-    result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+    result.releaseBuffer(length);
 
     if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
         buffer=result.getBuffer(length);
@@ -917,12 +899,16 @@ Locale::getDisplayName(const Locale &displayLocale,
         length=uloc_getDisplayName(fullName, displayLocale.fullName,
                                    buffer, result.getCapacity(),
                                    &errorCode);
-        result.releaseBuffer(U_SUCCESS(errorCode) ? length : 0);
+        result.releaseBuffer(length);
+    }
+
+    if(U_FAILURE(errorCode)) {
+        result.truncate(0);
     }
 
     return result;
 }
-const Locale* U_EXPORT2
+const Locale*
 Locale::getAvailableLocales(int32_t& count)
 {
     // for now, there is a hardcoded list, so just walk through that list and set it up.
@@ -937,7 +923,6 @@ Locale::getAvailableLocales(int32_t& count)
            newLocaleList = new Locale[locCount];
         }
         if (newLocaleList == NULL) {
-            count = 0;
             return NULL;
         }
 
@@ -952,7 +937,6 @@ Locale::getAvailableLocales(int32_t& count)
             availableLocaleListCount = count;
             availableLocaleList = newLocaleList;
             newLocaleList = NULL;
-            ucln_common_registerCleanup(UCLN_COMMON_LOCALE, locale_cleanup);
         }
         umtx_unlock(NULL);
         delete []newLocaleList;
@@ -961,12 +945,12 @@ Locale::getAvailableLocales(int32_t& count)
     return availableLocaleList;
 }
 
-const char* const* U_EXPORT2 Locale::getISOCountries()
+const char* const* Locale::getISOCountries()
 {
     return uloc_getISOCountries();
 }
 
-const char* const* U_EXPORT2 Locale::getISOLanguages()
+const char* const* Locale::getISOLanguages()
 {
     return uloc_getISOLanguages();
 }
@@ -974,131 +958,131 @@ const char* const* U_EXPORT2 Locale::getISOLanguages()
 // Set the locale's data based on a posix id.
 void Locale::setFromPOSIXID(const char *posixID)
 {
-    init(posixID, TRUE);
+    init(posixID);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getEnglish(void)
 {
     return getLocale(eENGLISH);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getFrench(void)
 {
     return getLocale(eFRENCH);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getGerman(void)
 {
     return getLocale(eGERMAN);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getItalian(void)
 {
     return getLocale(eITALIAN);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getJapanese(void)
 {
     return getLocale(eJAPANESE);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getKorean(void)
 {
     return getLocale(eKOREAN);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getChinese(void)
 {
     return getLocale(eCHINESE);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getSimplifiedChinese(void)
 {
     return getLocale(eCHINA);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getTraditionalChinese(void)
 {
     return getLocale(eTAIWAN);
 }
 
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getFrance(void)
 {
     return getLocale(eFRANCE);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getGermany(void)
 {
     return getLocale(eGERMANY);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getItaly(void)
 {
     return getLocale(eITALY);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getJapan(void)
 {
     return getLocale(eJAPAN);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getKorea(void)
 {
     return getLocale(eKOREA);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getChina(void)
 {
     return getLocale(eCHINA);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getPRC(void)
 {
     return getLocale(eCHINA);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getTaiwan(void)
 {
     return getLocale(eTAIWAN);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getUK(void)
 {
     return getLocale(eUK);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getUS(void)
 {
     return getLocale(eUS);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getCanada(void)
 {
     return getLocale(eCANADA);
 }
 
-const Locale & U_EXPORT2
+const Locale &
 Locale::getCanadaFrench(void)
 {
     return getLocale(eCANADA_FRENCH);
@@ -1108,13 +1092,13 @@ const Locale &
 Locale::getLocale(int locid)
 {
     Locale *localeCache = getLocaleCache();
-    U_ASSERT((locid < eMAX_LOCALES)&&(locid>=0));
+    U_ASSERT(locid < eMAX_LOCALES);
     if (localeCache == NULL) {
         // Failure allocating the locale cache.
         //   The best we can do is return a NULL reference.
         locid = 0;
     }
-    return localeCache[locid]; /*operating on NULL*/
+    return localeCache[locid];
 }
 
 /*
@@ -1129,7 +1113,7 @@ Locale::getLocaleCache(void)
     umtx_unlock(NULL);
 
     if (needInit) {
-        Locale *tLocaleCache = new Locale[(int)eMAX_LOCALES];
+        Locale *tLocaleCache = new Locale[eMAX_LOCALES];
         if (tLocaleCache == NULL) {
             return NULL;
         }
@@ -1156,7 +1140,6 @@ Locale::getLocaleCache(void)
         if (gLocaleCache == NULL) {
             gLocaleCache = tLocaleCache;
             tLocaleCache = NULL;
-            ucln_common_registerCleanup(UCLN_COMMON_LOCALE, locale_cleanup);
         }
         umtx_unlock(NULL);
         if (tLocaleCache) {
@@ -1175,7 +1158,7 @@ private:
     static const char fgClassID;/* Warning this is used beyond the typical RTTI usage. */
 
 public:
-    static UClassID U_EXPORT2 getStaticClassID(void) { return (UClassID)&fgClassID; }
+    static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
     virtual UClassID getDynamicClassID(void) const { return getStaticClassID(); }
 public:
     KeywordEnumeration(const char *keys, int32_t keywordLen, int32_t currentIndex, UErrorCode &status)
@@ -1198,7 +1181,7 @@ public:
         }
     }
 
-    virtual ~KeywordEnumeration() {
+    ~KeywordEnumeration() {
         uprv_free(keywords);
     }
 
@@ -1208,7 +1191,7 @@ public:
         return new KeywordEnumeration(keywords, length, (int32_t)(current - keywords), status);
     }
 
-    virtual int32_t count(UErrorCode &/*status*/) const {
+    int32_t count(UErrorCode &/*status*/) const {
         char *kw = keywords;
         int32_t result = 0;
         while(*kw) {
@@ -1218,12 +1201,12 @@ public:
         return result;
     }
 
-    virtual const char* next(int32_t* resultLength, UErrorCode& status) {
+    const char* next(int32_t* resultLength, UErrorCode& status) {
         const char* result;
         int32_t len;
         if(U_SUCCESS(status) && *current != 0) {
             result = current;
-            len = (int32_t)uprv_strlen(current);
+            len = uprv_strlen(current);
             current += len+1;
             if(resultLength != NULL) {
                 *resultLength = len;
@@ -1237,13 +1220,13 @@ public:
         return result;
     }
 
-    virtual const UnicodeString* snext(UErrorCode& status) {
+    const UnicodeString* snext(UErrorCode& status) {
         int32_t resultLength = 0;
         const char *s = next(&resultLength, status);
         return setChars(s, resultLength, status);
     }
 
-    virtual void reset(UErrorCode& /*status*/) {
+    void reset(UErrorCode& /*status*/) {
         current = keywords;
     }
 };
@@ -1260,7 +1243,7 @@ Locale::createKeywords(UErrorCode &status) const
     const char* variantStart = uprv_strchr(fullName, '@');
     const char* assignment = uprv_strchr(fullName, '=');
     if(variantStart) {
-        if(assignment > variantStart) {
+        if(assignment) {
             int32_t keyLen = locale_getKeywords(variantStart+1, '@', keywords, keywordCapacity, NULL, 0, NULL, FALSE, &status);
             if(keyLen) {
                 result = new KeywordEnumeration(keywords, keyLen, 0, status);
