@@ -3,7 +3,7 @@
 //
 /*
 ***************************************************************************
-*   Copyright (C) 2002-2005 International Business Machines Corporation   *
+*   Copyright (C) 2002-2003 International Business Machines Corporation   *
 *   and others. All rights reserved.                                      *
 ***************************************************************************
 */
@@ -94,7 +94,6 @@ RBBISetBuilder::RBBISetBuilder(RBBIRuleBuilder *rb)
     fTrie           = 0;
     fTrieSize       = 0;
     fGroupCount     = 0;
-    fSawBOF         = FALSE;
 }
 
 
@@ -148,7 +147,7 @@ void RBBISetBuilder::build() {
     //  Find the set of non-overlapping ranges of characters
     //
     int  ni;
-    for (ni=0; ; ni++) {        // Loop over each of the UnicodeSets encountered in the input rules
+    for (ni=0; ; ni++) {
         usetNode = (RBBINode *)this->fRB->fUSetNodes->elementAt(ni);
         if (usetNode==NULL) {
             break;
@@ -223,11 +222,6 @@ void RBBISetBuilder::build() {
     //    The groups are numbered, and these group numbers are the set of
     //    input symbols recognized by the run-time state machine.
     //
-    //    Numbering: # 0  (state table column 0) is unused.
-    //               # 1  is reserved - table column 1 is for end-of-input
-    //               # 2  is reserved - table column 2 is for beginning-in-input
-    //               # 3  is the first range list.
-    //
     RangeDescriptor *rlSearchRange;
     for (rlRange = fRangeList; rlRange!=0; rlRange=rlRange->fNext) {
         for (rlSearchRange=fRangeList; rlSearchRange != rlRange; rlSearchRange=rlSearchRange->fNext) {
@@ -238,42 +232,12 @@ void RBBISetBuilder::build() {
         }
         if (rlRange->fNum == 0) {
             fGroupCount ++;
-            rlRange->fNum = fGroupCount+2; 
+            rlRange->fNum = fGroupCount;
             rlRange->setDictionaryFlag();
-            addValToSets(rlRange->fIncludesSets, fGroupCount+2);
+            addValToSets(rlRange->fIncludesSets, fGroupCount);
         }
     }
 
-    // Handle input sets that contain the special string {eof}.
-    //   Column 1 of the state table is reserved for EOF on input.
-    //   Column 2 is reserved for before-the-start-input.
-    //            (This column can be optimized away later if there are no rule
-    //             references to {bof}.)
-    //   Add this column value (1 or 2) to the equivalent expression
-    //     subtree for each UnicodeSet that contains the string {eof}
-    //   Because {bof} and {eof} are not a characters in the normal sense,
-    //   they doesn't affect the computation of ranges or TRIE.
-    static const UChar eofUString[] = {0x65, 0x6f, 0x66, 0};
-    static const UChar bofUString[] = {0x62, 0x6f, 0x66, 0};
-
-    UnicodeString eofString(eofUString);
-    UnicodeString bofString(bofUString);
-    for (ni=0; ; ni++) {        // Loop over each of the UnicodeSets encountered in the input rules
-        usetNode = (RBBINode *)this->fRB->fUSetNodes->elementAt(ni);
-        if (usetNode==NULL) {
-            break;
-        }
-        UnicodeSet      *inputSet = usetNode->fInputSet;
-        if (inputSet->contains(eofString)) {
-            addValToSet(usetNode, 1);
-        }
-        if (inputSet->contains(bofString)) {
-            addValToSet(usetNode, 2);
-            fSawBOF = TRUE;
-        }
-    }
-
-
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "rgroup")) {printRangeGroups();}
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "esets")) {printSets();}
 
@@ -301,7 +265,7 @@ void RBBISetBuilder::build() {
 //  getTrieSize()    Return the size that will be required to serialize the Trie.
 //
 //-----------------------------------------------------------------------------------
-int32_t RBBISetBuilder::getTrieSize() /*const*/ {
+int32_t RBBISetBuilder::getTrieSize() {
     fTrieSize  = utrie_serialize(fTrie,
                                     NULL,                // Buffer
                                     0,                   // Capacity
@@ -332,7 +296,7 @@ void RBBISetBuilder::serializeTrie(uint8_t *where) {
 //------------------------------------------------------------------------
 //
 //  addValToSets     Add a runtime-mapped input value to each uset from a
-//                   list of uset nodes. (val corresponds to a state table column.)
+//                   list of uset nodes.
 //                   For each of the original Unicode sets - which correspond
 //                   directly to uset nodes - a logically equivalent expression
 //                   is constructed in terms of the remapped runtime input
@@ -348,58 +312,46 @@ void  RBBISetBuilder::addValToSets(UVector *sets, uint32_t val) {
 
     for (ix=0; ix<sets->size(); ix++) {
         RBBINode *usetNode = (RBBINode *)sets->elementAt(ix);
-        addValToSet(usetNode, val);
-    }
-}
-
-void  RBBISetBuilder::addValToSet(RBBINode *usetNode, uint32_t val) {
-    RBBINode *leafNode = new RBBINode(RBBINode::leafChar);
-    leafNode->fVal = (unsigned short)val;
-    if (usetNode->fLeftChild == NULL) {
-        usetNode->fLeftChild = leafNode;
-        leafNode->fParent    = usetNode;
-    } else {
-        // There are already input symbols present for this set.
-        // Set up an OR node, with the previous stuff as the left child
-        //   and the new value as the right child.
-        RBBINode *orNode = new RBBINode(RBBINode::opOr);
-        orNode->fLeftChild  = usetNode->fLeftChild;
-        orNode->fRightChild = leafNode;
-        orNode->fLeftChild->fParent  = orNode;
-        orNode->fRightChild->fParent = orNode;
-        usetNode->fLeftChild = orNode;
-        orNode->fParent = usetNode;
+        RBBINode *leafNode = new RBBINode(RBBINode::leafChar);
+        leafNode->fVal = (unsigned short)val;
+        if (usetNode->fLeftChild == NULL) {
+            usetNode->fLeftChild = leafNode;
+            leafNode->fParent    = usetNode;
+        } else {
+            // There are already input symbols present for this set.
+            // Set up an OR node, with the previous stuff as the left child
+            //   and the new value as the right child.
+            RBBINode *orNode = new RBBINode(RBBINode::opOr);
+            orNode->fLeftChild  = usetNode->fLeftChild;
+            orNode->fRightChild = leafNode;
+            orNode->fLeftChild->fParent  = orNode;
+            orNode->fRightChild->fParent = orNode;
+            usetNode->fLeftChild = orNode;
+            orNode->fParent = usetNode;
+        }
     }
 }
 
 
-//------------------------------------------------------------------------
-//
-//   getNumCharCategories
-//
-//------------------------------------------------------------------------
-int32_t  RBBISetBuilder::getNumCharCategories() const {
-    return fGroupCount + 3;
-}
-
 
 //------------------------------------------------------------------------
 //
-//   sawBOF
+//   getNumOutputSets
 //
 //------------------------------------------------------------------------
-UBool  RBBISetBuilder::sawBOF() const {
-    return fSawBOF;
+int32_t  RBBISetBuilder::getNumCharCategories() {
+    return fGroupCount + 1;
 }
 
 
+
 //------------------------------------------------------------------------
 //
 //   getFirstChar      Given a runtime RBBI character category, find
 //                     the first UChar32 that is in the set of chars 
 //                     in the category.
 //------------------------------------------------------------------------
-UChar32  RBBISetBuilder::getFirstChar(int32_t category) const {
+UChar32  RBBISetBuilder::getFirstChar(int32_t category) {
     RangeDescriptor   *rlRange;
     UChar32            retVal = (UChar32)-1;
     for (rlRange = fRangeList; rlRange!=0; rlRange=rlRange->fNext) {
@@ -430,7 +382,7 @@ void RBBISetBuilder::printRanges() {
 
         for (i=0; i<rlRange->fIncludesSets->size(); i++) {
             RBBINode       *usetNode    = (RBBINode *)rlRange->fIncludesSets->elementAt(i);
-            UnicodeString   setName = UNICODE_STRING("anon", 4);
+            UnicodeString   setName = "anon";
             RBBINode       *setRef = usetNode->fParent;
             if (setRef != NULL) {
                 RBBINode *varRef = setRef->fParent;
@@ -470,7 +422,7 @@ void RBBISetBuilder::printRangeGroups() {
 
             for (i=0; i<rlRange->fIncludesSets->size(); i++) {
                 RBBINode       *usetNode    = (RBBINode *)rlRange->fIncludesSets->elementAt(i);
-                UnicodeString   setName = UNICODE_STRING("anon", 4);
+                UnicodeString   setName = "anon";
                 RBBINode       *setRef = usetNode->fParent;
                 if (setRef != NULL) {
                     RBBINode *varRef = setRef->fParent;
@@ -521,7 +473,7 @@ void RBBISetBuilder::printSets() {
         }
 
         RBBIDebugPrintf("%3d    ", i);
-        setName = UNICODE_STRING("anonymous", 9);
+        setName = "anonymous";
         setRef = usetNode->fParent;
         if (setRef != NULL) {
             varRef = setRef->fParent;
@@ -670,7 +622,7 @@ void RangeDescriptor::setDictionaryFlag() {
                 setName = varRef->fText;
             }
         }
-        if (setName.compare(UNICODE_STRING("dictionary", 10)) == 0) {   // TODO:  no string literals.
+        if (setName.compare("dictionary") == 0) {   // TODO:  no string literals.
             this->fNum |= 0x4000;
             break;
         }
