@@ -1,13 +1,13 @@
-/*
-**********************************************************************
-*   Copyright (c) 2002-2006, International Business Machines
-*   Corporation and others.  All Rights Reserved.
-**********************************************************************
-*/
 //
 //  rbbitblb.cpp
 //
 
+/*
+**********************************************************************
+*   Copyright (c) 2002-2003, International Business Machines
+*   Corporation and others.  All Rights Reserved.
+**********************************************************************
+*/
 
 #include "unicode/utypes.h"
 
@@ -75,28 +75,10 @@ void  RBBITableBuilder::build() {
     //   parse tree for the substition expression.
     //
     fTree = fTree->flattenVariables();
-#ifdef RBBI_DEBUG
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "ftree")) {
-        RBBIDebugPuts("Parse tree after flattening variable references.");
+        RBBIDebugPrintf("Parse tree after flattening variable references.\n");
         fTree->printTree(TRUE);
     }
-#endif
-
-    //
-    // If the rules contained any references to {bof} 
-    //   add a {bof} <cat> <former root of tree> to the
-    //   tree.  Means that all matches must start out with the 
-    //   {bof} fake character.
-    // 
-    if (fRB->fSetBuilder->sawBOF()) {
-        RBBINode *bofTop    = new RBBINode(RBBINode::opCat);
-        RBBINode *bofLeaf   = new RBBINode(RBBINode::leafChar);
-        bofTop->fLeftChild  = bofLeaf;
-        bofTop->fRightChild = fTree;
-        bofLeaf->fParent    = bofTop;
-        bofLeaf->fVal       = 2;      // Reserved value for {bof}.
-        fTree               = bofTop;
-    }
 
     //
     // Add a unique right-end marker to the expression.
@@ -115,12 +97,10 @@ void  RBBITableBuilder::build() {
     //      expression.
     //
     fTree->flattenSets();
-#ifdef RBBI_DEBUG
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "stree")) {
-        RBBIDebugPuts("Parse tree after flattening Unicode Set references.");
+        RBBIDebugPrintf("Parse tree after flattening Unicode Set references.\n");
         fTree->printTree(TRUE);
     }
-#endif
 
 
     //
@@ -135,7 +115,7 @@ void  RBBITableBuilder::build() {
     calcLastPos(fTree);
     calcFollowPos(fTree);
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "pos")) {
-        RBBIDebugPuts("\n");
+        RBBIDebugPrintf("\n\n");
         printPosSets(fTree);
     }
 
@@ -146,13 +126,6 @@ void  RBBITableBuilder::build() {
         calcChainedFollowPos(fTree);
     }
 
-    //
-    //  BOF (start of input) test fixup.
-    //
-    if (fRB->fSetBuilder->sawBOF()) {
-        bofFixup();
-    }
-
     //
     // Build the DFA state transition tables.
     //
@@ -160,15 +133,8 @@ void  RBBITableBuilder::build() {
     flagAcceptingStates();
     flagLookAheadStates();
     flagTaggedStates();
-
-    //
-    // Update the global table of rule status {tag} values
-    // The rule builder has a global vector of status values that are common
-    //    for all tables.  Merge the ones from this table into the global set.
-    //
-    mergeRuleStatusVals();
-
     if (fRB->fDebugEnv && uprv_strstr(fRB->fDebugEnv, "states")) {printStates();};
+
 }
 
 
@@ -370,21 +336,14 @@ void RBBITableBuilder::calcChainedFollowPos(RBBINode *tree) {
     // get a list of all endmarker nodes.
     tree->findNodes(&endMarkerNodes, RBBINode::endMark, *fStatus);
 
-    // get a list all leaf nodes
+    // get a list all leaf nodes 
     tree->findNodes(&leafNodes, RBBINode::leafChar, *fStatus);
     if (U_FAILURE(*fStatus)) {
         return;
     }
 
-    // Get all nodes that can be the start a match, which is FirstPosition()
-    // of the portion of the tree corresponding to user-written rules.
-    // See the tree description in bofFixup().
-    RBBINode *userRuleRoot = tree;
-    if (fRB->fSetBuilder->sawBOF()) {
-        userRuleRoot = tree->fLeftChild->fRightChild;
-    }
-    U_ASSERT(userRuleRoot != NULL);
-    UVector *matchStartNodes = userRuleRoot->fFirstPosSet;
+    // Get all nodes that can be the start a match, which is FirstPosition(root)
+    UVector *matchStartNodes = tree->fFirstPosSet;
 
 
     // Iteratate over all leaf nodes,
@@ -417,12 +376,10 @@ void RBBITableBuilder::calcChainedFollowPos(RBBINode *tree) {
         //        into the rule file.
         if (fRB->fLBCMNoChain) {
             UChar32 c = this->fRB->fSetBuilder->getFirstChar(endNode->fVal);
-            if (c != -1) {
-                // c == -1 occurs with sets containing only the {eof} marker string.
-                ULineBreak cLBProp = (ULineBreak)u_getIntPropertyValue(c, UCHAR_LINE_BREAK);
-                if (cLBProp == U_LB_COMBINING_MARK) {
-                    continue;
-                }
+            U_ASSERT(c != -1);
+            ULineBreak cLBProp = (ULineBreak)u_getIntPropertyValue(c, UCHAR_LINE_BREAK);
+            if (cLBProp == U_LB_COMBINING_MARK) {
+                continue;
             }
         }
 
@@ -451,62 +408,6 @@ void RBBITableBuilder::calcChainedFollowPos(RBBINode *tree) {
 }
 
 
-//-----------------------------------------------------------------------------
-//
-//   bofFixup.    Fixup for state tables that include {bof} beginning of input testing.
-//                Do an swizzle similar to chaining, modifying the followPos set of
-//                the bofNode to include the followPos nodes from other {bot} nodes
-//                scattered through the tree.
-//
-//                This function has much in common with calcChainedFollowPos().
-//
-//-----------------------------------------------------------------------------
-void RBBITableBuilder::bofFixup() {
-
-    if (U_FAILURE(*fStatus)) {
-        return;
-    }
-
-    //   The parse tree looks like this ...
-    //         fTree root  --->       <cat>
-    //                               /     \       .
-    //                            <cat>   <#end node>
-    //                           /     \  .
-    //                     <bofNode>   rest
-    //                               of tree
-    //
-    //    We will be adding things to the followPos set of the <bofNode>
-    //
-    RBBINode  *bofNode = fTree->fLeftChild->fLeftChild;
-    U_ASSERT(bofNode->fType == RBBINode::leafChar);
-    U_ASSERT(bofNode->fVal == 2);
-
-    // Get all nodes that can be the start a match of the user-written rules
-    //  (excluding the fake bofNode)
-    //  We want the nodes that can start a match in the
-    //     part labeled "rest of tree"
-    // 
-    UVector *matchStartNodes = fTree->fLeftChild->fRightChild->fFirstPosSet;
-
-    RBBINode *startNode;
-    int       startNodeIx;
-    for (startNodeIx = 0; startNodeIx<matchStartNodes->size(); startNodeIx++) {
-        startNode = (RBBINode *)matchStartNodes->elementAt(startNodeIx);
-        if (startNode->fType != RBBINode::leafChar) {
-            continue;
-        }
-
-        if (startNode->fVal == bofNode->fVal) {
-            //  We found a leaf node corresponding to a {bof} that was
-            //    explicitly written into a rule.
-            //  Add everything from the followPos set of this node to the
-            //    followPos set of the fake bofNode at the start of the tree.
-            //  
-            setAdd(bofNode->fFollowPos, startNode->fFollowPos);
-        }
-    }
-}
-
 //-----------------------------------------------------------------------------
 //
 //   buildStateTable()    Determine the set of runtime DFA states and the
@@ -664,29 +565,14 @@ void     RBBITableBuilder::flagAcceptingStates() {
                 // Any non-zero value for fAccepting means this is an accepting node.
                 // The value is what will be returned to the user as the break status.
                 // If no other value was specified, force it to -1.
-
-                if (sd->fAccepting==0) {
-                    // State hasn't been marked as accepting yet.  Do it now.
-                    sd->fAccepting = endMarker->fVal;
-                    if (sd->fAccepting == 0) {
-                        sd->fAccepting = -1;
-                    }
+                sd->fAccepting = endMarker->fVal;
+                if (sd->fAccepting == 0) {
+                    sd->fAccepting = -1;
                 }
-                if (sd->fAccepting==-1 && endMarker->fVal != 0) {
-                    // Both lookahead and non-lookahead accepting for this state.
-                    // Favor the look-ahead.  Expedient for line break.
-                    // TODO:  need a more elegant resolution for conflicting rules.
-                    sd->fAccepting = endMarker->fVal;
-                }
-                // implicit else:
-                // if sd->fAccepting already had a value other than 0 or -1, leave it be.
 
                 // If the end marker node is from a look-ahead rule, set
                 //   the fLookAhead field or this state also.
                 if (endMarker->fLookAheadEnd) {
-                    // TODO:  don't change value if already set?
-                    // TODO:  allow for more than one active look-ahead rule in engine.
-                    //        Make value here an index to a side array in engine?
                     sd->fLookAhead = sd->fAccepting;
                 }
             }
@@ -755,138 +641,14 @@ void     RBBITableBuilder::flagTaggedStates() {
         for (n=0; n<fDStates->size(); n++) {              //    For each state  s (row in the state table)
             RBBIStateDescriptor *sd = (RBBIStateDescriptor *)fDStates->elementAt(n);
             if (sd->fPositions->indexOf(tagNode) >= 0) {  //       if  s include the tag node t
-                sortedAdd(&sd->fTagVals, tagNode->fVal);
+                if (sd->fTagVal < tagNode->fVal) {
+                    // If more than one rule tag applies to this state, the larger
+                    //   tag takes precedence.
+                sd->fTagVal = tagNode->fVal;
             }
         }
     }
 }
-
-
-
-
-//-----------------------------------------------------------------------------
-//
-//  mergeRuleStatusVals
-//
-//      Update the global table of rule status {tag} values
-//      The rule builder has a global vector of status values that are common
-//      for all tables.  Merge the ones from this table into the global set.
-//
-//-----------------------------------------------------------------------------
-void  RBBITableBuilder::mergeRuleStatusVals() {
-    //
-    //  The basic outline of what happens here is this...
-    //
-    //    for each state in this state table
-    //       if the status tag list for this state is in the global statuses list
-    //           record where and
-    //           continue with the next state
-    //       else
-    //           add the tag list for this state to the global list.
-    //
-    int i;
-    int n;
-
-    // Pre-set a single tag of {0} into the table.
-    //   We will need this as a default, for rule sets with no explicit tagging.
-    if (fRB->fRuleStatusVals->size() == 0) {
-        fRB->fRuleStatusVals->addElement(1, *fStatus);  // Num of statuses in group
-        fRB->fRuleStatusVals->addElement((int32_t)0, *fStatus);  //   and our single status of zero
-    }
-
-    //    For each state
-    for (n=0; n<fDStates->size(); n++) {
-        RBBIStateDescriptor *sd = (RBBIStateDescriptor *)fDStates->elementAt(n);
-        UVector *thisStatesTagValues = sd->fTagVals;
-        if (thisStatesTagValues == NULL) {
-            // No tag values are explicitly associated with this state.
-            //   Set the default tag value.
-            sd->fTagsIdx = 0;
-            continue;
-        }
-
-        // There are tag(s) associated with this state.
-        //   fTagsIdx will be the index into the global tag list for this state's tag values.
-        //   Initial value of -1 flags that we haven't got it set yet.
-        sd->fTagsIdx = -1;
-        int32_t  thisTagGroupStart = 0;   // indexes into the global rule status vals list
-        int32_t  nextTagGroupStart = 0;
-
-        // Loop runs once per group of tags in the global list
-        while (nextTagGroupStart < fRB->fRuleStatusVals->size()) {
-            thisTagGroupStart = nextTagGroupStart;
-            nextTagGroupStart += fRB->fRuleStatusVals->elementAti(thisTagGroupStart) + 1;
-            if (thisStatesTagValues->size() != fRB->fRuleStatusVals->elementAti(thisTagGroupStart)) {
-                // The number of tags for this state is different from
-                //    the number of tags in this group from the global list.
-                //    Continue with the next group from the global list.
-                continue;
-            }
-            // The lengths match, go ahead and compare the actual tag values
-            //    between this state and the group from the global list.
-            for (i=0; i<thisStatesTagValues->size(); i++) {
-                if (thisStatesTagValues->elementAti(i) !=
-                    fRB->fRuleStatusVals->elementAti(thisTagGroupStart + 1 + i) ) {
-                    // Mismatch.
-                    break;
-                }
-            }
-
-            if (i == thisStatesTagValues->size()) {
-                // We found a set of tag values in the global list that match
-                //   those for this state.  Use them.
-                sd->fTagsIdx = thisTagGroupStart;
-                break;
-            }
-        }
-
-        if (sd->fTagsIdx == -1) {
-            // No suitable entry in the global tag list already.  Add one
-            sd->fTagsIdx = fRB->fRuleStatusVals->size();
-            fRB->fRuleStatusVals->addElement(thisStatesTagValues->size(), *fStatus);
-            for (i=0; i<thisStatesTagValues->size(); i++) {
-                fRB->fRuleStatusVals->addElement(thisStatesTagValues->elementAti(i), *fStatus);
-            }
-        }
-    }
-}
-
-
-
-
-
-
-
-//-----------------------------------------------------------------------------
-//
-//  sortedAdd  Add a value to a vector of sorted values (ints).
-//             Do not replicate entries; if the value is already there, do not
-//                add a second one.
-//             Lazily create the vector if it does not already exist.
-//
-//-----------------------------------------------------------------------------
-void RBBITableBuilder::sortedAdd(UVector **vector, int32_t val) {
-    int32_t i;
-
-    if (*vector == NULL) {
-        *vector = new UVector(*fStatus);
-    }
-    if (*vector == NULL || U_FAILURE(*fStatus)) {
-        return;
-    }
-    UVector *vec = *vector;
-    int32_t  vSize = vec->size();
-    for (i=0; i<vSize; i++) {
-        int32_t valAtI = vec->elementAti(i);
-        if (valAtI == val) {
-            // The value is already in the vector.  Don't add it again.
-            return;
-        }
-        if (valAtI > val) {
-            break;
-        }
-    }
-    vec->insertElementAt(val, i, *fStatus);
 }
 
 
@@ -916,7 +678,6 @@ void RBBITableBuilder::setAdd(UVector *dest, UVector *source) {
 }
 
 
-
 //-----------------------------------------------------------------------------
 //
 //  setEqual    Set operation on UVector.
@@ -994,7 +755,7 @@ void RBBITableBuilder::printPosSets(RBBINode *n) {
 //                     state transition table.
 //
 //-----------------------------------------------------------------------------
-int32_t  RBBITableBuilder::getTableSize() const {
+int32_t  RBBITableBuilder::getTableSize() {
     int32_t    size = 0;
     int32_t    numRows;
     int32_t    numCols;
@@ -1048,9 +809,6 @@ void RBBITableBuilder::exportTable(void *where) {
     if (fRB->fLookAheadHardBreak) {
         table->fFlags  |= RBBI_LOOKAHEAD_HARD_BREAK;
     }
-    if (fRB->fSetBuilder->sawBOF()) {
-        table->fFlags  |= RBBI_BOF_REQUIRED;
-    }
     table->fReserved  = 0;
 
     for (state=0; state<table->fNumStates; state++) {
@@ -1060,7 +818,7 @@ void RBBITableBuilder::exportTable(void *where) {
         U_ASSERT (-32768 < sd->fLookAhead && sd->fLookAhead <= 32767);
         row->fAccepting = (int16_t)sd->fAccepting;
         row->fLookAhead = (int16_t)sd->fLookAhead;
-        row->fTagIdx    = (int16_t)sd->fTagsIdx;
+        row->fTag       = (int16_t)sd->fTagVal;
         for (col=0; col<fRB->fSetBuilder->getNumCharCategories(); col++) {
             row->fNextState[col] = (uint16_t)sd->fDtran->elementAti(col);
         }
@@ -1098,20 +856,16 @@ void RBBITableBuilder::printStates() {
 
     RBBIDebugPrintf("state |           i n p u t     s y m b o l s \n");
     RBBIDebugPrintf("      | Acc  LA    Tag");
-    for (c=0; c<fRB->fSetBuilder->getNumCharCategories(); c++) {
-        RBBIDebugPrintf(" %2d", c);
-    }
+    for (c=0; c<fRB->fSetBuilder->getNumCharCategories(); c++) {RBBIDebugPrintf(" %2d", c);};
     RBBIDebugPrintf("\n");
     RBBIDebugPrintf("      |---------------");
-    for (c=0; c<fRB->fSetBuilder->getNumCharCategories(); c++) {
-        RBBIDebugPrintf("---");
-    }
+    for (c=0; c<fRB->fSetBuilder->getNumCharCategories(); c++) {RBBIDebugPrintf("---");};
     RBBIDebugPrintf("\n");
 
     for (n=0; n<fDStates->size(); n++) {
         RBBIStateDescriptor *sd = (RBBIStateDescriptor *)fDStates->elementAt(n);
         RBBIDebugPrintf("  %3d | " , n);
-        RBBIDebugPrintf("%3d %3d %5d ", sd->fAccepting, sd->fLookAhead, sd->fTagsIdx);
+        RBBIDebugPrintf("%3d %3d %5d ", sd->fAccepting, sd->fLookAhead, sd->fTagVal);
         for (c=0; c<fRB->fSetBuilder->getNumCharCategories(); c++) {
             RBBIDebugPrintf(" %2d", sd->fDtran->elementAti(c));
         }
@@ -1123,33 +877,6 @@ void RBBITableBuilder::printStates() {
 
 
 
-//-----------------------------------------------------------------------------
-//
-//   printRuleStatusTable    Debug Function.  Dump the common rule status table
-//
-//-----------------------------------------------------------------------------
-#ifdef RBBI_DEBUG
-void RBBITableBuilder::printRuleStatusTable() {
-    int32_t  thisRecord = 0;
-    int32_t  nextRecord = 0;
-    int      i;
-    UVector  *tbl = fRB->fRuleStatusVals;
-
-    RBBIDebugPrintf("index |  tags \n");
-    RBBIDebugPrintf("-------------------\n");
-
-    while (nextRecord < tbl->size()) {
-        thisRecord = nextRecord;
-        nextRecord = thisRecord + tbl->elementAti(thisRecord) + 1;
-        RBBIDebugPrintf("%4d   ", thisRecord);
-        for (i=thisRecord+1; i<nextRecord; i++) {
-            RBBIDebugPrintf("  %5d", tbl->elementAti(i));
-        }
-        RBBIDebugPrintf("\n");
-    }
-    RBBIDebugPrintf("\n\n");
-}
-#endif
 
 
 //-----------------------------------------------------------------------------
@@ -1163,15 +890,19 @@ RBBIStateDescriptor::RBBIStateDescriptor(int lastInputSymbol, UErrorCode *fStatu
     fMarked    = FALSE;
     fAccepting = 0;
     fLookAhead = 0;
-    fTagsIdx   = 0;
-    fTagVals   = NULL;
+    fTagVal    = 0;
     fPositions = NULL;
     fDtran     = NULL;
-
-    fDtran     = new UVector(lastInputSymbol+1, *fStatus);
+    
+    UErrorCode status = U_ZERO_ERROR;
+    fDtran     = new UVector(lastInputSymbol+1, status);
     if (U_FAILURE(*fStatus)) {
         return;
     }
+    if (U_FAILURE(status)) {
+        *fStatus = status;
+        return;
+    }
     if (fDtran == NULL) {
         *fStatus = U_MEMORY_ALLOCATION_ERROR;
         return;
@@ -1186,10 +917,8 @@ RBBIStateDescriptor::RBBIStateDescriptor(int lastInputSymbol, UErrorCode *fStatu
 RBBIStateDescriptor::~RBBIStateDescriptor() {
     delete       fPositions;
     delete       fDtran;
-    delete       fTagVals;
     fPositions = NULL;
     fDtran     = NULL;
-    fTagVals   = NULL;
 }
 
 U_NAMESPACE_END
