@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *                                                                            *
-* Copyright (C) 2001-2004, International Business Machines                   *
+* Copyright (C) 2001-2003, International Business Machines                   *
 *                Corporation and others. All Rights Reserved.                *
 *                                                                            *
 ******************************************************************************
@@ -50,16 +50,14 @@
 typedef enum ECleanupLibraryType {
     UCLN_START = -1,
     UCLN_CUSTOM,    /* Custom is for anyone else. */
-    UCLN_LAYOUTEX,
     UCLN_LAYOUT,
-    UCLN_IO,
+    UCLN_LAYOUTEX,
+    UCLN_USTDIO,
     UCLN_I18N,
     UCLN_COMMON /* This must be the last one to cleanup. */
 } ECleanupLibraryType;
 
-U_CDECL_BEGIN
-typedef UBool U_CALLCONV cleanupFunc(void);
-U_CDECL_END
+typedef UBool cleanupFunc(void);
 
 U_CAPI void U_EXPORT2 ucln_registerCleanup(ECleanupLibraryType type,
                                            cleanupFunc *func);
