@@ -1,7 +1,7 @@
 /*
  ********************************************************************
- * COPYRIGHT:
- * Copyright (c) 1996-2008, International Business Machines Corporation and
+ * COPYRIGHT: 
+ * Copyright (c) 1996-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************
  *
@@ -12,20 +12,17 @@
  *  uses uconv_io.h routines to access disk information
  *  is used by ucnv.h to implement public API create/delete/flushCache routines
  * Modification History:
- *
+ * 
  *   Date        Name        Description
- *
+ * 
  *   06/20/2000  helena      OS/400 port changes; mostly typecast.
  *   06/29/2000  helena      Major rewrite of the callback interface.
 */
 
-#include "unicode/utypes.h"
 
-#if !UCONFIG_NO_CONVERSION
-
-#include "unicode/putil.h"
 #include "unicode/udata.h"
 #include "unicode/ucnv.h"
+#include "unicode/ucnv_err.h"
 #include "unicode/uloc.h"
 #include "utracimp.h"
 #include "ucnv_io.h"
@@ -39,7 +36,7 @@
 #include "cstring.h"
 #include "cmemory.h"
 #include "ucln_cmn.h"
-#include "ustr_cnv.h"
+#include "ustr_imp.h"
 
 
 
@@ -155,61 +152,30 @@ static UMTX        cnvCacheMutex = NULL;  /* Mutex for synchronizing cnv cache a
                                           /*  Note:  the global mutex is used for      */
                                           /*         reference count updates.          */
 
-static const char **gAvailableConverters = NULL;
-static uint16_t gAvailableConverterCount = 0;
-
-/* This contains the resolved converter name. So no further alias lookup is needed again. */
-static char gDefaultConverterNameBuffer[UCNV_MAX_CONVERTER_NAME_LENGTH + 1]; /* +1 for NULL */
-static const char *gDefaultConverterName = NULL;
-
-/*
-If the default converter is an algorithmic converter, this is the cached value.
-We don't cache a full UConverter and clone it because ucnv_clone doesn't have
-less overhead than an algorithmic open. We don't cache non-algorithmic converters
-because ucnv_flushCache must be able to unload the default converter and its table.
-*/
-static const UConverterSharedData *gDefaultAlgorithmicSharedData = NULL;
-
-/* Does gDefaultConverterName have a converter option and require extra parsing? */
-static UBool gDefaultConverterContainsOption;
-
 
 static const char DATA_TYPE[] = "cnv";
 
-static void
-ucnv_flushAvailableConverterCache() {
-    if (gAvailableConverters) {
-        umtx_lock(&cnvCacheMutex);
-        gAvailableConverterCount = 0;
-        uprv_free((char **)gAvailableConverters);
-        gAvailableConverters = NULL;
-        umtx_unlock(&cnvCacheMutex);
+/* ucnv_cleanup - delete all storage held by the converter cache, except any in use    */
+/*                by open converters.                                                  */
+/*                Not thread safe.                                                     */
+/*                Not supported  API.  Marked U_CAPI only for use by test programs.    */
+U_CFUNC UBool U_EXPORT2 ucnv_cleanup(void) {
+    if (SHARED_DATA_HASHTABLE != NULL) {
+        ucnv_flushCache();
+        if (SHARED_DATA_HASHTABLE != NULL && uhash_count(SHARED_DATA_HASHTABLE) == 0) {
+            uhash_close(SHARED_DATA_HASHTABLE);
+            SHARED_DATA_HASHTABLE = NULL;
+        }
     }
+
+    umtx_destroy(&cnvCacheMutex);           /* Don't worry about destroying the mutex even  */
+                                            /*  if the hash table still exists.  The mutex  */
+                                            /*  will lazily re-init  itself if needed.      */
+    return (SHARED_DATA_HASHTABLE == NULL);
 }
 
-/* ucnv_cleanup - delete all storage held by the converter cache, except any  */
-/*                in use by open converters.                                  */
-/*                Not thread safe.                                            */
-/*                Not supported API.                                          */
-static UBool U_CALLCONV ucnv_cleanup(void) {
-    ucnv_flushCache();
-    if (SHARED_DATA_HASHTABLE != NULL && uhash_count(SHARED_DATA_HASHTABLE) == 0) {
-        uhash_close(SHARED_DATA_HASHTABLE);
-        SHARED_DATA_HASHTABLE = NULL;
-    }
-
-    /* Isn't called from flushCache because other threads may have preexisting references to the table. */
-    ucnv_flushAvailableConverterCache();
-
-    gDefaultConverterName = NULL;
-    gDefaultConverterNameBuffer[0] = 0;
-    gDefaultConverterContainsOption = FALSE;
-    gDefaultAlgorithmicSharedData = NULL;
-
-    umtx_destroy(&cnvCacheMutex);    /* Don't worry about destroying the mutex even  */
-                                     /*  if the hash table still exists.  The mutex  */
-                                     /*  will lazily re-init  itself if needed.      */
-    return (SHARED_DATA_HASHTABLE == NULL);
+U_CFUNC void ucnv_init(UErrorCode *status) {
+    umtx_init(&cnvCacheMutex);
 }
 
 static UBool U_CALLCONV
@@ -282,7 +248,7 @@ ucnv_data_unFlattenClone(UConverterLoadArgs *pArgs, UDataMemory *pData, UErrorCo
 #endif
 
     data->staticData = source;
-
+    
     data->sharedDataCached = FALSE;
 
     /* fill in fields from the loaded data */
@@ -310,7 +276,7 @@ static UConverterSharedData *createConverterFromFile(UConverterLoadArgs *pArgs,
 
     UTRACE_ENTRY_OC(UTRACE_UCNV_LOAD);
 
-    if (U_FAILURE (*err)) {
+    if (err == NULL || U_FAILURE (*err)) {
         UTRACE_EXIT_STATUS(*err);
         return NULL;
     }
@@ -343,13 +309,30 @@ static UConverterSharedData *createConverterFromFile(UConverterLoadArgs *pArgs,
     return sharedData;
 }
 
+int32_t 
+ucnv_copyPlatformString(char *platformString, UConverterPlatform pltfrm)
+{
+    switch (pltfrm)
+    {
+    case UCNV_IBM:
+        uprv_strcpy(platformString, "ibm-");
+        return 4;
+    case UCNV_UNKNOWN:
+        break;
+    }
+
+    /* default to empty string */
+    *platformString = 0;
+    return 0;
+}
+
 /*returns a converter type from a string
  */
 static const UConverterSharedData *
 getAlgorithmicTypeFromName(const char *realName)
 {
     uint32_t mid, start, limit;
-    uint32_t lastMid;
+	uint32_t lastMid;
     int result;
     char strippedName[UCNV_MAX_CONVERTER_NAME_LENGTH];
 
@@ -360,14 +343,14 @@ getAlgorithmicTypeFromName(const char *realName)
     start = 0;
     limit = sizeof(cnvNameType)/sizeof(cnvNameType[0]);
     mid = limit;
-    lastMid = UINT32_MAX;
+	lastMid = UINT32_MAX;
 
     for (;;) {
         mid = (uint32_t)((start + limit) / 2);
-        if (lastMid == mid) {   /* Have we moved? */
-            break;  /* We haven't moved, and it wasn't found. */
-        }
-        lastMid = mid;
+		if (lastMid == mid) {	/* Have we moved? */
+			break;	/* We haven't moved, and it wasn't found. */
+		}
+		lastMid = mid;
         result = uprv_strcmp(strippedName, cnvNameType[mid].name);
 
         if (result < 0) {
@@ -382,16 +365,6 @@ getAlgorithmicTypeFromName(const char *realName)
     return NULL;
 }
 
-/*
-* Based on the number of known converters, this determines how many times larger
-* the shared data hash table should be. When on small platforms, or just a couple
-* of converters are used, this number should be 2. When memory is plentiful, or
-* when ucnv_countAvailable is ever used with a lot of available converters,
-* this should be 4.
-* Larger numbers reduce the number of hash collisions, but use more memory.
-*/
-#define UCNV_CACHE_LOAD_FACTOR 2
-
 /* Puts the shared data in the static hashtable SHARED_DATA_HASHTABLE */
 /*   Will always be called with the cnvCacheMutex alrady being held   */
 /*     by the calling function.                                       */
@@ -407,12 +380,10 @@ ucnv_shareConverterData(UConverterSharedData * data)
 
     if (SHARED_DATA_HASHTABLE == NULL)
     {
-        SHARED_DATA_HASHTABLE = uhash_openSize(uhash_hashChars, uhash_compareChars, NULL,
-                            ucnv_io_countKnownConverters(&err)*UCNV_CACHE_LOAD_FACTOR,
+        SHARED_DATA_HASHTABLE = uhash_openSize(uhash_hashChars, uhash_compareChars,
+                            ucnv_io_countAvailableAliases(&err),
                             &err);
-        ucln_common_registerCleanup(UCLN_COMMON_UCNV, ucnv_cleanup);
-
-        if (U_FAILURE(err))
+        if (U_FAILURE(err)) 
             return;
     }
 
@@ -426,7 +397,7 @@ ucnv_shareConverterData(UConverterSharedData * data)
     }
     UCNV_DEBUG_LOG("put:chk",data->staticData->name,sanity);
     */
-
+    
     /* Mark it shared */
     data->sharedDataCached = TRUE;
 
@@ -488,7 +459,7 @@ ucnv_deleteSharedConverterData(UConverterSharedData * deadSharedData)
     if (deadSharedData->impl->unload != NULL) {
         deadSharedData->impl->unload(deadSharedData);
     }
-
+    
     if(deadSharedData->dataMemory != NULL)
     {
         UDataMemory *data = (UDataMemory*)deadSharedData->dataMemory;
@@ -573,7 +544,7 @@ ucnv_unload(UConverterSharedData *sharedData) {
         if (sharedData->referenceCounter > 0) {
             sharedData->referenceCounter--;
         }
-
+    
         if((sharedData->referenceCounter <= 0)&&(sharedData->sharedDataCached == FALSE)) {
             ucnv_deleteSharedConverterData(sharedData);
         }
@@ -691,8 +662,6 @@ ucnv_loadSharedData(const char *converterName, UConverterLookupData *lookup, UEr
     UConverterLookupData stackLookup;
     UConverterSharedData *mySharedConverterData = NULL;
     UErrorCode internalErrorCode = U_ZERO_ERROR;
-    UBool mayContainOption = TRUE;
-    UBool checkForAlgorithmic = TRUE;
 
     if (U_FAILURE (*err)) {
         return NULL;
@@ -707,29 +676,13 @@ ucnv_loadSharedData(const char *converterName, UConverterLookupData *lookup, UEr
 
     /* In case "name" is NULL we want to open the default converter. */
     if (converterName == NULL) {
-        /* Call ucnv_getDefaultName first to query the name from the OS. */
-        lookup->realName = ucnv_getDefaultName();
+        lookup->realName = ucnv_io_getDefaultConverterName();
         if (lookup->realName == NULL) {
             *err = U_MISSING_RESOURCE_ERROR;
             return NULL;
         }
-        mySharedConverterData = (UConverterSharedData *)gDefaultAlgorithmicSharedData;
-        checkForAlgorithmic = FALSE;
-        mayContainOption = gDefaultConverterContainsOption;
         /* the default converter name is already canonical */
-    }
-    else if((converterName[0] == 'U' ?
-            (                           converterName[1] == 'T' && converterName[2] == 'F') :
-            (converterName[0] == 'u' && converterName[1] == 't' && converterName[2] == 'f'))
-        &&
-        (converterName[3] == '-' ?
-            (converterName[4] == '8' && converterName[5] == 0) :
-            (converterName[3] == '8' && converterName[4] == 0)))
-    {
-        /* fastpath for UTF-8 */
-        return (UConverterSharedData *)converterData[UCNV_UTF8];
-    }
-    else {
+    } else {
         /* separate the converter name from the options */
         parseConverterOptions(converterName, lookup->cnvName, lookup->locale, &lookup->options, err);
         if (U_FAILURE(*err)) {
@@ -738,7 +691,7 @@ ucnv_loadSharedData(const char *converterName, UConverterLookupData *lookup, UEr
         }
 
         /* get the canonical converter name */
-        lookup->realName = ucnv_io_getConverterName(lookup->cnvName, &mayContainOption, &internalErrorCode);
+        lookup->realName = ucnv_io_getConverterName(lookup->cnvName, &internalErrorCode);
         if (U_FAILURE(internalErrorCode) || lookup->realName == NULL) {
             /*
             * set the input name in case the converter was added
@@ -749,15 +702,13 @@ ucnv_loadSharedData(const char *converterName, UConverterLookupData *lookup, UEr
     }
 
     /* separate the converter name from the options */
-    if(mayContainOption && lookup->realName != lookup->cnvName) {
+    if(lookup->realName != lookup->cnvName) {
         parseConverterOptions(lookup->realName, lookup->cnvName, lookup->locale, &lookup->options, err);
         lookup->realName = lookup->cnvName;
     }
-
+    
     /* get the shared data for an algorithmic converter, if it is one */
-    if (checkForAlgorithmic) {
-        mySharedConverterData = (UConverterSharedData *)getAlgorithmicTypeFromName(lookup->realName);
-    }
+    mySharedConverterData = (UConverterSharedData *)getAlgorithmicTypeFromName(lookup->realName);
     if (mySharedConverterData == NULL)
     {
         /* it is a data-based converter, get its shared data.               */
@@ -807,14 +758,9 @@ ucnv_createConverter(UConverter *myUConverter, const char *converterName, UError
             if(U_SUCCESS(*err)) {
                 UTRACE_EXIT_PTR_STATUS(myUConverter, *err);
                 return myUConverter;
-            }
-            /*
-            else mySharedConverterData was already cleaned up by
-            ucnv_createConverterFromSharedData.
-            */
-            /*else {
+            } else {
                 ucnv_unloadSharedDataIfReady(mySharedConverterData);
-            }*/
+            }
         }
     }
 
@@ -872,7 +818,7 @@ ucnv_createConverterFromPackage(const char *packageName, const char *converterNa
 
     if(U_FAILURE(*err)) {
         UTRACE_EXIT_STATUS(*err);
-        return NULL;
+        return NULL; 
     }
 
     UTRACE_DATA2(UTRACE_OPEN_CLOSE, "open converter %s from package %s", converterName, packageName);
@@ -889,24 +835,24 @@ ucnv_createConverterFromPackage(const char *packageName, const char *converterNa
         return NULL;
     }
     args.name=cnvName;
-
+    
     /* open the data, unflatten the shared structure */
     mySharedConverterData = createConverterFromFile(&args, err);
-
+    
     if (U_FAILURE(*err)) {
         UTRACE_EXIT_STATUS(*err);
-        return NULL;
+        return NULL; 
     }
 
     /* create the actual converter */
     myUConverter = ucnv_createConverterFromSharedData(NULL, mySharedConverterData, cnvName, locale, args.options, err);
-
+    
     if (U_FAILURE(*err)) {
         ucnv_close(myUConverter);
         UTRACE_EXIT_STATUS(*err);
-        return NULL;
+        return NULL; 
     }
-
+    
     UTRACE_EXIT_PTR_STATUS(myUConverter, *err);
     return myUConverter;
 }
@@ -936,22 +882,20 @@ ucnv_createConverterFromSharedData(UConverter *myUConverter,
     /* initialize the converter */
     uprv_memset(myUConverter, 0, sizeof(UConverter));
     myUConverter->isCopyLocal = isCopyLocal;
-    /*myUConverter->isExtraLocal = FALSE;*/ /* Set by the memset call */
+    myUConverter->isExtraLocal = FALSE;
     myUConverter->sharedData = mySharedConverterData;
     myUConverter->options = options;
+    myUConverter->fromCharErrorBehaviour = (UConverterToUCallback) UCNV_TO_U_CALLBACK_SUBSTITUTE;
+    myUConverter->fromUCharErrorBehaviour = (UConverterFromUCallback) UCNV_FROM_U_CALLBACK_SUBSTITUTE;
+    myUConverter->toUnicodeStatus = myUConverter->sharedData->toUnicodeStatus;
+    myUConverter->maxBytesPerUChar = myUConverter->sharedData->staticData->maxBytesPerChar;
+    myUConverter->subChar1 = myUConverter->sharedData->staticData->subChar1;
+    myUConverter->subCharLen = myUConverter->sharedData->staticData->subCharLen;
+    uprv_memcpy (myUConverter->subChar, myUConverter->sharedData->staticData->subChar, myUConverter->subCharLen);
     myUConverter->preFromUFirstCP = U_SENTINEL;
-    myUConverter->fromCharErrorBehaviour = UCNV_TO_U_DEFAULT_CALLBACK;
-    myUConverter->fromUCharErrorBehaviour = UCNV_FROM_U_DEFAULT_CALLBACK;
-    myUConverter->toUnicodeStatus = mySharedConverterData->toUnicodeStatus;
-    myUConverter->maxBytesPerUChar = mySharedConverterData->staticData->maxBytesPerChar;
-    myUConverter->subChar1 = mySharedConverterData->staticData->subChar1;
-    myUConverter->subCharLen = mySharedConverterData->staticData->subCharLen;
-    myUConverter->subChars = (uint8_t *)myUConverter->subUChars;
-    uprv_memcpy(myUConverter->subChars, mySharedConverterData->staticData->subChar, myUConverter->subCharLen);
-    myUConverter->toUCallbackReason = UCNV_ILLEGAL; /* default reason to invoke (*fromCharErrorBehaviour) */
 
-    if(mySharedConverterData->impl->open != NULL) {
-        mySharedConverterData->impl->open(myUConverter, realName, locale, options, err);
+    if(myUConverter != NULL && myUConverter->sharedData->impl->open != NULL) {
+        myUConverter->sharedData->impl->open(myUConverter, realName, locale,options, err);
         if(U_FAILURE(*err)) {
             ucnv_close(myUConverter);
             return NULL;
@@ -1015,9 +959,9 @@ ucnv_flushCache ()
             if (mySharedData->referenceCounter == 0)
             {
                 tableDeletedNum++;
-
+            
                 UCNV_DEBUG_LOG("del",mySharedData->staticData->name,mySharedData);
-
+            
                 uhash_removeElement(SHARED_DATA_HASHTABLE, e);
                 mySharedData->sharedDataCached = FALSE;
                 ucnv_deleteSharedConverterData (mySharedData);
@@ -1030,216 +974,12 @@ ucnv_flushCache ()
 
     UTRACE_DATA1(UTRACE_INFO, "ucnv_flushCache() exits with %d converters remaining", remaining);
 
+    ucnv_io_flushAvailableConverterCache();
+
     UTRACE_EXIT_VALUE(tableDeletedNum);
     return tableDeletedNum;
 }
 
-/* available converters list --------------------------------------------------- */
-
-static UBool haveAvailableConverterList(UErrorCode *pErrorCode) {
-    int needInit;
-    UMTX_CHECK(&cnvCacheMutex, (gAvailableConverters == NULL), needInit);
-    if (needInit) {
-        UConverter tempConverter;
-        UEnumeration *allConvEnum = NULL;
-        uint16_t idx;
-        uint16_t localConverterCount;
-        uint16_t allConverterCount;
-        UErrorCode localStatus;
-        const char *converterName;
-        const char **localConverterList;
-
-        allConvEnum = ucnv_openAllNames(pErrorCode);
-        allConverterCount = uenum_count(allConvEnum, pErrorCode);
-        if (U_FAILURE(*pErrorCode)) {
-            return FALSE;
-        }
-
-        /* We can't have more than "*converterTable" converters to open */
-        localConverterList = (const char **) uprv_malloc(allConverterCount * sizeof(char*));
-        if (!localConverterList) {
-            *pErrorCode = U_MEMORY_ALLOCATION_ERROR;
-            return FALSE;
-        }
-
-        /* Open the default converter to make sure that it has first dibs in the hash table. */
-        localStatus = U_ZERO_ERROR;
-        ucnv_close(ucnv_createConverter(&tempConverter, NULL, &localStatus));
-
-        localConverterCount = 0;
-
-        for (idx = 0; idx < allConverterCount; idx++) {
-            localStatus = U_ZERO_ERROR;
-            converterName = uenum_next(allConvEnum, NULL, &localStatus);
-            ucnv_close(ucnv_createConverter(&tempConverter, converterName, &localStatus));
-            if (U_SUCCESS(localStatus)) {
-                localConverterList[localConverterCount++] = converterName;
-            }
-        }
-        uenum_close(allConvEnum);
-
-        umtx_lock(&cnvCacheMutex);
-        if (gAvailableConverters == NULL) {
-            gAvailableConverters = localConverterList;
-            gAvailableConverterCount = localConverterCount;
-            ucln_common_registerCleanup(UCLN_COMMON_UCNV, ucnv_cleanup);
-        }
-        else {
-            uprv_free((char **)localConverterList);
-        }
-        umtx_unlock(&cnvCacheMutex);
-    }
-    return TRUE;
-}
-
-U_CFUNC uint16_t
-ucnv_bld_countAvailableConverters(UErrorCode *pErrorCode) {
-    if (haveAvailableConverterList(pErrorCode)) {
-        return gAvailableConverterCount;
-    }
-    return 0;
-}
-
-U_CFUNC const char *
-ucnv_bld_getAvailableConverter(uint16_t n, UErrorCode *pErrorCode) {
-    if (haveAvailableConverterList(pErrorCode)) {
-        if (n < gAvailableConverterCount) {
-            return gAvailableConverters[n];
-        }
-        *pErrorCode = U_INDEX_OUTOFBOUNDS_ERROR;
-    }
-    return NULL;
-}
-
-/* default converter name --------------------------------------------------- */
-
-/*
-Copy the canonical converter name.
-ucnv_getDefaultName must be thread safe, which can call this function.
-
-ucnv_setDefaultName calls this function and it doesn't have to be
-thread safe because there is no reliable/safe way to reset the
-converter in use in all threads. If you did reset the converter, you
-would not be sure that retrieving a default converter for one string
-would be the same type of default converter for a successive string.
-Since the name is a returned via ucnv_getDefaultName without copying,
-you shouldn't be modifying or deleting the string from a separate thread.
-*/
-static U_INLINE void
-internalSetName(const char *name, UErrorCode *status) {
-    UConverterLookupData lookup;
-    int32_t length=(int32_t)(uprv_strlen(name));
-    UBool containsOption = (UBool)(uprv_strchr(name, UCNV_OPTION_SEP_CHAR) != NULL);
-    const UConverterSharedData *algorithmicSharedData;
-
-    lookup.locale[0] = 0;
-    lookup.options = 0;
-    lookup.realName = name;
-    if(containsOption) {
-        parseConverterOptions(lookup.realName, lookup.cnvName, lookup.locale, &lookup.options, status);
-        lookup.realName = lookup.cnvName;
-    }
-    algorithmicSharedData = getAlgorithmicTypeFromName(lookup.realName);
-
-    umtx_lock(&cnvCacheMutex);
-
-    gDefaultAlgorithmicSharedData = algorithmicSharedData;
-    gDefaultConverterContainsOption = containsOption;
-    uprv_memcpy(gDefaultConverterNameBuffer, name, length);
-    gDefaultConverterNameBuffer[length]=0;
-    gDefaultConverterName = gDefaultConverterNameBuffer;
-
-    ucln_common_registerCleanup(UCLN_COMMON_UCNV, ucnv_cleanup);
-
-    umtx_unlock(&cnvCacheMutex);
-}
-
-/*
- * In order to be really thread-safe, the get function would have to take
- * a buffer parameter and copy the current string inside a mutex block.
- * This implementation only tries to be really thread-safe while
- * setting the name.
- * It assumes that setting a pointer is atomic.
- */
-
-U_CAPI const char*  U_EXPORT2
-ucnv_getDefaultName() {
-    /* local variable to be thread-safe */
-    const char *name;
-
-    /*
-    Multiple calls to ucnv_getDefaultName must be thread safe,
-    but ucnv_setDefaultName is not thread safe.
-    */
-    UMTX_CHECK(&cnvCacheMutex, gDefaultConverterName, name);
-    if(name==NULL) {
-        UErrorCode errorCode = U_ZERO_ERROR;
-        UConverter *cnv = NULL;
-
-        name = uprv_getDefaultCodepage();
-
-        /* if the name is there, test it out and get the canonical name with options */
-        if(name != NULL) {
-            cnv = ucnv_open(name, &errorCode);
-            if(U_SUCCESS(errorCode) && cnv != NULL) {
-                name = ucnv_getName(cnv, &errorCode);
-            }
-        }
-
-        if(name == NULL || name[0] == 0
-            || U_FAILURE(errorCode) || cnv == NULL
-            || uprv_strlen(name)>=sizeof(gDefaultConverterNameBuffer))
-        {
-            /* Panic time, let's use a fallback. */
-#if (U_CHARSET_FAMILY == U_ASCII_FAMILY)
-            name = "US-ASCII";
-            /* there is no 'algorithmic' converter for EBCDIC */
-#elif defined(OS390)
-            name = "ibm-1047_P100-1995" UCNV_SWAP_LFNL_OPTION_STRING;
-#else
-            name = "ibm-37_P100-1995";
-#endif
-        }
-
-        internalSetName(name, &errorCode);
-
-        /* The close may make the current name go away. */
-        ucnv_close(cnv);
-    }
-
-    return name;
-}
-
-/*
-This function is not thread safe, and it can't be thread safe.
-See internalSetName or the API reference for details.
-*/
-U_CAPI void U_EXPORT2
-ucnv_setDefaultName(const char *converterName) {
-    if(converterName==NULL) {
-        /* reset to the default codepage */
-        gDefaultConverterName=NULL;
-    } else {
-        UErrorCode errorCode = U_ZERO_ERROR;
-        UConverter *cnv = NULL;
-        const char *name = NULL;
-
-        /* if the name is there, test it out and get the canonical name with options */
-        cnv = ucnv_open(converterName, &errorCode);
-        if(U_SUCCESS(errorCode) && cnv != NULL) {
-            name = ucnv_getName(cnv, &errorCode);
-        }
-
-        if(U_SUCCESS(errorCode) && name!=NULL) {
-            internalSetName(name, &errorCode);
-        }
-        /* else this converter is bad to use. Don't change it to a bad value. */
-
-        /* The close may make the current name go away. */
-        ucnv_close(cnv);
-    }
-}
-
 /* data swapping ------------------------------------------------------------ */
 
 /* most of this might belong more properly into ucnvmbcs.c, but that is so large */
@@ -1265,13 +1005,8 @@ ucnv_swap(const UDataSwapper *ds,
     const _MBCSHeader *inMBCSHeader;
     _MBCSHeader *outMBCSHeader;
     _MBCSHeader mbcsHeader;
-    uint32_t mbcsHeaderLength;
-    UBool noFromU=FALSE;
-
     uint8_t outputType;
 
-    int32_t maxFastUChar, mbcsIndexLength;
-
     const int32_t *inExtIndexes;
     int32_t extOffset;
 
@@ -1331,10 +1066,11 @@ ucnv_swap(const UDataSwapper *ds,
         ds->swapArray32(ds, &inStaticData->codepage, 4,
                            &outStaticData->codepage, pErrorCode);
 
-        ds->swapInvChars(ds, inStaticData->name, (int32_t)uprv_strlen(inStaticData->name),
+        ds->swapInvChars(ds, inStaticData->name, uprv_strlen(inStaticData->name),
                             outStaticData->name, pErrorCode);
         if(U_FAILURE(*pErrorCode)) {
-            udata_printError(ds, "ucnv_swap(): error swapping converter name\n");
+            udata_printError(ds, "ucnv_swap(): error swapping converter name - %s\n",
+                             u_errorName(*pErrorCode));
             return 0;
         }
     }
@@ -1351,21 +1087,7 @@ ucnv_swap(const UDataSwapper *ds,
         inMBCSHeader=(const _MBCSHeader *)inBytes;
         outMBCSHeader=(_MBCSHeader *)outBytes;
 
-        if(0<=length && length<sizeof(_MBCSHeader)) {
-            udata_printError(ds, "ucnv_swap(): too few bytes (%d after headers) for an ICU MBCS .cnv conversion table\n",
-                                length);
-            *pErrorCode=U_INDEX_OUTOFBOUNDS_ERROR;
-            return 0;
-        }
-        if(inMBCSHeader->version[0]==4 && inMBCSHeader->version[1]>=1) {
-            mbcsHeaderLength=MBCS_HEADER_V4_LENGTH;
-        } else if(inMBCSHeader->version[0]==5 && inMBCSHeader->version[1]>=3 &&
-                  ((mbcsHeader.options=ds->readUInt32(inMBCSHeader->options))&
-                   MBCS_OPT_UNKNOWN_INCOMPATIBLE_MASK)==0
-        ) {
-            mbcsHeaderLength=mbcsHeader.options&MBCS_OPT_LENGTH_MASK;
-            noFromU=(UBool)((mbcsHeader.options&MBCS_OPT_NO_FROM_U)!=0);
-        } else {
+        if(!(inMBCSHeader->version[0]==4 || inMBCSHeader->version[1]>=1)) {
             udata_printError(ds, "ucnv_swap(): unsupported _MBCSHeader.version %d.%d\n",
                              inMBCSHeader->version[0], inMBCSHeader->version[1]);
             *pErrorCode=U_UNSUPPORTED_ERROR;
@@ -1380,15 +1102,9 @@ ucnv_swap(const UDataSwapper *ds,
         mbcsHeader.offsetFromUBytes=    ds->readUInt32(inMBCSHeader->offsetFromUBytes);
         mbcsHeader.flags=               ds->readUInt32(inMBCSHeader->flags);
         mbcsHeader.fromUBytesLength=    ds->readUInt32(inMBCSHeader->fromUBytesLength);
-        /* mbcsHeader.options have been read above */
 
-        extOffset=(int32_t)(mbcsHeader.flags>>8);
+        extOffset=(int32_t)mbcsHeader.flags>>8;
         outputType=(uint8_t)mbcsHeader.flags;
-        if(noFromU && outputType==MBCS_OUTPUT_1) {
-            udata_printError(ds, "ucnv_swap(): unsupported combination of makeconv --small with SBCS\n");
-            *pErrorCode=U_UNSUPPORTED_ERROR;
-            return 0;
-        }
 
         /* make sure that the output type is known */
         switch(outputType) {
@@ -1410,27 +1126,8 @@ ucnv_swap(const UDataSwapper *ds,
         }
 
         /* calculate the length of the MBCS data */
-
-        /*
-         * utf8Friendly MBCS files (mbcsHeader.version 4.3)
-         * contain an additional mbcsIndex table:
-         *   uint16_t[(maxFastUChar+1)>>6];
-         * where maxFastUChar=((mbcsHeader.version[2]<<8)|0xff).
-         */
-        maxFastUChar=0;
-        mbcsIndexLength=0;
-        if( outputType!=MBCS_OUTPUT_EXT_ONLY && outputType!=MBCS_OUTPUT_1 &&
-            mbcsHeader.version[1]>=3 && (maxFastUChar=mbcsHeader.version[2])!=0
-        ) {
-            maxFastUChar=(maxFastUChar<<8)|0xff;
-            mbcsIndexLength=((maxFastUChar+1)>>6)*2;  /* number of bytes */
-        }
-
         if(extOffset==0) {
-            size=(int32_t)(mbcsHeader.offsetFromUBytes+mbcsIndexLength);
-            if(!noFromU) {
-                size+=(int32_t)mbcsHeader.fromUBytesLength;
-            }
+            size=(int32_t)(mbcsHeader.offsetFromUBytes+mbcsHeader.fromUBytesLength);
 
             /* avoid compiler warnings - not otherwise necessary, and the value does not matter */
             inExtIndexes=NULL;
@@ -1460,9 +1157,8 @@ ucnv_swap(const UDataSwapper *ds,
                 uprv_memcpy(outBytes, inBytes, size);
             }
 
-            /* swap the MBCSHeader, except for the version field */
-            count=mbcsHeaderLength*4;
-            ds->swapArray32(ds, &inMBCSHeader->countStates, count-4,
+            /* swap the _MBCSHeader */
+            ds->swapArray32(ds, &inMBCSHeader->countStates, 7*4,
                                &outMBCSHeader->countStates, pErrorCode);
 
             if(outputType==MBCS_OUTPUT_EXT_ONLY) {
@@ -1472,23 +1168,18 @@ ucnv_swap(const UDataSwapper *ds,
                  */
 
                 /* swap the base name, between the header and the extension data */
-                const char *inBaseName=(const char *)inBytes+count;
-                char *outBaseName=(char *)outBytes+count;
-                ds->swapInvChars(ds, inBaseName, (int32_t)uprv_strlen(inBaseName),
-                                    outBaseName, pErrorCode);
+                ds->swapInvChars(ds, inMBCSHeader+1, uprv_strlen((const char *)(inMBCSHeader+1)),
+                                    outMBCSHeader+1, pErrorCode);
             } else {
                 /* normal file with base table data */
 
                 /* swap the state table, 1kB per state */
-                offset=count;
-                count=mbcsHeader.countStates*1024;
-                ds->swapArray32(ds, inBytes+offset, (int32_t)count,
-                                   outBytes+offset, pErrorCode);
+                ds->swapArray32(ds, inMBCSHeader+1, (int32_t)(mbcsHeader.countStates*1024),
+                                   outMBCSHeader+1, pErrorCode);
 
                 /* swap the toUFallbacks[] */
-                offset+=count;
-                count=mbcsHeader.countToUFallbacks*8;
-                ds->swapArray32(ds, inBytes+offset, (int32_t)count,
+                offset=sizeof(_MBCSHeader)+mbcsHeader.countStates*1024;
+                ds->swapArray32(ds, inBytes+offset, (int32_t)(mbcsHeader.countToUFallbacks*8),
                                    outBytes+offset, pErrorCode);
 
                 /* swap the unicodeCodeUnits[] */
@@ -1525,7 +1216,7 @@ ucnv_swap(const UDataSwapper *ds,
 
                     /* stage 3/result bytes: sometimes uint16_t[] or uint32_t[] */
                     offset=mbcsHeader.offsetFromUBytes;
-                    count= noFromU ? 0 : mbcsHeader.fromUBytesLength;
+                    count=mbcsHeader.fromUBytesLength;
                     switch(outputType) {
                     case MBCS_OUTPUT_2:
                     case MBCS_OUTPUT_3_EUC:
@@ -1541,13 +1232,6 @@ ucnv_swap(const UDataSwapper *ds,
                         /* just uint8_t[], nothing to swap */
                         break;
                     }
-
-                    if(mbcsIndexLength!=0) {
-                        offset+=count;
-                        count=mbcsIndexLength;
-                        ds->swapArray16(ds, inBytes+offset, (int32_t)count,
-                                           outBytes+offset, pErrorCode);
-                    }
                 }
             }
 
@@ -1609,5 +1293,3 @@ ucnv_swap(const UDataSwapper *ds,
 }
 
 #endif /* #if !UCONFIG_NO_LEGACY_CONVERSION */
-
-#endif
