@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 2003-2004, International Business Machines
+*   Copyright (C) 2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -20,9 +20,6 @@
 #define __UCNV_EXT_H__
 
 #include "unicode/utypes.h"
-
-#if !UCONFIG_NO_CONVERSION
-
 #include "unicode/ucnv.h"
 #include "ucnv_cnv.h"
 
@@ -384,7 +381,7 @@ ucnv_extContinueMatchFromU(UConverter *cnv,
 
 U_CFUNC void
 ucnv_extGetUnicodeSet(const UConverterSharedData *sharedData,
-                      const USetAdder *sa,
+                      USet *set,
                       UConverterUnicodeSet which,
                       UErrorCode *pErrorCode);
 
@@ -459,5 +456,3 @@ ucnv_extGetUnicodeSet(const UConverterSharedData *sharedData,
 #define UCNV_EXT_FROM_U_GET_DATA(value) ((value)&UCNV_EXT_FROM_U_DATA_MASK)
 
 #endif
-
-#endif
