@@ -1,6 +1,6 @@
 /*  
 **********************************************************************
-*   Copyright (C) 2000-2009, International Business Machines
+*   Copyright (C) 2000-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   file name:  ucnvhz.c
@@ -16,15 +16,15 @@
 
 #include "unicode/utypes.h"
 
-#if !UCONFIG_NO_CONVERSION && !UCONFIG_NO_LEGACY_CONVERSION
+#if !UCONFIG_NO_LEGACY_CONVERSION
 
 #include "cmemory.h"
+#include "unicode/ucnv_err.h"
 #include "unicode/ucnv.h"
 #include "unicode/ucnv_cb.h"
 #include "unicode/uset.h"
 #include "ucnv_bld.h"
 #include "ucnv_cnv.h"
-#include "ucnv_imp.h"
 
 #define UCNV_TILDE 0x7E          /* ~ */
 #define UCNV_OPEN_BRACE 0x7B     /* { */
@@ -54,39 +54,33 @@
 
 
 typedef struct{
-    UConverter* gbConverter;
     int32_t targetIndex;
     int32_t sourceIndex;
     UBool isEscapeAppended;
+    UConverter* gbConverter;
     UBool isStateDBCS;
     UBool isTargetUCharDBCS;
-    UBool isEmptySegment;
 }UConverterDataHZ;
 
 
 
 static void 
-_HZOpen(UConverter *cnv, UConverterLoadArgs *pArgs, UErrorCode *errorCode){
-    UConverter *gbConverter;
-    if(pArgs->onlyTestIsLoadable) {
-        ucnv_canCreateConverter("GBK", errorCode);  /* errorCode carries result */
-        return;
-    }
-    gbConverter = ucnv_open("GBK", errorCode);
-    if(U_FAILURE(*errorCode)) {
-        return;
-    }
+_HZOpen(UConverter *cnv, const char *name,const char *locale,uint32_t options, UErrorCode *errorCode){
     cnv->toUnicodeStatus = 0;
     cnv->fromUnicodeStatus= 0;
     cnv->mode=0;
     cnv->fromUChar32=0x0000;
-    cnv->extraInfo = uprv_malloc(sizeof(UConverterDataHZ));
+    cnv->extraInfo = uprv_malloc (sizeof (UConverterDataHZ));
     if(cnv->extraInfo != NULL){
-        uprv_memset(cnv->extraInfo, 0, sizeof(UConverterDataHZ));
-        ((UConverterDataHZ*)cnv->extraInfo)->gbConverter = gbConverter;
+        ((UConverterDataHZ*)cnv->extraInfo)->gbConverter = ucnv_open("ibm-1386",errorCode);
+        ((UConverterDataHZ*)cnv->extraInfo)->isStateDBCS = FALSE;
+        ((UConverterDataHZ*)cnv->extraInfo)->isEscapeAppended = FALSE;
+        ((UConverterDataHZ*)cnv->extraInfo)->targetIndex = 0;
+        ((UConverterDataHZ*)cnv->extraInfo)->sourceIndex = 0;
+        ((UConverterDataHZ*)cnv->extraInfo)->isTargetUCharDBCS = FALSE;
     }
+    /* test for NULL */
     else {
-        ucnv_close(gbConverter);
         *errorCode = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
@@ -110,7 +104,6 @@ _HZReset(UConverter *cnv, UConverterResetChoice choice){
         cnv->mode=0;
         if(cnv->extraInfo != NULL){
             ((UConverterDataHZ*)cnv->extraInfo)->isStateDBCS = FALSE;
-            ((UConverterDataHZ*)cnv->extraInfo)->isEmptySegment = FALSE;
         }
     }
     if(choice!=UCNV_RESET_TO_UNICODE) {
@@ -143,10 +136,6 @@ _HZReset(UConverter *cnv, UConverterResetChoice choice){
 *   from-GB code '~}' ($7E7D) is outside the defined GB range.)
 *
 *   Source: RFC 1842
-*
-*   Note that the formal syntax in RFC 1842 is invalid. I assume that the
-*   intended definition of single-byte-segment is as follows (pedberg):
-*   single-byte-segment = single-byte-seq 1*single-byte-char
 */
 
 
@@ -158,16 +147,13 @@ UConverter_toUnicode_HZ_OFFSETS_LOGIC(UConverterToUnicodeArgs *args,
     UChar *myTarget = args->target;
     const char *mySourceLimit = args->sourceLimit;
     UChar32 targetUniChar = 0x0000;
-    int32_t mySourceChar = 0x0000;
+    UChar mySourceChar = 0x0000;
     UConverterDataHZ* myData=(UConverterDataHZ*)(args->converter->extraInfo);
-    tempBuf[0]=0; 
-    tempBuf[1]=0;
-
-    /* Calling code already handles this situation. */
-    /*if ((args->converter == NULL) || (args->targetLimit < args->target) || (mySourceLimit < args->source)){
+       
+    if ((args->converter == NULL) || (args->targetLimit < args->target) || (mySourceLimit < args->source)){
         *err = U_ILLEGAL_ARGUMENT_ERROR;
         return;
-    }*/
+    }
     
     while(mySource< mySourceLimit){
         
@@ -175,123 +161,90 @@ UConverter_toUnicode_HZ_OFFSETS_LOGIC(UConverterToUnicodeArgs *args,
             
             mySourceChar= (unsigned char) *mySource++;
 
-            if(args->converter->mode == UCNV_TILDE) {
-                /* second byte after ~ */
-                args->converter->mode=0;
-                switch(mySourceChar) {
+            switch(mySourceChar){
                 case 0x0A:
-                    /* no output for ~\n (line-continuation marker) */
-                    continue;
-                case UCNV_TILDE:
-                    if(args->offsets) {
-                        args->offsets[myTarget - args->target]=(int32_t)(mySource - args->source - 2);
+                    if(args->converter->mode ==UCNV_TILDE){
+                        args->converter->mode=0;
+                        
                     }
                     *(myTarget++)=(UChar)mySourceChar;
-                    myData->isEmptySegment = FALSE;
                     continue;
-                case UCNV_OPEN_BRACE:
-                case UCNV_CLOSE_BRACE:
-                    myData->isStateDBCS = (mySourceChar == UCNV_OPEN_BRACE);
-                    if (myData->isEmptySegment) {
-                        myData->isEmptySegment = FALSE; /* we are handling it, reset to avoid future spurious errors */
-                        *err = U_ILLEGAL_ESCAPE_SEQUENCE;
-                        args->converter->toUCallbackReason = UCNV_IRREGULAR;
-                        args->converter->toUBytes[0] = UCNV_TILDE;
-                        args->converter->toUBytes[1] = mySourceChar;
-                        args->converter->toULength = 2;
-                        args->target = myTarget;
-                        args->source = mySource;
-                        return;
+            
+                case UCNV_TILDE:
+                    if(args->converter->mode ==UCNV_TILDE){
+                        *(myTarget++)=(UChar)mySourceChar;
+                        args->converter->mode=0;
+                        continue;
+                        
                     }
-                    myData->isEmptySegment = TRUE;
-                    continue;
+                    else if(args->converter->toUnicodeStatus !=0){
+                        args->converter->mode=0;
+                        break;
+                    }
+                    else{
+                        args->converter->mode = UCNV_TILDE;
+                        continue;
+                    }
+                
+                
+                case UCNV_OPEN_BRACE:
+                    if(args->converter->mode == UCNV_TILDE){
+                        args->converter->mode=0;
+                        myData->isStateDBCS = TRUE;
+                        continue;
+                    }
+                    else{
+                        break;
+                    }
+               
+                
+                case UCNV_CLOSE_BRACE:
+                    if(args->converter->mode == UCNV_TILDE){
+                        args->converter->mode=0;
+                         myData->isStateDBCS = FALSE;
+                        continue;
+                    }
+                    else{
+                        break;
+                    }
+                
                 default:
                      /* if the first byte is equal to TILDE and the trail byte
                      * is not a valid byte then it is an error condition
                      */
-                    /*
-                     * Ticket 5691: consistent illegal sequences:
-                     * - We include at least the first byte in the illegal sequence.
-                     * - If any of the non-initial bytes could be the start of a character,
-                     *   we stop the illegal sequence before the first one of those.
-                     */
-                    myData->isEmptySegment = FALSE; /* different error here, reset this to avoid spurious future error */
-                    *err = U_ILLEGAL_ESCAPE_SEQUENCE;
-                    args->converter->toUBytes[0] = UCNV_TILDE;
-                    if( myData->isStateDBCS ?
-                            (0x21 <= mySourceChar && mySourceChar <= 0x7e) :
-                            mySourceChar <= 0x7f
-                    ) {
-                        /* The current byte could be the start of a character: Back it out. */
-                        args->converter->toULength = 1;
-                        --mySource;
-                    } else {
-                        /* Include the current byte in the illegal sequence. */
-                        args->converter->toUBytes[1] = mySourceChar;
-                        args->converter->toULength = 2;
+                    if(args->converter->mode == UCNV_TILDE){
+                        args->converter->mode=0;
+                        mySourceChar= (UChar)(((UCNV_TILDE+0x80) << 8) | ((mySourceChar & 0x00ff)+0x80));
+                        goto SAVE_STATE;
                     }
-                    args->target = myTarget;
-                    args->source = mySource;
-                    return;
-                }
-            } else if(myData->isStateDBCS) {
+                    
+                    break;
+
+            }
+             
+            if(myData->isStateDBCS){
                 if(args->converter->toUnicodeStatus == 0x00){
-                    /* lead byte */
-                    if(mySourceChar == UCNV_TILDE) {
-                        args->converter->mode = UCNV_TILDE;
-                    } else {
-                        /* add another bit to distinguish a 0 byte from not having seen a lead byte */
-                        args->converter->toUnicodeStatus = (uint32_t) (mySourceChar | 0x100);
-                        myData->isEmptySegment = FALSE; /* the segment has something, either valid or will produce a different error, so reset this */
-                    }
+                    args->converter->toUnicodeStatus = (UChar) mySourceChar;
                     continue;
                 }
                 else{
-                    /* trail byte */
-                    int leadIsOk, trailIsOk;
-                    uint32_t leadByte = args->converter->toUnicodeStatus & 0xff;
-                    targetUniChar = 0xffff;
-                    /*
-                     * Ticket 5691: consistent illegal sequences:
-                     * - We include at least the first byte in the illegal sequence.
-                     * - If any of the non-initial bytes could be the start of a character,
-                     *   we stop the illegal sequence before the first one of those.
-                     *
-                     * In HZ DBCS, if the second byte is in the 21..7e range,
-                     * we report only the first byte as the illegal sequence.
-                     * Otherwise we convert or report the pair of bytes.
-                     */
-                    leadIsOk = (uint8_t)(leadByte - 0x21) <= (0x7d - 0x21);
-                    trailIsOk = (uint8_t)(mySourceChar - 0x21) <= (0x7e - 0x21);
-                    if (leadIsOk && trailIsOk) {
-                        tempBuf[0] = (char) (leadByte+0x80) ;
-                        tempBuf[1] = (char) (mySourceChar+0x80);
-                        targetUniChar = ucnv_MBCSSimpleGetNextUChar(myData->gbConverter->sharedData,
-                            tempBuf, 2, args->converter->useFallback);
-                        mySourceChar= (leadByte << 8) | mySourceChar;
-                    } else if (trailIsOk) {
-                        /* report a single illegal byte and continue with the following DBCS starter byte */
-                        --mySource;
-                        mySourceChar = (int32_t)leadByte;
-                    } else {
-                        /* report a pair of illegal bytes if the second byte is not a DBCS starter */
-                        /* add another bit so that the code below writes 2 bytes in case of error */
-                        mySourceChar= 0x10000 | (leadByte << 8) | mySourceChar;
-                    }
+                    tempBuf[0] = (char) (args->converter->toUnicodeStatus+0x80) ;
+                    tempBuf[1] = (char) (mySourceChar+0x80);
+                    mySourceChar= (UChar)(((args->converter->toUnicodeStatus+0x80) << 8) | ((mySourceChar & 0x00ff)+0x80));
                     args->converter->toUnicodeStatus =0x00;
+                    targetUniChar = _MBCSSimpleGetNextUChar(myData->gbConverter->sharedData,
+                        tempBuf, 2, args->converter->useFallback);
                 }
             }
             else{
-                if(mySourceChar == UCNV_TILDE) {
-                    args->converter->mode = UCNV_TILDE;
-                    continue;
-                } else if(mySourceChar <= 0x7f) {
-                    targetUniChar = (UChar)mySourceChar;  /* ASCII */
-                    myData->isEmptySegment = FALSE; /* the segment has something valid */
-                } else {
-                    targetUniChar = 0xffff;
-                    myData->isEmptySegment = FALSE; /* different error here, reset this to avoid spurious future error */
+                if(args->converter->fromUnicodeStatus == 0x00){
+                    targetUniChar = _MBCSSimpleGetNextUChar(myData->gbConverter->sharedData,
+                        mySource - 1, 1, args->converter->useFallback);
                 }
+                else{
+                    goto SAVE_STATE;
+                }
+
             }
             if(targetUniChar < 0xfffe){
                 if(args->offsets) {
@@ -300,16 +253,17 @@ UConverter_toUnicode_HZ_OFFSETS_LOGIC(UConverterToUnicodeArgs *args,
 
                 *(myTarget++)=(UChar)targetUniChar;
             }
-            else /* targetUniChar>=0xfffe */ {
+            else if(targetUniChar>=0xfffe){
+SAVE_STATE:
                 if(targetUniChar == 0xfffe){
                     *err = U_INVALID_CHAR_FOUND;
                 }
                 else{
                     *err = U_ILLEGAL_CHAR_FOUND;
                 }
-                if(mySourceChar > 0xff){
-                    args->converter->toUBytes[0] = (uint8_t)(mySourceChar >> 8);
-                    args->converter->toUBytes[1] = (uint8_t)mySourceChar;
+                if(myData->isStateDBCS){
+                    args->converter->toUBytes[0] = (uint8_t)(tempBuf[0]-0x80);
+                    args->converter->toUBytes[1] = (uint8_t)(tempBuf[1]-0x80);
                     args->converter->toULength=2;
                 }
                 else{
@@ -342,18 +296,18 @@ UConverter_fromUnicode_HZ_OFFSETS_LOGIC (UConverterFromUnicodeArgs * args,
     int32_t mySourceLength = (int32_t)(args->sourceLimit - args->source);
     int32_t length=0;
     uint32_t targetUniChar = 0x0000;
-    UChar32 mySourceChar = 0x0000;
+    UChar32 mySourceChar = 0x0000,c=0x0000;
     UConverterDataHZ *myConverterData=(UConverterDataHZ*)args->converter->extraInfo;
     UBool isTargetUCharDBCS = (UBool) myConverterData->isTargetUCharDBCS;
     UBool oldIsTargetUCharDBCS = isTargetUCharDBCS;
+    UBool isEscapeAppended =FALSE;
     int len =0;
     const char* escSeq=NULL;
     
-    /* Calling code already handles this situation. */
-    /*if ((args->converter == NULL) || (args->targetLimit < myTarget) || (args->sourceLimit < args->source)){
+    if ((args->converter == NULL) || (args->targetLimit < myTarget) || (args->sourceLimit < args->source)){
         *err = U_ILLEGAL_ARGUMENT_ERROR;
         return;
-    }*/
+    }
     if(args->converter->fromUChar32!=0 && myTargetIndex < targetLength) {
         goto getTrail;
     }
@@ -362,7 +316,7 @@ UConverter_fromUnicode_HZ_OFFSETS_LOGIC (UConverterFromUnicodeArgs * args,
         targetUniChar = missingCharMarker;
         if (myTargetIndex < targetLength){
             
-            mySourceChar = (UChar) mySource[mySourceIndex++];
+            c=mySourceChar = (UChar) mySource[mySourceIndex++];
             
 
             oldIsTargetUCharDBCS = isTargetUCharDBCS;
@@ -372,21 +326,16 @@ UConverter_fromUnicode_HZ_OFFSETS_LOGIC (UConverterFromUnicodeArgs * args,
                 escSeq = TILDE_ESCAPE;
                 CONCAT_ESCAPE_MACRO(args, myTargetIndex, targetLength, escSeq,err,len,mySourceIndex);
                 continue;
-            } else if(mySourceChar <= 0x7f) {
-                length = 1;
-                targetUniChar = mySourceChar;
-            } else {
-                length= ucnv_MBCSFromUChar32(myConverterData->gbConverter->sharedData,
+            }
+            else{
+                length= _MBCSFromUChar32(myConverterData->gbConverter->sharedData,
                     mySourceChar,&targetUniChar,args->converter->useFallback);
-                /* we can only use lead bytes 21..7D and trail bytes 21..7E */
-                if( length == 2 &&
-                    (uint16_t)(targetUniChar - 0xa1a1) <= (0xfdfe - 0xa1a1) &&
-                    (uint8_t)(targetUniChar - 0xa1) <= (0xfe - 0xa1)
-                ) {
-                    targetUniChar -= 0x8080;
-                } else {
-                    targetUniChar = missingCharMarker;
-                }
+
+            }
+            /* only DBCS or SBCS characters are expected*/
+            /* DB haracters with high bit set to 1 are expected */
+            if(length > 2 || length==0 ||(((targetUniChar & 0x8080) != 0x8080)&& length==2)){
+                targetUniChar= missingCharMarker;
             }
             if (targetUniChar != missingCharMarker){
                myConverterData->isTargetUCharDBCS = isTargetUCharDBCS = (UBool)(targetUniChar>0x00FF);     
@@ -396,35 +345,35 @@ UConverter_fromUnicode_HZ_OFFSETS_LOGIC (UConverterFromUnicodeArgs * args,
                         len =ESC_LEN;
                         escSeq = SB_ESCAPE;
                         CONCAT_ESCAPE_MACRO(args, myTargetIndex, targetLength, escSeq,err,len,mySourceIndex);
-                        myConverterData->isEscapeAppended = TRUE;
+                        myConverterData->isEscapeAppended =isEscapeAppended =TRUE;
                     }
                     else{ /* Shifting from a single byte to double byte mode*/
                         len =ESC_LEN;
                         escSeq = DB_ESCAPE;
                         CONCAT_ESCAPE_MACRO(args, myTargetIndex, targetLength, escSeq,err,len,mySourceIndex);
-                        myConverterData->isEscapeAppended = TRUE;
+                        myConverterData->isEscapeAppended =isEscapeAppended =TRUE;
                         
                     }
                 }
             
                 if(isTargetUCharDBCS){
                     if( myTargetIndex <targetLength){
-                        myTarget[myTargetIndex++] =(char) (targetUniChar >> 8);
+                        myTarget[myTargetIndex++] =(char) ((targetUniChar >> 8) -0x80);
                         if(offsets){
                             *(offsets++) = mySourceIndex-1;
                         }
                         if(myTargetIndex < targetLength){
-                            myTarget[myTargetIndex++] =(char) targetUniChar;
+                            myTarget[myTargetIndex++] =(char) ((targetUniChar & 0x00FF) -0x80);
                             if(offsets){
                                 *(offsets++) = mySourceIndex-1;
                             }
                         }else{
-                            args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] = (char) targetUniChar;
+                            args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] = (char) ((targetUniChar & 0x00FF) -0x80);
                             *err = U_BUFFER_OVERFLOW_ERROR;
                         } 
                     }else{
-                        args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] =(char) (targetUniChar >> 8);
-                        args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] = (char) targetUniChar;
+                        args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] =(char) ((targetUniChar >> 8) -0x80);
+                        args->converter->charErrorBuffer[args->converter->charErrorBufferLength++] = (char) ((targetUniChar & 0x00FF) -0x80);
                         *err = U_BUFFER_OVERFLOW_ERROR;
                     }
 
@@ -509,28 +458,20 @@ _HZ_WriteSub(UConverterFromUnicodeArgs *args, int32_t offsetIndex, UErrorCode *e
         *p++= UCNV_CLOSE_BRACE;
         convData->isTargetUCharDBCS=FALSE;
     }
-    *p++= (char)cnv->subChars[0];
+    *p++= cnv->subChar[0];
 
     ucnv_cbFromUWriteBytes(args,
                            buffer, (int32_t)(p - buffer),
                            offsetIndex, err);
 }
 
-/*
- * Structure for cloning an HZ converter into a single memory block.
- * ucnv_safeClone() of the HZ converter will align the entire cloneHZStruct,
- * and then ucnv_safeClone() of the sub-converter may additionally align
- * subCnv inside the cloneHZStruct, for which we need the deadSpace after
- * subCnv. This is because UAlignedMemory may be larger than the actually
- * necessary alignment size for the platform.
- * The other cloneHZStruct fields will not be moved around,
- * and are aligned properly with cloneHZStruct's alignment.
- */
-struct cloneHZStruct
+/* structure for SafeClone calculations */
+struct cloneStruct
 {
     UConverter cnv;
+    UAlignedMemory deadSpace1;
     UConverter subCnv;
-    UAlignedMemory deadSpace;
+    UAlignedMemory deadSpace2;
     UConverterDataHZ mydata;
 };
 
@@ -541,8 +482,8 @@ _HZ_SafeClone(const UConverter *cnv,
               int32_t *pBufferSize, 
               UErrorCode *status)
 {
-    struct cloneHZStruct * localClone;
-    int32_t size, bufferSizeNeeded = sizeof(struct cloneHZStruct);
+    struct cloneStruct * localClone;
+    int32_t size, bufferSizeNeeded = sizeof(struct cloneStruct);
 
     if (U_FAILURE(*status)){
         return 0;
@@ -553,15 +494,15 @@ _HZ_SafeClone(const UConverter *cnv,
         return 0;
     }
 
-    localClone = (struct cloneHZStruct *)stackBuffer;
-    /* ucnv.c/ucnv_safeClone() copied the main UConverter already */
+    localClone = (struct cloneStruct *)stackBuffer;
+    uprv_memcpy(&localClone->cnv, cnv, sizeof(UConverter));
 
     uprv_memcpy(&localClone->mydata, cnv->extraInfo, sizeof(UConverterDataHZ));
     localClone->cnv.extraInfo = &localClone->mydata;
     localClone->cnv.isExtraLocal = TRUE;
 
     /* deep-clone the sub-converter */
-    size = (int32_t)(sizeof(UConverter) + sizeof(UAlignedMemory)); /* include size of padding */
+    size = (int32_t)sizeof(UConverter);
     ((UConverterDataHZ*)localClone->cnv.extraInfo)->gbConverter =
         ucnv_safeClone(((UConverterDataHZ*)cnv->extraInfo)->gbConverter, &localClone->subCnv, &size, status);
 
@@ -570,17 +511,17 @@ _HZ_SafeClone(const UConverter *cnv,
 
 static void
 _HZ_GetUnicodeSet(const UConverter *cnv,
-                  const USetAdder *sa,
+                  USet *set,
                   UConverterUnicodeSet which,
                   UErrorCode *pErrorCode) {
-    /* HZ converts all of ASCII */
-    sa->addRange(sa->set, 0, 0x7f);
+    /* the tilde '~' is hardcoded in the converter */
+    uset_add(set, 0x7e);
 
     /* add all of the code points that the sub-converter handles */
-    ucnv_MBCSGetFilteredUnicodeSetForUnicode(
-        ((UConverterDataHZ*)cnv->extraInfo)->gbConverter->sharedData,
-        sa, which, UCNV_SET_FILTER_HZ,
-        pErrorCode);
+    ((UConverterDataHZ*)cnv->extraInfo)->
+        gbConverter->sharedData->impl->
+            getUnicodeSet(((UConverterDataHZ*)cnv->extraInfo)->gbConverter,
+                          set, which, pErrorCode);
 }
 
 static const UConverterImpl _HZImpl={
