@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 2000-2009, International Business Machines
+*   Copyright (C) 2000-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -20,9 +20,6 @@
 */
 
 #include "unicode/utypes.h"
-
-#if !UCONFIG_NO_CONVERSION
-
 #include "unicode/ucnv.h"
 #include "unicode/ucnv_cb.h"
 #include "ucnv_bld.h"
@@ -190,12 +187,10 @@ _SCSUReset(UConverter *cnv, UConverterResetChoice choice) {
 
 static void
 _SCSUOpen(UConverter *cnv,
-          UConverterLoadArgs *pArgs,
+          const char *name,
+          const char *locale,
+          uint32_t options,
           UErrorCode *pErrorCode) {
-    const char *locale=pArgs->locale;
-    if(pArgs->onlyTestIsLoadable) {
-        return;
-    }
     cnv->extraInfo=uprv_malloc(sizeof(SCSUData));
     if(cnv->extraInfo!=NULL) {
         if(locale!=NULL && locale[0]=='j' && locale[1]=='a' && (locale[2]==0 || locale[2]=='_')) {
@@ -207,10 +202,6 @@ _SCSUOpen(UConverter *cnv,
     } else {
         *pErrorCode=U_MEMORY_ALLOCATION_ERROR;
     }
-
-    /* Set the substitution character U+fffd as a Unicode string. */
-    cnv->subUChars[0]=0xfffd;
-    cnv->subCharLen=-1;
 }
 
 static void
@@ -1439,14 +1430,11 @@ outputBytes:
          * first to the overflow buffer what does not fit into the
          * regular target.
          */
-        /* we know that 0<=targetCapacity<length<=4 */
-        /* targetCapacity==0 when SCU+supplementary where SCU used up targetCapacity==1 */
+        /* we know that 1<=targetCapacity<length<=4 */
         length-=targetCapacity;
         p=(uint8_t *)cnv->charErrorBuffer;
         switch(length) {
             /* each branch falls through to the next one */
-        case 4:
-            *p++=(uint8_t)(c>>24);
         case 3:
             *p++=(uint8_t)(c>>16);
         case 2:
@@ -1479,6 +1467,7 @@ outputBytes:
                 *offsets++=sourceIndex;
             }
         default:
+            /* will never occur */
             break;
         }
 
@@ -1876,14 +1865,11 @@ outputBytes:
          * first to the overflow buffer what does not fit into the
          * regular target.
          */
-        /* we know that 0<=targetCapacity<length<=4 */
-        /* targetCapacity==0 when SCU+supplementary where SCU used up targetCapacity==1 */
+        /* we know that 1<=targetCapacity<length<=4 */
         length-=targetCapacity;
         p=(uint8_t *)cnv->charErrorBuffer;
         switch(length) {
             /* each branch falls through to the next one */
-        case 4:
-            *p++=(uint8_t)(c>>24);
         case 3:
             *p++=(uint8_t)(c>>16);
         case 2:
@@ -1907,6 +1893,7 @@ outputBytes:
         case 1:
             *target++=(uint8_t)c;
         default:
+            /* will never occur */
             break;
         }
 
@@ -1932,8 +1919,32 @@ _SCSUGetName(const UConverter *cnv) {
     }
 }
 
+static void
+_SCSUWriteSub(UConverterFromUnicodeArgs *pArgs,
+               int32_t offsetIndex,
+               UErrorCode *pErrorCode) {
+    static const char squ_fffd[]={ (char)SQU, (char)0xffu, (char)0xfdu };
+
+    /*
+     * The substitution character is U+fffd={ ff, fd }.
+     * If the SCSU converter is in Unicode mode, then these two bytes just need to
+     * be written. Otherwise, this character is quoted.
+     */
+    if(((SCSUData *)pArgs->converter->extraInfo)->fromUIsSingleByteMode) {
+        /* single-byte mode: quote Unicode */
+        ucnv_cbFromUWriteBytes(pArgs,
+                               squ_fffd, 3,
+                               offsetIndex, pErrorCode);
+    } else {
+        /* Unicode mode: just write U+fffd */
+        ucnv_cbFromUWriteBytes(pArgs,
+                               squ_fffd+1, 2,
+                               offsetIndex, pErrorCode);
+    }
+}
+
 /* structure for SafeClone calculations */
-struct cloneSCSUStruct
+struct cloneStruct
 {
     UConverter cnv;
     SCSUData mydata;
@@ -1945,8 +1956,8 @@ _SCSUSafeClone(const UConverter *cnv,
                int32_t *pBufferSize, 
                UErrorCode *status)
 {
-    struct cloneSCSUStruct * localClone;
-    int32_t bufferSizeNeeded = sizeof(struct cloneSCSUStruct);
+    struct cloneStruct * localClone;
+    int32_t bufferSizeNeeded = sizeof(struct cloneStruct);
 
     if (U_FAILURE(*status)){
         return 0;
@@ -1957,7 +1968,7 @@ _SCSUSafeClone(const UConverter *cnv,
         return 0;
     }
 
-    localClone = (struct cloneSCSUStruct *)stackBuffer;
+    localClone = (struct cloneStruct *)stackBuffer;
     /* ucnv.c/ucnv_safeClone() copied the main UConverter already */
 
     uprv_memcpy(&localClone->mydata, cnv->extraInfo, sizeof(SCSUData));
@@ -1986,7 +1997,7 @@ static const UConverterImpl _SCSUImpl={
 
     NULL,
     _SCSUGetName,
-    NULL,
+    _SCSUWriteSub,
     _SCSUSafeClone,
     ucnv_getCompleteUnicodeSet
 };
@@ -1994,12 +2005,14 @@ static const UConverterImpl _SCSUImpl={
 static const UConverterStaticData _SCSUStaticData={
     sizeof(UConverterStaticData),
     "SCSU",
-    1212, /* CCSID for SCSU */
+    0, /* CCSID for SCSU */
     UCNV_IBM, UCNV_SCSU,
     1, 3, /* one UChar generates at least 1 byte and at most 3 bytes */
     /*
-     * The subchar here is ignored because _SCSUOpen() sets U+fffd as a Unicode
-     * substitution string.
+     * ### TODO the subchar really must be written by an SCSU function
+     * however, currently SCSU's fromUnicode() never causes errors, therefore
+     * no callbacks will be called and no subchars written
+     * See Jitterbug 2837 - RFE: forbid converting surrogate code points in all charsets
      */
     { 0x0e, 0xff, 0xfd, 0 }, 3,
     FALSE, FALSE,
@@ -2013,5 +2026,3 @@ const UConverterSharedData _SCSUData={
     NULL, NULL, &_SCSUStaticData, FALSE, &_SCSUImpl,
     0
 };
-
-#endif
