@@ -1,7 +1,7 @@
 /*
  *******************************************************************************
  *
- *   Copyright (C) 2003-2007, International Business Machines
+ *   Copyright (C) 2003, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -24,7 +24,6 @@
 #include "punycode.h"
 #include "ustr_imp.h"
 #include "cmemory.h"
-#include "uassert.h"
 #include "sprpimpl.h"
 
 /* it is official IDNA ACE Prefix is "xn--" */
@@ -32,19 +31,15 @@ static const UChar ACE_PREFIX[] ={ 0x0078,0x006E,0x002d,0x002d } ;
 #define ACE_PREFIX_LENGTH 4
 
 #define MAX_LABEL_LENGTH 63
-/* The Max length of the labels should not be more than MAX_LABEL_LENGTH */
-#define MAX_LABEL_BUFFER_SIZE 100
+#define HYPHEN      0x002D
+/* The Max length of the labels should not be more than 64 */
+#define MAX_LABEL_BUFFER_SIZE 100 
+#define MAX_IDN_BUFFER_SIZE   300
 
-#define MAX_DOMAIN_NAME_LENGTH 255
-/* The Max length of the domain names should not be more than MAX_DOMAIN_NAME_LENGTH */
-#define MAX_IDN_BUFFER_SIZE   MAX_DOMAIN_NAME_LENGTH+1
-
-#define LOWER_CASE_DELTA 0x0020
-#define HYPHEN           0x002D
-#define FULL_STOP        0x002E
 #define CAPITAL_A        0x0041
 #define CAPITAL_Z        0x005A
-
+#define LOWER_CASE_DELTA 0x0020
+#define FULL_STOP        0x002E
 #define DATA_FILE_NAME   "uidna"
 
 inline static UChar 
@@ -71,6 +66,12 @@ startsWithPrefix(const UChar* src , int32_t srcLength){
     return startsWithPrefix;
 }
 
+inline static void 
+toASCIILower(UChar* src, int32_t srcLen){
+    for(int32_t i=0; i<srcLen; i++){
+        src[i] = toASCIILower(src[i]);
+    }
+}
 
 inline static int32_t
 compareCaseInsensitiveASCII(const UChar* s1, int32_t s1Len, 
@@ -126,7 +127,7 @@ compareCaseInsensitiveASCII(const UChar* s1, int32_t s1Len,
  * 
  * @param ch The code point to be ascertained
  * @return true if the char is a label separator
- * @stable ICU 2.8
+ * @draft ICU 2.8
  */
 static inline UBool isLabelSeparator(UChar ch){
     switch(ch){
@@ -144,8 +145,9 @@ static inline UBool isLabelSeparator(UChar ch){
 // if *limit == separator then the length returned does not include 
 // the separtor.
 static inline int32_t
-getNextSeparator(UChar *src, int32_t srcLength,
-                 UChar **limit, UBool *done){
+getNextSeparator(UChar *src,int32_t srcLength,
+                 UChar **limit,
+                 UBool *done){
     if(srcLength == -1){
         int32_t i;
         for(i=0 ; ;i++){
@@ -198,14 +200,12 @@ _internal_toASCII(const UChar* src, int32_t srcLength,
                   int32_t options,
                   UStringPrepProfile* nameprep,
                   UParseError* parseError,
-                  UErrorCode* status)
-{
+                  UErrorCode* status){
 
-    // TODO Revisit buffer handling. The label should not be over 63 ASCII characters. ICU4J may need to be updated too.
     UChar b1Stack[MAX_LABEL_BUFFER_SIZE], b2Stack[MAX_LABEL_BUFFER_SIZE];
     //initialize pointers to stack buffers
     UChar  *b1 = b1Stack, *b2 = b2Stack;
-    int32_t b1Len=0, b2Len, 
+    int32_t b1Len, b2Len, 
             b1Capacity = MAX_LABEL_BUFFER_SIZE, 
             b2Capacity = MAX_LABEL_BUFFER_SIZE ,
             reqLength=0;
@@ -224,79 +224,45 @@ _internal_toASCII(const UChar* src, int32_t srcLength,
     UBool useSTD3ASCIIRules = (UBool)((options & UIDNA_USE_STD3_RULES) != 0);
 
     int32_t failPos = -1;
+
+    // step 2    
+    b1Len = usprep_prepare(nameprep, src, srcLength, b1, b1Capacity, namePrepOptions, parseError, status);
     
-    if(srcLength == -1){
-        srcLength = u_strlen(src);
-    }
-    
-    if(srcLength > b1Capacity){
-        b1 = (UChar*) uprv_malloc(srcLength * U_SIZEOF_UCHAR);
+    if(*status == U_BUFFER_OVERFLOW_ERROR){
+        // redo processing of string
+        // we do not have enough room so grow the buffer
+        b1 = (UChar*) uprv_malloc(b1Len * U_SIZEOF_UCHAR);
         if(b1==NULL){
             *status = U_MEMORY_ALLOCATION_ERROR;
             goto CLEANUP;
         }
-        b1Capacity = srcLength;
-    }
 
-    // step 1 
-    for( j=0;j<srcLength;j++){
-        if(src[j] > 0x7F){
-            srcIsASCII = FALSE;
-        }
-        b1[b1Len++] = src[j];
-    }
-    
-    // step 2 is performed only if the source contains non ASCII
-    if(srcIsASCII == FALSE){
+        *status = U_ZERO_ERROR; // reset error
         
-        // step 2    
-        b1Len = usprep_prepare(nameprep, src, srcLength, b1, b1Capacity, namePrepOptions, parseError, status);
-
-        if(*status == U_BUFFER_OVERFLOW_ERROR){
-            // redo processing of string
-            // we do not have enough room so grow the buffer
-            if(b1 != b1Stack){
-                uprv_free(b1);
-            }
-            b1 = (UChar*) uprv_malloc(b1Len * U_SIZEOF_UCHAR);
-            if(b1==NULL){
-                *status = U_MEMORY_ALLOCATION_ERROR;
-                goto CLEANUP;
-            }
-
-            *status = U_ZERO_ERROR; // reset error
-            
-            b1Len = usprep_prepare(nameprep, src, srcLength, b1, b1Len, namePrepOptions, parseError, status);
-        }
+        b1Len = usprep_prepare(nameprep, src, srcLength, b1, b1Len, namePrepOptions, parseError, status);
     }
     // error bail out
     if(U_FAILURE(*status)){
         goto CLEANUP;
     }
-    if(b1Len == 0){
-        *status = U_IDNA_ZERO_LENGTH_LABEL_ERROR;
-        goto CLEANUP;
-    }
 
-    // for step 3 & 4
-    srcIsASCII = TRUE;
+    // step 3 & 4
     for( j=0;j<b1Len;j++){
-        // check if output of usprep_prepare is all ASCII 
         if(b1[j] > 0x7F){
             srcIsASCII = FALSE;
-        }else if(isLDHChar(b1[j])==FALSE){  // if the char is in ASCII range verify that it is an LDH character
+        }
+        // here we do not assemble surrogates
+        // since we know that LDH code points
+        // are in the ASCII range only
+        if(isLDHChar(b1[j])==FALSE){
             srcIsLDH = FALSE;
             failPos = j;
         }
     }
+    
     if(useSTD3ASCIIRules == TRUE){
         // verify 3a and 3b
-        // 3(a) Verify the absence of non-LDH ASCII code points; that is, the
-        //  absence of 0..2C, 2E..2F, 3A..40, 5B..60, and 7B..7F.
-        // 3(b) Verify the absence of leading and trailing hyphen-minus; that
-        //  is, the absence of U+002D at the beginning and end of the
-        //  sequence.
-        if( srcIsLDH == FALSE /* source at this point should not contain anyLDH characters */
+        if( srcIsLDH == FALSE /* source contains some non-LDH characters */
             || b1[0] ==  HYPHEN || b1[b1Len-1] == HYPHEN){
             *status = U_IDNA_STD3_ASCII_RULES_ERROR;
 
@@ -315,7 +281,6 @@ _internal_toASCII(const UChar* src, int32_t srcLength,
             goto CLEANUP;
         }
     }
-    // Step 4: if the source is ASCII then proceed to step 8
     if(srcIsASCII){
         if(b1Len <= destCapacity){
             uprv_memmove(dest, b1, b1Len * U_SIZEOF_UCHAR);
@@ -375,7 +340,7 @@ _internal_toASCII(const UChar* src, int32_t srcLength,
             goto CLEANUP;
         }
     }
-    // step 8: verify the length of label
+
     if(reqLength > MAX_LABEL_LENGTH){
         *status = U_IDNA_LABEL_TOO_LONG_ERROR;
     }
@@ -398,14 +363,12 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
                     int32_t options,
                     UStringPrepProfile* nameprep,
                     UParseError* parseError,
-                    UErrorCode* status)
-{
+                    UErrorCode* status){
 
     //get the options
-    //UBool useSTD3ASCIIRules = (UBool)((options & UIDNA_USE_STD3_RULES) != 0);
+    UBool useSTD3ASCIIRules = (UBool)((options & UIDNA_USE_STD3_RULES) != 0);
     int32_t namePrepOptions = ((options & UIDNA_ALLOW_UNASSIGNED) != 0) ? USPREP_ALLOW_UNASSIGNED: 0; 
-
-    // TODO Revisit buffer handling. The label should not be over 63 ASCII characters. ICU4J may need to be updated too.
+    
     UChar b1Stack[MAX_LABEL_BUFFER_SIZE], b2Stack[MAX_LABEL_BUFFER_SIZE], b3Stack[MAX_LABEL_BUFFER_SIZE];
 
     //initialize pointers to stack buffers
@@ -420,8 +383,8 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
     UBool* caseFlags = NULL;
 
     UBool srcIsASCII = TRUE;
-    /*UBool srcIsLDH = TRUE;
-    int32_t failPos =0;*/
+    UBool srcIsLDH = TRUE;
+    int32_t failPos =0;
 
     // step 1: find out if all the codepoints in src are ASCII  
     if(srcLength==-1){
@@ -429,31 +392,31 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
         for(;src[srcLength]!=0;){
             if(src[srcLength]> 0x7f){
                 srcIsASCII = FALSE;
-            }/*else if(isLDHChar(src[srcLength])==FALSE){
-                // here we do not assemble surrogates
-                // since we know that LDH code points
-                // are in the ASCII range only
+            }
+            // here we do not assemble surrogates
+            // since we know that LDH code points
+            // are in the ASCII range only
+            if(isLDHChar(src[srcLength])==FALSE){
                 srcIsLDH = FALSE;
                 failPos = srcLength;
-            }*/
+            }
             srcLength++;
         }
-    }else if(srcLength > 0){
+    }else{
         for(int32_t j=0; j<srcLength; j++){
             if(src[j]> 0x7f){
                 srcIsASCII = FALSE;
-            }/*else if(isLDHChar(src[j])==FALSE){
-                // here we do not assemble surrogates
-                // since we know that LDH code points
-                // are in the ASCII range only
+            }
+            // here we do not assemble surrogates
+            // since we know that LDH code points
+            // are in the ASCII range only
+            if(isLDHChar(src[j])==FALSE){
                 srcIsLDH = FALSE;
                 failPos = j;
-            }*/
+            }
         }
-    }else{
-        return 0;
     }
-    
+
     if(srcIsASCII == FALSE){
         // step 2: process the string
         b1Len = usprep_prepare(nameprep, src, srcLength, b1, b1Capacity, namePrepOptions, parseError, status);
@@ -481,14 +444,8 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
         b1Len = srcLength;
     }
 
-    // The RFC states that 
-    // <quote>
-    // ToUnicode never fails. If any step fails, then the original input
-    // is returned immediately in that step.
-    // </quote>
-
     //step 3: verify ACE Prefix
-    if(startsWithPrefix(b1,b1Len)){
+    if(startsWithPrefix(src,srcLength)){
 
         //step 4: Remove the ACE Prefix
         b1Prime = b1 + ACE_PREFIX_LENGTH;
@@ -496,7 +453,7 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
 
         //step 5: Decode using punycode
         b2Len = u_strFromPunycode(b1Prime, b1PrimeLen, b2, b2Capacity, caseFlags,status);
-
+        
         if(*status == U_BUFFER_OVERFLOW_ERROR){
             // redo processing of string
             /* we do not have enough room so grow the buffer*/
@@ -507,13 +464,14 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
             }
 
             *status = U_ZERO_ERROR; // reset error
-
+            
             b2Len =  u_strFromPunycode(b1Prime, b1PrimeLen, b2, b2Len, caseFlags, status);
+            
         }
-
-
+        
+        
         //step 6:Apply toASCII
-        b3Len = uidna_toASCII(b2, b2Len, b3, b3Capacity, options, parseError, status);
+        b3Len = uidna_toASCII(b2, b2Len, b3, b3Capacity,options,parseError, status);
 
         if(*status == U_BUFFER_OVERFLOW_ERROR){
             // redo processing of string
@@ -525,9 +483,9 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
             }
 
             *status = U_ZERO_ERROR; // reset error
-
+            
             b3Len =  uidna_toASCII(b2,b2Len,b3,b3Len,options,parseError, status);
-
+        
         }
         //bail out on error
         if(U_FAILURE(*status)){
@@ -536,8 +494,7 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
 
         //step 7: verify
         if(compareCaseInsensitiveASCII(b1, b1Len, b3, b3Len) !=0){
-            // Cause the original to be returned.
-            *status = U_IDNA_VERIFICATION_ERROR;
+            *status = U_IDNA_VERIFICATION_ERROR; 
             goto CLEANUP;
         }
 
@@ -546,16 +503,14 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
         if(b2Len <= destCapacity) {
             uprv_memmove(dest, b2, b2Len * U_SIZEOF_UCHAR);
         }
-    }
-    else{
-        // See the start of this if statement for why this is commented out.
+    }else{
         // verify that STD3 ASCII rules are satisfied
-        /*if(useSTD3ASCIIRules == TRUE){
-            if( srcIsLDH == FALSE // source contains some non-LDH characters
+        if(useSTD3ASCIIRules == TRUE){
+            if( srcIsLDH == FALSE /* source contains some non-LDH characters */
                 || src[0] ==  HYPHEN || src[srcLength-1] == HYPHEN){
                 *status = U_IDNA_STD3_ASCII_RULES_ERROR;
 
-                // populate the parseError struct
+                /* populate the parseError struct */
                 if(srcIsLDH==FALSE){
                     // failPos is always set the index of failure
                     uprv_syntaxError(src,failPos, srcLength,parseError);
@@ -569,8 +524,7 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
 
                 goto CLEANUP;
             }
-        }*/
-        // just return the source
+        }
         //copy the source to destination
         if(srcLength <= destCapacity){
             uprv_memmove(dest,src,srcLength * U_SIZEOF_UCHAR);
@@ -578,7 +532,6 @@ _internal_toUnicode(const UChar* src, int32_t srcLength,
         reqLength = srcLength;
     }
 
-
 CLEANUP:
 
     if(b1 != b1Stack && b1!=src){
@@ -588,7 +541,8 @@ CLEANUP:
         uprv_free(b2);
     }
     uprv_free(caseFlags);
-
+    
+   
     // The RFC states that 
     // <quote>
     // ToUnicode never fails. If any step fails, then the original input
@@ -598,12 +552,13 @@ CLEANUP:
     if(U_FAILURE(*status)){
         //copy the source to destination
         if(dest && srcLength <= destCapacity){
-            // srcLength should have already been set earlier.
-            U_ASSERT(srcLength >= 0);
+          if(srcLength == -1) {
+            uprv_memmove(dest,src,u_strlen(src)* U_SIZEOF_UCHAR);
+          } else {
             uprv_memmove(dest,src,srcLength * U_SIZEOF_UCHAR);
+          }
         }
         reqLength = srcLength;
-        *status = U_ZERO_ERROR;
     }
 
     return u_terminateUChars(dest, destCapacity, reqLength, status);
@@ -652,7 +607,7 @@ uidna_toUnicode(const UChar* src, int32_t srcLength,
         *status = U_ILLEGAL_ARGUMENT_ERROR;
         return 0;
     }  
-
+    
     UStringPrepProfile* nameprep = usprep_open(NULL, DATA_FILE_NAME, status);
     
     if(U_FAILURE(*status)){
@@ -703,19 +658,16 @@ uidna_IDNToASCII(  const UChar *src, int32_t srcLength,
     for(;;){
 
         labelLen = getNextSeparator(labelStart,remainingLen, &delimiter,&done);
-        labelReqLength = 0;
-        if(!(labelLen==0 && done)){// make sure this is not a root label separator.
         
-            labelReqLength = _internal_toASCII( labelStart, labelLen, 
-                                                currentDest, remainingDestCapacity, 
-                                                options, nameprep, 
-                                                parseError, status);
-    
-            if(*status == U_BUFFER_OVERFLOW_ERROR){
-                
-                *status = U_ZERO_ERROR; // reset error
-                remainingDestCapacity = 0;
-            }
+        labelReqLength = _internal_toASCII( labelStart, labelLen, 
+                                            currentDest, remainingDestCapacity, 
+                                            options, nameprep, 
+                                            parseError, status);
+
+        if(*status == U_BUFFER_OVERFLOW_ERROR){
+            
+            *status = U_ZERO_ERROR; // reset error
+            remainingDestCapacity = 0;
         }
 
     
@@ -732,7 +684,6 @@ uidna_IDNToASCII(  const UChar *src, int32_t srcLength,
             // should never occur
             remainingDestCapacity = 0;
         }
-
         if(done == TRUE){
             break;
         }
@@ -742,19 +693,15 @@ uidna_IDNToASCII(  const UChar *src, int32_t srcLength,
             *currentDest++ = FULL_STOP;
             remainingDestCapacity--;
         }
-        reqLength++;
+        reqLength++;           
 
         labelStart = delimiter;
         if(remainingLen >0 ){
-            remainingLen = (int32_t)(srcLength - (delimiter - src));
+            remainingLen = srcLength - (delimiter - src);
         }
 
     }
-
-    if(reqLength > MAX_DOMAIN_NAME_LENGTH){
-        *status = U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR;
-    }
-
+   
     usprep_close(nameprep);
     
     return u_terminateUChars(dest, destCapacity, reqLength, status);
@@ -792,31 +739,23 @@ uidna_IDNToUnicode(  const UChar* src, int32_t srcLength,
     int32_t labelLen = 0, labelReqLength = 0;
     UBool done = FALSE;
 
+
     for(;;){
 
         labelLen = getNextSeparator(labelStart,remainingLen, &delimiter,&done);
         
-        // The RFC states that 
-        // <quote>
-        // ToUnicode never fails. If any step fails, then the original input
-        // is returned immediately in that step.
-        // </quote>
-        // _internal_toUnicode will copy the label.
-        /*if(labelLen==0 && done==FALSE){ 
-            *status = U_IDNA_ZERO_LENGTH_LABEL_ERROR;
-            break;
-        }*/
-        
         labelReqLength = _internal_toUnicode(labelStart, labelLen, 
                                              currentDest, remainingDestCapacity, 
                                              options, nameprep, 
                                              parseError, status);
 
         if(*status == U_BUFFER_OVERFLOW_ERROR){
+            
             *status = U_ZERO_ERROR; // reset error
             remainingDestCapacity = 0;
         }
 
+    
         if(U_FAILURE(*status)){
             break;
         }
@@ -836,24 +775,19 @@ uidna_IDNToUnicode(  const UChar* src, int32_t srcLength,
         }
 
         // add the label separator
-        // Unlike the ToASCII operation we don't normalize the label separators
         if(remainingDestCapacity > 0){
-            *currentDest++ = *(labelStart + labelLen);
+            *currentDest++ = FULL_STOP;
             remainingDestCapacity--;
         }
-        reqLength++;
+        reqLength++;           
 
         labelStart = delimiter;
         if(remainingLen >0 ){
-            remainingLen = (int32_t)(srcLength - (delimiter - src));
+            remainingLen = srcLength - (delimiter - src);
         }
 
     }
-
-    if(reqLength > MAX_DOMAIN_NAME_LENGTH){
-        *status = U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR;
-    }
-
+   
     usprep_close(nameprep);
     
     return u_terminateUChars(dest, destCapacity, reqLength, status);
