@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1997-2008, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -18,30 +18,10 @@
 ******************************************************************************
 */
 
-#include "unicode/utypes.h"
-#include "uassert.h"
-#include "ucln_cmn.h"
-
-#if defined(U_DARWIN)
-#include <AvailabilityMacros.h>
-#if (ICU_USE_THREADS == 1) && defined(MAC_OS_X_VERSION_10_4) && defined(MAC_OS_X_VERSION_MIN_REQUIRED) && (MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4)
-#if defined(__STRICT_ANSI__)
-#define UPRV_REMAP_INLINE
-#define inline
-#endif
-#include <libkern/OSAtomic.h>
-#define USE_MAC_OS_ATOMIC_INCREMENT 1
-#if defined(UPRV_REMAP_INLINE)
-#undef inline
-#undef UPRV_REMAP_INLINE
-#endif
-#endif
-#endif
-
 /* Assume POSIX, and modify as necessary below */
 #define POSIX
 
-#if defined(U_WINDOWS)
+#if defined(_WIN32)
 #undef POSIX
 #endif
 #if defined(macintosh)
@@ -51,14 +31,20 @@
 #undef POSIX
 #endif
 
+
+#include "unicode/utypes.h"
+#include "uassert.h"
+#include "ucln_cmn.h"
+
+
 #if defined(POSIX) && (ICU_USE_THREADS==1)
 # include <pthread.h> /* must be first, so that we get the multithread versions of things. */
 
 #endif /* POSIX && (ICU_USE_THREADS==1) */
 
-#ifdef U_WINDOWS
+#ifdef WIN32
 # define WIN32_LEAN_AND_MEAN
-# define VC_EXTRALEAN
+# define NOGDI
 # define NOUSER
 # define NOSERVICE
 # define NOIME
@@ -98,93 +84,34 @@
  *
  */ 
 
-#define  MAX_MUTEXES  40
-static UMTX              gGlobalMutex          = NULL;
-static UMTX              gIncDecMutex          = NULL;       
-#if (ICU_USE_THREADS == 1)
-static UBool             gMutexPoolInitialized = FALSE;
-static char              gMutexesInUse[MAX_MUTEXES];   
-
-#if defined(U_WINDOWS) 
-/*-------------------------------------------------------------
- *
- *   WINDOWS  platform variable declarations
- *
- *-------------------------------------------------------------*/
-static CRITICAL_SECTION  gMutexes[MAX_MUTEXES];
-static CRITICAL_SECTION  gGlobalWinMutex;
-
-
-/* On WIN32 mutexes are reentrant.  This makes it difficult to debug
- * deadlocking problems that show up on POSIXy platforms, where
- * mutexes deadlock upon reentry.  ICU contains checking code for
- * the global mutex as well as for other mutexes in the pool.
- *
- * This is for debugging purposes.
- *
- * This has no effect on non-WIN32 platforms, non-DEBUG builds, and
- * non-ICU_USE_THREADS builds.
- *
- * Note: The CRITICAL_SECTION structure already has a RecursionCount
- * member that can be used for this purpose, but portability to
- * Win98/NT/2K needs to be tested before use.  Works fine on XP.
- * After portability is confirmed, the built-in RecursionCount can be
- * used, and the gRecursionCountPool can be removed.
- *
- * Note: Non-global mutex checking only happens if there is no custom
- * pMutexLockFn defined.  Use one function, not two (don't use
- * pMutexLockFn and pMutexUnlockFn) so the increment and decrement of
- * the recursion count don't get out of sync.  Users might set just
- * one function, e.g., to perform a custom action, followed by a
- * standard call to EnterCriticalSection.
- */
-#if defined(U_DEBUG) && (ICU_USE_THREADS==1)
-static int32_t gRecursionCount = 0; /* detect global mutex locking */      
-static int32_t gRecursionCountPool[MAX_MUTEXES]; /* ditto for non-global */
-#endif
-
+/* The global ICU mutex.   */
+#if defined(WIN32) 
+static UMTX              gGlobalMutex      = NULL;
 
 #elif defined(POSIX) 
-/*-------------------------------------------------------------
- *
- *   POSIX   platform variable declarations
- *
- *-------------------------------------------------------------*/
-static pthread_mutex_t   gMutexes[MAX_MUTEXES] = {
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER,
-    PTHREAD_MUTEX_INITIALIZER, PTHREAD_MUTEX_INITIALIZER
-};
+#if (ICU_USE_THREADS == 1)
+static pthread_mutex_t   gGlobalPosixMutex = PTHREAD_MUTEX_INITIALIZER;
+static UMTX              gGlobalMutex      = &gGlobalPosixMutex;
+static UMTX              gIncDecMutex      = NULL;
+#else
+static UMTX              gGlobalMutex      = NULL;
+static UMTX              gIncDecMutex      = NULL;
+#endif  
 
 #else 
-/*-------------------------------------------------------------
- *
- *   UNKNOWN   platform  declarations
- *
- *-------------------------------------------------------------*/
-static void *gMutexes[MAX_MUTEXES] = {
-    NULL, NULL, NULL,
-    NULL, NULL, NULL,
-    NULL, NULL, NULL,
-    NULL, NULL, NULL,
-    NULL, NULL, NULL,
-    NULL, NULL, NULL,
-    NULL, NULL };
-
 /* Unknown platform.  OK so long as ICU_USE_THREAD is not set.  
                       Note that user can still set mutex functions at run time,
                       and that the global mutex variable is still needed in that case. */
+static UMTX              gGlobalMutex      = NULL;
 #if (ICU_USE_THREADS == 1)
 #error no ICU mutex implementation for this platform
 #endif
 #endif
-#endif /* ICU_USE_THREADS==1 */
-
 
+/* Detect Recursive locking of the global mutex.  For debugging only. */
+#if defined(WIN32) && defined(_DEBUG) && (ICU_USE_THREADS==1)
+static int32_t gRecursionCount = 0;       
+#endif
 
 
 /*
@@ -220,7 +147,7 @@ umtx_lock(UMTX *mutex)
     } else {
 
 #if (ICU_USE_THREADS == 1)
-#if defined(U_WINDOWS)
+#if defined(WIN32)
         EnterCriticalSection((CRITICAL_SECTION*) *mutex);
 #elif defined(POSIX)
         pthread_mutex_lock((pthread_mutex_t*) *mutex);
@@ -228,27 +155,12 @@ umtx_lock(UMTX *mutex)
 #endif /* ICU_USE_THREADS==1 */
     }
 
-#if defined(U_WINDOWS) && defined(U_DEBUG) && (ICU_USE_THREADS==1)
-    if (mutex == &gGlobalMutex) {         /* Detect Reentrant locking of the global mutex.      */
-        gRecursionCount++;                /* Recursion causes deadlocks on Unixes.              */
-        U_ASSERT(gRecursionCount == 1);   /* Detection works on Windows.  Debug problems there. */
-    }
-    /* This handles gGlobalMutex too, but only if there is no pMutexLockFn */
-    else if (pMutexLockFn == NULL) { /* see comments above */
-        size_t i = ((CRITICAL_SECTION*)*mutex) - &gMutexes[0];
-        U_ASSERT(i >= 0 && i < MAX_MUTEXES);
-        ++gRecursionCountPool[i];
-
-        U_ASSERT(gRecursionCountPool[i] == 1); /* !Detect Deadlock! */
-
-        /* This works and is fast, but needs testing on Win98/NT/2K.
-           See comments above. [alan]
-          U_ASSERT((CRITICAL_SECTION*)*mutex >= &gMutexes[0] &&
-                   (CRITICAL_SECTION*)*mutex <= &gMutexes[MAX_MUTEXES]);
-          U_ASSERT(((CRITICAL_SECTION*)*mutex)->RecursionCount == 1);
-        */
-    }
-#endif /*U_DEBUG*/
+#if defined(WIN32) && defined(_DEBUG) && (ICU_USE_THREADS==1)
+        if (mutex == &gGlobalMutex) {         /* Detect Reentrant locking of the global mutex.      */
+            gRecursionCount++;                /* Recursion causes deadlocks on Unixes.              */
+            U_ASSERT(gRecursionCount == 1);   /* Detection works on Windows.  Debug problems there. */
+        }
+#endif /*_DEBUG*/
 }
 
 
@@ -264,40 +176,22 @@ umtx_unlock(UMTX* mutex)
     }
 
     if(*mutex == NULL)    {
-#if (ICU_USE_THREADS == 1)
         U_ASSERT(FALSE);  /* This mutex is not initialized.     */
-#endif
         return; 
     }
 
-#if defined (U_WINDOWS) && defined (U_DEBUG) && (ICU_USE_THREADS==1)
+#if defined (WIN32) && defined (_DEBUG) && (ICU_USE_THREADS==1)
     if (mutex == &gGlobalMutex) {
         gRecursionCount--;
         U_ASSERT(gRecursionCount == 0);  /* Detect unlock of an already unlocked mutex */
     }
-    /* This handles gGlobalMutex too, but only if there is no pMutexLockFn */
-    else if (pMutexLockFn == NULL) { /* see comments above */
-        size_t i = ((CRITICAL_SECTION*)*mutex) - &gMutexes[0];
-        U_ASSERT(i >= 0 && i < MAX_MUTEXES);
-        --gRecursionCountPool[i];
-
-        U_ASSERT(gRecursionCountPool[i] == 0); /* !Detect Deadlock! */
-
-        /* This works and is fast, but needs testing on Win98/NT/2K.
-           Note that RecursionCount will be 1, not 0, since we haven't
-           left the CRITICAL_SECTION yet.  See comments above. [alan]
-          U_ASSERT((CRITICAL_SECTION*)*mutex >= &gMutexes[0] &&
-                   (CRITICAL_SECTION*)*mutex <= &gMutexes[MAX_MUTEXES]);
-          U_ASSERT(((CRITICAL_SECTION*)*mutex)->RecursionCount == 1);
-        */
-    }
 #endif
 
     if (pMutexUnlockFn) {
         (*pMutexUnlockFn)(gMutexContext, mutex);
     } else {
 #if (ICU_USE_THREADS==1)
-#if defined (U_WINDOWS)
+#if defined (WIN32)
         LeaveCriticalSection((CRITICAL_SECTION*)*mutex);
 #elif defined (POSIX)
         pthread_mutex_unlock((pthread_mutex_t*)*mutex);
@@ -308,6 +202,50 @@ umtx_unlock(UMTX* mutex)
 
 
 
+/*
+ *   umtx_raw_init    Do the platform specific mutex allocation and initialization
+ *                    for all ICU mutexes _except_ the ICU global mutex.
+ */
+static void umtx_raw_init(UMTX *mutex) {
+    if (pMutexInitFn != NULL) {
+        UErrorCode status = U_ZERO_ERROR;
+        (*pMutexInitFn)(gMutexContext, mutex, &status);
+        if (U_FAILURE(status)) {
+            /* TODO:  how should errors here be handled? */
+            return;
+        }
+    } else {
+
+#if (ICU_USE_THREADS == 1)
+    #if defined (WIN32)
+        CRITICAL_SECTION *cs = uprv_malloc(sizeof(CRITICAL_SECTION));
+        if (cs == NULL) {
+            return;
+        }
+        InitializeCriticalSection(cs);
+        *mutex = cs;
+    #elif defined( POSIX )
+        pthread_mutex_t *m = (pthread_mutex_t *)uprv_malloc(sizeof(pthread_mutex_t));
+        if (m == NULL) {
+            return;
+        }
+        # if defined (HPUX_CMA)
+            pthread_mutex_init(m, pthread_mutexattr_default);
+        # else
+            pthread_mutex_init(m, NULL);
+        # endif
+        *mutex = m;
+    #endif /* cascade of platforms */
+#else  /* ICU_USE_THREADS */
+        *mutex = mutex;      /* With no threads, we must still set the mutex to
+                              * some non-null value to make the rest of the
+                              *   (not ifdefed) mutex code think that it is initialized.
+                              */
+#endif /* ICU_USE_THREADS */
+    }
+}
+
+
 
 /*
  *   initGlobalMutex    Do the platform specific initialization of the ICU global mutex.
@@ -317,8 +255,8 @@ umtx_unlock(UMTX* mutex)
  */
 static void initGlobalMutex() {
     /*
-     * If User Supplied mutex functions are in use
-     *    init the icu global mutex using them.  
+     * Call user mutex init function if one has been specified and the global mutex
+     *  is not already initialized.  
      */
     if (pMutexInitFn != NULL) {
         if (gGlobalMutex==NULL) {
@@ -336,49 +274,36 @@ static void initGlobalMutex() {
      *   Use default ICU mutex implementations.
      */
 #if (ICU_USE_THREADS == 1)
-    /*
-     *  for Windows, init the pool of critical sections that we
-     *    will use as needed for ICU mutexes.
-     */
-#if defined (U_WINDOWS)
-    if (gMutexPoolInitialized == FALSE) {
-        int i;
-        for (i=0; i<MAX_MUTEXES; i++) {
-            InitializeCriticalSection(&gMutexes[i]);
-#if defined (U_DEBUG)
-            gRecursionCountPool[i] = 0; /* see comments above */
+    #if defined (WIN32)
+    {
+        void              *t;
+        CRITICAL_SECTION  *ourCritSec = uprv_malloc(sizeof(CRITICAL_SECTION)); 
+        InitializeCriticalSection(ourCritSec);
+#if defined (InterlockedCompareExchangePointer) || defined (_WIN64)
+        t = InterlockedCompareExchangePointer(&gGlobalMutex, ourCritSec, NULL);
+#else
+        /* Note that the headers from Microsoft's WIndows SDK define InterlockedCompareExchangePointer
+         * for all platforms, but the old headers included with MSVC 6 do not.
+         */
+        t = (void *)InterlockedCompareExchange(&gGlobalMutex, ourCritSec, NULL);
 #endif
+        if (t != NULL) {
+            /* Some other thread stored into gGlobalMutex first.  Discard the critical
+             *  section we just created; the system will go with the other one.
+             */
+            DeleteCriticalSection(ourCritSec);
+            uprv_free(ourCritSec);
         }
-        gMutexPoolInitialized = TRUE;
     }
-#elif defined (U_DARWIN)
-    /* PTHREAD_MUTEX_INITIALIZER works, don't need to call pthread_mutex_init
-     * as below (which is subject to a race condition)
-     */
-    gMutexPoolInitialized = TRUE;
-#elif defined (POSIX)
-    /*  TODO:  experimental code.  Shouldn't need to explicitly init the mutexes. */
-    if (gMutexPoolInitialized == FALSE) {
-        int i;
-        for (i=0; i<MAX_MUTEXES; i++) {
-            pthread_mutex_init(&gMutexes[i], NULL);
-        }
-        gMutexPoolInitialized = TRUE;
-    }
-#endif 
-
-    /*
-     * for both Windows & POSIX, the first mutex in the array is used
-     *   for the ICU global mutex.
-     */
-    gGlobalMutex = &gMutexes[0];
-    gMutexesInUse[0] = 1;
-
+    #elif defined( POSIX )
+        /*  No Action Required.  Global mutex set up with C static initialization. */
+        U_ASSERT(gGlobalMutex == &gGlobalPosixMutex);
+    #endif /* cascade of platforms */
 #else  /* ICU_USE_THREADS */
-    gGlobalMutex = &gGlobalMutex;  /* With no threads, we must still set the mutex to
-                                    * some non-null value to make the rest of the
-                                    *   (not ifdefed) mutex code think that it is initialized.
-                                    */
+        gGlobalMutex = &gGlobalMutex;  /* With no threads, we must still set the mutex to
+                                        * some non-null value to make the rest of the
+                                        *   (not ifdefed) mutex code think that it is initialized.
+                                        */
 #endif /* ICU_USE_THREADS */
 }
 
@@ -392,41 +317,32 @@ umtx_init(UMTX *mutex)
     if (mutex == NULL || mutex == &gGlobalMutex) {
         initGlobalMutex();
     } else {
+        /* 
+         *  Thread safe initialization of mutexes other than the global one,
+         *  using the global mutex.
+         */  
+        UBool isInitialized; 
+        UMTX tMutex = NULL;
+
         umtx_lock(NULL);
-        if (*mutex != NULL) {
-            /* Another thread initialized this mutex first. */
-            umtx_unlock(NULL);
+        isInitialized = (*mutex != NULL);
+        umtx_unlock(NULL);
+        if (isInitialized) {  
             return;
         }
 
-        if (pMutexInitFn != NULL) {
-            UErrorCode status = U_ZERO_ERROR;
-            (*pMutexInitFn)(gMutexContext, mutex, &status);
-            /* TODO:  how to report failure on init? */
-            umtx_unlock(NULL);
-            return;
-        }
-        else {
-#if (ICU_USE_THREADS == 1)
-            /*  Search through our pool of pre-allocated mutexes for one that is not
-            *  already in use.    */
-            int i;
-            for (i=0; i<MAX_MUTEXES; i++) {
-                if (gMutexesInUse[i] == 0) {
-                    gMutexesInUse[i] = 1;
-                    *mutex = &gMutexes[i];
-                    break;
-                }
-            }
-#endif
+        umtx_raw_init(&tMutex);
+
+        umtx_lock(NULL);
+        if (*mutex == NULL) {
+            *mutex = tMutex;
+            tMutex = NULL;
         }
         umtx_unlock(NULL);
-
-#if (ICU_USE_THREADS == 1)
-        /* No more mutexes were available from our pre-allocated pool.  */
-        /*   TODO:  how best to deal with this?                    */
-        U_ASSERT(*mutex != NULL);
-#endif
+        
+        if (tMutex != NULL) {
+            umtx_destroy(&tMutex); 
+        }
     }
 }
 
@@ -448,32 +364,38 @@ umtx_destroy(UMTX *mutex) {
         return;
     }
 
-    /*  The life of the inc/dec mutex is tied to that of the global mutex.  */
+#if defined (POSIX)
+    /*  The life of the inc/dec mutex for POSIX is tied to that of the global mutex. */
     if (mutex == &gGlobalMutex) {
         umtx_destroy(&gIncDecMutex);
     }
+#endif
 
     if (pMutexDestroyFn != NULL) {
-        /* Mutexes are being managed by the app.  Call back to it for the destroy. */
         (*pMutexDestroyFn)(gMutexContext, mutex);
-    }
-    else {
+        *mutex = NULL;
+    } else {
 #if (ICU_USE_THREADS == 1)
-        /* Return this mutex to the pool of available mutexes, if it came from the
-         *  pool in the first place.
-         */
-        /* TODO use pointer math here, instead of iterating! */
-        int i;
-        for (i=0; i<MAX_MUTEXES; i++)  {
-            if (*mutex == &gMutexes[i]) {
-                gMutexesInUse[i] = 0;
-                break;
-            }
-        }
-#endif
+#if defined (WIN32)
+        DeleteCriticalSection((CRITICAL_SECTION*)*mutex);
+        uprv_free(*mutex);
+        *mutex = NULL;
+#elif defined (POSIX)
+        if (*mutex != &gGlobalPosixMutex) {
+            /* Only POSIX mutexes other than the ICU global mutex get destroyed. */
+            pthread_mutex_destroy((pthread_mutex_t*)*mutex);
+            uprv_free(*mutex);
+            *mutex = NULL;
+        } 
+#endif /* chain of platforms */
+#else  /* ICU_USE_THREADS==1 */
+           /* NO ICU Threads.  We still need to zero out the mutex pointer, so that
+            * it appears to be uninitialized     */
+        *mutex = NULL;
+#endif   /* ICU_USE_THREADS */
+        
     }
 
-    *mutex = NULL;
 }
 
 
@@ -521,7 +443,7 @@ u_setMutexFunctions(const void *context, UMtxInitFn *i, UMtxFn *d, UMtxFn *l, UM
 /* Pointers to user-supplied inc/dec functions.  Null if no funcs have been set.  */
 static UMtxAtomicFn  *pIncFn = NULL;
 static UMtxAtomicFn  *pDecFn = NULL;
-static const void *gIncDecContext  = NULL;
+static void *gIncDecContext  = NULL;
 
 
 U_CAPI int32_t U_EXPORT2
@@ -530,10 +452,8 @@ umtx_atomic_inc(int32_t *p)  {
     if (pIncFn) {
         retVal = (*pIncFn)(gIncDecContext, p);
     } else {
-        #if defined (U_WINDOWS) && ICU_USE_THREADS == 1
-            retVal = InterlockedIncrement((LONG*)p);
-        #elif defined(USE_MAC_OS_ATOMIC_INCREMENT)
-            retVal = OSAtomicIncrement32Barrier(p);
+        #if defined (WIN32) && ICU_USE_THREADS == 1
+            retVal = InterlockedIncrement(p);
         #elif defined (POSIX) && ICU_USE_THREADS == 1
             umtx_lock(&gIncDecMutex);
             retVal = ++(*p);
@@ -552,10 +472,8 @@ umtx_atomic_dec(int32_t *p) {
     if (pDecFn) {
         retVal = (*pDecFn)(gIncDecContext, p);
     } else {
-        #if defined (U_WINDOWS) && ICU_USE_THREADS == 1
-            retVal = InterlockedDecrement((LONG*)p);
-        #elif defined(USE_MAC_OS_ATOMIC_INCREMENT)
-            retVal = OSAtomicDecrement32Barrier(p);
+        #if defined (WIN32) && ICU_USE_THREADS == 1
+            retVal = InterlockedDecrement(p);
         #elif defined (POSIX) && ICU_USE_THREADS == 1
             umtx_lock(&gIncDecMutex);
             retVal = --(*p);
@@ -577,6 +495,7 @@ umtx_atomic_dec(int32_t *p) {
 U_CAPI void U_EXPORT2
 u_setAtomicIncDecFunctions(const void *context, UMtxAtomicFn *ip, UMtxAtomicFn *dp,
                                 UErrorCode *status) {
+    int32_t   testInt;
     if (U_FAILURE(*status)) {
         return;
     }
@@ -593,17 +512,12 @@ u_setAtomicIncDecFunctions(const void *context, UMtxAtomicFn *ip, UMtxAtomicFn *
 
     pIncFn = ip;
     pDecFn = dp;
-    gIncDecContext = context;
 
-#if !U_RELEASE
-    {
-        int32_t   testInt = 0;
-        U_ASSERT(umtx_atomic_inc(&testInt) == 1);     /* Sanity Check.    Do the functions work at all? */
-        U_ASSERT(testInt == 1);
-        U_ASSERT(umtx_atomic_dec(&testInt) == 0);
-        U_ASSERT(testInt == 0);
-    }
-#endif
+    testInt = 0;
+    U_ASSERT(umtx_atomic_inc(&testInt) == 1);     /* Sanity Check.    Do the functions work at all? */
+    U_ASSERT(testInt == 1);
+    U_ASSERT(umtx_atomic_dec(&testInt) == 0);
+    U_ASSERT(testInt == 0);
 }
 
 
@@ -620,28 +534,16 @@ U_CFUNC UBool umtx_cleanup(void) {
     pMutexLockFn    = NULL;
     pMutexUnlockFn  = NULL;
     gMutexContext   = NULL;
-    gGlobalMutex    = NULL;
-    pIncFn          = NULL;
-    pDecFn          = NULL;
-    gIncDecContext  = NULL;
-    gIncDecMutex    = NULL;
 
+    gGlobalMutex    = NULL;
+#if defined (POSIX)
+    gIncDecMutex    = NULL;
 #if (ICU_USE_THREADS == 1)
-    if (gMutexPoolInitialized) {
-        int i;
-        for (i=0; i<MAX_MUTEXES; i++) {
-            if (gMutexesInUse[i]) {
-#if defined (U_WINDOWS)
-                DeleteCriticalSection(&gMutexes[i]);
-#elif defined (POSIX)
-                pthread_mutex_destroy(&gMutexes[i]);
+    gGlobalMutex    = &gGlobalPosixMutex;
 #endif
-                gMutexesInUse[i] = 0;
-            }
-        }
-    }
-    gMutexPoolInitialized = FALSE;
 #endif
+    pIncFn         = NULL;
+    pDecFn         = NULL;
 
     return TRUE;
 }
