@@ -1,10 +1,10 @@
 /*
-*****************************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+********************************************************************************
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
-*****************************************************************************************
+********************************************************************************
 *
-* File BRKITER.H
+* File brkiter.h
 *
 * Modification History:
 *
@@ -14,20 +14,39 @@
 *   07/09/97    jfitz       Renamed BreakIterator and interface synced with JDK
 *   08/11/98    helena      Sync-up JDK1.2.
 *   01/13/2000  helena      Added UErrorCode parameter to createXXXInstance methods.
-*****************************************************************************************
+********************************************************************************
 */
 
 #ifndef BRKITER_H
 #define BRKITER_H
 
+#include "unicode/utypes.h"
 
+#if UCONFIG_NO_BREAK_ITERATION
+
+U_NAMESPACE_BEGIN
+
+/*
+ * Allow the declaration of APIs with pointers to BreakIterator
+ * even when break iteration is removed from the build.
+ */
+class BreakIterator;
+
+U_NAMESPACE_END
+
+#else
+
+#include "unicode/uobject.h"
 #include "unicode/unistr.h"
 #include "unicode/chariter.h"
 #include "unicode/locid.h"
-
+#include "unicode/ubrk.h"
+#include "unicode/strenum.h"
 
 U_NAMESPACE_BEGIN
 
+typedef const void* URegistryKey;
+
 /**
  * The BreakIterator class implements methods for finding the location
  * of boundaries in text. BreakIterator is an abstract base class.
@@ -63,13 +82,13 @@ U_NAMESPACE_BEGIN
  * <P>
  * Helper function to output text
  * <pre>
- * \code 
+ * \code
  *    void printTextRange( BreakIterator& iterator, int32_t start, int32_t end )
  *    {
  *        UnicodeString textBuffer, temp;
  *        CharacterIterator *strIter = iterator.createText();
  *        strIter->getText(temp);
- *        cout << " " << start << " " << end << " |" 
+ *        cout << " " << start << " " << end << " |"
  *             << temp.extractBetween(start, end, textBuffer)
  *             << "|" << endl;
  *        delete strIter;
@@ -147,7 +166,7 @@ U_NAMESPACE_BEGIN
  *           BreakIterator* boundary;
  *           UnicodeString stringToExamine("Aaa bbb ccc. Ddd eee fff.");
  *           cout << "Examining: " << stringToExamine << endl;
- * 
+ *
  *           //print each sentence in forward and reverse order
  *           boundary = BreakIterator::createSentenceInstance( Locale::US );
  *           boundary->setText(stringToExamine);
@@ -156,7 +175,7 @@ U_NAMESPACE_BEGIN
  *           cout << "----- backward: ----------" << endl;
  *           printEachBackward(*boundary);
  *           delete boundary;
- * 
+ *
  *           //print each word in order
  *           boundary = BreakIterator::createWordInstance();
  *           boundary->setText(stringToExamine);
@@ -171,14 +190,18 @@ U_NAMESPACE_BEGIN
  *           //print word at charpos 10
  *           cout << "----- at pos 10: ---------" << endl;
  *           printAt(*boundary, 10 );
- * 
+ *
  *           delete boundary;
  *       }
  * \endcode
  * </pre>
  */
-class U_COMMON_API BreakIterator {
+class U_COMMON_API BreakIterator : public UObject {
 public:
+    /**
+     *  destructor
+     *  @stable ICU 2.0
+     */
     virtual ~BreakIterator();
 
     /**
@@ -192,81 +215,76 @@ public:
      * BreakIterator, as the argument.  Text is considered the same if
      * it contains the same characters, it need not be the same
      * object, and styles are not considered.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const BreakIterator&) const = 0;
 
     /**
      * Returns the complement of the result of operator==
-     * @stable
+     * @param rhs The BreakIterator to be compared for inequality
+     * @return the complement of the result of operator==
+     * @stable ICU 2.0
      */
     UBool operator!=(const BreakIterator& rhs) const { return !operator==(rhs); }
 
     /**
      * Return a polymorphic copy of this object.  This is an abstract
      * method which subclasses implement.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual BreakIterator* clone(void) const = 0;
 
     /**
      * Return a polymorphic class ID for this object. Different subclasses
      * will return distinct unequal values.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UClassID getDynamicClassID(void) const = 0;
 
     /**
      * Return a CharacterIterator over the text being analyzed.
-     * @stable
+     * Changing the state of the returned iterator can have undefined consequences
+     * on the operation of the break iterator.  If you need to change it, clone it first.
+     * @stable ICU 2.0
      */
     virtual const CharacterIterator& getText(void) const = 0;
 
-#ifdef ICU_ENABLE_DEPRECATED_BREAKITERATOR
-    /**
-     * Get the text for which this object is finding the boundaries.
-     * @deprecated This will be removed after 2000-Dec-31. Use getText() instead.
-     */
-    virtual CharacterIterator* createText(void) const = 0;
-
     /**
      * Change the text over which this operates. The text boundary is
      * reset to the start.
-     * @deprecated Use setText(const UnicodeString&) instead.
-     */
-    virtual void  setText(const UnicodeString* it) = 0;
-#endif
-
-    /**
-     * Change the text over which this operates. The text boundary is
-     * reset to the start.
-     * @stable
+     * @param text The UnicodeString used to change the text.
+     * @stable ICU 2.0
      */
     virtual void  setText(const UnicodeString &text) = 0;
 
     /**
      * Change the text over which this operates. The text boundary is
      * reset to the start.
-     * @stable
+     * @param it The CharacterIterator used to change the text.
+     * @stable ICU 2.0
      */
     virtual void  adoptText(CharacterIterator* it) = 0;
 
     /**
      * DONE is returned by previous() and next() after all valid
      * boundaries have been returned.
-     * @stable
+     * @stable ICU 2.0
      */
+#ifdef U_CYGWIN
+    static U_COMMON_API const int32_t DONE;
+#else
     static const int32_t DONE;
+#endif
 
     /**
      * Return the index of the first character in the text being scanned.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t first(void) = 0;
 
     /**
      * Return the index immediately BEYOND the last character in the text being scanned.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t last(void) = 0;
 
@@ -274,7 +292,7 @@ public:
      * Return the boundary preceding the current boundary.
      * @return The character index of the previous text boundary or DONE if all
      * boundaries have been returned.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t previous(void) = 0;
 
@@ -282,15 +300,14 @@ public:
      * Return the boundary following the current boundary.
      * @return The character index of the next text boundary or DONE if all
      * boundaries have been returned.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t next(void) = 0;
 
     /**
-     * Return character index of the text boundary that was most recently
-     * returned by next(), previous(), first(), or last()
+     * Return character index of the current interator position within the text.
      * @return The boundary most recently returned.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t current(void) const = 0;
 
@@ -300,7 +317,7 @@ public:
      * the value BreakIterator.DONE
      * @param offset the offset to begin scanning.
      * @return The first boundary after the specified offset.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t following(int32_t offset) = 0;
 
@@ -310,15 +327,17 @@ public:
      * the value BreakIterator.DONE
      * @param offset the offset to begin scanning.
      * @return The first boundary before the specified offset.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t preceding(int32_t offset) = 0;
- 
+
     /**
      * Return true if the specfied position is a boundary position.
+     * As a side effect, the current position of the iterator is set
+     * to the first boundary position at or following the specified offset.
      * @param offset the offset to check.
      * @return True if "offset" is a boundary position.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UBool isBoundary(int32_t offset) = 0;
 
@@ -329,7 +348,7 @@ public:
      * and positive values move to later boundaries.
      * @return The index of the nth boundary from the current position, or
      * DONE if there are fewer than |n| boundaries in the specfied direction.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual int32_t next(int32_t n) = 0;
 
@@ -337,22 +356,22 @@ public:
      * Create BreakIterator for word-breaks using the given locale.
      * Returns an instance of a BreakIterator implementing word breaks.
      * WordBreak is useful for word selection (ex. double click)
-     * @param where the locale. 
+     * @param where the locale.
      * @param status the error code
-     * @return A BreakIterator for word-breaks.  The UErrorCode& status 
+     * @return A BreakIterator for word-breaks.  The UErrorCode& status
      * parameter is used to return status information to the user.
      * To check whether the construction succeeded or not, you should check
      * the value of U_SUCCESS(err).  If you wish more detailed information, you
      * can check for informational error results which still indicate success.
-     * U_USING_FALLBACK_ERROR indicates that a fall back locale was used.  For
+     * U_USING_FALLBACK_WARNING indicates that a fall back locale was used.  For
      * example, 'de_CH' was requested, but nothing was found there, so 'de' was
-     * used.  U_USING_DEFAULT_ERROR indicates that the default locale data was
+     * used.  U_USING_DEFAULT_WARNING indicates that the default locale data was
      * used; neither the requested locale nor any of its fall back locales
      * could be found.
      * The caller owns the returned object and is responsible for deleting it.
-     * @stable
+     * @stable ICU 2.0
      */
-    static BreakIterator* createWordInstance(const Locale& where, 
+    static BreakIterator* createWordInstance(const Locale& where,
                                                    UErrorCode& status);
 
     /**
@@ -361,89 +380,99 @@ public:
      * breaks are logically possible line breaks, actual line breaks are
      * usually determined based on display width.
      * LineBreak is useful for word wrapping text.
-     * @param where the locale. .
-     * @return A BreakIterator for line-breaks.  The UErrorCode& status 
+     * @param where the locale.
+     * @param status The error code.
+     * @return A BreakIterator for line-breaks.  The UErrorCode& status
      * parameter is used to return status information to the user.
      * To check whether the construction succeeded or not, you should check
      * the value of U_SUCCESS(err).  If you wish more detailed information, you
      * can check for informational error results which still indicate success.
-     * U_USING_FALLBACK_ERROR indicates that a fall back locale was used.  For
+     * U_USING_FALLBACK_WARNING indicates that a fall back locale was used.  For
      * example, 'de_CH' was requested, but nothing was found there, so 'de' was
-     * used.  U_USING_DEFAULT_ERROR indicates that the default locale data was
+     * used.  U_USING_DEFAULT_WARNING indicates that the default locale data was
      * used; neither the requested locale nor any of its fall back locales
      * could be found.
      * The caller owns the returned object and is responsible for deleting it.
-     * @stable
+     * @stable ICU 2.0
      */
-    static BreakIterator* createLineInstance(const Locale& where, 
+    static BreakIterator* createLineInstance(const Locale& where,
                                                    UErrorCode& status);
 
     /**
      * Create BreakIterator for character-breaks using specified locale
      * Returns an instance of a BreakIterator implementing character breaks.
      * Character breaks are boundaries of combining character sequences.
-     * @param where the locale. 
-     * @return A BreakIterator for character-breaks.  The UErrorCode& status 
+     * @param where the locale.
+     * @param status The error code.
+     * @return A BreakIterator for character-breaks.  The UErrorCode& status
      * parameter is used to return status information to the user.
      * To check whether the construction succeeded or not, you should check
      * the value of U_SUCCESS(err).  If you wish more detailed information, you
      * can check for informational error results which still indicate success.
-     * U_USING_FALLBACK_ERROR indicates that a fall back locale was used.  For
+     * U_USING_FALLBACK_WARNING indicates that a fall back locale was used.  For
      * example, 'de_CH' was requested, but nothing was found there, so 'de' was
-     * used.  U_USING_DEFAULT_ERROR indicates that the default locale data was
+     * used.  U_USING_DEFAULT_WARNING indicates that the default locale data was
      * used; neither the requested locale nor any of its fall back locales
      * could be found.
      * The caller owns the returned object and is responsible for deleting it.
-     * @stable
+     * @stable ICU 2.0
      */
-    static BreakIterator* createCharacterInstance(const Locale& where, 
+    static BreakIterator* createCharacterInstance(const Locale& where,
                                                         UErrorCode& status);
 
     /**
      * Create BreakIterator for sentence-breaks using specified locale
      * Returns an instance of a BreakIterator implementing sentence breaks.
-     * @param where the locale. 
-     * @return A BreakIterator for sentence-breaks.  The UErrorCode& status 
+     * @param where the locale.
+     * @param status The error code.
+     * @return A BreakIterator for sentence-breaks.  The UErrorCode& status
      * parameter is used to return status information to the user.
      * To check whether the construction succeeded or not, you should check
      * the value of U_SUCCESS(err).  If you wish more detailed information, you
      * can check for informational error results which still indicate success.
-     * U_USING_FALLBACK_ERROR indicates that a fall back locale was used.  For
+     * U_USING_FALLBACK_WARNING indicates that a fall back locale was used.  For
      * example, 'de_CH' was requested, but nothing was found there, so 'de' was
-     * used.  U_USING_DEFAULT_ERROR indicates that the default locale data was
+     * used.  U_USING_DEFAULT_WARNING indicates that the default locale data was
      * used; neither the requested locale nor any of its fall back locales
      * could be found.
      * The caller owns the returned object and is responsible for deleting it.
-     * @stable
+     * @stable ICU 2.0
      */
-    static BreakIterator* createSentenceInstance(const Locale& where, 
+    static BreakIterator* createSentenceInstance(const Locale& where,
                                                        UErrorCode& status);
 
     /**
      * Create BreakIterator for title-casing breaks using the specified locale
      * Returns an instance of a BreakIterator implementing title breaks.
-     * @param where the locale. 
-     * @return A BreakIterator for title-breaks.  The UErrorCode& status 
+     * The iterator returned locates title boundaries as described for 
+     * Unicode 3.2 only. For Unicode 4.0 and above title boundary iteration,
+     * please use Word Boundary iterator.{@link createWordInstance()}
+     *
+     * @param where the locale.
+     * @param status The error code.
+     * @return A BreakIterator for title-breaks.  The UErrorCode& status
      * parameter is used to return status information to the user.
      * To check whether the construction succeeded or not, you should check
      * the value of U_SUCCESS(err).  If you wish more detailed information, you
      * can check for informational error results which still indicate success.
-     * U_USING_FALLBACK_ERROR indicates that a fall back locale was used.  For
+     * U_USING_FALLBACK_WARNING indicates that a fall back locale was used.  For
      * example, 'de_CH' was requested, but nothing was found there, so 'de' was
-     * used.  U_USING_DEFAULT_ERROR indicates that the default locale data was
+     * used.  U_USING_DEFAULT_WARNING indicates that the default locale data was
      * used; neither the requested locale nor any of its fall back locales
      * could be found.
      * The caller owns the returned object and is responsible for deleting it.
-     * @stable
+     * @stable ICU 2.1
      */
-    static BreakIterator* createTitleInstance(const Locale& where, 
+    static BreakIterator* createTitleInstance(const Locale& where,
                                                        UErrorCode& status);
 
     /**
-     * Get the set of Locales for which TextBoundaries are installed
+     * Get the set of Locales for which TextBoundaries are installed.
+     * <p><b>Note:</b> this will not return locales added through the register
+     * call.</p>
      * @param count the output parameter of number of elements in the locale list
      * @return available locales
-     * @stable
+     * @stable ICU 2.0
      */
     static const Locale* getAvailableLocales(int32_t& count);
 
@@ -454,7 +483,7 @@ public:
      * @param name the fill-in parameter of the return value
      * Uses best match.
      * @return user-displayable name
-     * @stable
+     * @stable ICU 2.0
      */
     static UnicodeString& getDisplayName(const Locale& objectLocale,
                                          const Locale& displayLocale,
@@ -466,7 +495,7 @@ public:
      * @param objectLocale must be from getMatchingLocales
      * @param name the fill-in parameter of the return value
      * @return user-displayable name
-     * @stable
+     * @stable ICU 2.0
      */
     static UnicodeString& getDisplayName(const Locale& objectLocale,
                                          UnicodeString& name);
@@ -474,37 +503,112 @@ public:
     /**
      * Thread safe client-buffer-based cloning operation
      *    Do NOT call delete on a safeclone, since 'new' is not used to create it.
-     * @param stackBuffer user allocated space for the new clone. If NULL new memory will be allocated. 
+     * @param stackBuffer user allocated space for the new clone. If NULL new memory will be allocated.
      * If buffer is not large enough, new memory will be allocated.
-     * @param BufferSize reference to size of allocated space. 
-     * If BufferSize == 0, a sufficient size for use in cloning will 
+     * @param BufferSize reference to size of allocated space.
+     * If BufferSize == 0, a sufficient size for use in cloning will
      * be returned ('pre-flighting')
-     * If BufferSize is not enough for a stack-based safe clone, 
+     * If BufferSize is not enough for a stack-based safe clone,
      * new memory will be allocated.
      * @param status to indicate whether the operation went on smoothly or there were errors
-     *  An informational status value, U_SAFECLONE_ALLOCATED_ERROR, is used if any allocations were 
+     *  An informational status value, U_SAFECLONE_ALLOCATED_ERROR, is used if any allocations were
      *  necessary.
      * @return pointer to the new clone
-     *  
-     * @draft ICU 1.8
+     *
+     * @stable ICU 2.0
      */
     virtual BreakIterator *  createBufferClone(void *stackBuffer,
                                                int32_t &BufferSize,
                                                UErrorCode &status) = 0;
 
+    /**
+     *   Determine whether the BreakIterator was created in user memory by
+     *   createBufferClone(), and thus should not be deleted.  Such objects
+     *   must be closed by an explicit call to the destructor (not delete).
+     *  @stable ICU 2.0
+     */
     inline UBool isBufferClone(void);
 
+    /**
+     * Register a new break iterator of the indicated kind, to use in the given locale.
+     * The break iterator will be adopted.  Clones of the iterator will be returned
+     * if a request for a break iterator of the given kind matches or falls back to
+     * this locale.
+     * @param toAdopt the BreakIterator instance to be adopted
+     * @param locale the Locale for which this instance is to be registered
+     * @param kind the type of iterator for which this instance is to be registered
+     * @param status the in/out status code, no special meanings are assigned
+     * @return a registry key that can be used to unregister this instance
+     * @draft ICU 2.4
+     */
+    static URegistryKey registerInstance(BreakIterator* toAdopt, const Locale& locale, UBreakIteratorType kind, UErrorCode& status);
+
+    /**
+     * Unregister a previously-registered BreakIterator using the key returned from the
+     * register call.  Key becomes invalid after a successful call and should not be used again.
+     * The BreakIterator corresponding to the key will be deleted.
+     * @param key the registry key returned by a previous call to registerInstance
+     * @param status the in/out status code, no special meanings are assigned
+     * @return TRUE if the iterator for the key was successfully unregistered
+     * @draft ICU 2.4
+     */
+    static UBool unregister(URegistryKey key, UErrorCode& status);
+
+    /**
+     * Return a StringEnumeration over the locales available at the time of the call, 
+     * including registered locales.
+     * @return a StringEnumeration over the locales available at the time of the call
+     * @draft ICU 2.4
+     */
+    static StringEnumeration* getAvailableLocales(void);
+
+    /**
+     * Returns the locale for this break iterator. Two flavors are available: valid and 
+     * actual locale. 
+     * @draft ICU 2.8 likely to change in ICU 3.0, based on feedback
+     */
+    Locale getLocale(ULocDataLocaleType type, UErrorCode& status) const;
+
+    /** Get the locale for this break iterator object. You can choose between valid and actual locale.
+     *  @param type type of the locale we're looking for (valid or actual) 
+     *  @param status error code for the operation
+     *  @return the locale
+     *  @internal
+     */
+    const char *getLocaleID(ULocDataLocaleType type, UErrorCode& status) const;
+
+ private:
+    static BreakIterator* makeCharacterInstance(const Locale& loc, UErrorCode& status);
+    static BreakIterator* makeWordInstance(const Locale& loc, UErrorCode& status);
+    static BreakIterator* makeLineInstance(const Locale& loc, UErrorCode& status);
+    static BreakIterator* makeSentenceInstance(const Locale& loc, UErrorCode& status);
+    static BreakIterator* makeTitleInstance(const Locale& loc, UErrorCode& status);
+
+    static BreakIterator* createInstance(const Locale& loc, UBreakIteratorType kind, UErrorCode& status);
+    static BreakIterator* makeInstance(const Locale& loc, int32_t kind, UErrorCode& status);
+
+    friend class ICUBreakIteratorFactory;
+    friend class ICUBreakIteratorService;
 
 protected:
+    /** @internal */
     BreakIterator();
+    /** @internal */
     UBool fBufferClone;
+    /** @internal */
+    BreakIterator (const BreakIterator &other) : UObject(other), fBufferClone(FALSE) {}
+
 private:
+
+    /** @internal */
+    char actualLocale[ULOC_FULLNAME_CAPACITY];
+    char validLocale[ULOC_FULLNAME_CAPACITY];
+
     /**
-     * The copy constructor and assignment operator have no real implementation.
-     * They are provided to make the compiler happy. Do not call.
+     * The assignment operator has no real implementation.
+     * It's provided to make the compiler happy. Do not call.
      */
     BreakIterator& operator=(const BreakIterator&) { return *this; }
-    BreakIterator (const BreakIterator&) {}
 };
 
 inline UBool BreakIterator::isBufferClone()
@@ -514,6 +618,8 @@ inline UBool BreakIterator::isBufferClone()
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_BREAK_ITERATION */
+
 #endif // _BRKITER
 //eof
 
