@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -42,27 +42,97 @@
  * functions may have to be re-implemented.
  */
 
-/* Floating point utilities 
+/**
+ * Floating point utility to determine if a double is Not a Number (NaN).
+ * @internal
+ */
+U_CAPI UBool   U_EXPORT2 uprv_isNaN(double d);
+/**
+ * Floating point utility to determine if a double has an infinite value.
+ * @internal
+ */
+U_CAPI UBool   U_EXPORT2 uprv_isInfinite(double d);
+/**
+ * Floating point utility to determine if a double has a positive infinite value.
+ * @internal
+ */
+U_CAPI UBool   U_EXPORT2 uprv_isPositiveInfinity(double d);
+/**
+ * Floating point utility to determine if a double has a negative infinite value.
+ * @internal
+ */
+U_CAPI UBool   U_EXPORT2 uprv_isNegativeInfinity(double d);
+/**
+ * Floating point utility that returns a Not a Number (NaN) value.
+ * @internal
  */
-U_CAPI UBool   U_EXPORT2 uprv_isNaN(double);
-U_CAPI UBool   U_EXPORT2 uprv_isInfinite(double);
-U_CAPI UBool   U_EXPORT2 uprv_isPositiveInfinity(double);
-U_CAPI UBool   U_EXPORT2 uprv_isNegativeInfinity(double);
 U_CAPI double  U_EXPORT2 uprv_getNaN(void);
+/**
+ * Floating point utility that returns an infinite value.
+ * @internal
+ */
 U_CAPI double  U_EXPORT2 uprv_getInfinity(void);
 
+/**
+ * Floating point utility to truncate a double.
+ * @internal
+ */
 U_CAPI double  U_EXPORT2 uprv_trunc(double d);
-U_CAPI double  U_EXPORT2 uprv_floor(double x);
-U_CAPI double  U_EXPORT2 uprv_ceil(double x);
-U_CAPI double  U_EXPORT2 uprv_fabs(double x);
-U_CAPI double  U_EXPORT2 uprv_modf(double x, double* y);
-U_CAPI double  U_EXPORT2 uprv_fmod(double x, double y);
-U_CAPI double  U_EXPORT2 uprv_pow(double x, double y);
-U_CAPI double  U_EXPORT2 uprv_pow10(int32_t x);
-U_CAPI double  U_EXPORT2 uprv_fmax(double x, double y);
-U_CAPI double  U_EXPORT2 uprv_fmin(double x, double y);
-U_CAPI int32_t U_EXPORT2 uprv_max(int32_t x, int32_t y);
-U_CAPI int32_t U_EXPORT2 uprv_min(int32_t x, int32_t y);
+/**
+ * Floating point utility to calculate the floor of a double.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_floor(double d);
+/**
+ * Floating point utility to calculate the ceiling of a double.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_ceil(double d);
+/**
+ * Floating point utility to calculate the absolute value of a double.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_fabs(double d);
+/**
+ * Floating point utility to calculate the fractional and integer parts of a double.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_modf(double d, double* pinteger);
+/**
+ * Floating point utility to calculate the remainder of a double divided by another double.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_fmod(double d, double y);
+/**
+ * Floating point utility to calculate d to the power of exponent (d^exponent).
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_pow(double d, double exponent);
+/**
+ * Floating point utility to calculate 10 to the power of exponent (10^exponent).
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_pow10(int32_t exponent);
+/**
+ * Floating point utility to calculate the maximum value of two doubles.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_fmax(double d, double y);
+/**
+ * Floating point utility to calculate the minimum value of two doubles.
+ * @internal
+ */
+U_CAPI double  U_EXPORT2 uprv_fmin(double d, double y);
+/**
+ * Private utility to calculate the maximum value of two integers.
+ * @internal
+ */
+U_CAPI int32_t U_EXPORT2 uprv_max(int32_t d, int32_t y);
+/**
+ * Private utility to calculate the minimum value of two integers.
+ * @internal
+ */
+U_CAPI int32_t U_EXPORT2 uprv_min(int32_t d, int32_t y);
 
 #if U_IS_BIG_ENDIAN
 #   define uprv_isNegative(number) (*((signed char *)&(number))<0)
@@ -73,6 +143,7 @@ U_CAPI int32_t U_EXPORT2 uprv_min(int32_t x, int32_t y);
 /**
  * Return the largest positive number that can be represented by an integer
  * type of arbitrary bit length.
+ * @internal
  */
 U_CAPI double  U_EXPORT2 uprv_maxMantissa(void);
 
@@ -85,18 +156,30 @@ U_CAPI double  U_EXPORT2 uprv_maxMantissa(void);
  *
  * @param d the double value to apply the common log function for.
  * @return the log of value d.
+ * @internal
  */
 U_CAPI int16_t  U_EXPORT2 uprv_log10(double d);
 
+/**
+ * Floating point utility to calculate the logarithm of a double.
+ * @internal
+ */
 U_CAPI double  U_EXPORT2 uprv_log(double d);
 
-/** Does common notion of rounding e.g. uprv_floor(x + 0.5); */
+/**
+ * Does common notion of rounding e.g. uprv_floor(x + 0.5);
+ * @param x the double number
+ * @return the rounded double
+ * @internal
+ */
 U_CAPI double  U_EXPORT2 uprv_round(double x);
 
 /**
  * Returns the number of digits after the decimal point in a double number x.
  *
  * @param x the double number
+ * @return the number of digits after the decimal point in a double number x.
+ * @internal
  */
 U_CAPI int32_t  U_EXPORT2 uprv_digitsAfterDecimal(double x);
 
@@ -130,13 +213,15 @@ U_CAPI int32_t  U_EXPORT2 uprv_digitsAfterDecimal(double x);
  * information specified by the operating system. Under Windows NT
  * and Windows 95, this information is specified in the Control Panels
  * Date/Time application.
- * @stable
+ * @internal
  */
 U_CAPI void     U_EXPORT2 uprv_tzset(void);
 
 /**
  * Difference in seconds between coordinated universal
  * time and local time. E.g., -28,800 for PST (GMT-8hrs)
+ * @return the difference in seconds between coordinated universal time and local time.
+ * @internal
  */
 U_CAPI int32_t  U_EXPORT2 uprv_timezone(void);
 
@@ -146,12 +231,14 @@ U_CAPI int32_t  U_EXPORT2 uprv_timezone(void);
  *   tzname(1)  Three-letter DST zone name derived from TZ environment
  *              variable.  E.g., "PDT". If DST zone is omitted from TZ,
  *              tzname(1) is an empty string.
+ * @internal
  */
 U_CAPI char*    U_EXPORT2 uprv_tzname(int n);
 
 /**
- * Get UTC (GMT) time measured in seconds since 0:00 on 1/1/70. 
- * @stable
+ * Get UTC (GMT) time measured in seconds since 0:00 on 1/1/70.
+ * @return the UTC time measured in seconds 
+ * @internal
  */
 U_CAPI int32_t  U_EXPORT2 uprv_getUTCtime(void);
 
@@ -166,15 +253,14 @@ U_CAPI int32_t  U_EXPORT2 uprv_getUTCtime(void);
  * The data directory is determined as follows:
  *    If u_setDataDirectory() has been called, that is it, otherwise
  *    if the ICU_DATA environment variable is set, use that, otherwise
- *    On Windows, if the SOFTWARE\ICU\Unicode\Data registry entry exists, use that
- *           (use of the registry in this way is not recommended.)  otherwise
- *    If a data directory was specifed at ICU build time, use that
+ *    If a data directory was specifed at ICU build time
+ *      (#define ICU_DATA_DIR "path"), use that,
  *    otherwise no data directory is available.
  *
  * @return the data directory, or an empty string ("") if no data directory has
  *         been specified.
  *   
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI const char* U_EXPORT2 u_getDataDirectory(void);
 
@@ -187,55 +273,64 @@ U_CAPI const char* U_EXPORT2 u_getDataDirectory(void);
  *   does not need to be set.
  *
  * This function should be called at most once in a process, before the
- * first ICU operation that will require the loading of an ICU data file.
+ * first ICU operation (e.g., u_init()) that will require the loading of an
+ * ICU data file.
+ * This function is not thread-safe. Use it before calling ICU APIs from
+ * multiple threads.
  *
- * @stable
+ * @param directory The directory to be set.
+ *
+ * @see u_init
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 u_setDataDirectory(const char *directory);
 
 /**
- * Return the default codepage for this platform and locale 
- * @stable
+ * Please use ucnv_getDefaultName() instead.
+ * Return the default codepage for this platform and locale.
+ * This function can call setlocale() on Unix platforms. Please read the
+ * platform documentation on setlocale() before calling this function.
+ * @return the default codepage for this platform 
+ * @internal
  */
 U_CAPI const char*  U_EXPORT2 uprv_getDefaultCodepage(void);
 
 /**
+ * Please use uloc_getDefault() instead.
  * Return the default locale ID string by querying ths system, or
  *     zero if one cannot be found. 
- * @stable
+ * This function can call setlocale() on Unix platforms. Please read the
+ * platform documentation on setlocale() before calling this function.
+ * @return the default locale ID string
+ * @internal
  */
 U_CAPI const char*  U_EXPORT2 uprv_getDefaultLocaleID(void);
 
-/*
- * Finds the least double greater than d (if positive == true),
- * or the greatest double less than d (if positive == false).
- *
- * This is a special purpose function defined by the ChoiceFormat API
- * documentation.
- * It is not a general purpose function and not defined for NaN or Infinity
- * @deprecated This will be removed after 2002-Jun-30. Use the ChoiceFormat closures API instead.
- */
-U_CAPI double       U_EXPORT2 uprv_nextDouble(double d, UBool positive);
-
 /**
  * Filesystem file and path separator characters.
  * Example: '/' and ':' on Unix, '\\' and ';' on Windows.
- * @stable
+ * @stable ICU 2.0
  */
 #ifdef XP_MAC
 #   define U_FILE_SEP_CHAR ':'
+#   define U_FILE_ALT_SEP_CHAR ':'
 #   define U_PATH_SEP_CHAR ';'
 #   define U_FILE_SEP_STRING ":"
+#   define U_FILE_ALT_SEP_STRING ":"
 #   define U_PATH_SEP_STRING ";"
 #elif defined(WIN32) || defined(OS2)
 #   define U_FILE_SEP_CHAR '\\'
+#   define U_FILE_ALT_SEP_CHAR '/'
 #   define U_PATH_SEP_CHAR ';'
 #   define U_FILE_SEP_STRING "\\"
+#   define U_FILE_ALT_SEP_STRING "/"
 #   define U_PATH_SEP_STRING ";"
 #else
 #   define U_FILE_SEP_CHAR '/'
+#   define U_FILE_ALT_SEP_CHAR '/'
 #   define U_PATH_SEP_CHAR ':'
 #   define U_FILE_SEP_STRING "/"
+#   define U_FILE_ALT_SEP_STRING "/"
 #   define U_PATH_SEP_STRING ":"
 #endif
 
@@ -253,7 +348,7 @@ U_CAPI double       U_EXPORT2 uprv_nextDouble(double d, UBool positive);
  *           Unicode characters.
  * @param length The number of characters to convert; this may
  *               include the terminating <code>NUL</code>.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2
 u_charsToUChars(const char *cs, UChar *us, int32_t length);
@@ -273,12 +368,42 @@ u_charsToUChars(const char *cs, UChar *us, int32_t length);
  *           character bytes.
  * @param length The number of characters to convert; this may
  *               include the terminating <code>NUL</code>.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2
 u_UCharsToChars(const UChar *us, char *cs, int32_t length);
 
-/* Define U_UPPER_ORDINAL */
+/**
+ * Check if a char string only contains invariant characters.
+ * See utypes.h for details.
+ *
+ * @param s Input string pointer.
+ * @param length Length of the string, can be -1 if NUL-terminated.
+ * @return TRUE if s contains only invariant characters.
+ *
+ * @internal (ICU 2.8)
+ */
+U_CAPI UBool U_EXPORT2
+uprv_isInvariantString(const char *s, int32_t length);
+
+/**
+ * Check if a Unicode string only contains invariant characters.
+ * See utypes.h for details.
+ *
+ * @param s Input string pointer.
+ * @param length Length of the string, can be -1 if NUL-terminated.
+ * @return TRUE if s contains only invariant characters.
+ *
+ * @internal (ICU 2.8)
+ */
+U_CAPI UBool U_EXPORT2
+uprv_isInvariantUString(const UChar *s, int32_t length);
+
+/**
+ * \def U_UPPER_ORDINAL
+ * Get the ordinal number of an uppercase invariant character
+ * @internal
+ */
 #if U_CHARSET_FAMILY==U_ASCII_FAMILY
 #   define U_UPPER_ORDINAL(x) ((x)-'A')
 #elif U_CHARSET_FAMILY==U_EBCDIC_FAMILY
@@ -321,7 +446,7 @@ u_UCharsToChars(const UChar *us, char *cs, int32_t length);
  */
 #    define U_MAX_PTR(base) ((void *)(((char *)base)-((int32_t)(base))+((int32_t)0xffefff)))
 #  else
-#    define U_MAX_PTR(base) ((void *)(((char *)(base)+0x7fffffff) > (char *)(base) ? ((char *)(base)+0x7fffffff) : (char *)-1))
+#    define U_MAX_PTR(base) ((void *)(((char *)(base)+0x7fffffffu) > (char *)(base) ? ((char *)(base)+0x7fffffffu) : (char *)-1))
 #  endif
 #endif
 
