@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1998-2001, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -33,7 +33,7 @@ U_NAMESPACE_BEGIN
  * compare equal).
  * @see CharacterIterator
  * @see ForwardCharacterIterator
- * @stable
+ * @stable ICU 2.0
  */
 class U_COMMON_API StringCharacterIterator : public UCharCharacterIterator {
 public:
@@ -41,7 +41,8 @@ public:
    * Create an iterator over the UnicodeString referred to by "textStr".
    * The UnicodeString object is copied.
    * The iteration range is the whole string, and the starting position is 0.
-   * @stable
+   * @param textStr The unicode string used to create an iterator
+   * @stable ICU 2.0
    */
   StringCharacterIterator(const UnicodeString& textStr);
 
@@ -50,7 +51,9 @@ public:
    * The iteration range is the whole string, and the starting
    * position is specified by "textPos".  If "textPos" is outside the valid
    * iteration range, the behavior of this object is undefined.  
-   * @stable
+   * @textStr The unicode string used to create an iterator 
+   * @textPos The starting position of the iteration
+   * @stable ICU 2.0
    */
   StringCharacterIterator(const UnicodeString&    textStr,
               int32_t              textPos);
@@ -65,7 +68,11 @@ public:
    * textBegin >= textEnd or either is negative or greater than text.size()),
    * or "textPos" is outside the range defined by "textBegin" and "textEnd",
    * the behavior of this iterator is undefined.  
-   * @stable
+   * @param textStr    The unicode string used to create the StringCharacterIterator
+   * @param textBegin  The begin position of the iteration range
+   * @param textEnd    The end position of the iteration range
+   * @param textPos    The starting position of the iteration
+   * @stable ICU 2.0
    */
   StringCharacterIterator(const UnicodeString&    textStr,
               int32_t              textBegin,
@@ -77,21 +84,24 @@ public:
    * of the same string as "that", and its initial position is the
    * same as "that"'s current position.  
    * The UnicodeString object in "that" is copied.
-   * @stable
+   * @param that The StringCharacterIterator to be copied
+   * @stable ICU 2.0
    */
   StringCharacterIterator(const StringCharacterIterator&  that);
 
   /**
    * Destructor.  
-   * @stable
+   * @stable ICU 2.0
    */
   virtual ~StringCharacterIterator();
 
   /**
    * Assignment operator.  *this is altered to iterate over the same
    * range of the same string as "that", and refers to the same
-   * character within that string as "that" does.  
-   * @stable
+   * character within that string as "that" does.
+   * @param that The object to be copied.
+   * @return the newly created object.  
+   * @stable ICU 2.0
    */
   StringCharacterIterator&
   operator=(const StringCharacterIterator&    that);
@@ -99,21 +109,26 @@ public:
   /**
    * Returns true if the iterators iterate over the same range of the
    * same string and are pointing at the same character.  
-   * @stable
+   * @param that The ForwardCharacterIterator to be compared for equality
+   * @return true if the iterators iterate over the same range of the
+   * same string and are pointing at the same character.  
+   * @stable ICU 2.0
    */
   virtual UBool          operator==(const ForwardCharacterIterator& that) const;
 
   /**
    * Returns a new StringCharacterIterator referring to the same
    * character in the same range of the same string as this one.  The
-   * caller must delete the new iterator.  
-   * @stable
+   * caller must delete the new iterator. 
+   * @return the newly cloned object. 
+   * @stable ICU 2.0
    */
   virtual CharacterIterator* clone(void) const;
                                 
   /**
    * Sets the iterator to iterate over the provided string.
-   * @stable
+   * @param newText The string to be iterated over
+   * @stable ICU 2.0
    */
   void setText(const UnicodeString& newText);
 
@@ -122,31 +137,45 @@ public:
    * referred to by "result".  Even if this iterator iterates across
    * only a part of this string, the whole string is copied.
    * @param result Receives a copy of the text under iteration.  
-   * @stable
+   * @stable ICU 2.0
    */
   virtual void            getText(UnicodeString& result);
 
   /**
    * Return a class ID for this object (not really public) 
-   * @stable
+   * @return a class ID for this object.
+   * @stable ICU 2.0
    */
-  virtual UClassID         getDynamicClassID(void) const 
-    { return getStaticClassID(); }
+  virtual UClassID         getDynamicClassID(void) const;
 
   /**
    * Return a class ID for this class (not really public) 
-   * @stable
+   * @return a class ID for this class
+   * @stable ICU 2.0
    */
-  static UClassID          getStaticClassID(void) 
-    { return (UClassID)(&fgClassID); }
+  static UClassID   getStaticClassID(void);
 
 protected:
+  /**
+   * Default constructor, iteration over empty string.
+   * @stable ICU 2.0
+   */
   StringCharacterIterator();
+
+  /**
+   * Sets the iterator to iterate over the provided string.
+   * @param newText The string to be iterated over
+   * @param newTextLength The length of the String
+   * @stable ICU 2.0
+   */
   void setText(const UChar* newText, int32_t newTextLength);
-        
+
+  /**
+   * Copy of the iterated string object.
+   * @stable ICU 2.0
+   */
   UnicodeString            text;
-private:
-  static const char        fgClassID;
+
 };
 
 U_NAMESPACE_END
