@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 1996-2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 1996-2003, International Business Machines Corporation and others. All Rights Reserved.
 *****************************************************************************************
 */
 
@@ -7,6 +7,25 @@
 #define UBRK_H
 
 #include "unicode/utypes.h"
+#include "unicode/uloc.h"
+
+/**
+ * A text-break iterator.
+ *  For usage in C programs.
+ */
+#ifndef UBRK_TYPEDEF_UBREAK_ITERATOR
+#   define UBRK_TYPEDEF_UBREAK_ITERATOR
+    /**
+     *  Opaque type representing an ICU Break iterator object.
+     *  @stable ICU 2.0
+     */
+    typedef void UBreakIterator;
+#endif
+
+#if !UCONFIG_NO_BREAK_ITERATION
+
+#include "unicode/parseerr.h"
+
 /**
  * \file
  * \brief C API: BreakIterator
@@ -45,7 +64,7 @@
  * typically starts of words, that should be set to Title Case
  * when title casing the text.
  * <P>
- * 
+ *
  * This is the interface for all text boundaries.
  * <P>
  * Examples:
@@ -164,35 +183,115 @@
  * </pre>
  */
 
-/**
- * A text-break iterator.
- *  For usage in C programs.
- */
-#ifndef UBRK_TYPEDEF_UBREAK_ITERATOR
-#   define UBRK_TYPEDEF_UBREAK_ITERATOR
-    typedef void* UBreakIterator;
-#endif
-
-/** The possible types of text boundaries. */
-enum UBreakIteratorType {
-  /** Character breaks */
+/** The possible types of text boundaries.  @stable ICU 2.0 */
+typedef enum UBreakIteratorType {
+  /** Character breaks  @stable ICU 2.0 */
   UBRK_CHARACTER,
-  /** Word breaks */
+  /** Word breaks @stable ICU 2.0 */
   UBRK_WORD,
-  /** Line breaks */
+  /** Line breaks @stable ICU 2.0 */
   UBRK_LINE,
-  /** Sentence breaks */
+  /** Sentence breaks @stable ICU 2.0 */
   UBRK_SENTENCE,
-  /** Title Case breaks */
+  /** 
+   * Title Case breaks 
+   * The iterator created using this type locates title boundaries as described for 
+   * Unicode 3.2 only. For Unicode 4.0 and above title boundary iteration,
+   * please use Word Boundary iterator.
+   *
+   * @deprecated ICU 2.8 Use the word break iterator for titlecasing for Unicode 4 and later.
+   */
   UBRK_TITLE
-};
-typedef enum UBreakIteratorType UBreakIteratorType;
+} UBreakIteratorType;
 
 /** Value indicating all text boundaries have been returned.
- *
+ *  @stable ICU 2.0 
  */
 #define UBRK_DONE ((int32_t) -1)
 
+
+/**
+ *  Enum constants for the word break tags returned by
+ *  getRuleStatus().  A range of values is defined for each category of
+ *  word, to allow for further subdivisions of a category in future releases.
+ *  Applications should check for tag values falling within the range, rather
+ *  than for single individual values.
+ *  @stable ICU 2.2
+*/
+typedef enum UWordBreak {
+    /** Tag value for "words" that do not fit into any of other categories. 
+     *  Includes spaces and most punctuation. */
+    UBRK_WORD_NONE           = 0,
+    /** Upper bound for tags for uncategorized words. */
+    UBRK_WORD_NONE_LIMIT     = 100,
+    /** Tag value for words that appear to be numbers, lower limit.    */
+    UBRK_WORD_NUMBER         = 100,
+    /** Tag value for words that appear to be numbers, upper limit.    */
+    UBRK_WORD_NUMBER_LIMIT   = 200,
+    /** Tag value for words that contain letters, excluding
+     *  hiragana, katakana or ideographic characters, lower limit.    */
+    UBRK_WORD_LETTER         = 200,
+    /** Tag value for words containing letters, upper limit  */
+    UBRK_WORD_LETTER_LIMIT   = 300,
+    /** Tag value for words containing kana characters, lower limit */
+    UBRK_WORD_KANA           = 300,
+    /** Tag value for words containing kana characters, upper limit */
+    UBRK_WORD_KANA_LIMIT     = 400,
+    /** Tag value for words containing ideographic characters, lower limit */
+    UBRK_WORD_IDEO           = 400,
+    /** Tag value for words containing ideographic characters, upper limit */
+    UBRK_WORD_IDEO_LIMIT     = 500
+} UWordBreak;
+
+/**
+ *  Enum constants for the line break tags returned by getRuleStatus().
+ *  A range of values is defined for each category of
+ *  word, to allow for further subdivisions of a category in future releases.
+ *  Applications should check for tag values falling within the range, rather
+ *  than for single individual values.
+ *  @draft ICU 2.8
+*/
+typedef enum ULineBreakTag {
+    /** Tag value for soft line breaks, positions at which a line break
+      *  is acceptable but not required                */
+    UBRK_LINE_SOFT            = 0,
+    /** Upper bound for soft line breaks.              */
+    UBRK_LINE_SOFT_LIMIT      = 100,
+    /** Tag value for a hard, or mandatory line break  */
+    UBRK_LINE_HARD            = 100,
+    /** Upper bound for hard line breaks.              */
+    UBRK_LINE_HARD_LIMIT      = 200
+} ULineBreakTag;
+
+
+
+/**
+ *  Enum constants for the sentence break tags returned by getRuleStatus().
+ *  A range of values is defined for each category of
+ *  sentence, to allow for further subdivisions of a category in future releases.
+ *  Applications should check for tag values falling within the range, rather
+ *  than for single individual values.
+ *  @draft ICU 2.8
+*/
+typedef enum USentenceBreakTag {
+    /** Tag value for for sentences  ending with a sentence terminator
+      * ('.', '?', '!', etc.) character, possibly followed by a
+      * hard separator (CR, LF, PS, etc.)
+      */
+    UBRK_SENTENCE_TERM       = 0,
+    /** Upper bound for tags for sentences ended by sentence terminators.    */
+    UBRK_SENTENCE_TERM_LIMIT = 100,
+    /** Tag value for for sentences that do not contain an ending
+      * sentence terminator ('.', '?', '!', etc.) character, but 
+      * are ended only by a hard separator (CR, LF, PS, etc.) or end of input.
+      */
+    UBRK_SENTENCE_SEP        = 100,
+    /** Upper bound for tags for sentences ended by a separator.              */
+    UBRK_SENTENCE_SEP_LIMIT  = 200
+    /** Tag value for a hard, or mandatory line break  */
+} USentenceBreakTag;
+
+
 /**
  * Open a new UBreakIterator for locating text boundaries for a specified locale.
  * A UBreakIterator may be used for detecting character, line, word,
@@ -205,9 +304,9 @@ typedef enum UBreakIteratorType UBreakIteratorType;
  * @param status A UErrorCode to receive any errors.
  * @return A UBreakIterator for the specified locale.
  * @see ubrk_openRules
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI UBreakIterator* U_EXPORT2 
+U_CAPI UBreakIterator* U_EXPORT2
 ubrk_open(UBreakIteratorType type,
       const char *locale,
       const UChar *text,
@@ -219,19 +318,23 @@ ubrk_open(UBreakIteratorType type,
  * The rule syntax is ... (TBD)
  * @param rules A set of rules specifying the text breaking conventions.
  * @param rulesLength The number of characters in rules, or -1 if null-terminated.
- * @param text The text to be iterated over.
+ * @param text The text to be iterated over.  May be null, in which case ubrk_setText() is
+ *        used to specify the text to be iterated.
  * @param textLength The number of characters in text, or -1 if null-terminated.
+ * @param parseErr   Receives position and context information for any syntax errors
+ *                   detected while parsing the rules.
  * @param status A UErrorCode to receive any errors.
  * @return A UBreakIterator for the specified rules.
  * @see ubrk_open
- * @stable
+ * @stable ICU 2.2
  */
-U_CAPI UBreakIterator* U_EXPORT2 
-ubrk_openRules(const UChar *rules,
-           int32_t rulesLength,
-           const UChar *text,
-           int32_t textLength,
-           UErrorCode *status);
+U_CAPI UBreakIterator* U_EXPORT2
+ubrk_openRules(const UChar     *rules,
+               int32_t         rulesLength,
+               const UChar     *text,
+               int32_t          textLength,
+               UParseError     *parseErr,
+               UErrorCode      *status);
 
 /**
  * Thread safe cloning operation
@@ -247,31 +350,39 @@ ubrk_openRules(const UChar *rules,
  * @param status to indicate whether the operation went on smoothly or there were errors
  *  An informational status value, U_SAFECLONE_ALLOCATED_ERROR, is used if any allocations were necessary.
  * @return pointer to the new clone
- * @draft ICU 1.8
+ * @stable ICU 2.0
  */
-U_CAPI UBreakIterator * U_EXPORT2 
+U_CAPI UBreakIterator * U_EXPORT2
 ubrk_safeClone(
           const UBreakIterator *bi,
           void *stackBuffer,
           int32_t *pBufferSize,
           UErrorCode *status);
 
+/**
+  * A recommended size (in bytes) for the memory buffer to be passed to ubrk_saveClone().
+  * @stable ICU 2.0
+  */
 #define U_BRK_SAFECLONE_BUFFERSIZE 512
 
 /**
 * Close a UBreakIterator.
 * Once closed, a UBreakIterator may no longer be used.
 * @param bi The break iterator to close.
- * @stable
+ * @stable ICU 2.0
 */
-U_CAPI void U_EXPORT2 
+U_CAPI void U_EXPORT2
 ubrk_close(UBreakIterator *bi);
 
 /**
  * Sets an existing iterator to point to a new piece of text
- * @stable
+ * @param bi The iterator to use
+ * @param text The text to be set
+ * @param textLength The length of the text
+ * @param status The error code
+ * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_CAPI void U_EXPORT2
 ubrk_setText(UBreakIterator* bi,
              const UChar*    text,
              int32_t         textLength,
@@ -283,9 +394,9 @@ ubrk_setText(UBreakIterator* bi,
  * @param bi The break iterator to use.
  * @return The character index most recently returned by \Ref{ubrk_next}, \Ref{ubrk_previous},
  * \Ref{ubrk_first}, or \Ref{ubrk_last}.
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_current(const UBreakIterator *bi);
 
 /**
@@ -295,9 +406,9 @@ ubrk_current(const UBreakIterator *bi);
  * @return The character index of the next text boundary, or UBRK_DONE
  * if all text boundaries have been returned.
  * @see ubrk_previous
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_next(UBreakIterator *bi);
 
 /**
@@ -307,9 +418,9 @@ ubrk_next(UBreakIterator *bi);
  * @return The character index of the preceding text boundary, or UBRK_DONE
  * if all text boundaries have been returned.
  * @see ubrk_next
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_previous(UBreakIterator *bi);
 
 /**
@@ -318,9 +429,9 @@ ubrk_previous(UBreakIterator *bi);
  * @param bi The break iterator to use.
  * @return The character index of the first character in the text being scanned.
  * @see ubrk_last
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_first(UBreakIterator *bi);
 
 /**
@@ -331,9 +442,9 @@ ubrk_first(UBreakIterator *bi);
  * @return The character offset immediately <EM>beyond</EM> the last character in the
  * text being scanned.
  * @see ubrk_first
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_last(UBreakIterator *bi);
 
 /**
@@ -343,9 +454,9 @@ ubrk_last(UBreakIterator *bi);
  * @param offset The offset to begin scanning.
  * @return The text boundary preceding offset, or UBRK_DONE.
  * @see ubrk_following
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_preceding(UBreakIterator *bi,
            int32_t offset);
 
@@ -356,9 +467,9 @@ ubrk_preceding(UBreakIterator *bi,
  * @param offset The offset to begin scanning.
  * @return The text boundary following offset, or UBRK_DONE.
  * @see ubrk_preceding
- * @stable
+ * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_following(UBreakIterator *bi,
            int32_t offset);
 
@@ -369,9 +480,9 @@ ubrk_following(UBreakIterator *bi,
 * @param index The index of the desired locale.
 * @return A locale for which number text breaking information is available, or 0 if none.
 * @see ubrk_countAvailable
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI const char* U_EXPORT2 
+U_CAPI const char* U_EXPORT2
 ubrk_getAvailable(int32_t index);
 
 /**
@@ -380,9 +491,9 @@ ubrk_getAvailable(int32_t index);
 * calls to \Ref{ubrk_getAvailable}.
 * @return The number of locales for which text breaking information is available.
 * @see ubrk_getAvailable
-* @stable
+* @stable ICU 2.0
 */
-U_CAPI int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2
 ubrk_countAvailable(void);
 
 
@@ -393,8 +504,36 @@ ubrk_countAvailable(void);
 * @param bi The break iterator to use.
 * @param offset the offset to check.
 * @return True if "offset" is a boundary position.
+* @stable ICU 2.0
 */
-U_CAPI  UBool U_EXPORT2 
+U_CAPI  UBool U_EXPORT2
 ubrk_isBoundary(UBreakIterator *bi, int32_t offset);
 
+/**
+ * Return the status from the break rule that determined the most recently
+ * returned break position.  The values appear in the rule source
+ * within brackets, {123}, for example.  For rules that do not specify a
+ * status, a default value of 0 is returned.
+ * <p>
+ * For word break iterators, the possible values are defined in enum UWordBreak.
+ * @stable ICU 2.2
+ */
+U_CAPI  int32_t U_EXPORT2
+ubrk_getRuleStatus(UBreakIterator *bi);
+
+/**
+ * Return the locale of the break iterator. You can choose between the valid and
+ * the actual locale.
+ * @param bi break iterator
+ * @param type locale type (valid or actual)
+ * @param status error code
+ * @return locale string
+ * @draft ICU 2.8 likely to change in ICU 3.0, based on feedback
+ */
+U_CAPI const char* U_EXPORT2
+ubrk_getLocaleByType(const UBreakIterator *bi, ULocDataLocaleType type, UErrorCode* status);
+
+
+#endif /* #if !UCONFIG_NO_BREAK_ITERATION */
+
 #endif
