@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 1998-2001, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 */
@@ -23,7 +23,7 @@ U_NAMESPACE_BEGIN
  * compare equal).
  * @see CharacterIterator
  * @see ForwardCharacterIterator
- * @stable
+ * @stable ICU 2.0
  */
 class U_COMMON_API UCharCharacterIterator : public CharacterIterator {
 public:
@@ -32,7 +32,9 @@ public:
    * The iteration range is 0 to <code>length-1</code>.
    * text is only aliased, not adopted (the
    * destructor will not delete it).
-   * @stable
+   * @param textPtr The UChar array to be iterated over
+   * @param length The length of the UChar array
+   * @stable ICU 2.0
    */
   UCharCharacterIterator(const UChar* textPtr, int32_t length);
 
@@ -44,7 +46,10 @@ public:
    * The starting
    * position is specified by "position". If "position" is outside the valid
    * iteration range, the behavior of this object is undefined.
-   * @stable
+   * @param textPtr The UChar array to be iteratd over
+   * @param length The length of the UChar array
+   * @param position The starting position of the iteration
+   * @stable ICU 2.0
    */
   UCharCharacterIterator(const UChar* textPtr, int32_t length,
                          int32_t position);
@@ -58,7 +63,12 @@ public:
    * position is specified by "position". If begin and end do not
    * form a valid iteration range or "position" is outside the valid
    * iteration range, the behavior of this object is undefined.
-   * @stable
+   * @param textPtr The UChar array to be iterated over
+   * @param length The length of the UChar array
+   * @param textBegin  The begin position of the iteration range
+   * @param textEnd    The end position of the iteration range
+   * @param position    The starting position of the iteration
+   * @stable ICU 2.0
    */
   UCharCharacterIterator(const UChar* textPtr, int32_t length,
                          int32_t textBegin,
@@ -69,21 +79,24 @@ public:
    * Copy constructor.  The new iterator iterates over the same range
    * of the same string as "that", and its initial position is the
    * same as "that"'s current position.  
-   * @stable
+   * @param that The UCharCharacterIterator to be copied
+   * @stable ICU 2.0
    */
   UCharCharacterIterator(const UCharCharacterIterator&  that);
 
   /**
    * Destructor. 
-   * @stable
+   * @stable ICU 2.0
    */
   ~UCharCharacterIterator();
 
   /**
    * Assignment operator.  *this is altered to iterate over the sane
    * range of the same string as "that", and refers to the same
-   * character within that string as "that" does.  
-   * @stable
+   * character within that string as "that" does. 
+   * @param that The object to be copied
+   * @return the newly created object 
+   * @stable ICU 2.0
    */
   UCharCharacterIterator&
   operator=(const UCharCharacterIterator&    that);
@@ -91,13 +104,17 @@ public:
   /**
    * Returns true if the iterators iterate over the same range of the
    * same string and are pointing at the same character.  
-   * @stable
+   * @param that The ForwardCharacterIterator used to be compared for equality
+   * @return true if the iterators iterate over the same range of the
+   * same string and are pointing at the same character.
+   * @stable ICU 2.0
    */
   virtual UBool          operator==(const ForwardCharacterIterator& that) const;
 
   /**
    * Generates a hash code for this iterator.  
-   * @stable
+   * @return the hash code.
+   * @stable ICU 2.0
    */
   virtual int32_t         hashCode(void) const;
 
@@ -105,7 +122,8 @@ public:
    * Returns a new UCharCharacterIterator referring to the same
    * character in the same range of the same string as this one.  The
    * caller must delete the new iterator.  
-   * @stable
+   * @return the CharacterIterator newly created
+   * @stable ICU 2.0
    */
   virtual CharacterIterator* clone(void) const;
                                 
@@ -113,7 +131,8 @@ public:
    * Sets the iterator to refer to the first code unit in its
    * iteration range, and returns that code unit.
    * This can be used to begin an iteration with next().
-   * @stable
+   * @return the first code unit in its iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar         first(void);
 
@@ -122,7 +141,8 @@ public:
    * iteration range, returns that code unit, and moves the position
    * to the second code unit. This is an alternative to setToStart()
    * for forward iteration with nextPostInc().
-   * @stable
+   * @return the first code unit in its iteration range
+   * @stable ICU 2.0
    */
   virtual UChar         firstPostInc(void);
 
@@ -132,7 +152,8 @@ public:
    * This can be used to begin an iteration with next32().
    * Note that an iteration with next32PostInc(), beginning with,
    * e.g., setToStart() or firstPostInc(), is more efficient.
-   * @stable
+   * @return the first code point in its iteration range
+   * @stable ICU 2.0
    */
   virtual UChar32       first32(void);
 
@@ -141,7 +162,8 @@ public:
    * iteration range, returns that code point, and moves the position
    * to the second code point. This is an alternative to setToStart()
    * for forward iteration with next32PostInc().
-   * @stable
+   * @return the first code point in its iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar32       first32PostInc(void);
 
@@ -149,7 +171,8 @@ public:
    * Sets the iterator to refer to the last code unit in its
    * iteration range, and returns that code unit.
    * This can be used to begin an iteration with previous().
-   * @stable
+   * @return the last code unit in its iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar         last(void);
 
@@ -157,7 +180,8 @@ public:
    * Sets the iterator to refer to the last code point in its
    * iteration range, and returns that code unit.
    * This can be used to begin an iteration with previous32().
-   * @stable
+   * @return the last code point in its iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar32       last32(void);
 
@@ -165,7 +189,9 @@ public:
    * Sets the iterator to refer to the "position"-th code unit
    * in the text-storage object the iterator refers to, and
    * returns that code unit.  
-   * @stable
+   * @position the position within the text-storage object
+   * @return the code unit
+   * @stable ICU 2.0
    */
   virtual UChar         setIndex(int32_t position);
 
@@ -176,27 +202,32 @@ public:
    * returns that code point.
    * The current position is adjusted to the beginning of the code point
    * (its first code unit).
-   * @stable
+   * @position the position within the text-storage object
+   * @return the code unit
+   * @stable ICU 2.0
    */
   virtual UChar32       setIndex32(int32_t position);
 
   /**
    * Returns the code unit the iterator currently refers to.  
-   * @stable
+   * @return the code unit the iterator currently refers to.
+   * @stable ICU 2.0
    */
   virtual UChar         current(void) const;
 
   /**
    * Returns the code point the iterator currently refers to.  
-   * @stable
+   * @return the code point the iterator currently refers to.
+   * @stable ICU 2.0
    */
   virtual UChar32       current32(void) const;
 
   /**
    * Advances to the next code unit in the iteration range (toward
    * endIndex()), and returns that code unit.  If there are no more
-   * code units to return, returns DONE.  
-   * @stable
+   * code units to return, returns DONE. 
+   * @return the next code unit in the iteration range. 
+   * @stable ICU 2.0
    */
   virtual UChar         next(void);
 
@@ -205,7 +236,8 @@ public:
    * in the iteration range
    * (toward endIndex()).  If there are
    * no more code units to return, returns DONE.
-   * @stable
+   * @return the current code unit.
+   * @stable ICU 2.0
    */
   virtual UChar         nextPostInc(void);
         
@@ -216,7 +248,8 @@ public:
    * Note that iteration with "pre-increment" semantics is less
    * efficient than iteration with "post-increment" semantics
    * that is provided by next32PostInc().
-   * @stable
+   * @return the next code point in the iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar32       next32(void);
 
@@ -225,7 +258,8 @@ public:
    * in the iteration range
    * (toward endIndex()).  If there are
    * no more code points to return, returns DONE.
-   * @stable
+   * @return the current point.
+   * @stable ICU 2.0
    */
   virtual UChar32       next32PostInc(void);
         
@@ -234,23 +268,27 @@ public:
    * at or after the current position in the iteration range.
    * This is used with nextPostInc() or next32PostInc() in forward
    * iteration.
-   * @stable
+   * @return FALSE if there are no more code units or code points
+   * at or after the current position in the iteration range.
+   * @stable ICU 2.0
    */
   virtual UBool        hasNext();
 
   /**
-   * Advances to the previous code unit in the iteration rance (toward
+   * Advances to the previous code unit in the iteration range (toward
    * startIndex()), and returns that code unit.  If there are no more
    * code units to return, returns DONE.  
-   * @stable
+   * @return the previous code unit in the iteration range.
+   * @stable ICU 2.0
    */
   virtual UChar         previous(void);
 
   /**
-   * Advances to the previous code point in the iteration rance (toward
+   * Advances to the previous code point in the iteration range (toward
    * startIndex()), and returns that code point.  If there are no more
-   * code points to return, returns DONE.  
-   * @stable
+   * code points to return, returns DONE.
+   * @return the previous code point in the iteration range.  
+   * @stable ICU 2.0
    */
   virtual UChar32       previous32(void);
 
@@ -259,7 +297,9 @@ public:
    * before the current position in the iteration range.
    * This is used with previous() or previous32() in backward
    * iteration.
-   * @stable
+   * @return FALSE if there are no more code units or code points
+   * before the current position in the iteration range.
+   * @stable ICU 2.0
    */
   virtual UBool        hasPrevious();
 
@@ -268,8 +308,11 @@ public:
    * iteration range, or relative to the current position itself.
    * The movement is expressed in numbers of code units forward
    * or backward by specifying a positive or negative delta.
+   * @delta the position relative to origin. A positive delta means forward;
+   * a negative delta means backward.
+   * @origin Origin enumeration {kStart, kCurrent, kEnd}
    * @return the new position
-   * @stable
+   * @stable ICU 2.0
    */
   virtual int32_t      move(int32_t delta, EOrigin origin);
 
@@ -278,14 +321,17 @@ public:
    * iteration range, or relative to the current position itself.
    * The movement is expressed in numbers of code points forward
    * or backward by specifying a positive or negative delta.
+   * @delta the position relative to origin. A positive delta means forward;
+   * a negative delta means backward.
+   * @origin Origin enumeration {kStart, kCurrent, kEnd}
    * @return the new position
-   * @stable
+   * @stable ICU 2.0
    */
   virtual int32_t      move32(int32_t delta, EOrigin origin);
 
   /**
    * Sets the iterator to iterate over a new range of text
-   * @stable
+   * @stable ICU 2.0
    */
   void setText(const UChar* newText, int32_t newTextLength);
   
@@ -294,31 +340,36 @@ public:
    * referred to by "result".  Even if this iterator iterates across
    * only a part of this string, the whole string is copied.
    * @param result Receives a copy of the text under iteration.  
-   * @stable
+   * @stable ICU 2.0
    */
   virtual void            getText(UnicodeString& result);
 
   /**
-   * Return a class ID for this object (not really public)   
-   * @stable
+   * Return a class ID for this class (not really public) 
+   * @return a class ID for this class  
+   * @stable ICU 2.0
    */
-  virtual UClassID         getDynamicClassID(void) const 
-    { return getStaticClassID(); }
+  static UClassID         getStaticClassID(void);
 
   /**
-   * Return a class ID for this class (not really public)   
-   * @stable
+   * Return a class ID for this object (not really public) 
+   * @return a class ID for this object.  
+   * @stable ICU 2.0
    */
-  static UClassID          getStaticClassID(void) 
-    { return (UClassID)(&fgClassID); }
+  virtual UClassID        getDynamicClassID(void) const;
 
 protected:
+  /**
+   * Protected constructor
+   * @stable ICU 2.0
+   */
   UCharCharacterIterator();
-        
+  /**
+   * Protected member text
+   * @stable ICU 2.0
+   */      
   const UChar*            text;
 
-private:
-  static const char       fgClassID;
 };
 
 U_NAMESPACE_END
