@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (c) 1996-2001, International Business Machines Corporation
+* Copyright (c) 1996-2003, International Business Machines Corporation
 *               and others. All Rights Reserved.
 *******************************************************************************
 * File unorm.h
@@ -16,6 +16,9 @@
 #define UNORM_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_NORMALIZATION
+
 #include "unicode/uiter.h"
 
 /**
@@ -119,90 +122,62 @@
 
 /**
  * Constants for normalization modes.
- * @stable except for deprecated constants
+ * @stable ICU 2.0
  */
 typedef enum {
-  /** No decomposition/composition. @stable */
+  /** No decomposition/composition. @stable ICU 2.0 */
   UNORM_NONE = 1, 
-  /** Canonical decomposition. @stable */
+  /** Canonical decomposition. @stable ICU 2.0 */
   UNORM_NFD = 2,
-  /** Compatibility decomposition. @stable */
+  /** Compatibility decomposition. @stable ICU 2.0 */
   UNORM_NFKD = 3,
-  /** Canonical decomposition followed by canonical composition. @stable */
+  /** Canonical decomposition followed by canonical composition. @stable ICU 2.0 */
   UNORM_NFC = 4,
-  /** Default normalization. @stable */
+  /** Default normalization. @stable ICU 2.0 */
   UNORM_DEFAULT = UNORM_NFC, 
-  /** Compatibility decomposition followed by canonical composition. @stable */
+  /** Compatibility decomposition followed by canonical composition. @stable ICU 2.0 */
   UNORM_NFKC =5,
-  /** "Fast C or D" form. @draft ICU 2.0 */
+  /** "Fast C or D" form. @stable ICU 2.0 */
   UNORM_FCD = 6,
 
-  /** One more than the highest normalization mode constant. @stable */
-  UNORM_MODE_COUNT,
-
-  /* *** The rest of this enum is entirely deprecated. *** */
-
-  /**
-   * No decomposition/composition
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NONE.
-   */
-  UCOL_NO_NORMALIZATION = 1,
-  /**
-   * Canonical decomposition
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NFD.
-   */
-  UCOL_DECOMP_CAN = 2,
-  /**
-   * Compatibility decomposition
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NFKD.
-   */
-  UCOL_DECOMP_COMPAT = 3,
-  /**
-   * Default normalization
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NFKD or UNORM_DEFAULT.
-   */
-  UCOL_DEFAULT_NORMALIZATION = UCOL_DECOMP_COMPAT, 
-  /**
-   * Canonical decomposition followed by canonical composition
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NFC.
-   */
-  UCOL_DECOMP_CAN_COMP_COMPAT = 4,
-  /**
-   * Compatibility decomposition followed by canonical composition
-   * @deprecated To be removed after 2002-sep-30, use UNORM_NFKC.
-   */
-  UCOL_DECOMP_COMPAT_COMP_CAN =5,
-
-  /**
-   * Do not normalize Hangul.
-   * @deprecated To be removed without replacement after 2002-mar-31.
-   */
-  UCOL_IGNORE_HANGUL    = 16,
-  /**
-   * Do not normalize Hangul.
-   * @deprecated To be removed without replacement after 2002-mar-31.
-   */
-  UNORM_IGNORE_HANGUL    = 16
+  /** One more than the highest normalization mode constant. @stable ICU 2.0 */
+  UNORM_MODE_COUNT
 } UNormalizationMode;
 
+/**
+ * Constants for options flags for normalization.
+ * Use 0 for default options,
+ * including normalization according to the Unicode version
+ * that is currently supported by ICU (see u_getUnicodeVersion).
+ * @draft ICU 2.6
+ */
+enum {
+    /**
+     * Options bit set value to select Unicode 3.2 normalization
+     * (except NormalizationCorrections).
+     * At most one Unicode version can be selected at a time.
+     * @draft ICU 2.6
+     */
+    UNORM_UNICODE_3_2=0x20
+};
+
 /**
  * Normalize a string.
  * The string will be normalized according the specified normalization mode
- * and options (there are currently no options defined).
+ * and options.
  *
  * @param source The string to normalize.
  * @param sourceLength The length of source, or -1 if NUL-terminated.
  * @param mode The normalization mode; one of UNORM_NONE, 
  *             UNORM_NFD, UNORM_NFC, UNORM_NFKC, UNORM_NFKD, UNORM_DEFAULT.
- * @param options The normalization options, ORed together (0 for no options);
- *                currently there is no option defined.
+ * @param options The normalization options, ORed together (0 for no options).
  * @param result A pointer to a buffer to receive the result string.
  *               The result string is NUL-terminated if possible.
  * @param resultLength The maximum size of result.
  * @param status A pointer to a UErrorCode to receive any errors.
  * @return The total buffer size needed; if greater than resultLength,
  *         the output was truncated, and the error code is set to U_BUFFER_OVERFLOW_ERROR.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 unorm_normalize(const UChar *source, int32_t sourceLength,
@@ -210,17 +185,10 @@ unorm_normalize(const UChar *source, int32_t sourceLength,
                 UChar *result, int32_t resultLength,
                 UErrorCode *status);
 
-/**
- * The function u_normalize() has been renamed to unorm_normalize()
- * for consistency. The old name is deprecated.
- * @deprecated To be removed after 2002-mar-31.
- */
-#define u_normalize unorm_normalize
-
 /**
  * Result values for unorm_quickCheck().
  * For details see Unicode Technical Report 15.
- * @stable
+ * @stable ICU 2.0
  */
 typedef enum UNormalizationCheckResult {
   /** 
@@ -251,16 +219,87 @@ typedef enum UNormalizationCheckResult {
  *
  * @param source       string for determining if it is in a normalized format
  * @param sourcelength length of source to test, or -1 if NUL-terminated
- * @paran mode         which normalization form to test for
+ * @param mode         which normalization form to test for
  * @param status       a pointer to a UErrorCode to receive any errors
  * @return UNORM_YES, UNORM_NO or UNORM_MAYBE
- * @stable
+ *
+ * @see unorm_isNormalized
+ * @stable ICU 2.0
  */
 U_CAPI UNormalizationCheckResult U_EXPORT2
 unorm_quickCheck(const UChar *source, int32_t sourcelength,
                  UNormalizationMode mode,
                  UErrorCode *status);
 
+/**
+ * Performing quick check on a string; same as unorm_quickCheck but
+ * takes an extra options parameter like most normalization functions.
+ *
+ * @param src        String that is to be tested if it is in a normalization format.
+ * @param srcLength  Length of source to test, or -1 if NUL-terminated.
+ * @param mode       Which normalization form to test for.
+ * @param options    The normalization options, ORed together (0 for no options).
+ * @param pErrorCode ICU error code in/out parameter.
+ *                   Must fulfill U_SUCCESS before the function call.
+ * @return UNORM_YES, UNORM_NO or UNORM_MAYBE
+ *
+ * @see unorm_quickCheck
+ * @see unorm_isNormalized
+ * @draft ICU 2.6
+ */
+U_CAPI UNormalizationCheckResult U_EXPORT2
+unorm_quickCheckWithOptions(const UChar *src, int32_t srcLength, 
+                            UNormalizationMode mode, int32_t options,
+                            UErrorCode *pErrorCode);
+
+/**
+ * Test if a string is in a given normalization form.
+ * This is semantically equivalent to source.equals(normalize(source, mode)) .
+ *
+ * Unlike unorm_quickCheck(), this function returns a definitive result,
+ * never a "maybe".
+ * For NFD, NFKD, and FCD, both functions work exactly the same.
+ * For NFC and NFKC where quickCheck may return "maybe", this function will
+ * perform further tests to arrive at a TRUE/FALSE result.
+ *
+ * @param src        String that is to be tested if it is in a normalization format.
+ * @param srcLength  Length of source to test, or -1 if NUL-terminated.
+ * @param mode       Which normalization form to test for.
+ * @param pErrorCode ICU error code in/out parameter.
+ *                   Must fulfill U_SUCCESS before the function call.
+ * @return Boolean value indicating whether the source string is in the
+ *         "mode" normalization form.
+ *
+ * @see unorm_quickCheck
+ * @stable ICU 2.2
+ */
+U_CAPI UBool U_EXPORT2
+unorm_isNormalized(const UChar *src, int32_t srcLength,
+                   UNormalizationMode mode,
+                   UErrorCode *pErrorCode);
+
+/**
+ * Test if a string is in a given normalization form; same as unorm_isNormalized but
+ * takes an extra options parameter like most normalization functions.
+ *
+ * @param src        String that is to be tested if it is in a normalization format.
+ * @param srcLength  Length of source to test, or -1 if NUL-terminated.
+ * @param mode       Which normalization form to test for.
+ * @param options    The normalization options, ORed together (0 for no options).
+ * @param pErrorCode ICU error code in/out parameter.
+ *                   Must fulfill U_SUCCESS before the function call.
+ * @return Boolean value indicating whether the source string is in the
+ *         "mode/options" normalization form.
+ *
+ * @see unorm_quickCheck
+ * @see unorm_isNormalized
+ * @draft ICU 2.6
+ */
+U_CAPI UBool U_EXPORT2
+unorm_isNormalizedWithOptions(const UChar *src, int32_t srcLength,
+                              UNormalizationMode mode, int32_t options,
+                              UErrorCode *pErrorCode);
+
 /**
  * Iterative normalization forward.
  * This function (together with unorm_previous) is somewhat
@@ -318,7 +357,7 @@ unorm_quickCheck(const UChar *source, int32_t sourcelength,
  * @param dest The output buffer; can be NULL if destCapacity==0 for pure preflighting.
  * @param destCapacity The number of UChars that fit into dest.
  * @param mode The normalization mode.
- * @param options A bit set of normalization options.
+ * @param options The normalization options, ORed together (0 for no options).
  * @param doNormalize Indicates if the source text up to the next boundary
  *                    is to be normalized (TRUE) or just copied (FALSE).
  * @param pNeededToNormalize Output flag indicating if the normalization resulted in
@@ -332,7 +371,7 @@ unorm_quickCheck(const UChar *source, int32_t sourcelength,
  * @see unorm_previous
  * @see unorm_normalize
  *
- * @draft ICU 2.1
+ * @stable ICU 2.1
  */
 U_CAPI int32_t U_EXPORT2
 unorm_next(UCharIterator *src,
@@ -351,7 +390,7 @@ unorm_next(UCharIterator *src,
  * @param dest The output buffer; can be NULL if destCapacity==0 for pure preflighting.
  * @param destCapacity The number of UChars that fit into dest.
  * @param mode The normalization mode.
- * @param options A bit set of normalization options.
+ * @param options The normalization options, ORed together (0 for no options).
  * @param doNormalize Indicates if the source text up to the next boundary
  *                    is to be normalized (TRUE) or just copied (FALSE).
  * @param pNeededToNormalize Output flag indicating if the normalization resulted in
@@ -365,7 +404,7 @@ unorm_next(UCharIterator *src,
  * @see unorm_next
  * @see unorm_normalize
  *
- * @draft ICU 2.1
+ * @stable ICU 2.1
  */
 U_CAPI int32_t U_EXPORT2
 unorm_previous(UCharIterator *src,
@@ -374,15 +413,15 @@ unorm_previous(UCharIterator *src,
                UBool doNormalize, UBool *pNeededToNormalize,
                UErrorCode *pErrorCode);
 
-/*
+/**
  * Concatenate normalized strings, making sure that the result is normalized as well.
  *
  * If both the left and the right strings are in
- * the normalization form according to "mode",
+ * the normalization form according to "mode/options",
  * then the result will be
  *
  * \code
- *     dest=normalize(left+right, mode)
+ *     dest=normalize(left+right, mode, options)
  * \endcode
  *
  * With the input strings already being normalized,
@@ -400,7 +439,7 @@ unorm_previous(UCharIterator *src,
  * @param dest The output buffer; can be NULL if destCapacity==0 for pure preflighting.
  * @param destCapacity The number of UChars that fit into dest.
  * @param mode The normalization mode.
- * @param options A bit set of normalization options.
+ * @param options The normalization options, ORed together (0 for no options).
  * @param pErrorCode ICU error code in/out parameter.
  *                   Must fulfill U_SUCCESS before the function call.
  * @return Length of output (number of UChars) when successful or buffer overflow.
@@ -409,7 +448,7 @@ unorm_previous(UCharIterator *src,
  * @see unorm_next
  * @see unorm_previous
  *
- * @draft ICU 2.1
+ * @stable ICU 2.1
  */
 U_CAPI int32_t U_EXPORT2
 unorm_concatenate(const UChar *left, int32_t leftLength,
@@ -418,4 +457,119 @@ unorm_concatenate(const UChar *left, int32_t leftLength,
                   UNormalizationMode mode, int32_t options,
                   UErrorCode *pErrorCode);
 
+/**
+ * Option bit for unorm_compare:
+ * Both input strings are assumed to fulfill FCD conditions.
+ * @stable ICU 2.2
+ */
+#define UNORM_INPUT_IS_FCD          0x20000
+
+/**
+ * Option bit for unorm_compare:
+ * Perform case-insensitive comparison.
+ * @stable ICU 2.2
+ */
+#define U_COMPARE_IGNORE_CASE       0x10000
+
+#ifndef U_COMPARE_CODE_POINT_ORDER
+/* see also unistr.h and ustring.h */
+/**
+ * Option bit for u_strCaseCompare, u_strcasecmp, unorm_compare, etc:
+ * Compare strings in code point order instead of code unit order.
+ * @stable ICU 2.2
+ */
+#define U_COMPARE_CODE_POINT_ORDER  0x8000
+#endif
+
+/**
+ * Lowest-order bit number of unorm_compare() options bits corresponding to
+ * normalization options bits.
+ *
+ * The options parameter for unorm_compare() uses most bits for
+ * itself and for various comparison and folding flags.
+ * The most significant bits, however, are shifted down and passed on
+ * to the normalization implementation.
+ * (That is, from unorm_compare(..., options, ...),
+ * options>>UNORM_COMPARE_NORM_OPTIONS_SHIFT will be passed on to the
+ * internal normalization functions.)
+ *
+ * @see unorm_compare
+ * @draft ICU 2.6
+ */
+#define UNORM_COMPARE_NORM_OPTIONS_SHIFT 20
+
+/**
+ * Compare two strings for canonical equivalence.
+ * Further options include case-insensitive comparison and
+ * code point order (as opposed to code unit order).
+ *
+ * Canonical equivalence between two strings is defined as their normalized
+ * forms (NFD or NFC) being identical.
+ * This function compares strings incrementally instead of normalizing
+ * (and optionally case-folding) both strings entirely,
+ * improving performance significantly.
+ *
+ * Bulk normalization is only necessary if the strings do not fulfill the FCD
+ * conditions. Only in this case, and only if the strings are relatively long,
+ * is memory allocated temporarily.
+ * For FCD strings and short non-FCD strings there is no memory allocation.
+ *
+ * Semantically, this is equivalent to
+ *   strcmp[CodePointOrder](NFD(foldCase(NFD(s1))), NFD(foldCase(NFD(s2))))
+ * where code point order and foldCase are all optional.
+ *
+ * UAX 21 2.5 Caseless Matching specifies that for a canonical caseless match
+ * the case folding must be performed first, then the normalization.
+ *
+ * @param s1 First source string.
+ * @param length1 Length of first source string, or -1 if NUL-terminated.
+ *
+ * @param s2 Second source string.
+ * @param length2 Length of second source string, or -1 if NUL-terminated.
+ *
+ * @param options A bit set of options:
+ *   - U_FOLD_CASE_DEFAULT or 0 is used for default options:
+ *     Case-sensitive comparison in code unit order, and the input strings
+ *     are quick-checked for FCD.
+ *
+ *   - UNORM_INPUT_IS_FCD
+ *     Set if the caller knows that both s1 and s2 fulfill the FCD conditions.
+ *     If not set, the function will quickCheck for FCD
+ *     and normalize if necessary.
+ *
+ *   - U_COMPARE_CODE_POINT_ORDER
+ *     Set to choose code point order instead of code unit order
+ *     (see u_strCompare for details).
+ *
+ *   - U_COMPARE_IGNORE_CASE
+ *     Set to compare strings case-insensitively using case folding,
+ *     instead of case-sensitively.
+ *     If set, then the following case folding options are used.
+ *
+ *   - Options as used with case-insensitive comparisons, currently:
+ *
+ *   - U_FOLD_CASE_EXCLUDE_SPECIAL_I
+ *    (see u_strCaseCompare for details)
+ *
+ *   - regular normalization options shifted left by UNORM_COMPARE_NORM_OPTIONS_SHIFT
+ *
+ * @param pErrorCode ICU error code in/out parameter.
+ *                   Must fulfill U_SUCCESS before the function call.
+ * @return <0 or 0 or >0 as usual for string comparisons
+ *
+ * @see unorm_normalize
+ * @see UNORM_FCD
+ * @see u_strCompare
+ * @see u_strCaseCompare
+ *
+ * @stable ICU 2.2
+ */
+U_CAPI int32_t U_EXPORT2
+unorm_compare(const UChar *s1, int32_t length1,
+              const UChar *s2, int32_t length2,
+              uint32_t options,
+              UErrorCode *pErrorCode);
+
+#endif /* #if !UCONFIG_NO_NORMALIZATION */
+
 #endif
