@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 2002-2007, International Business Machines
+*   Copyright (C) 2002-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -43,22 +43,12 @@ U_NAMESPACE_BEGIN
 /**  U_HAVE_PLACEMENT_NEW - Define this to define the placement new and
  *                          delete in UMemory for STL.
  *
- * @stable ICU 2.6
+ * @draft ICU 2.6
  */                              
 #ifndef U_HAVE_PLACEMENT_NEW
 #define U_HAVE_PLACEMENT_NEW 1
 #endif
 
-
-/**  U_HAVE_DEBUG_LOCATION_NEW - Define this to define the MFC debug
- * version of the operator new.
- *
- * @stable ICU 3.4
- */                              
-#ifndef U_HAVE_DEBUG_LOCATION_NEW
-#define U_HAVE_DEBUG_LOCATION_NEW 0
-#endif
-
 /**
  * UMemory is the common ICU base class.
  * All other ICU C++ classes are derived from UMemory (starting with ICU 2.4).
@@ -72,7 +62,7 @@ U_NAMESPACE_BEGIN
  * UMemory does not contain any virtual functions.
  * Common "boilerplate" functions are defined in UObject.
  *
- * @stable ICU 2.4
+ * @draft ICU 2.4
  */
 class U_COMMON_API UMemory {
 public:
@@ -84,16 +74,16 @@ public:
      * (uprv_malloc(), uprv_free(), uprv_realloc());
      * they or something else could be used here to implement C++ new/delete
      * for ICU4C C++ classes
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
-    static void * U_EXPORT2 operator new(size_t size);
+    static void *operator new(size_t size);
 
     /**
      * Override for ICU4C C++ memory management.
      * See new().
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
-    static void * U_EXPORT2 operator new[](size_t size);
+    static void *operator new[](size_t size);
 
     /**
      * Override for ICU4C C++ memory management.
@@ -101,50 +91,32 @@ public:
      * (uprv_malloc(), uprv_free(), uprv_realloc());
      * they or something else could be used here to implement C++ new/delete
      * for ICU4C C++ classes
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
-    static void U_EXPORT2 operator delete(void *p);
+    static void operator delete(void *p);
 
     /**
      * Override for ICU4C C++ memory management.
      * See delete().
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
-    static void U_EXPORT2 operator delete[](void *p);
+    static void operator delete[](void *p);
 
 #if U_HAVE_PLACEMENT_NEW
     /**
      * Override for ICU4C C++ memory management for STL.
      * See new().
-     * @stable ICU 2.6
+     * @draft ICU 2.6
      */
-    static inline void * U_EXPORT2 operator new(size_t, void *ptr) { return ptr; }
+    static inline void * operator new(size_t, void *ptr) { return ptr; }
 
     /**
      * Override for ICU4C C++ memory management for STL.
      * See delete().
-     * @stable ICU 2.6
+     * @draft ICU 2.6
      */
-    static inline void U_EXPORT2 operator delete(void *, void *) {}
+    static inline void operator delete(void *, void *) {}
 #endif /* U_HAVE_PLACEMENT_NEW */
-#if U_HAVE_DEBUG_LOCATION_NEW
-    /**
-      * This method overrides the MFC debug version of the operator new
-      * 
-      * @param size   The requested memory size
-      * @param file   The file where the allocation was requested
-      * @param line   The line where the allocation was requested 
-      */ 
-    static void * U_EXPORT2 operator new(size_t size, const char* file, int line);
-    /**
-      * This method provides a matching delete for the MFC debug new
-      * 
-      * @param p      The pointer to the allocated memory
-      * @param file   The file where the allocation was requested
-      * @param line   The line where the allocation was requested 
-      */ 
-    static void U_EXPORT2 operator delete(void* p, const char* file, int line);
-#endif /* U_HAVE_DEBUG_LOCATION_NEW */
 #endif /* U_OVERRIDE_CXX_ALLOCATION */
 
     /*
@@ -209,9 +181,8 @@ protected:
     // commented out because UObject is abstract (see getDynamicClassID)
     // inline UObject(const UObject &other) {}
 
-#if 0
-    // TODO Sometime in the future. Implement operator==().
-    // (This comment inserted in 2.2)
+#if U_ICU_VERSION_MAJOR_NUM>2
+    // TODO post ICU 2.4  (This comment inserted in 2.2)
     // some or all of the following "boilerplate" functions may be made public
     // in a future ICU4C release when all subclasses implement them
 
@@ -241,19 +212,6 @@ protected:
      * here would be to declare and empty-implement a protected or public one.
     UObject &UObject::operator=(const UObject &);
      */
-
-// Future implementation for RTTI that support subtyping. [alan]
-// 
-//  public:
-//     /**
-//      * @internal
-//      */
-//     static UClassID getStaticClassID();
-// 
-//     /**
-//      * @internal
-//      */
-//     UBool instanceOf(UClassID type) const;
 };
 
 /**
@@ -264,44 +222,14 @@ protected:
  * @internal
  */
 #define UOBJECT_DEFINE_RTTI_IMPLEMENTATION(myClass) \
-    UClassID U_EXPORT2 myClass::getStaticClassID() { \
-        static char classID = 0; \
+    UClassID myClass::getStaticClassID() { \
+        static const char classID = 0; \
         return (UClassID)&classID; \
     } \
     UClassID myClass::getDynamicClassID() const \
     { return myClass::getStaticClassID(); }
 
 
-/**
- * This macro adds ICU RTTI to an ICU abstract class implementation.
- * This macro should be invoked in *.cpp files.  The corresponding
- * header should declare getStaticClassID.
- *
- * @param myClass The name of the class that needs RTTI defined.
- * @internal
- */
-#define UOBJECT_DEFINE_ABSTRACT_RTTI_IMPLEMENTATION(myClass) \
-    UClassID U_EXPORT2 myClass::getStaticClassID() { \
-        static char classID = 0; \
-        return (UClassID)&classID; \
-    }
-
-// /**
-//  * This macro adds ICU RTTI to an ICU concrete class implementation.
-//  * This macro should be invoked in *.cpp files.  The corresponding
-//  * header should declare getDynamicClassID and getStaticClassID.
-//  *
-//  * @param myClass The name of the class that needs RTTI defined.
-//  * @param myParent The name of the myClass's parent.
-//  * @internal
-//  */
-/*#define UOBJECT_DEFINE_RTTI_IMPLEMENTATION(myClass, myParent) \
-    UOBJECT_DEFINE_ABSTRACT_RTTI_IMPLEMENTATION(myClass, myParent) \
-    UClassID myClass::getDynamicClassID() const { \
-        return myClass::getStaticClassID(); \
-    }
-*/
-
 
 U_NAMESPACE_END
 
