@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2002-2010, International Business Machines
+*   Copyright (C) 2002-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -14,7 +14,7 @@
 *   created by: Markus W. Scherer
 *
 *   Constants for mostly non-core Unicode character properties
-*   stored in uprops.icu.
+*   stored in uprops.dat.
 */
 
 #ifndef __UPROPS_H__
@@ -22,7 +22,6 @@
 
 #include "unicode/utypes.h"
 #include "unicode/uset.h"
-#include "uset_imp.h"
 #include "udataswp.h"
 
 /* indexes[] entries */
@@ -48,34 +47,44 @@ enum {
 /* definitions for the main properties words */
 enum {
     /* general category shift==0                                0 (5 bits) */
-    /* reserved                                                 5 (1 bit) */
-    UPROPS_NUMERIC_TYPE_VALUE_SHIFT=6                       /*  6 (10 bits) */
+    UPROPS_EXCEPTION_SHIFT=5,                               /*  5 (1 bit)  */
+    UPROPS_BIDI_SHIFT,                                      /*  6 (5 bits) */
+    UPROPS_MIRROR_SHIFT=UPROPS_BIDI_SHIFT+5,                /* 11 (1 bit)  */
+    UPROPS_NUMERIC_TYPE_SHIFT,                              /* 12 (3 bits) */
+    UPROPS_CASE_SENSITIVE_SHIFT=UPROPS_NUMERIC_TYPE_SHIFT+3,/* 15 (1 bit) format version 3.2 */
+    UPROPS_RESERVED_SHIFT,                                  /* 16 (4 bits) */
+    UPROPS_VALUE_SHIFT=20,                                  /* 20 */
+
+    UPROPS_EXCEPTION_BIT=1UL<<UPROPS_EXCEPTION_SHIFT,
+    UPROPS_VALUE_BITS=32-UPROPS_VALUE_SHIFT,
+
+    UPROPS_MIN_VALUE=-(1L<<(UPROPS_VALUE_BITS-1)),
+    UPROPS_MAX_VALUE=(1L<<(UPROPS_VALUE_BITS-1))-1,
+    UPROPS_MAX_EXCEPTIONS_COUNT=1L<<UPROPS_VALUE_BITS
 };
 
+#define PROPS_VALUE_IS_EXCEPTION(props) ((props)&UPROPS_EXCEPTION_BIT)
 #define GET_CATEGORY(props) ((props)&0x1f)
+#define GET_BIDI_CLASS(props) ((props>>UPROPS_BIDI_SHIFT)&0x1f)
+#define GET_NUMERIC_TYPE(props) (((props)>>UPROPS_NUMERIC_TYPE_SHIFT)&7)
+#define GET_UNSIGNED_VALUE(props) ((props)>>UPROPS_VALUE_SHIFT)
+#define GET_SIGNED_VALUE(props) ((int32_t)(props)>>UPROPS_VALUE_SHIFT)
+#define GET_EXCEPTIONS(props) (exceptionsTable+GET_UNSIGNED_VALUE(props))
+
 #define CAT_MASK(props) U_MASK(GET_CATEGORY(props))
 
-#define GET_NUMERIC_TYPE_VALUE(props) ((props)>>UPROPS_NUMERIC_TYPE_VALUE_SHIFT)
-
-/* constants for the storage form of numeric types and values */
 enum {
-    UPROPS_NTV_NONE=0,
-    UPROPS_NTV_DECIMAL_START=1,
-    UPROPS_NTV_DIGIT_START=11,
-    UPROPS_NTV_NUMERIC_START=21,
-    UPROPS_NTV_FRACTION_START=0xb0,
-    UPROPS_NTV_LARGE_START=0x1e0,
-    UPROPS_NTV_RESERVED_START=0x300,
-
-    UPROPS_NTV_MAX_SMALL_INT=UPROPS_NTV_FRACTION_START-UPROPS_NTV_NUMERIC_START-1
+    EXC_UPPERCASE,
+    EXC_LOWERCASE,
+    EXC_TITLECASE,
+    EXC_UNUSED,
+    EXC_NUMERIC_VALUE,
+    EXC_DENOMINATOR_VALUE,
+    EXC_MIRROR_MAPPING,
+    EXC_SPECIAL_CASING,
+    EXC_CASE_FOLDING
 };
 
-#define UPROPS_NTV_GET_TYPE(ntv) \
-    ((ntv==UPROPS_NTV_NONE) ? U_NT_NONE : \
-    (ntv<UPROPS_NTV_DIGIT_START) ?  U_NT_DECIMAL : \
-    (ntv<UPROPS_NTV_NUMERIC_START) ? U_NT_DIGIT : \
-    U_NT_NUMERIC)
-
 /* number of properties vector words */
 #define UPROPS_VECTOR_WORDS     3
 
@@ -83,23 +92,27 @@ enum {
  * Properties in vector word 0
  * Bits
  * 31..24   DerivedAge version major/minor one nibble each
- * 23..20   reserved
- * 19..17   East Asian Width
- * 16.. 8   UBlockCode
- *  7.. 0   UScriptCode
+ * 23       reserved
+ * 22..18   Line Break
+ * 17..15   East Asian Width
+ * 14.. 7   UBlockCode
+ *  6.. 0   UScriptCode
  */
 
 /* derived age: one nibble each for major and minor version numbers */
 #define UPROPS_AGE_MASK         0xff000000
 #define UPROPS_AGE_SHIFT        24
 
-#define UPROPS_EA_MASK          0x000e0000
-#define UPROPS_EA_SHIFT         17
+#define UPROPS_LB_MASK          0x007C0000
+#define UPROPS_LB_SHIFT         18
 
-#define UPROPS_BLOCK_MASK       0x0001ff00
-#define UPROPS_BLOCK_SHIFT      8
+#define UPROPS_EA_MASK          0x00038000
+#define UPROPS_EA_SHIFT         15
 
-#define UPROPS_SCRIPT_MASK      0x000000ff
+#define UPROPS_BLOCK_MASK       0x00007f80
+#define UPROPS_BLOCK_SHIFT      7
+
+#define UPROPS_SCRIPT_MASK      0x0000007f
 
 /*
  * Properties in vector word 1
@@ -114,6 +127,8 @@ enum {
  */
 enum {
     UPROPS_WHITE_SPACE,
+    UPROPS_BIDI_CONTROL,
+    UPROPS_JOIN_CONTROL,
     UPROPS_DASH,
     UPROPS_HYPHEN,
     UPROPS_QUOTATION_MARK,
@@ -125,6 +140,8 @@ enum {
     UPROPS_IDEOGRAPHIC,
     UPROPS_DIACRITIC,
     UPROPS_EXTENDER,
+    UPROPS_LOWERCASE,
+    UPROPS_UPPERCASE,
     UPROPS_NONCHARACTER_CODE_POINT,
     UPROPS_GRAPHEME_EXTEND,
     UPROPS_GRAPHEME_LINK,
@@ -134,42 +151,28 @@ enum {
     UPROPS_UNIFIED_IDEOGRAPH,
     UPROPS_DEFAULT_IGNORABLE_CODE_POINT,
     UPROPS_DEPRECATED,
+    UPROPS_SOFT_DOTTED,
     UPROPS_LOGICAL_ORDER_EXCEPTION,
     UPROPS_XID_START,
     UPROPS_XID_CONTINUE,
     UPROPS_ID_START,                            /* ICU 2.6, uprops format version 3.2 */
     UPROPS_ID_CONTINUE,
     UPROPS_GRAPHEME_BASE,
-    UPROPS_S_TERM,                              /* new in ICU 3.0 and Unicode 4.0.1 */
-    UPROPS_VARIATION_SELECTOR,
-    UPROPS_PATTERN_SYNTAX,                      /* new in ICU 3.4 and Unicode 4.1 */
-    UPROPS_PATTERN_WHITE_SPACE,
-    UPROPS_RESERVED,                            /* reserved & unused */
     UPROPS_BINARY_1_TOP                         /* ==32 - full! */
 };
 
 /*
  * Properties in vector word 2
  * Bits
- * 31..26   reserved
- * 25..20   Line Break
- * 19..15   Sentence Break
- * 14..10   Word Break
- *  9.. 5   Grapheme Cluster Break
+ * 13..11   Joining Type
+ * 10.. 5   Joining Group
  *  4.. 0   Decomposition Type
  */
-#define UPROPS_LB_MASK          0x03f00000
-#define UPROPS_LB_SHIFT         20
-#define UPROPS_LB_VWORD         2
+#define UPROPS_JT_MASK          0x00003800
+#define UPROPS_JT_SHIFT         11
 
-#define UPROPS_SB_MASK          0x000f8000
-#define UPROPS_SB_SHIFT         15
-
-#define UPROPS_WB_MASK          0x00007c00
-#define UPROPS_WB_SHIFT         10
-
-#define UPROPS_GCB_MASK         0x000003e0
-#define UPROPS_GCB_SHIFT        5
+#define UPROPS_JG_MASK          0x000007e0
+#define UPROPS_JG_SHIFT         5
 
 #define UPROPS_DT_MASK          0x0000001f
 
@@ -199,29 +202,32 @@ U_CFUNC int32_t
 uprv_getMaxValues(int32_t column);
 
 /**
- * Checks if c is alphabetic, or a decimal digit; implements UCHAR_POSIX_ALNUM.
+ * \var uprv_comparePropertyNames
+ * Unicode property names and property value names are compared
+ * "loosely". Property[Value]Aliases.txt say:
+ *   "With loose matching of property names, the case distinctions, whitespace,
+ *    and '_' are ignored."
+ *
+ * This function does just that, for (char *) name strings.
+ * It is almost identical to ucnv_compareNames() but also ignores
+ * C0 White_Space characters (U+0009..U+000d, and U+0085 on EBCDIC).
+ *
  * @internal
  */
-U_CFUNC UBool
-u_isalnumPOSIX(UChar32 c);
 
-/**
- * Checks if c is in
- * [^\p{space}\p{gc=Control}\p{gc=Surrogate}\p{gc=Unassigned}]
- * with space=\p{Whitespace} and Control=Cc.
- * Implements UCHAR_POSIX_GRAPH.
- * @internal
- */
-U_CFUNC UBool
-u_isgraphPOSIX(UChar32 c);
+U_CAPI int32_t U_EXPORT2
+uprv_compareASCIIPropertyNames(const char *name1, const char *name2);
 
-/**
- * Checks if c is in \p{graph}\p{blank} - \p{cntrl}.
- * Implements UCHAR_POSIX_PRINT.
- * @internal
- */
-U_CFUNC UBool
-u_isprintPOSIX(UChar32 c);
+U_CAPI int32_t U_EXPORT2
+uprv_compareEBCDICPropertyNames(const char *name1, const char *name2);
+
+#if U_CHARSET_FAMILY==U_ASCII_FAMILY
+#   define uprv_comparePropertyNames uprv_compareASCIIPropertyNames
+#elif U_CHARSET_FAMILY==U_EBCDIC_FAMILY
+#   define uprv_comparePropertyNames uprv_compareEBCDICPropertyNames
+#else
+#   error U_CHARSET_FAMILY is not valid
+#endif
 
 /** Turn a bit index into a bit flag. @internal */
 #define FLAG(n) ((uint32_t)1<<(n))
@@ -232,7 +238,7 @@ u_isprintPOSIX(UChar32 c);
 #define _Ll     FLAG(U_LOWERCASE_LETTER)
 #define _Lt     FLAG(U_TITLECASE_LETTER)
 #define _Lm     FLAG(U_MODIFIER_LETTER)
-/* #define _Lo     FLAG(U_OTHER_LETTER) -- conflicts with MS Visual Studio 9.0 xiosbase */
+#define _Lo     FLAG(U_OTHER_LETTER)
 #define _Mn     FLAG(U_NON_SPACING_MARK)
 #define _Me     FLAG(U_ENCLOSING_MARK)
 #define _Mc     FLAG(U_COMBINING_SPACING_MARK)
@@ -248,15 +254,15 @@ u_isprintPOSIX(UChar32 c);
 #define _Cs     FLAG(U_SURROGATE)
 #define _Pd     FLAG(U_DASH_PUNCTUATION)
 #define _Ps     FLAG(U_START_PUNCTUATION)
-/* #define _Pe     FLAG(U_END_PUNCTUATION) -- conflicts with MS Visual Studio 9.0 xlocnum */
-/* #define _Pc     FLAG(U_CONNECTOR_PUNCTUATION) -- conflicts with MS Visual Studio 9.0 streambuf */
+#define _Pe     FLAG(U_END_PUNCTUATION)
+#define _Pc     FLAG(U_CONNECTOR_PUNCTUATION)
 #define _Po     FLAG(U_OTHER_PUNCTUATION)
 #define _Sm     FLAG(U_MATH_SYMBOL)
 #define _Sc     FLAG(U_CURRENCY_SYMBOL)
 #define _Sk     FLAG(U_MODIFIER_SYMBOL)
 #define _So     FLAG(U_OTHER_SYMBOL)
 #define _Pi     FLAG(U_INITIAL_PUNCTUATION)
-/* #define _Pf     FLAG(U_FINAL_PUNCTUATION) -- conflicts with MS Visual Studio 9.0 streambuf */
+#define _Pf     FLAG(U_FINAL_PUNCTUATION)
 
 /** Some code points. @internal */
 enum {
@@ -265,10 +271,8 @@ enum {
     FF      =0x000c,
     CR      =0x000d,
     U_A     =0x0041,
-    U_F     =0x0046,
     U_Z     =0x005a,
     U_a     =0x0061,
-    U_f     =0x0066,
     U_z     =0x007a,
     DEL     =0x007f,
     NL      =0x0085,
@@ -283,15 +287,24 @@ enum {
     WJ      =0x2060,
     INHSWAP =0x206a,
     NOMDIG  =0x206f,
-    U_FW_A  =0xff21,
-    U_FW_F  =0xff26,
-    U_FW_Z  =0xff3a,
-    U_FW_a  =0xff41,
-    U_FW_f  =0xff46,
-    U_FW_z  =0xff5a,
     ZWNBSP  =0xfeff
 };
 
+/**
+ * Is this character a "white space" in the sense of ICU rule parsers?
+ * @internal
+ */
+U_CAPI UBool U_EXPORT2
+uprv_isRuleWhiteSpace(UChar32 c);
+
+/**
+ * Get the set of "white space" characters in the sense of ICU rule
+ * parsers.  Caller must close/delete result.
+ * @internal
+ */
+U_CAPI USet* U_EXPORT2
+uprv_openRuleWhiteSpaceSet(UErrorCode* ec);
+
 /**
  * Get the maximum length of a (regular/1.0/extended) character name.
  * @return 0 if no character names available.
@@ -299,84 +312,61 @@ enum {
 U_CAPI int32_t U_EXPORT2
 uprv_getMaxCharNameLength(void);
 
+#if 0
+/* 
+Currently not used but left for future use. Probably by UnicodeSet. 
+urename.h and unames.c changed accordingly. 
+*/
+/**
+ * Get the maximum length of an ISO comment.
+ * @return 0 if no ISO comments available.
+ */
+U_CAPI int32_t U_EXPORT2
+uprv_getMaxISOCommentLength();
+#endif
+
 /**
  * Fills set with characters that are used in Unicode character names.
  * Includes all characters that are used in regular/Unicode 1.0/extended names.
  * Just empties the set if no character names are available.
- * @param sa USetAdder to receive characters.
+ * @param set USet to receive characters. Existing contents are deleted.
  */
 U_CAPI void U_EXPORT2
-uprv_getCharNameCharacters(const USetAdder *sa);
+uprv_getCharNameCharacters(USet* set);
 
+#if 0
+/* 
+Currently not used but left for future use. Probably by UnicodeSet. 
+urename.h and unames.c changed accordingly. 
+*/
 /**
- * Constants for which data and implementation files provide which properties.
- * Used by UnicodeSet for service-specific property enumeration.
- * @internal
+ * Fills set with characters that are used in Unicode character names.
+ * Just empties the set if no ISO comments are available.
+ * @param set USet to receive characters. Existing contents are deleted.
  */
-enum UPropertySource {
-    /** No source, not a supported property. */
-    UPROPS_SRC_NONE,
-    /** From uchar.c/uprops.icu main trie */
-    UPROPS_SRC_CHAR,
-    /** From uchar.c/uprops.icu properties vectors trie */
-    UPROPS_SRC_PROPSVEC,
-    /** From unames.c/unames.icu */
-    UPROPS_SRC_NAMES,
-    /** From unorm.cpp/unorm.icu */
-    UPROPS_SRC_NORM,
-    /** From ucase.c/ucase.icu */
-    UPROPS_SRC_CASE,
-    /** From ubidi_props.c/ubidi.icu */
-    UPROPS_SRC_BIDI,
-    /** From uchar.c/uprops.icu main trie as well as properties vectors trie */
-    UPROPS_SRC_CHAR_AND_PROPSVEC,
-    /** From ucase.c/ucase.icu as well as unorm.cpp/unorm.icu */
-    UPROPS_SRC_CASE_AND_NORM,
-    /** From normalizer2impl.cpp/nfc.nrm */
-    UPROPS_SRC_NFC,
-    /** From normalizer2impl.cpp/nfkc.nrm */
-    UPROPS_SRC_NFKC,
-    /** From normalizer2impl.cpp/nfkc_cf.nrm */
-    UPROPS_SRC_NFKC_CF,
-    /** One more than the highest UPropertySource (UPROPS_SRC_) constant. */
-    UPROPS_SRC_COUNT
-};
-typedef enum UPropertySource UPropertySource;
+U_CAPI void U_EXPORT2
+uprv_getISOCommentCharacters(USet* set);
+*/
+#endif
 
 /**
- * @see UPropertySource
- * @internal
- */
-U_CFUNC UPropertySource U_EXPORT2
-uprops_getSource(UProperty which);
-
-/**
- * Enumerate uprops.icu's main data trie and add the
+ * Enumerate each core properties data trie and add the
  * start of each range of same properties to the set.
  * @internal
  */
-U_CFUNC void U_EXPORT2
-uchar_addPropertyStarts(const USetAdder *sa, UErrorCode *pErrorCode);
-
-/**
- * Enumerate uprops.icu's properties vectors trie and add the
- * start of each range of same properties to the set.
- * @internal
- */
-U_CFUNC void U_EXPORT2
-upropsvec_addPropertyStarts(const USetAdder *sa, UErrorCode *pErrorCode);
+U_CAPI void U_EXPORT2
+uchar_addPropertyStarts(USet *set, UErrorCode *pErrorCode);
 
 /**
  * Return a set of characters for property enumeration.
  * For each two consecutive characters (start, limit) in the set,
  * all of the properties for start..limit-1 are all the same.
  *
- * @param sa USetAdder to receive result. Existing contents are lost.
+ * @param set USet to receive result. Existing contents are lost.
  * @internal
  */
-/*U_CFUNC void U_EXPORT2
-uprv_getInclusions(const USetAdder *sa, UErrorCode *pErrorCode);
-*/
+U_CAPI void U_EXPORT2
+uprv_getInclusions(USet* set, UErrorCode *pErrorCode);
 
 /**
  * Swap the ICU Unicode properties file. See uchar.c.
@@ -396,18 +386,4 @@ uchar_swapNames(const UDataSwapper *ds,
                 const void *inData, int32_t length, void *outData,
                 UErrorCode *pErrorCode);
 
-#ifdef XP_CPLUSPLUS
-
-U_NAMESPACE_BEGIN
-
-class UnicodeSet;
-
-// implemented in uniset_props.cpp
-U_CFUNC UnicodeSet *
-uniset_getUnicode32Instance(UErrorCode &errorCode);
-
-U_NAMESPACE_END
-
-#endif
-
 #endif
