@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2002-2006, International Business Machines
+*   Copyright (C) 2002-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -34,162 +34,168 @@ uset_open(UChar32 start, UChar32 end) {
     return (USet*) new UnicodeSet(start, end);
 }
 
+U_CAPI USet* U_EXPORT2
+uset_openPattern(const UChar* pattern, int32_t patternLength,
+                 UErrorCode* ec)
+{
+    UnicodeString pat(patternLength==-1, pattern, patternLength);
+    UnicodeSet* set = new UnicodeSet(pat, *ec);
+    /* test for NULL */
+    if(set == 0) {
+        *ec = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
+    
+    if (U_FAILURE(*ec)) {
+        delete set;
+        set = NULL;
+    }
+    return (USet*) set;
+}
+
+U_CAPI USet* U_EXPORT2
+uset_openPatternOptions(const UChar* pattern, int32_t patternLength,
+                 uint32_t options,
+                 UErrorCode* ec)
+{
+    UnicodeString pat(patternLength==-1, pattern, patternLength);
+    UnicodeSet* set = new UnicodeSet(pat, options, NULL, *ec);
+    /* test for NULL */
+    if(set == 0) {
+        *ec = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
+    
+    if (U_FAILURE(*ec)) {
+        delete set;
+        set = NULL;
+    }
+    return (USet*) set;
+}
+
+
 U_CAPI void U_EXPORT2
 uset_close(USet* set) {
     delete (UnicodeSet*) set;
 }
 
-U_CAPI void U_EXPORT2
-uset_set(USet* set,
-     UChar32 start, UChar32 end) {
-    ((UnicodeSet*) set)->UnicodeSet::set(start, end);
+U_CAPI int32_t U_EXPORT2 
+uset_applyPattern(USet *set,
+                  const UChar *pattern, int32_t patternLength,
+                  uint32_t options,
+                  UErrorCode *status){
+
+    // status code needs to be checked since we 
+    // dereference it
+    if(status == NULL || U_FAILURE(*status)){
+        return 0;
+    }
+
+    // check only the set paramenter
+    // if pattern is NULL or null terminate
+    // UnicodeString constructor takes care of it
+    if(set == NULL){
+        *status = U_ILLEGAL_ARGUMENT_ERROR;
+        return 0;
+    }
+
+    UnicodeString pat(pattern, patternLength);
+
+    ParsePosition pos;
+   
+    ((UnicodeSet*) set)->applyPattern(pat, pos, options, NULL, *status);
+    
+    return pos.getIndex();
+}
+
+U_CAPI int32_t U_EXPORT2
+uset_toPattern(const USet* set,
+               UChar* result, int32_t resultCapacity,
+               UBool escapeUnprintable,
+               UErrorCode* ec) {
+    UnicodeString pat;
+    ((const UnicodeSet*) set)->toPattern(pat, escapeUnprintable);
+    return pat.extract(result, resultCapacity, *ec);
 }
 
 U_CAPI void U_EXPORT2
 uset_addAll(USet* set, const USet *additionalSet) {
-    ((UnicodeSet*) set)->UnicodeSet::addAll(*((const UnicodeSet*)additionalSet));
+    ((UnicodeSet*) set)->addAll(*((const UnicodeSet*)additionalSet));
 }
 
 U_CAPI void U_EXPORT2
 uset_add(USet* set, UChar32 c) {
-    ((UnicodeSet*) set)->UnicodeSet::add(c);
+    ((UnicodeSet*) set)->add(c);
 }
 
 U_CAPI void U_EXPORT2
 uset_addRange(USet* set, UChar32 start, UChar32 end) {
-    ((UnicodeSet*) set)->UnicodeSet::add(start, end);    
+    ((UnicodeSet*) set)->add(start, end);    
 }
 
 U_CAPI void U_EXPORT2
 uset_addString(USet* set, const UChar* str, int32_t strLen) {
-    // WRONG! Do not alias, it will stay aliased, even after 
-    // copying. TODO: do we need a copy ctor that unaliases
+  // WRONG! Do not alias, it will stay aliased, even after 
+  // copying. TODO: do we need a copy ctor that unaliases
     //UnicodeString s(strLen==-1, str, strLen);
-
-    // UnicodeString handles -1 for strLen
+  // We promised -1 for zero terminated
+    if(strLen == -1) {
+      strLen = u_strlen(str);
+    }
     UnicodeString s(str, strLen);
-    ((UnicodeSet*) set)->UnicodeSet::add(s);
-}
-
-U_CAPI void U_EXPORT2
-uset_addAllCodePoints(USet* set, const UChar *str, int32_t strLen) {
-    // UnicodeString handles -1 for strLen
-    UnicodeString s(str, strLen);
-    ((UnicodeSet*) set)->UnicodeSet::addAll(s);
+    ((UnicodeSet*) set)->add(s);
 }
 
 U_CAPI void U_EXPORT2
 uset_remove(USet* set, UChar32 c) {
-    ((UnicodeSet*) set)->UnicodeSet::remove(c);
+    ((UnicodeSet*) set)->remove(c);
 }
 
 U_CAPI void U_EXPORT2
 uset_removeRange(USet* set, UChar32 start, UChar32 end) {
-    ((UnicodeSet*) set)->UnicodeSet::remove(start, end);
+    ((UnicodeSet*) set)->remove(start, end);
 }
 
 U_CAPI void U_EXPORT2
 uset_removeString(USet* set, const UChar* str, int32_t strLen) {
     UnicodeString s(strLen==-1, str, strLen);
-    ((UnicodeSet*) set)->UnicodeSet::remove(s);
-}
-
-U_CAPI void U_EXPORT2
-uset_removeAll(USet* set, const USet* remove) {
-    ((UnicodeSet*) set)->UnicodeSet::removeAll(*(const UnicodeSet*)remove);
-}
-
-U_CAPI void U_EXPORT2
-uset_retain(USet* set, UChar32 start, UChar32 end) {
-    ((UnicodeSet*) set)->UnicodeSet::retain(start, end);
-}
-
-U_CAPI void U_EXPORT2
-uset_retainAll(USet* set, const USet* retain) {
-    ((UnicodeSet*) set)->UnicodeSet::retainAll(*(const UnicodeSet*)retain);
-}
-
-U_CAPI void U_EXPORT2
-uset_compact(USet* set) {
-    ((UnicodeSet*) set)->UnicodeSet::compact();
+    ((UnicodeSet*) set)->remove(s);
 }
 
 U_CAPI void U_EXPORT2
 uset_complement(USet* set) {
-    ((UnicodeSet*) set)->UnicodeSet::complement();
-}
-
-U_CAPI void U_EXPORT2
-uset_complementAll(USet* set, const USet* complement) {
-    ((UnicodeSet*) set)->UnicodeSet::complementAll(*(const UnicodeSet*)complement);
+    ((UnicodeSet*) set)->complement();
 }
 
 U_CAPI void U_EXPORT2
 uset_clear(USet* set) {
-    ((UnicodeSet*) set)->UnicodeSet::clear();
+    ((UnicodeSet*) set)->clear();
 }
 
 U_CAPI UBool U_EXPORT2
 uset_isEmpty(const USet* set) {
-    return ((const UnicodeSet*) set)->UnicodeSet::isEmpty();
+    return ((const UnicodeSet*) set)->isEmpty();
 }
 
 U_CAPI UBool U_EXPORT2
 uset_contains(const USet* set, UChar32 c) {
-    return ((const UnicodeSet*) set)->UnicodeSet::contains(c);
+    return ((const UnicodeSet*) set)->contains(c);
 }
 
 U_CAPI UBool U_EXPORT2
 uset_containsRange(const USet* set, UChar32 start, UChar32 end) {
-    return ((const UnicodeSet*) set)->UnicodeSet::contains(start, end);
+    return ((const UnicodeSet*) set)->contains(start, end);
 }
 
 U_CAPI UBool U_EXPORT2
 uset_containsString(const USet* set, const UChar* str, int32_t strLen) {
     UnicodeString s(strLen==-1, str, strLen);
-    return ((const UnicodeSet*) set)->UnicodeSet::contains(s);
-}
-
-U_CAPI UBool U_EXPORT2
-uset_containsAll(const USet* set1, const USet* set2) {
-    return ((const UnicodeSet*) set1)->UnicodeSet::containsAll(* (const UnicodeSet*) set2);
-}
-
-U_CAPI UBool U_EXPORT2
-uset_containsAllCodePoints(const USet* set, const UChar *str, int32_t strLen) {
-    // Create a string alias, since nothing is being added to the set.
-    UnicodeString s(strLen==-1, str, strLen);
-    return ((const UnicodeSet*) set)->UnicodeSet::containsAll(s);
-}
-
-U_CAPI UBool U_EXPORT2
-uset_containsNone(const USet* set1, const USet* set2) {
-    return ((const UnicodeSet*) set1)->UnicodeSet::containsNone(* (const UnicodeSet*) set2);
-}
-
-U_CAPI UBool U_EXPORT2
-uset_containsSome(const USet* set1, const USet* set2) {
-    return ((const UnicodeSet*) set1)->UnicodeSet::containsSome(* (const UnicodeSet*) set2);
-}
-
-U_CAPI UBool U_EXPORT2
-uset_equals(const USet* set1, const USet* set2) {
-    return *(const UnicodeSet*)set1 == *(const UnicodeSet*)set2;
-}
-
-U_CAPI int32_t U_EXPORT2
-uset_indexOf(const USet* set, UChar32 c) {
-    return ((UnicodeSet*) set)->UnicodeSet::indexOf(c);
-}
-
-U_CAPI UChar32 U_EXPORT2
-uset_charAt(const USet* set, int32_t index) {
-    return ((UnicodeSet*) set)->UnicodeSet::charAt(index);
+    return ((const UnicodeSet*) set)->contains(s);
 }
 
 U_CAPI int32_t U_EXPORT2
 uset_size(const USet* set) {
-    return ((const UnicodeSet*) set)->UnicodeSet::size();
+    return ((const UnicodeSet*) set)->size();
 }
 
 U_NAMESPACE_BEGIN
@@ -265,18 +271,6 @@ uset_getItem(const USet* uset, int32_t itemIndex,
 //    return TRUE;
 //}
 
-U_CAPI USet* U_EXPORT2
-uprv_openRuleWhiteSpaceSet(UErrorCode* ec) {
-    if(U_FAILURE(*ec)) {
-        return NULL;
-    }
-    // create a set with the Pattern_White_Space characters,
-    // without a pattern for fewer code dependencies
-    UnicodeSet *set=new UnicodeSet(9, 0xd);
-    set->UnicodeSet::add(0x20).add(0x85).add(0x200e, 0x200f).add(0x2028, 0x2029);
-    return (USet *)set;
-}
-
 /*
  * Serialize a USet into 16-bit units.
  * Store BMP code points as themselves with one 16-bit unit each.
@@ -301,7 +295,7 @@ uset_serialize(const USet* set, uint16_t* dest, int32_t destCapacity, UErrorCode
         return 0;
     }
 
-    return ((const UnicodeSet*) set)->UnicodeSet::serialize(dest, destCapacity,* ec);
+    return ((const UnicodeSet*) set)->serialize(dest, destCapacity,* ec);
 }
 
 U_CAPI UBool U_EXPORT2
@@ -382,50 +376,19 @@ uset_serializedContains(const USerializedSet* set, UChar32 c) {
     array=set->array;
     if(c<=0xffff) {
         /* find c in the BMP part */
-        int32_t lo = 0;
-        int32_t hi = set->bmpLength-1;
-        if (c < array[0]) {
-            hi = 0;
-        } else if (c < array[hi]) {
-            for(;;) {
-                int32_t i = (lo + hi) >> 1;
-                if (i == lo) {
-                    break;  // Done!
-                } else if (c < array[i]) {
-                    hi = i;
-                } else {
-                    lo = i;
-                }
-            }
-        } else {
-            hi += 1;
-        }
-        return (UBool)(hi&1);
+        int32_t i, bmpLength=set->bmpLength;
+        for(i=0; i<bmpLength && (uint16_t)c>=array[i]; ++i) {}
+        return (UBool)(i&1);
     } else {
         /* find c in the supplementary part */
+        int32_t i, length=set->length;
         uint16_t high=(uint16_t)(c>>16), low=(uint16_t)c;
-        int32_t base = set->bmpLength;
-        int32_t lo = 0;
-        int32_t hi = set->length - 2 - base;
-        if (high < array[base] || (high==array[base] && low<array[base+1])) {
-            hi = 0;
-        } else if (high < array[base+hi] || (high==array[base+hi] && low<array[base+hi+1])) {
-            for (;;) {
-                int32_t i = ((lo + hi) >> 1) & ~1;  // Guarantee even result
-                int32_t iabs = i + base;
-                if (i == lo) {
-                    break;  // Done!
-                } else if (high < array[iabs] || (high==array[iabs] && low<array[iabs+1])) {
-                    hi = i;
-                } else {
-                    lo = i;
-                }
-            }
-        } else {
-            hi += 2;
-        }
+        for(i=set->bmpLength;
+            i<length && (high>array[i] || (high==array[i] && low>=array[i+1]));
+            i+=2) {}
+
         /* count pairs of 16-bit units even per BMP and check if the number of pairs is odd */
-        return (UBool)(((hi+(base<<1))&2)!=0);
+        return (UBool)(((i+set->bmpLength)&2)!=0);
     }
 }
 
