@@ -1,6 +1,6 @@
 /*****************************************************************************
 *
-*   Copyright (C) 1999-2011, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************/
@@ -8,7 +8,7 @@
 /*
  * uconv(1): an iconv(1)-like converter using ICU.
  *
- * Original code by Jonas Utterstr&#x00F6;m <jonas.utterstrom@vittran.norrnod.se>
+ * Original code by Jonas Utterstrm <jonas.utterstrom@vittran.norrnod.se>
  * contributed in 1999.
  *
  * Conversion to the C conversion API and many improvements by
@@ -19,7 +19,6 @@
  */
 
 #include <unicode/utypes.h>
-#include <unicode/putil.h>
 #include <unicode/ucnv.h>
 #include <unicode/uenum.h>
 #include <unicode/unistr.h>
@@ -38,24 +37,10 @@
 
 #include "unicode/uwmsg.h"
 
-U_NAMESPACE_USE
-
-#if (defined(U_WINDOWS) || defined(U_CYGWIN) || defined(U_MINGW)) && !defined(__STRICT_ANSI__)
+#ifdef WIN32
+#include <string.h>
 #include <io.h>
 #include <fcntl.h>
-#if defined(U_WINDOWS)
-#define USE_FILENO_BINARY_MODE 1
-/* Windows likes to rename Unix-like functions */
-#ifndef fileno
-#define fileno _fileno
-#endif
-#ifndef setmode
-#define setmode _setmode
-#endif
-#ifndef O_BINARY
-#define O_BINARY _O_BINARY
-#endif
-#endif
 #endif
 
 #ifdef UCONVMSG_LINK
@@ -259,7 +244,7 @@ static int printConverters(const char *pname, const char *lookfor,
             }
             if (U_FAILURE(err)) {
                 u_wmsg(stderr, "cantGetTag", u_wmsg_errorName(err));
-                goto error_cleanup;
+                return -1;
             }
         }
         if (canon) {
@@ -291,7 +276,7 @@ static int printConverters(const char *pname, const char *lookfor,
             putchar('\t');
             u_wmsg(stderr, "cantGetAliases", str.getTerminatedBuffer(),
                 u_wmsg_errorName(err));
-            goto error_cleanup;
+            return -1;
         } else {
             uint16_t a, s, t;
 
@@ -305,7 +290,7 @@ static int printConverters(const char *pname, const char *lookfor,
                     putchar('\t');
                     u_wmsg(stderr, "cantGetAliases", str.getTerminatedBuffer(),
                         u_wmsg_errorName(err));
-                    goto error_cleanup;
+                    return -1;
                 }
 
                 /* Print the current alias so that it looks right. */
@@ -363,9 +348,6 @@ static int printConverters(const char *pname, const char *lookfor,
     /* Success. */
 
     return 0;
-error_cleanup:
-    uprv_free(stds);
-    return -1;
 }
 
 /* Print all available transliterators. If canon is non zero, print
@@ -377,30 +359,56 @@ static int printTransliterators(UBool canon)
     printf("no transliterators available because of UCONFIG_NO_TRANSLITERATION, see uconfig.h\n");
     return 1;
 #else
-    UErrorCode status = U_ZERO_ERROR;
-    UEnumeration *ids = utrans_openIDs(&status);
-    int32_t i, numtrans = uenum_count(ids, &status);
+    int32_t numtrans = utrans_countAvailableIDs(), i;
+    int buflen = 512;
+    char *buf = (char *) uprv_malloc(buflen);
+    char staticbuf[512];
 
     char sepchar = canon ? '\n' : ' ';
 
-    for (i = 0; U_SUCCESS(status)&& (i < numtrans); ++i) {
-    	int32_t len;
-    	const char *nextTrans = uenum_next(ids, &len, &status);
+    if (!buf) {
+        buf = staticbuf;
+        buflen = sizeof(staticbuf);
+    }
 
-        printf("%s", nextTrans);
+    for (i = 0; i < numtrans; ++i) {
+        int32_t len = utrans_getAvailableID(i, buf, buflen);
+        if (len >= buflen - 1) {
+            if (buf != staticbuf) {
+                buflen <<= 1;
+                if (buflen < len) {
+                    buflen = len + 64;
+                }
+                buf = (char *) uprv_realloc(buf, buflen);
+                if (!buf) {
+                    buf = staticbuf;
+                    buflen = sizeof(staticbuf);
+                }
+            }
+            utrans_getAvailableID(i, buf, buflen);
+            if (len >= buflen) {
+                uprv_strcpy(buf + buflen - 4, "..."); /* Truncate the name. */
+            }
+        }
+
+        printf("%s", buf);
         if (i < numtrans - 1) {
             putchar(sepchar);
         }
     }
 
-    uenum_close(ids);
-
     /* Add a terminating newline if needed. */
 
     if (sepchar != '\n') {
         putchar('\n');
     }
 
+    /* Free temporary data. */
+
+    if (buf != staticbuf) {
+        uprv_free(buf);
+    }
+
     /* Success. */
 
     return 0;
@@ -631,7 +639,7 @@ ConvertFile::convertFile(const char *pname,
     } else {
         infilestr = "-";
         infile = stdin;
-#ifdef USE_FILENO_BINARY_MODE
+#ifdef WIN32
         if (setmode(fileno(stdin), O_BINARY) == -1) {
             initMsg(pname);
             u_wmsg(stderr, "cantSetInBinMode");
@@ -770,7 +778,7 @@ ConvertFile::convertFile(const char *pname,
                 buf + rd, useOffsets ? fromoffsets : NULL, flush, &err);
 
             ulen = (int32_t)(unibufp - unibuf);
-            u.releaseBuffer(U_SUCCESS(err) ? ulen : 0);
+            u.releaseBuffer(ulen);
 
             // fromSawEndOfBytes indicates that ucnv_toUnicode() is done
             // converting all of the input bytes.
@@ -982,7 +990,7 @@ ConvertFile::convertFile(const char *pname,
                         errtag = "problemCvtFromUOut";
                     }
 
-                    length = (int8_t)sprintf(pos, "%u", (int)ferroffset);
+                    length = (int8_t)sprintf(pos, "%u", ferroffset);
 
                     // output the code points that caused the error
                     UnicodeString str;
@@ -1134,7 +1142,7 @@ main(int argc, char **argv)
 
     // Get and prettify pname.
     pname = uprv_strrchr(*argv, U_FILE_SEP_CHAR);
-#ifdef U_WINDOWS
+#ifdef WIN32
     if (!pname) {
         pname = uprv_strrchr(*argv, '/');
     }
@@ -1332,7 +1340,7 @@ main(int argc, char **argv)
     } else {
         outfilestr = "-";
         outfile = stdout;
-#ifdef USE_FILENO_BINARY_MODE
+#ifdef WIN32
         if (setmode(fileno(outfile), O_BINARY) == -1) {
             u_wmsg(stderr, "cantSetOutBinMode");
             exit(-1);
@@ -1367,11 +1375,7 @@ main(int argc, char **argv)
 
     goto normal_exit;
 error_exit:
-#if !UCONFIG_NO_LEGACY_CONVERSION
     ret = 1;
-#else 
-    fprintf(stderr, "uconv error: UCONFIG_NO_LEGACY_CONVERSION is on. See uconfig.h\n");
-#endif
 normal_exit:
 
     if (outfile != stdout) {
