@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1998-2001, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -22,38 +22,71 @@
 #include "unicode/ustdio.h"
 #include "ufile.h"
 #include "unicode/uloc.h"
-#include "loccache.h"
 #include "unicode/ures.h"
 #include "unicode/ucnv.h"
 #include "cstring.h"
 #include "cmemory.h"
 
 
-static UBool hasICUData(const char *cp) {
-    UErrorCode status = U_ZERO_ERROR;
-    UConverter *cnv = NULL;
-#if 0
-    UResourceBundle *r = NULL;
-
-    r = ures_open(NULL, NULL, &status);
-    if(U_FAILURE(status)) {
-        return FALSE;
-    } else {
-        ures_close(r);
+U_CAPI UFILE* U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
+u_finit(FILE        *f,
+        const char    *locale,
+        const char    *codepage)
+{
+    UErrorCode status     = U_ZERO_ERROR;
+    UBool     useSysCP    = (UBool)(locale == NULL && codepage == NULL);
+    UFILE     *result     = (UFILE*) uprv_malloc(sizeof(UFILE));
+    if(result == NULL || f == NULL) {
+        return 0;
     }
+
+    uprv_memset(result, 0, sizeof(UFILE));
+
+#ifdef WIN32
+    result->fFile = &_iob[_fileno(f)];
+#else
+    result->fFile = f;
 #endif
-    cnv = ucnv_open(cp, &status);
-    if(cnv == NULL) {
-        return FALSE;
-    } else {
-        ucnv_close(cnv);
+    result->fUCPos     = result->fUCBuffer;
+    result->fUCLimit     = result->fUCBuffer;
+
+#if !UCONFIG_NO_FORMATTING
+        /* if locale is 0, use the default */
+        if(locale == 0) {
+            locale = uloc_getDefault();
+        }
+
+        if(u_locbund_init(&result->fBundle, locale) == 0) {
+            /* DO NOT FCLOSE HERE! */
+            uprv_free(result);
+            return 0;
+        }
+#endif
+
+    /* if the codepage is NULL, use the default for the locale */
+    if(codepage == NULL) {
+        if(!useSysCP) { /* if both locale and codepage are NULL, use the system default codepage */
+            codepage = uprv_defaultCodePageForLocale(locale);
+        }
+
+        /* if the codepage is still NULL, the default codepage will be used */
+        result->fConverter = ucnv_open(codepage, &status);
+    } else if (*codepage != '\0') {
+        result->fConverter = ucnv_open(codepage, &status);
     }
 
-    return TRUE;
+    if(U_FAILURE(status)) {
+#if !UCONFIG_NO_FORMATTING
+        u_locbund_close(&result->fBundle);
+#endif
+        /* DO NOT fclose here!!!!!! */
+        uprv_free(result);
+        result = NULL;
+    }
+
+    return result;
 }
 
-
-
 U_CAPI UFILE* U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fopen(const char    *filename,
         const char    *perm,
@@ -75,74 +108,6 @@ u_fopen(const char    *filename,
     return result;
 }
 
-U_CAPI UFILE* U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
-u_finit(FILE        *f,
-        const char    *locale,
-        const char    *codepage)
-{
-    UErrorCode status     = U_ZERO_ERROR;
-    UBool     useSysCP    = (UBool)(locale == NULL && codepage == NULL);
-    UFILE     *result     = (UFILE*) uprv_malloc(sizeof(UFILE));
-    if(result == NULL || f == NULL) {
-        return 0;
-    }
-
-
-#ifdef WIN32
-    result->fFile = &_iob[_fileno(f)];
-#else
-    result->fFile = f;
-#endif
-    result->fTranslit = NULL;
-    result->fOwnFile = FALSE;
-    result->fOwnBundle     = FALSE;
-    result->fUCPos     = result->fUCBuffer;
-    result->fUCLimit     = result->fUCBuffer;
-    result->fConverter = NULL;
-    result->fBundle = NULL;
-
-    if(hasICUData(codepage)) {
-        /* if locale is 0, use the default */
-        if(locale == 0) {
-            locale = uloc_getDefault();
-        }
-
-        result->fBundle = u_loccache_get(locale);
-        if(result->fBundle == 0) {
-            /* DO NOT FCLOSE HERE! */
-            uprv_free(result);
-            return 0;
-        }
-    } else {
-        /* bootstrap mode */
-        return result;
-    }
-
-    /* if the codepage is NULL, use the default for the locale */
-    if(codepage == NULL) {
-        if(!useSysCP) { /* if both locale and codepage are NULL, use the system default codepage */
-            codepage = uprv_defaultCodePageForLocale(locale);
-        }
-
-        /* if the codepage is still NULL, the default codepage will be used */
-        result->fConverter = ucnv_open(codepage, &status);
-        if(U_FAILURE(status) || result->fConverter == NULL) {
-            /* DO NOT fclose here!!!!!! */
-            uprv_free(result);
-            return 0;
-        }
-    } else if (*codepage != '\0') {
-        result->fConverter = ucnv_open(codepage, &status);
-        if(U_FAILURE(status) || result->fConverter == NULL) {
-            /* DO NOT fclose here!!!!!! */
-            uprv_free(result);
-            return 0;
-        }
-    }
-    return result;
-}
-
-
 U_CAPI void U_EXPORT2
 u_fflush(UFILE *file)
 {
@@ -151,6 +116,16 @@ u_fflush(UFILE *file)
   /* TODO: flush input */
 }
 
+U_CAPI void
+u_frewind(UFILE *file)
+{
+    u_fflush(file);
+    rewind(file->fFile);
+    ucnv_reset(file->fConverter);
+    file->fUCPos   = file->fUCBuffer;
+    file->fUCLimit = file->fUCBuffer;
+}
+
 U_CAPI void U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fclose(UFILE *file)
 {
@@ -160,8 +135,9 @@ u_fclose(UFILE *file)
     if(file->fOwnFile)
         fclose(file->fFile);
 
-    if(file->fOwnBundle)
-        u_locbund_delete(file->fBundle);
+#if !UCONFIG_NO_FORMATTING
+    u_locbund_close(&file->fBundle);
+#endif
 
     ucnv_close(file->fConverter);
     uprv_free(file);
@@ -173,25 +149,25 @@ u_fgetfile(    UFILE         *f)
     return f->fFile;
 }
 
+#if !UCONFIG_NO_FORMATTING
+
 U_CAPI const char*  U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fgetlocale(    UFILE        *file)
 {
-    return file->fBundle->fLocale;
+    return file->fBundle.fLocale;
 }
 
 U_CAPI int32_t U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fsetlocale(const char        *locale,
              UFILE        *file)
 {
-    if(file->fOwnBundle)
-        u_locbund_delete(file->fBundle);
+    u_locbund_close(&file->fBundle);
 
-    file->fBundle     = u_loccache_get(locale);
-    file->fOwnBundle     = FALSE;
-
-    return file->fBundle == 0 ? -1 : 0;
+    return u_locbund_init(&file->fBundle, locale) == 0 ? -1 : 0;
 }
 
+#endif
+
 U_CAPI const char* U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fgetcodepage(UFILE        *file)
 {
@@ -211,19 +187,26 @@ u_fsetcodepage(    const char    *codepage,
                UFILE        *file)
 {
     UErrorCode status = U_ZERO_ERROR;
+    int32_t retVal = -1;
 
+    /* We use the normal default codepage for this system, and not the one for the locale. */
+#if 0 /* !UCONFIG_NO_FORMATTING */
     /* if the codepage is 0, use the default for the locale */
     if(codepage == 0) {
-        codepage = uprv_defaultCodePageForLocale(file->fBundle->fLocale);
+        codepage = uprv_defaultCodePageForLocale(file->fBundle.fLocale);
 
         /* if the codepage is still 0, fall back on the default codepage */
     }
+#endif
 
-    ucnv_close(file->fConverter);
-    file->fConverter = ucnv_open(codepage, &status);
-    if(U_FAILURE(status))
-        return -1;
-    return 0;
+    if ((file->fUCPos == file->fUCBuffer) && (file->fUCLimit == file->fUCBuffer)) {
+        ucnv_close(file->fConverter);
+        file->fConverter = ucnv_open(codepage, &status);
+        if(U_SUCCESS(status)) {
+            retVal = 0;
+        }
+    }
+    return retVal;
 }
 
 
@@ -232,3 +215,4 @@ u_fgetConverter(UFILE *file)
 {
     return file->fConverter;
 }
+
