@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1998-2001, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -27,21 +27,41 @@
 
 #include <string.h>
 
-#define DELIM_CR 0x000D
 #define DELIM_LF 0x000A
+#define DELIM_VT 0x000B
+#define DELIM_FF 0x000C
+#define DELIM_CR 0x000D
+#define DELIM_NEL 0x0085
+#define DELIM_LS 0x2028
+#define DELIM_PS 0x2029
 
+/* Leave this copyright notice here! */
+static const char copyright[] = U_COPYRIGHT_STRING;
+
+/* TODO: is this correct for all codepages? Should we just use \n and let the converter handle it? */
 #ifdef WIN32
 static const UChar DELIMITERS [] = { DELIM_CR, DELIM_LF, 0x0000 };
 static const uint32_t DELIMITERS_LEN = 2;
+#elif (U_CHARSET_FAMILY == U_EBCDIC_FAMILY)
+static const UChar DELIMITERS [] = { DELIM_NEL, 0x0000 };
+static const uint32_t DELIMITERS_LEN = 1;
 #else
 static const UChar DELIMITERS [] = { DELIM_LF, 0x0000 };
 static const uint32_t DELIMITERS_LEN = 1;
 #endif
 
-#define IS_STRING_DELIMITER(s)    (UBool)(    (s) == DELIM_CR || \
-(s) == DELIM_LF    )
+#define IS_FIRST_STRING_DELIMITER(c1) \
+ (UBool)((DELIM_LF <= (c1) && (c1) <= DELIM_CR) \
+        || (c1) == DELIM_NEL \
+        || (c1) == DELIM_LS \
+        || (c1) == DELIM_PS)
+#define CAN_HAVE_COMBINED_STRING_DELIMITER(c1) (UBool)((c1) == DELIM_CR)
+#define IS_COMBINED_STRING_DELIMITER(c1, c2) \
+ (UBool)((c1) == DELIM_CR && (c2) == DELIM_LF)
 
 
+#if !UCONFIG_NO_TRANSLITERATION
+
 U_CAPI UTransliterator* U_EXPORT2
 u_fsettransliterator(UFILE *file, UFileDirection direction,
                      UTransliterator *adopt, UErrorCode *status)
@@ -105,7 +125,6 @@ u_fsettransliterator(UFILE *file, UFileDirection direction,
     return old;
 }
 
-
 static const UChar * u_file_translit(UFILE *f, const UChar *src, int32_t *count, UBool flush)
 {
     int32_t newlen;
@@ -173,11 +192,13 @@ static const UChar * u_file_translit(UFILE *f, const UChar *src, int32_t *count,
             &pos,
             &status);
 
+#ifdef _DEBUG
         if(U_FAILURE(status))
         {
             fprintf(stderr, " Gack. Translit blew up with a %s\n", u_errorName(status));
             return src;
         }
+#endif
 
         /* now: start/limit point to the transliterated text */
         /* Transliterated is [buffer..pos.start) */
@@ -200,11 +221,13 @@ static const UChar * u_file_translit(UFILE *f, const UChar *src, int32_t *count,
             &textLimit,
             &status);
 
+#ifdef _DEBUG
         if(U_FAILURE(status))
         {
             fprintf(stderr, " Gack. Translit(flush) blew up with a %s\n", u_errorName(status));
             return src;
         }
+#endif
 
         /* out: converted len */
         *count = textLimit;
@@ -217,12 +240,15 @@ static const UChar * u_file_translit(UFILE *f, const UChar *src, int32_t *count,
     }
 }
 
+#endif
 
 void
 ufile_flush_translit(UFILE *f)
 {
+#if !UCONFIG_NO_TRANSLITERATION
     if((!f)||(!f->fTranslit))
         return;
+#endif
 
     u_file_write_flush(NULL, 0, f, TRUE);
 }
@@ -231,11 +257,14 @@ ufile_flush_translit(UFILE *f)
 void
 ufile_close_translit(UFILE *f)
 {
+#if !UCONFIG_NO_TRANSLITERATION
     if((!f)||(!f->fTranslit))
         return;
+#endif
 
     ufile_flush_translit(f);
 
+#if !UCONFIG_NO_TRANSLITERATION
     if(f->fTranslit->translit)
         utrans_close(f->fTranslit->translit);
 
@@ -246,6 +275,7 @@ ufile_close_translit(UFILE *f)
 
     uprv_free(f->fTranslit);
     f->fTranslit = NULL;
+#endif
 }
 
 
@@ -275,15 +305,21 @@ u_file_write_flush(    const UChar     *chars,
                    UBool         flush)
 {
     /* Set up conversion parameters */
-    UErrorCode         status        = U_ZERO_ERROR;
-    const UChar        *mySource       = chars;
-    const UChar        *sourceAlias       = chars;
-    const UChar        *mySourceEnd     = chars + count;
-    char            *myTarget     = f->fCharBuffer;
-    int32_t        bufferSize    = UFILE_CHARBUFFER_SIZE;
-    int32_t        written        = 0;
-    int32_t        numConverted   = 0;
+    UErrorCode         status       = U_ZERO_ERROR;
+    const UChar        *mySource    = chars;
+    const UChar        *sourceAlias = chars;
+    const UChar        *mySourceEnd;
+    char            *myTarget   = f->fCharBuffer;
+    int32_t        bufferSize   = UFILE_CHARBUFFER_SIZE;
+    int32_t        written      = 0;
+    int32_t        numConverted = 0;
 
+    if (count < 0) {
+        count = u_strlen(chars);
+    }
+    mySourceEnd     = chars + count;
+
+#if !UCONFIG_NO_TRANSLITERATION
     if((f->fTranslit) && (f->fTranslit->translit))
     {
         /* Do the transliteration */
@@ -291,6 +327,7 @@ u_file_write_flush(    const UChar     *chars,
         sourceAlias = mySource;
         mySourceEnd = mySource + count;
     }
+#endif
 
     /* Perform the conversion in a loop */
     do {
@@ -309,7 +346,7 @@ u_file_write_flush(    const UChar     *chars,
             u_UCharsToChars(mySource, myTarget, count);
             myTarget += count;
         }
-        numConverted = (myTarget - f->fCharBuffer);
+        numConverted = (int32_t)(myTarget - f->fCharBuffer);
 
         if (numConverted > 0) {
             /* write the converted bytes */
@@ -353,7 +390,7 @@ ufile_fill_uchar_buffer(UFILE *f)
 
 
     /* shift the buffer if it isn't empty */
-    dataSize = f->fUCLimit - f->fUCPos;
+    dataSize = (int32_t)(f->fUCLimit - f->fUCPos);
     if(dataSize != 0) {
         memmove(f->fUCBuffer,
             f->fUCPos,
@@ -369,17 +406,17 @@ ufile_fill_uchar_buffer(UFILE *f)
     maxCPBytes = availLength / (f->fConverter!=NULL?(2*ucnv_getMinCharSize(f->fConverter)):1);
 
     /* Read in the data to convert */
-    bytesRead = fread(f->fCharBuffer,
+    bytesRead = (int32_t)fread(f->fCharBuffer,
         sizeof(char),
         ufmt_min(maxCPBytes, UFILE_CHARBUFFER_SIZE),
         f->fFile);
 
     /* Set up conversion parameters */
-    status    = U_ZERO_ERROR;
-    mySource       = f->fCharBuffer;
-    mySourceEnd     = f->fCharBuffer + bytesRead;
-    myTarget     = f->fUCBuffer + dataSize;
-    bufferSize    = UFILE_UCHARBUFFER_SIZE;
+    status      = U_ZERO_ERROR;
+    mySource    = f->fCharBuffer;
+    mySourceEnd = f->fCharBuffer + bytesRead;
+    myTarget    = f->fUCBuffer + dataSize;
+    bufferSize  = UFILE_UCHARBUFFER_SIZE;
 
     if(f->fConverter != NULL) { /* We have a valid converter */
         /* Perform the conversion */
@@ -403,103 +440,81 @@ ufile_fill_uchar_buffer(UFILE *f)
 }
 
 U_CAPI UChar* U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
-u_fgets(UFILE        *f,
-        int32_t        n,
-        UChar        *s)
+u_fgets(UChar        *s,
+        int32_t       n,
+        UFILE        *f)
 {
     int32_t dataSize;
-    int32_t read;
     int32_t count;
     UChar *alias;
+    UChar *limit;
+    UChar *sItr;
+    UChar currDelim = 0;
 
     if (n <= 0) {
         /* Caller screwed up. We need to write the null terminatior. */
         return NULL;
     }
 
-    /* fill the buffer */
-    ufile_fill_uchar_buffer(f);
+    /* fill the buffer if needed */
+    if (f->fUCPos >= f->fUCLimit) {
+        ufile_fill_uchar_buffer(f);
+    }
 
     /* subtract 1 from n to compensate for the terminator */
     --n;
 
     /* determine the amount of data in the buffer */
-    dataSize = f->fUCLimit - f->fUCPos;
+    dataSize = (int32_t)(f->fUCLimit - f->fUCPos);
 
-    /* if the buffer contains more data than requested, operate on the buffer */
-    if(dataSize > n) {
-
-        /* find the first occurrence of a delimiter character */
-        alias = f->fUCPos;
-        count = 0;
-        while( ! IS_STRING_DELIMITER(*alias) && count < n) {
-            count++;
-            alias++;
-        }
-        /* Preserve the newline */
-        if (IS_STRING_DELIMITER(*alias) && count < n) {
-            count++;
-            alias++;
-        }
-
-        /* copy the characters into the target*/
-        memcpy(s, f->fUCPos, count * sizeof(UChar));
-
-        /* add the terminator */
-        s[count] = 0x0000;
-
-        /* update the current buffer position */
-        f->fUCPos += count;
-
-        /* refill the buffer */
-        ufile_fill_uchar_buffer(f);
-
-        /* skip over any remaining delimiters */
-        while(IS_STRING_DELIMITER(*(f->fUCPos)) && f->fUCPos < f->fUCLimit)
-            (f->fUCPos)++;
-
-        /* return s */
-        return s;
-    }
+    /* if 0 characters were left, return 0 */
+    if (dataSize == 0)
+        return NULL;
 
     /* otherwise, iteratively fill the buffer and copy */
-    read = 0;
-    do {
-
-        /* determine the amount of data in the buffer */
-        dataSize = f->fUCLimit - f->fUCPos;
-
-        /* find the first occurrence of a delimiter character, if present */
+    count = 0;
+    sItr = s;
+    currDelim = 0;
+    while (dataSize > 0 && count < n) {
         alias = f->fUCPos;
-        count = 0;
-        while( ! IS_STRING_DELIMITER(*alias) && alias < f->fUCLimit && count < n) {
-            ++count;
-            alias++;
+
+        /* Find how much to copy */
+        if (dataSize < n) {
+            limit = f->fUCLimit;
         }
-        /* Preserve the newline */
-        if (IS_STRING_DELIMITER(*alias) && count < n) {
-            count++;
-            alias++;
+        else {
+            limit = alias + n;
         }
 
-        /* copy the current data in the buffer */
-        memcpy(s + read, f->fUCPos, count * sizeof(UChar));
-
-        /* update number of items read */
-        read += count;
+        if (!currDelim) {
+            /* Copy UChars until we find the first occurrence of a delimiter character */
+            while (alias < limit && !IS_FIRST_STRING_DELIMITER(*alias)) {
+                count++;
+                *(sItr++) = *(alias++);
+            }
+            /* Preserve the newline */
+            if (alias < limit && IS_FIRST_STRING_DELIMITER(*alias)) {
+                if (CAN_HAVE_COMBINED_STRING_DELIMITER(*alias)) {
+                    currDelim = *alias;
+                }
+                count++;
+                *(sItr++) = *(alias++);
+            }
+        }
+        /* If we have a CRLF combination, preserve that too. */
+        if (alias < limit) {
+            if (currDelim && IS_COMBINED_STRING_DELIMITER(currDelim, *alias)) {
+                count++;
+                *(sItr++) = *(alias++);
+            }
+            currDelim = 0;
+        }
 
         /* update the current buffer position */
-        f->fUCPos += count;
+        f->fUCPos = alias;
 
         /* if we found a delimiter */
-        if(alias < f->fUCLimit) {
-
-            /* refill the buffer */
-            ufile_fill_uchar_buffer(f);
-
-            /* skip over any remaining delimiters */
-            while(IS_STRING_DELIMITER(*(f->fUCPos)) && f->fUCPos < f->fUCLimit)
-                (f->fUCPos)++;
+        if (alias < f->fUCLimit && !currDelim) {
 
             /* break out */
             break;
@@ -508,14 +523,12 @@ u_fgets(UFILE        *f,
         /* refill the buffer */
         ufile_fill_uchar_buffer(f);
 
-    } while(dataSize != 0 && read < n);
-
-    /* if 0 characters were read, return 0 */
-    if(read == 0)
-        return 0;
+        /* determine the amount of data in the buffer */
+        dataSize = (int32_t)(f->fUCLimit - f->fUCPos);
+    }
 
     /* add the terminator and return s */
-    s[read] = 0x0000;
+    *sItr = 0x0000;
     return s;
 }
 
@@ -528,77 +541,70 @@ u_fgetc(UFILE        *f)
     /* otherwise, fill the buffer and return the next character */
     else {
         ufile_fill_uchar_buffer(f);
-        if(f->fUCPos < f->fUCLimit)
+        if(f->fUCPos < f->fUCLimit) {
             return *(f->fUCPos)++;
-        else
-            return 0xFFFF;
+        }
+        else {
+            return U_EOF;
+        }
     }
 }
 
-/* u_unescapeAt() callback to return a UChar from a UFILE */
-static UChar _charAt(int32_t offset, void *context) {
-    return ((UFILE*) context)->fUCPos[offset];
-}
-
 /* Read a UChar from a UFILE and process escape sequences */
 U_CAPI UChar32 U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
 u_fgetcx(UFILE        *f)
 {
-    int32_t length;
-    int32_t offset;
     UChar32 c32;
-    UChar c16;
 
     /* Fill the buffer if it is empty */
-    if (f->fUCPos >= f->fUCLimit) {
+    if (f->fUCPos + 1 >= f->fUCLimit) {
         ufile_fill_uchar_buffer(f);
     }
 
     /* Get the next character in the buffer */
     if (f->fUCPos < f->fUCLimit) {
-        c16 = *(f->fUCPos)++;
-    } else {
-        c16 = U_EOF;
+        c32 = *(f->fUCPos)++;
+    }
+    else {
+        c32 = U_EOF;
     }
 
-    /* If it isn't a backslash, return it */
-    if (c16 != 0x005C /*'\\'*/) {
-        return c16;
+    if (U_IS_LEAD(c32)) {
+        if (f->fUCPos < f->fUCLimit) {
+            UChar c16 = *(f->fUCPos)++;
+            c32 = U16_GET_SUPPLEMENTARY(c32, c16);
+        }
+        else {
+            c32 = U_EOF;
+        }
     }
 
-    /* Determine the amount of data in the buffer */
-    length = f->fUCLimit - f->fUCPos;
-
-    /* The longest escape sequence is \Uhhhhhhhh; make sure
-    we have at least that many characters */
-    if (length < 10) {
-        /* fill the buffer */
-        ufile_fill_uchar_buffer(f);
-        length = f->fUCLimit - f->fUCPos;
-    }
-
-    /* Process the escape */
-    offset = 0;
-    c32 = u_unescapeAt(_charAt, &offset, length, (void*)f);
-
-    /* Update the current buffer position */
-    f->fUCPos += offset;
-
     return c32;
 }
 
-U_CAPI UChar U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
-u_fungetc(UChar        c,
+U_CAPI UChar32 U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
+u_fungetc(UChar32        ch,
     UFILE        *f)
 {
     /* if we're at the beginning of the buffer, sorry! */
-    if(f->fUCPos == f->fUCBuffer)
-        return 0xFFFF;
-    /* otherwise, put the character back */
-    else {
-        *--(f->fUCPos) = c;
-        return c;
+    if (f->fUCPos == f->fUCBuffer
+        || (U_IS_LEAD(ch) && (f->fUCPos - 1) == f->fUCBuffer))
+    {
+        ch = U_EOF;
     }
+    else {
+        /* otherwise, put the character back */
+        /* TODO: Maybe we shouldn't be writing to the buffer and just verify the contents */
+        if (U_IS_LEAD(ch)) {
+            /* Remember, put them back on in the reverse order. */
+            *--(f->fUCPos) = U16_TRAIL(ch);
+            *--(f->fUCPos) = U16_LEAD(ch);
+        }
+        else {
+            *--(f->fUCPos) = (UChar)ch;
+        }
+    }
+    return ch;
 }
 
 U_CAPI int32_t U_EXPORT2 /* U_CAPI ... U_EXPORT2 added by Peter Kirk 17 Nov 2001 */
@@ -613,7 +619,7 @@ u_file_read(    UChar        *chars,
     ufile_fill_uchar_buffer(f);
 
     /* determine the amount of data in the buffer */
-    dataSize = f->fUCLimit - f->fUCPos;
+    dataSize = (int32_t)(f->fUCLimit - f->fUCPos);
 
     /* if the buffer contains the amount requested, just copy */
     if(dataSize > count) {
@@ -631,7 +637,7 @@ u_file_read(    UChar        *chars,
     do {
 
         /* determine the amount of data in the buffer */
-        dataSize = f->fUCLimit - f->fUCPos;
+        dataSize = (int32_t)(f->fUCLimit - f->fUCPos);
 
         /* copy the current data in the buffer */
         memcpy(chars + read, f->fUCPos, dataSize * sizeof(UChar));
