@@ -1,41 +1,41 @@
 /*
- ******************************************************************************
- * Copyright (C) 1996-2007, International Business Machines Corporation and   *
- * others. All Rights Reserved.                                               *
- ******************************************************************************
- */
+******************************************************************************
+* Copyright (C) 1996-2003, International Business Machines Corporation and   *
+* others. All Rights Reserved.                                               *
+******************************************************************************
+*/
 
 /**
- * File coll.cpp
- *
- * Created by: Helena Shih
- *
- * Modification History:
- *
- *  Date        Name        Description
- *  2/5/97      aliu        Modified createDefault to load collation data from
- *                          binary files when possible.  Added related methods
- *                          createCollationFromFile, chopLocale, createPathName.
- *  2/11/97     aliu        Added methods addToCache, findInCache, which implement
- *                          a Collation cache.  Modified createDefault to look in
- *                          cache first, and also to store newly created Collation
- *                          objects in the cache.  Modified to not use gLocPath.
- *  2/12/97     aliu        Modified to create objects from RuleBasedCollator cache.
- *                          Moved cache out of Collation class.
- *  2/13/97     aliu        Moved several methods out of this class and into
- *                          RuleBasedCollator, with modifications.  Modified
- *                          createDefault() to call new RuleBasedCollator(Locale&)
- *                          constructor.  General clean up and documentation.
- *  2/20/97     helena      Added clone, operator==, operator!=, operator=, and copy
- *                          constructor.
- * 05/06/97     helena      Added memory allocation error detection.
- * 05/08/97     helena      Added createInstance().
- *  6/20/97     helena      Java class name change.
- * 04/23/99     stephen     Removed EDecompositionMode, merged with 
- *                          Normalizer::EMode
- * 11/23/9      srl         Inlining of some critical functions
- * 01/29/01     synwee      Modified into a C++ wrapper calling C APIs (ucol.h)
- */
+* File coll.cpp
+*
+* Created by: Helena Shih
+*
+* Modification History:
+*
+*  Date        Name        Description
+*  2/5/97      aliu        Modified createDefault to load collation data from
+*                          binary files when possible.  Added related methods
+*                          createCollationFromFile, chopLocale, createPathName.
+*  2/11/97     aliu        Added methods addToCache, findInCache, which implement
+*                          a Collation cache.  Modified createDefault to look in
+*                          cache first, and also to store newly created Collation
+*                          objects in the cache.  Modified to not use gLocPath.
+*  2/12/97     aliu        Modified to create objects from RuleBasedCollator cache.
+*                          Moved cache out of Collation class.
+*  2/13/97     aliu        Moved several methods out of this class and into
+*                          RuleBasedCollator, with modifications.  Modified
+*                          createDefault() to call new RuleBasedCollator(Locale&)
+*                          constructor.  General clean up and documentation.
+*  2/20/97     helena      Added clone, operator==, operator!=, operator=, and copy
+*                          constructor.
+* 05/06/97     helena      Added memory allocation error detection.
+* 05/08/97     helena      Added createInstance().
+*  6/20/97     helena      Java class name change.
+* 04/23/99     stephen     Removed EDecompositionMode, merged with 
+*                          Normalizer::EMode
+* 11/23/9      srl         Inlining of some critical functions
+* 01/29/01     synwee      Modified into a C++ wrapper calling C APIs (ucol.h)
+*/
 
 #include "unicode/utypes.h"
 
@@ -43,44 +43,13 @@
 
 #include "unicode/coll.h"
 #include "unicode/tblcoll.h"
-#include "ucol_imp.h"
-#include "cstring.h"
 #include "cmemory.h"
-#include "umutex.h"
-#include "servloc.h"
-#include "ustrenum.h"
+#include "mutex.h"
+#include "iculserv.h"
 #include "ucln_in.h"
 
-static U_NAMESPACE_QUALIFIER Locale* availableLocaleList = NULL;
-static int32_t  availableLocaleListCount;
-static U_NAMESPACE_QUALIFIER ICULocaleService* gService = NULL;
-
-/**
- * Release all static memory held by collator.
- */
-U_CDECL_BEGIN
-static UBool U_CALLCONV collator_cleanup(void) {
-#if !UCONFIG_NO_SERVICE
-    if (gService) {
-        delete gService;
-        gService = NULL;
-    }
-#endif
-    if (availableLocaleList) {
-        delete []availableLocaleList;
-        availableLocaleList = NULL;
-    }
-    availableLocaleListCount = 0;
-
-    return TRUE;
-}
-
-U_CDECL_END
-
 U_NAMESPACE_BEGIN
 
-#if !UCONFIG_NO_SERVICE
-
 // ------------------------------------------
 //
 // Registration
@@ -88,10 +57,6 @@ U_NAMESPACE_BEGIN
 
 //-------------------------------------------
 
-CollatorFactory::~CollatorFactory() {}
-
-//-------------------------------------------
-
 UBool
 CollatorFactory::visible(void) const {
     return TRUE;
@@ -110,9 +75,6 @@ CollatorFactory::getDisplayName(const Locale& objectLocale,
 // -------------------------------------
 
 class ICUCollatorFactory : public ICUResourceBundleFactory {
- public:
-    ICUCollatorFactory():  ICUResourceBundleFactory(UnicodeString(U_ICUDATA_COLL, -1, US_INV)) { } 
- protected:
     virtual UObject* create(const ICUServiceKey& key, const ICUService* service, UErrorCode& status) const;
 };
 
@@ -136,7 +98,7 @@ ICUCollatorFactory::create(const ICUServiceKey& key, const ICUService* /* servic
 class ICUCollatorService : public ICULocaleService {
 public:
     ICUCollatorService()
-        : ICULocaleService(UNICODE_STRING_SIMPLE("Collator"))
+        : ICULocaleService("Collator")
     {
         UErrorCode status = U_ZERO_ERROR;
         registerFactory(new ICUCollatorFactory(), status);
@@ -189,26 +151,31 @@ public:
 
 // -------------------------------------
 
+class ICUCollatorService;
+
+static ICULocaleService* gService = NULL;
+
 static ICULocaleService* 
 getService(void)
 {
     UBool needInit;
-    UMTX_CHECK(NULL, (UBool)(gService == NULL), needInit);
+    {
+        Mutex mutex;
+        needInit = (UBool)(gService == NULL);
+    }
     if(needInit) {
         ICULocaleService *newservice = new ICUCollatorService();
         if(newservice) {
-            umtx_lock(NULL);
+            Mutex mutex;
             if(gService == NULL) {
                 gService = newservice;
                 newservice = NULL;
             }
-            umtx_unlock(NULL);
         }
         if(newservice) {
             delete newservice;
-        }
-        else {
-            ucln_i18n_registerCleanup(UCLN_I18N_COLLATOR, collator_cleanup);
+        } else {
+            ucln_i18n_registerCleanup();
         }
     }
     return gService;
@@ -216,12 +183,11 @@ getService(void)
 
 // -------------------------------------
 
-static inline UBool
+static UBool
 hasService(void) 
 {
-    UBool retVal;
-    UMTX_CHECK(NULL, gService != NULL, retVal);
-    return retVal;
+    Mutex mutex;
+    return gService != NULL;
 }
 
 // -------------------------------------
@@ -247,73 +213,22 @@ Collator::createUCollator(const char *loc,
     }
     return result;
 }
-#endif /* UCONFIG_NO_SERVICE */
-
-static UBool isAvailableLocaleListInitialized(UErrorCode &status) {
-    // for now, there is a hardcoded list, so just walk through that list and set it up.
-    UBool needInit;
-    UMTX_CHECK(NULL, availableLocaleList == NULL, needInit);
-
-    if (needInit) {
-        UResourceBundle *index = NULL;
-        UResourceBundle installed;
-        Locale * temp;
-        int32_t i = 0;
-        int32_t localeCount;
-        
-        ures_initStackObject(&installed);
-        index = ures_openDirect(U_ICUDATA_COLL, "res_index", &status);
-        ures_getByKey(index, "InstalledLocales", &installed, &status);
-        
-        if(U_SUCCESS(status)) {
-            localeCount = ures_getSize(&installed);
-            temp = new Locale[localeCount];
-            
-            if (temp != NULL) {
-                ures_resetIterator(&installed);
-                while(ures_hasNext(&installed)) {
-                    const char *tempKey = NULL;
-                    ures_getNextString(&installed, NULL, &tempKey, &status);
-                    temp[i++] = Locale(tempKey);
-                }
-                
-                umtx_lock(NULL);
-                if (availableLocaleList == NULL)
-                {
-                    availableLocaleList = temp;
-                    availableLocaleListCount = localeCount;
-                    temp = NULL;
-                    ucln_i18n_registerCleanup(UCLN_I18N_COLLATOR, collator_cleanup);
-                } 
-                umtx_unlock(NULL);
-
-                needInit = FALSE;
-                if (temp) {
-                    delete []temp;
-                }
-            }
-
-            ures_close(&installed);
-        }
-        ures_close(index);
-    }
-    return !needInit;
-}
 
 // Collator public methods -----------------------------------------------
 
-Collator* U_EXPORT2 Collator::createInstance(UErrorCode& success) 
+Collator* Collator::createInstance(UErrorCode& success) 
 {
+    if (U_FAILURE(success))
+        return 0;
     return createInstance(Locale::getDefault(), success);
 }
 
-Collator* U_EXPORT2 Collator::createInstance(const Locale& desiredLocale,
+Collator* Collator::createInstance(const Locale& desiredLocale,
                                    UErrorCode& status)
 {
     if (U_FAILURE(status)) 
         return 0;
     
-#if !UCONFIG_NO_SERVICE
     if (hasService()) {
         Locale actualLoc;
         Collator *result =
@@ -329,7 +244,6 @@ Collator* U_EXPORT2 Collator::createInstance(const Locale& desiredLocale,
         }
         return result;
     }
-#endif
     return makeInstance(desiredLocale, status);
 }
 
@@ -452,34 +366,22 @@ UBool Collator::greater(const UnicodeString& source,
 
 // this API  ignores registered collators, since it returns an
 // array of indefinite lifetime
-const Locale* U_EXPORT2 Collator::getAvailableLocales(int32_t& count) 
+const Locale* Collator::getAvailableLocales(int32_t& count) 
 {
-    UErrorCode status = U_ZERO_ERROR;
-    Locale *result = NULL;
-    count = 0;
-    if (isAvailableLocaleListInitialized(status))
-    {
-        result = availableLocaleList;
-        count = availableLocaleListCount;
-    }
-    return result;
+    return Locale::getAvailableLocales(count);
 }
 
-UnicodeString& U_EXPORT2 Collator::getDisplayName(const Locale& objectLocale,
+UnicodeString& Collator::getDisplayName(const Locale& objectLocale,
                                         const Locale& displayLocale,
                                         UnicodeString& name)
 {
-#if !UCONFIG_NO_SERVICE
     if (hasService()) {
-        UnicodeString locNameStr;
-        LocaleUtility::initNameFromLocale(objectLocale, locNameStr);
-        return gService->getDisplayName(locNameStr, name, displayLocale);
+        return gService->getDisplayName(objectLocale.getName(), name, displayLocale);
     }
-#endif
     return objectLocale.getDisplayName(displayLocale, name);
 }
 
-UnicodeString& U_EXPORT2 Collator::getDisplayName(const Locale& objectLocale,
+UnicodeString& Collator::getDisplayName(const Locale& objectLocale,
                                         UnicodeString& name)
 {   
     return getDisplayName(objectLocale, Locale::getDefault(), name);
@@ -540,7 +442,7 @@ UBool Collator::operator!=(const Collator& other) const
     return (UBool)!(*this == other);
 }
 
-int32_t U_EXPORT2 Collator::getBound(const uint8_t       *source,
+int32_t Collator::getBound(const uint8_t       *source,
                            int32_t             sourceLength,
                            UColBoundMode       boundType,
                            uint32_t            noOfLevels,
@@ -566,8 +468,7 @@ UnicodeSet *Collator::getTailoredSet(UErrorCode &status) const
 
 // -------------------------------------
 
-#if !UCONFIG_NO_SERVICE
-URegistryKey U_EXPORT2
+URegistryKey
 Collator::registerInstance(Collator* toAdopt, const Locale& locale, UErrorCode& status) 
 {
     if (U_SUCCESS(status)) {
@@ -657,7 +558,7 @@ CFactory::getDisplayName(const UnicodeString& id, const Locale& locale, UnicodeS
     return result;
 }
 
-URegistryKey U_EXPORT2
+URegistryKey 
 Collator::registerFactory(CollatorFactory* toAdopt, UErrorCode& status)
 {
     if (U_SUCCESS(status)) {
@@ -672,7 +573,7 @@ Collator::registerFactory(CollatorFactory* toAdopt, UErrorCode& status)
 
 // -------------------------------------
 
-UBool U_EXPORT2
+UBool 
 Collator::unregister(URegistryKey key, UErrorCode& status) 
 {
     if (U_SUCCESS(status)) {
@@ -683,116 +584,13 @@ Collator::unregister(URegistryKey key, UErrorCode& status)
     }
     return FALSE;
 }
-#endif /* UCONFIG_NO_SERVICE */
-
-class CollationLocaleListEnumeration : public StringEnumeration {
-private:
-    int32_t index;
-public:
-    static UClassID U_EXPORT2 getStaticClassID(void);
-    virtual UClassID getDynamicClassID(void) const;
-public:
-    CollationLocaleListEnumeration()
-        : index(0)
-    {
-        // The global variables should already be initialized.
-        //isAvailableLocaleListInitialized(status);
-    }
-
-    virtual ~CollationLocaleListEnumeration() {
-    }
-
-    virtual StringEnumeration * clone() const
-    {
-        CollationLocaleListEnumeration *result = new CollationLocaleListEnumeration();
-        result->index = index;
-        return result;
-    }
-
-    virtual int32_t count(UErrorCode &/*status*/) const {
-        return availableLocaleListCount;
-    }
-
-    virtual const char* next(int32_t* resultLength, UErrorCode& /*status*/) {
-        const char* result;
-        if(index < availableLocaleListCount) {
-            result = availableLocaleList[index++].getName();
-            if(resultLength != NULL) {
-                *resultLength = uprv_strlen(result);
-            }
-        } else {
-            if(resultLength != NULL) {
-                *resultLength = 0;
-            }
-            result = NULL;
-        }
-        return result;
-    }
-
-    virtual const UnicodeString* snext(UErrorCode& status) {
-        int32_t resultLength = 0;
-        const char *s = next(&resultLength, status);
-        return setChars(s, resultLength, status);
-    }
-
-    virtual void reset(UErrorCode& /*status*/) {
-        index = 0;
-    }
-};
-
-UOBJECT_DEFINE_RTTI_IMPLEMENTATION(CollationLocaleListEnumeration)
-
 
 // -------------------------------------
 
-StringEnumeration* U_EXPORT2
+StringEnumeration* 
 Collator::getAvailableLocales(void)
 {
-#if !UCONFIG_NO_SERVICE
-    if (hasService()) {
-        return getService()->getAvailableLocales();
-    }
-#endif /* UCONFIG_NO_SERVICE */
-    UErrorCode status = U_ZERO_ERROR;
-    if (isAvailableLocaleListInitialized(status)) {
-        return new CollationLocaleListEnumeration();
-    }
-    return NULL;
-}
-
-StringEnumeration* U_EXPORT2
-Collator::getKeywords(UErrorCode& status) {
-    // This is a wrapper over ucol_getKeywords
-    UEnumeration* uenum = ucol_getKeywords(&status);
-    if (U_FAILURE(status)) {
-        uenum_close(uenum);
-        return NULL;
-    }
-    return new UStringEnumeration(uenum);
-}
-
-StringEnumeration* U_EXPORT2
-Collator::getKeywordValues(const char *keyword, UErrorCode& status) {
-    // This is a wrapper over ucol_getKeywordValues
-    UEnumeration* uenum = ucol_getKeywordValues(keyword, &status);
-    if (U_FAILURE(status)) {
-        uenum_close(uenum);
-        return NULL;
-    }
-    return new UStringEnumeration(uenum);
-}
-
-Locale U_EXPORT2
-Collator::getFunctionalEquivalent(const char* keyword, const Locale& locale,
-                                  UBool& isAvailable, UErrorCode& status) {
-    // This is a wrapper over ucol_getFunctionalEquivalent
-    char loc[ULOC_FULLNAME_CAPACITY];
-    /*int32_t len =*/ ucol_getFunctionalEquivalent(loc, sizeof(loc),
-                    keyword, locale.getName(), &isAvailable, &status);
-    if (U_FAILURE(status)) {
-        *loc = 0; // root
-    }
-    return Locale::createFromName(loc);
+    return getService()->getAvailableLocales();
 }
 
 // UCollator private data members ----------------------------------------
@@ -804,6 +602,19 @@ Collator::getFunctionalEquivalent(const char* keyword, const Locale& locale,
 
 U_NAMESPACE_END
 
+// defined in ucln_cmn.h
+
+/**
+ * Release all static memory held by collator.
+ */
+U_CFUNC UBool collator_cleanup(void) {
+    if (gService) {
+        delete gService;
+        gService = NULL;
+    }
+    return TRUE;
+}
+
 #endif /* #if !UCONFIG_NO_COLLATION */
 
 /* eof */
