@@ -1,18 +1,24 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   11/17/99    aliu        Creation.
 **********************************************************************
 */
-#include "unicode/cpdtrans.h"
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/unifilt.h"
 #include "unicode/unifltlg.h"
 #include "unicode/uniset.h"
+#include "cpdtrans.h"
 #include "uvector.h"
 #include "tridpars.h"
+#include "cmemory.h"
 
 // keep in sync with Transliterator
 static const UChar ID_SEP   = 0x002D; /*-*/
@@ -24,6 +30,8 @@ static const UChar EMPTY[] = {0}; //""
 
 U_NAMESPACE_BEGIN
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(CompoundTransliterator)
+
 /**
  * Constructs a new compound transliterator given an array of
  * transliterators.  The array of transliterators may be of any
@@ -180,7 +188,12 @@ void CompoundTransliterator::init(UVector& list,
     // Allocate array
     if (U_SUCCESS(status)) {
         count = list.size();
-        trans = new Transliterator*[count];
+        trans = (Transliterator **)uprv_malloc(count * sizeof(Transliterator *));
+        /* test for NULL */
+        if (trans == 0) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
     }
 
     if (U_FAILURE(status) || trans == 0) {
@@ -254,7 +267,7 @@ void CompoundTransliterator::freeTransliterators(void) {
         for (int32_t i=0; i<count; ++i) {
             delete trans[i];
         }
-        delete[] trans;
+        uprv_free(trans);
     }
     trans = 0;
     count = 0;
@@ -272,8 +285,8 @@ CompoundTransliterator& CompoundTransliterator::operator=(
         trans[i] = 0;
     }
     if (t.count > count) {
-        delete[] trans;
-        trans = new Transliterator*[t.count];
+        uprv_free(trans);
+        trans = (Transliterator **)uprv_malloc(t.count * sizeof(Transliterator *));
     }
     count = t.count;
     for (i=0; i<count; ++i) {
@@ -309,7 +322,7 @@ const Transliterator& CompoundTransliterator::getTransliterator(int32_t index) c
 
 void CompoundTransliterator::setTransliterators(Transliterator* const transliterators[],
                                                 int32_t transCount) {
-    Transliterator** a = new Transliterator*[transCount];
+    Transliterator** a = (Transliterator **)uprv_malloc(transCount * sizeof(Transliterator *));
     for (int32_t i=0; i<transCount; ++i) {
         a[i] = transliterators[i]->clone();
     }
@@ -366,6 +379,41 @@ UnicodeString& CompoundTransliterator::toRules(UnicodeString& rulesSource,
     return rulesSource;
 }
 
+/**
+ * Implement Transliterator framework
+ */
+void CompoundTransliterator::handleGetSourceSet(UnicodeSet& result) const {
+    UnicodeSet set;
+    result.clear();
+    for (int32_t i=0; i<count; ++i) {
+    result.addAll(trans[i]->getSourceSet(set));
+    // Take the example of Hiragana-Latin.  This is really
+    // Hiragana-Katakana; Katakana-Latin.  The source set of
+    // these two is roughly [:Hiragana:] and [:Katakana:].
+    // But the source set for the entire transliterator is
+    // actually [:Hiragana:] ONLY -- that is, the first
+    // non-empty source set.
+
+    // This is a heuristic, and not 100% reliable.
+    if (!result.isEmpty()) {
+        break;
+    }
+    }
+}
+
+/**
+ * Override Transliterator framework
+ */
+UnicodeSet& CompoundTransliterator::getTargetSet(UnicodeSet& result) const {
+    UnicodeSet set;
+    result.clear();
+    for (int32_t i=0; i<count; ++i) {
+    // This is a heuristic, and not 100% reliable.
+    result.addAll(trans[i]->getTargetSet(set));
+    }
+    return result;
+}
+
 /**
  * Implements {@link Transliterator#handleTransliterate}.
  */
@@ -506,4 +554,6 @@ void CompoundTransliterator::computeMaximumContextLength(void) {
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 /* eof */
