@@ -1,34 +1,29 @@
 /*
- *******************************************************************************
- * Copyright (C) 1997-2011, International Business Machines Corporation and    *
- * others. All Rights Reserved.                                                *
- *******************************************************************************
- *
- * File DATEFMT.CPP
- *
- * Modification History:
- *
- *   Date        Name        Description
- *   02/19/97    aliu        Converted from java.
- *   03/31/97    aliu        Modified extensively to work with 50 locales.
- *   04/01/97    aliu        Added support for centuries.
- *   08/12/97    aliu        Fixed operator== to use Calendar::equivalentTo.
- *   07/20/98    stephen     Changed ParsePosition initialization
- ********************************************************************************
- */
+*******************************************************************************
+* Copyright (C) 1997-2003, International Business Machines Corporation and    *
+* others. All Rights Reserved.                                                *
+*******************************************************************************
+*
+* File DATEFMT.CPP
+*
+* Modification History:
+*
+*   Date        Name        Description
+*   02/19/97    aliu        Converted from java.
+*   03/31/97    aliu        Modified extensively to work with 50 locales.
+*   04/01/97    aliu        Added support for centuries.
+*   08/12/97    aliu        Fixed operator== to use Calendar::equivalentTo.
+*    07/20/98    stephen        Changed ParsePosition initialization
+********************************************************************************
+*/
 
 #include "unicode/utypes.h"
 
 #if !UCONFIG_NO_FORMATTING
 
-#include "unicode/ures.h"
+#include "unicode/resbund.h"
 #include "unicode/datefmt.h"
 #include "unicode/smpdtfmt.h"
-#include "unicode/dtptngen.h"
-#include "reldtfmt.h"
-
-#include "cstring.h"
-#include "windtfmt.h"
 
 #if defined( U_DEBUG_CALSVC ) || defined (U_DEBUG_CAL)
 #include <stdio.h>
@@ -95,13 +90,14 @@ DateFormat::operator==(const Format& other) const
     // which have confirmed that the other object being compared against is
     // an instance of a sublcass of DateFormat.  THIS IS IMPORTANT.
 
-    // Format::operator== guarantees that this cast is safe
+    // We only dereference this pointer after we have confirmed below that
+    // 'other' is a DateFormat subclass.
     DateFormat* fmt = (DateFormat*)&other;
 
     return (this == fmt) ||
-        (Format::operator==(other) &&
+        ((getDynamicClassID() == other.getDynamicClassID()) &&
          fCalendar&&(fCalendar->isEquivalentTo(*fmt->fCalendar)) &&
-         (fNumberFormat && *fNumberFormat == *fmt->fNumberFormat));
+         (fNumberFormat&&(*fNumberFormat == *fmt->fNumberFormat)) );
 }
 
 //----------------------------------------------------------------------
@@ -141,55 +137,6 @@ DateFormat::format(const Formattable& obj,
 
 //----------------------------------------------------------------------
 
-UnicodeString&
-DateFormat::format(const Formattable& obj,
-                   UnicodeString& appendTo,
-                   FieldPositionIterator* posIter,
-                   UErrorCode& status) const
-{
-    if (U_FAILURE(status)) return appendTo;
-
-    // if the type of the Formattable is double or long, treat it as if it were a Date
-    UDate date = 0;
-    switch (obj.getType())
-    {
-    case Formattable::kDate:
-        date = obj.getDate();
-        break;
-    case Formattable::kDouble:
-        date = (UDate)obj.getDouble();
-        break;
-    case Formattable::kLong:
-        date = (UDate)obj.getLong();
-        break;
-    default:
-        status = U_ILLEGAL_ARGUMENT_ERROR;
-        return appendTo;
-    }
-
-    // Is this right?
-    //if (fieldPosition.getBeginIndex() == fieldPosition.getEndIndex())
-    //  status = U_ILLEGAL_ARGUMENT_ERROR;
-
-    return format(date, appendTo, posIter, status);
-}
-
-//----------------------------------------------------------------------
-
-// Default implementation for backwards compatibility, subclasses should implement.
-UnicodeString&
-DateFormat::format(Calendar& /* unused cal */,
-                   UnicodeString& appendTo,
-                   FieldPositionIterator* /* unused posIter */,
-                   UErrorCode& status) const {
-    if (U_SUCCESS(status)) {
-        status = U_UNSUPPORTED_ERROR;
-    }
-    return appendTo;
-}
-
-//----------------------------------------------------------------------
-
 UnicodeString&
 DateFormat::format(UDate date, UnicodeString& appendTo, FieldPosition& fieldPosition) const {
     if (fCalendar != NULL) {
@@ -205,20 +152,6 @@ DateFormat::format(UDate date, UnicodeString& appendTo, FieldPosition& fieldPosi
 
 //----------------------------------------------------------------------
 
-UnicodeString&
-DateFormat::format(UDate date, UnicodeString& appendTo, FieldPositionIterator* posIter,
-                   UErrorCode& status) const {
-    if (fCalendar != NULL) {
-        fCalendar->setTime(date, status);
-        if (U_SUCCESS(status)) {
-            return format(*fCalendar, appendTo, posIter, status);
-        }
-    }
-    return appendTo;
-}
-
-//----------------------------------------------------------------------
-
 UnicodeString&
 DateFormat::format(UDate date, UnicodeString& appendTo) const
 {
@@ -234,32 +167,24 @@ UDate
 DateFormat::parse(const UnicodeString& text,
                   ParsePosition& pos) const
 {
-    UDate d = 0; // Error return UDate is 0 (the epoch)
     if (fCalendar != NULL) {
         int32_t start = pos.getIndex();
-
-        // Parse may update TimeZone used by the calendar.
-        TimeZone *tzsav = (TimeZone*)fCalendar->getTimeZone().clone();
-
         fCalendar->clear();
         parse(text, *fCalendar, pos);
         if (pos.getIndex() != start) {
             UErrorCode ec = U_ZERO_ERROR;
-            d = fCalendar->getTime(ec);
-            if (U_FAILURE(ec)) {
-                // We arrive here if fCalendar is non-lenient and there
-                // is an out-of-range field.  We don't know which field
-                // was illegal so we set the error index to the start.
-                pos.setIndex(start);
-                pos.setErrorIndex(start);
-                d = 0;
+            UDate d = fCalendar->getTime(ec);
+            if (U_SUCCESS(ec)) {
+                return d; // Successful function exit
             }
+            // We arrive here if fCalendar is non-lenient and there
+            // is an out-of-range field.  We don't know which field
+            // was illegal so we set the error index to the start.
+            pos.setIndex(start);
+            pos.setErrorIndex(start);
         }
-
-        // Restore TimeZone
-        fCalendar->adoptTimeZone(tzsav);
     }
-    return d;
+    return 0; // Error return UDate is 0 (the epoch)
 }
 
 //----------------------------------------------------------------------
@@ -294,7 +219,7 @@ DateFormat::parseObject(const UnicodeString& source,
 
 //----------------------------------------------------------------------
 
-DateFormat* U_EXPORT2
+DateFormat*
 DateFormat::createTimeInstance(DateFormat::EStyle style,
                                const Locale& aLocale)
 {
@@ -303,36 +228,36 @@ DateFormat::createTimeInstance(DateFormat::EStyle style,
 
 //----------------------------------------------------------------------
 
-DateFormat* U_EXPORT2
+DateFormat*
 DateFormat::createDateInstance(DateFormat::EStyle style,
                                const Locale& aLocale)
 {
-    // +4 to set the correct index for getting data out of
-    // LocaleElements.
-    if(style != kNone)
-    {
-        style = (EStyle) (style + kDateOffset);
-    }
-    return create(kNone, (EStyle) (style), aLocale);
+  // +4 to set the correct index for getting data out of
+  // LocaleElements.
+  if(style != kNone)
+  {
+    style = (EStyle) (style + kDateOffset);
+  }
+  return create(kNone, (EStyle) (style), aLocale);
 }
 
 //----------------------------------------------------------------------
 
-DateFormat* U_EXPORT2
+DateFormat*
 DateFormat::createDateTimeInstance(EStyle dateStyle,
                                    EStyle timeStyle,
                                    const Locale& aLocale)
 {
-    if(dateStyle != kNone)
-    {
-        dateStyle = (EStyle) (dateStyle + kDateOffset);
-    }
-    return create(timeStyle, dateStyle, aLocale);
+  if(dateStyle != kNone)
+  {
+    dateStyle = (EStyle) (dateStyle + kDateOffset);
+  }
+  return create(timeStyle, dateStyle, aLocale);
 }
 
 //----------------------------------------------------------------------
 
-DateFormat* U_EXPORT2
+DateFormat*
 DateFormat::createInstance()
 {
     return create(kShort, (EStyle) (kShort + kDateOffset), Locale::getDefault());
@@ -340,35 +265,11 @@ DateFormat::createInstance()
 
 //----------------------------------------------------------------------
 
-DateFormat* U_EXPORT2
+DateFormat*
 DateFormat::create(EStyle timeStyle, EStyle dateStyle, const Locale& locale)
 {
-    UErrorCode status = U_ZERO_ERROR;
-#ifdef U_WINDOWS
-    char buffer[8];
-    int32_t count = locale.getKeywordValue("compat", buffer, sizeof(buffer), status);
-
-    // if the locale has "@compat=host", create a host-specific DateFormat...
-    if (count > 0 && uprv_strcmp(buffer, "host") == 0) {
-        Win32DateFormat *f = new Win32DateFormat(timeStyle, dateStyle, locale, status);
-
-        if (U_SUCCESS(status)) {
-            return f;
-        }
-
-        delete f;
-    }
-#endif
-
-    // is it relative?
-    if(/*((timeStyle!=UDAT_NONE)&&(timeStyle & UDAT_RELATIVE)) || */((dateStyle!=kNone)&&((dateStyle-kDateOffset) & UDAT_RELATIVE))) {
-        RelativeDateFormat *r = new RelativeDateFormat((UDateFormatStyle)timeStyle, (UDateFormatStyle)(dateStyle-kDateOffset), locale, status);
-        if(U_SUCCESS(status)) return r;
-        delete r;
-        status = U_ZERO_ERROR;
-    }
-
     // Try to create a SimpleDateFormat of the desired style.
+    UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *f = new SimpleDateFormat(timeStyle, dateStyle, locale, status);
     if (U_SUCCESS(status)) return f;
     delete f;
@@ -388,7 +289,7 @@ DateFormat::create(EStyle timeStyle, EStyle dateStyle, const Locale& locale)
 
 //----------------------------------------------------------------------
 
-const Locale* U_EXPORT2
+const Locale*
 DateFormat::getAvailableLocales(int32_t& count)
 {
     // Get the list of installed locales.
@@ -410,10 +311,7 @@ DateFormat::adoptCalendar(Calendar* newCalendar)
 void
 DateFormat::setCalendar(const Calendar& newCalendar)
 {
-    Calendar* newCalClone = newCalendar.clone();
-    if (newCalClone != NULL) {
-        adoptCalendar(newCalClone);
-    }
+    adoptCalendar(newCalendar.clone());
 }
 
 //----------------------------------------------------------------------
@@ -438,10 +336,7 @@ DateFormat::adoptNumberFormat(NumberFormat* newNumberFormat)
 void
 DateFormat::setNumberFormat(const NumberFormat& newNumberFormat)
 {
-    NumberFormat* newNumFmtClone = (NumberFormat*)newNumberFormat.clone();
-    if (newNumFmtClone != NULL) {
-        adoptNumberFormat(newNumFmtClone);
-    }
+    adoptNumberFormat((NumberFormat*)newNumberFormat.clone());
 }
 
 //----------------------------------------------------------------------
@@ -457,18 +352,14 @@ DateFormat::getNumberFormat() const
 void
 DateFormat::adoptTimeZone(TimeZone* zone)
 {
-    if (fCalendar != NULL) {
-        fCalendar->adoptTimeZone(zone);
-    }
+    fCalendar->adoptTimeZone(zone);
 }
 //----------------------------------------------------------------------
 
 void
 DateFormat::setTimeZone(const TimeZone& zone)
 {
-    if (fCalendar != NULL) {
-        fCalendar->setTimeZone(zone);
-    }
+    fCalendar->setTimeZone(zone);
 }
 
 //----------------------------------------------------------------------
@@ -476,12 +367,7 @@ DateFormat::setTimeZone(const TimeZone& zone)
 const TimeZone&
 DateFormat::getTimeZone() const
 {
-    if (fCalendar != NULL) {
-        return fCalendar->getTimeZone();
-    }
-    // If calendar doesn't exists, create default timezone.
-    // fCalendar is rarely null
-    return *(TimeZone::createDefault());
+    return fCalendar->getTimeZone();
 }
 
 //----------------------------------------------------------------------
@@ -489,9 +375,7 @@ DateFormat::getTimeZone() const
 void
 DateFormat::setLenient(UBool lenient)
 {
-    if (fCalendar != NULL) {
-        fCalendar->setLenient(lenient);
-    }
+    fCalendar->setLenient(lenient);
 }
 
 //----------------------------------------------------------------------
@@ -499,11 +383,7 @@ DateFormat::setLenient(UBool lenient)
 UBool
 DateFormat::isLenient() const
 {
-    if (fCalendar != NULL) {
-        return fCalendar->isLenient();
-    }
-    // fCalendar is rarely null
-    return FALSE;
+    return fCalendar->isLenient();
 }
 
 U_NAMESPACE_END
