@@ -1,24 +1,31 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   11/17/99    aliu        Creation.
 **********************************************************************
 */
-#include "unicode/hextouni.h"
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "unicode/rep.h"
 #include "unicode/unifilt.h"
 #include "unicode/uchar.h"
+#include "hextouni.h"
 
 
 U_NAMESPACE_BEGIN
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(HexToUnicodeTransliterator)
+
 /**
  * ID for this transliterator.
  */
-const char HexToUnicodeTransliterator::_ID[] = "Hex-Any";
+const char CURR_ID[] = "Hex-Any";
 
 /**
  * This pattern encodes the following specs for the default constructor:
@@ -29,7 +36,7 @@ const char HexToUnicodeTransliterator::_ID[] = "Hex-Any";
  * The multiple backslashes resolve to a single backslash
  * in the effective prefix.
  */
-const UChar HexToUnicodeTransliterator::DEFAULT_PATTERN[] = {
+static const UChar DEFAULT_PATTERN[] = {
     0x5C, 0x5C, 0x75, 0x30, 0x30, 0x30, 0x30, 0x3B,  /* "\\u0000;" */
     0x5C, 0x5C, 0x55, 0x30, 0x30, 0x30, 0x30, 0x3B,  /* "\\U0000;" */
     0x75, 0x2B, 0x30, 0x30, 0x30, 0x30, 0x3B,        /* "u+0000;" */
@@ -40,11 +47,13 @@ static const UChar gQuadA[] = {
     0x41, 0x41, 0x41, 0x41, 0
 };  /* "AAAA" */
 
+HexToUnicodeTransliterator::~HexToUnicodeTransliterator() {}
+
 /**
  * Constructs a transliterator.
  */
 HexToUnicodeTransliterator::HexToUnicodeTransliterator(UnicodeFilter* adoptedFilter) :
-    Transliterator(_ID, adoptedFilter) {
+    Transliterator(UnicodeString(CURR_ID, ""), adoptedFilter) {
     // We don't need to pass the status back to the caller because
     // we know that the DEFAULT_PATTERN parses.
     UErrorCode status = U_ZERO_ERROR;
@@ -56,7 +65,7 @@ HexToUnicodeTransliterator::HexToUnicodeTransliterator(UnicodeFilter* adoptedFil
  */
 HexToUnicodeTransliterator::HexToUnicodeTransliterator(const UnicodeString& thePattern,
                                                        UErrorCode& status) :
-    Transliterator(_ID, 0) {
+    Transliterator(UnicodeString(CURR_ID, ""), 0) {
     applyPattern(thePattern, status);
 }
 
@@ -66,7 +75,7 @@ HexToUnicodeTransliterator::HexToUnicodeTransliterator(const UnicodeString& theP
 HexToUnicodeTransliterator::HexToUnicodeTransliterator(const UnicodeString& thePattern,
                                                        UnicodeFilter* adoptedFilter,
                                                        UErrorCode& status) :
-    Transliterator(_ID, adoptedFilter) {
+    Transliterator(UnicodeString(CURR_ID, ""), adoptedFilter) {
     applyPattern(thePattern, status);
 }
 
@@ -366,3 +375,4 @@ void HexToUnicodeTransliterator::handleTransliterate(Replaceable& text, UTransPo
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
