@@ -1,6 +1,6 @@
 /*
 ******************************************************************************
-*   Copyright (C) 1997-2011, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ******************************************************************************
 *   file name:  nfrs.cpp
@@ -20,12 +20,13 @@
 #include "unicode/uchar.h"
 #include "nfrule.h"
 #include "nfrlist.h"
-#include "patternprops.h"
 
 #ifdef RBNF_DEBUG
 #include "cmemory.h"
 #endif
 
+#include "uprops.h"
+
 U_NAMESPACE_BEGIN
 
 #if 0
@@ -119,7 +120,6 @@ NFRuleSet::NFRuleSet(UnicodeString* descriptions, int32_t index, UErrorCode& sta
   , negativeNumberRule(NULL)
   , fIsFractionRuleSet(FALSE)
   , fIsPublic(FALSE)
-  , fRecursionCount(0)
 {
     for (int i = 0; i < 3; ++i) {
         fractionRules[i] = NULL;
@@ -131,12 +131,6 @@ NFRuleSet::NFRuleSet(UnicodeString* descriptions, int32_t index, UErrorCode& sta
 
     UnicodeString& description = descriptions[index]; // !!! make sure index is valid
 
-    if (description.length() == 0) {
-        // throw new IllegalArgumentException("Empty rule set description");
-        status = U_PARSE_ERROR;
-        return;
-    }
-
     // if the description begins with a rule set name (the rule set
     // name can be omitted in formatter descriptions that consist
     // of only one rule set), copy it out into our "name" member
@@ -148,12 +142,12 @@ NFRuleSet::NFRuleSet(UnicodeString* descriptions, int32_t index, UErrorCode& sta
             status = U_PARSE_ERROR;
         } else {
             name.setTo(description, 0, pos);
-            while (pos < description.length() && PatternProps::isWhiteSpace(description.charAt(++pos))) {
+            while (pos < description.length() && uprv_isRuleWhiteSpace(description.charAt(++pos))) {
             }
             description.remove(0, pos);
         }
     } else {
-        name.setTo(UNICODE_STRING_SIMPLE("%default"));
+        name.setTo("%default");
     }
 
     if (description.length() == 0) {
@@ -214,7 +208,7 @@ NFRuleSet::parseRules(UnicodeString& description, const RuleBasedNumberFormat* o
             // same as the preceding rule's base value in fraction
             // rule sets)
         case NFRule::kNoBase:
-            rule->setBaseValue(defaultBaseValue, status);
+            rule->setBaseValue(defaultBaseValue);
             if (!isFractionRuleSet()) {
                 ++defaultBaseValue;
             }
@@ -306,38 +300,18 @@ NFRuleSet::operator==(const NFRuleSet& rhs) const
     return FALSE;
 }
 
-#define RECURSION_LIMIT 50
-
 void
 NFRuleSet::format(int64_t number, UnicodeString& toAppendTo, int32_t pos) const
 {
     NFRule *rule = findNormalRule(number);
-    if (rule) { // else error, but can't report it
-        NFRuleSet* ncThis = (NFRuleSet*)this;
-        if (ncThis->fRecursionCount++ >= RECURSION_LIMIT) {
-            // stop recursion
-            ncThis->fRecursionCount = 0;
-        } else {
-            rule->doFormat(number, toAppendTo, pos);
-            ncThis->fRecursionCount--;
-        }
-    }
+    rule->doFormat(number, toAppendTo, pos);
 }
 
 void
 NFRuleSet::format(double number, UnicodeString& toAppendTo, int32_t pos) const
 {
     NFRule *rule = findDoubleRule(number);
-    if (rule) { // else error, but can't report it
-        NFRuleSet* ncThis = (NFRuleSet*)this;
-        if (ncThis->fRecursionCount++ >= RECURSION_LIMIT) {
-            // stop recursion
-            ncThis->fRecursionCount = 0;
-        } else {
-            rule->doFormat(number, toAppendTo, pos);
-            ncThis->fRecursionCount--;
-        }
-    }
+    rule->doFormat(number, toAppendTo, pos);
 }
 
 NFRule*
@@ -434,10 +408,6 @@ NFRuleSet::findNormalRule(int64_t number) const
                 lo = mid + 1;
             }
         }
-        if (hi == 0) { // bad rule set, minimum base > 0
-            return NULL; // want to throw exception here
-        }
-
         NFRule *result = rules[hi - 1];
 
         // use shouldRollBack() to see whether we need to invoke the
@@ -446,9 +416,6 @@ NFRuleSet::findNormalRule(int64_t number) const
         // one rule and return that one instead of the one we'd normally
         // return
         if (result->shouldRollBack((double)number)) {
-            if (hi == 1) { // bad rule set, no prior rule to rollback to from this base
-                return NULL;
-            }
             result = rules[hi - 2];
         }
         return result;
@@ -571,12 +538,10 @@ NFRuleSet::findFractionRuleSetRule(double number) const
 static void dumpUS(FILE* f, const UnicodeString& us) {
   int len = us.length();
   char* buf = (char *)uprv_malloc((len+1)*sizeof(char)); //new char[len+1];
-  if (buf != NULL) {
-	  us.extract(0, len, buf);
-	  buf[len] = 0;
-	  fprintf(f, "%s", buf);
-	  uprv_free(buf); //delete[] buf;
-  }
+  us.extract(0, len, buf);
+  buf[len] = 0;
+  fprintf(f, "%s", buf);
+  uprv_free(buf); //delete[] buf;
 }
 #endif
 
@@ -702,14 +667,14 @@ NFRuleSet::appendRules(UnicodeString& result) const
     // followed by the regular rules...
     for (uint32_t i = 0; i < rules.size(); i++) {
         result.append(gFourSpaces);
-        rules[i]->_appendRuleText(result);
+        rules[i]->appendRuleText(result);
         result.append(gLineFeed);
     }
 
     // followed by the special rules (if they exist)
     if (negativeNumberRule) {
         result.append(gFourSpaces);
-        negativeNumberRule->_appendRuleText(result);
+        negativeNumberRule->appendRuleText(result);
         result.append(gLineFeed);
     }
 
@@ -717,7 +682,7 @@ NFRuleSet::appendRules(UnicodeString& result) const
         for (uint32_t i = 0; i < 3; ++i) {
             if (fractionRules[i]) {
                 result.append(gFourSpaces);
-                fractionRules[i]->_appendRuleText(result);
+                fractionRules[i]->appendRuleText(result);
                 result.append(gLineFeed);
             }
         }
@@ -771,7 +736,6 @@ static const uint8_t asciiDigits[] = {
 
 static const UChar kUMinus = (UChar)0x002d;
 
-#ifdef RBNF_DEBUG
 static const char kMinus = '-';
 
 static const uint8_t digitInfo[] = {
@@ -793,6 +757,7 @@ static const uint8_t digitInfo[] = {
     0xa1u, 0xa2u, 0xa3u,     0,     0,     0,     0,     0,
 };
 
+#ifdef RBNF_DEBUG
 int64_t util64_atoi(const char* str, uint32_t radix)
 {
     if (radix > 36) {
@@ -818,6 +783,7 @@ int64_t util64_atoi(const char* str, uint32_t radix)
     }
     return result;
 }
+#endif
 
 int64_t util64_utoi(const UChar* str, uint32_t radix)
 {
@@ -846,6 +812,7 @@ int64_t util64_utoi(const UChar* str, uint32_t radix)
     return result;
 }
 
+#ifdef RBNF_DEBUG
 uint32_t util64_toa(int64_t w, char* buf, uint32_t len, uint32_t radix, UBool raw)
 {    
     if (radix > 36) {
