@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -18,15 +18,11 @@
 #include "rbt_data.h"
 #include "rbt_rule.h"
 #include "rbt.h"
-#include "umutex.h"
 
 U_NAMESPACE_BEGIN
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(RuleBasedTransliterator)
 
-static UMTX  transliteratorDataMutex = NULL;
-static Replaceable *gLockedText = NULL;
-
 void RuleBasedTransliterator::_construct(const UnicodeString& rules,
                                          UTransDirection direction,
                                          UParseError& parseError,
@@ -37,20 +33,19 @@ void RuleBasedTransliterator::_construct(const UnicodeString& rules,
         return;
     }
 
-    TransliteratorParser parser(status);
+    TransliteratorParser parser;
     parser.parse(rules, direction, parseError, status);
     if (U_FAILURE(status)) {
         return;
     }
 
-    if (parser.idBlockVector.size() != 0 ||
-        parser.compoundFilter != NULL ||
-        parser.dataVector.size() == 0) {
+    if (parser.idBlock.length() != 0 ||
+        parser.compoundFilter != NULL) {
         status = U_INVALID_RBT_SYNTAX; // ::ID blocks disallowed in RBT
         return;
     }
 
-    fData = (TransliterationRuleData*)parser.dataVector.orphanElementAt(0);
+    fData = parser.orphanData();
     setMaximumContextLength(fData->ruleSet.getMaximumContextLength());
 }
 
@@ -232,41 +227,16 @@ RuleBasedTransliterator::handleTransliterate(Replaceable& text, UTransPosition&
         loopLimit <<= 4;
     }
 
-    // Transliterator locking.  Rule-based Transliterators are not thread safe; concurrent
-    //   operations must be prevented.  
-    // A Complication: compound transliterators can result in recursive entries to this
-    //   function, sometimes with different "This" objects, always with the same text. 
-    //   Double-locking must be prevented in these cases.
-    //   
-
-    // If the transliteration data is exclusively owned by this transliterator object,
-    //   we don't need to do any locking.  No sharing between transliterators is possible,
-    //   so no concurrent access from multiple threads is possible.
-    UBool    lockedMutexAtThisLevel = FALSE;
-    if (isDataOwned == FALSE) {
-        umtx_lock(NULL);
-            // Test whether this request is operating on the same text string as some
-            //   some other transliteration that is still in progress and holding the 
-            //   transliteration mutex.  If so, do not lock the transliteration
-            //    mutex again.
-            UBool needToLock = (&text != gLockedText);
-        umtx_unlock(NULL);
-        if (needToLock) {
-            umtx_lock(&transliteratorDataMutex);
-            gLockedText = &text;
-            lockedMutexAtThisLevel = TRUE;
-        }
+    if (isDataOwned==FALSE) {
+        fData->lock();
     }
-    
-
     while (index.start < index.limit &&
            loopCount <= loopLimit &&
            fData->ruleSet.transliterate(text, index, isIncremental)) {
         ++loopCount;
     }
-    if (lockedMutexAtThisLevel) {
-        gLockedText = NULL;
-        umtx_unlock(&transliteratorDataMutex);
+    if (isDataOwned==FALSE) {
+        fData->unlock();
     }
 }
 
