@@ -1,9 +1,9 @@
 //
-//  file:  repattrn.cpp
+//  file:  repattrn.cpp    
 //
 /*
 ***************************************************************************
-*   Copyright (C) 2002-2010 International Business Machines Corporation   *
+*   Copyright (C) 2002-2003 International Business Machines Corporation   *
 *   and others. All rights reserved.                                      *
 ***************************************************************************
 */
@@ -17,7 +17,6 @@
 #include "uassert.h"
 #include "uvector.h"
 #include "uvectr32.h"
-#include "uvectr64.h"
 #include "regexcmp.h"
 #include "regeximp.h"
 #include "regexst.h"
@@ -32,10 +31,12 @@ U_NAMESPACE_BEGIN
 RegexPattern::RegexPattern() {
     UErrorCode status = U_ZERO_ERROR;
     u_init(&status);
-
     // Init all of this instances data.
     init();
-}
+
+    // Lazy init of all shared global sets.
+    RegexStaticSets::initGlobals(&fDeferredStatus);
+};
 
 
 //--------------------------------------------------------------------------
@@ -45,7 +46,7 @@ RegexPattern::RegexPattern() {
 //
 //--------------------------------------------------------------------------
 RegexPattern::RegexPattern(const RegexPattern &other) :  UObject(other) {
-    init();
+    init(); 
     *this = other;
 }
 
@@ -53,7 +54,7 @@ RegexPattern::RegexPattern(const RegexPattern &other) :  UObject(other) {
 
 //--------------------------------------------------------------------------
 //
-//    Assignment Operator
+//    Assignmenet Operator
 //
 //--------------------------------------------------------------------------
 RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
@@ -69,18 +70,7 @@ RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
     init();
 
     // Copy simple fields
-    if ( other.fPatternString == NULL ) {
-        fPatternString = NULL;
-        fPattern      = utext_clone(fPattern, other.fPattern, FALSE, TRUE, &fDeferredStatus);
-    } else {
-        fPatternString = new UnicodeString(*(other.fPatternString));
-        UErrorCode status = U_ZERO_ERROR;
-        fPattern      = utext_openConstUnicodeString(NULL, fPatternString, &status);
-        if (U_FAILURE(status)) {
-            fDeferredStatus = U_MEMORY_ALLOCATION_ERROR;
-            return *this;
-        }
-    }
+    fPattern          = other.fPattern;
     fFlags            = other.fFlags;
     fLiteralText      = other.fLiteralText;
     fDeferredStatus   = other.fDeferredStatus;
@@ -88,32 +78,27 @@ RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
     fFrameSize        = other.fFrameSize;
     fDataSize         = other.fDataSize;
     fMaxCaptureDigits = other.fMaxCaptureDigits;
-    fStaticSets       = other.fStaticSets;
+    fStaticSets       = other.fStaticSets; 
     fStaticSets8      = other.fStaticSets8;
-
+    
     fStartType        = other.fStartType;
     fInitialStringIdx = other.fInitialStringIdx;
     fInitialStringLen = other.fInitialStringLen;
     *fInitialChars    = *other.fInitialChars;
     fInitialChar      = other.fInitialChar;
     *fInitialChars8   = *other.fInitialChars8;
-    fNeedsAltInput    = other.fNeedsAltInput;
 
     //  Copy the pattern.  It's just values, nothing deep to copy.
     fCompiledPat->assign(*other.fCompiledPat, fDeferredStatus);
     fGroupMap->assign(*other.fGroupMap, fDeferredStatus);
 
-    //  Copy the Unicode Sets.
+    //  Copy the Unicode Sets.  
     //    Could be made more efficient if the sets were reference counted and shared,
-    //    but I doubt that pattern copying will be particularly common.
+    //    but I doubt that pattern copying will be particularly common. 
     //    Note:  init() already added an empty element zero to fSets
     int32_t i;
     int32_t  numSets = other.fSets->size();
     fSets8 = new Regex8BitSet[numSets];
-    if (fSets8 == NULL) {
-    	fDeferredStatus = U_MEMORY_ALLOCATION_ERROR;
-    	return *this;
-    }
     for (i=1; i<numSets; i++) {
         if (U_FAILURE(fDeferredStatus)) {
             return *this;
@@ -139,6 +124,7 @@ RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
 //
 //--------------------------------------------------------------------------
 void RegexPattern::init() {
+    fPattern.remove();
     fFlags            = 0;
     fCompiledPat      = 0;
     fLiteralText.remove();
@@ -149,7 +135,7 @@ void RegexPattern::init() {
     fFrameSize        = 0;
     fDataSize         = 0;
     fGroupMap         = NULL;
-    fMaxCaptureDigits = 1;
+    fMaxCaptureDigits = 1;  
     fStaticSets       = NULL;
     fStaticSets8      = NULL;
     fStartType        = START_NO_INFO;
@@ -158,11 +144,8 @@ void RegexPattern::init() {
     fInitialChars     = NULL;
     fInitialChar      = 0;
     fInitialChars8    = NULL;
-    fNeedsAltInput    = FALSE;
-
-    fPattern          = NULL; // will be set later
-    fPatternString    = NULL; // may be set later
-    fCompiledPat      = new UVector64(fDeferredStatus);
+    
+    fCompiledPat      = new UVector32(fDeferredStatus);
     fGroupMap         = new UVector32(fDeferredStatus);
     fSets             = new UVector(fDeferredStatus);
     fInitialChars     = new UnicodeSet;
@@ -183,7 +166,7 @@ void RegexPattern::init() {
 
 //--------------------------------------------------------------------------
 //
-//   zap            Delete everything owned by this RegexPattern.
+//   zap            Delete everything owned by this RegexPattern. 
 //
 //--------------------------------------------------------------------------
 void RegexPattern::zap() {
@@ -207,14 +190,6 @@ void RegexPattern::zap() {
     fInitialChars = NULL;
     delete fInitialChars8;
     fInitialChars8 = NULL;
-    if (fPattern != NULL) {
-        utext_close(fPattern);
-        fPattern = NULL;
-    }
-    if (fPatternString != NULL) {
-        delete fPatternString;
-        fPatternString = NULL;
-    }
 }
 
 
@@ -225,7 +200,7 @@ void RegexPattern::zap() {
 //--------------------------------------------------------------------------
 RegexPattern::~RegexPattern() {
     zap();
-}
+};
 
 
 //--------------------------------------------------------------------------
@@ -233,113 +208,49 @@ RegexPattern::~RegexPattern() {
 //   Clone
 //
 //--------------------------------------------------------------------------
-RegexPattern  *RegexPattern::clone() const {
+RegexPattern  *RegexPattern::clone() const { 
     RegexPattern  *copy = new RegexPattern(*this);
     return copy;
-}
+};
 
 
 //--------------------------------------------------------------------------
 //
 //   operator ==   (comparison)    Consider to patterns to be == if the
 //                                 pattern strings and the flags are the same.
-//                                 Note that pattern strings with the same
-//                                 characters can still be considered different.
 //
 //--------------------------------------------------------------------------
 UBool   RegexPattern::operator ==(const RegexPattern &other) const {
-    if (this->fFlags == other.fFlags && this->fDeferredStatus == other.fDeferredStatus) {
-        if (this->fPatternString != NULL && other.fPatternString != NULL) {
-            return *(this->fPatternString) == *(other.fPatternString);
-        } else if (this->fPattern == NULL) {
-            if (other.fPattern == NULL) {
-                return TRUE;
-            }
-        } else if (other.fPattern != NULL) {
-            UTEXT_SETNATIVEINDEX(this->fPattern, 0);
-            UTEXT_SETNATIVEINDEX(other.fPattern, 0);
-            return utext_equals(this->fPattern, other.fPattern);
-        }
-    }
-    return FALSE;
+    UBool r = this->fFlags    == other.fFlags &&
+              this->fPattern  == other.fPattern &&
+              this->fDeferredStatus == other.fDeferredStatus;
+    return r;
 }
 
 //---------------------------------------------------------------------
 //
-//   compile
+//   compile        
 //
 //---------------------------------------------------------------------
-RegexPattern * U_EXPORT2
-RegexPattern::compile(const UnicodeString &regex,
-                      uint32_t             flags,
-                      UParseError          &pe,
-                      UErrorCode           &status)
-{
-    if (U_FAILURE(status)) {
-        return NULL;
-    }
-    
-    const uint32_t allFlags = UREGEX_CANON_EQ | UREGEX_CASE_INSENSITIVE | UREGEX_COMMENTS |
-    UREGEX_DOTALL   | UREGEX_MULTILINE        | UREGEX_UWORD |
-    UREGEX_ERROR_ON_UNKNOWN_ESCAPES           | UREGEX_UNIX_LINES | UREGEX_LITERAL;
-    
-    if ((flags & ~allFlags) != 0) {
-        status = U_REGEX_INVALID_FLAG;
-        return NULL;
-    }
-    
-    if ((flags & (UREGEX_CANON_EQ | UREGEX_LITERAL)) != 0) {
-        status = U_REGEX_UNIMPLEMENTED;
-        return NULL;
-    }
-    
-    RegexPattern *This = new RegexPattern;
-    if (This == NULL) {
-        status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
-    }
-    if (U_FAILURE(This->fDeferredStatus)) {
-        status = This->fDeferredStatus;
-        delete This;
-        return NULL;
-    }
-    This->fFlags = flags;
-    
-    RegexCompile     compiler(This, status);
-    compiler.compile(regex, pe, status);
-    
-    if (U_FAILURE(status)) {
-        delete This;
-        This = NULL;
-    }
-    
-    return This;
-}
+RegexPattern  *RegexPattern::compile(
+                             const UnicodeString &regex,
+                             uint32_t             flags,
+                             UParseError          &pe,
+                             UErrorCode           &status)  {
 
-
-//
-//   compile, UText mode
-//
-RegexPattern * U_EXPORT2
-RegexPattern::compile(UText                *regex,
-                      uint32_t             flags,
-                      UParseError          &pe,
-                      UErrorCode           &status)
-{
     if (U_FAILURE(status)) {
         return NULL;
     }
 
     const uint32_t allFlags = UREGEX_CANON_EQ | UREGEX_CASE_INSENSITIVE | UREGEX_COMMENTS |
-                              UREGEX_DOTALL   | UREGEX_MULTILINE        | UREGEX_UWORD |
-                              UREGEX_ERROR_ON_UNKNOWN_ESCAPES           | UREGEX_UNIX_LINES | UREGEX_LITERAL;
+                              UREGEX_DOTALL   | UREGEX_MULTILINE;
 
     if ((flags & ~allFlags) != 0) {
         status = U_REGEX_INVALID_FLAG;
         return NULL;
     }
 
-    if ((flags & (UREGEX_CANON_EQ | UREGEX_LITERAL)) != 0) {
+    if ((flags & UREGEX_CANON_EQ) != 0) {
         status = U_REGEX_UNIMPLEMENTED;
         return NULL;
     }
@@ -351,71 +262,40 @@ RegexPattern::compile(UText                *regex,
     }
     if (U_FAILURE(This->fDeferredStatus)) {
         status = This->fDeferredStatus;
-        delete This;
         return NULL;
     }
     This->fFlags = flags;
 
     RegexCompile     compiler(This, status);
     compiler.compile(regex, pe, status);
-    
-    if (U_FAILURE(status)) {
-        delete This;
-        This = NULL;
-    }
 
     return This;
-}
-
+};
+    
 //
 //   compile with default flags.
 //
-RegexPattern * U_EXPORT2
-RegexPattern::compile(const UnicodeString &regex,
-                      UParseError         &pe,
-                      UErrorCode          &err)
+RegexPattern *RegexPattern::compile( const UnicodeString &regex,
+        UParseError          &pe,
+        UErrorCode           &err) 
 {
-    return compile(regex, 0, pe, err);
+    return compile(regex, 0, pe, err); 
 }
 
 
-//
-//   compile with default flags, UText mode
-//
-RegexPattern * U_EXPORT2
-RegexPattern::compile(UText               *regex,
-                      UParseError         &pe,
-                      UErrorCode          &err)
-{
-    return compile(regex, 0, pe, err);
-}
-
 
 //
 //   compile with no UParseErr parameter.
 //
-RegexPattern * U_EXPORT2
-RegexPattern::compile(const UnicodeString &regex,
-                      uint32_t             flags,
-                      UErrorCode          &err)
+RegexPattern *RegexPattern::compile( const UnicodeString &regex,
+        uint32_t             flags,
+        UErrorCode           &err) 
 {
     UParseError pe;
-    return compile(regex, flags, pe, err);
+    return compile(regex, flags, pe, err); 
 }
 
 
-//
-//   compile with no UParseErr parameter, UText mode
-//
-RegexPattern * U_EXPORT2
-RegexPattern::compile(UText                *regex,
-                      uint32_t             flags,
-                      UErrorCode           &err)
-{
-    UParseError pe;
-    return compile(regex, flags, pe, err);
-}
-
 
 //---------------------------------------------------------------------
 //
@@ -436,37 +316,12 @@ RegexMatcher *RegexPattern::matcher(const UnicodeString &input,
                                     UErrorCode          &status)  const {
     RegexMatcher    *retMatcher = matcher(status);
     if (retMatcher != NULL) {
-        retMatcher->fDeferredStatus = status;
         retMatcher->reset(input);
     }
     return retMatcher;
-}
+};
 
-//
-//   matcher, UText mode
-//
-RegexMatcher *RegexPattern::matcher(UText               *input,
-                                    PatternIsUTextFlag  /*flag*/,
-                                    UErrorCode          &status)  const {
-    RegexMatcher    *retMatcher = matcher(status);
-    if (retMatcher != NULL) {
-        retMatcher->fDeferredStatus = status;
-        retMatcher->reset(input);
-    }
-    return retMatcher;
-}
 
-#if 0
-RegexMatcher *RegexPattern::matcher(const UChar * /*input*/,
-                                    UErrorCode          &status)  const
-{
-    /* This should never get called. The API with UnicodeString should be called instead. */
-    if (U_SUCCESS(status)) {
-        status = U_UNSUPPORTED_ERROR;
-    }
-    return NULL;
-}
-#endif
 
 //---------------------------------------------------------------------
 //
@@ -484,13 +339,13 @@ RegexMatcher *RegexPattern::matcher(UErrorCode &status)  const {
         return NULL;
     }
 
-    retMatcher = new RegexMatcher(this);
+    retMatcher = new RegexMatcher(this); 
     if (retMatcher == NULL) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return NULL;
     }
     return retMatcher;
-}
+};
 
 
 
@@ -500,7 +355,7 @@ RegexMatcher *RegexPattern::matcher(UErrorCode &status)  const {
 //                  with a pattern string and a data string.
 //
 //---------------------------------------------------------------------
-UBool U_EXPORT2 RegexPattern::matches(const UnicodeString   &regex,
+UBool RegexPattern::matches(const UnicodeString   &regex,
               const UnicodeString   &input,
                     UParseError     &pe,
                     UErrorCode      &status) {
@@ -521,31 +376,6 @@ UBool U_EXPORT2 RegexPattern::matches(const UnicodeString   &regex,
 }
 
 
-//
-//   matches, UText mode
-//
-UBool U_EXPORT2 RegexPattern::matches(UText                *regex,
-                    UText           *input,
-                    UParseError     &pe,
-                    UErrorCode      &status) {
-
-    if (U_FAILURE(status)) {return FALSE;}
-
-    UBool         retVal;
-    RegexPattern *pat     = NULL;
-    RegexMatcher *matcher = NULL;
-
-    pat     = RegexPattern::compile(regex, 0, pe, status);
-    matcher = pat->matcher(input, PATTERN_IS_UTEXT, status);
-    retVal  = matcher->matches(status);
-
-    delete matcher;
-    delete pat;
-    return retVal;
-}
-
-
-
 
 
 //---------------------------------------------------------------------
@@ -554,45 +384,12 @@ UBool U_EXPORT2 RegexPattern::matches(UText                *regex,
 //
 //---------------------------------------------------------------------
 UnicodeString RegexPattern::pattern() const {
-    if (fPatternString != NULL) {
-        return *fPatternString;
-    } else if (fPattern == NULL) {
-        return UnicodeString();
-    } else {
-        UErrorCode status = U_ZERO_ERROR;
-        int64_t nativeLen = utext_nativeLength(fPattern);
-        int32_t len16 = utext_extract(fPattern, 0, nativeLen, NULL, 0, &status); // buffer overflow error
-        UnicodeString result;
-        
-        status = U_ZERO_ERROR;
-        UChar *resultChars = result.getBuffer(len16);
-        utext_extract(fPattern, 0, nativeLen, resultChars, len16, &status); // unterminated warning
-        result.releaseBuffer(len16);
-        
-        return result;
-    }
+    return fPattern;
 }
 
 
 
 
-//---------------------------------------------------------------------
-//
-//   patternText
-//
-//---------------------------------------------------------------------
-UText *RegexPattern::patternText() const {
-    if (fPattern != NULL) {
-        return fPattern;
-    } else {
-        UErrorCode status = U_ZERO_ERROR;
-        RegexStaticSets::initGlobals(&status);
-        return RegexStaticSets::gStaticSets->fEmptyText;
-    }
-}
-
-
-
 //---------------------------------------------------------------------
 //
 //   split
@@ -601,39 +398,14 @@ UText *RegexPattern::patternText() const {
 int32_t  RegexPattern::split(const UnicodeString &input,
         UnicodeString    dest[],
         int32_t          destCapacity,
-        UErrorCode      &status) const
+        UErrorCode       &status) const
 {
     if (U_FAILURE(status)) {
         return 0;
     };
 
     RegexMatcher  m(this);
-    int32_t r = 0;
-    // Check m's status to make sure all is ok.
-    if (U_SUCCESS(m.fDeferredStatus)) {
-    	r = m.split(input, dest, destCapacity, status);
-    }
-    return r;
-}
-
-//
-//   split, UText mode
-//
-int32_t  RegexPattern::split(UText *input,
-        UText           *dest[],
-        int32_t          destCapacity,
-        UErrorCode      &status) const
-{
-    if (U_FAILURE(status)) {
-        return 0;
-    };
-
-    RegexMatcher  m(this);
-    int32_t r = 0;
-    // Check m's status to make sure all is ok.
-    if (U_SUCCESS(m.fDeferredStatus)) {
-    	r = m.split(input, dest, destCapacity, status);
-    }
+    int32_t r = m.split(input, dest, destCapacity, status);
     return r;
 }
 
@@ -652,15 +424,17 @@ void   RegexPattern::dumpOp(int32_t index) const {
     int32_t val         = URX_VAL(op);
     int32_t type        = URX_TYPE(op);
     int32_t pinnedType  = type;
-    if ((uint32_t)pinnedType >= sizeof(opNames)/sizeof(char *)) {
+    if (pinnedType >= sizeof(opNames)/sizeof(char *)) {
         pinnedType = 0;
     }
-
+    
     REGEX_DUMP_DEBUG_PRINTF(("%4d   %08x    %-15s  ", index, op, opNames[pinnedType]));
     switch (type) {
     case URX_NOP:
     case URX_DOTANY:
     case URX_DOTANY_ALL:
+    case URX_DOTANY_PL:
+    case URX_DOTANY_ALL_PL:
     case URX_FAIL:
     case URX_CARET:
     case URX_DOLLAR:
@@ -671,7 +445,7 @@ void   RegexPattern::dumpOp(int32_t index) const {
     case URX_CARET_M:
         // Types with no operand field of interest.
         break;
-
+        
     case URX_RESERVED_OP:
     case URX_START_CAPTURE:
     case URX_END_CAPTURE:
@@ -707,12 +481,12 @@ void   RegexPattern::dumpOp(int32_t index) const {
         // types with an integer operand field.
         REGEX_DUMP_DEBUG_PRINTF(("%d", val));
         break;
-
+        
     case URX_ONECHAR:
     case URX_ONECHAR_I:
         REGEX_DUMP_DEBUG_PRINTF(("%c", val<256?val:'?'));
         break;
-
+        
     case URX_STRING:
     case URX_STRING_I:
         {
@@ -756,7 +530,7 @@ void   RegexPattern::dumpOp(int32_t index) const {
         }
         break;
 
-
+        
     default:
         REGEX_DUMP_DEBUG_PRINTF(("??????"));
         break;
@@ -767,30 +541,23 @@ void   RegexPattern::dumpOp(int32_t index) const {
 
 
 #if defined(REGEX_DEBUG)
-U_CAPI void  U_EXPORT2
+U_CAPI void  U_EXPORT2 
 RegexPatternDump(const RegexPattern *This) {
     int      index;
     int      i;
 
     REGEX_DUMP_DEBUG_PRINTF(("Original Pattern:  "));
-    UChar32 c = utext_next32From(This->fPattern, 0);
-    while (c != U_SENTINEL) {
-        if (c<32 || c>256) {
-            c = '.';
-        }
-        REGEX_DUMP_DEBUG_PRINTF(("%c", c));
-        
-        c = UTEXT_NEXT32(This->fPattern);
+    for (i=0; i<This->fPattern.length(); i++) {
+        REGEX_DUMP_DEBUG_PRINTF(("%c", This->fPattern.charAt(i)));
     }
     REGEX_DUMP_DEBUG_PRINTF(("\n"));
     REGEX_DUMP_DEBUG_PRINTF(("   Min Match Length:  %d\n", This->fMinMatchLen));
-    REGEX_DUMP_DEBUG_PRINTF(("   Match Start Type:  %s\n", START_OF_MATCH_STR(This->fStartType)));
+    REGEX_DUMP_DEBUG_PRINTF(("   Match Start Type:  %s\n", START_OF_MATCH_STR(This->fStartType)));   
     if (This->fStartType == START_STRING) {
-        REGEX_DUMP_DEBUG_PRINTF(("    Initial match string: \""));
+        REGEX_DUMP_DEBUG_PRINTF(("    Initial match sting: \""));
         for (i=This->fInitialStringIdx; i<This->fInitialStringIdx+This->fInitialStringLen; i++) {
             REGEX_DUMP_DEBUG_PRINTF(("%c", This->fLiteralText[i]));   // TODO:  non-printables, surrogates.
         }
-        REGEX_DUMP_DEBUG_PRINTF(("\"\n"));
 
     } else if (This->fStartType == START_SET) {
         int32_t numSetChars = This->fInitialChars->size();
@@ -800,7 +567,7 @@ RegexPatternDump(const RegexPattern *This) {
         REGEX_DUMP_DEBUG_PRINTF(("     Match First Chars : "));
         for (i=0; i<numSetChars; i++) {
             UChar32 c = This->fInitialChars->charAt(i);
-            if (0x20<c && c <0x7e) {
+            if (0x20<c && c <0x7e) { 
                 REGEX_DUMP_DEBUG_PRINTF(("%c ", c));
             } else {
                 REGEX_DUMP_DEBUG_PRINTF(("%#x ", c));
@@ -826,12 +593,24 @@ RegexPatternDump(const RegexPattern *This) {
         This->dumpOp(index);
     }
     REGEX_DUMP_DEBUG_PRINTF(("\n\n"));
-}
+};
 #endif
 
 
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(RegexPattern)
 
+//----------------------------------------------------------------------------------
+//
+//   regex_cleanup      Memory cleanup function, free/delete all
+//                      cached memory.  Called by ICU's u_cleanup() function.
+//
+//----------------------------------------------------------------------------------
+U_CFUNC UBool 
+regex_cleanup(void) {
+    RegexCompile::cleanup();
+    return TRUE;
+};
+
 U_NAMESPACE_END
 #endif  // !UCONFIG_NO_REGULAR_EXPRESSIONS
