@@ -1,19 +1,26 @@
 /*
-* Copyright (C) 2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 2001-2003, International Business Machines Corporation and others. All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   07/23/01    aliu        Creation.
 **********************************************************************
 */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "strmatch.h"
 #include "rbt_data.h"
 #include "util.h"
+#include "unicode/uniset.h"
 
 U_NAMESPACE_BEGIN
 
 const UChar EMPTY[] = { 0 }; // empty string: ""
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(StringMatcher)
+
 StringMatcher::StringMatcher(const UnicodeString& theString,
                              int32_t start,
                              int32_t limit,
@@ -28,7 +35,9 @@ StringMatcher::StringMatcher(const UnicodeString& theString,
 }
 
 StringMatcher::StringMatcher(const StringMatcher& o) :
+    UnicodeFunctor(o),
     UnicodeMatcher(o),
+    UnicodeReplacer(o),
     pattern(o.pattern),
     data(o.data),
     segmentNumber(o.segmentNumber),
@@ -180,13 +189,29 @@ UBool StringMatcher::matchesIndexValue(uint8_t v) const {
     return (m == 0) ? ((c & 0xFF) == v) : m->matchesIndexValue(v);
 }
 
+/**
+ * Implement UnicodeMatcher
+ */
+void StringMatcher::addMatchSetTo(UnicodeSet& toUnionTo) const {
+    UChar32 ch;
+    for (int32_t i=0; i<pattern.length(); i+=UTF_CHAR_LENGTH(ch)) {
+        ch = pattern.char32At(i);
+        const UnicodeMatcher* matcher = data->lookupMatcher(ch);
+        if (matcher == NULL) {
+            toUnionTo.add(ch);
+        } else {
+            matcher->addMatchSetTo(toUnionTo);
+        }
+    }
+}
+
 /**
  * UnicodeReplacer API
  */
 int32_t StringMatcher::replace(Replaceable& text,
                                int32_t start,
                                int32_t limit,
-                               int32_t& cursor) {
+                               int32_t& /*cursor*/) {
     
     int32_t outLen = 0;
     
@@ -210,7 +235,7 @@ int32_t StringMatcher::replace(Replaceable& text,
  * UnicodeReplacer API
  */
 UnicodeString& StringMatcher::toReplacerPattern(UnicodeString& rule,
-                                                UBool escapeUnprintable) const {
+                                                UBool /*escapeUnprintable*/) const {
     // assert(segmentNumber > 0);
     rule.truncate(0);
     rule.append((UChar)0x0024 /*$*/);
@@ -226,6 +251,19 @@ UnicodeString& StringMatcher::toReplacerPattern(UnicodeString& rule,
     matchStart = matchLimit = -1;
 }
 
+/**
+ * Union the set of all characters that may output by this object
+ * into the given set.
+ * @param toUnionTo the set into which to union the output characters
+ */
+void StringMatcher::addReplacementSetTo(UnicodeSet& /*toUnionTo*/) const {
+    // The output of this replacer varies; it is the source text between
+    // matchStart and matchLimit.  Since this varies depending on the
+    // input text, we can't compute it here.  We can either do nothing
+    // or we can add ALL characters to the set.  It's probably more useful
+    // to do nothing.
+}
+
 /**
  * Implement UnicodeFunctor
  */
@@ -244,4 +282,6 @@ void StringMatcher::setData(const TransliterationRuleData* d) {
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 //eof
