@@ -1,24 +1,31 @@
 /*
 **********************************************************************
-*   Copyright (c) 2001, International Business Machines
+*   Copyright (c) 2001-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
 *   08/10/2001  aliu        Creation.
 **********************************************************************
 */
-#include "transreg.h"
-#include "rbt_data.h"
-#include "rbt_pars.h"
-#include "tridpars.h"
-#include "unicode/cpdtrans.h"
-#include "unicode/nultrans.h"
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
+#include "unicode/utypes.h"
+#include "unicode/uobject.h"
 #include "unicode/parseerr.h"
-#include "unicode/rbt.h"
 #include "unicode/resbund.h"
 #include "unicode/translit.h"
 #include "unicode/uniset.h"
 #include "unicode/uscript.h"
+#include "rbt.h"
+#include "cpdtrans.h"
+#include "nultrans.h"
+#include "transreg.h"
+#include "rbt_data.h"
+#include "rbt_pars.h"
+#include "tridpars.h"
 #include "charstr.h"
 
 // Enable the following symbol to add debugging code that tracks the
@@ -85,6 +92,11 @@ Transliterator* TransliteratorAlias::create(UParseError& pe,
     } else {
         t = new CompoundTransliterator(ID, aliasID, idSplitPoint,
                                        trans, ec);
+        /* test for NULL */
+        if (t == 0) {
+            ec = U_MEMORY_ALLOCATION_ERROR;
+            return 0;
+        }
         trans = 0; // so we don't delete it later
         if (compoundFilter) {
             t->adoptFilter((UnicodeSet*) compoundFilter->clone());
@@ -112,7 +124,7 @@ Transliterator* TransliteratorAlias::create(UParseError& pe,
  * canonical form, or the script is transformed from an abbreviation
  * to a full name.
  */
-class Spec {
+class Spec : public UMemory {
  public:
     Spec(const UnicodeString& spec);
     ~Spec();
@@ -138,6 +150,9 @@ class Spec {
     UBool isSpecLocale; // TRUE if spec is a locale
     UBool isNextLocale; // TRUE if nextSpec is a locale
     ResourceBundle* res;
+
+    Spec(const Spec &other); // forbid copying of this class
+    Spec &operator=(const Spec &other); // forbid copying of this class
 };
 
 Spec::Spec(const UnicodeString& theSpec) : top(theSpec) {
@@ -145,8 +160,11 @@ Spec::Spec(const UnicodeString& theSpec) : top(theSpec) {
     CharString topch(top);
     Locale toploc(topch);
     res = new ResourceBundle(u_getDataDirectory(), toploc, status);
-    if (U_FAILURE(status) ||
-        status == U_USING_DEFAULT_ERROR) {
+    /* test for NULL */
+    if (res == 0) {
+        return;
+    }
+    if (U_FAILURE(status) || status == U_USING_DEFAULT_WARNING) {
         delete res;
         res = 0;
     }
@@ -315,7 +333,7 @@ static void DEBUG_useEntry(Entry* e) {
  * for it.  We could easily add this if there is a need for it in the
  * future.
  */
-class Entry {
+class Entry : public UMemory {
 public:
     enum Type {
         RULES_FORWARD,
@@ -346,6 +364,11 @@ public:
     void adoptPrototype(Transliterator* adopted);
     void setFactory(Transliterator::Factory factory,
                     Transliterator::Token context);
+
+private:
+
+    Entry(const Entry &other); // forbid copying of this class
+    Entry &operator=(const Entry &other); // forbid copying of this class
 };
 
 Entry::Entry() {
@@ -465,7 +488,7 @@ void TransliteratorRegistry::remove(const UnicodeString& ID) {
     TransliteratorIDParser::IDtoSTV(ID, source, target, variant, sawSource);
     // Only need to do this if ID.indexOf('-') < 0
     UnicodeString id;
-    STVtoID(source, target, variant, id);
+    TransliteratorIDParser::STVtoID(source, target, variant, id);
     registry.remove(id);
     removeSTV(source, target, variant);
     availableIDs.removeElement((void*) &id);
@@ -585,25 +608,6 @@ UnicodeString& TransliteratorRegistry::getAvailableVariant(int32_t index,
 // class TransliteratorRegistry: internal
 //----------------------------------------------------------------------
 
-/**
- * Given source, target, and variant strings, concatenate them into a
- * full ID.  If the source is empty, then "Any" will be used for the
- * source, so the ID will always be of the form s-t/v or s-t.
- */
-void TransliteratorRegistry::STVtoID(const UnicodeString& source,
-                                     const UnicodeString& target,
-                                     const UnicodeString& variant,
-                                     UnicodeString& id) {
-    id = source;
-    if (id.length() == 0) {
-        id = ANY;
-    }
-    id.append(ID_SEP).append(target);
-    if (variant.length() != 0) {
-        id.append(VARIANT_SEP).append(variant);
-    }
-}
-
 /**
  * Convenience method.  Calls 6-arg registerEntry().
  */
@@ -617,7 +621,7 @@ void TransliteratorRegistry::registerEntry(const UnicodeString& source,
     if (s.length() == 0) {
         s = ANY;
     }
-    STVtoID(source, target, variant, ID);
+    TransliteratorIDParser::STVtoID(source, target, variant, ID);
     registerEntry(ID, s, target, variant, adopted, visible);
 }
 
@@ -632,7 +636,7 @@ void TransliteratorRegistry::registerEntry(const UnicodeString& ID,
     TransliteratorIDParser::IDtoSTV(ID, source, target, variant, sawSource);
     // Only need to do this if ID.indexOf('-') < 0
     UnicodeString id;
-    STVtoID(source, target, variant, id);
+    TransliteratorIDParser::STVtoID(source, target, variant, id);
     registerEntry(id, source, target, variant, adopted, visible);
 }
 
@@ -651,7 +655,10 @@ void TransliteratorRegistry::registerEntry(const UnicodeString& ID,
     if (visible) {
         registerSTV(source, target, variant);
         if (!availableIDs.contains((void*) &ID)) {
-            availableIDs.addElement(new UnicodeString(ID), status);
+            UnicodeString *newID = (UnicodeString *)ID.clone();
+            // NUL-terminate the ID string
+            newID->getTerminatedBuffer();
+            availableIDs.addElement(newID, status);
         }
     } else {
         removeSTV(source, target, variant);
@@ -737,7 +744,7 @@ Entry* TransliteratorRegistry::findInDynamicStore(const Spec& src,
                                                   const Spec& trg,
                                                   const UnicodeString& variant) {
     UnicodeString ID;
-    STVtoID(src, trg, variant, ID);
+    TransliteratorIDParser::STVtoID(src, trg, variant, ID);
     Entry *e = (Entry*) registry.get(ID);
     DEBUG_useEntry(e);
     return e;
@@ -817,8 +824,7 @@ Entry* TransliteratorRegistry::findInBundle(const Spec& specToOpen,
         
         UErrorCode status = U_ZERO_ERROR;
         ResourceBundle subres(specToOpen.getBundle().get(tag, status));
-        if (U_FAILURE(status) ||
-            status == U_USING_DEFAULT_ERROR) {
+        if (U_FAILURE(status) || status == U_USING_DEFAULT_WARNING) {
             continue;
         }
         
@@ -978,12 +984,21 @@ Transliterator* TransliteratorRegistry::instantiateEntry(const UnicodeString& ID
             return entry->u.prototype->clone();
         } else if (entry->entryType == Entry::ALIAS) {
             aliasReturn = new TransliteratorAlias(entry->stringArg);
+            /* test for NULL */
+            if (aliasReturn == 0) {
+                status = U_MEMORY_ALLOCATION_ERROR;
+            }
             return 0;
         } else if (entry->entryType == Entry::FACTORY) {
             return entry->u.factory.function(ID, entry->u.factory.context);
         } else if (entry->entryType == Entry::COMPOUND_RBT) {
             UnicodeString id("_", "");
             Transliterator *t = new RuleBasedTransliterator(id, entry->u.data);
+            /* test for NULL */
+            if (t == 0) {
+                status = U_MEMORY_ALLOCATION_ERROR;
+                return 0;
+            }
             aliasReturn = new TransliteratorAlias(ID, entry->stringArg, t, entry->intArg, entry->compoundFilter);
             return 0;
         }
@@ -1071,4 +1086,6 @@ Transliterator* TransliteratorRegistry::instantiateEntry(const UnicodeString& ID
 }
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
+
 //eof
