@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (c) 2002-2006, International Business Machines Corporation
+*   Copyright (c) 2002-2003, International Business Machines Corporation
 *   and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -33,7 +33,7 @@ static const UChar VARIANT_SEP = 0x002F; // /
 static const UChar OPEN_REV    = 0x0028; // (
 static const UChar CLOSE_REV   = 0x0029; // )
 
-//static const UChar EMPTY[]     = {0}; // ""
+static const UChar EMPTY[]     = {0}; // ""
 static const UChar ANY[]       = {65,110,121,0}; // "Any"
 static const UChar ANY_NULL[]  = {65,110,121,45,78,117,108,108,0}; // "Any-Null"
 
@@ -105,7 +105,7 @@ Transliterator* TransliteratorIDParser::SingleID::createInstance() {
  */
 TransliteratorIDParser::SingleID*
 TransliteratorIDParser::parseSingleID(const UnicodeString& id, int32_t& pos,
-                                      int32_t dir, UErrorCode& status) {
+                                      int32_t dir) {
 
     int32_t start = pos;
 
@@ -167,7 +167,7 @@ TransliteratorIDParser::parseSingleID(const UnicodeString& id, int32_t& pos,
         if (dir == FORWARD) {
             single = specsToID(specsA, FORWARD);
         } else {
-            single = specsToSpecialInverse(*specsA, status);
+            single = specsToSpecialInverse(*specsA);
             if (single == NULL) {
                 single = specsToID(specsA, REVERSE);
             }
@@ -301,7 +301,7 @@ static void U_CALLCONV _deleteSingleID(void* obj) {
     delete (TransliteratorIDParser::SingleID*) obj;
 }
 
-static void U_CALLCONV _deleteTransliteratorTrIDPars(void* obj) {
+static void U_CALLCONV _deleteTransliterator(void* obj) {
     delete (Transliterator*) obj;
 }
 U_CDECL_END
@@ -360,7 +360,7 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
 
     UBool sawDelimiter = TRUE;
     for (;;) {
-        SingleID* single = parseSingleID(id, pos, dir, ec);
+        SingleID* single = parseSingleID(id, pos, dir);
         if (single == NULL) {
             break;
         }
@@ -437,23 +437,41 @@ UBool TransliteratorIDParser::parseCompoundID(const UnicodeString& id, int32_t d
  * the reverse.  THIS MAY RESULT IN AN EMPTY VECTOR.  Convert
  * SingleID entries to actual transliterators.
  *
+ * Also, optionally, insert the given transliterator at the given
+ * position.  This effectively happens before anything else.
+ *
  * @param list vector of SingleID objects.  On exit, vector
  * of one or more Transliterators.
+ * @param insert Transliterator to insert, or NULL if none.
+ * Adopted.
+ * @param insertIndex index from 0..list.size()-1, at which
+ * to place 'insert', or -1 if none.
  * @return new value of insertIndex.  The index will shift if
  * there are empty items, like "(Lower)", with indices less than
  * insertIndex.
  */
-void TransliteratorIDParser::instantiateList(UVector& list,
+int32_t TransliteratorIDParser::instantiateList(UVector& list,
+                                                Transliterator* insert,
+                                                int32_t insertIndex,
                                                 UErrorCode& ec) {
     UVector tlist(ec);
     if (U_FAILURE(ec)) {
         goto RETURN;
     }
-    tlist.setDeleter(_deleteTransliteratorTrIDPars);
+    tlist.setDeleter(_deleteTransliterator);
 
     Transliterator* t;
     int32_t i;
     for (i=0; i<=list.size(); ++i) { // [sic]: i<=list.size()
+        if (insertIndex == i) {
+            insertIndex = tlist.size();
+            tlist.addElement(insert, ec);
+            if (U_FAILURE(ec)) {
+                goto RETURN;
+            }
+            insert = NULL;
+        }
+
         // We run the loop too long by one, so we can
         // do an insert after the last element
         if (i==list.size()) {
@@ -494,7 +512,7 @@ void TransliteratorIDParser::instantiateList(UVector& list,
     list.removeAllElements();
 
     if (U_SUCCESS(ec)) {
-        list.setDeleter(_deleteTransliteratorTrIDPars);
+        list.setDeleter(_deleteTransliterator);
 
         while (tlist.size() > 0) {
             t = (Transliterator*) tlist.orphanElementAt(0);
@@ -507,7 +525,9 @@ void TransliteratorIDParser::instantiateList(UVector& list,
         }
     }
 
+    delete insert; // Clean up in case of failure
     list.setDeleter(save);
+    return insertIndex;
 }
 
 /**
@@ -581,10 +601,6 @@ void TransliteratorIDParser::STVtoID(const UnicodeString& source,
     if (variant.length() != 0) {
         id.append(VARIANT_SEP).append(variant);
     }
-    // NUL-terminate the ID string for getTerminatedBuffer.
-    // This prevents valgrind and Purify warnings.
-    id.append((UChar)0);
-    id.truncate(id.length()-1);
 }
 
 /**
@@ -621,12 +637,8 @@ void TransliteratorIDParser::STVtoID(const UnicodeString& source,
  */
 void TransliteratorIDParser::registerSpecialInverse(const UnicodeString& target,
                                                     const UnicodeString& inverseTarget,
-                                                    UBool bidirectional,
-                                                    UErrorCode &status) {
-    init(status);
-    if (U_FAILURE(status)) {
-        return;
-    }
+                                                    UBool bidirectional) {
+    init();
 
     // If target == inverseTarget then force bidirectional => FALSE
     if (bidirectional && 0==target.caseCompare(inverseTarget, U_FOLD_CASE_DEFAULT)) {
@@ -636,9 +648,10 @@ void TransliteratorIDParser::registerSpecialInverse(const UnicodeString& target,
     umtx_init(&LOCK);
     Mutex lock(&LOCK);
 
-    SPECIAL_INVERSES->put(target, new UnicodeString(inverseTarget), status);
+    UErrorCode ec = U_ZERO_ERROR;
+    SPECIAL_INVERSES->put(target, new UnicodeString(inverseTarget), ec);
     if (bidirectional) {
-        SPECIAL_INVERSES->put(inverseTarget, new UnicodeString(target), status);
+        SPECIAL_INVERSES->put(inverseTarget, new UnicodeString(target), ec);
     }
 }
 
@@ -817,11 +830,11 @@ TransliteratorIDParser::specsToID(const Specs* specs, int32_t dir) {
  * 'filter' field of NULL.
  */
 TransliteratorIDParser::SingleID*
-TransliteratorIDParser::specsToSpecialInverse(const Specs& specs, UErrorCode &status) {
+TransliteratorIDParser::specsToSpecialInverse(const Specs& specs) {
     if (0!=specs.source.caseCompare(ANY, U_FOLD_CASE_DEFAULT)) {
         return NULL;
     }
-    init(status);
+    init();
 
     UnicodeString* inverseTarget;
 
@@ -867,12 +880,12 @@ Transliterator* TransliteratorIDParser::createBasicInstance(const UnicodeString&
 /**
  * Initialize static memory.
  */
-void TransliteratorIDParser::init(UErrorCode &status) {
+void TransliteratorIDParser::init() {
     if (SPECIAL_INVERSES != NULL) {
         return;
     }
 
-    Hashtable* special_inverses = new Hashtable(TRUE, status);
+    Hashtable* special_inverses = new Hashtable(TRUE);
     special_inverses->setValueDeleter(uhash_deleteUnicodeString);
 
     umtx_init(&LOCK);
@@ -882,9 +895,9 @@ void TransliteratorIDParser::init(UErrorCode &status) {
         special_inverses = NULL;
     }
     umtx_unlock(&LOCK);
-    delete special_inverses; /*null instance*/
+    delete special_inverses;
 
-    ucln_i18n_registerCleanup(UCLN_I18N_TRANSLITERATOR, transliterator_cleanup);
+    ucln_i18n_registerCleanup();
 }
 
 /**
