@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001-2006, International Business Machines
+*   Copyright (C) 2001, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -26,6 +26,8 @@
 #include "ucol_cnt.h"
 #include "cmemory.h"
 
+U_NAMESPACE_BEGIN
+
 static void uprv_growTable(ContractionTable *tbl, UErrorCode *status) {
     if(tbl->position == tbl->size) {
         uint32_t *newData = (uint32_t *)uprv_realloc(tbl->CEs, 2*tbl->size*sizeof(uint32_t));
@@ -53,15 +55,15 @@ uprv_cnttab_open(UNewTrie *mapping, UErrorCode *status) {
     }
     CntTable *tbl = (CntTable *)uprv_malloc(sizeof(CntTable));
     if(tbl == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return NULL;
     }
     tbl->mapping = mapping;
     tbl->elements = (ContractionTable **)uprv_malloc(INIT_EXP_TABLE_SIZE*sizeof(ContractionTable *));
     if(tbl->elements == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(tbl);
-        return NULL;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(tbl);
+      return NULL;
     }
     tbl->capacity = INIT_EXP_TABLE_SIZE;
     uprv_memset(tbl->elements, 0, INIT_EXP_TABLE_SIZE*sizeof(ContractionTable *));
@@ -77,22 +79,22 @@ uprv_cnttab_open(UNewTrie *mapping, UErrorCode *status) {
 static ContractionTable *addATableElement(CntTable *table, uint32_t *key, UErrorCode *status) {
     ContractionTable *el = (ContractionTable *)uprv_malloc(sizeof(ContractionTable));
     if(el == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return NULL;
     }
     el->CEs = (uint32_t *)uprv_malloc(INIT_EXP_TABLE_SIZE*sizeof(uint32_t));
     if(el->CEs == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(el);
-        return NULL;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(el);
+      return NULL;
     }
 
     el->codePoints = (UChar *)uprv_malloc(INIT_EXP_TABLE_SIZE*sizeof(UChar));
     if(el->codePoints == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(el->CEs);
-        uprv_free(el);
-        return NULL;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(el->CEs);
+      uprv_free(el);
+      return NULL;
     }
 
     el->position = 0;
@@ -109,20 +111,20 @@ static ContractionTable *addATableElement(CntTable *table, uint32_t *key, UError
     if(table->size == table->capacity) {
         ContractionTable **newElements = (ContractionTable **)uprv_malloc(table->capacity*2*sizeof(ContractionTable *));
         // do realloc
-        /*        table->elements = (ContractionTable **)realloc(table->elements, table->capacity*2*sizeof(ContractionTable *));*/
+/*        table->elements = (ContractionTable **)realloc(table->elements, table->capacity*2*sizeof(ContractionTable *));*/
         if(newElements == NULL) {
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(el->codePoints);
-            uprv_free(el->CEs);
-            uprv_free(el);
-            return NULL;
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          uprv_free(el->codePoints);
+          uprv_free(el->CEs);
+          uprv_free(el);
+          return NULL;
         } else {
-            ContractionTable **oldElements = table->elements;
-            uprv_memcpy(newElements, oldElements, table->capacity*sizeof(ContractionTable *));
-            uprv_memset(newElements+table->capacity, 0, table->capacity*sizeof(ContractionTable *));
-            table->capacity *= 2;
-            table->elements = newElements;
-            uprv_free(oldElements);
+          ContractionTable **oldElements = table->elements;
+          uprv_memcpy(newElements, oldElements, table->capacity*sizeof(ContractionTable *));
+          uprv_memset(newElements+table->capacity, 0, table->capacity*sizeof(ContractionTable *));
+          table->capacity *= 2;
+          table->elements = newElements;
+          uprv_free(oldElements);
         }
     }
 
@@ -143,8 +145,8 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
     }
     table->offsets = (int32_t *)uprv_malloc(table->size*sizeof(int32_t));
     if(table->offsets == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        return 0;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return 0;
     }
 
 
@@ -160,10 +162,10 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
     }
     table->CEs = (uint32_t *)uprv_malloc(table->position*sizeof(uint32_t));
     if(table->CEs == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(table->offsets);
-        table->offsets = NULL;
-        return 0;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(table->offsets);
+      table->offsets = NULL;
+      return 0;
     }
     uprv_memset(table->CEs, '?', table->position*sizeof(uint32_t));
 
@@ -172,12 +174,12 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
     }
     table->codePoints = (UChar *)uprv_malloc(table->position*sizeof(UChar));
     if(table->codePoints == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(table->offsets);
-        table->offsets = NULL;
-        uprv_free(table->CEs);
-        table->CEs = NULL;
-        return 0;
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(table->offsets);
+      table->offsets = NULL;
+      uprv_free(table->CEs);
+      table->CEs = NULL;
+      return 0;
     }
     uprv_memset(table->codePoints, '?', table->position*sizeof(UChar));
 
@@ -189,14 +191,14 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
         int32_t size = table->elements[i]->position;
         uint8_t ccMax = 0, ccMin = 255, cc = 0;
         for(j = 1; j<size; j++) {
-            cc = u_getCombiningClass(table->elements[i]->codePoints[j]);
-            if(cc>ccMax) {
-                ccMax = cc;
-            }
-            if(cc<ccMin) {
-                ccMin = cc;
-            }
-            *(cpPointer+j) = table->elements[i]->codePoints[j];
+          cc = u_getCombiningClass(table->elements[i]->codePoints[j]);
+          if(cc>ccMax) {
+            ccMax = cc;
+          }
+          if(cc<ccMin) {
+            ccMin = cc;
+          }
+          *(cpPointer+j) = table->elements[i]->codePoints[j];
         }
         *cpPointer = ((ccMin==ccMax)?1:0 << 8) | ccMax;
 
@@ -210,11 +212,7 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
         CEPointer += size;
     }
 
-    // TODO: this one apparently updates the contraction CEs to point to a real address (relative to the 
-    // start of the flat file). However, what is done below is just wrong and it affects building of 
-    // tailorings that have constructions in a bad way. At least, one should enumerate the trie. Also,
-    // keeping a list of code points that are contractions might be smart, although I'm not sure if it's
-    // feasible.
+
     uint32_t CE;
     for(i = 0; i<=0x10FFFF; i++) {
         /*CE = ucmpe32_get(table->mapping, i);*/
@@ -231,112 +229,97 @@ uprv_cnttab_constructTable(CntTable *table, uint32_t mainOffset, UErrorCode *sta
 }
 
 static ContractionTable *uprv_cnttab_cloneContraction(ContractionTable *t, UErrorCode *status) {
-    ContractionTable *r = (ContractionTable *)uprv_malloc(sizeof(ContractionTable));
-    if(r == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
-    }
+  ContractionTable *r = (ContractionTable *)uprv_malloc(sizeof(ContractionTable));
+  if(r == NULL) {
+    *status = U_MEMORY_ALLOCATION_ERROR;
+    return NULL;
+  }
 
-    r->position = t->position;
-    r->size = t->size;
+  r->position = t->position;
+  r->size = t->size;
 
-    r->codePoints = (UChar *)uprv_malloc(sizeof(UChar)*t->size);
-    if(r->codePoints == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(r);
-        return NULL;
-    }
-    r->CEs = (uint32_t *)uprv_malloc(sizeof(uint32_t)*t->size);
-    if(r->CEs == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(r->codePoints);
-        uprv_free(r);
-        return NULL;
-    }
-    uprv_memcpy(r->codePoints, t->codePoints, sizeof(UChar)*t->size);
-    uprv_memcpy(r->CEs, t->CEs, sizeof(uint32_t)*t->size);
+  r->codePoints = (UChar *)uprv_malloc(sizeof(UChar)*t->size);
+  r->CEs = (uint32_t *)uprv_malloc(sizeof(uint32_t)*t->size);
+  
+  /* test for NULL */
+  if((r->codePoints == NULL) || (r->CEs == NULL)) {
+    *status = U_MEMORY_ALLOCATION_ERROR;
+    return NULL;
+  }
+  uprv_memcpy(r->codePoints, t->codePoints, sizeof(UChar)*t->size);
+  uprv_memcpy(r->CEs, t->CEs, sizeof(uint32_t)*t->size);
 
-    return r;
+  return r;
 
 }
 
 U_CAPI CntTable* U_EXPORT2
 uprv_cnttab_clone(CntTable *t, UErrorCode *status) {
-    if(U_FAILURE(*status)) {
-        return NULL;
-    }
-    int32_t i = 0;
-    CntTable *r = (CntTable *)uprv_malloc(sizeof(CntTable));
+  if(U_FAILURE(*status)) {
+    return NULL;
+  }
+  int32_t i = 0;
+  CntTable *r = (CntTable *)uprv_malloc(sizeof(CntTable));
+  /* test for NULL */
+  if (r == NULL) {
+    *status = U_MEMORY_ALLOCATION_ERROR;
+    return NULL;
+  }
+  r->position = t->position;
+  r->size = t->size;
+  r->capacity = t->capacity;
+
+  r->mapping = t->mapping;
+
+  r->elements = (ContractionTable **)uprv_malloc(t->capacity*sizeof(ContractionTable *));
+  /* test for NULL */
+  if (r->elements == NULL) {
+    *status = U_MEMORY_ALLOCATION_ERROR;
+    return NULL;
+  }
+  //uprv_memcpy(r->elements, t->elements, t->capacity*sizeof(ContractionTable *));
+
+  for(i = 0; i<t->size; i++) {
+    r->elements[i] = uprv_cnttab_cloneContraction(t->elements[i], status);
+  }
+
+  if(t->CEs != NULL) {
+    r->CEs = (uint32_t *)uprv_malloc(t->position*sizeof(uint32_t));
     /* test for NULL */
-    if (r == NULL) {
+    if (r->CEs == NULL) {
         *status = U_MEMORY_ALLOCATION_ERROR;
         return NULL;
     }
-    r->position = t->position;
-    r->size = t->size;
-    r->capacity = t->capacity;
+    uprv_memcpy(r->CEs, t->CEs, t->position*sizeof(uint32_t));
+  } else {
+    r->CEs = NULL;
+  }
 
-    r->mapping = t->mapping;
-
-    r->elements = (ContractionTable **)uprv_malloc(t->capacity*sizeof(ContractionTable *));
+  if(t->codePoints != NULL) {
+    r->codePoints = (UChar *)uprv_malloc(t->position*sizeof(UChar));
     /* test for NULL */
-    if (r->elements == NULL) {
+    if (r->codePoints == NULL) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(r);
         return NULL;
     }
-    //uprv_memcpy(r->elements, t->elements, t->capacity*sizeof(ContractionTable *));
+    uprv_memcpy(r->codePoints, t->codePoints, t->position*sizeof(UChar));
+  } else {
+    r->codePoints = NULL;
+  }
 
-    for(i = 0; i<t->size; i++) {
-        r->elements[i] = uprv_cnttab_cloneContraction(t->elements[i], status);
+  if(t->offsets != NULL) {
+    r->offsets = (int32_t *)uprv_malloc(t->size*sizeof(int32_t));
+    /* test for NULL */
+    if (r->offsets == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
     }
+    uprv_memcpy(r->offsets, t->offsets, t->size*sizeof(int32_t));
+  } else {
+    r->offsets = NULL;
+  }
 
-    if(t->CEs != NULL) {
-        r->CEs = (uint32_t *)uprv_malloc(t->position*sizeof(uint32_t));
-        /* test for NULL */
-        if (r->CEs == NULL) {
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(r->elements);
-            uprv_free(r);
-            return NULL;
-        }
-        uprv_memcpy(r->CEs, t->CEs, t->position*sizeof(uint32_t));
-    } else {
-        r->CEs = NULL;
-    }
-
-    if(t->codePoints != NULL) {
-        r->codePoints = (UChar *)uprv_malloc(t->position*sizeof(UChar));
-        /* test for NULL */
-        if (r->codePoints == NULL) {
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(r->CEs);
-            uprv_free(r->elements);
-            uprv_free(r);
-            return NULL;
-        }
-        uprv_memcpy(r->codePoints, t->codePoints, t->position*sizeof(UChar));
-    } else {
-        r->codePoints = NULL;
-    }
-
-    if(t->offsets != NULL) {
-        r->offsets = (int32_t *)uprv_malloc(t->size*sizeof(int32_t));
-        /* test for NULL */
-        if (r->offsets == NULL) {
-            *status = U_MEMORY_ALLOCATION_ERROR;
-            uprv_free(r->codePoints);
-            uprv_free(r->CEs);
-            uprv_free(r->elements);
-            uprv_free(r);
-            return NULL;
-        }
-        uprv_memcpy(r->offsets, t->offsets, t->size*sizeof(int32_t));
-    } else {
-        r->offsets = NULL;
-    }
-
-    return r;
+  return r;
 }
 
 U_CAPI void  U_EXPORT2
@@ -365,7 +348,7 @@ uprv_cnttab_changeLastCE(CntTable *table, uint32_t element, uint32_t value, UErr
     }
 
     if((element == 0xFFFFFF) || (tbl = table->elements[element]) == NULL) {
-        return 0;
+      return 0;
     }
 
     tbl->CEs[tbl->position-1] = value;
@@ -378,12 +361,12 @@ uprv_cnttab_changeLastCE(CntTable *table, uint32_t element, uint32_t value, UErr
 U_CAPI uint32_t  U_EXPORT2
 uprv_cnttab_insertContraction(CntTable *table, uint32_t element, UChar codePoint, uint32_t value, UErrorCode *status) {
 
+    element &= 0xFFFFFF;
     ContractionTable *tbl = NULL;
 
     if(U_FAILURE(*status)) {
         return 0;
     }
-    element &= 0xFFFFFF;
 
     if((element == 0xFFFFFF) || (tbl = table->elements[element]) == NULL) {
         tbl = addATableElement(table, &element, status);
@@ -470,40 +453,40 @@ static ContractionTable *_cnttab_getContractionTable(CntTable *table, uint32_t e
     ContractionTable *tbl = NULL;
 
     if((element == 0xFFFFFF) || (tbl = table->elements[element]) == NULL) {
-        return NULL;
+      return NULL;
     } else {
-        return tbl;
+      return tbl;
     }
 }
 
 static int32_t _cnttab_findCP(ContractionTable *tbl, UChar codePoint) {
     uint32_t position = 0;
     if(tbl == NULL) {
-        return -1;
+      return -1;
     }
 
     while(codePoint > tbl->codePoints[position]) {
-        position++;
-        if(position > tbl->position) {
-            return -1;
-        }
+      position++;
+      if(position > tbl->position) {
+        return -1;
+      }
     }
     if (codePoint == tbl->codePoints[position]) {
-        return position;
+      return position;
     } else {
-        return -1;
+      return -1;
     }
 }
 
 static uint32_t _cnttab_getCE(ContractionTable *tbl, int32_t position) {
-    if(tbl == NULL) {
-        return UCOL_NOT_FOUND;
-    }
-    if((uint32_t)position > tbl->position || position == -1) {
-        return UCOL_NOT_FOUND;
-    } else {
-        return tbl->CEs[position];
-    }
+  if(tbl == NULL) {
+    return UCOL_NOT_FOUND;
+  }
+  if((uint32_t)position > tbl->position || position == -1) {
+    return UCOL_NOT_FOUND;
+  } else {
+    return tbl->CEs[position];
+  }
 }
 
 U_CAPI int32_t  U_EXPORT2
@@ -541,16 +524,20 @@ uprv_cnttab_isTailored(CntTable *table, uint32_t element, UChar *ztString, UErro
     }
 
     while(*(ztString)!=0) {
-        element = uprv_cnttab_findCE(table, element, *(ztString), status);
-        if(element == UCOL_NOT_FOUND) {
-            return FALSE;
-        }
-        if(!isCntTableElement(element)) {
-            return TRUE;
-        }
-        ztString++;
+      element = uprv_cnttab_findCE(table, element, *(ztString), status);
+      if(element == UCOL_NOT_FOUND) {
+        return FALSE;
+      }
+      if(!isCntTableElement(element)) {
+        return TRUE;
+      }
+      ztString++;
+    }
+    if(uprv_cnttab_getCE(table, element, 0, status) != UCOL_NOT_FOUND) {
+      return TRUE;
+    } else {
+      return FALSE; 
     }
-    return (UBool)(uprv_cnttab_getCE(table, element, 0, status) != UCOL_NOT_FOUND);
 }
 
 U_CAPI uint32_t  U_EXPORT2
@@ -564,23 +551,25 @@ uprv_cnttab_changeContraction(CntTable *table, uint32_t element, UChar codePoint
     }
 
     if((element == 0xFFFFFF) || (tbl = table->elements[element]) == NULL) {
-        return 0;
+      return 0;
     }
 
     uint32_t position = 0;
 
     while(codePoint > tbl->codePoints[position]) {
-        position++;
-        if(position > tbl->position) {
-            return UCOL_NOT_FOUND;
-        }
+      position++;
+      if(position > tbl->position) {
+        return UCOL_NOT_FOUND;
+      }
     }
     if (codePoint == tbl->codePoints[position]) {
-        tbl->CEs[position] = newCE;
-        return element;
+      tbl->CEs[position] = newCE;
+      return element;
     } else {
-        return UCOL_NOT_FOUND;
+      return UCOL_NOT_FOUND;
     }
 }
 
+U_NAMESPACE_END
+
 #endif /* #if !UCONFIG_NO_COLLATION */
