@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1998-2010, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -33,21 +33,13 @@
 #define INVC_DATA_TYPE "icu"
 #define INVC_DATA_NAME "invuca"
 
-/**
- * Convenience string denoting the Collation data tree
- * @internal ICU 3.0
- */
-#define U_ICUDATA_COLL U_ICUDATA_NAME U_TREE_SEPARATOR_STRING "coll"
-
 #if !UCONFIG_NO_COLLATION
 
-#ifdef XP_CPLUSPLUS
-#include "unicode/normalizer2.h"
-#include "unicode/unistr.h"
-#endif
 #include "unicode/ucol.h"
 #include "utrie.h"
-#include "cmemory.h"
+#include "unicode/ures.h"
+#include "unicode/udata.h"
+#include "unicode/uiter.h"
 
 /* This is the internal header file which contains important declarations for 
  * the collation framework. 
@@ -60,7 +52,7 @@
  * The following describes the formats for collation binaries
  * (UCA & tailorings) and for the inverse UCA table.
  * Substructures are described in the collation design document at
- * http://source.icu-project.org/repos/icu/icuhtml/trunk/design/collation/ICU_collation_design.htm
+ * http://oss.software.ibm.com/cvs/icu/~checkout~/icuhtml/design/collation/ICU_collation_design.htm
  *
  * -------------------------------------------------------------
  *
@@ -172,28 +164,42 @@
 #define UCOL_HEADER_MAGIC 0x20030618
 
 /* UDataInfo for UCA mapping table */
-/* dataFormat="UCol"            */
-#define UCA_DATA_FORMAT_0 ((uint8_t)0x55)
-#define UCA_DATA_FORMAT_1 ((uint8_t)0x43)
-#define UCA_DATA_FORMAT_2 ((uint8_t)0x6f)
-#define UCA_DATA_FORMAT_3 ((uint8_t)0x6c)
+static const UDataInfo ucaDataInfo={
+    sizeof(UDataInfo),
+    0,
 
-#define UCA_FORMAT_VERSION_0 ((uint8_t)2)
-#define UCA_FORMAT_VERSION_1 ((uint8_t)3)
-#define UCA_FORMAT_VERSION_2 ((uint8_t)0)
-#define UCA_FORMAT_VERSION_3 ((uint8_t)0)
+    U_IS_BIG_ENDIAN,
+    U_CHARSET_FAMILY,
+    sizeof(UChar),
+    0,
+
+    {0x55, 0x43, 0x6f, 0x6c},     /* dataFormat="UCol"            */
+    /* 03/26/2002 bumped up version since format has changed */
+    /* 09/16/2002 bumped up version since we went from UColAttributeValue */
+    /*            to int32_t in UColOptionSet */
+    /* 05/13/2003 This one also updated since we added UCA and UCD versions */
+    /*            to header */
+    /* 09/11/2003 Adding information required by data swapper */
+    {2, 3, 0, 0},                 /* formatVersion                */
+    {3, 0, 0, 0}                  /* dataVersion = Unicode Version*/
+};
 
 /* UDataInfo for inverse UCA table */
-/* dataFormat="InvC"            */
-#define INVUCA_DATA_FORMAT_0 ((uint8_t)0x49)
-#define INVUCA_DATA_FORMAT_1 ((uint8_t)0x6E)
-#define INVUCA_DATA_FORMAT_2 ((uint8_t)0x76)
-#define INVUCA_DATA_FORMAT_3 ((uint8_t)0x43)
+static const UDataInfo invUcaDataInfo={
+    sizeof(UDataInfo),
+    0,
 
-#define INVUCA_FORMAT_VERSION_0 ((uint8_t)2)
-#define INVUCA_FORMAT_VERSION_1 ((uint8_t)1)
-#define INVUCA_FORMAT_VERSION_2 ((uint8_t)0)
-#define INVUCA_FORMAT_VERSION_3 ((uint8_t)0)
+    U_IS_BIG_ENDIAN,
+    U_CHARSET_FAMILY,
+    sizeof(UChar),
+    0,
+
+    {0x49, 0x6E, 0x76, 0x43},     /* dataFormat="InvC"            */
+    /* 03/26/2002 bumped up version since format has changed */
+    /* 04/29/2003 2.1 format - we have added UCA version to header */
+    {2, 1, 0, 0},                 /* formatVersion                */
+    {3, 0, 0, 0}                  /* dataVersion = Unicode Version*/
+};
 
 /* This is the size of the stack allocated buffer for sortkey generation and similar operations */
 /* if it is too small, heap allocation will occur.*/
@@ -221,13 +227,10 @@ minimum number for special Jamo
 /* This is the size of the buffer for expansion CE's */
 /* In reality we should not have to deal with expm sequences longer then 16 */
 /* you can change this value if you need memory */
-/* WARNING THIS BUFFER DOES HAVE MALLOC FALLBACK. If you make it too small, you'll get into performance trouble */
+/* WARNING THIS BUFFER DOES NOT HAVE MALLOC FALLBACK. If you make it too small, you'll get in trouble */
 /* Reasonable small value is around 10, if you don't do Arabic or other funky collations that have long expansion sequence */
 /* This is the longest expansion sequence we can handle without bombing out */
-#define UCOL_EXPAND_CE_BUFFER_SIZE 64
-
-/* This is the size to increase the buffer for expansion CE's */
-#define UCOL_EXPAND_CE_BUFFER_EXTEND_SIZE 64
+#define UCOL_EXPAND_CE_BUFFER_SIZE 512 /* synwee :TODO revert back 64*/
 
 
 /* Unsafe UChar hash table table size.                                           */
@@ -264,95 +267,49 @@ minimum number for special Jamo
                               /* by index */
 #define UCOL_USE_ITERATOR   64
 
-#define UCOL_FORCE_HAN_IMPLICIT 128
-
 #define NFC_ZERO_CC_BLOCK_LIMIT_  0x300
 
-#ifdef XP_CPLUSPLUS
-
-typedef struct collIterate : public U_NAMESPACE_QUALIFIER UMemory {
-  const UChar *string; /* Original string */
+struct collIterate {
+  UChar *string; /* Original string */
   /* UChar *start;  Pointer to the start of the source string. Either points to string
                     or to writableBuffer */
-  const UChar *endp; /* string end ptr.  Is undefined for null terminated strings */
-  const UChar *pos; /* This is position in the string.  Can be to original or writable buf */
+  UChar *endp;   /* string end ptr.  Is undefined for null terminated strings */
+  UChar *pos; /* This is position in the string.  Can be to original or writable buf */
 
   uint32_t *toReturn; /* This is the CE from CEs buffer that should be returned */
   uint32_t *CEpos; /* This is the position to which we have stored processed CEs */
-
-  int32_t *offsetReturn; /* This is the offset to return, if non-NULL */
-  int32_t *offsetStore;  /* This is the pointer for storing offsets */
-  int32_t offsetRepeatCount;  /* Repeat stored offset if non-zero */
-  int32_t offsetRepeatValue;  /* offset value to repeat */
-
-  U_NAMESPACE_QUALIFIER UnicodeString writableBuffer;
-  const UChar *fcdPosition; /* Position in the original string to continue FCD check from. */
+  UChar *writableBuffer;
+  uint32_t writableBufSize;
+  UChar *fcdPosition; /* Position in the original string to continue FCD check from. */
   const UCollator *coll;
-  const U_NAMESPACE_QUALIFIER Normalizer2 *nfd;
   uint8_t   flags;
   uint8_t   origFlags;
-  uint32_t *extendCEs; /* This is use if CEs is not big enough */
-  int32_t extendCEsSize; /* Holds the size of the dynamic CEs buffer */
   uint32_t CEs[UCOL_EXPAND_CE_BUFFER_SIZE]; /* This is where we store CEs */
-
-  int32_t *offsetBuffer;    /* A dynamic buffer to hold offsets */
-  int32_t offsetBufferSize; /* The size of the offset buffer */
-
+  UChar stackWritableBuffer[UCOL_WRITABLE_BUFFER_SIZE]; /* A writable buffer. */
   UCharIterator *iterator;
   /*int32_t iteratorIndex;*/
-} collIterate;
-
-#else
-
-typedef struct collIterate collIterate;
-
-#endif
-
-#define paddedsize(something) ((something)+((((something)%4)!=0)?(4-(something)%4):0))
-#define headersize (paddedsize(sizeof(UCATableHeader))+paddedsize(sizeof(UColOptionSet)))
+};
 
 /* 
 struct used internally in getSpecial*CE.
 data similar to collIterate.
 */
 struct collIterateState {
-    const UChar *pos; /* This is position in the string.  Can be to original or writable buf */
-    const UChar *returnPos;
-    const UChar *fcdPosition; /* Position in the original string to continue FCD check from. */
-    const UChar *bufferaddress; /* address of the normalization buffer */
-    int32_t  buffersize;
+    UChar    *pos; /* This is position in the string.  Can be to original or writable buf */
+    UChar    *fcdPosition; /* Position in the original string to continue FCD check from. */
+    UChar    *bufferaddress; /* address of the normalization buffer */
+    uint32_t  buffersize;
     uint8_t   flags;
     uint8_t   origFlags;
     uint32_t   iteratorIndex;
     int32_t    iteratorMove;
 };
 
-U_CAPI void U_EXPORT2
-uprv_init_collIterate(const UCollator *collator,
-                      const UChar *sourceString, int32_t sourceLen,
-                      collIterate *s, UErrorCode *status);
+U_CAPI void U_EXPORT2 
+uprv_init_collIterate(const UCollator *collator, const UChar *sourceString, int32_t sourceLen, collIterate *s);
 
-/* Internal functions for C test code. */
-U_CAPI collIterate * U_EXPORT2
-uprv_new_collIterate(UErrorCode *status);
 
-U_CAPI void U_EXPORT2
-uprv_delete_collIterate(collIterate *s);
-
-/* @return s->pos == s->endp */
-U_CAPI UBool U_EXPORT2
-uprv_collIterateAtEnd(collIterate *s);
-
-#ifdef XP_CPLUSPLUS
-
-U_NAMESPACE_BEGIN
-
-struct UCollationPCE;
-typedef struct UCollationPCE UCollationPCE;
-
-U_NAMESPACE_END
-
-struct UCollationElements : public U_NAMESPACE_QUALIFIER UMemory
+struct UCollationElements
 {
   /**
   * Struct wrapper for source data
@@ -366,17 +323,8 @@ struct UCollationElements : public U_NAMESPACE_QUALIFIER UMemory
   * Indicates if the data should be deleted.
   */
         UBool              isWritable;
-
-/**
- * Data for getNextProcessed, getPreviousProcessed.
- */
-        U_NAMESPACE_QUALIFIER UCollationPCE     *pce;
 };
 
-#endif
-
-U_CAPI void U_EXPORT2
-uprv_init_pce(const struct UCollationElements *elems);
 
 #define UCOL_LEVELTERMINATOR 1
 
@@ -419,27 +367,6 @@ uprv_init_pce(const struct UCollationElements *elems);
                          (((uint32_t)(ch) - 0x1161) <= (0x1175 - 0x1161)) || \
                          (((uint32_t)(ch) - 0x11A8) <= (0x11C2 - 0x11A8)))
 
-/* Han character ranges */
-#define UCOL_FIRST_HAN 0x4E00
-#define UCOL_LAST_HAN  0x9FFF
-#define UCOL_FIRST_HAN_A 0x3400
-#define UCOL_LAST_HAN_A  0x4DBF
-#define UCOL_FIRST_HAN_COMPAT 0xFAE0
-#define UCOL_LAST_HAN_COMPAT  0xFA2F
-
-/* Han extension B is in plane 2 */
-#define UCOL_FIRST_HAN_B       0x20000
-#define UCOL_LAST_HAN_B        0x2A6DF
-
-/* Hangul range */
-#define UCOL_FIRST_HANGUL 0xAC00
-#define UCOL_LAST_HANGUL  0xD7AF
-
-/* Jamo ranges */
-#define UCOL_FIRST_L_JAMO 0x1100
-#define UCOL_FIRST_V_JAMO 0x1161
-#define UCOL_FIRST_T_JAMO 0x11A8
-#define UCOL_LAST_T_JAMO  0x11F9
 
 
 #if 0
@@ -507,7 +434,7 @@ U_CFUNC
 uint32_t ucol_prv_getSpecialPrevCE(const UCollator *coll, UChar ch, uint32_t CE,
                           collIterate *source, UErrorCode *status);
 U_CAPI uint32_t U_EXPORT2 ucol_getNextCE(const UCollator *coll, collIterate *collationSource, UErrorCode *status);
-U_CFUNC uint32_t U_EXPORT2 ucol_getPrevCE(const UCollator *coll,
+U_CAPI uint32_t U_EXPORT2 ucol_getPrevCE(const UCollator *coll,
                                          collIterate *collationSource,
                                          UErrorCode *status);
 /* function used by C++ getCollationKey to prevent restarting the calculation */
@@ -555,15 +482,15 @@ ucol_getSortKeySize(const UCollator *coll, collIterate *s,
  * @return memory, owned by the caller, of size 'length' bytes.
  * @internal INTERNAL USE ONLY
  */
-U_CFUNC uint8_t* U_EXPORT2 
+U_CAPI uint8_t* U_EXPORT2 
 ucol_cloneRuleData(const UCollator *coll, int32_t *length, UErrorCode *status);
 
 /**
  * Used to set requested and valid locales on a collator returned by the collator
  * service.
  */
-U_CFUNC void U_EXPORT2
-ucol_setReqValidLocales(UCollator *coll, char *requestedLocaleToAdopt, char *validLocaleToAdopt, char *actualLocaleToAdopt);
+U_CAPI void U_EXPORT2
+ucol_setReqValidLocales(UCollator *coll, char *requestedLocaleToAdopt, char *validLocaleToAdopt);
 
 #define UCOL_SPECIAL_FLAG 0xF0000000
 #define UCOL_TAG_SHIFT 24
@@ -639,7 +566,7 @@ enum {
     UCOL_BYTE_FIRST_TAILORED = 0x04,
     UCOL_BYTE_COMMON = 0x05,
     UCOL_BYTE_FIRST_UCA = UCOL_BYTE_COMMON,
-    UCOL_CODAN_PLACEHOLDER = 0x27,
+    UCOL_CODAN_PLACEHOLDER = 0x24,
     UCOL_BYTE_LAST_LATIN_PRIMARY = 0x4C,
     UCOL_BYTE_FIRST_NON_LATIN_PRIMARY = 0x4D,
     UCOL_BYTE_UNSHIFTED_MAX = 0xFF
@@ -739,8 +666,8 @@ typedef enum {
                              /* It fits in a single 32 bit CE and is used instead of expansion to save */
                              /* space without affecting the performance (hopefully) */
                              
-    DIGIT_TAG = 13,          /* COllate Digits As Numbers (CODAN) implementation */
-    
+	DIGIT_TAG = 13,			/* COllate Digits As Numbers (CODAN) implementation */
+	
     CE_TAGS_COUNT
 } UColCETags;
 
@@ -929,27 +856,22 @@ SortKeyGenerator(const    UCollator    *coll,
         UBool allocatePrimary,
         UErrorCode *status);
 
-typedef void U_CALLCONV
-ResourceCleaner(UCollator *coll);
-
-
 struct UCollator {
     UColOptionSet  *options;
     SortKeyGenerator *sortKeyGen;
     uint32_t *latinOneCEs;
-    char* actualLocale;
     char* validLocale;
     char* requestedLocale;
-    const UChar *rules;
-    const UChar *ucaRules;
-    const UCollator *UCA;
+    UResourceBundle *rb;
+    UResourceBundle *elements;
     const UCATableHeader *image;
-    UTrie mapping;
+    /*CompactEIntArray *mapping;*/
+    UTrie *mapping;
     const uint32_t *latinOneMapping;
     const uint32_t *expansion;
     const UChar    *contractionIndex;
     const uint32_t *contractionCEs;
-    /*const uint8_t  *scriptOrder;*/
+    const uint8_t  *scriptOrder;
 
     const uint32_t *endExpansionCE;    /* array of last ces in an expansion ce.
                                           corresponds to expansionCESize */
@@ -963,9 +885,12 @@ struct UCollator {
     UChar          minUnsafeCP;        /* Smallest unsafe Code Point. */
     UChar          minContrEndCP;      /* Smallest code point at end of a contraction */
 
+    const UChar *rules;
     int32_t rulesLength;
     int32_t latinOneTableLen;
 
+    /*UErrorCode errorCode;*/             /* internal error code */
+
     uint32_t variableTopValue;
     UColAttributeValue frenchCollation;
     UColAttributeValue alternateHandling; /* attribute for handling variable elements*/
@@ -997,7 +922,7 @@ struct UCollator {
     UBool latinOneRegenTable;
     UBool latinOneFailed;
 
-    int8_t tertiaryAddition; /* when switching case, we need to add or subtract different values */
+    int32_t tertiaryAddition; /* when switching case, we need to add or subtract different values */
     uint8_t caseSwitch;
     uint8_t tertiaryCommon;
     uint8_t tertiaryMask;
@@ -1006,7 +931,9 @@ struct UCollator {
     uint8_t tertiaryTopCount;
     uint8_t tertiaryBottomCount;
 
-    UVersionInfo dataVersion;               /* Data info of UCA table */
+    UDataInfo dataInfo;               /* Data info of UCA table */
+    const UCollator *UCA;
+
 };
 
 U_CDECL_END
@@ -1034,19 +961,20 @@ void ucol_putOptionsToHeader(UCollator* result, UColOptionSet * opts, UErrorCode
 U_CFUNC
 void ucol_updateInternalState(UCollator *coll, UErrorCode *status);
 
-U_CFUNC uint32_t U_EXPORT2 ucol_getFirstCE(const UCollator *coll, UChar u, UErrorCode *status);
+U_CAPI uint32_t U_EXPORT2 ucol_getFirstCE(const UCollator *coll, UChar u, UErrorCode *status);
+U_CAPI char* U_EXPORT2 ucol_sortKeyToString(const UCollator *coll, const uint8_t *sortkey, char *buffer, uint32_t *len);
 U_CAPI UBool U_EXPORT2 ucol_isTailored(const UCollator *coll, const UChar u, UErrorCode *status);
 
 U_CAPI const InverseUCATableHeader* U_EXPORT2 ucol_initInverseUCA(UErrorCode *status);
 
 U_CAPI void U_EXPORT2 
-uprv_uca_initImplicitConstants(UErrorCode *status);
+uprv_uca_initImplicitConstants(int32_t minPrimary, int32_t maxPrimary, UErrorCode *status);
 
 U_CAPI uint32_t U_EXPORT2
 uprv_uca_getImplicitFromRaw(UChar32 cp);
 
-/*U_CFUNC uint32_t U_EXPORT2
-uprv_uca_getImplicitPrimary(UChar32 cp);*/
+U_CAPI uint32_t U_EXPORT2
+uprv_uca_getImplicitPrimary(UChar32 cp);
 
 U_CAPI UChar32 U_EXPORT2
 uprv_uca_getRawFromImplicit(uint32_t implicit);
@@ -1057,6 +985,16 @@ uprv_uca_getRawFromCodePoint(UChar32 i);
 U_CAPI UChar32 U_EXPORT2
 uprv_uca_getCodePointFromRaw(UChar32 i);
 
+/** Check whether two collators are equal. Collators are considered equal if they
+ *  will sort strings the same. This means that both the current attributes and the
+ *  rules must be equivalent. Currently used for RuleBasedCollator::operator==.
+ *  @param source first collator
+ *  @param target second collator
+ *  @return TRUE or FALSE
+ */
+U_CAPI UBool U_EXPORT2
+ucol_equals(const UCollator *source, const UCollator *target);
+
 
 
 #ifdef XP_CPLUSPLUS
@@ -1078,20 +1016,18 @@ static inline UBool ucol_unsafeCP(UChar c, const UCollator *coll) {
 
     hash = c;
     if (hash >= UCOL_UNSAFECP_TABLE_SIZE*8) {
-        if(UTF_IS_SURROGATE(c)) {
-            /*  Lead or trail surrogate             */
+      if(UTF_IS_LEAD(c) || UTF_IS_TRAIL(c)) {
+            /*  Trail surrogate                     */
             /*  These are always considered unsafe. */
             return TRUE;
         }
         hash = (hash & UCOL_UNSAFECP_TABLE_MASK) + 256;
     }
     htbyte = coll->unsafeCP[hash>>3];
-    return ((htbyte >> (hash & 7)) & 1);
+    return (((htbyte >> (hash & 7)) & 1) == 1);
 }
 #endif /* XP_CPLUSPLUS */
 
-/* The offsetBuffer in collIterate might need to be freed to avoid memory leaks. */
-void ucol_freeOffsetBuffer(collIterate *s); 
 
 #endif /* #if !UCONFIG_NO_COLLATION */
 
