@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001-2010, International Business Machines
+*   Copyright (C) 2001-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -43,7 +43,7 @@
 
 /* this is space for the extra strings that need to be unquoted */
 /* during the parsing of the rules */
-#define UCOL_TOK_EXTRA_RULE_SPACE_SIZE 4096
+#define UCOL_TOK_EXTRA_RULE_SPACE_SIZE 2048
 typedef struct UColToken UColToken;
 
 typedef struct  {
@@ -83,7 +83,6 @@ struct UColToken {
   UColToken* previous;
   UColToken* next;
   UChar *rulesToParse;
-  uint16_t flags;
 };
 
 /* 
@@ -122,8 +121,6 @@ typedef struct {
   UColToken *varTop;
   USet *copySet;
   USet *removeSet;
-  UBool buildCCTabFlag;  /* Tailoring rule requirs building combining class table. */
-  uint32_t prevStrength;
 } UColTokenParser;
 
 typedef struct {
@@ -171,12 +168,11 @@ U_CAPI int32_t U_EXPORT2 ucol_inv_getNextCE(const UColTokenParser *src,
                                             uint32_t CE, uint32_t contCE,
                                             uint32_t *nextCE, uint32_t *nextContCE,
                                             uint32_t strength);
-U_CFUNC int32_t U_EXPORT2 ucol_inv_getPrevCE(const UColTokenParser *src,
+U_CAPI int32_t U_EXPORT2 ucol_inv_getPrevCE(const UColTokenParser *src,
                                             uint32_t CE, uint32_t contCE,
                                             uint32_t *prevCE, uint32_t *prevContCE,
                                             uint32_t strength);
 
-
 #endif /* #if !UCONFIG_NO_COLLATION */
 
 #endif
