@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2011, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -29,29 +29,6 @@
 #include "unicode/msgfmt.h"
 #include "unicode/unistr.h"
 #include "cpputils.h"
-#include "uassert.h"
-#include "ustr_imp.h"
-
-U_NAMESPACE_BEGIN
-/**
- * This class isolates our access to private internal methods of
- * MessageFormat.  It is never instantiated; it exists only for C++
- * access management.
- */
-class MessageFormatAdapter {
-public:
-    static const Formattable::Type* getArgTypeList(const MessageFormat& m,
-                                                   int32_t& count);
-    static UBool hasArgTypeConflicts(const MessageFormat& m) {
-        return m.hasArgTypeConflicts;
-    }
-};
-const Formattable::Type*
-MessageFormatAdapter::getArgTypeList(const MessageFormat& m,
-                                     int32_t& count) {
-    return m.getArgTypeList(count);
-}
-U_NAMESPACE_END
 
 U_NAMESPACE_USE
 
@@ -238,23 +215,25 @@ umsg_open(  const UChar     *pattern,
     }
 
     UParseError tErr;
+   
     if(parseError==NULL)
     {
         parseError = &tErr;
     }
+        
+    UMessageFormat* retVal = 0;
 
     int32_t len = (patternLength == -1 ? u_strlen(pattern) : patternLength);
-    UnicodeString patString(patternLength == -1, pattern, len);
+    
+    UnicodeString patString((patternLength == -1 ? TRUE:FALSE), pattern,len);
 
-    MessageFormat* retVal = new MessageFormat(patString,Locale(locale),*parseError,*status);
-    if(retVal == NULL) {
+    retVal = (UMessageFormat*) new MessageFormat(pattern,Locale(locale),*parseError,*status);
+    
+    if(retVal == 0) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
+        return 0;
     }
-    if (U_SUCCESS(*status) && MessageFormatAdapter::hasArgTypeConflicts(*retVal)) {
-        *status = U_ARGUMENT_TYPE_MISMATCH;
-    }
-    return (UMessageFormat*)retVal;
+    return retVal;
 }
 
 U_CAPI void U_EXPORT2
@@ -298,13 +277,13 @@ umsg_setLocale(UMessageFormat *fmt, const char* locale)
 }
 
 U_CAPI const char*  U_EXPORT2
-umsg_getLocale(const UMessageFormat *fmt)
+umsg_getLocale(UMessageFormat *fmt)
 {
     //check arguments
     if(fmt==NULL){
         return "";
     }
-    return ((const MessageFormat*)fmt)->getLocale().getName();
+    return ((MessageFormat*)fmt)->getLocale().getName();
 }
 
 U_CAPI void  U_EXPORT2
@@ -335,7 +314,7 @@ umsg_applyPattern(UMessageFormat *fmt,
 }
 
 U_CAPI int32_t  U_EXPORT2
-umsg_toPattern(const UMessageFormat *fmt,
+umsg_toPattern(UMessageFormat *fmt,
                UChar* result, 
                int32_t resultLength,
                UErrorCode* status)
@@ -356,12 +335,12 @@ umsg_toPattern(const UMessageFormat *fmt,
         // otherwise, alias the destination buffer
         res.setTo(result, 0, resultLength);
     }
-    ((const MessageFormat*)fmt)->toPattern(res);
+    ((MessageFormat*)fmt)->toPattern(res);
     return res.extract(result, resultLength, *status);
 }
 
 U_CAPI int32_t
-umsg_format(    const UMessageFormat *fmt,
+umsg_format(    UMessageFormat *fmt,
                 UChar          *result,
                 int32_t        resultLength,
                 UErrorCode     *status,
@@ -385,8 +364,26 @@ umsg_format(    const UMessageFormat *fmt,
     return actLen;
 }
 
+U_NAMESPACE_BEGIN
+/**
+ * This class isolates our access to private internal methods of
+ * MessageFormat.  It is never instantiated; it exists only for C++
+ * access management.
+ */
+class MessageFormatAdapter {
+public:
+    static const Formattable::Type* getArgTypeList(const MessageFormat& m,
+                                                   int32_t& count);
+};
+const Formattable::Type*
+MessageFormatAdapter::getArgTypeList(const MessageFormat& m,
+                                     int32_t& count) {
+    return m.getArgTypeList(count);
+}
+U_NAMESPACE_END
+
 U_CAPI int32_t U_EXPORT2
-umsg_vformat(   const UMessageFormat *fmt,
+umsg_vformat(   UMessageFormat *fmt,
                 UChar          *result,
                 int32_t        resultLength,
                 va_list        ap,
@@ -404,7 +401,7 @@ umsg_vformat(   const UMessageFormat *fmt,
 
     int32_t count =0;
     const Formattable::Type* argTypes =
-        MessageFormatAdapter::getArgTypeList(*(const MessageFormat*)fmt, count);
+        MessageFormatAdapter::getArgTypeList(*(MessageFormat*)fmt, count);
     // Allocate at least one element.  Allocating an array of length
     // zero causes problems on some platforms (e.g. Win32).
     Formattable* args = new Formattable[count ? count : 1];
@@ -456,22 +453,13 @@ umsg_vformat(   const UMessageFormat *fmt,
             va_arg(ap, int);
             break;
 
-        case Formattable::kObject:
-        default:
-            // This will never happen because MessageFormat doesn't
-            // support kObject.  When MessageFormat is changed to
-            // understand MeasureFormats, modify this code to do the
-            // right thing. [alan]
-            U_ASSERT(FALSE);
-            *status=U_ILLEGAL_ARGUMENT_ERROR;
-            break;
         }
     }
     UnicodeString resultStr;
     FieldPosition fieldPosition(0);
     
     /* format the message */
-    ((const MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
+    ((MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
 
     delete[] args;
 
@@ -483,7 +471,7 @@ umsg_vformat(   const UMessageFormat *fmt,
 }
 
 U_CAPI void
-umsg_parse( const UMessageFormat *fmt,
+umsg_parse( UMessageFormat *fmt,
             const UChar    *source,
             int32_t        sourceLength,
             int32_t        *count,
@@ -505,7 +493,7 @@ umsg_parse( const UMessageFormat *fmt,
 }
 
 U_CAPI void U_EXPORT2
-umsg_vparse(const UMessageFormat *fmt,
+umsg_vparse(UMessageFormat *fmt,
             const UChar    *source,
             int32_t        sourceLength,
             int32_t        *count,
@@ -526,7 +514,7 @@ umsg_vparse(const UMessageFormat *fmt,
     }
 
     UnicodeString srcString(source,sourceLength);
-    Formattable *args = ((const MessageFormat*)fmt)->parse(source,*count,*status);
+    Formattable *args = ((MessageFormat*)fmt)->parse(source,*count,*status);
     UDate *aDate;
     double *aDouble;
     UChar *aString;
@@ -586,17 +574,9 @@ umsg_vparse(const UMessageFormat *fmt,
             }
             break;
 
-        case Formattable::kObject:
-            // This will never happen because MessageFormat doesn't
-            // support kObject.  When MessageFormat is changed to
-            // understand MeasureFormats, modify this code to do the
-            // right thing. [alan]
-            U_ASSERT(FALSE);
-            break;
-
         // better not happen!
         case Formattable::kArray:
-            U_ASSERT(FALSE);
+            // DIE
             break;
         }
     }
@@ -605,105 +585,18 @@ umsg_vparse(const UMessageFormat *fmt,
     delete [] args;
 }
 
-#define SINGLE_QUOTE      ((UChar)0x0027)
-#define CURLY_BRACE_LEFT  ((UChar)0x007B)
-#define CURLY_BRACE_RIGHT ((UChar)0x007D)
-
-#define STATE_INITIAL 0
-#define STATE_SINGLE_QUOTE 1
-#define STATE_IN_QUOTE 2
-#define STATE_MSG_ELEMENT 3
-
-#define MAppend(c) if (len < destCapacity) dest[len++] = c; else len++
-
-int32_t umsg_autoQuoteApostrophe(const UChar* pattern, 
-                 int32_t patternLength,
-                 UChar* dest,
-                 int32_t destCapacity,
-                 UErrorCode* ec)
+U_CAPI const char* U_EXPORT2
+umsg_getLocaleByType(const UMessageFormat *fmt,
+                     ULocDataLocaleType type,
+                     UErrorCode* status)
 {
-    int32_t state = STATE_INITIAL;
-    int32_t braceCount = 0;
-    int32_t len = 0;
-
-    if (ec == NULL || U_FAILURE(*ec)) {
-        return -1;
-    }
-
-    if (pattern == NULL || patternLength < -1 || (dest == NULL && destCapacity > 0)) {
-        *ec = U_ILLEGAL_ARGUMENT_ERROR;
-        return -1;
-    }
-
-    if (patternLength == -1) {
-        patternLength = u_strlen(pattern);
-    }
-
-    for (int i = 0; i < patternLength; ++i) {
-        UChar c = pattern[i];
-        switch (state) {
-        case STATE_INITIAL:
-            switch (c) {
-            case SINGLE_QUOTE:
-                state = STATE_SINGLE_QUOTE;
-                break;
-            case CURLY_BRACE_LEFT:
-                state = STATE_MSG_ELEMENT;
-                ++braceCount;
-                break;
-            }
-            break;
-
-        case STATE_SINGLE_QUOTE:
-            switch (c) {
-            case SINGLE_QUOTE:
-                state = STATE_INITIAL;
-                break;
-            case CURLY_BRACE_LEFT:
-            case CURLY_BRACE_RIGHT:
-                state = STATE_IN_QUOTE;
-                break;
-            default:
-                MAppend(SINGLE_QUOTE);
-                state = STATE_INITIAL;
-                break;
-            }
-        break;
-
-        case STATE_IN_QUOTE:
-            switch (c) {
-            case SINGLE_QUOTE:
-                state = STATE_INITIAL;
-                break;
-            }
-            break;
-
-        case STATE_MSG_ELEMENT:
-            switch (c) {
-            case CURLY_BRACE_LEFT:
-                ++braceCount;
-                break;
-            case CURLY_BRACE_RIGHT:
-                if (--braceCount == 0) {
-                    state = STATE_INITIAL;
-                }
-                break;
-            }
-            break;
-
-        default: // Never happens.
-            break;
+    if (fmt == NULL) {
+        if (U_SUCCESS(*status)) {
+            *status = U_ILLEGAL_ARGUMENT_ERROR;
         }
-
-        MAppend(c);
+        return NULL;
     }
-
-    // End of scan
-    if (state == STATE_SINGLE_QUOTE || state == STATE_IN_QUOTE) {
-        MAppend(SINGLE_QUOTE);
-    }
-
-    return u_terminateUChars(dest, destCapacity, len, ec);
+    return ((Format*)fmt)->getLocaleID(type, *status);
 }
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
