@@ -1,6 +1,6 @@
 /*
 ********************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ********************************************************************************
 *
@@ -15,15 +15,17 @@
 *                           boolean array.
 *   4/10/97     aliu        Clean up.  Modified to work on AIX.
 *   8/6/97      nos         Removed overloaded constructor, member var 'buffer'.
-*    07/22/98    stephen        Removed operator!= (implemented in Format)
+*   07/22/98    stephen     Removed operator!= (implemented in Format)
 ********************************************************************************
 */
  
 #ifndef CHOICFMT_H
 #define CHOICFMT_H
  
-
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/unistr.h"
 #include "unicode/numfmt.h"
 #include "unicode/fieldpos.h"
@@ -34,7 +36,7 @@ U_NAMESPACE_BEGIN
 class MessageFormat;
 
 /**
- * <p><code>ChoiceFormat</code> converts between ranges of numeric values
+ * ChoiceFormat converts between ranges of numeric values
  * and string names for those ranges. A <code>ChoiceFormat</code> splits
  * the real number line <code>-Inf</code> to <code>+Inf</code> into two
  * or more contiguous ranges. Each range is mapped to a
@@ -53,21 +55,27 @@ class MessageFormat;
  * <p>In most cases, the preferred way to define a
  * <code>ChoiceFormat</code> is with a pattern. Here is an example of a
  * <code>ChoiceFormat</code> pattern:</p>
+ *
+ * \htmlonly<pre>    0&#x2264;are no files|1&#x2264;is one file|1&lt;are many files</pre>\endhtmlonly
+ * 
+ * <p>or equivalently,</p>
  * 
  * <pre>    0#are no files|1#is one file|1&lt;are many files</pre>
  * 
  * <p>The pattern consists of a number or <em>range specifiers</em>
- * separated by vertical bars U+007C (<code>|</code>). There is no
+ * separated by vertical bars '|' (U+007C). There is no
  * vertical bar after the last range.  Each range specifier is of the
- * form <em>number separator string</em>.</p>
+ * form:</p>
+ *
+ * \htmlonly<blockquote><em>Number Separator String</em></blockquote>\endhtmlonly
  * 
  * <p><em>Number</em> is a floating point number that can be parsed by a
  * default <code>NumberFormat</code> for the US locale. It gives the
  * lower limit of this range. The lower limit is either inclusive or
- * exclusive, depending on the <em>separator</em>. (The upper limit is
- * given by the lower limit of the next range.)  The Unicode infinity
- * sign U+221E is recognized for positive infinity. It may be preceded by
- * '<code>-</code>' (U+002D) to indicate negative infinity.</p>
+ * exclusive, depending on the <em>separator</em>. The upper limit is
+ * given by the lower limit of the next range.  The Unicode infinity
+ * sign \htmlonly&#x221E \endhtmlonly (U+221E) is recognized for positive infinity. It may be preceded by
+ * '-' (U+002D) to indicate negative infinity.</p>
  * 
  * <p><em>String</em> is the format string for this range, with special
  * characters enclosed in single quotes (<code>'The #
@@ -77,15 +85,18 @@ class MessageFormat;
  * <p><em>Separator</em> is one of the following single characters:
  * 
  * <ul>
- *   <li>U+0023 (<code>#</code>) indicates that the lower limit given by
- *     <em>number</em> is inclusive.  That is, the limit value belongs to
- *     this range.  Another way of saying this is that the corresponding
- *     closure is <code>FALSE</code>.  The Unicode less than or equals
- *     sign U+2264 may be used in place of <code>#</code>.</li>
- *   <li>U+003C (<code>&lt;</code>) indicates that the lower limit given
- *     by <em>number</em> is exclusive.  This means that the limit
- *     belongs to the prior range.</li> Another way of saying this is
- *     that the corresponding closure is <code>TRUE</code>.
+ *   <li>\htmlonly'&#x2264;' \endhtmlonly (U+2264) or '#' (U+0023)
+ *   indicates that the lower limit given by <em>Number</em> is
+ *   inclusive.  (The two characters are equivalent to ChoiceFormat.)
+ *   This means that the limit value <em>Number</em> belongs to this
+ *   range.  Another way of saying this is that the corresponding
+ *   closure is <code>FALSE</code>.</li>
+ *
+ *   <li>'<' (U+003C) indicates that the lower limit given by
+ *   <em>Number</em> is exclusive.  This means that the value
+ *   <em>Number</em> belongs to the prior range.</li> Another way of
+ *   saying this is that the corresponding closure is
+ *   <code>TRUE</code>.
  * </ul>
  * 
  * <p>See below for more information about closures.</p>
@@ -175,70 +186,66 @@ class MessageFormat;
  * 
  * <p>Here is a simple example that shows formatting and parsing: </p>
  * 
- * <pre>
  * \code
- *   #include &lt;unicode/choicfmt.h&gt;
- *   #include &lt;unicode/unistr.h&gt;
- *   #include &lt;iostream.h&gt;
+ *   #include <unicode/choicfmt.h>
+ *   #include <unicode/unistr.h>
+ *   #include <iostream.h>
  *   
  *   int main(int argc, char *argv[]) {
  *       double limits[] = {1,2,3,4,5,6,7};
  *       UnicodeString monthNames[] = {
- *           &quot;Sun&quot;,&quot;Mon&quot;,&quot;Tue&quot;,&quot;Wed&quot;,&quot;Thu&quot;,&quot;Fri&quot;,&quot;Sat&quot;};
+ *           "Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
  *       ChoiceFormat fmt(limits, monthNames, 7);
  *       UnicodeString str;
  *       char buf[256];
- *       for (double x = 1.0; x &lt;= 8.0; x += 1.0) {
+ *       for (double x = 1.0; x <= 8.0; x += 1.0) {
  *           fmt.format(x, str);
- *           str.extract(0, str.length(), buf, 256, &quot;&quot;);
+ *           str.extract(0, str.length(), buf, 256, "");
  *           str.truncate(0);
- *           cout &lt;&lt; x &lt;&lt; &quot; -&gt; &quot;
- *                &lt;&lt; buf &lt;&lt; endl;
+ *           cout << x << " -> "
+ *                << buf << endl;
  *       }
- *       cout &lt;&lt; endl;
+ *       cout << endl;
  *       return 0;
  *   }
  * \endcode
- * </pre>
  * 
  * <p>Here is a more complex example using a <code>ChoiceFormat</code>
  * constructed from a pattern together with a
  * <code>MessageFormat</code>.</p>
  * 
- * <pre>
  * \code
- *   #include &lt;unicode/choicfmt.h&gt;
- *   #include &lt;unicode/msgfmt.h&gt;
- *   #include &lt;unicode/unistr.h&gt;
- *   #include &lt;iostream.h&gt;
+ *   #include <unicode/choicfmt.h>
+ *   #include <unicode/msgfmt.h>
+ *   #include <unicode/unistr.h>
+ *   #include <iostream.h>
  * 
  *   int main(int argc, char *argv[]) {
  *       UErrorCode status = U_ZERO_ERROR;
  *       double filelimits[] = {0,1,2};
  *       UnicodeString filepart[] =
- *           {&quot;are no files&quot;,&quot;is one file&quot;,&quot;are {0} files&quot;};
+ *           {"are no files","is one file","are {0} files"};
  *       ChoiceFormat* fileform = new ChoiceFormat(filelimits, filepart, 3 );
  *       Format* testFormats[] =
  *           {fileform, NULL, NumberFormat::createInstance(status)};
- *       MessageFormat pattform(&quot;There {0} on {1}&quot;, status );
+ *       MessageFormat pattform("There {0} on {1}", status );
  *       pattform.adoptFormats(testFormats, 3);
- *       Formattable testArgs[] = {0L, &quot;Disk A&quot;};
+ *       Formattable testArgs[] = {0L, "Disk A"};
  *       FieldPosition fp(0);
  *       UnicodeString str;
  *       char buf[256];
- *       for (int32_t i = 0; i &lt; 4; ++i) {
+ *       for (int32_t i = 0; i < 4; ++i) {
  *           Formattable fInt(i);
  *           testArgs[0] = fInt;
  *           pattform.format(testArgs, 2, str, fp, status );
- *           str.extract(0, str.length(), buf, &quot;&quot;);
+ *           str.extract(0, str.length(), buf, "");
  *           str.truncate(0);
- *           cout &lt;&lt; &quot;Output for i=&quot; &lt;&lt; i &lt;&lt; &quot; : &quot; &lt;&lt; buf &lt;&lt; endl;
+ *           cout << "Output for i=" << i << " : " << buf << endl;
  *       }
- *       cout &lt;&lt; endl;
+ *       cout << endl;
  *       return 0;
  *   }
  * \endcode
- * </pre>
  */
 class U_I18N_API ChoiceFormat: public NumberFormat {
 public:
@@ -249,9 +256,9 @@ public:
      * @param pattern   Pattern used to construct object.
      * @param status    Output param to receive success code.  If the
      *                  pattern cannot be parsed, set to failure code.
-     * @stable
+     * @stable ICU 2.0
      */
-    ChoiceFormat(const UnicodeString& newPattern,
+    ChoiceFormat(const UnicodeString& pattern,
                  UErrorCode& status);
 
 
@@ -262,7 +269,7 @@ public:
      * @param limits    Array of limit values.
      * @param formats   Array of formats.
      * @param count     Size of 'limits' and 'formats' arrays.
-     * @stable
+     * @stable ICU 2.0
      */
     
     ChoiceFormat(const double* limits,
@@ -284,6 +291,7 @@ public:
      * then the limit belongs to the range below it.
      * @param formats Array of formats
      * @param count Size of 'limits', 'closures', and 'formats' arrays
+     * @draft ICU 2.4
      */
     ChoiceFormat(const double* limits,
                  const UBool* closures,
@@ -292,33 +300,42 @@ public:
 
     /**
      * Copy constructor.
-     * @stable
+     *
+     * @param that   ChoiceFormat object to be copied from
+     * @stable ICU 2.0
      */
-    ChoiceFormat(const ChoiceFormat&);
+    ChoiceFormat(const ChoiceFormat& that);
 
     /**
      * Assignment operator.
-     * @stable
+     *
+     * @param that   ChoiceFormat object to be copied
+     * @stable ICU 2.0
      */
-    const ChoiceFormat& operator=(const ChoiceFormat&);
+    const ChoiceFormat& operator=(const ChoiceFormat& that);
 
     /**
      * Destructor.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual ~ChoiceFormat();
 
     /**
      * Clone this Format object polymorphically. The caller owns the
      * result and should delete it when done.
-     * @stable
+     *
+     * @return a copy of this object
+     * @stable ICU 2.0
      */
     virtual Format* clone(void) const;
 
     /**
      * Return true if the given Format objects are semantically equal.
      * Objects of different subclasses are considered unequal.
-     * @stable
+     *
+     * @param other    ChoiceFormat object to be compared 
+     * @return         true if other is the same as this. 
+     * @stable ICU 2.0
      */
     virtual UBool operator==(const Format& other) const;
 
@@ -328,7 +345,7 @@ public:
      * @param status    Output param set to success/failure code on
      *                  exit. If the pattern is invalid, this will be
      *                  set to a failure result.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void applyPattern(const UnicodeString& pattern,
                               UErrorCode& status);
@@ -341,17 +358,22 @@ public:
      * @param status     Output param set to success/failure code on
      *                   exit. If the pattern is invalid, this will be
      *                   set to a failure result.
-     * @draft
+     * @stable ICU 2.0
      */
     virtual void applyPattern(const UnicodeString& pattern,
                              UParseError& parseError,
                              UErrorCode& status);
     /**
      * Gets the pattern.
-     * @stable
+     * 
+     * @param pattern    Output param which will recieve the pattern
+     *                   Previous contents are deleted.
+     * @return    A reference to 'pattern'
+     * @stable ICU 2.0
      */
     virtual UnicodeString& toPattern(UnicodeString &pattern) const;
 
+#ifdef U_USE_CHOICE_FORMAT_DEPRECATES
     /**
      * Set the choices to be used in formatting.  The arrays are adopted and
      * should not be deleted by the caller.
@@ -363,7 +385,7 @@ public:
      *                          &lt;= X &lt; limit[i+1].
      * @param formatsToAdopt    The format strings you want to use for each limit.
      * @param count             The size of the above arrays.
-     * @deprecated Remove after 2003-mar-25. Use setChoices instead.
+     * @obsolete ICU 2.6. Use setChoices instead since this API will be removed in that release.
      */
     virtual void adoptChoices(double* limitsToAdopt,
                               UnicodeString* formatsToAdopt,
@@ -377,13 +399,14 @@ public:
      * @param closuresToAdopt Array of limit booleans to adopt
      * @param formatsToAdopt Array of format string to adopt
      * @param count The size of the above arrays
-     * @deprecated Remove after 2003-mar-25. Use setChoices instead.
+     * @obsolete ICU 2.6. Use setChoices instead since this API will be removed in that release.
      */
     virtual void adoptChoices(double* limitsToAdopt,
                               UBool* closuresToAdopt,
                               UnicodeString* formatsToAdopt,
                               int32_t count);
-    
+#endif
+
     /**
      * Set the choices to be used in formatting.
      *
@@ -394,7 +417,7 @@ public:
      *                          &lt;= X &lt; limit[i+1].
      * @param formatsToCopy     The format strings you want to use for each limit.
      * @param count             The size of the above arrays.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual void setChoices(const double* limitsToCopy,
                             const UnicodeString* formatsToCopy,
@@ -407,6 +430,7 @@ public:
      * @param closures Array of limit booleans
      * @param formats Array of format string
      * @param count The size of the above arrays
+     * @draft ICU 2.4
      */
     virtual void setChoices(const double* limits,
                             const UBool* closures,
@@ -415,88 +439,156 @@ public:
 
     /**
      * Get the limits passed in the constructor.
+     *
+     * @param count    The size of the limits arrays
      * @return the limits.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const double* getLimits(int32_t& count) const;
     
     /**
      * Get the limit booleans passed in the constructor.  The caller
      * must not delete the result.
+     *
+     * @param count   The size of the arrays
      * @return the closures
+     * @draft ICU 2.4
      */
     virtual const UBool* getClosures(int32_t& count) const;
 
     /**
      * Get the formats passed in the constructor.
+     *
+     * @param count   The size of the arrays
      * @return the formats.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual const UnicodeString* getFormats(int32_t& count) const;
 
-   /**
-    * Format a double or long number using this object's choices.
-    *
-    * @param number     The value to be formatted.
-    * @param toAppendTo The string to append the formatted string to.
-    *                   This is an output parameter.
-    * @param pos        On input: an alignment field, if desired.
-    *                   On output: the offsets of the alignment field.
-    * @return           A reference to 'toAppendTo'.
-    * @stable
-    */
+    /**
+     * Format a double or long number using this object's choices.
+     *
+     * @param number    The value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
+     */
     virtual UnicodeString& format(double number,
-                                  UnicodeString& toAppendTo,
+                                  UnicodeString& appendTo,
                                   FieldPosition& pos) const;
-   /**
-    * Format a int_32t number using this object's choices.
-    *
-    * @stable
-    */
+    /**
+     * Format a int_32t number using this object's choices.
+     *
+     * @param number    The value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
+     */
     virtual UnicodeString& format(int32_t number,
-                                  UnicodeString& toAppendTo,
+                                  UnicodeString& appendTo,
                                   FieldPosition& pos) const;
-   /**
-    * Format an array of objects using this object's choices.
-    *
-    * @stable
-    */
+
+    /**
+     * Format an int64_t number using this object's choices.
+     *
+     * @param number    The value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @return          Reference to 'appendTo' parameter.
+     * @draft ICU 2.8
+     */
+    virtual UnicodeString& format(int64_t number,
+                                  UnicodeString& appendTo,
+                                  FieldPosition& pos) const;
+
+    /**
+     * Format an array of objects using this object's choices.
+     *
+     * @param objs      The array of objects to be formatted.
+     * @param cnt       The size of objs.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @param success   Output param set to success/failure code on
+     *                  exit. 
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
+     */
     virtual UnicodeString& format(const Formattable* objs,
                                   int32_t cnt,
-                                  UnicodeString& toAppendTo,
+                                  UnicodeString& appendTo,
                                   FieldPosition& pos,
                                   UErrorCode& success) const;
-   /**
-    * Format an object using this object's choices.
-    *
-    * @stable
-    */
+    /**
+     * Format an object using this object's choices.
+     *
+     *
+     * @param obj       The object to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @param status    Output param set to success/failure code on
+     *                  exit. 
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
+     */
     virtual UnicodeString& format(const Formattable& obj,
-                                  UnicodeString& toAppendTo,
+                                  UnicodeString& appendTo,
                                   FieldPosition& pos, 
                                   UErrorCode& status) const;
 
     /**
      * Redeclared NumberFormat method.
-     * @stable
+     *
+     * @param obj       The object to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @param status    Output param set to success/failure code on
+     *                  exit. 
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(const Formattable& obj,
-                          UnicodeString& result,
+                          UnicodeString& appendTo,
                           UErrorCode& status) const;
 
     /**
      * Redeclared NumberFormat method.
-     * @stable
+     * Format a double number. These methods call the NumberFormat
+     * pure virtual format() methods with the default FieldPosition.
+     *
+     * @param number    The value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(  double number,
-                            UnicodeString& output) const;
+                            UnicodeString& appendTo) const;
 
     /**
      * Redeclared NumberFormat method.
-     * @stable
+     * Format a long number. These methods call the NumberFormat
+     * pure virtual format() methods with the default FieldPosition.
+     *
+     * @param number    The value to be formatted.
+     * @param appendTo  Output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 2.0
      */
     UnicodeString& format(  int32_t number,
-                            UnicodeString& output) const;
+                            UnicodeString& appendTo) const;
 
    /**
     * Return a long if possible (e.g. within range LONG_MAX,
@@ -513,14 +605,29 @@ public:
     * @param parsePosition  The position to start parsing at on input.
     *                       On output, moved to after the last successfully
     *                       parse character. On parse failure, does not change.
-    * @return               A Formattable object of numeric type.  The caller
-    *                       owns this an must delete it.  NULL on failure.
     * @see                  NumberFormat::isParseIntegerOnly
-    * @stable
+    * @stable ICU 2.0
     */
     virtual void parse(const UnicodeString& text,
                        Formattable& result,
                        ParsePosition& parsePosition) const;
+    
+    /**
+    * Return a long if possible (e.g. within range LONG_MAX,
+    * LONG_MAX], and with no decimals), otherwise a double.  If
+    * IntegerOnly is set, will stop at a decimal point (or equivalent;
+    * e.g. for rational numbers "1 2/3", will stop after the 1).
+    * <P>
+    * If no object can be parsed, parsePosition is unchanged, and NULL is
+    * returned.
+    *
+    * @param text           The text to be parsed.
+    * @param result         Formattable to be set to the parse result.
+    *                       If parse fails, return contents are undefined.
+    * @param status         Output param with the formatted string.
+    * @see                  NumberFormat::isParseIntegerOnly
+    * @stable ICU 2.0
+    */
     virtual void parse(const UnicodeString& text,
                        Formattable& result,
                        UErrorCode& status) const;
@@ -536,7 +643,7 @@ public:
      * @return          The class ID for this object. All objects of a
      *                  given class have the same class ID.  Objects of
      *                  other classes have different class IDs.
-     * @stable
+     * @stable ICU 2.0
      */
     virtual UClassID getDynamicClassID(void) const;
 
@@ -549,36 +656,9 @@ public:
      * .           Derived::getStaticClassID()) ...
      * </pre>
      * @return          The class ID for all objects of this class.
-     * @stable
+     * @stable ICU 2.0
      */
-    static UClassID getStaticClassID(void) { return (UClassID)&fgClassID; }
-
-    /*
-     * Finds the least double greater than d (if positive == true),
-     * or the greatest double less than d (if positive == false).
-     * If NaN, returns same value.
-     * <P>
-     * Does not affect floating-point flags,
-     * @deprecated This will be removed after 2002-Jun-30. Use closures API instead.
-     */
-    static double nextDouble(double d, UBool positive);
-
-    /**
-     * Finds the least double greater than d.
-     * If NaN, returns same value.
-     * Used to make half-open intervals.
-     * @see ChoiceFormat::previousDouble
-     * @deprecated This will be removed after 2002-Jun-30. Use closures API instead.
-     */
-    static double nextDouble(double d );
-
-    /**
-     * Finds the greatest double less than d.
-     * If NaN, returns same value.
-     * @see ChoiceFormat::nextDouble
-     * @deprecated This will be removed after 2002-Jun-30. Use closures API instead.
-     */
-    static double previousDouble(double d );
+    static UClassID getStaticClassID(void);
 
 private:
     // static cache management (thread-safe)
@@ -589,7 +669,6 @@ private:
      * Converts a string to a double value using a default NumberFormat object
      * which is static (shared by all ChoiceFormat instances).
      * @param string the string to be converted with.
-     * @param status error code.
      * @return the converted double number.
      */
     static double stod(const UnicodeString& string);
@@ -597,27 +676,24 @@ private:
     /**
      * Converts a double value to a string using a default NumberFormat object
      * which is static (shared by all ChoiceFormat instances).
-     * @@param value the double number to be converted with.
-     * @@param string the result string.
-     * @@param status error code.
-     * @@return the converted string.
+     * @param value the double number to be converted with.
+     * @param string the result string.
+     * @return the converted string.
      */
     static UnicodeString& dtos(double value, UnicodeString& string);
 
-    //static UMTX fgMutex;
-    //static NumberFormat* fgNumberFormat;
-    static const char fgClassID;
+    ChoiceFormat(); // default constructor not implemented
 
-    static const UChar fgPositiveInfinity[];
-    static const UChar fgNegativeInfinity[];
     /**
      * Construct a new ChoiceFormat with the limits and the corresponding formats
      * based on the pattern.
      *
-     * @param pattern   Pattern used to construct object.
-     * @param status    Output param to receive success code.  If the
-     *                  pattern cannot be parsed, set to failure code.
-     * @stable
+     * @param newPattern   Pattern used to construct object.
+     * @param parseError   Struct to recieve information on position 
+     *                     of error if an error is encountered.
+     * @param status       Output param to receive success code.  If the
+     *                     pattern cannot be parsed, set to failure code.
+     * @stable ICU 2.0
      */
     ChoiceFormat(const UnicodeString& newPattern,
                  UParseError& parseError,
@@ -667,43 +743,29 @@ private:
     int32_t         fCount;
 };
  
-inline UClassID 
-ChoiceFormat::getDynamicClassID() const
-{ 
-    return ChoiceFormat::getStaticClassID(); 
-}
-
-inline double ChoiceFormat::nextDouble( double d )
-{
-    return ChoiceFormat::nextDouble( d, TRUE );
-}
-    
-inline double ChoiceFormat::previousDouble( double d )
-{
-    return ChoiceFormat::nextDouble( d, FALSE );
-}
-
 inline UnicodeString&
 ChoiceFormat::format(const Formattable& obj,
-                     UnicodeString& result,
+                     UnicodeString& appendTo,
                      UErrorCode& status) const {
     // Don't use Format:: - use immediate base class only,
     // in case immediate base modifies behavior later.
-    return NumberFormat::format(obj, result, status);
+    return NumberFormat::format(obj, appendTo, status);
 }
 
 inline UnicodeString&
 ChoiceFormat::format(double number,
-                     UnicodeString& output) const {
-    return NumberFormat::format(number, output);
+                     UnicodeString& appendTo) const {
+    return NumberFormat::format(number, appendTo);
 }
 
 inline UnicodeString&
 ChoiceFormat::format(int32_t number,
-                     UnicodeString& output) const {
-    return NumberFormat::format(number, output);
+                     UnicodeString& appendTo) const {
+    return NumberFormat::format(number, appendTo);
 }
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif // _CHOICFMT
 //eof
