@@ -1,6 +1,6 @@
 /*  
 ********************************************************************************
-*   Copyright (C) 1997-2011, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ********************************************************************************
 *
@@ -22,21 +22,14 @@
 
 #if !UCONFIG_NO_FORMATTING
 
-#include "unicode/calendar.h"
 #include "unicode/uobject.h"
 #include "unicode/locid.h"
-#include "unicode/ures.h"
-
-/**
- * \file 
- * \brief C++ API: Symbols for formatting dates.
- */
+#include "unicode/resbund.h"
 
 U_NAMESPACE_BEGIN
 
 /* forward declaration */
 class SimpleDateFormat;
-class Hashtable;
 
 /**
  * DateFormatSymbols is a public class for encapsulating localizable date-time
@@ -157,7 +150,7 @@ public:
      * subclassed.
      * @stable ICU 2.0
      */
-    virtual ~DateFormatSymbols();
+    ~DateFormatSymbols();
 
     /**
      * Return true if another object is semantically equal to this one.
@@ -178,7 +171,7 @@ public:
     UBool operator!=(const DateFormatSymbols& other) const { return !operator==(other); }
 
     /**
-     * Gets abbreviated era strings. For example: "AD" and "BC".
+     * Gets era strings. For example: "AD" and "BC".
      *
      * @param count    Filled in with length of the array.
      * @return         the era strings.
@@ -187,47 +180,13 @@ public:
     const UnicodeString* getEras(int32_t& count) const;
 
     /**
-     * Sets abbreviated era strings. For example: "AD" and "BC".
+     * Sets era strings. For example: "AD" and "BC".
      * @param eras  Array of era strings (DateFormatSymbols retains ownership.)
      * @param count Filled in with length of the array.
      * @stable ICU 2.0
      */
     void setEras(const UnicodeString* eras, int32_t count);
 
-    /**
-     * Gets era name strings. For example: "Anno Domini" and "Before Christ".
-     *
-     * @param count    Filled in with length of the array.
-     * @return         the era name strings.
-     * @stable ICU 3.4
-     */
-    const UnicodeString* getEraNames(int32_t& count) const;
-
-    /**
-     * Sets era name strings. For example: "Anno Domini" and "Before Christ".
-     * @param eraNames  Array of era name strings (DateFormatSymbols retains ownership.)
-     * @param count Filled in with length of the array.
-     * @stable ICU 3.6
-     */
-    void setEraNames(const UnicodeString* eraNames, int32_t count);
-
-    /**
-     * Gets narrow era strings. For example: "A" and "B".
-     *
-     * @param count    Filled in with length of the array.
-     * @return         the narrow era strings.
-     * @stable ICU 4.2
-     */
-    const UnicodeString* getNarrowEras(int32_t& count) const;
-
-    /**
-     * Sets narrow era strings. For example: "A" and "B".
-     * @param narrowEras  Array of narrow era strings (DateFormatSymbols retains ownership.)
-     * @param count Filled in with length of the array.
-     * @stable ICU 4.2
-     */
-    void setNarrowEras(const UnicodeString* narrowEras, int32_t count);
-
     /**
      * Gets month strings. For example: "January", "February", etc.
      * @param count Filled in with length of the array.
@@ -262,48 +221,6 @@ public:
      */
     void setShortMonths(const UnicodeString* shortMonths, int32_t count);
 
-    /**
-     * Selector for date formatting context
-     * @stable ICU 3.6
-     */
-    enum DtContextType {
-         FORMAT,
-         STANDALONE,
-         DT_CONTEXT_COUNT
-    };
-
-    /**
-     * Selector for date formatting width
-     * @stable ICU 3.6
-     */
-    enum DtWidthType {
-         ABBREVIATED,
-         WIDE,
-         NARROW,
-         DT_WIDTH_COUNT
-    };
-
-    /**
-     * Gets month strings by width and context. For example: "January", "February", etc.
-     * @param count Filled in with length of the array.
-     * @param context The formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
-     * @return the month strings. (DateFormatSymbols retains ownership.)
-     * @stable ICU 3.4
-     */
-    const UnicodeString* getMonths(int32_t& count, DtContextType context, DtWidthType width) const;
-
-    /**
-     * Sets month strings by width and context. For example: "January", "February", etc.
-     *
-     * @param months  The new month strings. (not adopted; caller retains ownership)
-     * @param count   Filled in with length of the array.
-     * @param context The formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
-     * @stable ICU 3.6
-     */
-    void setMonths(const UnicodeString* months, int32_t count, DtContextType context, DtWidthType width);
-
     /**
      * Gets weekday strings. For example: "Sunday", "Monday", etc.
      * @param count        Filled in with length of the array.
@@ -312,7 +229,6 @@ public:
      */
     const UnicodeString* getWeekdays(int32_t& count) const;
 
-
     /**
      * Sets weekday strings. For example: "Sunday", "Monday", etc.
      * @param weekdays     the new weekday strings. (not adopted; caller retains ownership)
@@ -337,49 +253,6 @@ public:
      */
     void setShortWeekdays(const UnicodeString* shortWeekdays, int32_t count);
 
-    /**
-     * Gets weekday strings by width and context. For example: "Sunday", "Monday", etc.
-     * @param count   Filled in with length of the array.
-     * @param context The formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW
-     * @return the month strings. (DateFormatSymbols retains ownership.)
-     * @stable ICU 3.4
-     */
-    const UnicodeString* getWeekdays(int32_t& count, DtContextType context, DtWidthType width) const;
-
-    /**
-     * Sets weekday strings by width and context. For example: "Sunday", "Monday", etc.
-     * @param weekdays  The new weekday strings. (not adopted; caller retains ownership)
-     * @param count     Filled in with length of the array.
-     * @param context   The formatting context, either FORMAT or STANDALONE
-     * @param width     The width of returned strings, either WIDE, ABBREVIATED, or NARROW
-     * @stable ICU 3.6
-     */
-    void setWeekdays(const UnicodeString* weekdays, int32_t count, DtContextType context, DtWidthType width);
-
-    /**
-     * Gets quarter strings by width and context. For example: "1st Quarter", "2nd Quarter", etc.
-     * @param count Filled in with length of the array.
-     * @param context The formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE or ABBREVIATED. There
-     *                are no NARROW quarters.
-     * @return the quarter strings. (DateFormatSymbols retains ownership.)
-     * @stable ICU 3.6
-     */
-    const UnicodeString* getQuarters(int32_t& count, DtContextType context, DtWidthType width) const;
-
-    /**
-     * Sets quarter strings by width and context. For example: "1st Quarter", "2nd Quarter", etc.
-     *
-     * @param quarters  The new quarter strings. (not adopted; caller retains ownership)
-     * @param count   Filled in with length of the array.
-     * @param context The formatting context, either FORMAT or STANDALONE
-     * @param width   The width of returned strings, either WIDE or ABBREVIATED. There
-     *                are no NARROW quarters.
-     * @stable ICU 3.6
-     */
-    void setQuarters(const UnicodeString* quarters, int32_t count, DtContextType context, DtWidthType width);
-
     /**
      * Gets AM/PM strings. For example: "AM" and "PM".
      * @param count        Filled in with length of the array.
@@ -401,16 +274,12 @@ public:
      * @param rowCount      Output param to receive number of rows.
      * @param columnCount   Output param to receive number of columns.
      * @return              The timezone strings as a 2-d array. (DateFormatSymbols retains ownership.)
-     * @deprecated ICU 3.6
+     * @stable ICU 2.0
      */
     const UnicodeString** getZoneStrings(int32_t& rowCount, int32_t& columnCount) const;
 
     /**
      * Sets timezone strings. These strings are stored in a 2-dimensional array.
-     * <p><b>Note:</b> SimpleDateFormat no longer use the zone strings stored in
-     * a DateFormatSymbols. Therefore, the time zone strings set by this mthod
-     * have no effects in an instance of SimpleDateFormat for formatting time
-     * zones.
      * @param strings       The timezone strings as a 2-d array to be copied. (not adopted; caller retains ownership)
      * @param rowCount      The number of rows (count of first index).
      * @param columnCount   The number of columns (count of second index).
@@ -423,14 +292,10 @@ public:
      * @return    the non-localized date-time pattern characters
      * @stable ICU 2.0
      */
-    static const UChar * U_EXPORT2 getPatternUChars(void);
+    static const UChar *getPatternUChars(void);
 
     /**
      * Gets localized date-time pattern characters. For example: 'u', 't', etc.
-     * <p>
-     * Note: ICU no longer provides localized date-time pattern characters for a locale
-     * starting ICU 3.8.  This method returns the non-localized date-time pattern
-     * characters unless user defined localized data is set by setLocalPatternChars.
      * @param result    Output param which will receive the localized date-time pattern characters.
      * @return          A reference to 'result'.
      * @stable ICU 2.0
@@ -448,7 +313,7 @@ public:
     /**
      * Returns the locale for this object. Two flavors are available:
      * valid and actual locale.
-     * @stable ICU 2.8
+     * @draft ICU 2.8 likely to change in ICU 3.0, based on feedback
      */
     Locale getLocale(ULocDataLocaleType type, UErrorCode& status) const;
 
@@ -464,7 +329,7 @@ public:
      *
      * @stable ICU 2.2
      */
-    static UClassID U_EXPORT2 getStaticClassID();
+    static UClassID getStaticClassID();
 
 private:
 
@@ -472,23 +337,11 @@ private:
     friend class DateFormatSymbolsSingleSetter; // see udat.cpp
 
     /**
-     * Abbreviated era strings. For example: "AD" and "BC".
+     * Era strings. For example: "AD" and "BC".
      */
     UnicodeString*  fEras;
     int32_t         fErasCount;
 
-    /**
-     * Era name strings. For example: "Anno Domini" and "Before Christ".
-     */
-    UnicodeString*  fEraNames;
-    int32_t         fEraNamesCount;
-
-    /**
-     * Narrow era strings. For example: "A" and "B".
-     */
-    UnicodeString*  fNarrowEras;
-    int32_t         fNarrowErasCount;
-
     /**
      * Month strings. For example: "January", "February", etc.
      */
@@ -501,30 +354,6 @@ private:
     UnicodeString*  fShortMonths;
     int32_t         fShortMonthsCount;
 
-    /**
-     * Narrow month strings. For example: "J", "F", etc.
-     */
-    UnicodeString*  fNarrowMonths;
-    int32_t         fNarrowMonthsCount;
-
-    /**
-     * Standalone Month strings. For example: "January", "February", etc.
-     */
-    UnicodeString*  fStandaloneMonths;
-    int32_t         fStandaloneMonthsCount;
-
-    /**
-     * Standalone Short month strings. For example: "Jan", "Feb", etc.
-     */
-    UnicodeString*  fStandaloneShortMonths;
-    int32_t         fStandaloneShortMonthsCount;
-
-    /**
-     * Standalone Narrow month strings. For example: "J", "F", etc.
-     */
-    UnicodeString*  fStandaloneNarrowMonths;
-    int32_t         fStandaloneNarrowMonthsCount;
-
     /**
      * Weekday strings. For example: "Sunday", "Monday", etc.
      */
@@ -537,30 +366,6 @@ private:
     UnicodeString*  fShortWeekdays;
     int32_t         fShortWeekdaysCount;
 
-    /**
-     * Narrow weekday strings. For example: "Sun", "Mon", etc.
-     */
-    UnicodeString*  fNarrowWeekdays;
-    int32_t         fNarrowWeekdaysCount;
-
-    /**
-     * Standalone Weekday strings. For example: "Sunday", "Monday", etc.
-     */
-    UnicodeString*  fStandaloneWeekdays;
-    int32_t         fStandaloneWeekdaysCount;
-
-    /**
-     * Standalone Short weekday strings. For example: "Sun", "Mon", etc.
-     */
-    UnicodeString*  fStandaloneShortWeekdays;
-    int32_t         fStandaloneShortWeekdaysCount;
-
-    /**
-     * Standalone Narrow weekday strings. For example: "Sun", "Mon", etc.
-     */
-    UnicodeString*  fStandaloneNarrowWeekdays;
-    int32_t         fStandaloneNarrowWeekdaysCount;
-
     /**
      * Ampm strings. For example: "AM" and "PM".
      */
@@ -568,97 +373,12 @@ private:
     int32_t         fAmPmsCount;
 
     /**
-     * Quarter strings. For example: "1st quarter", "2nd quarter", etc.
+     * The format data of all the timezones in this locale.
      */
-    UnicodeString  *fQuarters;
-    int32_t         fQuartersCount;
-
-    /**
-     * Short quarters. For example: "Q1", "Q2", etc.
-     */
-    UnicodeString  *fShortQuarters;
-    int32_t         fShortQuartersCount;
-
-    /**
-     * Standalone quarter strings. For example: "1st quarter", "2nd quarter", etc.
-     */
-    UnicodeString  *fStandaloneQuarters;
-    int32_t         fStandaloneQuartersCount;
-
-    /**
-     * Standalone short quarter strings. For example: "Q1", "Q2", etc.
-     */
-    UnicodeString  *fStandaloneShortQuarters;
-    int32_t         fStandaloneShortQuartersCount;
-
-    /**
-     * Localized names of time zones in this locale.  This is a
-     * two-dimensional array of strings of size n by m,
-     * where m is at least 5 and up to 7.  Each of the n rows is an
-     * entry containing the localized names for a single TimeZone.
-     *
-     * Each such row contains (with i ranging from 0..n-1):
-     * 
-     * zoneStrings[i][0] - time zone ID
-     *  example: America/Los_Angeles
-     * zoneStrings[i][1] - long name of zone in standard time
-     *  example: Pacific Standard Time
-     * zoneStrings[i][2] - short name of zone in standard time
-     *  example: PST
-     * zoneStrings[i][3] - long name of zone in daylight savings time
-     *  example: Pacific Daylight Time
-     * zoneStrings[i][4] - short name of zone in daylight savings time
-     *  example: PDT
-     * zoneStrings[i][5] - location name of zone
-     *  example: United States (Los Angeles)
-     * zoneStrings[i][6] - long generic name of zone
-     *  example: Pacific Time
-     * zoneStrings[i][7] - short generic of zone
-     *  example: PT
-     *
-     * The zone ID is not localized; it corresponds to the ID
-     * value associated with a system time zone object.  All other entries
-     * are localized names.  If a zone does not implement daylight savings
-     * time, the daylight savings time names are ignored.
-     *
-     * Note:CLDR 1.5 introduced metazone and its historical mappings.
-     * This simple two-dimensional array is no longer sufficient to represent
-     * localized names and its historic changes.  Since ICU 3.8.1, localized
-     * zone names extracted from ICU locale data is stored in a ZoneStringFormat
-     * instance.  But we still need to support the old way of customizing
-     * localized zone names, so we keep this field for the purpose.
-     */
-    UnicodeString   **fZoneStrings;         // Zone string array set by setZoneStrings
-    UnicodeString   **fLocaleZoneStrings;   // Zone string array created by the locale
+    UnicodeString** fZoneStrings;
     int32_t         fZoneStringsRowCount;
     int32_t         fZoneStringsColCount;
 
-    Locale                  fZSFLocale;         // Locale used for getting ZoneStringFormat
-
-    /**
-     * String used for localized GMT. For example, "GMT"
-     */
-    UnicodeString fGmtZero;
-
-    /**
-     * Pattern string used for localized time zone GMT format.  For example, "GMT{0}"
-     */
-    UnicodeString   fGmtFormat;
-
-    /**
-     * Pattern strings used for formatting zone offset in a localized time zone GMT string.
-     */
-    UnicodeString  *fGmtHourFormats;
-    int32_t         fGmtHourFormatsCount; 
-
-    enum GMTHourType {
-        GMT_NEGATIVE_HMS = 0,
-        GMT_NEGATIVE_HM,
-        GMT_POSITIVE_HMS,
-        GMT_POSITIVE_HM,
-        GMT_HOUR_COUNT
-    };
-
     /**
      * Localized date-time pattern characters. For example: use 'u' as 'y'.
      */
@@ -671,8 +391,43 @@ private:
     char validLocale[ULOC_FULLNAME_CAPACITY];
     char actualLocale[ULOC_FULLNAME_CAPACITY];
 
+
+    /* Sizes for the last resort string arrays */
+    typedef enum LastResortSize {
+        kMonthNum = 13,
+        kMonthLen = 3,
+
+        kDayNum = 8,
+        kDayLen = 2,
+
+        kAmPmNum = 2,
+        kAmPmLen = 3,
+
+        kEraNum = 2,
+        kEraLen = 3,
+
+        kZoneNum = 5,
+        kZoneLen = 4
+    } LastResortSize;
+
     DateFormatSymbols(); // default constructor not implemented
 
+    void initField(UnicodeString **field, int32_t& length, const ResourceBundle &data, UErrorCode &status);
+    void initField(UnicodeString **field, int32_t& length, const UChar *data, LastResortSize numStr, LastResortSize strLen, UErrorCode &status);
+
+    /**
+     * Load data for specified 'type', falling back to gregorian if needed
+     *
+     * @param rb ResourceBundle
+     * @param tag Resource key to data
+     * @param type Calendar type
+     * @param status Error Status
+     * @internal
+     */
+    static ResourceBundle
+      getData(ResourceBundle &rb, const char *tag, const char *type, UErrorCode& status);
+
+
     /**
      * Called by the constructors to actually load data from the resources
      *
@@ -682,7 +437,7 @@ private:
      *                             failure code upon return.
      * @param useLastResortData    determine if use last resort data
      */
-    void initializeData(const Locale& locale, const char *type, UErrorCode& status, UBool useLastResortData = FALSE);
+    void initializeData(const Locale&, const char *type, UErrorCode& status, UBool useLastResortData = FALSE);
 
     /**
      * Copy or alias an array in another object, as appropriate.
@@ -718,6 +473,21 @@ private:
      */
     void createZoneStrings(const UnicodeString *const * otherStrings);
 
+    /**
+     * Package private: used by SimpleDateFormat
+     * Gets the index for the given time zone ID to obtain the timezone
+     * strings for formatting. The time zone ID is just for programmatic
+     * lookup. NOT LOCALIZED!!!
+     * @param ID the given time zone ID.
+     * @return the index of the given time zone ID.  Returns -1 if
+     * the given time zone ID can't be located in the DateFormatSymbols object.
+     * @see java.util.SimpleTimeZone
+     */
+    int32_t getZoneIndex(const UnicodeString& ID) const;
+
+    // Internal method; see source for documentation
+    int32_t _getZoneIndex(const UnicodeString& id) const;
+
     /**
      * Delete all the storage owned by this object.
      */
@@ -729,11 +499,6 @@ private:
      */
     void copyData(const DateFormatSymbols& other);
 
-    /**
-     * Create zone strings array by locale if not yet available
-     */
-    void initZoneStringsArray(void);
-
     /**
      * Delete just the zone strings.
      */
