@@ -1,6 +1,6 @@
 /*
 ********************************************************************************
-*   Copyright (C) 1997-2006, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ********************************************************************************
 *
@@ -17,10 +17,6 @@
 
 #include "unicode/utypes.h"
 #include "unicode/unistr.h"
-/**
- * \file 
- * \brief C++ API: Formattable is a thin wrapper for primitive numeric types.
- */
 
 #if !UCONFIG_NO_FORMATTING
 
@@ -31,18 +27,18 @@ U_NAMESPACE_BEGIN
  * its subclasses for formatting.  Formattable is a thin wrapper
  * class which interconverts between the primitive numeric types
  * (double, long, etc.) as well as UDate and UnicodeString.
+ * <P>
+ * Note that this is fundamentally different from the Java behavior, since
+ * in this case the various formattable objects do not occupy a hierarchy,
+ * but are all wrapped within this one class.  Formattable encapsulates all
+ * the polymorphism in itself.
+ * <P>
+ * It would be easy to change this so that Formattable was an abstract base
+ * class of a genuine hierarchy, and that would clean up the code that
+ * currently must explicitly check for type, but that seems like overkill at
+ * this point.
  *
- * <p>Internally, a Formattable object is a union of primitive types.
- * As such, it can only store one flavor of data at a time.  To
- * determine what flavor of data it contains, use the getType method.
- *
- * <p>As of ICU 3.0, Formattable may also wrap a UObject pointer,
- * which it owns.  This allows an instance of any ICU class to be
- * encapsulated in a Formattable.  For legacy reasons and for
- * efficiency, primitive numeric types are still stored directly
- * within a Formattable.
- *
- * <p>The Formattable class is not suitable for subclassing.
+ * The Formattable class is not suitable for subclassing.
  */
 class U_I18N_API Formattable : public UObject {
 public:
@@ -53,13 +49,13 @@ public:
      * since UDate is currently typedefed to be either double or long.
      * If UDate is changed later to be a bonafide class
      * or struct, then we no longer need this enum.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     enum ISDATE { kIsDate };
 
     /**
      * Default constructor
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     Formattable(); // Type kLong, value 0
 
@@ -88,11 +84,10 @@ public:
     /**
      * Creates a Formattable object with an int64_t number
      * @param ll the int64_t number.
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */
     Formattable(int64_t ll);
 
-#if !UCONFIG_NO_CONVERSION
     /**
      * Creates a Formattable object with a char string pointer.
      * Assumes that the char string is null terminated.
@@ -100,7 +95,6 @@ public:
      * @stable ICU 2.0
      */
     Formattable(const char* strToCopy);
-#endif
 
     /**
      * Creates a Formattable object with a UnicodeString object to copy from.
@@ -124,13 +118,6 @@ public:
      */
     Formattable(const Formattable* arrayToCopy, int32_t count);
 
-    /**
-     * Creates a Formattable object that adopts the given UObject.
-     * @param objectToAdopt the UObject to set this object to
-     * @stable ICU 3.0
-     */
-    Formattable(UObject* objectToAdopt);
-
     /**
      * Copy constructor.
      * @stable ICU 2.0
@@ -176,65 +163,27 @@ public:
      * @return a clone of this object
      *
      * @see getDynamicClassID
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */
     Formattable *clone() const;
 
     /** 
-     * Selector for flavor of data type contained within a
-     * Formattable object.  Formattable is a union of several
-     * different types, and at any time contains exactly one type.
-     * @stable ICU 2.4
+     * The list of possible data types of this Formattable object.
+     * @draft ICU 2.4
      */
     enum Type {
-        /**
-         * Selector indicating a UDate value.  Use getDate to retrieve
-         * the value.
-         * @stable ICU 2.4
-         */
-        kDate,
-
-        /**
-         * Selector indicating a double value.  Use getDouble to
-         * retrieve the value.
-         * @stable ICU 2.4
-         */
-        kDouble,
-
-        /**
-         * Selector indicating a 32-bit integer value.  Use getLong to
-         * retrieve the value.
-         * @stable ICU 2.4
-         */
-        kLong,
-
-        /**
-         * Selector indicating a UnicodeString value.  Use getString
-         * to retrieve the value.
-         * @stable ICU 2.4
-         */
-        kString,
-
-        /**
-         * Selector indicating an array of Formattables.  Use getArray
-         * to retrieve the value.
-         * @stable ICU 2.4
-         */
-        kArray,
-
-        /**
-         * Selector indicating a 64-bit integer value.  Use getInt64
-         * to retrieve the value.
-         * @stable ICU 2.8
-         */
-        kInt64,
-
-        /**
-         * Selector indicating a UObject value.  Use getObject to
-         * retrieve the value.
-         * @stable ICU 3.0
-         */
-        kObject
+                /** @draft ICU 2.4 */
+        kDate,      // Date
+                /** @draft ICU 2.4 */
+        kDouble,    // double
+                /** @draft ICU 2.4 */
+        kLong,      // long
+                /** @draft ICU 2.4 */
+        kString,    // UnicodeString
+                /** @draft ICU 2.4 */
+        kArray,     // Formattable[]
+                /** @draft ICU 2.8 */
+                kInt64      // int64
    };
 
     /**
@@ -245,107 +194,81 @@ public:
     Type            getType(void) const;
     
     /**
-     * Returns TRUE if the data type of this Formattable object
-     * is kDouble, kLong, or kInt64.
-     * @return TRUE if this is a pure numeric object
-     * @stable ICU 3.0
-     */
-    UBool           isNumeric() const;
-    
-    /**
-     * Gets the double value of this object. If this object is not of type
-     * kDouble then the result is undefined.
+     * Gets the double value of this object.
      * @return    the double value of this object.
      * @stable ICU 2.0
      */ 
     double          getDouble(void) const { return fValue.fDouble; }
 
     /**
-     * Gets the double value of this object. If this object is of type
-     * long or int64 then a casting conversion is peformed, with
-     * possible loss of precision.  If the type is kObject and the
-     * object is a Measure, then the result of
-     * getNumber().getDouble(status) is returned.  If this object is
-     * neither a numeric type nor a Measure, then 0 is returned and
-     * the status is set to U_INVALID_FORMAT_ERROR.
+     * Gets the double value of this object.  This converts from long or
+     * int64 values as required (conversion from int64 can lose precision).
+     * If the type is not a numeric type, 0 is returned and the status
+     * is set to U_INVALID_FORMAT_ERROR.
      * @param status the error code
-     * @return the double value of this object.
-     * @stable ICU 3.0
+     * @return    the double value of this object.
+     * @draft ICU 2.8
      */ 
-    double          getDouble(UErrorCode& status) const;
+    double          getDouble(UErrorCode* status) const;
 
     /**
-     * Gets the long value of this object. If this object is not of type
-     * kLong then the result is undefined.
+     * Gets the long value of this object.
      * @return    the long value of this object.
      * @stable ICU 2.0
      */ 
     int32_t         getLong(void) const { return (int32_t)fValue.fInt64; }
 
     /**
-     * Gets the long value of this object. If the magnitude is too
-     * large to fit in a long, then the maximum or minimum long value,
-     * as appropriate, is returned and the status is set to
-     * U_INVALID_FORMAT_ERROR.  If this object is of type kInt64 and
-     * it fits within a long, then no precision is lost.  If it is of
-     * type kDouble, then a casting conversion is peformed, with
-     * truncation of any fractional part.  If the type is kObject and
-     * the object is a Measure, then the result of
-     * getNumber().getLong(status) is returned.  If this object is
-     * neither a numeric type nor a Measure, then 0 is returned and
+     * Gets the long value of this object.  This converts from double or
+     * int64 values as required.  If the magnitude is too large to fit in a long,
+     * the maximum or minimum long value, as appropriate, is returned and
      * the status is set to U_INVALID_FORMAT_ERROR.
+     * If the type is not a numeric type, 0 is returned and the status 
+     * is set to U_INVALID_FORMAT_ERROR.
      * @param status the error code
      * @return    the long value of this object.
-     * @stable ICU 3.0
+     * @stable ICU 2.0
      */ 
-    int32_t         getLong(UErrorCode& status) const;
+    int32_t         getLong(UErrorCode* status) const;
 
     /**
-     * Gets the int64 value of this object. If this object is not of type
-     * kInt64 then the result is undefined.
+     * Gets the int64 value of this object.
      * @return    the int64 value of this object.
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */ 
     int64_t         getInt64(void) const { return fValue.fInt64; }
 
     /**
-     * Gets the int64 value of this object. If this object is of type
-     * kDouble and the magnitude is too large to fit in an int64, then
-     * the maximum or minimum int64 value, as appropriate, is returned
-     * and the status is set to U_INVALID_FORMAT_ERROR.  If the
-     * magnitude fits in an int64, then a casting conversion is
-     * peformed, with truncation of any fractional part.  If the type
-     * is kObject and the object is a Measure, then the result of
-     * getNumber().getDouble(status) is returned.  If this object is
-     * neither a numeric type nor a Measure, then 0 is returned and
+     * Gets the int64 value of this object.  This converts from double or
+     * int64 values as required.  If the value value won't fit in an int64,
+     * the maximum or minimum in64 value, as appropriate, is returned and
      * the status is set to U_INVALID_FORMAT_ERROR.
+     * If the type is not a numeric type, 0 is returned and the status 
+     * is set to U_INVALID_FORMAT_ERROR.
      * @param status the error code
      * @return    the int64 value of this object.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */ 
-    int64_t         getInt64(UErrorCode& status) const;
+    int64_t         getInt64(UErrorCode* status) const;
 
     /**
-     * Gets the Date value of this object. If this object is not of type
-     * kDate then the result is undefined.
+     * Gets the Date value of this object.
      * @return    the Date value of this object.
      * @stable ICU 2.0
      */ 
     UDate           getDate() const { return fValue.fDate; }
 
     /**
-     * Gets the Date value of this object.  If the type is not a date,
-     * status is set to U_INVALID_FORMAT_ERROR and the return value is
-     * undefined.
-     * @param status the error code.
+     * Gets the Date value of this object.
+     * @param status the error code.  If the type is not a date, status
+     * is set to U_INVALID_FORMAT_ERROR and the return value is undefined.
      * @return    the Date value of this object.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */ 
-     UDate          getDate(UErrorCode& status) const;
+     UDate          getDate(UErrorCode* status) const;
 
     /**
-     * Gets the string value of this object. If this object is not of type
-     * kString then the result is undefined.
+     * Gets the string value of this object.
      * @param result    Output param to receive the Date value of this object.
      * @return          A reference to 'result'.
      * @stable ICU 2.0
@@ -354,56 +277,49 @@ public:
       { result=*fValue.fString; return result; }
 
     /**
-     * Gets the string value of this object. If the type is not a
-     * string, status is set to U_INVALID_FORMAT_ERROR and a bogus
-     * string is returned.
+     * Gets the string value of this object.
      * @param result    Output param to receive the Date value of this object.
-     * @param status    the error code. 
+     * @param status    the error code.  If the type is not a string, status
+     * is set to U_INVALID_FORMAT_ERROR and the result is set to bogus.
      * @return          A reference to 'result'.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */ 
-    UnicodeString&  getString(UnicodeString& result, UErrorCode& status) const;
+    UnicodeString&  getString(UnicodeString& result, UErrorCode* status) const;
 
     /**
-     * Gets a const reference to the string value of this object. If
-     * this object is not of type kString then the result is
-     * undefined.
+     * Gets a const reference to the string value of this object.
      * @return   a const reference to the string value of this object.
      * @stable ICU 2.0
      */
     inline const UnicodeString& getString(void) const;
 
     /**
-     * Gets a const reference to the string value of this object.  If
-     * the type is not a string, status is set to
-     * U_INVALID_FORMAT_ERROR and the result is a bogus string.
-     * @param status    the error code.
+     * Gets a const reference to the string value of this object.
+     * @param status    the error code.  If the type is not a string, status
+     * is set to U_INVALID_FORMAT_ERROR and the result is a bogus string.
      * @return   a const reference to the string value of this object.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */
-    const UnicodeString& getString(UErrorCode& status) const;
+    const UnicodeString& getString(UErrorCode* status) const;
 
     /**
-     * Gets a reference to the string value of this object. If this
-     * object is not of type kString then the result is undefined.
+     * Gets a reference to the string value of this object.
      * @return   a reference to the string value of this object.
      * @stable ICU 2.0
      */
     inline UnicodeString& getString(void);
 
     /**
-     * Gets a reference to the string value of this object. If the
-     * type is not a string, status is set to U_INVALID_FORMAT_ERROR
-     * and the result is a bogus string.
-     * @param status    the error code. 
+     * Gets a reference to the string value of this object.
+     * @param status    the error code.  If the type is not a string, status
+     * is set to U_INVALID_FORMAT_ERROR and the result is a bogus string.
      * @return   a reference to the string value of this object.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */
-    UnicodeString& getString(UErrorCode& status);
+    UnicodeString& getString(UErrorCode* status);
 
     /**
-     * Gets the array value and count of this object. If this object
-     * is not of type kArray then the result is undefined.
+     * Gets the array value and count of this object.
      * @param count    fill-in with the count of this object.
      * @return         the array value of this object.
      * @stable ICU 2.0
@@ -412,77 +328,60 @@ public:
       { count=fValue.fArrayAndCount.fCount; return fValue.fArrayAndCount.fArray; }
 
     /**
-     * Gets the array value and count of this object. If the type is
-     * not an array, status is set to U_INVALID_FORMAT_ERROR, count is
-     * set to 0, and the result is NULL.
+     * Gets the array value and count of this object.
      * @param count    fill-in with the count of this object.
-     * @param status the error code. 
+     * @param status the error code.  If the type is not an array, status
+     * is set to U_INVALID_FORMAT_ERROR, count is set to 0, and the result is NULL.
      * @return         the array value of this object.
-     * @stable ICU 3.0
+     * @draft ICU 2.8
      */ 
-    const Formattable* getArray(int32_t& count, UErrorCode& status) const;
+    const Formattable* getArray(int32_t& count, UErrorCode* status) const;
 
     /**
-     * Accesses the specified element in the array value of this
-     * Formattable object. If this object is not of type kArray then
-     * the result is undefined.
+     * Accesses the specified element in the array value of this Formattable object.
      * @param index the specified index.
      * @return the accessed element in the array.
      * @stable ICU 2.0
      */
     Formattable&    operator[](int32_t index) { return fValue.fArrayAndCount.fArray[index]; }
-       
-    /**
-     * Returns a pointer to the UObject contained within this
-     * formattable, or NULL if this object does not contain a UObject.
-     * @return a UObject pointer, or NULL
-     * @stable ICU 3.0
-     */
-    const UObject*  getObject() const;
 
     /**
-     * Sets the double value of this object and changes the type to
-     * kDouble.
+     * Sets the double value of this object.
      * @param d    the new double value to be set.
      * @stable ICU 2.0
      */ 
     void            setDouble(double d);
 
     /**
-     * Sets the long value of this object and changes the type to
-     * kLong.
+     * Sets the long value of this object.
      * @param l    the new long value to be set.
      * @stable ICU 2.0
      */ 
     void            setLong(int32_t l);
 
     /**
-     * Sets the int64 value of this object and changes the type to
-     * kInt64.
+     * Sets the int64 value of this object.
      * @param ll    the new int64 value to be set.
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */ 
     void            setInt64(int64_t ll);
 
     /**
-     * Sets the Date value of this object and changes the type to
-     * kDate.
+     * Sets the Date value of this object.
      * @param d    the new Date value to be set.
      * @stable ICU 2.0
      */ 
     void            setDate(UDate d);
 
     /**
-     * Sets the string value of this object and changes the type to
-     * kString.
+     * Sets the string value of this object.
      * @param stringToCopy    the new string value to be set.
      * @stable ICU 2.0
      */ 
     void            setString(const UnicodeString& stringToCopy);
 
     /**
-     * Sets the array value and count of this object and changes the
-     * type to kArray.
+     * Sets the array value and count of this object.
      * @param array    the array value.
      * @param count    the number of array elements to be copied.
      * @stable ICU 2.0
@@ -490,29 +389,18 @@ public:
     void            setArray(const Formattable* array, int32_t count);
 
     /**
-     * Sets and adopts the string value and count of this object and
-     * changes the type to kArray.
+     * Sets and adopts the string value and count of this object.
      * @param stringToAdopt    the new string value to be adopted.
      * @stable ICU 2.0
      */ 
     void            adoptString(UnicodeString* stringToAdopt);
 
     /**
-     * Sets and adopts the array value and count of this object and
-     * changes the type to kArray.
+     * Sets and adopts the array value and count of this object.
      * @stable ICU 2.0
      */ 
     void            adoptArray(Formattable* array, int32_t count);
-       
-    /**
-     * Sets and adopts the UObject value of this object and changes
-     * the type to kObject.  After this call, the caller must not
-     * delete the given object.
-     * @param objectToAdopt the UObject value to be adopted
-     * @stable ICU 3.0
-     */
-    void            adoptObject(UObject* objectToAdopt);
-
+        
     /**
      * ICU "poor man's RTTI", returns a UClassID for the actual class.
      *
@@ -525,15 +413,7 @@ public:
      *
      * @stable ICU 2.2
      */
-    static UClassID U_EXPORT2 getStaticClassID();
-
-    /**
-     * Deprecated variant of getLong(UErrorCode&).
-     * @param status the error code
-     * @return the long value of this object.
-     * @deprecated ICU 3.0 use getLong(UErrorCode&) instead
-     */ 
-    inline int32_t getLong(UErrorCode* status) const;
+    static UClassID getStaticClassID();
 
 private:
     /**
@@ -542,30 +422,48 @@ private:
      */
     void            dispose(void);
 
+    /**
+     * Creates a new Formattable array and copies the values from the specified
+     * original.
+     * @param array the original array
+     * @param count the original array count
+     * @return the new Formattable array.
+     */
+    static Formattable* createArrayCopy(const Formattable* array, int32_t count);
+
     UnicodeString* getBogus() const;
 
+    // Note: For now, we do not handle unsigned long and unsigned
+    // double types.  Smaller unsigned types, such as unsigned
+    // short, can fit within a long.
     union {
-        UObject*        fObject;
         UnicodeString*  fString;
         double          fDouble;
         int64_t         fInt64;
         UDate           fDate;
-        struct {
-          Formattable*  fArray;
-          int32_t       fCount;
+        struct
+        {
+            Formattable*  fArray;
+            int32_t       fCount;
         }               fArrayAndCount;
-    } fValue;
+    }                   fValue;
 
     Type                fType;
     UnicodeString       fBogus; // Bogus string when it's needed.
 };
 
-inline UDate Formattable::getDate(UErrorCode& status) const {
-    if (fType != kDate) {
-        if (U_SUCCESS(status)) {
-            status = U_INVALID_FORMAT_ERROR;
-        }
-        return 0;
+inline Formattable*
+Formattable::createArrayCopy(const Formattable* array, int32_t count)
+{
+    Formattable *result = new Formattable[count];
+    for (int32_t i=0; i<count; ++i) result[i] = array[i]; // Don't memcpy!
+    return result;
+}
+
+inline UDate Formattable::getDate(UErrorCode* status) const {
+    if (status && U_SUCCESS(*status) && fType != kDate) {
+      *status = U_INVALID_FORMAT_ERROR;
+      return 0;
     }
     return fValue.fDate;
 }
@@ -578,14 +476,10 @@ inline UnicodeString& Formattable::getString(void) {
     return *fValue.fString;
 }
 
-inline int32_t Formattable::getLong(UErrorCode* status) const {
-    return getLong(*status);
-}
-
 U_NAMESPACE_END
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 #endif //_FMTABLE
 //eof
-
+     
