@@ -1,47 +1,36 @@
 /*
- ********************************************************************************
- * Copyright (C) 1997-2007, International Business Machines                     *
- * Corporation and others. All Rights Reserved.                                 *
- ********************************************************************************
- *
- * File SIMPLETZ.H
- *
- * Modification History:
- *
- *   Date        Name        Description
- *   04/21/97    aliu        Overhauled header.
- *   08/10/98    stephen     JDK 1.2 sync
- *                           Added setStartRule() / setEndRule() overloads
- *                           Added hasSameRules()
- *   09/02/98    stephen     Added getOffset(monthLen)
- *                           Changed getOffset() to take UErrorCode
- *   07/09/99    stephen     Removed millisPerHour (unused, for HP compiler)
- *   12/02/99    aliu        Added TimeMode and constructor and setStart/EndRule
- *                           methods that take TimeMode. Added to docs.
- ********************************************************************************
- */
+* Copyright (C) {1997-2003}, International Business Machines Corporation and others. All Rights Reserved.
+*                                                                              *
+********************************************************************************
+*
+* File SIMPLETZ.H
+*
+* Modification History:
+*
+*   Date        Name        Description
+*   04/21/97    aliu        Overhauled header.
+*    08/10/98    stephen        JDK 1.2 sync
+*                            Added setStartRule() / setEndRule() overloads
+*                            Added hasSameRules()
+*    09/02/98    stephen        Added getOffset(monthLen)
+*                            Changed getOffset() to take UErrorCode
+*    07/09/99    stephen     Removed millisPerHour (unused, for HP compiler)
+*   12/02/99    aliu        Added TimeMode and constructor and setStart/EndRule
+*                           methods that take TimeMode. Added to docs.
+********************************************************************************
+*/
 
 #ifndef SIMPLETZ_H
 #define SIMPLETZ_H
 
 #include "unicode/utypes.h"
 
-/**
- * \file 
- * \brief C++ API: SimpleTimeZone is a concrete subclass of TimeZone.
- */
- 
 #if !UCONFIG_NO_FORMATTING
 
-#include "unicode/basictz.h"
+#include "unicode/timezone.h"
 
 U_NAMESPACE_BEGIN
 
-// forward declaration
-class InitialTimeZoneRule;
-class TimeZoneTransition;
-class AnnualTimeZoneRule;
-
 /**
  * <code>SimpleTimeZone</code> is a concrete subclass of <code>TimeZone</code>
  * that represents a time zone for use with a Gregorian calendar. This
@@ -58,7 +47,7 @@ class AnnualTimeZoneRule;
  * @see      TimeZone
  * @author   D. Goldsmith, Mark Davis, Chen-Lieh Huang, Alan Liu
  */
-class U_I18N_API SimpleTimeZone: public BasicTimeZone {
+class U_I18N_API SimpleTimeZone: public TimeZone {
 public:
 
     /**
@@ -273,30 +262,28 @@ public:
      * setStartRule(TimeFields.APRIL, 1, TimeFields.SUNDAY, 2*60*60*1000);
      * The dayOfWeekInMonth and dayOfWeek parameters together specify how to calculate
      * the exact starting date.  Their exact meaning depend on their respective signs,
-     * allowing various types of rules to be constructed, as follows:
-     * <ul>
+     * allowing various types of rules to be constructed, as follows:<ul>
      *   <li>If both dayOfWeekInMonth and dayOfWeek are positive, they specify the
      *       day of week in the month (e.g., (2, WEDNESDAY) is the second Wednesday
-     *       of the month).</li>
+     *       of the month).
      *   <li>If dayOfWeek is positive and dayOfWeekInMonth is negative, they specify
      *       the day of week in the month counting backward from the end of the month.
-     *       (e.g., (-1, MONDAY) is the last Monday in the month)</li>
+     *       (e.g., (-1, MONDAY) is the last Monday in the month)
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is positive, dayOfWeekInMonth
      *       specifies the day of the month, regardless of what day of the week it is.
-     *       (e.g., (10, 0) is the tenth day of the month)</li>
+     *       (e.g., (10, 0) is the tenth day of the month)
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is negative, dayOfWeekInMonth
      *       specifies the day of the month counting backward from the end of the
      *       month, regardless of what day of the week it is (e.g., (-2, 0) is the
-     *       next-to-last day of the month).</li>
+     *       next-to-last day of the month).
      *   <li>If dayOfWeek is negative and dayOfWeekInMonth is positive, they specify the
      *       first specified day of the week on or after the specfied day of the month.
      *       (e.g., (15, -SUNDAY) is the first Sunday after the 15th of the month
-     *       [or the 15th itself if the 15th is a Sunday].)</li>
+     *       [or the 15th itself if the 15th is a Sunday].)
      *   <li>If dayOfWeek and DayOfWeekInMonth are both negative, they specify the
      *       last specified day of the week on or before the specified day of the month.
      *       (e.g., (-20, -TUESDAY) is the last Tuesday before the 20th of the month
-     *       [or the 20th itself if the 20th is a Tuesday].)</li>
-     * </ul>
+     *       [or the 20th itself if the 20th is a Tuesday].)</ul>
      * @param month the daylight savings starting month. Month is 0-based.
      * eg, 0 for January.
      * @param dayOfWeekInMonth the daylight savings starting
@@ -317,30 +304,28 @@ public:
      * setStartRule(TimeFields.APRIL, 1, TimeFields.SUNDAY, 2*60*60*1000);
      * The dayOfWeekInMonth and dayOfWeek parameters together specify how to calculate
      * the exact starting date.  Their exact meaning depend on their respective signs,
-     * allowing various types of rules to be constructed, as follows:
-     * <ul>
+     * allowing various types of rules to be constructed, as follows:<ul>
      *   <li>If both dayOfWeekInMonth and dayOfWeek are positive, they specify the
      *       day of week in the month (e.g., (2, WEDNESDAY) is the second Wednesday
-     *       of the month).</li>
+     *       of the month).
      *   <li>If dayOfWeek is positive and dayOfWeekInMonth is negative, they specify
      *       the day of week in the month counting backward from the end of the month.
-     *       (e.g., (-1, MONDAY) is the last Monday in the month)</li>
+     *       (e.g., (-1, MONDAY) is the last Monday in the month)
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is positive, dayOfWeekInMonth
      *       specifies the day of the month, regardless of what day of the week it is.
-     *       (e.g., (10, 0) is the tenth day of the month)</li>
+     *       (e.g., (10, 0) is the tenth day of the month)
      *   <li>If dayOfWeek is zero and dayOfWeekInMonth is negative, dayOfWeekInMonth
      *       specifies the day of the month counting backward from the end of the
      *       month, regardless of what day of the week it is (e.g., (-2, 0) is the
-     *       next-to-last day of the month).</li>
+     *       next-to-last day of the month).
      *   <li>If dayOfWeek is negative and dayOfWeekInMonth is positive, they specify the
      *       first specified day of the week on or after the specfied day of the month.
      *       (e.g., (15, -SUNDAY) is the first Sunday after the 15th of the month
-     *       [or the 15th itself if the 15th is a Sunday].)</li>
+     *       [or the 15th itself if the 15th is a Sunday].)
      *   <li>If dayOfWeek and DayOfWeekInMonth are both negative, they specify the
      *       last specified day of the week on or before the specified day of the month.
      *       (e.g., (-20, -TUESDAY) is the last Tuesday before the 20th of the month
-     *       [or the 20th itself if the 20th is a Tuesday].)</li>
-     * </ul>
+     *       [or the 20th itself if the 20th is a Tuesday].)</ul>
      * @param month the daylight savings starting month. Month is 0-based.
      * eg, 0 for January.
      * @param dayOfWeekInMonth the daylight savings starting
@@ -611,7 +596,7 @@ public:
     /**
      * Redeclared TimeZone method.  This implementation simply calls
      * the base class method, which otherwise would be hidden.
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */
     virtual void getOffset(UDate date, UBool local, int32_t& rawOffset,
                            int32_t& dstOffset, UErrorCode& ec) const;
@@ -651,7 +636,7 @@ public:
      * are in effect. A positive number, typically one hour (3600000).
      * @stable ICU 2.0
      */
-    virtual int32_t getDSTSavings(void) const;
+    int32_t getDSTSavings(void) const;
 
     /**
      * Queries if this TimeZone uses Daylight Savings Time.
@@ -694,56 +679,6 @@ public:
      */
     virtual TimeZone* clone(void) const;
 
-    /**
-     * Gets the first time zone transition after the base time.
-     * @param base      The base time.
-     * @param inclusive Whether the base time is inclusive or not.
-     * @param result    Receives the first transition after the base time.
-     * @return  TRUE if the transition is found.
-     * @draft ICU 3.8
-     */
-    virtual UBool getNextTransition(UDate base, UBool inclusive, TimeZoneTransition& result) /*const*/;
-
-    /**
-     * Gets the most recent time zone transition before the base time.
-     * @param base      The base time.
-     * @param inclusive Whether the base time is inclusive or not.
-     * @param result    Receives the most recent transition before the base time.
-     * @return  TRUE if the transition is found.
-     * @draft ICU 3.8
-     */
-    virtual UBool getPreviousTransition(UDate base, UBool inclusive, TimeZoneTransition& result) /*const*/;
-
-    /**
-     * Returns the number of <code>TimeZoneRule</code>s which represents time transitions,
-     * for this time zone, that is, all <code>TimeZoneRule</code>s for this time zone except
-     * <code>InitialTimeZoneRule</code>.  The return value range is 0 or any positive value.
-     * @param status    Receives error status code.
-     * @return The number of <code>TimeZoneRule</code>s representing time transitions.
-     * @draft ICU 3.8
-     */
-    virtual int32_t countTransitionRules(UErrorCode& status) /*const*/;
-
-    /**
-     * Gets the <code>InitialTimeZoneRule</code> and the set of <code>TimeZoneRule</code>
-     * which represent time transitions for this time zone.  On successful return,
-     * the argument initial points to non-NULL <code>InitialTimeZoneRule</code> and
-     * the array trsrules is filled with 0 or multiple <code>TimeZoneRule</code>
-     * instances up to the size specified by trscount.  The results are referencing the
-     * rule instance held by this time zone instance.  Therefore, after this time zone
-     * is destructed, they are no longer available.
-     * @param initial       Receives the initial timezone rule
-     * @param trsrules      Receives the timezone transition rules
-     * @param trscount      On input, specify the size of the array 'transitions' receiving
-     *                      the timezone transition rules.  On output, actual number of
-     *                      rules filled in the array will be set.
-     * @param status        Receives error status code.
-     * @draft ICU 3.8
-     */
-    virtual void getTimeZoneRules(const InitialTimeZoneRule*& initial,
-        const TimeZoneRule* trsrules[], int32_t& trscount, UErrorCode& status) /*const*/;
-
-
 public:
 
     /**
@@ -769,7 +704,7 @@ public:
      * @return   The class ID for all objects of this class.
      * @stable ICU 2.0
      */
-    static UClassID U_EXPORT2 getStaticClassID(void);
+    static UClassID getStaticClassID(void);
 
 private:
     /**
@@ -861,16 +796,6 @@ private:
      * Typically one hour; sometimes 30 minutes.
      */
     int32_t dstSavings;
-
-    /* Private for BasicTimeZone implementation */
-    void initTransitionRules(UErrorCode& status);
-    void clearTransitionRules(void);
-    void deleteTransitionRules(void);
-    UBool   transitionRulesInitialized;
-    InitialTimeZoneRule*    initialRule;
-    TimeZoneTransition*     firstTransition;
-    AnnualTimeZoneRule*     stdRule;
-    AnnualTimeZoneRule*     dstRule;
 };
 
 inline void SimpleTimeZone::setStartRule(int32_t month, int32_t dayOfWeekInMonth,
