@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2001-2007 IBM and others. All rights reserved.
+*   Copyright (C) 2001-2003 IBM and others. All rights reserved.
 **********************************************************************
 *   Date        Name        Description
 *  03/22/2000   helena      Creation.
@@ -12,11 +12,6 @@
 
 #include "unicode/utypes.h"
 
-/**
- * \file 
- * \brief C++ API: Service for searching text based on RuleBasedCollator.
- */
- 
 #if !UCONFIG_NO_COLLATION
 
 #include "unicode/tblcoll.h"
@@ -25,20 +20,19 @@
 
 U_NAMESPACE_BEGIN
 
-/** 
- *
+/**
  * <tt>StringSearch</tt> is a <tt>SearchIterator</tt> that provides
  * language-sensitive text searching based on the comparison rules defined
  * in a {@link RuleBasedCollator} object.
  * StringSearch ensures that language eccentricity can be 
- * handled, e.g. for the German collator, characters &szlig; and SS will be matched 
+ * handled, e.g. for the German collator, characters  and SS will be matched 
  * if case is chosen to be ignored.
- * See the <a href="http://source.icu-project.org/repos/icu/icuhtml/trunk/design/collation/ICU_collation_design.htm">
+ * See the <a href="http://oss.software.ibm.com/cvs/icu/~checkout~/icuhtml/design/collation/ICU_collation_design.htm">
  * "ICU Collation Design Document"</a> for more information.
  * <p> 
  * The algorithm implemented is a modified form of the Boyer Moore's search.
  * For more information  see 
- * <a href="http://icu-project.org/docs/papers/efficient_text_searching_in_java.html">
+ * <a href=http://oss.software.ibm.com/icu/docs/papers/text-search.html>
  * "Efficient Text Searching in Java"</a>, published in <i>Java Report</i> 
  * in February, 1999, for further information on the algorithm.
  * <p>
@@ -55,7 +49,7 @@ U_NAMESPACE_BEGIN
  *           there exists no non-ignorable combining mark before or after S? 
  *           in S respectively. 
  * </pre>
- * Option 2. will be the default.
+ * Option 2. will be the default
  * <p>
  * This search has APIs similar to that of other text iteration mechanisms 
  * such as the break iterators in <tt>BreakIterator</tt>. Using these 
@@ -114,15 +108,13 @@ U_NAMESPACE_BEGIN
  * and examples of how to use instances of this class to implement text
  * searching.
  * <pre><code>
- * UnicodeString target("The quick brown fox jumps over the lazy dog.");
+ * UnicodeString target("The quick brown fox jumped over the lazy fox");
  * UnicodeString pattern("fox");
  *
+ * SearchIterator *iter  = new StringSearch(pattern, target);
  * UErrorCode      error = U_ZERO_ERROR;
- * StringSearch iter(pattern, target, Locale::getUS(), NULL, status);
- * for (int pos = iter.first(error);
- *      pos != USEARCH_DONE; 
- *      pos = iter.next(error))
- * {
+ * for (int pos = iter->first(error); pos != USEARCH_DONE; 
+ *                               pos = iter->next(error)) {
  *     printf("Found match at %d pos, length is %d\n", pos, 
  *                                             iter.getMatchLength());
  * }
@@ -278,7 +270,7 @@ public:
      * @return a clone of this object
      *
      * @see getDynamicClassID
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */
     StringSearch *clone() const;
 
@@ -363,7 +355,7 @@ public:
      * <p>
      * Caller may modify but <b>must not</b> delete the <tt>RuleBasedCollator</tt>!
      * Modifications to this collator will affect the original collator passed in to 
-     * the <tt>StringSearch></tt> constructor or to setCollator, if any.
+     * the <tt>StringSearch>/tt> constructor or to setCollator, if any.
      * @return collator used for string search
      * @stable ICU 2.0
      */
@@ -433,7 +425,7 @@ public:
      *
      * @stable ICU 2.2
      */
-    static UClassID U_EXPORT2 getStaticClassID();
+    static UClassID getStaticClassID();
 
 protected:
 
@@ -442,15 +434,15 @@ protected:
     /**
      * Search forward for matching text, starting at a given location.
      * Clients should not call this method directly; instead they should 
-     * call {@link SearchIterator#next }.
+     * call {@link SearchIterator#next}.
      * <p>
      * If a match is found, this method returns the index at which the match
-     * starts and calls {@link SearchIterator#setMatchLength } with the number 
+     * starts and calls {@link SearchIterator#setMatchLength} with the number 
      * of characters in the target text that make up the match. If no match 
      * is found, the method returns <tt>USEARCH_DONE</tt>.
      * <p>
      * The <tt>StringSearch</tt> is adjusted so that its current index 
-     * (as returned by {@link #getOffset }) is the match position if one was 
+     * (as returned by {@link #getOffset()}) is the match position if one was 
      * found.
      * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
      * the <tt>StringSearch</tt> will be adjusted to the index USEARCH_DONE.
@@ -469,12 +461,12 @@ protected:
      * <tt>SearchIterator.previous()</tt>, which this method overrides.
      * <p>
      * If a match is found, this method returns the index at which the match
-     * starts and calls {@link SearchIterator#setMatchLength } with the number 
+     * starts and calls {@link SearchIterator#setMatchLength} with the number 
      * of characters in the target text that make up the match. If no match 
      * is found, the method returns <tt>USEARCH_DONE</tt>.
      * <p>
      * The <tt>StringSearch</tt> is adjusted so that its current index 
-     * (as returned by {@link #getOffset }) is the match position if one was 
+     * (as returned by {@link #getOffset()}) is the match position if one was 
      * found.
      * If a match is not found, <tt>USEARCH_DONE</tt> will be returned and
      * the <tt>StringSearch</tt> will be adjusted to the index USEARCH_DONE.
@@ -503,6 +495,11 @@ private :
     */
     UnicodeString      m_pattern_;
     /**
+    * Corresponding collation rules
+    * @stable ICU 2.0
+    */
+    UnicodeString      m_collation_rules_;
+    /**
     * String search struct data
     * @stable ICU 2.0
     */
