@@ -1,7 +1,6 @@
-/*************************************************************************
-* Copyright (c) 1997-2007, International Business Machines Corporation
-* and others. All Rights Reserved.
-**************************************************************************
+/*
+* Copyright (C) {1997-2003}, International Business Machines Corporation and others. All Rights Reserved.
+********************************************************************************
 *
 * File TIMEZONE.H
 *
@@ -21,7 +20,7 @@
 *                           Hashtable replaced by new static data structures.
 *   12/14/99    aliu        Made GMT public.
 *   08/15/01    grhoten     Made GMT private and added the getGMT() function
-**************************************************************************
+********************************************************************************
 */
 
 #ifndef TIMEZONE_H
@@ -29,11 +28,6 @@
 
 #include "unicode/utypes.h"
 
-/**
- * \file 
- * \brief C++ API: TimeZone object
- */
-
 #if !UCONFIG_NO_FORMATTING
 
 #include "unicode/uobject.h"
@@ -45,7 +39,6 @@ U_NAMESPACE_BEGIN
 class StringEnumeration;
 
 /**
- *
  * <code>TimeZone</code> represents a time zone offset, and also figures out daylight
  * savings.
  *
@@ -60,11 +53,11 @@ class StringEnumeration;
  * with a time zone ID. For instance, the time zone ID for the Pacific
  * Standard Time zone is "PST". So, you can get a PST <code>TimeZone</code> object
  * with:
- * \htmlonly<blockquote>\endhtmlonly
+ * <blockquote>
  * <pre>
  * TimeZone *tz = TimeZone::createTimeZone("PST");
  * </pre>
- * \htmlonly</blockquote>\endhtmlonly
+ * </blockquote>
  * You can use <code>getAvailableIDs</code> method to iterate through
  * all the supported time zone IDs. You can then choose a
  * supported ID to get a <code>TimeZone</code>.
@@ -72,11 +65,11 @@ class StringEnumeration;
  * supported IDs, then you can create a custom time zone ID with
  * the following syntax:
  *
- * \htmlonly<blockquote>\endhtmlonly
+ * <blockquote>
  * <pre>
  * GMT[+|-]hh[[:]mm]
  * </pre>
- * \htmlonly</blockquote>\endhtmlonly
+ * </blockquote>
  *
  * For example, you might specify GMT+14:00 as a custom
  * time zone ID.  The <code>TimeZone</code> that is returned
@@ -85,16 +78,14 @@ class StringEnumeration;
  *
  * TimeZone is an abstract class representing a time zone.  A TimeZone is needed for
  * Calendar to produce local time for a particular time zone.  A TimeZone comprises
- * three basic pieces of information:
- * <ul>
+ * three basic pieces of information:<ul>
  *    <li>A time zone offset; that, is the number of milliseconds to add or subtract
  *      from a time expressed in terms of GMT to convert it to the same time in that
- *      time zone (without taking daylight savings time into account).</li>
+ *      time zone (without taking daylight savings time into account).
  *    <li>Logic necessary to take daylight savings time into account if daylight savings
  *      time is observed in that time zone (e.g., the days and hours on which daylight
- *      savings time begins and ends).</li>
- *    <li>An ID.  This is a text string that uniquely identifies the time zone.</li>
- * </ul>
+ *      savings time begins and ends).
+ *    <li>An ID.  This is a text string that uniquely identifies the time zone.</ul>
  *
  * (Only the ID is actually implemented in TimeZone; subclasses of TimeZone may handle
  * daylight savings time and GMT offset in different ways.  Currently we only have one
@@ -132,7 +123,7 @@ public:
      * @return the GMT time zone.
      * @stable ICU 2.0
      */
-    static const TimeZone* U_EXPORT2 getGMT(void);
+    static const TimeZone* getGMT(void);
 
     /**
      * Creates a <code>TimeZone</code> for the given ID.
@@ -145,16 +136,16 @@ public:
      * return result.
      * @stable ICU 2.0
      */
-    static TimeZone* U_EXPORT2 createTimeZone(const UnicodeString& ID);
+    static TimeZone* createTimeZone(const UnicodeString& ID);
 
     /**
      * Returns an enumeration over all recognized time zone IDs. (i.e.,
      * all strings that createTimeZone() accepts)
      *
      * @return an enumeration object, owned by the caller.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
-    static StringEnumeration* U_EXPORT2 createEnumeration();
+    static StringEnumeration* createEnumeration();
 
     /**
      * Returns an enumeration over time zone IDs with a given raw
@@ -170,10 +161,10 @@ public:
      *
      * @param rawOffset an offset from GMT in milliseconds, ignoring
      * the effect of daylight savings time, if any
-     * @return an enumeration object, owned by the caller
-     * @stable ICU 2.4
+     * @return an enumeration object, owned by the caller 
+     * @draft ICU 2.4
      */
-    static StringEnumeration* U_EXPORT2 createEnumeration(int32_t rawOffset);
+    static StringEnumeration* createEnumeration(int32_t rawOffset);
 
     /**
      * Returns an enumeration over time zone IDs associated with the
@@ -182,10 +173,10 @@ public:
      *
      * @param country The ISO 3166 two-letter country code, or NULL to
      * retrieve zones not affiliated with any country.
-     * @return an enumeration object, owned by the caller
-     * @stable ICU 2.4
+     * @return an enumeration object, owned by the caller 
+     * @draft ICU 2.4
      */
-    static StringEnumeration* U_EXPORT2 createEnumeration(const char* country);
+    static StringEnumeration* createEnumeration(const char* country);
 
 #ifdef U_USE_TIMEZONE_OBSOLETE_2_8
     /**
@@ -267,7 +258,7 @@ public:
      * @see #getEquivalentID
      * @stable ICU 2.0
      */
-    static int32_t U_EXPORT2 countEquivalentIDs(const UnicodeString& id);
+    static int32_t countEquivalentIDs(const UnicodeString& id);
 
     /**
      * Returns an ID in the equivalency group that
@@ -288,7 +279,7 @@ public:
      * @see #countEquivalentIDs
      * @stable ICU 2.0
      */
-    static const UnicodeString U_EXPORT2 getEquivalentID(const UnicodeString& id,
+    static const UnicodeString getEquivalentID(const UnicodeString& id,
                                                int32_t index);
 
     /**
@@ -303,10 +294,10 @@ public:
      *           object returned.
      * @stable ICU 2.0
      */
-    static TimeZone* U_EXPORT2 createDefault(void);
+    static TimeZone* createDefault(void);
 
     /**
-     * Sets the default time zone (i.e., what's returned by createDefault()) to be the
+     * Sets the default time zone (i.e., what's returned by getDefault()) to be the
      * specified time zone.  If NULL is specified for the time zone, the default time
      * zone is set to the default host time zone.  This call adopts the TimeZone object
      * passed in; the clent is no longer responsible for deleting it.
@@ -314,7 +305,7 @@ public:
      * @param zone  A pointer to the new TimeZone object to use as the default.
      * @stable ICU 2.0
      */
-    static void U_EXPORT2 adoptDefault(TimeZone* zone);
+    static void adoptDefault(TimeZone* zone);
 
     /**
      * Same as adoptDefault(), except that the TimeZone object passed in is NOT adopted;
@@ -323,15 +314,7 @@ public:
      * @param zone  The given timezone.
      * @system
      */
-    static void U_EXPORT2 setDefault(const TimeZone& zone);
-
-    /**
-     * Returns the timezone data version currently used by ICU.
-     * @param status Output param to filled in with a success or an error.
-     * @return the version string, such as "2007f"
-     * @draft ICU 3.8
-     */
-    static const char* U_EXPORT2 getTZDataVersion(UErrorCode& status);
+    static void setDefault(const TimeZone& zone);
 
     /**
      * Returns true if the two TimeZones are equal.  (The TimeZone version only compares
@@ -427,7 +410,7 @@ public:
      * typically one hour.
      * @param ec input-output error code
      *
-     * @stable ICU 2.8
+     * @draft ICU 2.8
      */
     virtual void getOffset(UDate date, UBool local, int32_t& rawOffset,
                            int32_t& dstOffset, UErrorCode& ec) const;
@@ -465,7 +448,7 @@ public:
      * blockquote><pre>
      * .     TimeZone* foo = TimeZone::createTimeZone("America/New_York");
      * .     foo.setID("America/Los_Angeles");
-     * </pre>\htmlonly</blockquote>\endhtmlonly
+     * </pre></blockquote>
      * the time zone's GMT offset and daylight-savings rules don't change to those for
      * Los Angeles.  They're still those for New York.  Only the ID has changed.)
      *
@@ -476,17 +459,17 @@ public:
 
     /**
      * Enum for use with getDisplayName
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     enum EDisplayType {
         /**
          * Selector for short display name
-         * @stable ICU 2.4
+         * @draft ICU 2.4
          */
         SHORT = 1,
         /**
          * Selector for long display name
-         * @stable ICU 2.4
+         * @draft ICU 2.4
          */
         LONG
     };
@@ -592,42 +575,21 @@ public:
     virtual TimeZone* clone(void) const = 0;
 
     /**
-     * Return the class ID for this class.  This is useful only for
-     * comparing to a return value from getDynamicClassID().
-     * @return The class ID for all objects of this class.
-     * @stable ICU 2.0
-     */
-    static UClassID U_EXPORT2 getStaticClassID(void);
-
-    /**
-     * Returns a unique class ID POLYMORPHICALLY. This method is to
+     * Returns a unique class ID POLYMORPHICALLY. Pure virtual method. This method is to
      * implement a simple version of RTTI, since not all C++ compilers support genuine
      * RTTI. Polymorphic operator==() and clone() methods call this method.
      * <P>
-     * Concrete subclasses of TimeZone must use the UOBJECT_DEFINE_RTTI_IMPLEMENTATION
-     *  macro from uobject.h in their implementation to provide correct RTTI information.
+     * Concrete subclasses of TimeZone must implement getDynamicClassID() and also a
+     * static method and data member:
+     * <pre>
+     * .     static UClassID getStaticClassID() { return (UClassID)&fgClassID; }
+     * .     static char fgClassID;
+     * </pre>
      * @return   The class ID for this object. All objects of a given class have the
      *           same class ID. Objects of other classes have different class IDs.
      * @stable ICU 2.0
      */
     virtual UClassID getDynamicClassID(void) const = 0;
-    
-    /**
-     * Returns the amount of time to be added to local standard time
-     * to get local wall clock time.
-     * <p>
-     * The default implementation always returns 3600000 milliseconds
-     * (i.e., one hour) if this time zone observes Daylight Saving
-     * Time. Otherwise, 0 (zero) is returned.
-     * <p>
-     * If an underlying TimeZone implementation subclass supports
-     * historical Daylight Saving Time changes, this method returns
-     * the known latest daylight saving value.
-     *
-     * @return the amount of saving time in milliseconds
-     * @stable ICU 3.6
-     */
-    virtual int32_t getDSTSavings() const;
 
 protected:
 
