@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 2001-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *
@@ -16,17 +16,23 @@
 #ifndef UCOLEITR_H
 #define UCOLEITR_H
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 /**  
  * This indicates an error has occured during processing or if no more CEs is 
  * to be returned.
+ * @stable ICU 2.0
  */
-#define UCOL_NULLORDER        0xFFFFFFFF
+#define UCOL_NULLORDER        ((int32_t)0xFFFFFFFF)
 
 #include "unicode/ucol.h"
 
 /** 
  * The UCollationElements struct.
  * For usage in C programs.
+ * @stable ICU 2.0
  */
 typedef struct UCollationElements UCollationElements;
 
@@ -99,7 +105,7 @@ typedef struct UCollationElements UCollationElements;
  * @param textLength The number of characters in text, or -1 if null-terminated
  * @param status A pointer to an UErrorCode to receive any errors.
  * @return a struct containing collation element information
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI UCollationElements* U_EXPORT2 
 ucol_openElements(const UCollator  *coll,
@@ -109,7 +115,10 @@ ucol_openElements(const UCollator  *coll,
 
 /**
  * get a hash code for a key... Not very useful!
- * @stable
+ * @param key    the given key.
+ * @param length the size of the key array.
+ * @return       the hash code.
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 ucol_keyHashCode(const uint8_t* key, int32_t length);
@@ -118,7 +127,7 @@ ucol_keyHashCode(const uint8_t* key, int32_t length);
  * Close a UCollationElements.
  * Once closed, a UCollationElements may no longer be used.
  * @param elems The UCollationElements to close.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 ucol_closeElements(UCollationElements *elems);
@@ -130,7 +139,7 @@ ucol_closeElements(UCollationElements *elems);
  * @param elems The UCollationElements to reset.
  * @see ucol_next
  * @see ucol_previous
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 ucol_reset(UCollationElements *elems);
@@ -142,7 +151,7 @@ ucol_reset(UCollationElements *elems);
  * @param status A pointer to an UErrorCode to receive any errors.
  * @return The next collation elements ordering, otherwise returns NULLORDER 
  *         if an error has occured or if the end of string has been reached
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 ucol_next(UCollationElements *elems, UErrorCode *status);
@@ -161,7 +170,7 @@ ucol_next(UCollationElements *elems, UErrorCode *status);
  * @return The previous collation elements ordering, otherwise returns 
  *         NULLORDER if an error has occured or if the start of string has 
  *         been reached.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 ucol_previous(UCollationElements *elems, UErrorCode *status);
@@ -175,7 +184,7 @@ ucol_previous(UCollationElements *elems, UErrorCode *status);
  * @return maximum size of the expansion sequences ending with the collation 
  *         element or 1 if collation element does not occur at the end of any 
  *         expansion sequence
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 ucol_getMaxExpansion(const UCollationElements *elems, int32_t order);
@@ -190,7 +199,7 @@ ucol_getMaxExpansion(const UCollationElements *elems, int32_t order);
  * @param textLength The length of text, or -1 if null-terminated.
  * @param status A pointer to an UErrorCode to receive any errors.
  * @see ucol_getText
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 ucol_setText(      UCollationElements *elems, 
@@ -205,7 +214,7 @@ ucol_setText(      UCollationElements *elems,
  * @param elems The UCollationElements to query.
  * @return The offset of the current source character.
  * @see ucol_setOffset
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 ucol_getOffset(const UCollationElements *elems);
@@ -220,11 +229,40 @@ ucol_getOffset(const UCollationElements *elems);
  * @param offset The desired character offset.
  * @param status A pointer to an UErrorCode to receive any errors.
  * @see ucol_getOffset
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 ucol_setOffset(UCollationElements *elems,
                int32_t        offset,
                UErrorCode         *status);
 
+/**
+* Get the primary order of a collation order.
+* @param order the collation order
+* @return the primary order of a collation order.
+* @draft ICU 2.6
+*/
+U_CAPI int32_t U_EXPORT2
+ucol_primaryOrder (int32_t order); 
+
+/**
+* Get the secondary order of a collation order.
+* @param order the collation order
+* @return the secondary order of a collation order.
+* @draft ICU 2.6
+*/
+U_CAPI int32_t U_EXPORT2
+ucol_secondaryOrder (int32_t order); 
+
+/**
+* Get the tertiary order of a collation order.
+* @param order the collation order
+* @return the tertiary order of a collation order.
+* @draft ICU 2.6
+*/
+U_CAPI int32_t U_EXPORT2
+ucol_tertiaryOrder (int32_t order); 
+
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif
