@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1996-2001, International Business Machines Corporation and others. All Rights Reserved.
+* Copyright (C) 1996-2003, International Business Machines Corporation and others. All Rights Reserved.
 *******************************************************************************
 *
 *   file name:  umsg.h
@@ -19,6 +19,10 @@
 #define UMSG_H
 
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/uloc.h"
 #include "unicode/parseerr.h"
 #include <stdarg.h>
 /**
@@ -194,7 +198,7 @@
  * @return The total buffer size needed; if greater than resultLength, the
  * output was truncated.
  * @see u_parseMessage
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 u_formatMessage(const char  *locale,
@@ -221,6 +225,7 @@ u_formatMessage(const char  *locale,
  * @return The total buffer size needed; if greater than resultLength, the
  * output was truncated.
  * @see u_parseMessage
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 u_vformatMessage(   const char  *locale,
@@ -245,7 +250,7 @@ u_vformatMessage(   const char  *locale,
  * @param ... A variable-length argument list containing the arguments
  * specified in pattern.
  * @see u_formatMessage
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 u_parseMessage( const char   *locale,
@@ -270,6 +275,7 @@ u_parseMessage( const char   *locale,
  * @param status A pointer to an UErrorCode to receive any errors
  * specified in pattern.
  * @see u_formatMessage
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 u_vparseMessage(const char  *locale,
@@ -298,7 +304,7 @@ u_vparseMessage(const char  *locale,
  * @return The total buffer size needed; if greater than resultLength, the
  * output was truncated.
  * @see u_parseMessage
- * @draft ICU 2.0
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 u_formatMessageWithError(   const char    *locale,
@@ -327,6 +333,7 @@ u_formatMessageWithError(   const char    *locale,
  * in pattern.
  * @return The total buffer size needed; if greater than resultLength, the
  * output was truncated.
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 u_vformatMessageWithError(  const char   *locale,
@@ -354,7 +361,7 @@ u_vformatMessageWithError(  const char   *locale,
  * @param ... A variable-length argument list containing the arguments
  * specified in pattern.
  * @see u_formatMessage
- * @draft ICU 2.0
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 u_parseMessageWithError(const char  *locale,
@@ -362,7 +369,7 @@ u_parseMessageWithError(const char  *locale,
                         int32_t     patternLength,
                         const UChar *source,
                         int32_t     sourceLength,
-                        UParseError *error,
+                        UParseError *parseError,
                         UErrorCode  *status,
                         ...);
 
@@ -382,7 +389,7 @@ u_parseMessageWithError(const char  *locale,
  * @param status A pointer to an UErrorCode to receive any errors
  * specified in pattern.
  * @see u_formatMessage
- * @draft ICU 2.0
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 u_vparseMessageWithError(const char  *locale,
@@ -391,11 +398,14 @@ u_vparseMessageWithError(const char  *locale,
                          const UChar *source,
                          int32_t     sourceLength,
                          va_list     ap,
-                         UParseError *error,
+                         UParseError *parseError,
                          UErrorCode* status);
 
 /*----------------------- New experimental API --------------------------- */
-
+/** 
+ * The message format object
+ * @stable ICU 2.0
+ */
 typedef void* UMessageFormat;
 
 
@@ -409,7 +419,7 @@ typedef void* UMessageFormat;
  * @param status        A pointer to an UErrorCode to receive any errors.
  * @return              A pointer to a UMessageFormat to use for formatting 
  *                      messages, or 0 if an error occurred. 
- * @draft ICU 2.0
+ * @stable ICU 2.0
  */
 U_CAPI UMessageFormat* U_EXPORT2 
 umsg_open(  const UChar     *pattern,
@@ -421,8 +431,8 @@ umsg_open(  const UChar     *pattern,
 /**
  * Close a UMessageFormat.
  * Once closed, a UMessageFormat may no longer be used.
- * @param fmt The formatter to close.
- * @stable
+ * @param format The formatter to close.
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 umsg_close(UMessageFormat* format);
@@ -433,7 +443,7 @@ umsg_close(UMessageFormat* format);
  * @param fmt The formatter to copy
  * @param status A pointer to an UErrorCode to receive any errors.
  * @return A pointer to a UDateFormat identical to fmt.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI UMessageFormat U_EXPORT2 
 umsg_clone(const UMessageFormat *fmt,
@@ -444,6 +454,7 @@ umsg_clone(const UMessageFormat *fmt,
  * format information.
  * @param fmt The formatter to set
  * @param locale The locale the formatter should use.
+ * @stable ICU 2.0
  */
 U_CAPI void  U_EXPORT2 
 umsg_setLocale(UMessageFormat *fmt,
@@ -452,8 +463,9 @@ umsg_setLocale(UMessageFormat *fmt,
 /**
  * Gets the locale. This locale is used for fetching default number or date
  * format information.
- * @param The formatter to querry
- * @stable
+ * @param fmt The formatter to querry
+ * @return the locale.
+ * @stable ICU 2.0
  */
 U_CAPI const char*  U_EXPORT2 
 umsg_getLocale(UMessageFormat *fmt);
@@ -468,7 +480,7 @@ umsg_getLocale(UMessageFormat *fmt);
  * @param status        Output param set to success/failure code on
  *                      exit. If the pattern is invalid, this will be
  *                      set to a failure result.
- * @draft
+ * @stable ICU 2.0
  */
 U_CAPI void  U_EXPORT2 
 umsg_applyPattern( UMessageFormat *fmt,
@@ -484,8 +496,9 @@ umsg_applyPattern( UMessageFormat *fmt,
  * @param resultLength The maximum size of result.
  * @param status       Output param set to success/failure code on
  *                     exit. If the pattern is invalid, this will be
- *                     set to a failure result.   
- * @draft
+ *                     set to a failure result.  
+ * @return the pattern of the format
+ * @stable ICU 2.0
  */
 U_CAPI int32_t  U_EXPORT2 
 umsg_toPattern(UMessageFormat *fmt,
@@ -506,7 +519,7 @@ umsg_toPattern(UMessageFormat *fmt,
  *                      specified in pattern.
  * @return              The total buffer size needed; if greater than resultLength, 
  *                      the output was truncated.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 umsg_format(    UMessageFormat *fmt,
@@ -528,6 +541,7 @@ umsg_format(    UMessageFormat *fmt,
  *                     specified in pattern.
  * @return             The total buffer size needed; if greater than resultLength, 
  *                     the output was truncated.
+ * @stable ICU 2.0
  */
 U_CAPI int32_t U_EXPORT2 
 umsg_vformat(   UMessageFormat *fmt,
@@ -548,7 +562,7 @@ umsg_vformat(   UMessageFormat *fmt,
  * @param status        A pointer to an UErrorCode to receive any errors
  * @param ...           A variable-length argument list containing the arguments
  *                      specified in pattern.
- * @stable
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 umsg_parse( UMessageFormat *fmt,
@@ -571,6 +585,7 @@ umsg_parse( UMessageFormat *fmt,
  * @param status        A pointer to an UErrorCode to receive any errors
  *                      specified in pattern.
  * @see u_formatMessage
+ * @stable ICU 2.0
  */
 U_CAPI void U_EXPORT2 
 umsg_vparse(UMessageFormat *fmt,
@@ -579,4 +594,22 @@ umsg_vparse(UMessageFormat *fmt,
             int32_t        *count,
             va_list        ap,
             UErrorCode     *status);
+
+
+/**
+ * Get the locale for this message format object.
+ * You can choose between valid and actual locale.
+ * @param fmt The formatter to get the locale from
+ * @param type type of the locale we're looking for (valid or actual) 
+ * @param status error code for the operation
+ * @return the locale name
+ * @draft ICU 2.8 likely to change in ICU 3.0, based on feedback
+ */
+U_CAPI const char* U_EXPORT2
+umsg_getLocaleByType(const UMessageFormat *fmt,
+                     ULocDataLocaleType type,
+                     UErrorCode* status); 
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif
