@@ -1,7 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2007, International Business Machines Corporation and others.
-* All Rights Reserved.
+* Copyright (C) 1997-2003, International Business Machines Corporation and others. All Rights Reserved.
 * Modification History:
 *
 *   Date        Name        Description
@@ -101,13 +100,9 @@
  * to get a format for displaying percentages. With this format, a
  * fraction from 0.53 is displayed as 53%.
  * <P>
- * Use a pattern to create either a DecimalFormat or a RuleBasedNumberFormat
- * formatter.  The pattern must conform to the syntax defined for those
- * formatters.
- * <P>
  * You can also control the display of numbers with such function as
- * unum_getAttribues() and unum_setAtributes(), which let you set the
- * miminum fraction digits, grouping, etc.
+ * unum_getAttribues() and unum_setAtributes().  where in you can set the
+ * miminum fraction digits, grouping used etc.
  * @see UNumberFormatAttributes for more details
  * <P>
  * You can also use forms of the parse and format methods with
@@ -132,11 +127,8 @@ typedef void* UNumberFormat;
  *  @stable ICU 2.0
  */
 typedef enum UNumberFormatStyle {
-    /**
-     * Decimal format defined by pattern 
-     * @stable ICU 3.0
-     */
-    UNUM_PATTERN_DECIMAL=0,
+    /** Ignore style specification and open the pattern */
+    UNUM_IGNORE=0,
     /** Decimal format */
     UNUM_DECIMAL=1,
     /** Currency format */
@@ -145,27 +137,10 @@ typedef enum UNumberFormatStyle {
     UNUM_PERCENT,
     /** Scientific format */
     UNUM_SCIENTIFIC,
-    /** Spellout rule-based format */
+    /** Spellout format */
     UNUM_SPELLOUT,
-    /** 
-     * Ordinal rule-based format 
-     * @stable ICU 3.0
-     */
-    UNUM_ORDINAL,
-    /** 
-     * Duration rule-based format 
-     * @stable ICU 3.0
-     */
-    UNUM_DURATION,
-    /** 
-     * Rule-based format defined by pattern 
-     * @stable ICU 3.0
-     */
-    UNUM_PATTERN_RULEBASED,
     /** Default format */
-    UNUM_DEFAULT = UNUM_DECIMAL,
-    /** (Alias for UNUM_PATTERN_DECIMAL) */
-    UNUM_IGNORE = UNUM_PATTERN_DECIMAL
+    UNUM_DEFAULT = UNUM_DECIMAL
 } UNumberFormatStyle;
 
 /** The possible number format rounding modes. 
@@ -176,18 +151,9 @@ typedef enum UNumberFormatRoundingMode {
     UNUM_ROUND_FLOOR,
     UNUM_ROUND_DOWN,
     UNUM_ROUND_UP,
-    /**
-     * Half-even rounding, misspelled name
-     * @deprecated, ICU 3.8
-     */
     UNUM_FOUND_HALFEVEN,
     UNUM_ROUND_HALFDOWN,
-    UNUM_ROUND_HALFUP,
-    /**
-     * Half-even rounding
-     * @stable, ICU 3.8
-     */
-    UNUM_ROUND_HALFEVEN = UNUM_FOUND_HALFEVEN
+    UNUM_ROUND_HALFUP
 } UNumberFormatRoundingMode;
 
 /** The possible number format pad positions. 
@@ -201,37 +167,25 @@ typedef enum UNumberFormatPadPosition {
 } UNumberFormatPadPosition;
 
 /**
- * Create and return a new UNumberFormat for formatting and parsing
- * numbers.  A UNumberFormat may be used to format numbers by calling
- * {@link #unum_format }, and to parse numbers by calling {@link #unum_parse }.
- * The caller must call {@link #unum_close } when done to release resources
- * used by this object.
- * @param style The type of number format to open: one of
- * UNUM_DECIMAL, UNUM_CURRENCY, UNUM_PERCENT, UNUM_SCIENTIFIC, UNUM_SPELLOUT,
- * UNUM_PATTERN_DECIMAL, UNUM_PATTERN_RULEBASED, or UNUM_DEFAULT.
- * If UNUM_PATTERN_DECIMAL or UNUM_PATTERN_RULEBASED is passed then the
- * number format is opened using the given pattern, which must conform
- * to the syntax described in DecimalFormat or RuleBasedNumberFormat,
- * respectively.
- * @param pattern A pattern specifying the format to use. 
- * This parameter is ignored unless the style is
- * UNUM_PATTERN_DECIMAL or UNUM_PATTERN_RULEBASED.
- * @param patternLength The number of characters in the pattern, or -1
- * if null-terminated. This parameter is ignored unless the style is
- * UNUM_PATTERN.
- * @param locale A locale identifier to use to determine formatting
- * and parsing conventions, or NULL to use the default locale.
- * @param parseErr A pointer to a UParseError struct to receive the
- * details of any parsing errors, or NULL if no parsing error details
- * are desired.
- * @param status A pointer to an input-output UErrorCode.
- * @return A pointer to a newly created UNumberFormat, or NULL if an
- * error occurred.
- * @see unum_close
- * @see DecimalFormat
- * @stable ICU 2.0
- */
-U_STABLE UNumberFormat* U_EXPORT2 
+* Open a new UNumberFormat for formatting and parsing numbers.
+* A UNumberFormat may be used to format numbers in calls to \Ref{unum_format},
+* and to parse numbers in calls to \Ref{unum_parse}.
+* @param style The type of number format to open: one of UNUM_DECIMAL, UNUM_CURRENCY,
+* UNUM_PERCENT, UNUM_SPELLOUT, UNUM_DEFAULT or UNUM_IGNORE. If UNUM_IGNORE is passed
+* then the style specification is ignored and a number format is opened with the pattern.
+* @param pattern A pattern specifying the format to use.
+* @param patternLength The number of characters in the pattern, or -1 if null-terminated.
+* @param locale The locale specifying the formatting conventions
+* @param parseErr a pointer to a UParseError struct to receive the
+* details of any parsing errors. This parameter may be NULL if no
+* parsing error details are desired.
+* @param status A pointer to an UErrorCode to receive any errors
+* @return A pointer to a UNumberFormat to use for formatting numbers, or 0 if
+* an error occurred.
+* @see unum_open
+* @stable ICU 2.0
+*/
+U_CAPI UNumberFormat* U_EXPORT2 
 unum_open(  UNumberFormatStyle    style,
             const    UChar*    pattern,
             int32_t            patternLength,
@@ -246,7 +200,7 @@ unum_open(  UNumberFormatStyle    style,
 * @param fmt The formatter to close.
 * @stable ICU 2.0
 */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 unum_close(UNumberFormat* fmt);
 
 /**
@@ -257,7 +211,7 @@ unum_close(UNumberFormat* fmt);
  * @return A pointer to a UNumberFormat identical to fmt.
  * @stable ICU 2.0
  */
-U_STABLE UNumberFormat* U_EXPORT2 
+U_CAPI UNumberFormat* U_EXPORT2 
 unum_clone(const UNumberFormat *fmt,
        UErrorCode *status);
 
@@ -282,7 +236,7 @@ unum_clone(const UNumberFormat *fmt,
 * @see UFieldPosition
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_format(    const    UNumberFormat*    fmt,
         int32_t            number,
         UChar*            result,
@@ -311,7 +265,7 @@ unum_format(    const    UNumberFormat*    fmt,
 * @see UFieldPosition
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_formatInt64(const UNumberFormat *fmt,
         int64_t         number,
         UChar*          result,
@@ -340,7 +294,7 @@ unum_formatInt64(const UNumberFormat *fmt,
 * @see UFieldPosition
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_formatDouble(    const    UNumberFormat*  fmt,
             double          number,
             UChar*          result,
@@ -348,36 +302,6 @@ unum_formatDouble(    const    UNumberFormat*  fmt,
             UFieldPosition  *pos, /* 0 if ignore */
             UErrorCode*     status);
 
-/**
- * Format a double currency amount using a UNumberFormat.
- * The double will be formatted according to the UNumberFormat's locale.
- * @param fmt the formatter to use
- * @param number the number to format
- * @param currency the 3-letter null-terminated ISO 4217 currency code
- * @param result a pointer to the buffer to receive the formatted number
- * @param resultLength the maximum number of UChars to write to result
- * @param pos a pointer to a UFieldPosition.  On input,
- * position->field is read.  On output, position->beginIndex and
- * position->endIndex indicate the beginning and ending indices of
- * field number position->field, if such a field exists.  This
- * parameter may be NULL, in which case it is ignored.
- * @param status a pointer to an input-output UErrorCode
- * @return the total buffer size needed; if greater than resultLength,
- * the output was truncated.
- * @see unum_formatDouble
- * @see unum_parseDoubleCurrency
- * @see UFieldPosition
- * @stable ICU 3.0
- */
-U_STABLE int32_t U_EXPORT2 
-unum_formatDoubleCurrency(const UNumberFormat* fmt,
-                          double number,
-                          UChar* currency,
-                          UChar* result,
-                          int32_t resultLength,
-                          UFieldPosition* pos, /* ignored if 0 */
-                          UErrorCode* status);
-
 /**
 * Parse a string into an integer using a UNumberFormat.
 * The string will be parsed according to the UNumberFormat's locale.
@@ -395,7 +319,7 @@ unum_formatDoubleCurrency(const UNumberFormat* fmt,
 * @see unum_formatDouble
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_parse(    const   UNumberFormat*  fmt,
         const   UChar*          text,
         int32_t         textLength,
@@ -417,9 +341,9 @@ unum_parse(    const   UNumberFormat*  fmt,
 * @see unum_format
 * @see unum_formatInt64
 * @see unum_formatDouble
-* @stable ICU 2.8
+* @draft ICU 2.8
 */
-U_STABLE int64_t U_EXPORT2 
+U_CAPI int64_t U_EXPORT2 
 unum_parseInt64(const UNumberFormat*  fmt,
         const UChar*  text,
         int32_t       textLength,
@@ -443,7 +367,7 @@ unum_parseInt64(const UNumberFormat*  fmt,
 * @see unum_formatDouble
 * @stable ICU 2.0
 */
-U_STABLE double U_EXPORT2 
+U_CAPI double U_EXPORT2 
 unum_parseDouble(    const   UNumberFormat*  fmt,
             const   UChar*          text,
             int32_t         textLength,
@@ -451,49 +375,19 @@ unum_parseDouble(    const   UNumberFormat*  fmt,
             UErrorCode      *status);
 
 /**
- * Parse a string into a double and a currency using a UNumberFormat.
- * The string will be parsed according to the UNumberFormat's locale.
- * @param fmt the formatter to use
- * @param text the text to parse
- * @param textLength the length of text, or -1 if null-terminated
- * @param parsePos a pointer to an offset index into text at which to
- * begin parsing. On output, *parsePos will point after the last
- * parsed character.  This parameter may be 0, in which case parsing
- * begins at offset 0.
- * @param currency a pointer to the buffer to receive the parsed null-
- * terminated currency.  This buffer must have a capacity of at least
- * 4 UChars.
- * @param status a pointer to an input-output UErrorCode
- * @return the parsed double
- * @see unum_parseDouble
- * @see unum_formatDoubleCurrency
- * @stable ICU 3.0
- */
-U_STABLE double U_EXPORT2
-unum_parseDoubleCurrency(const UNumberFormat* fmt,
-                         const UChar* text,
-                         int32_t textLength,
-                         int32_t* parsePos, /* 0 = start */
-                         UChar* currency,
-                         UErrorCode* status);
-
-/**
- * Set the pattern used by a UNumberFormat.  This can only be used
- * on a DecimalFormat, other formats return U_ILLEGAL_ARGUMENT_ERROR
- * in the status.
- * @param format The formatter to set.
- * @param localized TRUE if the pattern is localized, FALSE otherwise.
- * @param pattern The new pattern
- * @param patternLength The length of pattern, or -1 if null-terminated.
- * @param parseError A pointer to UParseError to recieve information
- * about errors occurred during parsing, or NULL if no parse error
- * information is desired.
- * @param status A pointer to an input-output UErrorCode.
- * @see unum_toPattern
- * @see DecimalFormat
- * @stable ICU 2.0
- */
-U_STABLE void U_EXPORT2 
+* Set the pattern used by an UNumberFormat.
+* The pattern should follow the pattern syntax rules.
+* @param format The formatter to set.
+* @param localized TRUE if the pattern is localized, FALSE otherwise.
+* @param pattern The new pattern
+* @param parseError  A pointer to UParseError to recieve information about errors
+*                    occurred during parsing.
+* @param patternLength The length of pattern, or -1 if null-terminated.
+* @param status A pointer to an UErrorCode to receive any errors
+* @see unum_toPattern
+* @stable ICU 2.0
+*/
+U_CAPI void U_EXPORT2 
 unum_applyPattern(          UNumberFormat  *format,
                             UBool          localized,
                     const   UChar          *pattern,
@@ -503,28 +397,26 @@ unum_applyPattern(          UNumberFormat  *format,
                                     );
 
 /**
-* Get a locale for which decimal formatting patterns are available.
+* Get a locale for which number formatting patterns are available.
 * A UNumberFormat in a locale returned by this function will perform the correct
-* formatting and parsing for the locale.  The results of this call are not
-* valid for rule-based number formats.
+* formatting and parsing for the locale.
 * @param index The index of the desired locale.
 * @return A locale for which number formatting patterns are available, or 0 if none.
 * @see unum_countAvailable
 * @stable ICU 2.0
 */
-U_STABLE const char* U_EXPORT2 
+U_CAPI const char* U_EXPORT2 
 unum_getAvailable(int32_t index);
 
 /**
-* Determine how many locales have decimal formatting patterns available.  The
-* results of this call are not valid for rule-based number formats.
-* This function is useful for determining the loop ending condition for
-* calls to {@link #unum_getAvailable }.
-* @return The number of locales for which decimal formatting patterns are available.
+* Determine how many locales have number formatting patterns available.
+* This function is most useful as determining the loop ending condition for
+* calls to \Ref{unum_getAvailable}.
+* @return The number of locales for which number formatting patterns are available.
 * @see unum_getAvailable
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_countAvailable(void);
 
 /** The possible UNumberFormat numeric attributes @stable ICU 2.0 */
@@ -560,20 +452,7 @@ typedef enum UNumberFormatAttribute {
   /** The position at which padding will take place. */
   UNUM_PADDING_POSITION,
   /** Secondary grouping size */
-  UNUM_SECONDARY_GROUPING_SIZE,
-  /** Use significant digits
-   * @stable ICU 3.0 */
-  UNUM_SIGNIFICANT_DIGITS_USED,
-  /** Minimum significant digits
-   * @stable ICU 3.0 */
-  UNUM_MIN_SIGNIFICANT_DIGITS,
-  /** Maximum significant digits
-   * @stable ICU 3.0 */
-  UNUM_MAX_SIGNIFICANT_DIGITS,
-  /** Lenient parse mode used by rule-based formats.
-   * @stable ICU 3.0
-   */
-  UNUM_LENIENT_PARSE
+  UNUM_SECONDARY_GROUPING_SIZE
 } UNumberFormatAttribute;
 
 /**
@@ -592,21 +471,18 @@ typedef enum UNumberFormatAttribute {
 * @see unum_setTextAttribute
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_getAttribute(const UNumberFormat*          fmt,
           UNumberFormatAttribute  attr);
 
 /**
 * Set a numeric attribute associated with a UNumberFormat.
-* An example of a numeric attribute is the number of integer digits a formatter will produce.  If the
-* formatter does not understand the attribute, the call is ignored.  Rule-based formatters only understand
-* the lenient-parse attribute.
+* An example of a numeric attribute is the number of integer digits a formatter will produce.
 * @param fmt The formatter to set.
 * @param attr The attribute to set; one of UNUM_PARSE_INT_ONLY, UNUM_GROUPING_USED,
 * UNUM_DECIMAL_ALWAYS_SHOWN, UNUM_MAX_INTEGER_DIGITS, UNUM_MIN_INTEGER_DIGITS, UNUM_INTEGER_DIGITS,
 * UNUM_MAX_FRACTION_DIGITS, UNUM_MIN_FRACTION_DIGITS, UNUM_FRACTION_DIGITS, UNUM_MULTIPLIER,
-* UNUM_GROUPING_SIZE, UNUM_ROUNDING_MODE, UNUM_FORMAT_WIDTH, UNUM_PADDING_POSITION, UNUM_SECONDARY_GROUPING_SIZE,
-* or UNUM_LENIENT_PARSE.
+* UNUM_GROUPING_SIZE, UNUM_ROUNDING_MODE, UNUM_FORMAT_WIDTH, UNUM_PADDING_POSITION, UNUM_SECONDARY_GROUPING_SIZE.
 * @param newValue The new value of attr.
 * @see unum_getAttribute
 * @see unum_getDoubleAttribute
@@ -615,7 +491,7 @@ unum_getAttribute(const UNumberFormat*          fmt,
 * @see unum_setTextAttribute
 * @stable ICU 2.0
 */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 unum_setAttribute(    UNumberFormat*          fmt,
             UNumberFormatAttribute  attr,
             int32_t                 newValue);
@@ -624,7 +500,6 @@ unum_setAttribute(    UNumberFormat*          fmt,
 /**
 * Get a numeric attribute associated with a UNumberFormat.
 * An example of a numeric attribute is the number of integer digits a formatter will produce.
-* If the formatter does not understand the attribute, -1 is returned.
 * @param fmt The formatter to query.
 * @param attr The attribute to query; e.g. UNUM_ROUNDING_INCREMENT.
 * @return The value of attr.
@@ -635,14 +510,13 @@ unum_setAttribute(    UNumberFormat*          fmt,
 * @see unum_setTextAttribute
 * @stable ICU 2.0
 */
-U_STABLE double U_EXPORT2 
+U_CAPI double U_EXPORT2 
 unum_getDoubleAttribute(const UNumberFormat*          fmt,
           UNumberFormatAttribute  attr);
 
 /**
 * Set a numeric attribute associated with a UNumberFormat.
 * An example of a numeric attribute is the number of integer digits a formatter will produce.
-* If the formatter does not understand the attribute, this call is ignored.
 * @param fmt The formatter to set.
 * @param attr The attribute to set; e.g. UNUM_ROUNDING_INCREMENT.
 * @param newValue The new value of attr.
@@ -653,7 +527,7 @@ unum_getDoubleAttribute(const UNumberFormat*          fmt,
 * @see unum_setTextAttribute
 * @stable ICU 2.0
 */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 unum_setDoubleAttribute(    UNumberFormat*          fmt,
             UNumberFormatAttribute  attr,
             double                 newValue);
@@ -671,30 +545,15 @@ typedef enum UNumberFormatTextAttribute {
   /** The character used to pad to the format width. */
   UNUM_PADDING_CHARACTER,
   /** The ISO currency code */
-  UNUM_CURRENCY_CODE,
-  /**
-   * The default rule set.  This is only available with rule-based formatters.
-   * @stable ICU 3.0
-   */
-  UNUM_DEFAULT_RULESET,
-  /**
-   * The public rule sets.  This is only available with rule-based formatters.
-   * This is a read-only attribute.  The public rulesets are returned as a
-   * single string, with each ruleset name delimited by ';' (semicolon).
-   * @stable ICU 3.0
-   */
-  UNUM_PUBLIC_RULESETS
+  UNUM_CURRENCY_CODE
 } UNumberFormatTextAttribute;
 
 /**
 * Get a text attribute associated with a UNumberFormat.
-* An example of a text attribute is the suffix for positive numbers.  If the formatter
-* does not understand the attributre, U_UNSUPPORTED_ERROR is returned as the status.
-* Rule-based formatters only understand UNUM_DEFAULT_RULESET and UNUM_PUBLIC_RULESETS.
+* An example of a text attribute is the suffix for positive numbers.
 * @param fmt The formatter to query.
 * @param tag The attribute to query; one of UNUM_POSITIVE_PREFIX, UNUM_POSITIVE_SUFFIX,
-* UNUM_NEGATIVE_PREFIX, UNUM_NEGATIVE_SUFFIX, UNUM_PADDING_CHARACTER, UNUM_CURRENCY_CODE,
-* UNUM_DEFAULT_RULESET, or UNUM_PUBLIC_RULESETS.
+* UNUM_NEGATIVE_PREFIX, UNUM_NEGATIVE_SUFFIX
 * @param result A pointer to a buffer to receive the attribute.
 * @param resultLength The maximum size of result.
 * @param status A pointer to an UErrorCode to receive any errors
@@ -704,7 +563,7 @@ typedef enum UNumberFormatTextAttribute {
 * @see unum_setAttribute
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 unum_getTextAttribute(    const    UNumberFormat*                    fmt,
             UNumberFormatTextAttribute      tag,
             UChar*                            result,
@@ -713,12 +572,10 @@ unum_getTextAttribute(    const    UNumberFormat*                    fmt,
 
 /**
 * Set a text attribute associated with a UNumberFormat.
-* An example of a text attribute is the suffix for positive numbers.  Rule-based formatters
-* only understand UNUM_DEFAULT_RULESET.
+* An example of a text attribute is the suffix for positive numbers.
 * @param fmt The formatter to set.
 * @param tag The attribute to set; one of UNUM_POSITIVE_PREFIX, UNUM_POSITIVE_SUFFIX,
-* UNUM_NEGATIVE_PREFIX, UNUM_NEGATIVE_SUFFIX, UNUM_PADDING_CHARACTER, UNUM_CURRENCY_CODE,
-* or UNUM_DEFAULT_RULESET.
+* UNUM_NEGATIVE_PREFIX, UNUM_NEGATIVE_SUFFIX
 * @param newValue The new value of attr.
 * @param newValueLength The length of newValue, or -1 if null-terminated.
 * @param status A pointer to an UErrorCode to receive any errors
@@ -727,7 +584,7 @@ unum_getTextAttribute(    const    UNumberFormat*                    fmt,
 * @see unum_setAttribute
 * @stable ICU 2.0
 */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 unum_setTextAttribute(    UNumberFormat*                    fmt,
             UNumberFormatTextAttribute      tag,
             const    UChar*                            newValue,
@@ -735,28 +592,25 @@ unum_setTextAttribute(    UNumberFormat*                    fmt,
             UErrorCode                        *status);
 
 /**
- * Extract the pattern from a UNumberFormat.  The pattern will follow
- * the DecimalFormat pattern syntax.
- * @param fmt The formatter to query.
- * @param isPatternLocalized TRUE if the pattern should be localized,
- * FALSE otherwise.  This is ignored if the formatter is a rule-based
- * formatter.
- * @param result A pointer to a buffer to receive the pattern.
- * @param resultLength The maximum size of result.
- * @param status A pointer to an input-output UErrorCode.
- * @return The total buffer size needed; if greater than resultLength,
- * the output was truncated.
- * @see unum_applyPattern
- * @see DecimalFormat
- * @stable ICU 2.0
- */
-U_STABLE int32_t U_EXPORT2 
+* Extract the pattern from a UNumberFormat.
+* The pattern will follow the pattern syntax.
+* @param fmt The formatter to query.
+* @param isPatternLocalized TRUE if the pattern should be localized, FALSE otherwise.
+* @param result A pointer to a buffer to receive the pattern.
+* @param resultLength The maximum size of result.
+* @param status A pointer to an UErrorCode to receive any errors
+* @return The total buffer size needed; if greater than resultLength, the output was truncated.
+* @stable ICU 2.0
+*/
+U_CAPI int32_t U_EXPORT2 
 unum_toPattern(    const    UNumberFormat*          fmt,
         UBool                  isPatternLocalized,
         UChar*                  result,
         int32_t                 resultLength,
         UErrorCode*             status);
 
+/** The maximum size for a textual number format symbol. @internal*/
+#define UNFSYMBOLSMAXSIZE 10
 
 /**
  * Constants for specifying a number format symbol.
@@ -764,53 +618,45 @@ unum_toPattern(    const    UNumberFormat*          fmt,
  */
 typedef enum UNumberFormatSymbol {
   /** The decimal separator */
-  UNUM_DECIMAL_SEPARATOR_SYMBOL = 0,
+  UNUM_DECIMAL_SEPARATOR_SYMBOL,
   /** The grouping separator */
-  UNUM_GROUPING_SEPARATOR_SYMBOL = 1,
+  UNUM_GROUPING_SEPARATOR_SYMBOL,
   /** The pattern separator */
-  UNUM_PATTERN_SEPARATOR_SYMBOL = 2,
+  UNUM_PATTERN_SEPARATOR_SYMBOL,
   /** The percent sign */
-  UNUM_PERCENT_SYMBOL = 3,
+  UNUM_PERCENT_SYMBOL,
   /** Zero*/
-  UNUM_ZERO_DIGIT_SYMBOL = 4,
+  UNUM_ZERO_DIGIT_SYMBOL,
   /** Character representing a digit in the pattern */
-  UNUM_DIGIT_SYMBOL = 5,
+  UNUM_DIGIT_SYMBOL,
   /** The minus sign */
-  UNUM_MINUS_SIGN_SYMBOL = 6,
+  UNUM_MINUS_SIGN_SYMBOL,
   /** The plus sign */
-  UNUM_PLUS_SIGN_SYMBOL = 7,
+  UNUM_PLUS_SIGN_SYMBOL,
   /** The currency symbol */
-  UNUM_CURRENCY_SYMBOL = 8,
+  UNUM_CURRENCY_SYMBOL,
   /** The international currency symbol */
-  UNUM_INTL_CURRENCY_SYMBOL = 9,
+  UNUM_INTL_CURRENCY_SYMBOL,
   /** The monetary separator */
-  UNUM_MONETARY_SEPARATOR_SYMBOL = 10,
+  UNUM_MONETARY_SEPARATOR_SYMBOL,
   /** The exponential symbol */
-  UNUM_EXPONENTIAL_SYMBOL = 11,
+  UNUM_EXPONENTIAL_SYMBOL,
   /** Per mill symbol */
-  UNUM_PERMILL_SYMBOL = 12,
+  UNUM_PERMILL_SYMBOL,
   /** Escape padding character */
-  UNUM_PAD_ESCAPE_SYMBOL = 13,
+  UNUM_PAD_ESCAPE_SYMBOL,
   /** Infinity symbol */
-  UNUM_INFINITY_SYMBOL = 14,
+  UNUM_INFINITY_SYMBOL,
   /** Nan symbol */
-  UNUM_NAN_SYMBOL = 15,
-  /** Significant digit symbol
-   * @stable ICU 3.0 */
-  UNUM_SIGNIFICANT_DIGIT_SYMBOL = 16,
-  /** The monetary grouping separator 
-   * @stable ICU 3.6
-   */
-  UNUM_MONETARY_GROUPING_SEPARATOR_SYMBOL = 17,  
+  UNUM_NAN_SYMBOL,
   /** count symbol constants */
-  UNUM_FORMAT_SYMBOL_COUNT = 18
+  UNUM_FORMAT_SYMBOL_COUNT
 } UNumberFormatSymbol;
 
 /**
 * Get a symbol associated with a UNumberFormat.
 * A UNumberFormat uses symbols to represent the special locale-dependent
-* characters in a number, for example the percent sign. This API is not
-* supported for rule-based formatters.
+* characters in a number, for example the percent sign.
 * @param fmt The formatter to query.
 * @param symbol The UNumberFormatSymbol constant for the symbol to get
 * @param buffer The string buffer that will receive the symbol string;
@@ -822,8 +668,8 @@ typedef enum UNumberFormatSymbol {
 * @see unum_setSymbol
 * @stable ICU 2.0
 */
-U_STABLE int32_t U_EXPORT2
-unum_getSymbol(const UNumberFormat *fmt,
+U_CAPI int32_t U_EXPORT2
+unum_getSymbol(UNumberFormat *fmt,
                UNumberFormatSymbol symbol,
                UChar *buffer,
                int32_t size,
@@ -832,8 +678,7 @@ unum_getSymbol(const UNumberFormat *fmt,
 /**
 * Set a symbol associated with a UNumberFormat.
 * A UNumberFormat uses symbols to represent the special locale-dependent
-* characters in a number, for example the percent sign.  This API is not
-* supported for rule-based formatters.
+* characters in a number, for example the percent sign.
 * @param fmt The formatter to set.
 * @param symbol The UNumberFormatSymbol constant for the symbol to set
 * @param value The string to set the symbol to
@@ -842,7 +687,7 @@ unum_getSymbol(const UNumberFormat *fmt,
 * @see unum_getSymbol
 * @stable ICU 2.0
 */
-U_STABLE void U_EXPORT2
+U_CAPI void U_EXPORT2
 unum_setSymbol(UNumberFormat *fmt,
                UNumberFormatSymbol symbol,
                const UChar *value,
@@ -857,9 +702,9 @@ unum_setSymbol(UNumberFormat *fmt,
  * @param type type of the locale we're looking for (valid or actual) 
  * @param status error code for the operation
  * @return the locale name
- * @stable ICU 2.8
+ * @draft ICU 2.8 likely to change in ICU 3.0, based on feedback
  */
-U_STABLE const char* U_EXPORT2
+U_CAPI const char* U_EXPORT2
 unum_getLocaleByType(const UNumberFormat *fmt,
                      ULocDataLocaleType type,
                      UErrorCode* status); 
