@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 1997-2009, International Business Machines
+*   Copyright (C) 1997-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   Date        Name        Description
@@ -15,7 +15,6 @@
 
 #if !UCONFIG_NO_TRANSLITERATION
 
-#include "unicode/localpointer.h"
 #include "unicode/urep.h"
 #include "unicode/parseerr.h"
 #include "unicode/uenum.h"
@@ -77,16 +76,16 @@ typedef void* UTransliterator;
 typedef enum UTransDirection {
     
     /**
-     * UTRANS_FORWARD means from &lt;source&gt; to &lt;target&gt; for a
-     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
+     * UTRANS_FORWARD means from <source> to <target> for a
+     * transliterator with ID <source>-<target>.  For a transliterator
      * opened using a rule, it means forward direction rules, e.g.,
      * "A > B".
      */
     UTRANS_FORWARD,
 
     /**
-     * UTRANS_REVERSE means from &lt;target&gt; to &lt;source&gt; for a
-     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
+     * UTRANS_REVERSE means from <target> to <source> for a
+     * transliterator with ID <source>-<target>.  For a transliterator
      * opened using a rule, it means reverse direction rules, e.g.,
      * "A < B".
      */
@@ -124,7 +123,7 @@ typedef struct UTransPosition {
      * anything before this index.  INPUT/OUTPUT parameter: This parameter
      * is updated by a transliteration operation to reflect the maximum
      * amount of antecontext needed by a transliterator.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     int32_t contextStart;
     
@@ -134,7 +133,7 @@ typedef struct UTransPosition {
      * anything at or after this index.  INPUT/OUTPUT parameter: This
      * parameter is updated to reflect changes in the length of the
      * text, but points to the same logical position in the text.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     int32_t contextLimit;
     
@@ -143,7 +142,7 @@ typedef struct UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is advanced past
      * characters that have already been transliterated by a
      * transliteration operation.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     int32_t start;
     
@@ -152,7 +151,7 @@ typedef struct UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is updated to reflect
      * changes in the length of the text, but points to the same
      * logical position in the text.
-     * @stable ICU 2.4
+     * @draft ICU 2.4
      */
     int32_t limit;
 
@@ -183,9 +182,9 @@ typedef struct UTransPosition {
  * @param pErrorCode a pointer to the UErrorCode
  * @return a transliterator pointer that may be passed to other
  *         utrans_xxx() functions, or NULL if the open call fails.
- * @stable ICU 2.8
+ * @draft ICU 2.8
  */
-U_STABLE UTransliterator* U_EXPORT2
+U_CAPI UTransliterator* U_EXPORT2
 utrans_openU(const UChar *id,
              int32_t idLength,
              UTransDirection dir,
@@ -209,7 +208,7 @@ utrans_openU(const UChar *id,
  * inverse of trans, or NULL if the open call fails.
  * @stable ICU 2.0
  */
-U_STABLE UTransliterator* U_EXPORT2 
+U_CAPI UTransliterator* U_EXPORT2 
 utrans_openInverse(const UTransliterator* trans,
                    UErrorCode* status);
 
@@ -223,7 +222,7 @@ utrans_openInverse(const UTransliterator* trans,
  * utrans_xxx() functions, or NULL if the clone call fails.
  * @stable ICU 2.0
  */
-U_STABLE UTransliterator* U_EXPORT2 
+U_CAPI UTransliterator* U_EXPORT2 
 utrans_clone(const UTransliterator* trans,
              UErrorCode* status);
 
@@ -233,28 +232,9 @@ utrans_clone(const UTransliterator* trans,
  * @param trans the transliterator to be closed.
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_close(UTransliterator* trans);
 
-#if U_SHOW_CPLUSPLUS_API
-
-U_NAMESPACE_BEGIN
-
-/**
- * \class LocalUTransliteratorPointer
- * "Smart pointer" class, closes a UTransliterator via utrans_close().
- * For most methods see the LocalPointerBase base class.
- *
- * @see LocalPointerBase
- * @see LocalPointer
- * @draft ICU 4.4
- */
-U_DEFINE_LOCAL_OPEN_POINTER(LocalUTransliteratorPointer, UTransliterator, utrans_close);
-
-U_NAMESPACE_END
-
-#endif
-
 /**
  * Return the programmatic identifier for this transliterator.
  * If this identifier is passed to utrans_openU(), it will open
@@ -264,12 +244,11 @@ U_NAMESPACE_END
  * @param trans the transliterator to return the ID of.
  * @param resultLength pointer to an output variable receiving the length
  *        of the ID string; can be NULL
- * @return the NUL-terminated ID string. This pointer remains
- * valid until utrans_close() is called on this transliterator.
+ * @return the NUL-terminated ID string
  *
- * @stable ICU 2.8
+ * @draft ICU 2.8
  */
-U_STABLE const UChar * U_EXPORT2
+U_CAPI const UChar * U_EXPORT2
 utrans_getUnicodeID(const UTransliterator *trans,
                     int32_t *resultLength);
 
@@ -287,7 +266,7 @@ utrans_getUnicodeID(const UTransliterator *trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_register(UTransliterator* adoptedTrans,
                 UErrorCode* status);
 
@@ -298,9 +277,9 @@ utrans_register(UTransliterator* adoptedTrans,
  *
  * @param id an ID to unregister
  * @param idLength the length of id, or -1 if id is zero-terminated
- * @stable ICU 2.8
+ * @draft ICU 2.8
  */
-U_STABLE void U_EXPORT2
+U_CAPI void U_EXPORT2
 utrans_unregisterID(const UChar* id, int32_t idLength);
 
 /**
@@ -321,7 +300,7 @@ utrans_unregisterID(const UChar* id, int32_t idLength);
  * @see UnicodeSet
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_setFilter(UTransliterator* trans,
                  const UChar* filterPattern,
                  int32_t filterPatternLen,
@@ -334,7 +313,7 @@ utrans_setFilter(UTransliterator* trans,
  * @return the number of system transliterators.
  * @stable ICU 2.0
  */
-U_STABLE int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 utrans_countAvailableIDs(void);
 
 /**
@@ -344,9 +323,9 @@ utrans_countAvailableIDs(void);
  * @return UEnumeration for the available transliterators.
  *         Close with uenum_close().
  *
- * @stable ICU 2.8
+ * @draft ICU 2.8
  */
-U_STABLE UEnumeration * U_EXPORT2
+U_CAPI UEnumeration * U_EXPORT2
 utrans_openIDs(UErrorCode *pErrorCode);
 
 /********************************************************************
@@ -376,7 +355,7 @@ utrans_openIDs(UErrorCode *pErrorCode);
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_trans(const UTransliterator* trans,
              UReplaceable* rep,
              UReplaceableCallbacks* repFunc,
@@ -427,7 +406,7 @@ utrans_trans(const UTransliterator* trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_transIncremental(const UTransliterator* trans,
                         UReplaceable* rep,
                         UReplaceableCallbacks* repFunc,
@@ -465,7 +444,7 @@ utrans_transIncremental(const UTransliterator* trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_transUChars(const UTransliterator* trans,
                    UChar* text,
                    int32_t* textLength,
@@ -500,7 +479,7 @@ utrans_transUChars(const UTransliterator* trans,
  * @see utrans_transIncremental
  * @stable ICU 2.0
  */
-U_STABLE void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_transIncrementalUChars(const UTransliterator* trans,
                               UChar* text,
                               int32_t* textLength,
@@ -535,7 +514,7 @@ utrans_transIncrementalUChars(const UTransliterator* trans,
  * utrans_xxx() functions, or NULL if the open call fails.
  * @deprecated ICU 2.8 Use utrans_openU() instead, see utrans.h
  */
-U_DEPRECATED UTransliterator* U_EXPORT2 
+U_CAPI UTransliterator* U_EXPORT2 
 utrans_open(const char* id,
             UTransDirection dir,
             const UChar* rules,         /* may be Null */
@@ -558,7 +537,7 @@ utrans_open(const char* id,
  * zero-termination.  This may be greater than bufCapacity.
  * @deprecated ICU 2.8 Use utrans_getUnicodeID() instead, see utrans.h
  */
-U_DEPRECATED int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 utrans_getID(const UTransliterator* trans,
              char* buf,
              int32_t bufCapacity);
@@ -572,7 +551,7 @@ utrans_getID(const UTransliterator* trans,
  * @param id a zero-terminated ID
  * @deprecated ICU 2.8 Use utrans_unregisterID() instead, see utrans.h
  */
-U_DEPRECATED void U_EXPORT2 
+U_CAPI void U_EXPORT2 
 utrans_unregister(const char* id);
 
 /**
@@ -593,7 +572,7 @@ utrans_unregister(const char* id);
  * zero-termination.  This may be greater than bufCapacity.
  * @deprecated ICU 2.8 Use utrans_openIDs() instead, see utrans.h
  */
-U_DEPRECATED int32_t U_EXPORT2 
+U_CAPI int32_t U_EXPORT2 
 utrans_getAvailableID(int32_t index,
                       char* buf,
                       int32_t bufCapacity);
