@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 1996-2004, International Business Machines
+*   Copyright (C) 1996-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 * Modification History:
@@ -23,8 +23,6 @@
 #include "unicode/ustring.h"
 #include "unicode/fmtable.h"
 #include "unicode/dcfmtsym.h"
-#include "unicode/curramt.h"
-#include "uassert.h"
 #include "cpputils.h"
 
 
@@ -39,109 +37,96 @@ unum_open(  UNumberFormatStyle    style,
             UParseError*       parseErr,
             UErrorCode*        status)
 {
-
+    
     if(U_FAILURE(*status))
-    {
+    { 
         return 0;
     }
-
-    UNumberFormat *retVal = 0;
-
-    switch(style) {
-    case UNUM_DECIMAL:
-        if(locale == 0)
-            retVal = (UNumberFormat*)NumberFormat::createInstance(*status);
-        else
-            retVal = (UNumberFormat*)NumberFormat::createInstance(Locale(locale),
-            *status);
-        break;
-
-    case UNUM_CURRENCY:
-        if(locale == 0)
-            retVal = (UNumberFormat*)NumberFormat::createCurrencyInstance(*status);
-        else
-            retVal = (UNumberFormat*)NumberFormat::createCurrencyInstance(Locale(locale),
-            *status);
-        break;
-
-    case UNUM_PERCENT:
-        if(locale == 0)
-            retVal = (UNumberFormat*)NumberFormat::createPercentInstance(*status);
-        else
-            retVal = (UNumberFormat*)NumberFormat::createPercentInstance(Locale(locale),
-            *status);
-        break;
-
-    case UNUM_SCIENTIFIC:
-        if(locale == 0)
-            retVal = (UNumberFormat*)NumberFormat::createScientificInstance(*status);
-        else
-            retVal = (UNumberFormat*)NumberFormat::createScientificInstance(Locale(locale),
-            *status);
-        break;
-
-    case UNUM_PATTERN_DECIMAL: {
+    if(style!=UNUM_IGNORE){
+        UNumberFormat *retVal = 0;
+        
+        switch(style) {
+        case UNUM_DECIMAL:
+            if(locale == 0)
+                retVal = (UNumberFormat*)NumberFormat::createInstance(*status);
+            else
+                retVal = (UNumberFormat*)NumberFormat::createInstance(Locale(locale),
+                *status);
+            break;
+            
+        case UNUM_CURRENCY:
+            if(locale == 0)
+                retVal = (UNumberFormat*)NumberFormat::createCurrencyInstance(*status);
+            else
+                retVal = (UNumberFormat*)NumberFormat::createCurrencyInstance(Locale(locale),
+                *status);
+            break;
+            
+        case UNUM_PERCENT:
+            if(locale == 0)
+                retVal = (UNumberFormat*)NumberFormat::createPercentInstance(*status);
+            else
+                retVal = (UNumberFormat*)NumberFormat::createPercentInstance(Locale(locale),
+                *status);
+            break;
+            
+        case UNUM_SCIENTIFIC:
+            if(locale == 0)
+                retVal = (UNumberFormat*)NumberFormat::createScientificInstance(*status);
+            else
+                retVal = (UNumberFormat*)NumberFormat::createScientificInstance(Locale(locale),
+                *status);
+            break;
+            
+        case UNUM_SPELLOUT:
+#if U_HAVE_RBNF
+            return (UNumberFormat*)new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale(locale), *status);
+#else
+            // fall through
+#endif
+        default:
+            *status = U_UNSUPPORTED_ERROR;
+            return 0;
+        }
+        
+        if(retVal == 0) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
+            return 0;
+        }
+        
+        return retVal;
+    }else{
+        /* we don't support RBNF patterns yet */
         UParseError tErr;
         /* UnicodeString can handle the case when patternLength = -1. */
         const UnicodeString pat(pattern, patternLength);
         DecimalFormatSymbols *syms = 0;
-
+        
         if(parseErr==NULL){
             parseErr = &tErr;
         }
-
+        
         if(locale == 0)
             syms = new DecimalFormatSymbols(*status);
         else
-            syms = new DecimalFormatSymbols(Locale(locale), *status);
-
+            syms = new DecimalFormatSymbols(Locale(locale),
+            *status);
+        
         if(syms == 0) {
             *status = U_MEMORY_ALLOCATION_ERROR;
             return 0;
         }
-
-        retVal = (UNumberFormat*)new DecimalFormat(pat, syms, *parseErr, *status);
-        if(retVal == 0) {
+        
+        DecimalFormat *fmt = 0;
+        fmt = new DecimalFormat(pat, syms, *parseErr, *status);
+        if(fmt == 0) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
             delete syms;
-        }
-                               } break;
-
-#if U_HAVE_RBNF
-    case UNUM_PATTERN_RULEBASED: {
-        UParseError tErr;
-        /* UnicodeString can handle the case when patternLength = -1. */
-        const UnicodeString pat(pattern, patternLength);
-        
-        if(parseErr==NULL){
-            parseErr = &tErr;
+            return 0;
         }
         
-        retVal = (UNumberFormat*)new RuleBasedNumberFormat(pat, Locale(locale), *parseErr, *status);
-    } break;
-
-    case UNUM_SPELLOUT:
-        retVal = (UNumberFormat*)new RuleBasedNumberFormat(URBNF_SPELLOUT, Locale(locale), *status);
-        break;
-
-    case UNUM_ORDINAL:
-        retVal = (UNumberFormat*)new RuleBasedNumberFormat(URBNF_ORDINAL, Locale(locale), *status);
-        break;
-
-    case UNUM_DURATION:
-        retVal = (UNumberFormat*)new RuleBasedNumberFormat(URBNF_DURATION, Locale(locale), *status);
-        break;
-#endif
-
-    default:
-        *status = U_UNSUPPORTED_ERROR;
-        return 0;
+        return (UNumberFormat*) fmt;
     }
-
-    if(retVal == 0 && U_SUCCESS(*status)) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-    }
-
-    return retVal;
 }
 
 U_CAPI void U_EXPORT2
@@ -157,14 +142,8 @@ unum_clone(const UNumberFormat *fmt,
     if(U_FAILURE(*status))
         return 0;
     
-    Format *res = 0;
-    if (((const NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-        res = ((const DecimalFormat*)fmt)->clone();
-    } else {
-        U_ASSERT(((const NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-        res = ((const RuleBasedNumberFormat*)fmt)->clone();
-    }
-
+    Format *res = ((DecimalFormat*)fmt)->clone();
+    
     if(res == 0) {
         *status = U_MEMORY_ALLOCATION_ERROR;
         return 0;
@@ -207,7 +186,7 @@ unum_formatInt64(const UNumberFormat* fmt,
     if(pos != 0)
         fp.setField(pos->field);
     
-    ((const NumberFormat*)fmt)->format(number, res, fp);
+    ((NumberFormat*)fmt)->format(number, res, fp);
     
     if(pos != 0) {
         pos->beginIndex = fp.getBeginIndex();
@@ -240,7 +219,7 @@ unum_formatDouble(    const    UNumberFormat*  fmt,
   if(pos != 0)
     fp.setField(pos->field);
   
-  ((const NumberFormat*)fmt)->format(number, res, fp);
+  ((NumberFormat*)fmt)->format(number, res, fp);
   
   if(pos != 0) {
     pos->beginIndex = fp.getBeginIndex();
@@ -250,47 +229,13 @@ unum_formatDouble(    const    UNumberFormat*  fmt,
   return res.extract(result, resultLength, *status);
 }
 
-U_DRAFT int32_t U_EXPORT2 
-unum_formatDoubleCurrency(const UNumberFormat* fmt,
-                          double number,
-                          UChar* currency,
-                          UChar* result,
-                          int32_t resultLength,
-                          UFieldPosition* pos, /* ignored if 0 */
-                          UErrorCode* status) {
-    if (U_FAILURE(*status)) return -1;
-
-    UnicodeString res;
-    if (!(result==NULL && resultLength==0)) {
-        // NULL destination for pure preflighting: empty dummy string
-        // otherwise, alias the destination buffer
-        res.setTo(result, 0, resultLength);
-    }
-    
-    FieldPosition fp;
-    if (pos != 0) {
-        fp.setField(pos->field);
-    }
-
-    Formattable n(new CurrencyAmount(number, currency, *status));
-    ((const NumberFormat*)fmt)->format(n, res, fp, *status);
-    
-    if (pos != 0) {
-        pos->beginIndex = fp.getBeginIndex();
-        pos->endIndex = fp.getEndIndex();
-    }
-  
-    return res.extract(result, resultLength, *status);
-}
-
 static void
 parseRes(Formattable& res,
-         const   UNumberFormat*  fmt,
-         const   UChar*          text,
-         int32_t         textLength,
-         int32_t         *parsePos /* 0 = start */,
-         UBool parseCurrency,
-         UErrorCode      *status)
+                const   UNumberFormat*  fmt,
+        const   UChar*          text,
+        int32_t         textLength,
+        int32_t         *parsePos /* 0 = start */,
+        UErrorCode      *status)
 {
     if(U_FAILURE(*status))
         return;
@@ -302,11 +247,7 @@ parseRes(Formattable& res,
     if(parsePos != 0)
         pp.setIndex(*parsePos);
     
-    if (parseCurrency) {
-        ((const NumberFormat*)fmt)->parseCurrency(src, res, pp);
-    } else {
-        ((const NumberFormat*)fmt)->parse(src, res, pp);
-    }
+    ((NumberFormat*)fmt)->parse(src, res, pp);
     
     if(parsePos != 0) {
         if(pp.getErrorIndex() == -1)
@@ -326,8 +267,8 @@ unum_parse(    const   UNumberFormat*  fmt,
         UErrorCode      *status)
 {
     Formattable res;
-    parseRes(res, fmt, text, textLength, parsePos, FALSE, status);
-    return res.getLong(*status);
+    parseRes(res, fmt, text, textLength, parsePos, status);
+    return res.getLong(status);
 }
 
 U_CAPI int64_t U_EXPORT2
@@ -338,8 +279,8 @@ unum_parseInt64(    const   UNumberFormat*  fmt,
         UErrorCode      *status)
 {
     Formattable res;
-    parseRes(res, fmt, text, textLength, parsePos, FALSE, status);
-    return res.getInt64(*status);
+    parseRes(res, fmt, text, textLength, parsePos, status);
+    return res.getInt64(status);
 }
 
 U_CAPI double U_EXPORT2
@@ -350,26 +291,8 @@ unum_parseDouble(    const   UNumberFormat*  fmt,
             UErrorCode      *status)
 {
     Formattable res;
-    parseRes(res, fmt, text, textLength, parsePos, FALSE, status);
-    return res.getDouble(*status);
-}
-
-U_DRAFT double U_EXPORT2
-unum_parseDoubleCurrency(const UNumberFormat* fmt,
-                         const UChar* text,
-                         int32_t textLength,
-                         int32_t* parsePos, /* 0 = start */
-                         UChar* currency,
-                         UErrorCode* status) {
-    Formattable res;
-    parseRes(res, fmt, text, textLength, parsePos, TRUE, status);
-    currency[0] = 0;
-    if (res.getType() == Formattable::kObject &&
-        res.getObject()->getDynamicClassID() == CurrencyAmount::getStaticClassID()) {
-        const CurrencyAmount* c = (const CurrencyAmount*) res.getObject();
-        u_strcpy(currency, c->getISOCurrency());
-    }
-    return res.getDouble(*status);
+    parseRes(res, fmt, text, textLength, parsePos, status);
+    return res.getDouble(status);
 }
 
 U_CAPI const char* U_EXPORT2
@@ -388,79 +311,56 @@ U_CAPI int32_t U_EXPORT2
 unum_getAttribute(const UNumberFormat*          fmt,
           UNumberFormatAttribute  attr)
 {
-  if (((const NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-    const DecimalFormat* df = (const DecimalFormat*) fmt;
     switch(attr) {
     case UNUM_PARSE_INT_ONLY:
-        return df->isParseIntegerOnly();
+        return ((NumberFormat*)fmt)->isParseIntegerOnly();
         
     case UNUM_GROUPING_USED:
-        return df->isGroupingUsed();
+        return ((NumberFormat*)fmt)->isGroupingUsed();
         
     case UNUM_DECIMAL_ALWAYS_SHOWN:
-        return df->isDecimalSeparatorAlwaysShown();    
+        return ((DecimalFormat*)fmt)->isDecimalSeparatorAlwaysShown();    
         
     case UNUM_MAX_INTEGER_DIGITS:
-        return df->getMaximumIntegerDigits();
+        return ((NumberFormat*)fmt)->getMaximumIntegerDigits();
         
     case UNUM_MIN_INTEGER_DIGITS:
-        return df->getMinimumIntegerDigits();
+        return ((NumberFormat*)fmt)->getMinimumIntegerDigits();
         
     case UNUM_INTEGER_DIGITS:
         // TBD: what should this return?
-        return df->getMinimumIntegerDigits();
+        return ((NumberFormat*)fmt)->getMinimumIntegerDigits();
         
     case UNUM_MAX_FRACTION_DIGITS:
-        return df->getMaximumFractionDigits();
+        return ((NumberFormat*)fmt)->getMaximumFractionDigits();
         
     case UNUM_MIN_FRACTION_DIGITS:
-        return df->getMinimumFractionDigits();
+        return ((NumberFormat*)fmt)->getMinimumFractionDigits();
         
     case UNUM_FRACTION_DIGITS:
         // TBD: what should this return?
-        return df->getMinimumFractionDigits();
-        
-    case UNUM_SIGNIFICANT_DIGITS_USED:
-        return df->areSignificantDigitsUsed();
-        
-    case UNUM_MAX_SIGNIFICANT_DIGITS:
-        return df->getMaximumSignificantDigits();
-        
-    case UNUM_MIN_SIGNIFICANT_DIGITS:
-        return df->getMinimumSignificantDigits();
+        return ((NumberFormat*)fmt)->getMinimumFractionDigits();
         
     case UNUM_MULTIPLIER:
-        return df->getMultiplier();    
+        return ((DecimalFormat*)fmt)->getMultiplier();    
         
     case UNUM_GROUPING_SIZE:
-        return df->getGroupingSize();    
+        return ((DecimalFormat*)fmt)->getGroupingSize();    
         
     case UNUM_ROUNDING_MODE:
-        return df->getRoundingMode();
+        return ((DecimalFormat*)fmt)->getRoundingMode();
         
     case UNUM_FORMAT_WIDTH:
-        return df->getFormatWidth();
+        return ((DecimalFormat*)fmt)->getFormatWidth();
         
+        /** The position at which padding will take place. */
     case UNUM_PADDING_POSITION:
-        return df->getPadPosition();
-        
-    case UNUM_SECONDARY_GROUPING_SIZE:
-        return df->getSecondaryGroupingSize();
+        return ((DecimalFormat*)fmt)->getPadPosition();
         
     default:
-        /* enums out of sync? unsupported enum? */
         break;
     }
-  } else {
-    U_ASSERT(((const NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-    if (attr == UNUM_LENIENT_PARSE) {
-#if !UCONFIG_NO_COLLATION
-      return ((const RuleBasedNumberFormat*)fmt)->isLenient();
-#endif
-    }
-  }
-
-  return -1;
+    return -1;
 }
 
 U_CAPI void U_EXPORT2
@@ -468,105 +368,82 @@ unum_setAttribute(    UNumberFormat*          fmt,
             UNumberFormatAttribute  attr,
             int32_t                 newValue)
 {
-  if (((NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-    DecimalFormat* df = (DecimalFormat*) fmt;
     switch(attr) {
     case UNUM_PARSE_INT_ONLY:
-        df->setParseIntegerOnly(newValue!=0);
+        ((NumberFormat*)fmt)->setParseIntegerOnly((UBool)newValue);
         break;
         
     case UNUM_GROUPING_USED:
-        df->setGroupingUsed(newValue!=0);
+        ((NumberFormat*)fmt)->setGroupingUsed((UBool)newValue);
         break;
         
     case UNUM_DECIMAL_ALWAYS_SHOWN:
-        df->setDecimalSeparatorAlwaysShown(newValue!=0);
+        ((DecimalFormat*)fmt)->setDecimalSeparatorAlwaysShown((UBool)newValue);
         break;
         
     case UNUM_MAX_INTEGER_DIGITS:
-        df->setMaximumIntegerDigits(newValue);
+        ((NumberFormat*)fmt)->setMaximumIntegerDigits(newValue);
         break;
         
     case UNUM_MIN_INTEGER_DIGITS:
-        df->setMinimumIntegerDigits(newValue);
+        ((NumberFormat*)fmt)->setMinimumIntegerDigits(newValue);
         break;
         
     case UNUM_INTEGER_DIGITS:
-        df->setMinimumIntegerDigits(newValue);
-        df->setMaximumIntegerDigits(newValue);
+        ((NumberFormat*)fmt)->setMinimumIntegerDigits(newValue);
+        ((NumberFormat*)fmt)->setMaximumIntegerDigits(newValue);
         break;
         
     case UNUM_MAX_FRACTION_DIGITS:
-        df->setMaximumFractionDigits(newValue);
+        ((NumberFormat*)fmt)->setMaximumFractionDigits(newValue);
         break;
         
     case UNUM_MIN_FRACTION_DIGITS:
-        df->setMinimumFractionDigits(newValue);
+        ((NumberFormat*)fmt)->setMinimumFractionDigits(newValue);
         break;
         
     case UNUM_FRACTION_DIGITS:
-        df->setMinimumFractionDigits(newValue);
-        df->setMaximumFractionDigits(newValue);
-        break;
-        
-    case UNUM_SIGNIFICANT_DIGITS_USED:
-        df->setSignificantDigitsUsed(newValue!=0);
-        break;
-
-    case UNUM_MAX_SIGNIFICANT_DIGITS:
-        df->setMaximumSignificantDigits(newValue);
-        break;
-        
-    case UNUM_MIN_SIGNIFICANT_DIGITS:
-        df->setMinimumSignificantDigits(newValue);
+        ((NumberFormat*)fmt)->setMinimumFractionDigits(newValue);
+        ((NumberFormat*)fmt)->setMaximumFractionDigits(newValue);
         break;
         
     case UNUM_MULTIPLIER:
-        df->setMultiplier(newValue);    
+        ((DecimalFormat*)fmt)->setMultiplier(newValue);    
         break;
         
     case UNUM_GROUPING_SIZE:
-        df->setGroupingSize(newValue);    
+        ((DecimalFormat*)fmt)->setGroupingSize(newValue);    
         break;
         
     case UNUM_ROUNDING_MODE:
-        df->setRoundingMode((DecimalFormat::ERoundingMode)newValue);
+        ((DecimalFormat*)fmt)->setRoundingMode((DecimalFormat::ERoundingMode)newValue);
         break;
         
     case UNUM_FORMAT_WIDTH:
-        df->setFormatWidth(newValue);
+        ((DecimalFormat*)fmt)->setFormatWidth(newValue);
         break;
         
-    case UNUM_PADDING_POSITION:
         /** The position at which padding will take place. */
-        df->setPadPosition((DecimalFormat::EPadPosition)newValue);
+    case UNUM_PADDING_POSITION:
+        ((DecimalFormat*)fmt)->setPadPosition((DecimalFormat::EPadPosition)newValue);
         break;
         
     case UNUM_SECONDARY_GROUPING_SIZE:
-        df->setSecondaryGroupingSize(newValue);
+        ((DecimalFormat*)fmt)->setSecondaryGroupingSize(newValue);
         break;
         
     default:
         /* Shouldn't get here anyway */
         break;
     }
-  } else {
-    U_ASSERT(((NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-    if (attr == UNUM_LENIENT_PARSE) {
-#if !UCONFIG_NO_COLLATION
-      ((RuleBasedNumberFormat*)fmt)->setLenient((UBool)newValue);
-#endif
-    }
-  }
 }
 
 U_CAPI double U_EXPORT2
 unum_getDoubleAttribute(const UNumberFormat*          fmt,
           UNumberFormatAttribute  attr)
 {
-    if (((const NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID() && 
-    attr == UNUM_ROUNDING_INCREMENT) {
-        return ((const DecimalFormat*)fmt)->getRoundingIncrement();
+    if (attr == UNUM_ROUNDING_INCREMENT) {
+        return ((DecimalFormat*)fmt)->getRoundingIncrement();
     } else {
         return -1.0;
     }
@@ -577,8 +454,7 @@ unum_setDoubleAttribute(    UNumberFormat*          fmt,
             UNumberFormatAttribute  attr,
             double                 newValue)
 {
-    if (((NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID() && 
-    attr == UNUM_ROUNDING_INCREMENT) {   
+    if (attr == UNUM_ROUNDING_INCREMENT) {   
         ((DecimalFormat*)fmt)->setRoundingIncrement(newValue);
     }
 }
@@ -592,62 +468,44 @@ unum_getTextAttribute(const UNumberFormat*  fmt,
 {
     if(U_FAILURE(*status))
         return -1;
-
+    
     UnicodeString res;
     if(!(result==NULL && resultLength==0)) {
         // NULL destination for pure preflighting: empty dummy string
         // otherwise, alias the destination buffer
         res.setTo(result, 0, resultLength);
     }
-
-    if (((const NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-        const DecimalFormat* df = (const DecimalFormat*) fmt;
-        switch(tag) {
-        case UNUM_POSITIVE_PREFIX:
-            df->getPositivePrefix(res);
-            break;
-
-        case UNUM_POSITIVE_SUFFIX:
-            df->getPositiveSuffix(res);
-            break;
-
-        case UNUM_NEGATIVE_PREFIX:
-            df->getNegativePrefix(res);
-            break;
-
-        case UNUM_NEGATIVE_SUFFIX:
-            df->getNegativeSuffix(res);
-            break;
-
-        case UNUM_PADDING_CHARACTER:
-            res = df->getPadCharacterString();
-            break;
-
-        case UNUM_CURRENCY_CODE:
-            res = UnicodeString(df->getCurrency());
-            break;
-
-        default:
-            *status = U_UNSUPPORTED_ERROR;
-            return -1;
-        }
-    } else {
-        U_ASSERT(((const NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-        const RuleBasedNumberFormat* rbnf = (const RuleBasedNumberFormat*)fmt;
-        if (tag == UNUM_DEFAULT_RULESET) {
-            res = rbnf->getDefaultRuleSetName();
-        } else if (tag == UNUM_PUBLIC_RULESETS) {
-            int32_t count = rbnf->getNumberOfRuleSetNames();
-            for (int i = 0; i < count; ++i) {
-                res += rbnf->getRuleSetName(i);
-                res += (UChar)0x003b; // semicolon
-            }
-        } else {
-            *status = U_UNSUPPORTED_ERROR;
-            return -1;
-        }
+    
+    switch(tag) {
+    case UNUM_POSITIVE_PREFIX:
+        ((DecimalFormat*)fmt)->getPositivePrefix(res);
+        break;
+        
+    case UNUM_POSITIVE_SUFFIX:
+        ((DecimalFormat*)fmt)->getPositiveSuffix(res);
+        break;
+        
+    case UNUM_NEGATIVE_PREFIX:
+        ((DecimalFormat*)fmt)->getNegativePrefix(res);
+        break;
+        
+    case UNUM_NEGATIVE_SUFFIX:
+        ((DecimalFormat*)fmt)->getNegativeSuffix(res);
+        break;
+        
+    case UNUM_PADDING_CHARACTER:
+        res = ((DecimalFormat*)fmt)->getPadCharacterString();
+        break;
+        
+    case UNUM_CURRENCY_CODE:
+        res = UnicodeString(((DecimalFormat*)fmt)->getCurrency());
+        break;
+        
+    default:
+        *status = U_UNSUPPORTED_ERROR;
+        return -1;
     }
-
+    
     return res.extract(result, resultLength, *status);
 }
 
@@ -664,44 +522,34 @@ unum_setTextAttribute(    UNumberFormat*                    fmt,
     int32_t len = (newValueLength == -1 ? u_strlen(newValue) : newValueLength);
     const UnicodeString val((UChar*)newValue, len, len);
     
-    if (((NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-      DecimalFormat* df = (DecimalFormat*) fmt;
-      switch(tag) {
-      case UNUM_POSITIVE_PREFIX:
-        df->setPositivePrefix(val);
+    switch(tag) {
+    case UNUM_POSITIVE_PREFIX:
+        ((DecimalFormat*)fmt)->setPositivePrefix(val);
         break;
         
-      case UNUM_POSITIVE_SUFFIX:
-        df->setPositiveSuffix(val);
+    case UNUM_POSITIVE_SUFFIX:
+        ((DecimalFormat*)fmt)->setPositiveSuffix(val);
         break;
         
-      case UNUM_NEGATIVE_PREFIX:
-        df->setNegativePrefix(val);
+    case UNUM_NEGATIVE_PREFIX:
+        ((DecimalFormat*)fmt)->setNegativePrefix(val);
         break;
         
-      case UNUM_NEGATIVE_SUFFIX:
-        df->setNegativeSuffix(val);
+    case UNUM_NEGATIVE_SUFFIX:
+        ((DecimalFormat*)fmt)->setNegativeSuffix(val);
         break;
         
-      case UNUM_PADDING_CHARACTER:
-        df->setPadCharacter(*newValue);
+    case UNUM_PADDING_CHARACTER:
+        ((DecimalFormat*)fmt)->setPadCharacter(*newValue);
         break;
         
-      case UNUM_CURRENCY_CODE:
-        df->setCurrency(newValue, *status);
+    case UNUM_CURRENCY_CODE:
+        ((DecimalFormat*)fmt)->setCurrency(newValue);
         break;
         
-      default:
+    default:
         *status = U_UNSUPPORTED_ERROR;
         break;
-      }
-    } else {
-      U_ASSERT(((NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-      if (tag == UNUM_DEFAULT_RULESET) {
-    ((RuleBasedNumberFormat*)fmt)->setDefaultRuleSet(newValue, *status);
-      } else {
-    *status = U_UNSUPPORTED_ERROR;
-      }
     }
 }
 
@@ -721,22 +569,17 @@ unum_toPattern(    const    UNumberFormat*          fmt,
         // otherwise, alias the destination buffer
         pat.setTo(result, 0, resultLength);
     }
-
-    if (((const NumberFormat*)fmt)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-      const DecimalFormat* df = (const DecimalFormat*) fmt;
-      if(isPatternLocalized)
-        df->toLocalizedPattern(pat);
-      else
-        df->toPattern(pat);
-    } else {
-      U_ASSERT(((const NumberFormat*)fmt)->getDynamicClassID() == RuleBasedNumberFormat::getStaticClassID());
-      pat = ((const RuleBasedNumberFormat*)fmt)->getRules();
-    }
+    
+    if(isPatternLocalized)
+        ((DecimalFormat*)fmt)->toLocalizedPattern(pat);
+    else
+        ((DecimalFormat*)fmt)->toPattern(pat);
+    
     return pat.extract(result, resultLength, *status);
 }
 
 U_CAPI int32_t U_EXPORT2
-unum_getSymbol(const UNumberFormat *fmt,
+unum_getSymbol(UNumberFormat *fmt,
                UNumberFormatSymbol symbol,
                UChar *buffer,
                int32_t size,
@@ -751,11 +594,6 @@ unum_getSymbol(const UNumberFormat *fmt,
         return 0;
     }
 
-    if (((const NumberFormat*)fmt)->getDynamicClassID() != DecimalFormat::getStaticClassID()) {
-      *status = U_UNSUPPORTED_ERROR;
-      return 0;
-    }
-
     return ((const DecimalFormat *)fmt)->
       getDecimalFormatSymbols()->
         getConstSymbol((DecimalFormatSymbols::ENumberFormatSymbol)symbol).
@@ -778,11 +616,6 @@ unum_setSymbol(UNumberFormat *fmt,
         return;
     }
     
-    if (((NumberFormat*)fmt)->getDynamicClassID() != DecimalFormat::getStaticClassID()) {
-      *status = U_UNSUPPORTED_ERROR;
-      return;
-    }
-
     DecimalFormatSymbols symbols(*((DecimalFormat *)fmt)->getDecimalFormatSymbols());
     symbols.setSymbol((DecimalFormatSymbols::ENumberFormatSymbol)symbol,
         UnicodeString(value, length));  /* UnicodeString can handle the case when length = -1. */
@@ -812,16 +645,15 @@ unum_applyPattern(  UNumberFormat  *format,
     const UnicodeString pat((UChar*)pattern, len, len);
     
     // Verify if the object passed is a DecimalFormat object
-    if (((NumberFormat*)format)->getDynamicClassID() == DecimalFormat::getStaticClassID()) {
-      if(localized) {
-        ((DecimalFormat*)format)->applyLocalizedPattern(pat,*parseError, *status);
-      } else {
-        ((DecimalFormat*)format)->applyPattern(pat,*parseError, *status);
-      }
-    } else {
-      *status = U_UNSUPPORTED_ERROR;
-      return;
+    if(((NumberFormat*)format)->getDynamicClassID()!= DecimalFormat::getStaticClassID()){
+        *status = U_ILLEGAL_ARGUMENT_ERROR;
+        return;
     }
+    
+    if(localized)
+        ((DecimalFormat*)format)->applyLocalizedPattern(pat,*parseError, *status);
+    else
+        ((DecimalFormat*)format)->applyPattern(pat,*parseError, *status);
 }
 
 U_CAPI const char* U_EXPORT2
@@ -835,7 +667,7 @@ unum_getLocaleByType(const UNumberFormat *fmt,
         }
         return NULL;
     }
-    return ((const Format*)fmt)->getLocaleID(type, *status);
+    return ((Format*)fmt)->getLocaleID(type, *status);
 }
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
