@@ -1,6 +1,7 @@
 /*
+ * @(#)AnchorTables.cpp	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2005 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -41,10 +42,9 @@ void AnchorTable::getAnchor(LEGlyphID glyphID, const LEFontInstance *fontInstanc
     }
 
     default:
-        // unknown format: just use x, y coordinate, like format 1...
-        const Format1AnchorTable *f1 = (const Format1AnchorTable *) this;
-
-        f1->getAnchor(fontInstance, anchor);
+		// Unknown format, set the anchor point to (0, 0)
+		anchor.fX = 0;
+		anchor.fY = 0;
         break;
     }
 }
