@@ -1,7 +1,7 @@
 /*
  * @(#)ArabicShaping.cpp    1.10 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -64,26 +64,53 @@ ArabicShaping::ShapeType ArabicShaping::getShapeType(LEUnicode c)
     return ST_NOSHAPE_NONE;
 }
 
-const LETag GlyphShaper::isolFeatureTag  = 0x69736F6C; // 'isol'
-const LETag GlyphShaper::initFeatureTag  = 0x696E6974; // 'init'
-const LETag GlyphShaper::mediFeatureTag  = 0x6D656469; // 'medi'
-const LETag GlyphShaper::finaFeatureTag  = 0x66696E61; // 'fina'
-const LETag GlyphShaper::ligaFeatureTag  = 0x6C696761; // 'liga'
-const LETag GlyphShaper::msetFeatureTag  = 0x6D736574; // 'mset'
-const LETag GlyphShaper::markFeatureTag  = 0x6D61726B; // 'mark'
+const LETag isolFeatureTag = LE_ISOL_FEATURE_TAG;
+const LETag initFeatureTag = LE_INIT_FEATURE_TAG;
+const LETag mediFeatureTag = LE_MEDI_FEATURE_TAG;
+const LETag finaFeatureTag = LE_FINA_FEATURE_TAG;
+const LETag ligaFeatureTag = LE_LIGA_FEATURE_TAG;
+const LETag msetFeatureTag = LE_MSET_FEATURE_TAG;
+const LETag markFeatureTag = LE_MARK_FEATURE_TAG;
+const LETag ccmpFeatureTag = LE_CCMP_FEATURE_TAG;
+const LETag rligFeatureTag = LE_RLIG_FEATURE_TAG;
+const LETag caltFeatureTag = LE_CALT_FEATURE_TAG;
+const LETag dligFeatureTag = LE_DLIG_FEATURE_TAG;
+const LETag cswhFeatureTag = LE_CSWH_FEATURE_TAG;
+const LETag cursFeatureTag = LE_CURS_FEATURE_TAG;
+const LETag kernFeatureTag = LE_KERN_FEATURE_TAG;
+const LETag mkmkFeatureTag = LE_MKMK_FEATURE_TAG;
 
-const LETag GlyphShaper::emptyTag        = 0x00000000; // ''
+const LETag emptyTag       = 0x00000000; // ''
+
+const LETag featureOrder[] = 
+{
+    ccmpFeatureTag, isolFeatureTag, finaFeatureTag, mediFeatureTag, initFeatureTag, rligFeatureTag,
+    caltFeatureTag, ligaFeatureTag, dligFeatureTag, cswhFeatureTag, msetFeatureTag, cursFeatureTag,
+    kernFeatureTag, markFeatureTag, mkmkFeatureTag, emptyTag
+};
 
 const LETag GlyphShaper::tagArray[] =
 {
-    isolFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, emptyTag,
-    finaFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, emptyTag,
-    initFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, emptyTag,
-    mediFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, emptyTag
+    isolFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, ccmpFeatureTag, rligFeatureTag,
+        caltFeatureTag, dligFeatureTag, cswhFeatureTag, cursFeatureTag, kernFeatureTag, mkmkFeatureTag, emptyTag,
+
+    finaFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, ccmpFeatureTag, rligFeatureTag,
+        caltFeatureTag, dligFeatureTag, cswhFeatureTag, cursFeatureTag, kernFeatureTag, mkmkFeatureTag, emptyTag,
+
+    initFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, ccmpFeatureTag, rligFeatureTag,
+        caltFeatureTag, dligFeatureTag, cswhFeatureTag, cursFeatureTag, kernFeatureTag, mkmkFeatureTag, emptyTag,
+
+    mediFeatureTag, ligaFeatureTag, msetFeatureTag, markFeatureTag, ccmpFeatureTag, rligFeatureTag,
+        caltFeatureTag, dligFeatureTag, cswhFeatureTag, cursFeatureTag, kernFeatureTag, mkmkFeatureTag, emptyTag
 };
 
 #define TAGS_PER_GLYPH (sizeof GlyphShaper::tagArray / sizeof(LETag) / 4)
 
+const LETag *ArabicShaping::getFeatureOrder()
+{
+    return featureOrder;
+}
+
 void ArabicShaping::shape(const LEUnicode *chars, le_int32 offset, le_int32 charCount, le_int32 charMax,
                           le_bool rightToLeft, Shaper &shaper)
 {
@@ -123,7 +150,7 @@ void ArabicShaping::shape(const LEUnicode *chars, le_int32 offset, le_int32 char
 
     // erout is effective right logical index
     le_int32 erout = -1;
-    le_bool rightShapes = false;
+    le_bool rightShapes = FALSE;
     le_bool rightCauses = (rightType & MASK_SHAPE_LEFT) != 0;
     le_int32 in, e, out = 0, dir = 1;
 
@@ -176,7 +203,7 @@ GlyphShaper::~GlyphShaper()
     // nothing to do
 }
 
-void GlyphShaper::init(LEUnicode ch, le_int32 outIndex, le_bool isloate)
+void GlyphShaper::init(LEUnicode /*ch*/, le_int32 outIndex, le_bool /*isloate*/)
 {
     charTags[outIndex] = tagArray;
 }
