@@ -1,13 +1,18 @@
 /*
  * @(#)ClassDefinitionTables.h	1.5 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
 #ifndef __CLASSDEFINITIONTABLES_H
 #define __CLASSDEFINITIONTABLES_H
 
+/**
+ * \file
+ * \internal
+ */
+
 #include "LETypes.h"
 #include "OpenTypeTables.h"
 
@@ -23,7 +28,7 @@ struct ClassDefinitionTable
 
 struct ClassDefFormat1Table : ClassDefinitionTable
 {
-    LEGlyphID  startGlyph;
+    TTGlyphID  startGlyph;
     le_uint16  glyphCount;
     le_uint16  classValueArray[ANY_NUMBER];
 
@@ -33,8 +38,8 @@ struct ClassDefFormat1Table : ClassDefinitionTable
 
 struct ClassRangeRecord
 {
-    LEGlyphID start;
-    LEGlyphID end;
+    TTGlyphID start;
+    TTGlyphID end;
     le_uint16 classValue;
 };
 
