@@ -1,6 +1,7 @@
 /*
+ * @(#)ContextualGlyphSubstProc.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -20,14 +21,13 @@
 
 U_NAMESPACE_BEGIN
 
-class LEGlyphStorage;
-
 class ContextualGlyphSubstitutionProcessor : public StateTableProcessor
 {
 public:
     virtual void beginStateTable();
 
-    virtual ByteOffset processStateEntry(LEGlyphStorage &glyphStorage, le_int32 &currGlyph, EntryTableIndex index);
+    virtual ByteOffset processStateEntry(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 &currGlyph,
+        le_int32 glyphCount, EntryTableIndex index);
 
     virtual void endStateTable();
 
@@ -39,14 +39,14 @@ public:
      *
      * @stable ICU 2.8
      */
-    virtual UClassID getDynamicClassID() const;
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
      *
      * @stable ICU 2.8
      */
-    static UClassID getStaticClassID();
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
 
 private:
     ContextualGlyphSubstitutionProcessor();
@@ -59,6 +59,13 @@ protected:
 
     const ContextualGlyphSubstitutionHeader *contextualGlyphSubstitutionHeader;
 
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 U_NAMESPACE_END
