@@ -1,7 +1,7 @@
 /*
  * @(#)ContextualSubstSubtables.cpp	1.11 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -44,34 +44,54 @@ void ContextualSubstitutionBase::applySubstitutionLookups(
     }
 }
 
-le_bool ContextualSubstitutionBase::matchGlyphIDs(const LEGlyphID *glyphArray, le_uint16 glyphCount,
-                                               GlyphIterator *glyphIterator)
+le_bool ContextualSubstitutionBase::matchGlyphIDs(const TTGlyphID *glyphArray, le_uint16 glyphCount,
+                                               GlyphIterator *glyphIterator, le_bool backtrack)
 {
-    for (le_uint16 match = 0; match < glyphCount; match += 1) {
-        if (! glyphIterator->next()) {
-            return false;
-        }
+    le_int32 direction = 1;
+    le_int32 match = 0;
 
-        LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
-
-        if (glyph != SWAPW(glyphArray[match])) {
-            return false;
-        }
+    if (backtrack) {
+        match = glyphCount -1;
+        direction = -1;
     }
 
-    return true;
+    while (glyphCount > 0) {
+        if (! glyphIterator->next()) {
+            return FALSE;
+        }
+
+        TTGlyphID glyph = (TTGlyphID) glyphIterator->getCurrGlyphID();
+
+        if (glyph != SWAPW(glyphArray[match])) {
+            return FALSE;
+        }
+
+        glyphCount -= 1;
+        match += direction;
+    }
+
+    return TRUE;
 }
 
 le_bool ContextualSubstitutionBase::matchGlyphClasses(const le_uint16 *classArray, le_uint16 glyphCount,
                                                GlyphIterator *glyphIterator,
-                                               const ClassDefinitionTable *classDefinitionTable)
+                                               const ClassDefinitionTable *classDefinitionTable,
+                                               le_bool backtrack)
 {
-    for (le_uint16 match = 0; match < glyphCount; match += 1) {
+    le_int32 direction = 1;
+    le_int32 match = 0;
+
+    if (backtrack) {
+        match = glyphCount - 1;
+        direction = -1;
+    }
+
+    while (glyphCount > 0) {
         if (! glyphIterator->next()) {
-            return false;
+            return FALSE;
         }
 
-        LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+        LEGlyphID glyph = glyphIterator->getCurrGlyphID();
         le_int32 glyphClass = classDefinitionTable->getGlyphClass(glyph);
         le_int32 matchClass = SWAPW(classArray[match]);
 
@@ -81,31 +101,45 @@ le_bool ContextualSubstitutionBase::matchGlyphClasses(const le_uint16 *classArra
             // table. If we're looking for such a class, pretend that
             // we found it.
             if (classDefinitionTable->hasGlyphClass(matchClass)) {
-                return false;
+                return FALSE;
             }
         }
+
+        glyphCount -= 1;
+        match += direction;
     }
 
-    return true;
+    return TRUE;
 }
 
 le_bool ContextualSubstitutionBase::matchGlyphCoverages(const Offset *coverageTableOffsetArray, le_uint16 glyphCount,
-                                                     GlyphIterator *glyphIterator, const char *offsetBase)
+                                                     GlyphIterator *glyphIterator, const char *offsetBase, le_bool backtrack)
 {
-    for (le_uint16 glyph = 0; glyph < glyphCount; glyph += 1) {
+    le_int32 direction = 1;
+    le_int32 glyph = 0;
+
+    if (backtrack) {
+        glyph = glyphCount - 1;
+        direction = -1;
+    }
+
+    while (glyphCount > 0) {
         Offset coverageTableOffset = SWAPW(coverageTableOffsetArray[glyph]);
         const CoverageTable *coverageTable = (const CoverageTable *) (offsetBase + coverageTableOffset);
 
         if (! glyphIterator->next()) {
-            return false;
+            return FALSE;
         }
 
         if (coverageTable->getGlyphCoverage((LEGlyphID) glyphIterator->getCurrGlyphID()) < 0) {
-            return false;
+            return FALSE;
         }
+
+        glyphCount -= 1;
+        glyph += direction;
     }
 
-    return true;
+    return TRUE;
 }
 
 le_uint32 ContextualSubstitutionSubtable::process(const LookupProcessor *lookupProcessor, GlyphIterator *glyphIterator,
@@ -145,7 +179,7 @@ le_uint32 ContextualSubstitutionSubtable::process(const LookupProcessor *lookupP
 le_uint32 ContextualSubstitutionFormat1Subtable::process(const LookupProcessor *lookupProcessor, GlyphIterator *glyphIterator,
                                                       const LEFontInstance *fontInstance) const
 {
-    LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+    LEGlyphID glyph = glyphIterator->getCurrGlyphID();
     le_int32 coverageIndex = getGlyphCoverage(glyph);
 
     if (coverageIndex >= 0) {
@@ -188,14 +222,14 @@ le_uint32 ContextualSubstitutionFormat1Subtable::process(const LookupProcessor *
 le_uint32 ContextualSubstitutionFormat2Subtable::process(const LookupProcessor *lookupProcessor, GlyphIterator *glyphIterator,
                                                       const LEFontInstance *fontInstance) const
 {
-    LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+    LEGlyphID glyph = glyphIterator->getCurrGlyphID();
     le_int32 coverageIndex = getGlyphCoverage(glyph);
 
     if (coverageIndex >= 0) {
         const ClassDefinitionTable *classDefinitionTable =
             (const ClassDefinitionTable *) ((char *) this + SWAPW(classDefTableOffset));
         le_uint16 scSetCount = SWAPW(subClassSetCount);
-        le_int32 setClass = classDefinitionTable->getGlyphClass((LEGlyphID) glyphIterator->getCurrGlyphID());
+        le_int32 setClass = classDefinitionTable->getGlyphClass(glyphIterator->getCurrGlyphID());
 
         if (setClass < scSetCount && subClassSetTableOffsetArray[setClass] != 0) {
             Offset subClassSetTableOffset = SWAPW(subClassSetTableOffsetArray[setClass]);
@@ -292,10 +326,12 @@ le_uint32 ChainingContextualSubstitutionSubtable::process(const LookupProcessor
     }
 }
 
+const LETag emptyTag = 0;
+
 le_uint32 ChainingContextualSubstitutionFormat1Subtable::process(const LookupProcessor *lookupProcessor, GlyphIterator *glyphIterator,
                                                               const LEFontInstance *fontInstance) const
 {
-    LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+    LEGlyphID glyph = glyphIterator->getCurrGlyphID();
     le_int32 coverageIndex = getGlyphCoverage(glyph);
 
     if (coverageIndex >= 0) {
@@ -307,7 +343,7 @@ le_uint32 ChainingContextualSubstitutionFormat1Subtable::process(const LookupPro
                 (const ChainSubRuleSetTable *) ((char *) this + chainSubRuleSetTableOffset);
             le_uint16 chainSubRuleCount = SWAPW(chainSubRuleSetTable->chainSubRuleCount);
             le_int32 position = glyphIterator->getCurrStreamPosition();
-            GlyphIterator tempIterator(*glyphIterator);
+            GlyphIterator tempIterator(*glyphIterator, emptyTag);
 
             for (le_uint16 subRule = 0; subRule < chainSubRuleCount; subRule += 1) {
                 Offset chainSubRuleTableOffset =
@@ -316,14 +352,19 @@ le_uint32 ChainingContextualSubstitutionFormat1Subtable::process(const LookupPro
                     (const ChainSubRuleTable *) ((char *) chainSubRuleSetTable + chainSubRuleTableOffset);
                 le_uint16 backtrackGlyphCount = SWAPW(chainSubRuleTable->backtrackGlyphCount);
                 le_uint16 inputGlyphCount = (le_uint16) SWAPW(chainSubRuleTable->backtrackGlyphArray[backtrackGlyphCount]) - 1;
-                const LEGlyphID *inputGlyphArray = &chainSubRuleTable->backtrackGlyphArray[backtrackGlyphCount + 1];
+                const TTGlyphID *inputGlyphArray = &chainSubRuleTable->backtrackGlyphArray[backtrackGlyphCount + 1];
                 le_uint16 lookaheadGlyphCount = (le_uint16) SWAPW(inputGlyphArray[inputGlyphCount]);
-                const LEGlyphID *lookaheadGlyphArray = &inputGlyphArray[inputGlyphCount + 1];
+                const TTGlyphID *lookaheadGlyphArray = &inputGlyphArray[inputGlyphCount + 1];
                 le_uint16 substCount = (le_uint16) SWAPW(lookaheadGlyphArray[lookaheadGlyphCount]);
 
                 tempIterator.setCurrStreamPosition(position);
-                tempIterator.prev(backtrackGlyphCount + 1);
-                if (! matchGlyphIDs(chainSubRuleTable->backtrackGlyphArray, backtrackGlyphCount, &tempIterator)) {
+
+                if (! tempIterator.prev(backtrackGlyphCount)) {
+                    continue;
+                }
+
+                tempIterator.prev();
+                if (! matchGlyphIDs(chainSubRuleTable->backtrackGlyphArray, backtrackGlyphCount, &tempIterator, TRUE)) {
                     continue;
                 }
 
@@ -355,7 +396,7 @@ le_uint32 ChainingContextualSubstitutionFormat1Subtable::process(const LookupPro
 le_uint32 ChainingContextualSubstitutionFormat2Subtable::process(const LookupProcessor *lookupProcessor, GlyphIterator *glyphIterator,
                                                               const LEFontInstance *fontInstance) const
 {
-    LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+    LEGlyphID glyph = glyphIterator->getCurrGlyphID();
     le_int32 coverageIndex = getGlyphCoverage(glyph);
 
     if (coverageIndex >= 0) {
@@ -366,7 +407,7 @@ le_uint32 ChainingContextualSubstitutionFormat2Subtable::process(const LookupPro
         const ClassDefinitionTable *lookaheadClassDefinitionTable =
             (const ClassDefinitionTable *) ((char *) this + SWAPW(lookaheadClassDefTableOffset));
         le_uint16 scSetCount = SWAPW(chainSubClassSetCount);
-        le_int32 setClass = inputClassDefinitionTable->getGlyphClass((LEGlyphID) glyphIterator->getCurrGlyphID());
+        le_int32 setClass = inputClassDefinitionTable->getGlyphClass(glyphIterator->getCurrGlyphID());
 
         if (setClass < scSetCount && chainSubClassSetTableOffsetArray[setClass] != 0) {
             Offset chainSubClassSetTableOffset = SWAPW(chainSubClassSetTableOffsetArray[setClass]);
@@ -374,7 +415,7 @@ le_uint32 ChainingContextualSubstitutionFormat2Subtable::process(const LookupPro
                 (const ChainSubClassSetTable *) ((char *) this + chainSubClassSetTableOffset);
             le_uint16 chainSubClassRuleCount = SWAPW(chainSubClassSetTable->chainSubClassRuleCount);
             le_int32 position = glyphIterator->getCurrStreamPosition();
-            GlyphIterator tempIterator(*glyphIterator);
+            GlyphIterator tempIterator(*glyphIterator, emptyTag);
 
             for (le_uint16 scRule = 0; scRule < chainSubClassRuleCount; scRule += 1) {
                 Offset chainSubClassRuleTableOffset =
@@ -390,9 +431,14 @@ le_uint32 ChainingContextualSubstitutionFormat2Subtable::process(const LookupPro
                 
 
                 tempIterator.setCurrStreamPosition(position);
-                tempIterator.prev(backtrackGlyphCount + 1);
+
+                if (! tempIterator.prev(backtrackGlyphCount)) {
+                    continue;
+                }
+
+                tempIterator.prev();
                 if (! matchGlyphClasses(chainSubClassRuleTable->backtrackClassArray, backtrackGlyphCount,
-                    &tempIterator, backtrackClassDefinitionTable)) {
+                    &tempIterator, backtrackClassDefinitionTable, TRUE)) {
                     continue;
                 }
 
@@ -431,11 +477,15 @@ le_uint32 ChainingContextualSubstitutionFormat3Subtable::process(const LookupPro
     const Offset *lookaheadCoverageTableOffsetArray = &inputCoverageTableOffsetArray[inputGlyphCount + 1];
     le_uint16 substCount = (le_uint16) SWAPW(lookaheadCoverageTableOffsetArray[lookaheadGlyphCount]);
     le_int32 position = glyphIterator->getCurrStreamPosition();
-    GlyphIterator tempIterator(*glyphIterator);
+    GlyphIterator tempIterator(*glyphIterator, emptyTag);
 
-    tempIterator.prev(backtrkGlyphCount + 1);
+    if (! tempIterator.prev(backtrkGlyphCount)) {
+        return 0;
+    }
+
+    tempIterator.prev();
     if (! ContextualSubstitutionBase::matchGlyphCoverages(backtrackCoverageTableOffsetArray,
-        backtrkGlyphCount, &tempIterator, (const char *) this)) {
+        backtrkGlyphCount, &tempIterator, (const char *) this, TRUE)) {
         return 0;
     }
 
@@ -459,7 +509,7 @@ le_uint32 ChainingContextualSubstitutionFormat3Subtable::process(const LookupPro
 
         ContextualSubstitutionBase::applySubstitutionLookups(lookupProcessor, substLookupRecordArray, substCount, glyphIterator, fontInstance, position);
 
-        return inputGlyphCount + 1;
+        return inputGlyphCount;
     }
 
     glyphIterator->setCurrStreamPosition(position);
