@@ -1,24 +1,32 @@
 /*
  * @(#)GDEFMarkFilter.h	1.5 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
 #ifndef __GDEFMARKFILTER__H
 #define __GDEFMARKFILTER__H
 
+/**
+ * \file
+ * \internal
+ */
+
 #include "LETypes.h"
 #include "LEGlyphFilter.h"
 #include "GlyphDefinitionTables.h"
 
 U_NAMESPACE_BEGIN
 
-class GDEFMarkFilter : public LEGlyphFilter
+class GDEFMarkFilter : public UMemory, public LEGlyphFilter
 {
 private:
     const GlyphClassDefinitionTable *classDefTable;
 
+    GDEFMarkFilter(const GDEFMarkFilter &other); // forbid copying of this class
+    GDEFMarkFilter &operator=(const GDEFMarkFilter &other); // forbid copying of this class
+
 public:
     GDEFMarkFilter(const GlyphDefinitionTableHeader *gdefTable);
     virtual ~GDEFMarkFilter();
