@@ -2,7 +2,7 @@
 /*
  * @(#)GXLayoutEngine.h	1.4 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -23,6 +23,8 @@ U_NAMESPACE_BEGIN
  * fonts. A font is a GX or AAT font if it contains a 'mort' table. See Apple's
  * TrueType Reference Manual (http://fonts.apple.com/TTRefMan/index.html) for details.
  * Information about 'mort' tables is in the chapter titled "Font Files."
+ *
+ * @internal
  */
 class GXLayoutEngine : public LayoutEngine
 {
@@ -43,18 +45,38 @@ public:
      *
      * @see LayoutEngine::layoutEngineFactory
      * @see ScriptAndLangaugeTags.h for script and language codes
+     *
+     * @internal
      */
     GXLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, const MorphTableHeader *morphTable);
 
     /**
      * The destructor, virtual for correct polymorphic invocation.
+     *
+     * @internal
      */
     virtual ~GXLayoutEngine();
 
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.8
+     */
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.8
+     */
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
+
 protected:
 
     /**
      * The address of the 'mort' table
+     *
+     * @internal
      */
     const MorphTableHeader *fMorphTable;
 
@@ -68,7 +90,7 @@ protected:
      * @param offset - the index of the first character to process
      * @param count - the number of characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the text is in a right to left directional run
+     * @param rightToLeft - TRUE if the text is in a right to left directional run
      *
      * Output parameters:
      * @param glyphs - the glyph index array
@@ -76,6 +98,8 @@ protected:
      * @param success - set to an error code if the operation fails
      *
      * @return the number of glyphs in the glyph index array
+     *
+     * @internal
      */
     virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
         LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
@@ -93,9 +117,19 @@ protected:
      * Output parameters:
      * @param positions - the output X and Y positions (two entries per glyph)
      * @param success - set to an error code if the operation fails
+     *
+     * @internal
      */
     virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphID glyphs[],
         le_int32 glyphCount, float positions[], LEErrorCode &success);
+
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 U_NAMESPACE_END
