@@ -1,5 +1,7 @@
 /*
- * (C) Copyright IBM Corp. 1998-2008 - All Rights Reserved
+ * @(#)GlyphPositioningTables.h	1.7 00/03/15
+ *
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -12,24 +14,26 @@
  */
 
 #include "LETypes.h"
+#include "LEFontInstance.h"
 #include "OpenTypeTables.h"
 #include "Lookups.h"
-#include "GlyphLookupTables.h"
+#include "GlyphDefinitionTables.h"
+#include "GlyphPositionAdjustments.h"
 
 U_NAMESPACE_BEGIN
 
-class  LEFontInstance;
-class  LEGlyphStorage;
-class  LEGlyphFilter;
-class  GlyphPositionAdjustments;
-struct GlyphDefinitionTableHeader;
-
-struct GlyphPositioningTableHeader : public GlyphLookupTableHeader
+struct GlyphPositioningTableHeader
 {
-    void    process(LEGlyphStorage &glyphStorage, GlyphPositionAdjustments *glyphPositionAdjustments,
+    fixed32 version;
+    Offset  scriptListOffset;
+    Offset  featureListOffset;
+    Offset  lookupListOffset;
+
+    void    process(LEGlyphID *glyphs, GlyphPositionAdjustment *glyphPositionAdjustments,
+                const LETag **glyphTags, le_int32 glyphCount,
                 le_bool rightToLeft, LETag scriptTag, LETag languageTag,
-                const GlyphDefinitionTableHeader *glyphDefinitionTableHeader, LEErrorCode &success,
-                const LEFontInstance *fontInstance, const FeatureMap *featureMap, le_int32 featureMapCount, le_bool featureOrder) const;
+                const GlyphDefinitionTableHeader *glyphDefinitionTableHeader,
+                const LEFontInstance *fontInstance, const LETag *featureOrder) const;
 };
 
 enum GlyphPositioningSubtableTypes
