@@ -1,5 +1,7 @@
 /*
- * (C) Copyright IBM Corp. 1998-2008 - All Rights Reserved
+ * @(#)GlyphPosnLookupProc.h	1.7 00/03/15
+ *
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -27,17 +29,12 @@ class GlyphPositioningLookupProcessor : public LookupProcessor
 {
 public:
     GlyphPositioningLookupProcessor(const GlyphPositioningTableHeader *glyphPositioningTableHeader,
-        LETag scriptTag, 
-        LETag languageTag, 
-        const FeatureMap *featureMap, 
-        le_int32 featureMapCount, 
-        le_bool featureOrder,
-        LEErrorCode& success);
+        LETag scriptTag, LETag languageTag, const LETag *featureOrder);
 
     virtual ~GlyphPositioningLookupProcessor();
 
     virtual le_uint32 applySubtable(const LookupSubtable *lookupSubtable, le_uint16 lookupType, GlyphIterator *glyphIterator,
-        const LEFontInstance *fontInstance, LEErrorCode& success) const;
+        const LEFontInstance *fontInstance) const;
 
 protected:
     GlyphPositioningLookupProcessor();
