@@ -2,7 +2,7 @@
 /*
  * @(#)IndicLayoutEngine.cpp	1.3 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -22,15 +22,17 @@
 
 U_NAMESPACE_BEGIN
 
+const char IndicOpenTypeLayoutEngine::fgClassID=0;
+
 IndicOpenTypeLayoutEngine::IndicOpenTypeLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode,
                     const GlyphSubstitutionTableHeader *gsubTable)
-    : OpenTypeLayoutEngine(fontInstance, scriptCode, languageCode, gsubTable)
+    : OpenTypeLayoutEngine(fontInstance, scriptCode, languageCode, gsubTable), fMPreFixups(NULL)
 {
     fFeatureOrder = IndicReordering::getFeatureOrder();
 }
 
 IndicOpenTypeLayoutEngine::IndicOpenTypeLayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode)
-    : OpenTypeLayoutEngine(fontInstance, scriptCode, languageCode)
+    : OpenTypeLayoutEngine(fontInstance, scriptCode, languageCode), fMPreFixups(NULL)
 {
     fFeatureOrder = IndicReordering::getFeatureOrder();
 }
@@ -42,14 +44,14 @@ IndicOpenTypeLayoutEngine::~IndicOpenTypeLayoutEngine()
 
 // Input: characters, tags
 // Output: glyphs, char indices
-le_int32 IndicOpenTypeLayoutEngine::glyphProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, const LETag **featureTags,
-                LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success)
+le_int32 IndicOpenTypeLayoutEngine::glyphProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
+					const LETag **&featureTags, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success)
 {
     if (LE_FAILURE(success)) {
         return 0;
     }
 
-    if (chars == NULL || offset < 0 || count < 0) {
+    if (chars == NULL || offset < 0 || count < 0 || max < 0 || offset >= max || offset + count > max) {
         success = LE_ILLEGAL_ARGUMENT_ERROR;
         return 0;
     }
@@ -60,7 +62,7 @@ le_int32 IndicOpenTypeLayoutEngine::glyphProcessing(const LEUnicode chars[], le_
         return 0;
     }
 
-    IndicReordering::adjustMPres(&chars[offset], count, glyphs, charIndices, fScriptCode);
+    IndicReordering::adjustMPres(fMPreFixups, glyphs, charIndices);
 
     return retCount;
 }
@@ -68,41 +70,46 @@ le_int32 IndicOpenTypeLayoutEngine::glyphProcessing(const LEUnicode chars[], le_
 // Input: characters
 // Output: characters, char indices, tags
 // Returns: output character count
-le_int32 IndicOpenTypeLayoutEngine::characterProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
+le_int32 IndicOpenTypeLayoutEngine::characterProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool /*rightToLeft*/,
         LEUnicode *&outChars, le_int32 *&charIndices, const LETag **&featureTags, LEErrorCode &success)
 {
     if (LE_FAILURE(success)) {
         return 0;
     }
 
+    if (chars == NULL || offset < 0 || count < 0 || max < 0 || offset >= max || offset + count > max) {
+        success = LE_ILLEGAL_ARGUMENT_ERROR;
+        return 0;
+    }
+
     le_int32 worstCase = count * IndicReordering::getWorstCaseExpansion(fScriptCode);
 
-    outChars = new LEUnicode[worstCase];
+    outChars = LE_NEW_ARRAY(LEUnicode, worstCase);
 
     if (outChars == NULL) {
         success = LE_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
 
-    charIndices = new le_int32[worstCase];
+    charIndices = LE_NEW_ARRAY(le_int32, worstCase);
     if (charIndices == NULL) {
-        delete[] outChars;
+        LE_DELETE_ARRAY(outChars);
         success = LE_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
 
-    featureTags = new const LETag*[worstCase];
+    featureTags = LE_NEW_ARRAY(const LETag *, worstCase);
 
     if (featureTags == NULL) {
-        delete[] charIndices;
-        delete[] outChars;
+        LE_DELETE_ARRAY(charIndices);
+        LE_DELETE_ARRAY(outChars);
         success = LE_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
 
     // NOTE: assumes this allocates featureTags...
     // (probably better than doing the worst case stuff here...)
-    return IndicReordering::reorder(&chars[offset], count, fScriptCode, outChars, charIndices, featureTags);
+    return IndicReordering::reorder(&chars[offset], count, fScriptCode, outChars, charIndices, featureTags, &fMPreFixups);
 }
 
 U_NAMESPACE_END
