@@ -1,22 +1,24 @@
+
 /*
- * (C) Copyright IBM Corp. 1998-2011 - All Rights Reserved
+ * %W% %W%
+ *
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
+ *
  */
 
 #ifndef __LAYOUTENGINE_H
 #define __LAYOUTENGINE_H
 
+#ifndef __LETYPES_H
 #include "LETypes.h"
+#endif
 
-/**
- * \file 
- * \brief C++ API: Virtual base class for complex text layout.
- */
+#include <string.h>
 
 U_NAMESPACE_BEGIN
 
 class LEFontInstance;
 class LEGlyphFilter;
-class LEGlyphStorage;
 
 /**
  * This is a virtual base class used to do complex text layout. The text must all
@@ -62,19 +64,38 @@ class LEGlyphStorage;
  * @stable ICU 2.8
  */
 class U_LAYOUT_API LayoutEngine : public UObject {
-public:
-    /** @internal Flag to request kerning. */
-    static const le_int32 kTypoFlagKern;
-    /** @internal Flag to request ligatures. */
-    static const le_int32 kTypoFlagLiga;
-
 protected:
     /**
-     * The object which holds the glyph storage
+     * The number of glyphs in the output
      *
      * @internal
      */
-    LEGlyphStorage *fGlyphStorage;
+    le_int32 fGlyphCount;
+
+    /**
+     * The output glyph array
+     *
+     * @internal
+     */
+    LEGlyphID *fGlyphs;
+
+    /**
+     * The character index array. One entry for each output glyph, giving the index
+     * in the input character array of the character which corresponds to this glyph.
+     *
+     * @internal
+     */
+    le_int32 *fCharIndices;
+
+    /**
+     * The glyph position array. There are two entries for each glyph giving the
+     * X and Y positions of the glyph. Thus, for glyph i, the X position is at index
+     * 2i, and the Y position is at index 2i + 1. There are also two entries for the
+     * X and Y position of the advance of the last glyph.
+     *
+     * @internal
+     */
+    float *fPositions;
 
     /**
      * The font instance for the text font.
@@ -103,42 +124,20 @@ protected:
      */
     le_int32 fLanguageCode;
 
-    /**
-     * The typographic control flags
-     *
-     * @internal
-     */
-    le_int32 fTypoFlags;
-
-    /**
-     * <code>TRUE</code> if <code>mapCharsToGlyphs</code> should replace ZWJ / ZWNJ with a glyph
-     * with no contours.
-     *
-     * @internal
-     */
-    le_bool fFilterZeroWidth;
-
     /**
      * This constructs an instance for a given font, script and language. Subclass constructors
      * must call this constructor.
      *
      * @param fontInstance - the font for the text
      * @param scriptCode - the script for the text
-     * @param languageCode - the language for the text
-     * @param typoFlags - the typographic control flags for the text (a bitfield).  Use kTypoFlagKern
-     * if kerning is desired, kTypoFlagLiga if ligature formation is desired.  Others are reserved.
-     * @param success - set to an error code if the operation fails
+     * @param langaugeCode - the language for the text
      *
      * @see LEFontInstance
      * @see ScriptAndLanguageTags.h
      *
      * @internal
      */
-    LayoutEngine(const LEFontInstance *fontInstance,
-                 le_int32 scriptCode,
-                 le_int32 languageCode,
-                 le_int32 typoFlags,
-                 LEErrorCode &success);
+    LayoutEngine(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode);
 
     /**
      * This overrides the default no argument constructor to make it
@@ -149,31 +148,6 @@ protected:
      */
     LayoutEngine();
 
-    /**
-     * This method does any required pre-processing to the input characters. It
-     * may generate output characters that differ from the input charcters due to
-     * insertions, deletions, or reorderings. In such cases, it will also generate an
-     * output character index array reflecting these changes.
-     *
-     * Subclasses must override this method.
-     *
-     * Input parameters:
-     * @param chars - the input character context
-     * @param offset - the index of the first character to process
-     * @param count - the number of characters to process
-     * @param max - the number of characters in the input context
-     * @param rightToLeft - TRUE if the characters are in a right to left directional run
-     * @param outChars - the output character array, if different from the input
-     * @param glyphStorage - the object that holds the per-glyph storage. The character index array may be set.
-     * @param success - set to an error code if the operation fails
-     *
-     * @return the output character count (input character count if no change)
-     *
-     * @internal
-     */
-    virtual le_int32 characterProcessing(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft,
-            LEUnicode *&outChars, LEGlyphStorage &glyphStorage, LEErrorCode &success);
-
     /**
      * This method does the glyph processing. It converts an array of characters
      * into an array of glyph indices and character indices. The characters to be
@@ -190,17 +164,17 @@ protected:
      * @param count - the number of characters to process
      * @param max - the number of characters in the context.
      * @param rightToLeft - TRUE if the text is in a right to left directional run
-     * @param glyphStorage - the object which holds the per-glyph storage. The glyph and char indices arrays
-     *                       will be set.
      *
      * Output parameters:
+     * @param glyphs - the glyph index array
+     * @param charIndices - the character index array
      * @param success - set to an error code if the operation fails
      *
      * @return the number of glyphs in the glyph index array
      *
      * @internal
      */
-    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphStorage &glyphStorage, LEErrorCode &success);
+    virtual le_int32 computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
 
     /**
      * This method does basic glyph positioning. The default implementation positions
@@ -208,14 +182,17 @@ protected:
      * is not expected that many subclasses will override this method.
      *
      * Input parameters:
-     * @param glyphStorage - the object which holds the per-glyph storage. The glyph position array will be set.
+     * @param glyphs - the input glyph array
+     * @param glyphCount - the number of glyphs in the glyph array
      * @param x - the starting X position
      * @param y - the starting Y position
-     * @param success - set to an error code if the operation fails
+     *
+     * Output parameters:
+     * @param positions - the output X and Y positions (two entries per glyph)
      *
      * @internal
      */
-    virtual void positionGlyphs(LEGlyphStorage &glyphStorage, float x, float y, LEErrorCode &success);
+    virtual void positionGlyphs(const LEGlyphID glyphs[], le_int32 glyphCount, float x, float y, float *&positions, LEErrorCode &success);
 
     /**
      * This method does positioning adjustments like accent positioning and
@@ -230,14 +207,18 @@ protected:
      * @param chars - the input character context
      * @param offset - the offset of the first character to process
      * @param count - the number of characters to process
-     * @param reverse - <code>TRUE</code> if the glyphs in the glyph array have been reordered
-     * @param glyphStorage - the object which holds the per-glyph storage. The glyph positions will be
-     *                       adjusted as needed.
+     * @param reverse - TRUE if the glyphs in the glyph array have been reordered
+     * @param glyphs - the input glyph array
+     * @param glyphCount - the number of glyphs
+     * @param positions - the position array, will be updated as needed
      * @param success - output parameter set to an error code if the operation fails
      *
+     * Note: the positions are passed as a plain array because this method should
+     * not need to reallocate them.
+     *
      * @internal
      */
-    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, LEGlyphStorage &glyphStorage, LEErrorCode &success);
+    virtual void adjustGlyphPositions(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool /*reverse*/, LEGlyphID glyphs[], le_int32 glyphCount, float positions[], LEErrorCode &success);
 
     /**
      * This method gets a table from the font associated with
@@ -268,53 +249,36 @@ protected:
      * @param chars - the input character context
      * @param offset - the offset of the first character to be mapped
      * @param count - the number of characters to be mapped
-     * @param reverse - if <code>TRUE</code>, the output will be in reverse order
-     * @param mirror - if <code>TRUE</code>, do character mirroring
-     * @param glyphStorage - the object which holds the per-glyph storage. The glyph and char
-     *                       indices arrays will be filled in.
+     * @param reverse - if TRUE, the output will be in reverse order
+     * @param mirror - if TRUE, do character mirroring
+     *
+     * Output parameters:
+     * @param glyphs - the glyph array
+     * @param charIndices - the character index array
      * @param success - set to an error code if the operation fails
      *
      * @see LEFontInstance
      *
      * @internal
      */
-    virtual void mapCharsToGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, le_bool mirror, LEGlyphStorage &glyphStorage, LEErrorCode &success);
+    virtual void mapCharsToGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_bool reverse, le_bool mirror, LEGlyphID *&glyphs, le_int32 *&charIndices, LEErrorCode &success);
 
     /**
      * This is a convenience method that forces the advance width of mark
      * glyphs to be zero, which is required for proper selection and highlighting.
      * 
-     * @param glyphStorage - the object containing the per-glyph storage. The positions array will be modified.
+     * @param glyphs - the glyph array
+     * @param glyphCount - the number of glyphs
+     * @param reverse - TRUE if the glyph array has been reordered
      * @param markFilter - used to identify mark glyphs
+     * @param positions - the glyph position array - updated as required
      * @param success - output parameter set to an error code if the operation fails
      *
      * @see LEGlyphFilter
      *
      * @internal
      */
-    static void adjustMarkGlyphs(LEGlyphStorage &glyphStorage, LEGlyphFilter *markFilter, LEErrorCode &success);
-
-
-    /**
-     * This is a convenience method that forces the advance width of mark
-     * glyphs to be zero, which is required for proper selection and highlighting.
-     * This method uses the input characters to identify marks. This is required in
-     * cases where the font does not contain enough information to identify them based
-     * on the glyph IDs.
-     * 
-     * @param chars - the array of input characters
-     * @param charCount - the number of input characers
-     * @param glyphStorage - the object containing the per-glyph storage. The positions array will be modified.
-     * @param reverse - <code>TRUE</code> if the glyph array has been reordered
-     * @param markFilter - used to identify mark glyphs
-     * @param success - output parameter set to an error code if the operation fails
-     *
-     * @see LEGlyphFilter
-     *
-     * @internal
-     */
-    static void adjustMarkGlyphs(const LEUnicode chars[], le_int32 charCount, le_bool reverse, LEGlyphStorage &glyphStorage, LEGlyphFilter *markFilter, LEErrorCode &success);
-
+    static void adjustMarkGlyphs(const LEGlyphID glyphs[], le_int32 glyphCount, le_bool reverse, LEGlyphFilter *markFilter, float positions[], LEErrorCode &success);
 
 public:
     /**
@@ -329,7 +293,7 @@ public:
 
     /**
      * This method will invoke the layout steps in their correct order by calling
-     * the computeGlyphs, positionGlyphs and adjustGlyphPosition methods. It will
+     * the computeGlyphs, positionGlyphs and adjustGlyphPosition methods.. It will
      * compute the glyph, character index and position arrays.
      *
      * @param chars - the input character context
@@ -343,12 +307,8 @@ public:
      *
      * @return the number of glyphs in the glyph array
      *
-     * Note: The glyph, character index and position array can be accessed
-     * using the getter methods below.
-     *
-     * Note: If you call this method more than once, you must call the reset()
-     * method first to free the glyph, character index and position arrays
-     * allocated by the previous call.
+     * Note; the glyph, character index and position array can be accessed
+     * using the getter method below.
      *
      * @stable ICU 2.8
      */
@@ -363,7 +323,10 @@ public:
      *
      * @stable ICU 2.8
      */
-    le_int32 getGlyphCount() const;
+    le_int32 getGlyphCount() const
+    {
+        return fGlyphCount;
+    };
 
     /**
      * This method copies the glyph array into a caller supplied array.
@@ -422,7 +385,7 @@ public:
      * X and Y position for each glyph, plus an extra X and Y for the
      * advance of the last glyph.
      *
-     * @param positions - the destiniation position array
+     * @param glyphs - the destiniation position array
      * @param success - set to an error code if the operation fails
      *
      * @stable ICU 2.8
@@ -461,7 +424,7 @@ public:
      *
      * @param fontInstance - the font of the text
      * @param scriptCode - the script of the text
-     * @param languageCode - the language of the text
+     * @param langaugeCode - the language of the text
      * @param success - output parameter set to an error code if the operation fails
      *
      * @return a LayoutEngine which can layout text in the given font.
@@ -472,27 +435,29 @@ public:
      */
     static LayoutEngine *layoutEngineFactory(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, LEErrorCode &success);
 
-    /**
-     * Override of existing call that provides flags to control typography.
-     * @stable ICU 3.4
-     */
-    static LayoutEngine *layoutEngineFactory(const LEFontInstance *fontInstance, le_int32 scriptCode, le_int32 languageCode, le_int32 typo_flags, LEErrorCode &success);
-
     /**
      * ICU "poor man's RTTI", returns a UClassID for the actual class.
      *
      * @stable ICU 2.8
      */
-    virtual UClassID getDynamicClassID() const;
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
      *
      * @stable ICU 2.8
      */
-    static UClassID getStaticClassID();
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
 
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 U_NAMESPACE_END
 #endif
+
