@@ -1,7 +1,7 @@
 /*
  * %W% %E%
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -17,12 +17,11 @@ U_NAMESPACE_BEGIN
 
 le_uint32 MultipleSubstitutionSubtable::process(GlyphIterator *glyphIterator, const LEGlyphFilter *filter) const
 {
-    LEGlyphID glyph = (LEGlyphID) glyphIterator->getCurrGlyphID();
+    LEGlyphID glyph = glyphIterator->getCurrGlyphID();
     le_int32 coverageIndex = getGlyphCoverage(glyph);
     le_uint16 seqCount = SWAPW(sequenceCount);
 
-    if (coverageIndex >= 0 && coverageIndex < seqCount)
-    {
+    if (coverageIndex >= 0 && coverageIndex < seqCount) {
         Offset sequenceTableOffset = SWAPW(sequenceTableOffsetArray[coverageIndex]);
         const SequenceTable *sequenceTable = (const SequenceTable *) ((char *) this + sequenceTableOffset);
         le_uint16 glyphCount = SWAPW(sequenceTable->glyphCount);
@@ -30,10 +29,30 @@ le_uint32 MultipleSubstitutionSubtable::process(GlyphIterator *glyphIterator, co
         if (glyphCount == 0) {
             glyphIterator->setCurrGlyphID(0xFFFF);
             return 1;
-        }
-        else {
-            // Can't do this 'till there's a way to
-            // grow the glyph array...
+        } else if (glyphCount == 1) {
+            TTGlyphID substitute = SWAPW(sequenceTable->substituteArray[0]);
+
+            if (filter == NULL || filter->accept(LE_SET_GLYPH(glyph, substitute))) {
+                glyphIterator->setCurrGlyphID(substitute);
+            }
+
+            return 1;
+        } else {
+			LEGlyphID *newGlyphs = glyphIterator->insertGlyphs(glyphCount);
+			le_int32 insert = 0, direction = 1;
+
+			if (glyphIterator->isRightToLeft()) {
+				insert = glyphCount - 1;
+				direction = -1;
+			}
+
+			for (le_int32 i = 0; i < glyphCount; i += 1) {
+				TTGlyphID substitute = SWAPW(sequenceTable->substituteArray[i]);
+
+				newGlyphs[insert] = LE_SET_GLYPH(glyph, substitute);
+				insert += direction;
+			}
+
             return 1;
         }
     }
