@@ -1,6 +1,7 @@
 /*
+ * @(#)OpenTypeTables.h	1.7 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2005 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -21,14 +22,13 @@ U_NAMESPACE_BEGIN
 typedef le_uint16 Offset;
 typedef le_uint8  ATag[4];
 typedef le_uint32 fixed32;
-typedef le_uint32 FeatureMask;
 
 #define SWAPT(atag) ((LETag) ((atag[0] << 24) + (atag[1] << 16) + (atag[2] << 8) + atag[3]))
 
 struct TagAndOffsetRecord
 {
-    ATag   tag;
-    Offset offset;
+    ATag    tag;
+    Offset  offset;
 };
 
 struct GlyphRangeRecord
@@ -38,11 +38,5 @@ struct GlyphRangeRecord
     le_int16  rangeValue;
 };
 
-struct FeatureMap
-{
-    LETag       tag;
-    FeatureMask mask;
-};
-
 U_NAMESPACE_END
 #endif
