@@ -1,6 +1,7 @@
 /*
+ * @(#)SegmentArrayProcessor.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -20,12 +21,10 @@
 
 U_NAMESPACE_BEGIN
 
-class LEGlyphStorage;
-
 class SegmentArrayProcessor : public NonContextualGlyphSubstitutionProcessor
 {
 public:
-    virtual void process(LEGlyphStorage &glyphStorage);
+    virtual void process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyph);
 
     SegmentArrayProcessor(const MorphSubtableHeader *morphSubtableHeader);
 
@@ -36,14 +35,14 @@ public:
      *
      * @stable ICU 2.8
      */
-    virtual UClassID getDynamicClassID() const;
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
      *
      * @stable ICU 2.8
      */
-    static UClassID getStaticClassID();
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
 
 private:
     SegmentArrayProcessor();
@@ -51,6 +50,13 @@ private:
 protected:
     const SegmentArrayLookupTable *segmentArrayLookupTable;
 
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 U_NAMESPACE_END
