@@ -1,7 +1,7 @@
 /*
  * @(#)SimpleArrayProcessor.cpp	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -15,6 +15,8 @@
 
 U_NAMESPACE_BEGIN
 
+const char SimpleArrayProcessor::fgClassID=0;
+
 SimpleArrayProcessor::SimpleArrayProcessor()
 {
 }
@@ -31,15 +33,15 @@ SimpleArrayProcessor::~SimpleArrayProcessor()
 {
 }
 
-void SimpleArrayProcessor::process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyphCount)
+void SimpleArrayProcessor::process(LEGlyphID *glyphs, le_int32 * /*charIndices*/, le_int32 glyphCount)
 {
     le_int32 glyph;
 
     for (glyph = 0; glyph < glyphCount; glyph += 1) {
         if (glyphs[glyph] < 0xFFFF) {
-            le_int16 newGlyph = simpleArrayLookupTable->valueArray[glyphs[glyph]];
+            le_int16 newGlyph = SWAPW(simpleArrayLookupTable->valueArray[glyphs[glyph]]);
 
-            glyphs[glyph] = SWAPW(newGlyph);
+            glyphs[glyph] = LE_SET_GLYPH(glyphs[glyph], newGlyph);
         }
     }
 }
