@@ -1,6 +1,7 @@
 /*
+ * @(#)SingleTableProcessor.cpp	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -10,12 +11,11 @@
 #include "NonContextualGlyphSubst.h"
 #include "NonContextualGlyphSubstProc.h"
 #include "SingleTableProcessor.h"
-#include "LEGlyphStorage.h"
 #include "LESwaps.h"
 
 U_NAMESPACE_BEGIN
 
-UOBJECT_DEFINE_RTTI_IMPLEMENTATION(SingleTableProcessor)
+const char SingleTableProcessor::fgClassID=0;
 
 SingleTableProcessor::SingleTableProcessor()
 {
@@ -33,17 +33,16 @@ SingleTableProcessor::~SingleTableProcessor()
 {
 }
 
-void SingleTableProcessor::process(LEGlyphStorage &glyphStorage)
+void SingleTableProcessor::process(LEGlyphID *glyphs, le_int32 * /*charIndices*/, le_int32 glyphCount)
 {
     const LookupSingle *entries = singleTableLookupTable->entries;
     le_int32 glyph;
-    le_int32 glyphCount = glyphStorage.getGlyphCount();
 
     for (glyph = 0; glyph < glyphCount; glyph += 1) {
-        const LookupSingle *lookupSingle = singleTableLookupTable->lookupSingle(entries, glyphStorage[glyph]);
+        const LookupSingle *lookupSingle = singleTableLookupTable->lookupSingle(entries, glyphs[glyph]);
 
         if (lookupSingle != NULL) {
-            glyphStorage[glyph] = SWAPW(lookupSingle->value);
+            glyphs[glyph] = SWAPW(lookupSingle->value);
         }
     }
 } 
