@@ -1,20 +1,24 @@
 /*
  * @(#)SubtableProcessor.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
 #ifndef __SUBTABLEPROCESSOR_H
 #define __SUBTABLEPROCESSOR_H
 
+/**
+ * \file
+ * \internal
+ */
+
 #include "LETypes.h"
 #include "MorphTables.h"
 
 U_NAMESPACE_BEGIN
 
-class SubtableProcessor
-{
+class SubtableProcessor : public UMemory {
 public:
     virtual void process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyph) = 0;
     virtual ~SubtableProcessor();
@@ -29,6 +33,11 @@ protected:
     FeatureFlags subtableFeatures;
 
     const MorphSubtableHeader *subtableHeader;
+
+private:
+
+    SubtableProcessor(const SubtableProcessor &other); // forbid copying of this class
+    SubtableProcessor &operator=(const SubtableProcessor &other); // forbid copying of this class
 };
 
 U_NAMESPACE_END
