@@ -1,7 +1,7 @@
 /*
  * @(#)TrimmedArrayProcessor.cpp	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -15,6 +15,8 @@
 
 U_NAMESPACE_BEGIN
 
+const char TrimmedArrayProcessor::fgClassID=0;
+
 TrimmedArrayProcessor::TrimmedArrayProcessor()
 {
 }
@@ -33,15 +35,17 @@ TrimmedArrayProcessor::~TrimmedArrayProcessor()
 {
 }
 
-void TrimmedArrayProcessor::process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyphCount)
+void TrimmedArrayProcessor::process(LEGlyphID *glyphs, le_int32 * /*charIndices*/, le_int32 glyphCount)
 {
     le_int32 glyph;
 
     for (glyph = 0; glyph < glyphCount; glyph += 1) {
-        if ((glyphs[glyph] > firstGlyph) && (glyphs[glyph] < lastGlyph)) {
-            le_int16 newGlyph = trimmedArrayLookupTable->valueArray[glyphs[glyph] - firstGlyph];
+        TTGlyphID ttGlyph = (TTGlyphID) LE_GET_GLYPH(glyphs[glyph]);
 
-            glyphs[glyph] = SWAPW(newGlyph);
+        if ((ttGlyph > firstGlyph) && (ttGlyph < lastGlyph)) {
+            TTGlyphID newGlyph = SWAPW(trimmedArrayLookupTable->valueArray[ttGlyph - firstGlyph]);
+
+            glyphs[glyph] = LE_SET_GLYPH(glyphs[glyph], newGlyph);
         }
     }
 } 
