@@ -1,6 +1,7 @@
 /*
+ * @(#)TrimmedArrayProcessor.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -20,12 +21,10 @@
 
 U_NAMESPACE_BEGIN
 
-class LEGlyphStorage;
-
 class TrimmedArrayProcessor : public NonContextualGlyphSubstitutionProcessor
 {
 public:
-    virtual void process(LEGlyphStorage &glyphStorage);
+    virtual void process(LEGlyphID *glyphs, le_int32 *charIndices, le_int32 glyphCount);
 
     TrimmedArrayProcessor(const MorphSubtableHeader *morphSubtableHeader);
 
@@ -36,23 +35,30 @@ public:
      *
      * @stable ICU 2.8
      */
-    virtual UClassID getDynamicClassID() const;
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
      *
      * @stable ICU 2.8
      */
-    static UClassID getStaticClassID();
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
 
 private:
     TrimmedArrayProcessor();
 
 protected:
-    TTGlyphID firstGlyph;
-    TTGlyphID lastGlyph;
+    le_int16 firstGlyph;
+    le_int16 lastGlyph;
     const TrimmedArrayLookupTable *trimmedArrayLookupTable;
 
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 U_NAMESPACE_END
