@@ -1,7 +1,7 @@
 /*
  * @(#)loengine.h	1.0 00/12/11
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2003 - All Rights Reserved
  *
  */
 
@@ -9,6 +9,7 @@
 #define __LOENGINE_H
 
 #include "unicode/utypes.h"
+#include "unicode/uobject.h"
 #include "unicode/uscript.h"
 #include "unicode/unistr.h"
 
@@ -44,9 +45,10 @@ U_NAMESPACE_BEGIN
  *
  * @see LayoutEngine.h
  * @see LEFontInstance.h
+ *
+ * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
  */
-class U_LAYOUT_API ICULayoutEngine
-{
+class U_LAYOUT_API ICULayoutEngine : public UObject {
 private:
     /**
      * This holds the instance of LayoutEngine that does all
@@ -82,6 +84,8 @@ public:
      * don't know why this is...
      *
      * @see createInstance
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     virtual ~ICULayoutEngine();
 
@@ -93,7 +97,7 @@ public:
      * @param startOffset - the starting offset of the characters to process
      * @param endOffset - the ending offset of the characters to process
      * @param max - the number of characters in the input context
-     * @param rightToLeft - true if the characers are in a right to left directional run
+     * @param rightToLeft - TRUE if the characers are in a right to left directional run
      * @param x - the initial X position
      * @param y - the initial Y position
      * @param success - output parameter set to an error code if the operation fails
@@ -102,6 +106,8 @@ public:
      *
      * Note; the glyph, character index and position array can be accessed
      * using the getter method below.
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     int32_t layoutChars(const UChar chars[],
                         int32_t startOffset,
@@ -119,7 +125,7 @@ public:
      * @param str - the input character context
      * @param startOffset - the starting offset of the characters to process
      * @param endOffset - the ending offset of the characters to process
-     * @param rightToLeft - true if the characers are in a right to left directional run
+     * @param rightToLeft - TRUE if the characers are in a right to left directional run
      * @param x - the initial X position
      * @param y - the initial Y position
      * @param success - output parameter set to an error code if the operation fails
@@ -128,6 +134,8 @@ public:
      *
      * Note; the glyph, character index and position array can be accessed
      * using the getter method below.
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     int32_t layoutString(const UnicodeString &str,
                          int32_t startOffset,
@@ -142,6 +150,8 @@ public:
      * of characters used to create the LayoutEngine.
      *
      * @return the number of glyphs in the glyph array
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     int32_t countGlyphs() const;
 
@@ -152,8 +162,10 @@ public:
      *
      * @param glyphs - the destiniation glyph array
      * @param success - output parameter set to an error code if the operation fails
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
-    void getGlyphs(uint16_t glyphs[], UErrorCode &success);
+    void getGlyphs(uint32_t glyphs[], UErrorCode &success);
 
     /**
      * This method copies the character index array into a caller supplied array.
@@ -162,6 +174,8 @@ public:
      *
      * @param charIndices - the destiniation character index array
      * @param success - output parameter set to an error code if the operation fails
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     void getCharIndices(int32_t charIndices[], UErrorCode &success);
 
@@ -173,6 +187,8 @@ public:
      * @param charIndices - the destiniation character index array
      * @param indexBase - an offset which will be added to each index
      * @param success - output parameter set to an error code if the operation fails
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     void getCharIndices(int32_t charIndices[], int32_t indexBase, UErrorCode &success);
 
@@ -184,6 +200,8 @@ public:
      *
      * @param glyphs - the destiniation position array
      * @param success - output parameter set to an error code if the operation fails
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     void getGlyphPositions(float positions[], UErrorCode &success);
 
@@ -199,6 +217,7 @@ public:
      * @param y - the glyph's Y position
      * @param success - output parameter set to an error code if the operation fails
      *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     void getGlyphPosition(int32_t glyphIndex, float &x, float &y, UErrorCode &success);
 
@@ -216,10 +235,34 @@ public:
      * NOTE: currently, locale is ignored...
      *
      * @see LEFontInstance
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
      */
     static ICULayoutEngine *createInstance(const LEFontInstance *fontInstance,
-                                           UScriptCode script, Locale &locale,
+                                           UScriptCode scriptCode, Locale &locale,
                                            UErrorCode &success);
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
+     */
+    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @obsolete ICU 3.0. Use LayoutEngine.h instead since this API will be removed in that release.
+     */
+    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
+
+private:
+
+    /**
+     * The address of this static class variable serves as this class's ID
+     * for ICU "poor man's RTTI".
+     */
+    static const char fgClassID;
 };
 
 inline ICULayoutEngine::ICULayoutEngine()
@@ -265,25 +308,15 @@ inline int32_t ICULayoutEngine::layoutString(const UnicodeString &str,
                                             float x, float y,
                                             UErrorCode &success)
 {
-    int32_t glyphCount = 0;
-    int32_t max = str.length();
-    UChar *chars = new UChar[max];
-
-    str.extract(0, max, chars);
-
     // NOTE: call reset() so that clients can safely reuse
     fLayoutEngine->reset();
-    glyphCount = fLayoutEngine->layoutChars(chars,
+    return fLayoutEngine->layoutChars(str.getBuffer(),
                                       startOffset,
                                       endOffset - startOffset,
-                                      max,
+                                      str.length(),
                                       rightToLeft,
                                       x, y,
                                       (LEErrorCode &) success);
-
-    delete[] chars;
-
-    return glyphCount;
 }
 
 inline int32_t ICULayoutEngine::countGlyphs() const
@@ -291,7 +324,7 @@ inline int32_t ICULayoutEngine::countGlyphs() const
     return fLayoutEngine->getGlyphCount();
 }
 
-inline void ICULayoutEngine::getGlyphs(uint16_t glyphs[], UErrorCode &success)
+inline void ICULayoutEngine::getGlyphs(uint32_t glyphs[], UErrorCode &success)
 {
     fLayoutEngine->getGlyphs(glyphs, (LEErrorCode &) success);
 }
@@ -317,11 +350,11 @@ inline void ICULayoutEngine::getGlyphPosition(int32_t glyphIndex, float &x, floa
 }
 
 inline ICULayoutEngine *ICULayoutEngine::createInstance(const LEFontInstance *fontInstance,
-                                                        UScriptCode script,
+                                                        UScriptCode scriptCode,
                                                         Locale &locale, UErrorCode &success)
 {
     LayoutEngine *engine = LayoutEngine::layoutEngineFactory(fontInstance,
-                                                             (le_int32) script,
+                                                             (le_int32) scriptCode,
                                                              0,
                                                              (LEErrorCode &) success);
 
