@@ -1,6 +1,6 @@
 /*
  **********************************************************************
- *   Copyright (C) 2002-2007, International Business Machines
+ *   Copyright (C) 2002-2003, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
  */
@@ -10,7 +10,6 @@
  * BreakIterator...
  */
 #include "layout/LETypes.h"
-#include "layout/LEScripts.h"
 #include "layout/LELanguages.h"
 #include "layout/LayoutEngine.h"
 #include "layout/LEFontInstance.h"
@@ -30,9 +29,6 @@ U_NAMESPACE_BEGIN
 
 #define ARRAY_SIZE(array) (sizeof array  / sizeof array[0])
 
-/* Leave this copyright notice here! It needs to go somewhere in this library. */
-static const char copyright[] = U_COPYRIGHT_STRING;
-
 class StyleRuns
 {
 public:
@@ -135,7 +131,7 @@ le_int32 StyleRuns::getRuns(le_int32 runLimits[], le_int32 styleIndices[])
  * process, rather for all scripts which require
  * complex processing for correct rendering.
  */
-static const le_bool complexTable[scriptCodeCount] = {
+static const le_bool complexTable[] = {
     FALSE , /* Zyyy */
     FALSE,  /* Qaai */
     TRUE,   /* Arab */
@@ -143,7 +139,7 @@ static const le_bool complexTable[scriptCodeCount] = {
     TRUE,   /* Beng */
     FALSE,  /* Bopo */
     FALSE,  /* Cher */
-    FALSE,  /* Copt=Qaac */
+    FALSE,  /* Qaac */
     FALSE,  /* Cyrl */
     FALSE,  /* Dsrt */
     TRUE,   /* Deva */
@@ -189,69 +185,7 @@ static const le_bool complexTable[scriptCodeCount] = {
     FALSE,  /* Osma */
     FALSE,  /* Shaw */
     FALSE,  /* Tale */
-    FALSE,  /* Ugar */
-    FALSE,  /* Hrkt */
-    FALSE,  /* Bugi */
-    FALSE,  /* Glag */
-    FALSE,  /* Khar */
-    FALSE,  /* Sylo */
-    FALSE,  /* Talu */
-    FALSE,  /* Tfng */
-    FALSE,  /* Xpeo */
-    FALSE,  /* Bali */
-    FALSE,  /* Batk */
-    FALSE,  /* Blis */
-    FALSE,  /* Brah */
-    FALSE,  /* Cham */
-    FALSE,  /* Cirt */
-    FALSE,  /* Cyrs */
-    FALSE,  /* Egyd */
-    FALSE,  /* Egyh */
-    FALSE,  /* Egyp */
-    FALSE,  /* Geok */
-    FALSE,  /* Hans */
-    FALSE,  /* Hant */
-    FALSE,  /* Hmng */
-    FALSE,  /* Hung */
-    FALSE,  /* Inds */
-    FALSE,  /* Java */
-    FALSE,  /* Kali */
-    FALSE,  /* Latf */
-    FALSE,  /* Latg */
-    FALSE,  /* Lepc */
-    FALSE,  /* Lina */
-    FALSE,  /* Mand */
-    FALSE,  /* Maya */
-    FALSE,  /* Mero */
-    FALSE,  /* Nkoo */
-    FALSE,  /* Orkh */
-    FALSE,  /* Perm */
-    FALSE,  /* Phag */
-    FALSE,  /* Phnx */
-    FALSE,  /* Plrd */
-    FALSE,  /* Roro */
-    FALSE,  /* Sara */
-    FALSE,  /* Syre */
-    FALSE,  /* Syrj */
-    FALSE,  /* Syrn */
-    FALSE,  /* Teng */
-    FALSE,  /* Taii */
-    FALSE,  /* Visp */
-    FALSE,  /* Xsux */
-    FALSE,  /* Zxxx */
-    FALSE,  /* Zzzz */
-    FALSE,  /* Cari */
-    FALSE,  /* Jpan */
-    FALSE,  /* Lana */
-    FALSE,  /* Lyci */
-    FALSE,  /* Lydi */
-    FALSE,  /* Olck */
-    FALSE,  /* Rjng */
-    FALSE,  /* Saur */
-    FALSE,  /* Sgnw */
-    FALSE,  /* Sund */
-    FALSE,  /* Moon */
-    FALSE   /* Mtei */
+    FALSE   /* Ugar */
 };
 
 
@@ -283,7 +217,7 @@ ParagraphLayout::ParagraphLayout(const LEUnicode chars[], le_int32 count,
                                  const ValueRuns  *scriptRuns,
                                  const LocaleRuns *localeRuns,
                                  UBiDiLevel paragraphLevel, le_bool vertical,
-                                 LEErrorCode &status)
+								 LEErrorCode &status)
                                  : fChars(chars), fCharCount(count),
                                    fFontRuns(NULL), fLevelRuns(levelRuns), fScriptRuns(scriptRuns), fLocaleRuns(localeRuns),
                                    fVertical(vertical), fClientLevels(TRUE), fClientScripts(TRUE), fClientLocales(TRUE), fEmbeddingLevels(NULL),
@@ -296,10 +230,10 @@ ParagraphLayout::ParagraphLayout(const LEUnicode chars[], le_int32 count,
                                    fFirstVisualRun(-1), fLastVisualRun(-1),*/ fVisualRunLastX(0), fVisualRunLastY(0)
 {
 
-    if (LE_FAILURE(status)) {
-        fCharCount = -1;
-        return;
-    }
+	if (LE_FAILURE(status)) {
+		fCharCount = -1;
+		return;
+	}
 
     // FIXME: should check the limit arrays for consistency...
 
@@ -315,11 +249,11 @@ ParagraphLayout::ParagraphLayout(const LEUnicode chars[], le_int32 count,
 
     computeSubFonts(fontRuns, status);
 
-    if (LE_FAILURE(status)) {
-        //other stuff?
-        fCharCount = -1;
-        return;
-    }
+	if (LE_FAILURE(status)) {
+		//other stuff?
+		fCharCount = -1;
+		return;
+	}
 
     // now intersect the font, direction and script runs...
     const RunArray *styleRunArrays[] = {fFontRuns, fLevelRuns, fScriptRuns, fLocaleRuns};
@@ -367,13 +301,13 @@ ParagraphLayout::ParagraphLayout(const LEUnicode chars[], le_int32 count,
     // For each layout get the positions and convert them into glyph widths, in
     // logical order. Get the glyph-to-char mapping, offset by starting index in the
     // character array. Swap the glyph width and glyph-to-char arrays into logical order.
-    // Finally, fill in the char-to-glyph mappings.
+	// Finally, fill in the char-to-glyph mappings.
     fGlyphWidths       = LE_NEW_ARRAY(float, fGlyphCount);
     fGlyphToCharMap    = LE_NEW_ARRAY(le_int32, fGlyphCount + 1);
     fCharToMinGlyphMap = LE_NEW_ARRAY(le_int32, fCharCount + 1);
-    fCharToMaxGlyphMap = LE_NEW_ARRAY(le_int32, fCharCount + 1);
+	fCharToMaxGlyphMap = LE_NEW_ARRAY(le_int32, fCharCount + 1);
 
-    le_int32 glyph;
+	le_int32 glyph;
 
     for (runStart = 0, run = 0; run < fStyleRunCount; run += 1) {
         LayoutEngine *engine = fStyleRunInfo[run].engine;
@@ -404,21 +338,21 @@ ParagraphLayout::ParagraphLayout(const LEUnicode chars[], le_int32 count,
 
     fGlyphToCharMap[fGlyphCount] = fCharCount;
 
-    for (glyph = fGlyphCount - 1; glyph >= 0; glyph -= 1) {
-        le_int32 ch = fGlyphToCharMap[glyph];
+	for (glyph = fGlyphCount - 1; glyph >= 0; glyph -= 1) {
+		le_int32 ch = fGlyphToCharMap[glyph];
 
-        fCharToMinGlyphMap[ch] = glyph;
-    }
+		fCharToMinGlyphMap[ch] = glyph;
+	}
 
     fCharToMinGlyphMap[fCharCount] = fGlyphCount;
 
-    for (glyph = 0; glyph < fGlyphCount; glyph += 1) {
-        le_int32 ch = fGlyphToCharMap[glyph];
+	for (glyph = 0; glyph < fGlyphCount; glyph += 1) {
+		le_int32 ch = fGlyphToCharMap[glyph];
 
-        fCharToMaxGlyphMap[ch] = glyph;
-    }
+		fCharToMaxGlyphMap[ch] = glyph;
+	}
 
-    fCharToMaxGlyphMap[fCharCount] = fGlyphCount;
+	fCharToMaxGlyphMap[fCharCount] = fGlyphCount;
 }
 
 ParagraphLayout::~ParagraphLayout()
@@ -461,10 +395,10 @@ ParagraphLayout::~ParagraphLayout()
         fCharToMinGlyphMap = NULL;
     }
 
-    if (fCharToMaxGlyphMap != NULL) {
-        LE_DELETE_ARRAY(fCharToMaxGlyphMap);
-        fCharToMaxGlyphMap = NULL;
-    }
+	if (fCharToMaxGlyphMap != NULL) {
+		LE_DELETE_ARRAY(fCharToMaxGlyphMap);
+		fCharToMaxGlyphMap = NULL;
+	}
 
     if (fGlyphWidths != NULL) {
         LE_DELETE_ARRAY(fGlyphWidths);
@@ -515,17 +449,14 @@ le_bool ParagraphLayout::isComplex(const LEUnicode chars[], le_int32 count)
     UErrorCode scriptStatus = U_ZERO_ERROR;
     UScriptCode scriptCode  = USCRIPT_INVALID_CODE;
     UScriptRun *sr = uscript_openRun(chars, count, &scriptStatus);
-    le_bool result = FALSE;
 
     while (uscript_nextRun(sr, NULL, NULL, &scriptCode)) {
         if (isComplex(scriptCode)) {
-            result = TRUE;
-            break;
+            return TRUE;
         }
     }
 
-    uscript_closeRun(sr);
-    return result;
+    return FALSE;
 }
 
 le_int32 ParagraphLayout::getAscent() const
@@ -585,11 +516,11 @@ ParagraphLayout::Line *ParagraphLayout::nextLine(float width)
         fLineEnd = previousBreak(fGlyphToCharMap[glyph]);
 
         // If this break is at or before the last one,
-        // find a glyph, starting at the one which didn't
-        // fit, that produces a break after the last one.
-        while (fLineEnd <= fLineStart) {
-            fLineEnd = fGlyphToCharMap[glyph++];
-        }
+		// find a glyph, starting at the one which didn't
+		// fit, that produces a break after the last one.
+		while (fLineEnd <= fLineStart) {
+			fLineEnd = fGlyphToCharMap[glyph++];
+		}
     } else {
         fLineEnd = fCharCount;
     }
@@ -671,9 +602,9 @@ void ParagraphLayout::computeLocales()
 
 void ParagraphLayout::computeSubFonts(const FontRuns *fontRuns, LEErrorCode &status)
 {
-    if (LE_FAILURE(status)) {
-        return;
-    }
+	if (LE_FAILURE(status)) {
+		return;
+	}
 
     const RunArray *styleRunArrays[] = {fontRuns, fScriptRuns};
     le_int32 styleCount = sizeof styleRunArrays / sizeof styleRunArrays[0];
@@ -696,10 +627,10 @@ void ParagraphLayout::computeSubFonts(const FontRuns *fontRuns, LEErrorCode &sta
         while (offset < styleRunLimits[run]) {
             const LEFontInstance *subFont = runFont->getSubFont(fChars, &offset, styleRunLimits[run], script, status);
 
-            if (LE_FAILURE(status)) {
-                delete subFontRuns;
-                goto cleanUp;
-            }
+			if (LE_FAILURE(status)) {
+				delete subFontRuns;
+				goto cleanUp;
+			}
 
             subFontRuns->add(subFont, offset);
         }
@@ -824,7 +755,7 @@ le_int32 ParagraphLayout::getLanguageCode(const Locale *locale)
 
 le_bool ParagraphLayout::isComplex(UScriptCode script)
 {
-    if (script < 0 || script >= (UScriptCode) scriptCodeCount) {
+    if (script < 0 || script >= USCRIPT_CODE_LIMIT) {
         return FALSE;
     }
 
@@ -913,7 +844,7 @@ void ParagraphLayout::appendRun(ParagraphLayout::Line *line, le_int32 run, le_in
 
     for (ch = firstChar; ch <= lastChar; ch += 1) {
         le_int32 minGlyph = fCharToMinGlyphMap[ch];
-        le_int32 maxGlyph = fCharToMaxGlyphMap[ch];
+		le_int32 maxGlyph = fCharToMaxGlyphMap[ch];
 
         if (minGlyph < leftGlyph) {
             leftGlyph = minGlyph;
@@ -942,7 +873,7 @@ void ParagraphLayout::appendRun(ParagraphLayout::Line *line, le_int32 run, le_in
     // want the left-most glyph to start at the final x position of the
     // previous run, even though this glyph may be in the middle of the
     // run.
-    fVisualRunLastX -= fStyleRunInfo[run].positions[leftGlyph * 2];
+	fVisualRunLastX -= fStyleRunInfo[run].positions[leftGlyph * 2];
  
     // Make rightGlyph be the glyph just to the right of
     // the run's glyphs
@@ -971,13 +902,8 @@ void ParagraphLayout::appendRun(ParagraphLayout::Line *line, le_int32 run, le_in
             glyphToCharMap[outGlyph] = fGlyphToCharMap[glyphBase + inGlyph];
         }
     } else {
-        // Because fGlyphToCharMap is stored in logical order to facilitate line breaking,
-        // we need to map the physical glyph indices to logical indices while we copy the
-        // character indices.
-        le_int32 base = glyphBase + fStyleRunInfo[run].glyphCount - 1;
-
-        for (outGlyph = 0, inGlyph = leftGlyph; inGlyph < rightGlyph; inGlyph += 1, outGlyph += 1) {
-            glyphToCharMap[outGlyph] = fGlyphToCharMap[base - inGlyph];
+        for (outGlyph = 0, inGlyph = rightGlyph - 1; inGlyph >= leftGlyph; inGlyph -= 1, outGlyph += 1) {
+            glyphToCharMap[outGlyph] = fGlyphToCharMap[glyphBase + inGlyph];
         }
     }
 
@@ -1049,16 +975,11 @@ le_int32 ParagraphLayout::Line::getLeading() const
 
 le_int32 ParagraphLayout::Line::getWidth() const
 {
-    const VisualRun *lastRun = getVisualRun(fRunCount - 1);
-
-    if (lastRun == NULL) {
-        return 0;
-    }
-
-    le_int32 glyphCount = lastRun->getGlyphCount();
-    const float *positions = lastRun->getPositions();
-    
-    return (le_int32) positions[glyphCount * 2];
+	const VisualRun *lastRun = getVisualRun(fRunCount - 1);
+	le_int32 glyphCount = lastRun->getGlyphCount();
+	const float *positions = lastRun->getPositions();
+	
+	return (le_int32) positions[glyphCount * 2];
 }
 
 const ParagraphLayout::VisualRun *ParagraphLayout::Line::getVisualRun(le_int32 runIndex) const
