@@ -1,16 +1,16 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2010, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
-/*******************************************************************************
+/********************************************************************************
 *
 * File CALLCOLL.C
 *
 * Modification History:
 *        Name                     Description            
 *     Madhu Katragadda              Ported for C API
-********************************************************************************
+*********************************************************************************
 */
 
 /*
@@ -38,13 +38,9 @@
 
 #include "unicode/ucol.h"
 #include "unicode/uloc.h"
-#include "unicode/ures.h"
-#include "unicode/udata.h"
 #include "unicode/ucoleitr.h"
 #include "unicode/ustring.h"
 #include "unicode/uclean.h"
-#include "unicode/putil.h"
-#include "unicode/uenum.h"
 
 #include "cintltst.h"
 #include "ccolltst.h"
@@ -52,9 +48,7 @@
 #include "calldata.h"
 #include "cstring.h"
 #include "cmemory.h"
-
-/* set to 1 to test offsets in backAndForth() */
-#define TEST_OFFSETS 0
+#include "ucol_imp.h"
 
 /* perform test with strength PRIMARY */
 static void TestPrimary(void);
@@ -89,8 +83,6 @@ static void TestJitterbug1098(void);
 
 static void TestFCDCrash(void);
 
-static void TestJ5298(void);
-
 const UCollationResult results[] = {
     UCOL_LESS,
     UCOL_LESS, /*UCOL_GREATER,*/
@@ -134,95 +126,42 @@ const UCollationResult results[] = {
 };
 
 
-static
-void uprv_appendByteToHexString(char *dst, uint8_t val) {
-  uint32_t len = (uint32_t)uprv_strlen(dst);
-  *(dst+len) = T_CString_itosOffset((val >> 4));
-  *(dst+len+1) = T_CString_itosOffset((val & 0xF));
-  *(dst+len+2) = 0;
-}
-
-/* this function makes a string with representation of a sortkey */
-static char* U_EXPORT2 sortKeyToString(const UCollator *coll, const uint8_t *sortkey, char *buffer, uint32_t *len) {
-    int32_t strength = UCOL_PRIMARY;
-    uint32_t res_size = 0;
-    UBool doneCase = FALSE;
-    UErrorCode errorCode = U_ZERO_ERROR;
-
-    char *current = buffer;
-    const uint8_t *currentSk = sortkey;
-
-    uprv_strcpy(current, "[");
-
-    while(strength <= UCOL_QUATERNARY && strength <= ucol_getStrength(coll)) {
-        if(strength > UCOL_PRIMARY) {
-            uprv_strcat(current, " . ");
-        }
-        while(*currentSk != 0x01 && *currentSk != 0x00) { /* print a level */
-            uprv_appendByteToHexString(current, *currentSk++);
-            uprv_strcat(current, " ");
-        }
-        if(ucol_getAttribute(coll, UCOL_CASE_LEVEL, &errorCode) == UCOL_ON && strength == UCOL_SECONDARY && doneCase == FALSE) {
-            doneCase = TRUE;
-        } else if(ucol_getAttribute(coll, UCOL_CASE_LEVEL, &errorCode) == UCOL_OFF || doneCase == TRUE || strength != UCOL_SECONDARY) {
-            strength ++;
-        }
-        if (*currentSk) {
-            uprv_appendByteToHexString(current, *currentSk++); /* This should print '01' */
-        }
-        if(strength == UCOL_QUATERNARY && ucol_getAttribute(coll, UCOL_ALTERNATE_HANDLING, &errorCode) == UCOL_NON_IGNORABLE) {
-            break;
-        }
-    }
-
-    if(ucol_getStrength(coll) == UCOL_IDENTICAL) {
-        uprv_strcat(current, " . ");
-        while(*currentSk != 0) {
-            uprv_appendByteToHexString(current, *currentSk++);
-            uprv_strcat(current, " ");
-        }
-
-        uprv_appendByteToHexString(current, *currentSk++);
-    }
-    uprv_strcat(current, "]");
-
-    if(res_size > *len) {
-        return NULL;
-    }
-
-    return buffer;
-}
-
 void addAllCollTest(TestNode** root)
 {
+    
+    
     addTest(root, &TestPrimary, "tscoll/callcoll/TestPrimary");
     addTest(root, &TestSecondary, "tscoll/callcoll/TestSecondary");
     addTest(root, &TestTertiary, "tscoll/callcoll/TestTertiary");
     addTest(root, &TestIdentical, "tscoll/callcoll/TestIdentical");
     addTest(root, &TestExtra, "tscoll/callcoll/TestExtra");
-    addTest(root, &TestJB581, "tscoll/callcoll/TestJB581");
-    addTest(root, &TestVariableTop, "tscoll/callcoll/TestVariableTop");
+    addTest(root, &TestJB581, "tscoll/callcoll/TestJB581");      
+    addTest(root, &TestVariableTop, "tscoll/callcoll/TestVariableTop");      
     addTest(root, &TestSurrogates, "tscoll/callcoll/TestSurrogates");
     addTest(root, &TestInvalidRules, "tscoll/callcoll/TestInvalidRules");
-    addTest(root, &TestJB1401, "tscoll/callcoll/TestJB1401");
-    addTest(root, &TestJitterbug1098, "tscoll/callcoll/TestJitterbug1098");
+    addTest(root, &TestJB1401, "tscoll/callcoll/TestJB1401");      
+    addTest(root, &TestJitterbug1098, "tscoll/callcoll/TestJitterbug1098");     
     addTest(root, &TestFCDCrash, "tscoll/callcoll/TestFCDCrash");
-    addTest(root, &TestJ5298, "tscoll/callcoll/TestJ5298");
-}
+
+   }
 
 UBool hasCollationElements(const char *locName) {
 
   UErrorCode status = U_ZERO_ERROR;
+  UResourceBundle *ColEl = NULL;
 
-  UResourceBundle *loc = ures_open(U_ICUDATA_NAME U_TREE_SEPARATOR_STRING "coll", locName, &status);;
+  UResourceBundle *loc = ures_open(NULL, locName, &status);;
 
   if(U_SUCCESS(status)) {
     status = U_ZERO_ERROR;
-    loc = ures_getByKey(loc, "collations", loc, &status);
-    ures_close(loc);
+    ColEl = ures_getByKey(loc, "collations", ColEl, &status);
     if(status == U_ZERO_ERROR) { /* do the test - there are real elements */
+      ures_close(ColEl);
+      ures_close(loc);
       return TRUE;
     }
+    ures_close(ColEl);
+    ures_close(loc);
   }
   return FALSE;
 }
@@ -339,8 +278,7 @@ static void doTestVariant(UCollator* myCollation, const UChar source[], const UC
 
         partialSKResult = compareUsingPartials(myCollation, source, sLen, target, tLen, partialSizes[i], &status);
         if(partialSKResult != result) {
-          log_err("Partial sortkey comparison returned wrong result (%i exp. %i): %s, %s (size %i)\n", 
-            partialSKResult, result,
+          log_err("Partial sortkey comparison returned wrong result: %s, %s (size %i)\n", 
             aescstrdup(source,-1), aescstrdup(target,-1), partialSizes[i]);
         }
 
@@ -394,11 +332,11 @@ static void doTestVariant(UCollator* myCollation, const UChar source[], const UC
     gSortklen2 = uprv_strlen((const char *)sortKey2)+1;
     if(sortklen1 != gSortklen1){
         log_err("SortKey length does not match Expected: %i Got: %i\n",sortklen1, gSortklen1);
-        log_verbose("Generated sortkey: %s\n", sortKeyToString(myCollation, sortKey1, buffer, &len));
+        log_verbose("Generated sortkey: %s\n", ucol_sortKeyToString(myCollation, sortKey1, buffer, &len));
     }
     if(sortklen2!= gSortklen2){
         log_err("SortKey length does not match Expected: %i Got: %i\n", sortklen2, gSortklen2);
-        log_verbose("Generated sortkey: %s\n", sortKeyToString(myCollation, sortKey2, buffer, &len));
+        log_verbose("Generated sortkey: %s\n", ucol_sortKeyToString(myCollation, sortKey2, buffer, &len));
     }
 
     if(temp < 0) {
@@ -439,15 +377,14 @@ void doTest(UCollator* myCollation, const UChar source[], const UChar target[],
  * Return an integer array containing all of the collation orders
  * returned by calls to next on the specified iterator
  */
-OrderAndOffset* getOrders(UCollationElements *iter, int32_t *orderLength)
+int32_t* getOrders(UCollationElements *iter, int32_t *orderLength)
 {
     UErrorCode status;
     int32_t order;
     int32_t maxSize = 100;
     int32_t size = 0;
-    int32_t offset = ucol_getOffset(iter);
-    OrderAndOffset *temp;
-    OrderAndOffset *orders =(OrderAndOffset *)malloc(sizeof(OrderAndOffset) * maxSize);
+    int32_t *temp;
+    int32_t *orders =(int32_t*)malloc(sizeof(int32_t) * maxSize);
     status= U_ZERO_ERROR;
 
 
@@ -456,26 +393,22 @@ OrderAndOffset* getOrders(UCollationElements *iter, int32_t *orderLength)
         if (size == maxSize)
         {
             maxSize *= 2;
-            temp = (OrderAndOffset *)malloc(sizeof(OrderAndOffset) * maxSize);
+            temp = (int32_t*)malloc(sizeof(int32_t) * maxSize);
 
-            memcpy(temp, orders, size * sizeof(OrderAndOffset));
+            memcpy(temp, orders, size * sizeof(int32_t));
             free(orders);
             orders = temp;
 
         }
 
-        orders[size].order  = order;
-        orders[size].offset = offset;
-
-        offset = ucol_getOffset(iter);
-        size += 1;
+        orders[size++] = order;
     }
 
     if (maxSize > size && size > 0)
     {
-        temp = (OrderAndOffset *)malloc(sizeof(OrderAndOffset) * size);
+        temp = (int32_t*)malloc(sizeof(int32_t) * size);
 
-        memcpy(temp, orders, size * sizeof(OrderAndOffset));
+        memcpy(temp, orders, size * sizeof(int32_t));
         free(orders);
         orders = temp;
 
@@ -494,7 +427,8 @@ backAndForth(UCollationElements *iter)
     int32_t index, o;
     UErrorCode status = U_ZERO_ERROR;
     int32_t orderLength = 0;
-    OrderAndOffset *orders = getOrders(iter, &orderLength);
+    int32_t *orders;
+    orders= getOrders(iter, &orderLength);
 
 
     /* Now go through it backwards and make sure we get the same values */
@@ -502,67 +436,53 @@ backAndForth(UCollationElements *iter)
     ucol_reset(iter);
 
     /* synwee : changed */
-    while ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER) {
-#if TEST_OFFSETS
-      int32_t offset = 
-#endif
-        ucol_getOffset(iter);
-
-      index -= 1;
-      if (o != orders[index].order) {
+    while ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
+    {
+        if (o != orders[-- index])
+        {
         if (o == 0)
           index ++;
-        else {
-          while (index > 0 && orders[-- index].order == 0) {
-            /* nothing... */
+        else
+        {
+          while (index > 0 && orders[-- index] == 0)
+          {
           }
+          if (o != orders[index])
+          {
+            log_err("Mismatch at index : 0x%x\n", index);
+            return;
+        }
 
-          if (o != orders[index].order) {
-              log_err("Mismatched order at index %d: 0x%8.8X vs. 0x%8.8X\n", index,
-                orders[index].order, o);
-            goto bail;
-          }
         }
       }
-
-#if TEST_OFFSETS
-      if (offset != orders[index].offset) {
-        log_err("Mismatched offset at index %d: %d vs. %d\n", index,
-            orders[index].offset, offset);
-        goto bail;
-      }
-#endif
-
     }
 
-    while (index != 0 && orders[index - 1].order == 0) {
-      index -= 1;
+    while (index != 0 && orders[index - 1] == 0) {
+      index --;
     }
 
-    if (index != 0) {
+    if (index != 0)
+    {
         log_err("Didn't get back to beginning - index is %d\n", index);
 
         ucol_reset(iter);
         log_err("\nnext: ");
-
-        if ((o = ucol_next(iter, &status)) != UCOL_NULLORDER) {
+        if ((o = ucol_next(iter, &status)) != UCOL_NULLORDER)
+        {
             log_err("Error at %x\n", o);
         }
-
         log_err("\nprev: ");
-
-        if ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER) {
+        if ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
+        {
             log_err("Error at %x\n", o);
         }
-
         log_verbose("\n");
     }
 
-bail:
     free(orders);
 }
 
-void genericOrderingTestWithResult(UCollator *coll, const char * const s[], uint32_t size, UCollationResult result) {
+void genericOrderingTestWithResult(UCollator *coll, const char *s[], uint32_t size, UCollationResult result) {
   UChar t1[2048] = {0};
   UChar t2[2048] = {0};
   UCollationElements *iter;
@@ -593,11 +513,11 @@ void genericOrderingTestWithResult(UCollator *coll, const char * const s[], uint
   ucol_closeElements(iter);
 }
 
-void genericOrderingTest(UCollator *coll, const char * const s[], uint32_t size) {
+void genericOrderingTest(UCollator *coll, const char *s[], uint32_t size) {
   genericOrderingTestWithResult(coll, s, size, UCOL_LESS);
 }
 
-void genericLocaleStarter(const char *locale, const char * const s[], uint32_t size) {
+void genericLocaleStarter(const char *locale, const char *s[], uint32_t size) {
   UErrorCode status = U_ZERO_ERROR;
   UCollator *coll = ucol_open(locale, &status);
 
@@ -614,7 +534,7 @@ void genericLocaleStarter(const char *locale, const char * const s[], uint32_t s
   ucol_close(coll);
 }
 
-void genericLocaleStarterWithResult(const char *locale, const char * const s[], uint32_t size, UCollationResult result) {
+void genericLocaleStarterWithResult(const char *locale, const char *s[], uint32_t size, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   UCollator *coll = ucol_open(locale, &status);
 
@@ -631,8 +551,9 @@ void genericLocaleStarterWithResult(const char *locale, const char * const s[],
   ucol_close(coll);
 }
 
+#if 0
 /* currently not used with options */
-void genericRulesStarterWithOptionsAndResult(const char *rules, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+void genericRulesStarterWithOptions(const char *rules, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
   UErrorCode status = U_ZERO_ERROR;
   UChar rlz[RULE_BUFFER_LEN] = { 0 };
   uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
@@ -648,14 +569,15 @@ void genericRulesStarterWithOptionsAndResult(const char *rules, const char * con
       ucol_setAttribute(coll, attrs[i], values[i], &status);
     }
 
-    genericOrderingTestWithResult(coll, s, size, result);
+    genericOrderingTest(coll, s, size);
   } else {
-    log_err_status(status, "Unable to open collator with rules %s\n", rules);
+    log_err("Unable to open collator with rules %s\n", rules);
   }
   ucol_close(coll);
 }
+#endif
 
-void genericLocaleStarterWithOptionsAndResult(const char *locale, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+void genericLocaleStarterWithOptions(const char *locale, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
   UErrorCode status = U_ZERO_ERROR;
   uint32_t i;
 
@@ -670,18 +592,14 @@ void genericLocaleStarterWithOptionsAndResult(const char *locale, const char * c
       ucol_setAttribute(coll, attrs[i], values[i], &status);
     }
 
-    genericOrderingTestWithResult(coll, s, size, result);
+    genericOrderingTest(coll, s, size);
   } else {
-    log_err_status(status, "Unable to open collator for locale %s\n", locale);
+    log_err("Unable to open collator for locale %s\n", locale);
   }
   ucol_close(coll);
 }
 
-void genericLocaleStarterWithOptions(const char *locale, const char * const s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
-  genericLocaleStarterWithOptionsAndResult(locale, s, size, attrs, values, attsize, UCOL_LESS);
-}
-
-void genericRulesStarterWithResult(const char *rules, const char * const s[], uint32_t size, UCollationResult result) {
+void genericRulesTestWithResult(const char *rules, const char *s[], uint32_t size, UCollationResult result) {
   UErrorCode status = U_ZERO_ERROR;
   UChar rlz[RULE_BUFFER_LEN] = { 0 };
   uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
@@ -700,24 +618,24 @@ void genericRulesStarterWithResult(const char *rules, const char * const s[], ui
   }
 }
 
-void genericRulesStarter(const char *rules, const char * const s[], uint32_t size) {
-  genericRulesStarterWithResult(rules, s, size, UCOL_LESS);
+void genericRulesStarter(const char *rules, const char *s[], uint32_t size) {
+  genericRulesTestWithResult(rules, s, size, UCOL_LESS);
 }
 
 static void TestTertiary()
 {
     int32_t len,i;
+    UChar *rules;
     UCollator *myCollation;
     UErrorCode status=U_ZERO_ERROR;
-    static const char str[]="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
-    UChar rules[sizeof(str)];
+    const char* str="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
     len = strlen(str);
+    rules=(UChar*)malloc(sizeof(UChar*) * (len+1));
     u_uastrcpy(rules, str);
 
     myCollation=ucol_openRules(rules, len, UCOL_OFF, UCOL_DEFAULT_STRENGTH, NULL, &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of rule based collator :%s\n", myErrorName(status));
-        return;
+        log_err("ERROR: in creation of rule based collator :%s\n", myErrorName(status));
     }
    
     ucol_setStrength(myCollation, UCOL_TERTIARY);
@@ -725,6 +643,7 @@ static void TestTertiary()
     {
         doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
+    free(rules);
     ucol_close(myCollation);
     myCollation = 0;
 }
@@ -732,17 +651,17 @@ static void TestTertiary()
 static void TestPrimary( )
 {
     int32_t len,i;
+    UChar *rules;
     UCollator *myCollation;
     UErrorCode status=U_ZERO_ERROR;
-    static const char str[]="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";   
-    UChar rules[sizeof(str)];
+    const char* str="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";   
     len = strlen(str);
+    rules=(UChar*)malloc(sizeof(UChar*) * (len+1));
     u_uastrcpy(rules, str);
 
     myCollation=ucol_openRules(rules, len, UCOL_OFF, UCOL_DEFAULT_STRENGTH,NULL, &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of rule based collator :%s\n", myErrorName(status));
-        return;
+        log_err("ERROR: in creation of rule based collator :%s\n", myErrorName(status));
     }
     ucol_setStrength(myCollation, UCOL_PRIMARY);
     
@@ -751,6 +670,7 @@ static void TestPrimary( )
         
         doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
+    free(rules);
     ucol_close(myCollation);
     myCollation = 0;
 }
@@ -759,23 +679,24 @@ static void TestSecondary()
 {
     int32_t i;
     int32_t len;
+    UChar *rules;
     UCollator *myCollation;
     UErrorCode status=U_ZERO_ERROR;
-    static const char str[]="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
-    UChar rules[sizeof(str)];
+    const char* str="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
     len = strlen(str);
+    rules=(UChar*)malloc(sizeof(UChar*) * (len+1));
     u_uastrcpy(rules, str);
 
     myCollation=ucol_openRules(rules, len, UCOL_OFF, UCOL_DEFAULT_STRENGTH,NULL, &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of rule based collator :%s\n", myErrorName(status));
-        return;
+        log_err("ERROR: in creation of rule based collator :%s\n", myErrorName(status));
     }
     ucol_setStrength(myCollation, UCOL_SECONDARY);
     for (i = 26; i < 34 ; i++)
     {
         doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
+    free(rules);
     ucol_close(myCollation);
     myCollation = 0;
 }
@@ -784,22 +705,23 @@ static void TestIdentical()
 {
     int32_t i;
     int32_t len;
+    UChar *rules = 0;
     UCollator *myCollation;
     UErrorCode status=U_ZERO_ERROR;
-    static const char str[]="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
-    UChar rules[sizeof(str)];
+    const char* str="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
     len = strlen(str);
+    rules=(UChar*)malloc(sizeof(UChar*) * (len+1));
     u_uastrcpy(rules, str);
 
     myCollation=ucol_openRules(rules, len, UCOL_OFF, UCOL_IDENTICAL, NULL,&status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of rule based collator :%s\n", myErrorName(status));
-        return;
+        log_err("ERROR: in creation of rule based collator :%s\n", myErrorName(status));
     }
     for(i= 34; i<37; i++)
     {
         doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
     }
+    free(rules);
     ucol_close(myCollation);
     myCollation = 0;
 }
@@ -808,17 +730,17 @@ static void TestExtra()
 {
     int32_t i, j;
     int32_t len;
+    UChar *rules;
     UCollator *myCollation;
     UErrorCode status = U_ZERO_ERROR;
-    static const char str[]="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
-    UChar rules[sizeof(str)];
+    const char* str="& C < ch, cH, Ch, CH & Five, 5 & Four, 4 & one, 1 & Ampersand; '&' & Two, 2 ";
     len = strlen(str);
+    rules=(UChar*)malloc(sizeof(UChar*) * (len+1));
     u_uastrcpy(rules, str);
 
     myCollation=ucol_openRules(rules, len, UCOL_OFF, UCOL_DEFAULT_STRENGTH,NULL, &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of rule based collator :%s\n", myErrorName(status));
-        return;
+        log_err("ERROR: in creation of rule based collator :%s\n", myErrorName(status));
     }
     ucol_setStrength(myCollation, UCOL_TERTIARY);
     for (i = 0; i < COUNT_TEST_CASES-1 ; i++)
@@ -829,12 +751,14 @@ static void TestExtra()
             doTest(myCollation, testCases[i], testCases[j], UCOL_LESS);
         }
     }
+    free(rules);
     ucol_close(myCollation);
     myCollation = 0;
 }
 
 static void TestJB581(void)
 {
+    UChar       dispName    [100]; 
     int32_t     bufferLen   = 0;
     UChar       source      [100];
     UChar       target      [100];
@@ -853,7 +777,9 @@ static void TestJB581(void)
 
     myCollator = ucol_open("en_US", &status);
     if (U_FAILURE(status)){
-        log_err_status(status, "ERROR: Failed to create the collator : %s\n", u_errorName(status));
+        bufferLen = uloc_getDisplayName("en_US", 0, dispName, 100, &status);
+        /*Report the error with display name... */
+        log_err("ERROR: Failed to create the collator for : \"%s\"\n", dispName);
         return;
     }
     result = ucol_strcoll(myCollator, source, -1, target, -1);
@@ -873,8 +799,10 @@ static void TestJB581(void)
     /* Now, do the same comparison with keys */
     sourceKeyOut = ucol_getSortKey(myCollator, source, -1, sourceKeyArray, 100);
     targetKeyOut = ucol_getSortKey(myCollator, target, -1, targetKeyArray, 100);
+    result = 0;
     bufferLen = ((targetKeyOut > 100) ? 100 : targetKeyOut);
-    if (memcmp(sourceKeyArray, targetKeyArray, bufferLen) != 0)
+    result = memcmp(sourceKeyArray, targetKeyArray, bufferLen);
+    if (result != 0)
     {
         log_err("Comparing two strings with sort keys in C failed.\n");
     }
@@ -896,7 +824,11 @@ static void TestJB1401(void)
     
     myCollator = ucol_open("en_US", &status);
     if (U_FAILURE(status)){
-        log_err_status(status, "ERROR: Failed to create the collator : %s\n", u_errorName(status));
+        int32_t     bufferLen   = 0;
+        UChar       dispName    [100]; 
+        bufferLen = uloc_getDisplayName("en_US", 0, dispName, 100, &status);
+        /*Report the error with display name... */
+        log_err("ERROR: Failed to create the collator for : \"%s\"\n", dispName);
         return;
     }
     ucol_setAttribute(myCollator, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
@@ -982,9 +914,9 @@ static void TestJB1401(void)
 */
 static void TestVariableTop(void)
 {
-    static const char       str[]          = "&z = [variable top]";
+    const char       *str          = "&z = [variable top]";
           int         len          = strlen(str);
-          UChar      rules[sizeof(str)];
+          UChar      *rules;
           UCollator  *myCollation;
           UCollator  *enCollation;
           UErrorCode  status       = U_ZERO_ERROR;
@@ -993,18 +925,13 @@ static void TestVariableTop(void)
           uint8_t     result[20];
           uint8_t     expected[20];
 
+    rules = (UChar*)malloc(sizeof(UChar*) * (len + 1));
     u_uastrcpy(rules, str);
 
     enCollation = ucol_open("en_US", &status);
-    if (U_FAILURE(status)) {
-        log_err_status(status, "ERROR: in creation of collator :%s\n", 
-                myErrorName(status));
-        return;
-    }
     myCollation = ucol_openRules(rules, len, UCOL_OFF, 
                                  UCOL_PRIMARY,NULL, &status);
     if (U_FAILURE(status)) {
-        ucol_close(enCollation);
         log_err("ERROR: in creation of rule based collator :%s\n", 
                 myErrorName(status));
         return;
@@ -1044,6 +971,7 @@ static void TestVariableTop(void)
         ch ++;
     }
   
+    free(rules);
     ucol_close(enCollation);
     ucol_close(myCollation);
     enCollation = NULL;
@@ -1057,11 +985,11 @@ static void TestVariableTop(void)
   */
 static void TestSurrogates(void)
 {
-    static const char       str[]          = 
+    const char       *str          = 
                               "&z<'\\uD800\\uDC00'<'\\uD800\\uDC0A\\u0308'<A";
           int         len          = strlen(str);
           int         rlen         = 0;
-          UChar      rules[sizeof(str)];
+          UChar      *rules;
           UCollator  *myCollation;
           UCollator  *enCollation;
           UErrorCode  status       = U_ZERO_ERROR;
@@ -1074,18 +1002,13 @@ static void TestSurrogates(void)
           int enlen, mylen;
           
     /* tests for open rules with surrogate rules */
+    rules = (UChar*)malloc(sizeof(UChar*) * (len + 1));
     rlen = u_unescape(str, rules, len);
     
     enCollation = ucol_open("en_US", &status);
-    if (U_FAILURE(status)) {
-        log_err_status(status, "ERROR: in creation of collator :%s\n", 
-                myErrorName(status));
-        return;
-    }
     myCollation = ucol_openRules(rules, rlen, UCOL_OFF, 
                                  UCOL_TERTIARY,NULL, &status);
     if (U_FAILURE(status)) {
-        ucol_close(enCollation);
         log_err("ERROR: in creation of rule based collator :%s\n", 
                 myErrorName(status));
         return;
@@ -1118,6 +1041,7 @@ static void TestSurrogates(void)
         log_verbose("Failed : non-tailored supplementary characters should have the same value\n");
     }
 
+    free(rules);
     ucol_close(enCollation);
     ucol_close(myCollation);
     enCollation = NULL;
@@ -1163,10 +1087,10 @@ TestInvalidRules(){
         /* open the rules and test */
         coll = ucol_openRules(rules,u_strlen(rules),UCOL_OFF,UCOL_DEFAULT_STRENGTH,&parseError,&status);
         if(u_strcmp(parseError.preContext,preContextExp)!=0){
-            log_err_status(status, "preContext in UParseError for ucol_openRules does not match\n");
+            log_err("preContext in UParseError for ucol_openRules does not match\n");
         }
         if(u_strcmp(parseError.postContext,postContextExp)!=0){
-            log_err_status(status, "postContext in UParseError for ucol_openRules does not match\n");
+            log_err("postContext in UParseError for ucol_openRules does not match\n");
         }
     }  
 }
@@ -1208,13 +1132,13 @@ TestJitterbug1098(){
         u_uastrcpy(rule, rules[i]);
         c1 = ucol_openRules(rule, u_strlen(rule), UCOL_OFF, UCOL_DEFAULT_STRENGTH, &parseError, &status);
         if(U_FAILURE(status)){
-            log_err_status(status, "Could not parse the rules syntax. Error: %s\n", u_errorName(status));
+			log_err("Could not parse the rules syntax. Error: %s ", u_errorName(status));
 
-            if (status == U_PARSE_ERROR) {
-                u_UCharsToChars(parseError.preContext,preContext,20);
-                u_UCharsToChars(parseError.postContext,postContext,20);
-                log_verbose("\n\tPre-Context: %s \n\tPost-Context:%s \n",preContext,postContext);
-            }
+			if (status == U_PARSE_ERROR) {
+				u_UCharsToChars(parseError.preContext,preContext,20);
+				u_UCharsToChars(parseError.postContext,postContext,20);
+				log_verbose("\n\tPre-Context: %s \n\tPost-Context:%s \n",preContext,postContext);
+			}
 
             return;
         }
@@ -1225,75 +1149,49 @@ TestJitterbug1098(){
     }
 }
 
+/*  These tests do cleanup and reinitialize ICU in the course of their operation.
+ *    The ICU data directory must be preserved across these operations.
+ *    Here is a helper function to assist with that.
+ */
+static char *safeGetICUDataDirectory() {
+    const char *dataDir = u_getDataDirectory();  /* Returned string vanashes with u_cleanup */
+    char *retStr = NULL;
+    if (dataDir != NULL) {
+        retStr = (char *)malloc(strlen(dataDir)+1);
+        strcpy(retStr, dataDir);
+    }
+    return retStr;
+}
+
+
 static void
 TestFCDCrash(void) {
-    static const char *test[] = {
+    static char *test[] = {
     "Gr\\u00F6\\u00DFe",
     "Grossist"
     };
 
+    char *icuDataDir = safeGetICUDataDirectory();
     UErrorCode status = U_ZERO_ERROR;
     UCollator *coll = ucol_open("es", &status);
     if(U_FAILURE(status)) {
-        log_err_status(status, "Couldn't open collator -> %s\n", u_errorName(status));
+        log_err("Couldn't open collator\n");
         return;
     }
     ucol_close(coll);
     coll = NULL;
-    ctest_resetICU();
+    u_cleanup();
+    u_setDataDirectory(icuDataDir);
     coll = ucol_open("de_DE", &status);
     if(U_FAILURE(status)) {
-        log_err_status(status, "Couldn't open collator -> %s\n", u_errorName(status));
+        log_err("Couldn't open collator\n");
         return;
     }
     ucol_setAttribute(coll, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
     genericOrderingTest(coll, test, 2);
     ucol_close(coll);
+    free(icuDataDir);
+
 }
 
-/*static UBool
-find(UEnumeration* list, const char* str, UErrorCode* status){
-    const char* value = NULL;
-    int32_t length=0;
-    if(U_FAILURE(*status)){
-        return FALSE;
-    }
-    uenum_reset(list, status);
-    while( (value= uenum_next(list, &length, status))!=NULL){
-        if(strcmp(value, str)==0){
-            return TRUE;
-        }
-    }
-    return FALSE;
-}*/
-
-static void TestJ5298(void)
-{
-    UErrorCode status = U_ZERO_ERROR;
-    char input[256], output[256];
-    UBool isAvailable;
-    int32_t i = 0;
-    UEnumeration* values = NULL;
-    const char *keywordValue = NULL;
-    log_verbose("Number of collator locales returned : %i \n", ucol_countAvailable());
-    values = ucol_getKeywordValues("collation", &status);
-    for (i = 0; i < ucol_countAvailable(); i++) {
-        uenum_reset(values, &status);
-        while ((keywordValue = uenum_next(values, NULL, &status)) != NULL) {
-            strcpy(input, ucol_getAvailable(i));
-            if (strcmp(keywordValue, "standard") != 0) {
-                strcat(input, "@collation=");
-                strcat(input, keywordValue);
-            }
-
-            ucol_getFunctionalEquivalent(output, 256, "collation", input, &isAvailable, &status);
-            if (strcmp(input, output) == 0) { /* Unique locale, print it out */
-                log_verbose("%s, \n", output);
-            }
-        }
-    }
-    uenum_close(values);
-    log_verbose("\n");
-}
 #endif /* #if !UCONFIG_NO_COLLATION */
-
