@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*   file name:  cbiditst.cpp
@@ -30,10 +30,10 @@ static void
 doBiDiTest(void);
 
 static void
-doTests(UBiDi *pBiDi, UBiDi *pLine);
+doTests(UBiDi *pBiDi, UBiDi *pLine, UBool countRunsFirst);
 
 static void
-doTest(UBiDi *pBiDi, int testNumber, BiDiTestData *test, int32_t lineStart);
+doTest(UBiDi *pBiDi, int testNumber, BiDiTestData *test, int32_t lineStart, UBool countRunsFirst);
 
 static void
 testReordering(UBiDi *pBiDi, int testNumber);
@@ -100,7 +100,8 @@ doBiDiTest() {
     if(pBiDi!=NULL) {
         pLine=ubidi_open();
         if(pLine!=NULL) {
-            doTests(pBiDi, pLine);
+            doTests(pBiDi, pLine, FALSE);
+			doTests(pBiDi, pLine, TRUE);
         } else {
             log_err("ubidi_open() returned NULL, out of memory\n");
         }
@@ -119,7 +120,7 @@ doBiDiTest() {
 }
 
 static void
-doTests(UBiDi *pBiDi, UBiDi *pLine) {
+doTests(UBiDi *pBiDi, UBiDi *pLine, UBool countRunsFirst) {
     int i;
     UChar *s;
     UErrorCode errorCode;
@@ -136,13 +137,13 @@ doTests(UBiDi *pBiDi, UBiDi *pLine) {
                     i, paraLevel, ubidi_getDirection(pBiDi), ubidi_getParaLevel(pBiDi));
             lineStart=tests[i].lineStart;
             if(lineStart==-1) {
-                doTest(pBiDi, i, tests+i, 0);
+                doTest(pBiDi, i, tests+i, 0, countRunsFirst);
             } else {
                 ubidi_setLine(pBiDi, lineStart, tests[i].lineLimit, pLine, &errorCode);
                 if(U_SUCCESS(errorCode)) {
                     log_verbose("ubidi_setLine(%d, %d) ok, direction %d paraLevel=%d\n",
                             lineStart, tests[i].lineLimit, ubidi_getDirection(pLine), ubidi_getParaLevel(pLine));
-                    doTest(pLine, i, tests+i, lineStart);
+                    doTest(pLine, i, tests+i, lineStart, countRunsFirst);
                 } else {
                     log_err("ubidi_setLine(tests[%d], %d, %d) failed with errorCode %s\n",
                             i, lineStart, tests[i].lineLimit, myErrorName(errorCode));
@@ -242,10 +243,10 @@ static void TestReorder(){
     UBiDi* bidi = ubidi_open();
     int i=0;
     for(;i<(sizeof(logicalOrder)/sizeof(logicalOrder[0]));i++){
-        int32_t srcSize = uprv_strlen(logicalOrder[i]);
+        int32_t srcSize = (int32_t)uprv_strlen(logicalOrder[i]);
         int32_t destSize = srcSize*2;
-        UChar* src = (UChar*) uprv_malloc(sizeof(UChar)*srcSize );
-        UChar* dest = (UChar*) uprv_malloc(sizeof(UChar)*destSize);
+        UChar* src = (UChar*) malloc(sizeof(UChar)*srcSize );
+        UChar* dest = (UChar*) malloc(sizeof(UChar)*destSize);
         char* chars=NULL;
         ec = U_ZERO_ERROR;
         u_unescape(logicalOrder[i],src,srcSize);
@@ -273,15 +274,15 @@ static void TestReorder(){
 
             
         }
-        uprv_free(src);
-        uprv_free(dest);
+        free(src);
+        free(dest);
     }
     
     for(i=0;i<(sizeof(logicalOrder)/sizeof(logicalOrder[0]));i++){
-        int32_t srcSize = uprv_strlen(logicalOrder[i]);
+        int32_t srcSize = (int32_t)uprv_strlen(logicalOrder[i]);
         int32_t destSize = srcSize*2;
-        UChar* src = (UChar*) uprv_malloc(sizeof(UChar)*srcSize );
-        UChar* dest = (UChar*) uprv_malloc(sizeof(UChar)*destSize);
+        UChar* src = (UChar*) malloc(sizeof(UChar)*srcSize );
+        UChar* dest = (UChar*) malloc(sizeof(UChar)*destSize);
         char* chars=NULL;
         ec = U_ZERO_ERROR;
         u_unescape(logicalOrder[i],src,srcSize);
@@ -310,15 +311,15 @@ static void TestReorder(){
             
         }
         
-        uprv_free(src);
-        uprv_free(dest);
+        free(src);
+        free(dest);
     }
 
     for(i=0;i<(sizeof(logicalOrder)/sizeof(logicalOrder[0]));i++){
-        int32_t srcSize = uprv_strlen(logicalOrder[i]);
+        int32_t srcSize = (int32_t)uprv_strlen(logicalOrder[i]);
         int32_t destSize = srcSize*2;
-        UChar* src = (UChar*) uprv_malloc(sizeof(UChar)*srcSize );
-        UChar* dest = (UChar*) uprv_malloc(sizeof(UChar)*destSize);
+        UChar* src = (UChar*) malloc(sizeof(UChar)*srcSize );
+        UChar* dest = (UChar*) malloc(sizeof(UChar)*destSize);
         char* chars=NULL;
         ec = U_ZERO_ERROR;
         u_unescape(logicalOrder[i],src,srcSize);
@@ -349,15 +350,15 @@ static void TestReorder(){
             
         }
         
-        uprv_free(src);
-        uprv_free(dest);
+        free(src);
+        free(dest);
     }
         /* Max Explicit level */
     for(i=0;i<(sizeof(logicalOrder)/sizeof(logicalOrder[0]));i++){
-        int32_t srcSize = uprv_strlen(logicalOrder[i]);
+        int32_t srcSize = (int32_t)uprv_strlen(logicalOrder[i]);
         int32_t destSize = srcSize*2;
-        UChar* src = (UChar*) uprv_malloc(sizeof(UChar)*srcSize );
-        UChar* dest = (UChar*) uprv_malloc(sizeof(UChar)*destSize);
+        UChar* src = (UChar*) malloc(sizeof(UChar)*srcSize );
+        UChar* dest = (UChar*) malloc(sizeof(UChar)*destSize);
         char* chars=NULL;
         UBiDiLevel levels[UBIDI_MAX_EXPLICIT_LEVEL]={1,2,3,4,5,6,7,8,9,10};
         ec = U_ZERO_ERROR;
@@ -388,14 +389,14 @@ static void TestReorder(){
             
         }
         
-        uprv_free(src);
-        uprv_free(dest);
+        free(src);
+        free(dest);
     }
     for(i=0;i<(sizeof(logicalOrder)/sizeof(logicalOrder[0]));i++){
-        int32_t srcSize = uprv_strlen(logicalOrder[i]);
+        int32_t srcSize = (int32_t)uprv_strlen(logicalOrder[i]);
         int32_t destSize = srcSize*2;
-        UChar* src = (UChar*) uprv_malloc(sizeof(UChar)*srcSize );
-        UChar* dest = (UChar*) uprv_malloc(sizeof(UChar)*destSize);
+        UChar* src = (UChar*) malloc(sizeof(UChar)*srcSize );
+        UChar* dest = (UChar*) malloc(sizeof(UChar)*destSize);
         char* chars=NULL;
         UBiDiLevel levels[UBIDI_MAX_EXPLICIT_LEVEL]={1,2,3,4,5,6,7,8,9,10};
         ec = U_ZERO_ERROR;
@@ -425,21 +426,34 @@ static void TestReorder(){
             log_err("ubidi_writeReordered() did not give expected results for UBIDI_DO_MIRRORING+UBIDI_REMOVE_BIDI_CONTROLS. Expected: %s Got: %s At Index: %d\n",visualOrder[i],chars,i);
         }
         
-        uprv_free(src);
-        uprv_free(dest);
+        free(src);
+        free(dest);
     }
     ubidi_close(bidi);
 }
 
 static void
-doTest(UBiDi *pBiDi, int testNumber, BiDiTestData *test, int32_t lineStart) {
+doTest(UBiDi *pBiDi, int testNumber, BiDiTestData *test, int32_t lineStart, UBool countRunsFirst) {
     const uint8_t *dirProps=test->text+lineStart;
     const UBiDiLevel *levels=test->levels;
     const uint8_t *visualMap=test->visualMap;
-    int32_t i, len=ubidi_getLength(pBiDi), logicalIndex, runCount;
+    int32_t i, len=ubidi_getLength(pBiDi), logicalIndex, runCount = 0;
     UErrorCode errorCode=U_ZERO_ERROR;
     UBiDiLevel level, level2;
 
+	if (countRunsFirst) {
+		log_verbose("Calling ubidi_countRuns() first.\n");
+
+		runCount = ubidi_countRuns(pBiDi, &errorCode);
+
+		if(U_FAILURE(errorCode)) {
+			log_err("ubidi_countRuns(tests[%d]): error %s\n", testNumber, myErrorName(errorCode));
+			return;
+		}
+	} else {
+		log_verbose("Calling ubidi_getLogicalMap() first.\n");
+	}
+
     testReordering(pBiDi, testNumber);
 
     for(i=0; i<len; ++i) {
@@ -493,11 +507,13 @@ doTest(UBiDi *pBiDi, int testNumber, BiDiTestData *test, int32_t lineStart) {
         }
     }
 
-    runCount=ubidi_countRuns(pBiDi, &errorCode);
-    if(U_FAILURE(errorCode)) {
-        log_err("ubidi_countRuns(tests[%d]): error %s\n", testNumber, myErrorName(errorCode));
-        return;
-    }
+	if (! countRunsFirst) {
+		runCount=ubidi_countRuns(pBiDi, &errorCode);
+		if(U_FAILURE(errorCode)) {
+			log_err("ubidi_countRuns(tests[%d]): error %s\n", testNumber, myErrorName(errorCode));
+			return;
+		}
+	}
 
     for(logicalIndex=0; logicalIndex<len;) {
         level=ubidi_getLevelAt(pBiDi, logicalIndex);
