@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -20,11 +20,10 @@
 #include <string.h>
 #include "unicode/utypes.h"
 #include "unicode/putil.h"
-#include "cstring.h"
+
 #include "cintltst.h"
 #include "umutex.h"
-#include "uassert.h"
-#include "cmemory.h"
+
 #include "unicode/uchar.h"
 #include "unicode/ustring.h"
 #include "unicode/ucnv.h"
@@ -32,7 +31,6 @@
 #include "unicode/uclean.h"
 #include "unicode/ucal.h"
 #include "uoptions.h"
-#include "putilimp.h" /* for uprv_getUTCtime() */
 
 #ifdef XP_MAC_CONSOLE
 #   include <console.h>
@@ -106,12 +104,9 @@ int main(int argc, const char* const argv[])
     int nerrors = 0;
     int warnOnMissingData = 0;
     int i, j;
-    UBool   defaultDataFound;
     TestNode *root;
     const char *warnOrErr = "Failure"; 
     const char** argv2;
-    UDate startTime, endTime;
-    int32_t diffTime;
 
     /* initial check for the default converter */
     UErrorCode errorCode = U_ZERO_ERROR;
@@ -120,8 +115,6 @@ int main(int argc, const char* const argv[])
 
     U_MAIN_INIT_ARGS(argc, argv);
 
-    startTime = uprv_getUTCtime();
-
     argv2 = (const char**) malloc(sizeof(char*) * argc);
     if (argv2 == NULL) {
         printf("*** Error: Out of memory (too many cmd line args?)\n");
@@ -176,12 +169,10 @@ int main(int argc, const char* const argv[])
          *  Whether or not this test succeeds, we want to cleanup and reinitialize
          *  with a data path so that data loading from individual files can be tested.
          */
-        defaultDataFound = TRUE;
         u_init(&errorCode);
         if (U_FAILURE(errorCode)) {
             fprintf(stderr,
-                "#### Note:  ICU Init without build-specific setDataDirectory() failed. %s\n", u_errorName(errorCode));
-            defaultDataFound = FALSE;
+                "#### Note:  ICU Init without build-specific setDataDirectory() failed.\n");
         }
         u_cleanup();
         errorCode = U_ZERO_ERROR;
@@ -189,9 +180,7 @@ int main(int argc, const char* const argv[])
         utrace_setLevel(ICU_TRACE);
 
         /* Initialize ICU */
-        if (!defaultDataFound) {
-            ctest_setICU_DATA();    /* u_setDataDirectory() must happen Before u_init() */
-        }
+        ctest_setICU_DATA();    /* u_setDataDirectory() must happen Before u_init() */
         u_init(&errorCode);
         if (U_FAILURE(errorCode)) {
             fprintf(stderr,
@@ -240,25 +229,6 @@ int main(int argc, const char* const argv[])
             }
         }
 
-        errorCode = U_ZERO_ERROR;
-        rb = ures_open(NULL, NULL, &errorCode);
-        if(U_SUCCESS(errorCode)) {
-            /* ok */
-            if (errorCode == U_USING_DEFAULT_WARNING || errorCode == U_USING_FALLBACK_WARNING) {
-                fprintf(stderr,
-                        "#### Note: The default locale %s is not available\n", uloc_getDefault());
-            }
-            ures_close(rb);
-        } else {
-            fprintf(stderr,
-                    "*** %s! Can not open a resource bundle for the default locale %s\n", warnOrErr, uloc_getDefault());
-            if(warnOnMissingData == 0) {
-                fprintf(stderr, "*** Exitting.  Use the '-w' option if data files were\n"
-                    "*** purposely removed, to continue test anyway.\n");
-                u_cleanup();
-                return 1;
-            }
-        }
         fprintf(stdout, "Default locale for this run is %s\n", uloc_getDefault());
 
         /* Build a tree of all tests.   
@@ -283,15 +253,6 @@ int main(int argc, const char* const argv[])
     }  /* End of loop that repeats the entire test, if requested.  (Normally doesn't loop)  */
 
     free((void*)argv2);
-
-    endTime = uprv_getUTCtime();
-    diffTime = (int32_t)(endTime - startTime);
-    printf("Elapsed Time: %02d:%02d:%02d.%03d\n",
-        (int)((diffTime%U_MILLIS_PER_DAY)/U_MILLIS_PER_HOUR),
-        (int)((diffTime%U_MILLIS_PER_HOUR)/U_MILLIS_PER_MINUTE),
-        (int)((diffTime%U_MILLIS_PER_MINUTE)/U_MILLIS_PER_SECOND),
-        (int)(diffTime%U_MILLIS_PER_SECOND));
-
     return nerrors ? 1 : 0;
 }
 
@@ -334,6 +295,60 @@ static void ctest_appendToDataDirectory(const char *toAppend)
 }
 */
 
+void
+ctest_pathnameInContext( char* fullname, int32_t maxsize, const char* relPath )
+{
+    char mainDirBuffer[1024];
+    char* mainDir = NULL;
+    const char *dataDirectory = ctest_dataOutDir();
+    const char inpSepChar = '|';
+    char* tmp;
+    int32_t lenMainDir;
+    int32_t lenRelPath;
+
+#ifdef XP_MAC
+    Str255 volName;
+    int16_t volNum;
+    OSErr err = GetVol( volName, &volNum );
+    if (err != noErr)
+        volName[0] = 0;
+    mainDir = (char*) &(volName[1]);
+    mainDir[volName[0]] = 0;
+#else
+    if (dataDirectory != NULL) {
+        strcpy(mainDirBuffer, dataDirectory);
+        strcat(mainDirBuffer, ".." U_FILE_SEP_STRING);
+    } else {
+        mainDirBuffer[0]='\0';
+    }
+    mainDir = mainDirBuffer;
+#endif
+
+    lenMainDir = (int32_t)strlen(mainDir);
+    if(lenMainDir > 0 && mainDir[lenMainDir - 1] != U_FILE_SEP_CHAR) {
+        mainDir[lenMainDir++] = U_FILE_SEP_CHAR;
+        mainDir[lenMainDir] = 0;
+    }
+
+    if (relPath[0] == '|')
+        relPath++;
+    lenRelPath = (int32_t)strlen(relPath);
+    if (maxsize < lenMainDir + lenRelPath + 2) {
+        fullname[0] = 0;
+        return;
+    }
+    strcpy(fullname, mainDir);
+    /*strcat(fullname, U_FILE_SEP_STRING);*/
+    strcat(fullname, relPath);
+    strchr(fullname, inpSepChar);
+    tmp = strchr(fullname, inpSepChar);
+    while (tmp) {
+        *tmp = U_FILE_SEP_CHAR;
+        tmp = strchr(tmp+1, inpSepChar);
+    }
+}
+
+
 /* returns the path to icu/source/data */
 const char *  ctest_dataSrcDir()
 {
@@ -366,6 +381,7 @@ const char *  ctest_dataSrcDir()
         char *pBackSlash;
         int i;
 
+        dataSrcDir = p;
         strcpy(p, __FILE__);
         /* We want to back over three '\' chars.                            */
         /*   Only Windows should end up here, so looking for '\' is safe.   */
@@ -381,18 +397,10 @@ const char *  ctest_dataSrcDir()
              *  Now append "source\data" and set the environment
              */
             strcpy(pBackSlash, U_FILE_SEP_STRING "data" U_FILE_SEP_STRING );
-            dataSrcDir = p;
         }
         else {
             /* __FILE__ on MSVC7 does not contain the directory */
-            FILE *file = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "Makefile.in", "r");
-            if (file) {
-                fclose(file);
-                dataSrcDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
-            }
-            else {
-                dataSrcDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
-            }
+            strcpy(p, ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING);
         }
     }
 #endif
@@ -433,6 +441,7 @@ const char *ctest_dataOutDir()
         char *pBackSlash;
         int i;
 
+        dataOutDir = p;
         strcpy(p, __FILE__);
         /* We want to back over three '\' chars.                            */
         /*   Only Windows should end up here, so looking for '\' is safe.   */
@@ -448,18 +457,10 @@ const char *ctest_dataOutDir()
              *  Now append "source\data" and set the environment
              */
             strcpy(pBackSlash, U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "out" U_FILE_SEP_STRING);
-            dataOutDir = p;
         }
         else {
             /* __FILE__ on MSVC7 does not contain the directory */
-            FILE *file = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "Makefile.in", "r");
-            if (file) {
-                fclose(file);
-                dataOutDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "out" U_FILE_SEP_STRING;
-            }
-            else {
-                dataOutDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "out" U_FILE_SEP_STRING;
-            }
+            strcpy(p, ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "out" U_FILE_SEP_STRING);
         }
     }
 #endif
@@ -542,18 +543,12 @@ char *aescstrdup(const UChar* unichars,int32_t length){
 }
 
 const char* loadTestData(UErrorCode* err){
+    const char*      directory=NULL;
+    UResourceBundle* test =NULL;
+    char* tdpath=NULL;
+    const char* tdrelativepath = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING"out"U_FILE_SEP_STRING;
     if( _testDataPath == NULL){
-        const char*      directory=NULL;
-        UResourceBundle* test =NULL;
-        char* tdpath=NULL;
-        const char* tdrelativepath;
-#if defined (U_TOPBUILDDIR)
-        tdrelativepath = "test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING"out"U_FILE_SEP_STRING;
-        directory = U_TOPBUILDDIR;
-#else
-        tdrelativepath = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING"out"U_FILE_SEP_STRING;
         directory= ctest_dataOutDir();
-#endif
 
         tdpath = (char*) ctst_malloc(sizeof(char) *(( strlen(directory) * strlen(tdrelativepath)) + 10));
 
@@ -637,7 +632,7 @@ U_CFUNC void ctest_resetTimeZone(void) {
 #endif
 }
 
-#define CTST_MAX_ALLOC 8192
+#define CTST_MAX_ALLOC 10000
 /* Array used as a queue */
 static void * ctst_allocated_stuff[CTST_MAX_ALLOC] = {0};
 static int ctst_allocated = 0;
@@ -671,55 +666,4 @@ void ctst_freeAll() {
     ctst_allocated = 0;
     _testDataPath=NULL;
 }
-
-#define VERBOSE_ASSERTIONS
-
-U_CFUNC UBool assertSuccess(const char* msg, UErrorCode* ec) {
-    U_ASSERT(ec!=NULL);
-    if (U_FAILURE(*ec)) {
-        log_err("FAIL: %s (%s)\n", msg, u_errorName(*ec));
-        return FALSE;
-    }
-    return TRUE;
-}
-
-/* if 'condition' is a UBool, the compiler complains bitterly about
-   expressions like 'a > 0' which it evaluates as int */
-U_CFUNC UBool assertTrue(const char* msg, int /*not UBool*/ condition) { 
-    if (!condition) {
-        log_err("FAIL: assertTrue() failed: %s\n", msg);
-    }
-#ifdef VERBOSE_ASSERTIONS
-    else {
-        log_verbose("Ok: %s\n", msg);
-    }
-#endif
-    return (UBool)condition;   
-}
-
-U_CFUNC UBool assertEquals(const char* message, const char* expected,
-                           const char* actual) {
-    if (uprv_strcmp(expected, actual) != 0) {
-        log_err("FAIL: %s; got \"%s\"; expected \"%s\"\n",
-                message, actual, expected);
-        return FALSE;
-    }
-#ifdef VERBOSE_ASSERTIONS
-    else {
-        log_verbose("Ok: %s; got \"%s\"\n", message, actual);
-    }
-#endif
-    return TRUE;
-}
-/*--------------------------------------------------------------------
- * Time bomb - allows temporary behavior that expires at a given
- *             release
- *--------------------------------------------------------------------
- */
-
-U_CFUNC UBool isICUVersionAtLeast(const UVersionInfo x) {
-    UVersionInfo v;
-    u_getVersion(v);
-    return (uprv_memcmp(v, x, U_MAX_VERSION_LENGTH) >= 0);
-}
 #endif
