@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2009, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -25,7 +25,6 @@
 #include "unicode/uloc.h"
 #include "unicode/uchar.h"
 #include "unicode/ustring.h"
-#include "unicode/putil.h"
 #include "callcoll.h"
 #include "cmemory.h"
 #include "cintltst.h"
@@ -80,7 +79,7 @@ static void TestBug672() {
         UCollationElements *titer = ucol_openElements(coll, text, -1,
                                                      &status);
         if (U_FAILURE(status)) {
-            log_err_status(status, "ERROR: in creation of either the collator or the collation iterator :%s\n",
+            log_err("ERROR: in creation of either the collator or the collation iterator :%s\n",
                     myErrorName(status));
             return;
         }
@@ -156,7 +155,7 @@ static void TestBug672Normalize() {
         pitr = ucol_openElements(coll, pattern, -1, &status);
         titer = ucol_openElements(coll, text, -1, &status);
         if (U_FAILURE(status)) {
-            log_err_status(status, "ERROR: in creation of either the collator or the collation iterator :%s\n",
+            log_err("ERROR: in creation of either the collator or the collation iterator :%s\n",
                     myErrorName(status));
             return;
         }
@@ -226,7 +225,7 @@ static void TestUnicodeChar()
     UChar *test;
     en_us = ucol_open("en_US", &status);
     if (U_FAILURE(status)){
-       log_err_status(status, "ERROR: in creation of collation data using ucol_open()\n %s\n",
+       log_err("ERROR: in creation of collation data using ucol_open()\n %s\n",
               myErrorName(status));
        return;
     }
@@ -293,7 +292,7 @@ static void TestNormalizedUnicodeChar()
     /* thai should have normalization on */
     th_th = ucol_open("th_TH", &status);
     if (U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of thai collation using ucol_open()\n %s\n",
+        log_err("ERROR: in creation of thai collation using ucol_open()\n %s\n",
               myErrorName(status));
         return;
     }
@@ -369,7 +368,7 @@ static void TestNormalization()
     coll = ucol_openRules(rule, rulelen, UCOL_ON, UCOL_TERTIARY, NULL, &status);
     ucol_setAttribute(coll, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
     if (U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of collator using ucol_openRules()\n %s\n",
+        log_err("ERROR: in creation of collator using ucol_openRules()\n %s\n",
               myErrorName(status));
         return;
     }
@@ -424,9 +423,9 @@ static void TestPrevious()
     UCollator *c1, *c2, *c3;
     UCollationElements *iter;
     UErrorCode status = U_ZERO_ERROR;
-    UChar test1[50];
-    UChar test2[50];
 
+    test1=(UChar*)malloc(sizeof(UChar) * 50);
+    test2=(UChar*)malloc(sizeof(UChar) * 50);
     u_uastrcpy(test1, "What subset of all possible test cases?");
     u_uastrcpy(test2, "has the highest probability of detecting");
     coll = ucol_open("en_US", &status);
@@ -434,7 +433,7 @@ static void TestPrevious()
     iter=ucol_openElements(coll, test1, u_strlen(test1), &status);
     log_verbose("English locale testing back and forth\n");
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of collation element iterator using ucol_openElements()\n %s\n",
+        log_err("ERROR: in creation of collation element iterator using ucol_openElements()\n %s\n",
             myErrorName(status));
         ucol_close(coll);
         return;
@@ -551,6 +550,8 @@ static void TestPrevious()
     ucol_close(coll);
 
     free(source);
+    free(test1);
+    free(test2);
 }
 
 /**
@@ -562,19 +563,18 @@ static void TestOffset()
     UCollator *en_us=NULL;
     UCollationElements *iter, *pristine;
     int32_t offset;
-    OrderAndOffset *orders;
+    int32_t *orders;
     int32_t orderLength=0;
     int     count = 0;
-    UChar test1[50];
-    UChar test2[50];
-
+    test1=(UChar*)malloc(sizeof(UChar) * 50);
+    test2=(UChar*)malloc(sizeof(UChar) * 50);
     u_uastrcpy(test1, "What subset of all possible test cases?");
     u_uastrcpy(test2, "has the highest probability of detecting");
     en_us = ucol_open("en_US", &status);
     log_verbose("Testing getOffset and setOffset for collations\n");
     iter = ucol_openElements(en_us, test1, u_strlen(test1), &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of collation element iterator using ucol_openElements()\n %s\n",
+        log_err("ERROR: in creation of collation element iterator using ucol_openElements()\n %s\n",
             myErrorName(status));
         ucol_close(en_us);
         return;
@@ -649,7 +649,7 @@ static void TestOffset()
         switch (count) {
         case 0:
             if (ucol_getOffset(iter) != 1) {
-                log_err("ERROR: Offset of iteration should be 1\n");
+                log_err("ERROR: Offset of iteration should be 0\n");
             }
             break;
         case 3:
@@ -671,16 +671,10 @@ static void TestOffset()
         U_SUCCESS(status)) {
         switch (count) {
         case 0:
-        case 1:
             if (ucol_getOffset(iter) != 3) {
                 log_err("ERROR: Offset of iteration should be 3\n");
             }
             break;
-        case 2:
-            if (ucol_getOffset(iter) != 1) {
-                log_err("ERROR: Offset of iteration should be 1\n");
-            }
-            break;
         default:
             if (ucol_getOffset(iter) != 0) {
                 log_err("ERROR: Offset of iteration should be 0\n");
@@ -696,6 +690,8 @@ static void TestOffset()
 
     ucol_closeElements(iter);
     ucol_close(en_us);
+    free(test1);
+    free(test2);
 }
 
 /**
@@ -707,16 +703,15 @@ static void TestSetText()
     UErrorCode status = U_ZERO_ERROR;
     UCollator *en_us=NULL;
     UCollationElements *iter1, *iter2;
-    UChar test1[50];
-    UChar test2[50];
-
+    test1=(UChar*)malloc(sizeof(UChar) * 50);
+    test2=(UChar*)malloc(sizeof(UChar) * 50);
     u_uastrcpy(test1, "What subset of all possible test cases?");
     u_uastrcpy(test2, "has the highest probability of detecting");
     en_us = ucol_open("en_US", &status);
     log_verbose("testing setText for Collation elements\n");
     iter1=ucol_openElements(en_us, test1, u_strlen(test1), &status);
     if(U_FAILURE(status)){
-        log_err_status(status, "ERROR: in creation of collation element iterator1 using ucol_openElements()\n %s\n",
+        log_err("ERROR: in creation of collation element iterator1 using ucol_openElements()\n %s\n",
             myErrorName(status));
     ucol_close(en_us);
         return;
@@ -774,6 +769,8 @@ static void TestSetText()
     ucol_closeElements(iter2);
     ucol_closeElements(iter1);
     ucol_close(en_us);
+    free(test1);
+    free(test2);
 }
 
 /** @bug 4108762
@@ -905,7 +902,7 @@ static void TestMaxExpansion()
       ucol_closeElements(iter);
       ucol_close(coll);
     } else {
-      log_err_status(status, "Couldn't open collator -> %s\n", u_errorName(status));
+      log_data_err("Couldn't open collator\n");
     }
 
 }
@@ -943,7 +940,7 @@ static void TestSmallBuffer()
     UCollationElements *testiter,
                        *iter;
     int32_t             count = 0;
-    OrderAndOffset     *testorders,
+    int32_t            *testorders,
                        *orders;
 
     UChar teststr[500];
@@ -983,8 +980,8 @@ static void TestSmallBuffer()
 
       while (count != 0) {
           /* UCA collation element for 0x0F76 */
-          if ((count > 250 && testorders[-- count].order != orders[1].order) ||
-              (count <= 250 && testorders[-- count].order != orders[0].order)) {
+          if ((count > 250 && testorders[-- count] != orders[1]) ||
+              (count <= 250 && testorders[-- count] != orders[0])) {
               log_err("Error decomposition does not give the right collation element at %d count\n", count);
               break;
           }
@@ -1007,7 +1004,7 @@ static void TestSmallBuffer()
       ucol_closeElements(iter);
       ucol_close(coll);
     } else {
-      log_err_status(status, "Couldn't open collator -> %s\n", u_errorName(status));
+      log_data_err("Couldn't open collator\n");
     }
 }
 
@@ -1033,7 +1030,7 @@ static int32_t hex2num(char hex) {
 * @param codepoints array for storage, assuming size > 5
 * @return position at the end of the codepoint section
 */
-static char * getCodePoints(char *str, UChar *codepoints, UChar *contextCPs) {
+static char * getCodePoints(char *str, UChar *codepoints) {
     char *pStartCP = str;
     char *pEndCP   = str + 4;
 
@@ -1041,21 +1038,6 @@ static char * getCodePoints(char *str, UChar *codepoints, UChar *contextCPs) {
                           (hex2num(*(pStartCP + 1)) << 8) |
                           (hex2num(*(pStartCP + 2)) << 4) |
                           (hex2num(*(pStartCP + 3))));
-    if (*pEndCP == '|' || *(pEndCP+1) == '|') {
-        /* pre-context rule */
-        pStartCP = pEndCP;
-        while (*pStartCP==' ' || *pStartCP== '|' ) {
-            pStartCP++;
-        }
-        pEndCP = pStartCP+4;
-        *contextCPs = *codepoints;
-        *(++codepoints) = (UChar)((hex2num(*pStartCP) << 12) |
-                                  (hex2num(*(pStartCP + 1)) << 8) |
-                                  (hex2num(*(pStartCP + 2)) << 4) |
-                                  (hex2num(*(pStartCP + 3))));
-        contextCPs++;
-    }
-    *contextCPs = 0;
     codepoints ++;
     while (*pEndCP != ';') {
         pStartCP = pEndCP + 1;
@@ -1233,7 +1215,7 @@ static FileStream * getFractionalUCA(void)
     FileStream *result = NULL;
 
     /* Look inside ICU_DATA first */
-    uprv_strcpy(newPath, ctest_dataSrcDir());
+    uprv_strcpy(newPath, u_getDataDirectory());
     uprv_strcat(newPath, "unidata" U_FILE_SEP_STRING );
     uprv_strcat(newPath, "FractionalUCA.txt");
 
@@ -1269,15 +1251,14 @@ static void TestCEs() {
     FileStream *file = NULL;
     char        line[1024];
     char       *str;
-    UChar       codepoints[10];
+    UChar       codepoints[5];
     uint32_t    ces[20];
     UErrorCode  status = U_ZERO_ERROR;
     UCollator          *coll = ucol_open("", &status);
     uint32_t lineNo = 0;
-    UChar       contextCPs[5];    
 
     if (U_FAILURE(status)) {
-        log_err_status(status, "Error in opening root collator -> %s\n", u_errorName(status));
+        log_err("Error in opening root collator\n");
         return;
     }
 
@@ -1292,7 +1273,6 @@ static void TestCEs() {
     while (T_FileStream_readLine(file, line, sizeof(line)) != NULL) {
         int                 count = 0;
         UCollationElements *iter;
-        int32_t            preContextCeLen=0;
         lineNo++;
         /* skip this line if it is empty or a comment or is a return value
         or start of some variable section */
@@ -1301,7 +1281,7 @@ static void TestCEs() {
             continue;
         }
 
-        str = getCodePoints(line, codepoints, contextCPs);
+        str = getCodePoints(line, codepoints);
 
         /* these are 'fake' codepoints in the fractional UCA, and are used just 
          * for positioning of indirect values. They should not go through this
@@ -1310,19 +1290,8 @@ static void TestCEs() {
         if(*codepoints == 0xFDD0) {
           continue;
         }
-        if (*contextCPs != 0) {
-            iter = ucol_openElements(coll, contextCPs, -1, &status);
-            if (U_FAILURE(status)) {
-                log_err("Error in opening collation elements\n");
-                break;
-            }
-            while((ces[preContextCeLen] = ucol_next(iter, &status)) != (uint32_t)UCOL_NULLORDER) {
-                preContextCeLen++;
-            }
-            ucol_closeElements(iter);
-        }
 
-        getCEs(str, ces+preContextCeLen, &status);
+        getCEs(str, ces, &status);
         if (U_FAILURE(status)) {
             log_err("Error in parsing collation elements in FractionalUCA.txt\n");
             break;
@@ -1410,7 +1379,7 @@ static void TestDiscontiguos() {
     resultiter = ucol_openElements(coll, rule, 1, &status);
 
     if (U_FAILURE(status)) {
-        log_err_status(status, "Error opening collation rules -> %s\n", u_errorName(status));
+        log_err("Error opening collation rules\n");
         return;
     }
 
@@ -1480,7 +1449,7 @@ static void TestCEBufferOverflow()
     u_uastrcpy(rule, "&z < AB");
     coll = ucol_openRules(rule, u_strlen(rule), UCOL_OFF, UCOL_DEFAULT_STRENGTH, NULL,&status);
     if (U_FAILURE(status)) {
-        log_err_status(status, "Rule based collator not created for testing ce buffer overflow -> %s\n", u_errorName(status));
+        log_err("Rule based collator not created for testing ce buffer overflow\n");
         return;
     }
 
@@ -1492,9 +1461,9 @@ static void TestCEBufferOverflow()
     str[UCOL_EXPAND_CE_BUFFER_SIZE] = 0x0042;   /* 'B' */
     iter = ucol_openElements(coll, str, UCOL_EXPAND_CE_BUFFER_SIZE + 1,
                              &status);
-    if (ucol_previous(iter, &status) == UCOL_NULLORDER ||
-        status == U_BUFFER_OVERFLOW_ERROR) {
-        log_err("CE buffer should not overflow with long string of trail surrogates\n");
+    if (ucol_previous(iter, &status) != UCOL_NULLORDER ||
+        status != U_BUFFER_OVERFLOW_ERROR) {
+        log_err("CE buffer expected to overflow with long string of trail surrogates\n");
     }
     ucol_closeElements(iter);
     ucol_close(coll);
@@ -1653,19 +1622,17 @@ static void TestCEValidity()
     /* tailored locales */
     char        locale[][11] = {"fr_FR", "ko_KR", "sh_YU", "th_TH", "zh_CN", "zh__PINYIN"};
     const char *loc;
-    FileStream *file = NULL;
+    FileStream *file = getFractionalUCA();
     char        line[1024];
     UChar       codepoints[10];
     int         count = 0;
     int         maxCount = 0;
-    UChar       contextCPs[3];
     UParseError parseError;
     if (U_FAILURE(status)) {
-        log_err_status(status, "en_US collator creation failed -> %s\n", u_errorName(status));
+        log_err("en_US collator creation failed\n");
         return;
     }
     log_verbose("Testing UCA elements\n");
-    file = getFractionalUCA();
     if (file == NULL) {
         log_err("Fractional UCA data can not be opened\n");
         return;
@@ -1677,7 +1644,7 @@ static void TestCEValidity()
             continue;
         }
 
-        getCodePoints(line, codepoints, contextCPs);
+        getCodePoints(line, codepoints);
         checkCEValidity(coll, codepoints, u_strlen(codepoints), 5, 86);
     }
 
@@ -1850,19 +1817,17 @@ static void TestSortKeyValidity(void)
     /* en_US has no tailorings */
     UCollator  *coll        = ucol_open("en_US", &status);
     /* tailored locales */
-    char        locale[][6] = {"fr_FR", "ko_KR", "sh_YU", "th_TH", "zh_CN"};
-    FileStream *file = NULL;
+    char        locale[][6] = {"fr_FR\0", "ko_KR\0", "sh_YU\0", "th_TH\0", "zh_CN\0"};
+    FileStream *file = getFractionalUCA();
     char        line[1024];
     UChar       codepoints[10];
     int         count = 0;
-    UChar       contextCPs[5];
     UParseError parseError;
     if (U_FAILURE(status)) {
-        log_err_status(status, "en_US collator creation failed -> %s\n", u_errorName(status));
+        log_err("en_US collator creation failed\n");
         return;
     }
     log_verbose("Testing UCA elements\n");
-    file = getFractionalUCA();
     if (file == NULL) {
         log_err("Fractional UCA data can not be opened\n");
         return;
@@ -1874,7 +1839,7 @@ static void TestSortKeyValidity(void)
             continue;
         }
 
-        getCodePoints(line, codepoints, contextCPs);
+        getCodePoints(line, codepoints);
         checkSortKeyValidity(coll, codepoints, u_strlen(codepoints));
     }
 
