@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 2003-2006, International Business Machines Corporation and
+ * Copyright (c) 2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*
@@ -9,7 +9,6 @@
 */
 
 #include "unicode/utypes.h"
-#include "unicode/putil.h"
 #include "unicode/uclean.h"
 #include "unicode/uchar.h"
 #include "unicode/ures.h"
@@ -38,23 +37,21 @@ void addHeapMutexTest(TestNode **root);
 void
 addHeapMutexTest(TestNode** root)
 {
-    addTest(root, &TestHeapFunctions,       "hpmufn/TestHeapFunctions"  );
-    addTest(root, &TestMutexFunctions,      "hpmufn/TestMutexFunctions" );
-    addTest(root, &TestIncDecFunctions,     "hpmufn/TestIncDecFunctions");
+    addTest(root, &TestHeapFunctions,       "tsutil/hpmufn/TestHeapFunctions"  );
+    addTest(root, &TestMutexFunctions,      "tsutil/hpmufn/TestMutexFunctions" );
+    addTest(root, &TestIncDecFunctions,     "tsutil/hpmufn/TestIncDecFunctions");
 }
 
-static int32_t gMutexFailures = 0;
 
 #define TEST_STATUS(status, expected) \
 if (status != expected) { \
 log_err("FAIL at  %s:%d. Actual status = \"%s\";  Expected status = \"%s\"\n", \
-  __FILE__, __LINE__, u_errorName(status), u_errorName(expected)); gMutexFailures++; }
+__FILE__, __LINE__, u_errorName(status), u_errorName(expected)); }
 
 
 #define TEST_ASSERT(expr) \
 if (!(expr)) { \
     log_err("FAILED Assertion \"" #expr "\" at  %s:%d.\n", __FILE__, __LINE__); \
-    gMutexFailures++; \
 }
 
 
@@ -85,7 +82,7 @@ static char *safeGetICUDataDirectory() {
 int    gBlockCount = 0;
 const void  *gContext;
 
-static void * U_CALLCONV myMemAlloc(const void *context, size_t size) {
+static void   *myMemAlloc(const void *context, size_t size) {
     char *retPtr = (char *)malloc(size+sizeof(ctest_AlignedMemory));
     if (retPtr != NULL) {
         retPtr += sizeof(ctest_AlignedMemory);
@@ -94,7 +91,7 @@ static void * U_CALLCONV myMemAlloc(const void *context, size_t size) {
     return retPtr;
 }
 
-static void U_CALLCONV myMemFree(const void *context, void *mem) {
+static void  myMemFree(const void *context, void *mem) {
     char *freePtr = (char *)mem;
     if (freePtr != NULL) {
         freePtr -= sizeof(ctest_AlignedMemory);
@@ -104,7 +101,7 @@ static void U_CALLCONV myMemFree(const void *context, void *mem) {
 
 
 
-static void * U_CALLCONV myMemRealloc(const void *context, void *mem, size_t size) {
+static void  *myMemRealloc(const void *context, void *mem, size_t size) {
     char *p = (char *)mem;
     char *retPtr;
 
@@ -123,16 +120,18 @@ static void TestHeapFunctions() {
     UErrorCode       status = U_ZERO_ERROR;
     UResourceBundle *rb     = NULL;
     char            *icuDataDir;
-    UVersionInfo unicodeVersion = {0,0,0,0};
+
+    UTraceEntry   *traceEntryFunc;           /* Tracing function ptrs.  We need to save    */
+    UTraceExit    *traceExitFunc;            /*  and restore them across calls to          */
+    UTraceData    *traceDataFunc;            /*  u_cleanup() that we make in this test.    */
+    const void    *traceContext;
+    int32_t        traceLevel;
 
     icuDataDir = safeGetICUDataDirectory();   /* save icu data dir, so we can put it back
                                                *  after doing u_cleanup().                */
 
-
-    /* Verify that ICU can be cleaned up and reinitialized successfully.
-     *  Failure here usually means that some ICU service didn't clean up successfully,
-     *  probably because some earlier test accidently left something open. */
-    ctest_resetICU();
+    utrace_getFunctions(&traceContext, &traceEntryFunc, &traceExitFunc, &traceDataFunc);
+    traceLevel = utrace_getLevel();
 
     /* Can not set memory functions if ICU is already initialized */
     u_setMemoryFunctions(&gContext, myMemAlloc, myMemRealloc, myMemFree, &status);
@@ -140,6 +139,8 @@ static void TestHeapFunctions() {
 
     /* Un-initialize ICU */
     u_cleanup();
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
 
     /* Can not set memory functions with NULL values */
     status = U_ZERO_ERROR;
@@ -179,11 +180,10 @@ static void TestHeapFunctions() {
     ures_close(rb);
 
     /* Cleanup should put the heap back to its default implementation. */
-    ctest_resetICU();
-    u_getUnicodeVersion(unicodeVersion);
-    if (unicodeVersion[0] <= 0) {
-        log_err("Properties doesn't reinitialize without u_init.\n");
-    }
+    u_cleanup();
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
+    u_setDataDirectory(icuDataDir);
     status = U_ZERO_ERROR;
     u_init(&status);
     TEST_STATUS(status, U_ZERO_ERROR);
@@ -198,8 +198,6 @@ static void TestHeapFunctions() {
     }
     ures_close(rb);
     free(icuDataDir);
-
-    ctest_resetICU();
 }
 
 
@@ -218,7 +216,7 @@ typedef struct DummyMutex {
 } DummyMutex;
 
 
-static void U_CALLCONV myMutexInit(const void *context, UMTX *mutex, UErrorCode *status) {
+static void myMutexInit(const void *context, UMTX *mutex, UErrorCode *status) {
     DummyMutex *theMutex;
 
     TEST_STATUS(*status, U_ZERO_ERROR);
@@ -232,7 +230,7 @@ static void U_CALLCONV myMutexInit(const void *context, UMTX *mutex, UErrorCode
 }
 
 
-static void U_CALLCONV myMutexDestroy(const void *context, UMTX  *mutex) {
+static void myMutexDestroy(const void *context, UMTX  *mutex) {
     DummyMutex *This = *(DummyMutex **)mutex;
 
     gTotalMutexesActive--;
@@ -243,7 +241,7 @@ static void U_CALLCONV myMutexDestroy(const void *context, UMTX  *mutex) {
     free(This);
 }
 
-static void U_CALLCONV myMutexLock(const void *context, UMTX *mutex) {
+static void  myMutexLock(const void *context, UMTX *mutex) {
     DummyMutex *This = *(DummyMutex **)mutex;
 
     TEST_ASSERT(This->fMagic == 123456);
@@ -251,7 +249,7 @@ static void U_CALLCONV myMutexLock(const void *context, UMTX *mutex) {
     gAccumulatedLocks++;
 }
 
-static void U_CALLCONV myMutexUnlock(const void *context, UMTX *mutex) {
+static void  myMutexUnlock(const void *context, UMTX *mutex) {
     DummyMutex *This = *(DummyMutex **)mutex;
 
     TEST_ASSERT(This->fMagic == 123456);
@@ -266,17 +264,18 @@ static void TestMutexFunctions() {
     UResourceBundle *rb     = NULL;
     char            *icuDataDir;
 
-    gMutexFailures = 0;
+    UTraceEntry     *traceEntryFunc;           /* Tracing function ptrs.  We need to save    */
+    UTraceExit      *traceExitFunc;            /*  and restore them across calls to          */
+    UTraceData      *traceDataFunc;            /*  u_cleanup() that we make in this test.    */
+    const void      *traceContext;
+    int32_t          traceLevel;
 
     /*  Save initial ICU state so that it can be restored later.
      *  u_cleanup(), which is called in this test, resets ICU's state.
      */
     icuDataDir = safeGetICUDataDirectory();
-
-    /* Verify that ICU can be cleaned up and reinitialized successfully.
-     *  Failure here usually means that some ICU service didn't clean up successfully,
-     *  probably because some earlier test accidently left something open. */
-    ctest_resetICU();
+    utrace_getFunctions(&traceContext, &traceEntryFunc, &traceExitFunc, &traceDataFunc);
+    traceLevel = utrace_getLevel();
 
     /* Can not set mutex functions if ICU is already initialized */
     u_setMutexFunctions(&gContext, myMutexInit, myMutexDestroy, myMutexLock, myMutexUnlock, &status);
@@ -284,6 +283,8 @@ static void TestMutexFunctions() {
 
     /* Un-initialize ICU */
     u_cleanup();
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
 
     /* Can not set Mutex functions with NULL values */
     status = U_ZERO_ERROR;
@@ -326,7 +327,10 @@ static void TestMutexFunctions() {
     ures_close(rb);
 
     /* Cleanup should destroy all of the mutexes. */
-    ctest_resetICU();
+    u_cleanup();
+    u_setDataDirectory(icuDataDir);
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
     status = U_ZERO_ERROR;
     TEST_ASSERT(gTotalMutexesInitialized > 0);
     TEST_ASSERT(gTotalMutexesActive == 0);
@@ -346,11 +350,6 @@ static void TestMutexFunctions() {
 
     ures_close(rb);
     free(icuDataDir);
-
-    if(gMutexFailures) {
-      log_info("Note: these failures may be caused by ICU failing to initialize/uninitialize properly.\n");
-      log_verbose("Check for prior tests which may not have closed all open resources. See the internal function ures_flushCache()\n");
-    }
 }
 
 
@@ -363,20 +362,19 @@ static void TestMutexFunctions() {
 int         gIncCount             = 0;
 int         gDecCount             = 0;
 const void *gIncDecContext;
-const void *gExpectedContext = &gIncDecContext;
 
 
-static int32_t U_CALLCONV myIncFunc(const void *context, int32_t *p) {
+static int32_t myIncFunc(const void *context, int32_t *p) {
     int32_t  retVal;
-    TEST_ASSERT(context == gExpectedContext);
+    TEST_ASSERT(context == gIncDecContext);
     gIncCount++;
     retVal = ++(*p);
     return retVal;
 }
 
-static int32_t U_CALLCONV myDecFunc(const void *context, int32_t *p) {
+static int32_t myDecFunc(const void *context, int32_t *p) {
     int32_t  retVal;
-    TEST_ASSERT(context == gExpectedContext);
+    TEST_ASSERT(context == gIncDecContext);
     gDecCount++;
     retVal = --(*p);
     return retVal;
@@ -390,21 +388,23 @@ static void TestIncDecFunctions() {
     int32_t      t = 1; /* random value to make sure that Inc/dec works */
     char         *dataDir;
 
-    /* Save ICU's data dir and tracing functions so that they can be resored 
-       after cleanup and reinit.  */
-    dataDir = safeGetICUDataDirectory();
-
-    /* Verify that ICU can be cleaned up and reinitialized successfully.
-     *  Failure here usually means that some ICU service didn't clean up successfully,
-     *  probably because some earlier test accidently left something open. */
-    ctest_resetICU();
+    UTraceEntry     *traceEntryFunc;           /* Tracing function ptrs.  We need to save    */
+    UTraceExit      *traceExitFunc;            /*  and restore them across calls to          */
+    UTraceData      *traceDataFunc;            /*  u_cleanup() that we make in this test.    */
+    const void      *traceContext;
+    int32_t          traceLevel;
 
     /* Can not set mutex functions if ICU is already initialized */
     u_setAtomicIncDecFunctions(&gIncDecContext, myIncFunc, myDecFunc,  &status);
     TEST_STATUS(status, U_INVALID_STATE_ERROR);
 
-    /* Clean up ICU */
+    /* Un-initialize ICU */
+    dataDir = safeGetICUDataDirectory();
+    utrace_getFunctions(&traceContext, &traceEntryFunc, &traceExitFunc, &traceDataFunc);
+    traceLevel = utrace_getLevel();
     u_cleanup();
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
 
     /* Can not set functions with NULL values */
     status = U_ZERO_ERROR;
@@ -416,10 +416,8 @@ static void TestIncDecFunctions() {
 
     /* u_setIncDecFunctions() should work with null or non-null context pointer */
     status = U_ZERO_ERROR;
-    gExpectedContext = NULL;
     u_setAtomicIncDecFunctions(NULL, myIncFunc, myDecFunc,  &status);
     TEST_STATUS(status, U_ZERO_ERROR);
-    gExpectedContext = &gIncDecContext;
     u_setAtomicIncDecFunctions(&gIncDecContext, myIncFunc, myDecFunc,  &status);
     TEST_STATUS(status, U_ZERO_ERROR);
 
@@ -429,7 +427,6 @@ static void TestIncDecFunctions() {
     u_setDataDirectory(dataDir);
     u_init(&status);
     TEST_STATUS(status, U_ZERO_ERROR);
-    gExpectedContext = &gIncDecContext;
     u_setAtomicIncDecFunctions(&gIncDecContext, myIncFunc, myDecFunc,  &status);
     TEST_STATUS(status, U_INVALID_STATE_ERROR);
 
@@ -446,7 +443,9 @@ static void TestIncDecFunctions() {
 
     /* Cleanup should cancel use of our inc/dec functions. */
     /* Additional ICU operations should not use them */
-    ctest_resetICU();
+    u_cleanup();
+    utrace_setFunctions(traceContext, traceEntryFunc, traceExitFunc, traceDataFunc);
+    utrace_setLevel(traceLevel);
     gIncCount = 0;
     gDecCount = 0;
     status = U_ZERO_ERROR;
