@@ -1,16 +1,16 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2007, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ***************************************************************************/
-/*****************************************************************************
+/*******************************************************************************
 *
 * File NCNVCBTS
 *
 * Modification History:
 *      Name              Date                  Description            
 * Madhu Katragadda    06/23/2000     Tests for Conveter FallBack API and Functionality
-******************************************************************************
+**********************************************************************************
 */
 #include <stdio.h>
 #include "unicode/uloc.h"
@@ -23,7 +23,6 @@
 #include "cmemory.h"
 #include "cstring.h"
 
-#if !UCONFIG_NO_LEGACY_CONVERSION
 #define NEW_MAX_BUFFER 999
 
 
@@ -116,11 +115,11 @@ void addTestConverterFallBack(TestNode** root)
 
 static void setNuConvTestName(const char *codepage, const char *direction)
 {
-    sprintf(gNuConvTestName, "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
-        codepage,
-        direction,
-        (int)gInBufferSize,
-        (int)gOutBufferSize);
+  sprintf(gNuConvTestName, "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
+      codepage,
+      direction,
+      gInBufferSize,
+      gOutBufferSize);
 }
 
 
@@ -131,25 +130,25 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
 
     UErrorCode status = U_ZERO_ERROR;
     UConverter *conv = 0;
-    char junkout[NEW_MAX_BUFFER]; /* FIX */
+    uint8_t junkout[NEW_MAX_BUFFER]; /* FIX */
     int32_t junokout[NEW_MAX_BUFFER]; /* FIX */
     const UChar *src;
-    char *end;
-    char *targ;
+    uint8_t *end;
+    uint8_t *targ;
     int32_t *offs;
     int i;
     int32_t   realBufferSize;
-    char *realBufferEnd;
+    uint8_t *realBufferEnd;
     const UChar *realSourceEnd;
     const UChar *sourceLimit;
     UBool checkOffsets = TRUE;
     UBool doFlush;
     UBool action=FALSE;
-    char *p;
+    uint8_t *p;
 
 
     for(i=0;i<NEW_MAX_BUFFER;i++)
-        junkout[i] = (char)0xF0;
+        junkout[i] = 0xF0;
     for(i=0;i<NEW_MAX_BUFFER;i++)
         junokout[i] = 0xFF;
     setNuConvTestName(codepage, "FROM");
@@ -251,7 +250,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     {
         log_err("Expected %d chars out, got %d %s\n", expectLen, targ-junkout, gNuConvTestName);
         log_verbose("Expected %d chars out, got %d %s\n", expectLen, targ-junkout, gNuConvTestName);
-        printSeqErr((const unsigned char*)junkout, (int32_t)(targ-junkout));
+        printSeqErr((const unsigned char*)junkout, targ-junkout);
         printSeqErr((const unsigned char*)expect, expectLen);
         return FALSE;
     }
@@ -262,7 +261,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
         if(uprv_memcmp(junokout,expectOffsets,(targ-junkout) * sizeof(int32_t) )){
             log_err("\ndid not get the expected offsets while %s \n", gNuConvTestName);
             log_err("Got  : ");
-            printSeqErr((const unsigned char*)junkout, (int32_t)(targ-junkout));
+            printSeqErr((const unsigned char*)junkout, targ-junkout);
             for(p=junkout;p<targ;p++)
                 log_err("%d, ", junokout[p-junkout]); 
             log_err("\nExpected: ");
@@ -294,9 +293,9 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     UConverter *conv = 0;
     UChar   junkout[NEW_MAX_BUFFER]; /* FIX */
     int32_t junokout[NEW_MAX_BUFFER]; /* FIX */
-    const char *src;
-    const char *realSourceEnd;
-    const char *srcLimit;
+    const uint8_t *src;
+    const uint8_t *realSourceEnd;
+    const uint8_t *srcLimit;
     UChar *targ;
     UChar *end;
     int32_t *offs;
@@ -330,7 +329,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
 
     log_verbose("Converter opened..\n");
 
-    src = (const char *)source;
+    src = source;
     targ = junkout;
     offs = junokout;
 
@@ -425,7 +424,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
             for(i=0; i<(targ-junkout); i++)
                 log_err("0x%04X,", junkout[i]);
             log_err("");
-            for(i=0; i<(src-(const char *)source); i++)
+            for(i=0; i<(src-source); i++)
                 log_err("0x%04X,", (unsigned char)source[i]);
         }
     }
@@ -519,10 +518,9 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
     
     static const char* nativeCodePage[]={
         /*NLCS Mapping*/
+        "ibm-367",
         "ibm-437",
         "ibm-850",
-        "ibm-878",
-        "ibm-923",
         "ibm-1051",
         "ibm-1089",
         "ibm-1250",
@@ -533,7 +531,8 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         "ibm-1256",
         "ibm-1257",
         "ibm-1258",
-        "ibm-1276"
+        "ibm-1276",
+        "ibm-1277"
     };
 
     int32_t i=0;
@@ -746,10 +745,3 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
     }
 
 }
-#else
-void addTestConverterFallBack(TestNode** root)
-{
-  /* test nothing... */
- 
-}
-#endif
