@@ -1,5 +1,5 @@
 /********************************************************************
- * Copyright (c) 2001-2007 International Business Machines 
+ * Copyright (c) 2001-2003 International Business Machines 
  * Corporation and others. All Rights Reserved.
  ********************************************************************
  * File usrchtst.c
@@ -141,8 +141,20 @@ static char *toCharString(const UChar* unichars)
             *temp ++ = (char)ch;
         }
         else {
-            sprintf(temp, "\\u%04x", ch);
-            temp += 6; /* \uxxxx */
+            char digit[5];
+            int  zerosize;
+            *temp = 0;
+            strcat(temp, "\\u");
+            temp = temp + 2;
+            sprintf(digit, "%x", ch);
+            zerosize = 4 - strlen(digit);
+            while (zerosize != 0) {
+                *temp ++ = '0';
+                zerosize --;
+            }
+            *temp = 0;
+            strcat(temp, digit);
+            temp = temp + strlen(digit);
         }
     }
     *temp = 0;
@@ -356,7 +368,7 @@ static UBool assertEqualWithUStringSearch(      UStringSearch *strsrch,
         matchindex = search.offset[count];
     }
     usearch_next(strsrch, &status);
-    if (usearch_getMatchedStart(strsrch) != USEARCH_DONE ||
+    if ((uint32_t)usearch_getMatchedStart(strsrch) != USEARCH_DONE ||
         usearch_getMatchedLength(strsrch) != 0) {
         char *str = toCharString(usearch_getText(strsrch, &textlength));
         log_err("Text: %s\n", str);
@@ -399,7 +411,7 @@ static UBool assertEqualWithUStringSearch(      UStringSearch *strsrch,
         count --;
     }
     usearch_previous(strsrch, &status);
-    if (usearch_getMatchedStart(strsrch) != USEARCH_DONE ||
+    if ((uint32_t)usearch_getMatchedStart(strsrch) != USEARCH_DONE ||
         usearch_getMatchedLength(strsrch) != 0) {
         char *str = toCharString(usearch_getText(strsrch, &textlength));
         log_err("Text: %s\n", str);
@@ -1120,7 +1132,7 @@ static void TestGetSetOffset(void)
             count += 2;
         }
         usearch_next(strsrch, &status);
-        if (usearch_getMatchedStart(strsrch) != USEARCH_DONE) {
+        if ((uint32_t)usearch_getMatchedStart(strsrch) != USEARCH_DONE) {
             char *str = toCharString(usearch_getText(strsrch, &textlength));
             log_err("Text: %s\n", str);
             str = toCharString(usearch_getPattern(strsrch, &textlength));
@@ -2036,7 +2048,7 @@ static void TestGetSetOffsetCanonical(void)
             count += 2;
         }
         usearch_next(strsrch, &status);
-        if (usearch_getMatchedStart(strsrch) != USEARCH_DONE) {
+        if ((uint32_t)usearch_getMatchedStart(strsrch) != USEARCH_DONE) {
             char *str = toCharString(usearch_getText(strsrch, &textlength));
             log_err("Text: %s\n", str);
             str = toCharString(usearch_getPattern(strsrch, &textlength));
@@ -2069,10 +2081,10 @@ static void TestContractionCanonical(void)
     UChar          rules[128];
     UChar          pattern[128];
     UChar          text[128];
-    UCollator     *collator = NULL;
+    UCollator     *collator;
     UErrorCode     status = U_ZERO_ERROR;
     int            count = 0;
-    UStringSearch *strsrch = NULL;
+    UStringSearch *strsrch;
     memset(rules, 0, 128*sizeof(UChar));
     memset(pattern, 0, 128*sizeof(UChar));
     memset(text, 0, 128*sizeof(UChar));
@@ -2110,37 +2122,6 @@ static void TestContractionCanonical(void)
     ucol_close(collator);
 }
 
-static void TestNumeric(void) {
-    UCollator     *coll = NULL;
-    UStringSearch *strsrch = NULL;
-    UErrorCode     status = U_ZERO_ERROR;
-
-    UChar          pattern[128];
-    UChar          text[128];
-    memset(pattern, 0, 128*sizeof(UChar));
-    memset(text, 0, 128*sizeof(UChar));
-
-    coll = ucol_open("", &status);
-    if(U_FAILURE(status)) {
-        log_data_err("Could not open UCA. Is your data around?\n");
-        return;
-    }
-
-    ucol_setAttribute(coll, UCOL_NUMERIC_COLLATION, UCOL_ON, &status);
-
-    strsrch = usearch_openFromCollator(pattern, 1, text, 1, coll, NULL, &status);
-
-    if(status != U_UNSUPPORTED_ERROR || U_SUCCESS(status)) {
-        log_err("Expected U_UNSUPPORTED_ERROR when trying to instantiate a search object from a CODAN collator, got %s instead\n", u_errorName(status));
-        if(strsrch) {
-            usearch_close(strsrch);
-        }
-    }
-
-    ucol_close(coll);
-
-}
-
 void addSearchTest(TestNode** root)
 {
     addTest(root, &TestStart, "tscoll/usrchtst/TestStart");
@@ -2190,7 +2171,6 @@ void addSearchTest(TestNode** root)
     addTest(root, &TestContractionCanonical, 
                                  "tscoll/usrchtst/TestContractionCanonical");
     addTest(root, &TestEnd, "tscoll/usrchtst/TestEnd");
-    addTest(root, &TestNumeric, "tscoll/usrchtst/TestNumeric");
 }
 
 #endif /* #if !UCONFIG_NO_COLLATION */
