@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2007, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 //===============================================================================
@@ -39,11 +39,9 @@
 #include "unicode/chariter.h"
 #include "unicode/schriter.h"
 #include "unicode/ustring.h"
-#include "unicode/ucol.h"
 
 #include "sfwdchit.h"
 #include "cmemory.h"
-#include <stdlib.h>
 
 void
 CollationAPITest::doAssert(UBool condition, const char *message)
@@ -101,10 +99,8 @@ CollationAPITest::TestProperty(/* char* par */)
       ICU 2.2 currVersionArray = {0x21, 0x40, 0x04, 0x04};
       ICU 2.4 currVersionArray = {0x21, 0x40, 0x04, 0x04};
       ICU 2.6 currVersionArray = {0x21, 0x40, 0x03, 0x03};
-      ICU 2.8 currVersionArray = {0x29, 0x80, 0x00, 0x04};
-      ICU 3.4 currVersionArray = {0x31, 0xC0, 0x00, 0x04};
     */
-    UVersionInfo currVersionArray = {0x31, 0xC0, 0x00, 0x05};
+    UVersionInfo currVersionArray = {0x29, 0x80, 0x01, 0x04};
     UVersionInfo versionArray;
     int i = 0;
 
@@ -225,16 +221,11 @@ CollationAPITest::TestProperty(/* char* par */)
     if (U_FAILURE(success))
     {
         errln("Creating French collator failed.");
-        delete col;
-        delete junk;
+        delete col; delete junk;
         return;
     }
 
-    // If the default locale isn't French, the French and non-French collators
-    // should be different
-    if (frCol->getLocale(ULOC_ACTUAL_LOCALE, success) != Locale::getFrench()) {
-        doAssert((*frCol != *junk), "The junk is the same as the French collator.");
-    }
+    doAssert((*frCol != *junk), "The junk is the same as the French collator.");
     Collator *aFrCol = frCol->clone();
     doAssert((*frCol == *aFrCol), "The cloning of a French collator failed.");
     logln("Collator property test ended.");
@@ -364,8 +355,8 @@ CollationAPITest::TestRules()
     }
 
     coll->getRules(UCOL_TAILORING_ONLY, rules);
-    if (rules.length() != 0x0a) {
-      errln("English tailored rules failed - length is 0x%x expected 0x%x", rules.length(), 0x0e);
+    if (rules.length() != 0x0f) {
+        errln("English tailored rules failed");
     }
 
     coll->getRules(UCOL_FULL_RULES, rules);
@@ -390,17 +381,17 @@ CollationAPITest::TestDecomposition() {
   /* there is no reason to have canonical decomposition in en_US OR default locale */
   if (vi_VN->getAttribute(UCOL_NORMALIZATION_MODE, status) != UCOL_ON)
   {
-    errln("ERROR: vi_VN collation did not have canonical decomposition for normalization!\n");
+    errln("ERROR: vi_VN collation did not have cannonical decomposition for normalization!\n");
   }
 
   if (el_GR->getAttribute(UCOL_NORMALIZATION_MODE, status) != UCOL_ON)
   {
-    errln("ERROR: el_GR collation did not have canonical decomposition for normalization!\n");
+    errln("ERROR: el_GR collation did not have cannonical decomposition for normalization!\n");
   }
 
   if (en_US->getAttribute(UCOL_NORMALIZATION_MODE, status) != UCOL_OFF)
   {
-    errln("ERROR: en_US collation had canonical decomposition for normalization!\n");
+    errln("ERROR: en_US collation had cannonical decomposition for normalization!\n");
   }
 
   delete en_US;
@@ -919,43 +910,12 @@ CollationAPITest::TestDuplicate(/* char* par */)
     }
     Collator *col2 = col1->clone();
     doAssert((*col1 == *col2), "Cloned object is not equal to the orginal");
-    UnicodeString *ruleset = new UnicodeString("< a, A < b, B < c, C < d, D, e, E");
-    RuleBasedCollator *col3 = new RuleBasedCollator(*ruleset, status);
+    UnicodeString ruleset("< a, A < b, B < c, C < d, D, e, E");
+    RuleBasedCollator *col3 = new RuleBasedCollator(ruleset, status);
     doAssert((*col1 != *col3), "Cloned object is equal to some dummy");
     *col3 = *((RuleBasedCollator*)col1);
     doAssert((*col1 == *col3), "Copied object is not equal to the orginal");
-
-    if (U_FAILURE(status)) {
-        logln("Collation tailoring failed.");
-        return;
-    }
-
-    UCollationResult res;
-    UnicodeString first((UChar)0x0061);
-    UnicodeString second((UChar)0x0062);
-    UnicodeString copiedEnglishRules(((RuleBasedCollator*)col1)->getRules());
-
     delete col1;
-    delete ruleset;
-
-    // Try using the cloned collators after deleting the original data
-    res = col2->compare(first, second, status);
-    if(res != UCOL_LESS) {
-        errln("a should be less then b after tailoring");
-    }
-    if (((RuleBasedCollator*)col2)->getRules() != copiedEnglishRules) {
-        errln(UnicodeString("English rule difference. ")
-            + copiedEnglishRules + UnicodeString("\ngetRules=") + ((RuleBasedCollator*)col2)->getRules());
-    }
-    res = col3->compare(first, second, status);
-    if(res != UCOL_LESS) {
-        errln("a should be less then b after tailoring");
-    }
-    if (col3->getRules() != copiedEnglishRules) {
-        errln(UnicodeString("English rule difference. ")
-            + copiedEnglishRules + UnicodeString("\ngetRules=") + col3->getRules());
-    }
-
     delete col2;
     delete col3;
 }
@@ -1022,61 +982,15 @@ CollationAPITest::TestCompare(/* char* par */)
 void
 CollationAPITest::TestGetAll(/* char* par */)
 {
-    int32_t count1, count2;
-    UErrorCode status = U_ZERO_ERROR;
-
-    logln("Trying Collator::getAvailableLocales(int&)");
-
-    const Locale* list = Collator::getAvailableLocales(count1);
-    for (int32_t i = 0; i < count1; ++i) {
-        UnicodeString dispName;
-        logln(UnicodeString("Locale name: ")
-            + UnicodeString(list[i].getName())
-            + UnicodeString(" , the display name is : ")
-            + UnicodeString(list[i].getDisplayName(dispName)));
+    int32_t count;
+    const Locale* list = Collator::getAvailableLocales(count);
+    for (int32_t i = 0; i < count; ++i) {
+        UnicodeString locName, dispName;
+        log("Locale name: ");
+        log(list[i].getName());
+        log(" , the display name is : ");
+        logln(list[i].getDisplayName(dispName));
     }
-
-    if (count1 == 0 || list == NULL) {
-        errln("getAvailableLocales(int&) returned an empty list");
-    }
-
-    logln("Trying Collator::getAvailableLocales()");
-    StringEnumeration* localeEnum = Collator::getAvailableLocales();
-    const UnicodeString* locStr;
-    const char *locCStr;
-    count2 = 0;
-
-    if (localeEnum == NULL) {
-        errln("getAvailableLocales() returned NULL");
-        return;
-    }
-
-    while ((locStr = localeEnum->snext(status)) != NULL)
-    {
-        logln(UnicodeString("Locale name is: ") + *locStr);
-        count2++;
-    }
-    if (count1 != count2) {
-        errln("getAvailableLocales(int&) returned %d and getAvailableLocales() returned %d", count1, count2);
-    }
-
-    logln("Trying Collator::getAvailableLocales() clone");
-    count1 = 0;
-    StringEnumeration* localeEnum2 = localeEnum->clone();
-    localeEnum2->reset(status);
-    while ((locCStr = localeEnum2->next(NULL, status)) != NULL)
-    {
-        logln(UnicodeString("Locale name is: ") + UnicodeString(locCStr));
-        count1++;
-    }
-    if (count1 != count2) {
-        errln("getAvailableLocales(3rd time) returned %d and getAvailableLocales(2nd time) returned %d", count1, count2);
-    }
-    if (localeEnum->count(status) != count1) {
-        errln("localeEnum->count() returned %d and getAvailableLocales() returned %d", localeEnum->count(status), count1);
-    }
-    delete localeEnum;
-    delete localeEnum2;
 }
 
 void CollationAPITest::TestSortKey()
@@ -1103,23 +1017,11 @@ void CollationAPITest::TestSortKey()
     col->setAttribute(UCOL_STRENGTH, UCOL_IDENTICAL, status);
 
     uint8_t key2compat[] = {
-        /* 3.6 key, from UCA 5.0 */
-        0x29, 0x2b, 0x2d, 0x2f, 0x29, 0x01, 
-        0x09, 0x01, 0x09, 0x01, 0x28, 0x01, 
-        0x92, 0x93, 0x94, 0x95, 0x92, 0x00
-        
-        /* 3.4 key, from UCA 4.1 */
-        /*
-        0x28, 0x2a, 0x2c, 0x2e, 0x28, 0x01, 
-        0x09, 0x01, 0x09, 0x01, 0x27, 0x01, 
-        0x92, 0x93, 0x94, 0x95, 0x92, 0x00
-        */
-        /* 2.6.1 key */
-        /*
+            /* 2.6.1 key */
         0x26, 0x28, 0x2A, 0x2C, 0x26, 0x01, 
         0x09, 0x01, 0x09, 0x01, 0x25, 0x01, 
         0x92, 0x93, 0x94, 0x95, 0x92, 0x00 
-        */
+
         /* 2.2 key */
         /*
         0x1D, 0x1F, 0x21, 0x23, 0x1D, 0x01,
@@ -1678,7 +1580,7 @@ void CollationAPITest::TestBounds(void) {
     uint8_t sortkey[512], lower[512], upper[512];
     UChar buffer[512];
 
-    static const char * const test[] = {
+    const char *test[] = {
         "John Smith",
         "JOHN SMITH",
         "john SMITH",
@@ -1689,7 +1591,7 @@ void CollationAPITest::TestBounds(void) {
         "John Smithsonian"
     };
 
-    struct teststruct tests[] = {
+    static struct teststruct tests[] = {
         {"\\u010CAKI MIHALJ", {0}},
         {"\\u010CAKI MIHALJ", {0}},
         {"\\u010CAKI PIRO\\u0160KA", {0}},
@@ -2216,80 +2118,6 @@ void CollationAPITest::TestSubclass()
     }
 }
 
-void CollationAPITest::TestNULLCharTailoring()
-{
-    UErrorCode status = U_ZERO_ERROR;
-    UChar buf[256] = {0};
-    int32_t len = u_unescape("&a < '\\u0000'", buf, 256);
-    UnicodeString first((UChar)0x0061);
-    UnicodeString second((UChar)0);
-    RuleBasedCollator *coll = new RuleBasedCollator(UnicodeString(buf, len), status);
-    if(U_FAILURE(status)) {
-        errln("Failed to open collator");
-    }
-    UCollationResult res = coll->compare(first, second, status);
-    if(res != UCOL_LESS) {
-        errln("a should be less then NULL after tailoring");
-    }
-    delete coll;
-}
-
-void CollationAPITest::TestClone() {
-    logln("\ninit c0");
-    UErrorCode status = U_ZERO_ERROR;
-    RuleBasedCollator* c0 = (RuleBasedCollator*)Collator::createInstance(status);
-
-    if (U_FAILURE(status)) {
-        errln("Collator::CreateInstance(status) failed with %s", u_errorName(status));
-        return;
-    }
-
-    c0->setStrength(Collator::TERTIARY);
-    dump("c0", c0, status);
-
-    logln("\ninit c1");
-    RuleBasedCollator* c1 = (RuleBasedCollator*)Collator::createInstance(status);
-    c1->setStrength(Collator::TERTIARY);
-    UColAttributeValue val = c1->getAttribute(UCOL_CASE_FIRST, status);
-    if(val == UCOL_LOWER_FIRST){
-        c1->setAttribute(UCOL_CASE_FIRST, UCOL_UPPER_FIRST, status);
-    }else{
-        c1->setAttribute(UCOL_CASE_FIRST, UCOL_LOWER_FIRST, status);
-    }
-    dump("c0", c0, status);
-    dump("c1", c1, status);
-    
-    logln("\ninit c2");
-    RuleBasedCollator* c2 = (RuleBasedCollator*)c1->clone();
-    val = c2->getAttribute(UCOL_CASE_FIRST, status);
-    if(val == UCOL_LOWER_FIRST){
-        c2->setAttribute(UCOL_CASE_FIRST, UCOL_UPPER_FIRST, status);
-    }else{
-        c2->setAttribute(UCOL_CASE_FIRST, UCOL_LOWER_FIRST, status);
-    }
-    if(U_FAILURE(status)){
-        errln("set and get attributes of collator failed. %s\n", u_errorName(status));
-        return;
-    }
-    dump("c0", c0, status);
-    dump("c1", c1, status);
-    dump("c2", c2, status);
-    if(*c1 == *c2){
-        errln("The cloned objects refer to same data");
-    }
-    delete c0;
-    delete c1;
-    delete c2;
-}
-
- void CollationAPITest::dump(UnicodeString msg, RuleBasedCollator* c, UErrorCode& status) {
-    const char* bigone = "One";
-    const char* littleone = "one";
-  
-    logln(msg + " " + c->compare(bigone, littleone) +
-                        " s: " + c->getStrength() +
-                        " u: " + c->getAttribute(UCOL_CASE_FIRST, status));
-}
 void CollationAPITest::runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par */)
 {
     if (exec) logln("TestSuite CollationAPITest: ");
@@ -2316,8 +2144,6 @@ void CollationAPITest::runIndexedTest( int32_t index, UBool exec, const char* &n
         case 19: name = "TestGetTailoredSet"; if (exec) TestGetTailoredSet(); break;
         case 20: name = "TestUClassID"; if (exec) TestUClassID(); break;
         case 21: name = "TestSubclass"; if (exec) TestSubclass(); break;
-        case 22: name = "TestNULLCharTailoring"; if (exec) TestNULLCharTailoring(); break;
-        case 23: name = "TestClone"; if (exec) TestClone(); break;
         default: name = ""; break;
     }
 }
