@@ -1,7 +1,7 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2011, International Business Machines Corporation
- * and others. All Rights Reserved.
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * others. All Rights Reserved.
  ********************************************************************/
  
 #include "unicode/utypes.h"
@@ -81,9 +81,6 @@ CalendarRegressionTest::runIndexedTest( int32_t index, UBool exec, const char* &
         CASE(42,TestWeekShift);
         CASE(43,TestTimeZoneTransitionAdd);
         CASE(44,TestDeprecates);
-        CASE(45,TestT5555);
-        CASE(46,TestT6745);
-        CASE(47,TestT8057);
     default: name = ""; break;
     }
 }
@@ -114,7 +111,7 @@ UBool
 CalendarRegressionTest::failure(UErrorCode status, const char* msg)
 {
     if(U_FAILURE(status)) {
-        errcheckln(status, UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
+        errln(UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
         return TRUE;
     }
 
@@ -130,7 +127,7 @@ CalendarRegressionTest::test4100311()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = (GregorianCalendar*)Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -153,7 +150,7 @@ CalendarRegressionTest::test4074758()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -176,7 +173,7 @@ CalendarRegressionTest::test4028518()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal1 = new GregorianCalendar(status) ;
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal1;
       return;
     }
@@ -217,25 +214,7 @@ CalendarRegressionTest::test4031502()
     // require the host zone to be set; it can be set in Java.
     UErrorCode status = U_ZERO_ERROR;
     StringEnumeration* ids = TimeZone::createEnumeration();
-    if (ids == NULL) {
-        dataerrln("Unable to create TimeZone Enumeration.");
-        return;
-    }
     UBool bad = FALSE;
-    TimeZone* tz =TimeZone::createTimeZone("Asia/Riyadh87");
-    failure(status, "new TimeZone");
-    GregorianCalendar *cl = new GregorianCalendar(tz, status);
-    if (U_FAILURE(status)) {
-        dataerrln("Fail new GregorianCalendar: %s", u_errorName(status));
-        delete tz;
-        return;
-    }
-    cl->clear();
-    cl->set(1900, 15, 5, 5, 8, 13);
-    cl->get(UCAL_HOUR, status);
-    failure(status, "cl->get(UCAL_HOUR, status)");
-    status = U_ZERO_ERROR;
-    delete cl;
     for (int32_t i=0; i<ids->count(status); ++i) {
         TimeZone *zone = TimeZone::createTimeZone(*ids->snext(status));
         GregorianCalendar *cal = new GregorianCalendar(zone, status);
@@ -273,7 +252,7 @@ void CalendarRegressionTest::test4035301()
         c->before(*c,status) ||
         *c != *c ||
         *c != *d)
-        dataerrln("Fail");
+        errln("Fail");
     delete c;
     delete d;
 }
@@ -285,20 +264,13 @@ void CalendarRegressionTest::test4040996()
 {
     int32_t count = 0;
     StringEnumeration* ids = TimeZone::createEnumeration(-8 * 60 * 60 * 1000);
-    if (ids == NULL) {
-        dataerrln("Unable to create TimeZone enumeration.");
-        return;
-    }
     UErrorCode status = U_ZERO_ERROR;    
     count = ids->count(status);
     SimpleTimeZone *pdt = new SimpleTimeZone(-8 * 60 * 60 * 1000, *ids->snext(status));
     pdt->setStartRule(UCAL_APRIL, 1, UCAL_SUNDAY, 2 * 60 * 60 * 1000, status);
     pdt->setEndRule(UCAL_OCTOBER, -1, UCAL_SUNDAY, 2 * 60 * 60 * 1000, status);
     Calendar *calendar = new GregorianCalendar(pdt, status);
-    if (U_FAILURE(status)) {
-        dataerrln("Fail new GregorianCalendar: %s", u_errorName(status));
-        return;
-    }
+
     calendar->set(UCAL_MONTH,3);
     calendar->set(UCAL_DATE,18);
     calendar->set(UCAL_SECOND, 30);
@@ -336,7 +308,7 @@ void CalendarRegressionTest::test4051765()
     UErrorCode status = U_ZERO_ERROR;
     Calendar *cal = Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -419,7 +391,7 @@ void CalendarRegressionTest::test4059654() {
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *gc = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete gc;
       return;
     }
@@ -451,7 +423,7 @@ void CalendarRegressionTest::test4061476()
     Calendar *cal = Calendar::createInstance(TimeZone::createTimeZone("GMT"), 
                                     Locale::getUK(),status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       delete fmt;
       return;
@@ -481,8 +453,8 @@ void CalendarRegressionTest::test4070502()
 {
     UErrorCode status = U_ZERO_ERROR;
     Calendar *cal = new GregorianCalendar(status);
-    if(status == U_USING_FALLBACK_WARNING || U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+    if(status == U_USING_FALLBACK_WARNING) {
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -538,7 +510,7 @@ void CalendarRegressionTest::dowTest(UBool lenient)
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -563,12 +535,9 @@ void CalendarRegressionTest::dowTest(UBool lenient)
       return;
     }
 
-    if(cal->getActualMinimum(UCAL_DAY_OF_WEEK, status) != min) {
+    if(cal->getActualMinimum(UCAL_DAY_OF_WEEK) != min) {
         errln("FAIL: actual minimum differs from minimum");
     }
-    if(cal->getActualMinimum(Calendar::DAY_OF_WEEK, status) != min) {
-        errln("FAIL: actual minimum (Calendar::DAY_OF_WEEK, status) differs from minimum");
-    }
     if(cal->getActualMinimum(Calendar::DAY_OF_WEEK) != min) {
         errln("FAIL: actual minimum (Calendar::DAY_OF_WEEK) differs from minimum");
     }
@@ -595,7 +564,7 @@ void CalendarRegressionTest::test4071385()
     UErrorCode status = U_ZERO_ERROR;
     Calendar *cal = Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -616,7 +585,7 @@ void CalendarRegressionTest::test4073929()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *foo1 = new GregorianCalendar(1997, 8, 27,status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating Calendar: %s", u_errorName(status));
+      errln("Error creating Calendar: %s", u_errorName(status));
       delete foo1;
       return;
     }
@@ -672,7 +641,7 @@ void CalendarRegressionTest::test4083167()
     UDate firstDate = Calendar::getNow();
         Calendar *cal = new GregorianCalendar(status);
         if(U_FAILURE(status)) {
-          dataerrln("Error creating Calendar: %s", u_errorName(status));
+          errln("Error creating Calendar: %s", u_errorName(status));
           delete cal;
           return;
         }
@@ -722,7 +691,7 @@ void CalendarRegressionTest::test4086724()
         date = new SimpleDateFormat(UnicodeString("dd MMM yyy (zzzz) 'is in week' ww"),status); 
         Calendar *cal = Calendar::createInstance(status); 
         if(U_FAILURE(status)) {
-          dataerrln("Error creating Calendar: %s", u_errorName(status));
+          errln("Error creating Calendar: %s", u_errorName(status));
           delete cal;
           delete newZone;
           delete date;
@@ -761,11 +730,6 @@ delete saveZone;
 void CalendarRegressionTest::test4092362() {
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal1 = new GregorianCalendar(1997, 10, 11, 10, 20, 40,status); 
-    if (U_FAILURE(status)) {
-        dataerrln("Fail new GregorianCalendar: %s", u_errorName(status));
-        delete cal1;
-        return;
-    }
     /*cal1.set( Calendar::YEAR, 1997 ); 
     cal1.set( Calendar::MONTH, 10 ); 
     cal1.set( Calendar::DATE, 11 ); 
@@ -775,8 +739,7 @@ void CalendarRegressionTest::test4092362() {
 
     logln( UnicodeString(" Cal1 = ") + cal1->getTime(status) ); 
     logln( UnicodeString(" Cal1 time in ms = ") + cal1->get(UCAL_MILLISECOND,status) ); 
-    for (int32_t k = 0; k < 100 ; k++)
-        ;
+    for( int32_t k = 0; k < 100 ; k++ ); 
 
     GregorianCalendar *cal2 = new GregorianCalendar(1997, 10, 11, 10, 20, 40,status); 
     /*cal2.set( Calendar::YEAR, 1997 ); 
@@ -802,11 +765,6 @@ void CalendarRegressionTest::test4095407()
 {
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *a = new GregorianCalendar(1997,UCAL_NOVEMBER, 13,status);
-    if (U_FAILURE(status)) {
-        dataerrln("Fail new GregorianCalendar: %s", u_errorName(status));
-        delete a;
-        return;
-    }
     int32_t dow = a->get(UCAL_DAY_OF_WEEK, status);
     if (dow != UCAL_THURSDAY)
         errln("Fail: Want THURSDAY Got " + dow);
@@ -825,13 +783,6 @@ void CalendarRegressionTest::test4096231()
     int32_t sec = 0, min = 0, hr = 0, day = 1, month = 10, year = 1997;
                         
     Calendar *cal1 = new GregorianCalendar(*PST,status);
-    if (U_FAILURE(status)) {
-        dataerrln("Failure new GregorianCalendar: %s", u_errorName(status));
-        delete GMT;
-        delete PST;
-        delete cal1;
-        return;
-    }
     cal1->setTime(880698639000.0,status);
     // Issue 1: Changing the timezone doesn't change the
     //          represented time.  The old API, pre 1.2.2a requires 
@@ -846,8 +797,6 @@ void CalendarRegressionTest::test4096231()
 
     Calendar *cal2 = new GregorianCalendar(*GMT,status);
     Calendar *cal3 = new GregorianCalendar(*PST,status);
-    cal2->set(UCAL_MILLISECOND, 0);
-    cal3->set(UCAL_MILLISECOND, 0);
 
     cal2->set(cal1->get(UCAL_YEAR,status),
              cal1->get(UCAL_MONTH,status),
@@ -895,11 +844,6 @@ void CalendarRegressionTest::test4096539()
     for (int32_t x=0;x<12;x++) {
         GregorianCalendar *gc = new 
             GregorianCalendar(1997,x,y[x], status);
-        if (U_FAILURE(status)) {
-            dataerrln("Fail new GregorianCalendar: %s", u_errorName(status));
-            delete gc;
-            return;
-        }
         int32_t m1,m2;
         log(UnicodeString("") + (m1=gc->get(UCAL_MONTH,status)+1)+UnicodeString("/")+
                          gc->get(UCAL_DATE,status)+"/"+gc->get(UCAL_YEAR,status)+
@@ -925,7 +869,7 @@ void CalendarRegressionTest::test41003112()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = (GregorianCalendar*)Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar: %s", u_errorName(status));
+      errln("Error creating calendar: %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -949,7 +893,7 @@ void CalendarRegressionTest::test4103271()
     UnicodeString output, testDesc, str, str2; 
     GregorianCalendar *testCal = (GregorianCalendar*)Calendar::createInstance(status); 
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar: %s", u_errorName(status));
+      errln("Error creating calendar: %s", u_errorName(status));
       delete testCal;
       return;
     }
@@ -1210,7 +1154,7 @@ void CalendarRegressionTest::test4106136()
             for (int32_t j=0; j<3; ++j) {
                 UnicodeString temp;
                 if (n[j] == 0)
-                    dataerrln(UnicodeString("Fail: No locales for ") + locales[i].getName());
+                    errln(UnicodeString("Fail: No locales for ") + locales[i].getName());
             }
         }
     //}
@@ -1227,7 +1171,7 @@ void CalendarRegressionTest::test4108764()
     UErrorCode status = U_ZERO_ERROR;
     Calendar *cal = Calendar::createInstance(status); 
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -1274,10 +1218,10 @@ void CalendarRegressionTest::test4108764()
 void CalendarRegressionTest::test4114578() 
 {
     UErrorCode status = U_ZERO_ERROR;
-    double ONE_HOUR = 60*60*1000;
+    int32_t ONE_HOUR = 60*60*1000;
     Calendar *cal = Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -1345,7 +1289,7 @@ void CalendarRegressionTest::test4118384()
     UErrorCode status = U_ZERO_ERROR;
     Calendar *cal = Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -1403,12 +1347,11 @@ void CalendarRegressionTest::test4125881()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = (GregorianCalendar*) Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
     DateFormat *fmt = new SimpleDateFormat(UnicodeString("MMMM d, yyyy G"),status);
-    if(!assertSuccess("trying to construct", status))return;
     cal->clear();
     for (int32_t y=-20; y<=10; ++y) {
         cal->set(UCAL_ERA, y < 1 ? GregorianCalendar::BC : GregorianCalendar::AD);
@@ -1433,12 +1376,11 @@ void CalendarRegressionTest::test4125892() {
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = (GregorianCalendar*) Calendar::createInstance(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
     DateFormat *fmt = new SimpleDateFormat(UnicodeString("MMMM d, yyyy G"),status);
-    if(!assertSuccess("trying to construct", status))return;
     cal->clear();
     cal->set(UCAL_ERA, GregorianCalendar::BC);
     cal->set(UCAL_YEAR, 81); // 81 BC is a leap year (proleptically)
@@ -1464,7 +1406,7 @@ void CalendarRegressionTest::test4141665()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *cal = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete cal;
       return;
     }
@@ -1493,7 +1435,7 @@ void CalendarRegressionTest::test4142933()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *calendar = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete calendar;
       return;
     }
@@ -1527,7 +1469,7 @@ void CalendarRegressionTest::test4145158()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *calendar = new GregorianCalendar(status);
     if(status == U_USING_FALLBACK_WARNING || U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete calendar;
       return;
     }
@@ -1566,7 +1508,7 @@ void CalendarRegressionTest::test4145983()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *calendar = new GregorianCalendar(status);
     if(U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete calendar;
       return;
     }
@@ -1597,7 +1539,7 @@ void CalendarRegressionTest::test4147269()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *calendar = new GregorianCalendar(status);
     if(status == U_USING_FALLBACK_WARNING || U_FAILURE(status)) {
-      dataerrln("Error creating calendar %s", u_errorName(status));
+      errln("Error creating calendar %s", u_errorName(status));
       delete calendar;
       return;
     }
@@ -1651,7 +1593,7 @@ CalendarRegressionTest::Test4149677()
     for (int32_t i=0; i < 3; ++i) {
         GregorianCalendar *calendar = new GregorianCalendar(zones[i], status);
         if(U_FAILURE(status)) {
-            dataerrln("Couldnt' create calendar.: %s", u_errorName(status));
+            errln("Couldnt' create calendar.");
             return;
         }
 
@@ -1703,7 +1645,7 @@ CalendarRegressionTest::Test4162587()
     
     GregorianCalendar *cal = new GregorianCalendar(tz, status);
     if(U_FAILURE(status)) {
-        dataerrln("Couldn't create calendar.: %s", u_errorName(status));
+        errln("Couldn't create calendar");
         return;
     }
     UDate d0, dPlus, dMinus;
@@ -1755,7 +1697,7 @@ CalendarRegressionTest::Test4165343()
     UErrorCode status = U_ZERO_ERROR;
     GregorianCalendar *calendar = new GregorianCalendar(1996, UCAL_FEBRUARY, 29, status);
     if(U_FAILURE(status)) {
-        dataerrln("Couldn't create calendar.: %s", u_errorName(status));
+        errln("Couldn't create calendar");
         return;
     }
     UDate start = calendar->getTime(status);
@@ -1810,7 +1752,7 @@ CalendarRegressionTest::Test4166109()
 
     GregorianCalendar *calendar = new GregorianCalendar(Locale::getUS(), status);
     if(U_FAILURE(status)) {
-        dataerrln("Couldn't create calendar.: %s", u_errorName(status));
+        errln("Couldn't create calendar");
         return;
     }
     calendar->set(1998, UCAL_MARCH, 1);
@@ -1854,7 +1796,7 @@ CalendarRegressionTest::Test4167060()
     DateFormat *format = new SimpleDateFormat(UnicodeString("EEE MMM dd HH:mm:ss zzz yyyy G"),
         Locale::getUS(), status);
     if(U_FAILURE(status)) {
-        dataerrln("Couldn't create SimpleDateFormat - %s", u_errorName(status));
+        errln("Couldn't create SimpleDateFormat");
         return;
     }
 
@@ -1945,7 +1887,7 @@ void CalendarRegressionTest::Test4197699() {
                          Locale::getUS(), status);
     fmt.setCalendar(cal);
     if (U_FAILURE(status)) {
-        dataerrln("Couldn't initialize test - %s", u_errorName(status));
+        errln("Couldn't initialize test");
         return;
     }
 
@@ -2004,10 +1946,10 @@ void CalendarRegressionTest::TestJ81() {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString temp, temp2, temp3;
     int32_t i;
-    GregorianCalendar cal(TimeZone::createTimeZone("GMT"), Locale::getUS(), status);
+    GregorianCalendar cal(TimeZone::createTimeZone("GMT"), status);
     SimpleDateFormat fmt("HH:mm 'w'w 'd'D E d MMM yyyy", Locale::getUS(), status);
     if (U_FAILURE(status)) {
-        dataerrln("Error: Cannot create calendar or format - %s", u_errorName(status));
+        errln("Error: Cannot create calendar or format");
         return;
     }
     fmt.setCalendar(cal);
@@ -2223,7 +2165,7 @@ void CalendarRegressionTest::TestJ438(void) {
     int32_t DATA_length = (int32_t)(sizeof(DATA)/sizeof(DATA[0]));
     Calendar* pcal = Calendar::createInstance(Locale::getUS(), ec);
     if(U_FAILURE(ec)) {
-      dataerrln("Error creating calendar %s", u_errorName(ec));
+      errln("Error creating calendar %s", u_errorName(ec));
       delete pcal;
       return;
     }
@@ -2313,116 +2255,6 @@ void CalendarRegressionTest::TestJ438(void) {
     delete pcal;
 }
 
-void CalendarRegressionTest::TestT5555()
-{
-    UErrorCode ec = U_ZERO_ERROR;
-    Calendar *cal = Calendar::createInstance(ec);
-
-    if (cal == NULL || U_FAILURE(ec)) {
-        dataerrln("FAIL: Calendar::createInstance(): %s", u_errorName(ec));
-        delete cal;
-        return;
-    }
-
-    // Set to Wednesday, February 21, 2007
-    cal->set(2007, UCAL_FEBRUARY, 21);
-
-    // Advance three years
-    cal->add(UCAL_MONTH, 36, ec);
-
-    // Set to last Wednesday of the month
-    cal->set(UCAL_DAY_OF_WEEK_IN_MONTH, -1);
-
-    cal->getTime(ec);
-
-    int32_t yy, mm, dd, ee;
-
-    yy = cal->get(UCAL_YEAR, ec);
-    mm = cal->get(UCAL_MONTH, ec);
-    dd = cal->get(UCAL_DATE, ec);
-    ee = cal->get(UCAL_DAY_OF_WEEK, ec);
-
-    // Should be set to Wednesday, February 24, 2010
-    if (U_FAILURE(ec) || yy != 2010 || mm != UCAL_FEBRUARY || dd != 24 || ee != UCAL_WEDNESDAY) {
-        errln("FAIL: got date %4d/%02d/%02d, expected 210/02/24: ", yy, mm + 1, dd);
-    }
-    delete cal;
-}
-
-typedef struct {
-    int32_t             startYear;
-    int32_t             startMonth; // 0-based
-    int32_t             startDay;   // 1-based
-    UCalendarDateFields fieldToChange;
-    int32_t             fieldDelta;
-    int32_t             endYear;
-    int32_t             endMonth;   // 0-based
-    int32_t             endDay;     // 1-based
-} CoptEthCalTestItem;
-
-// year 1724 in coptic calendar =
-// year 2000 in ethiopic calendar (276 more than coptic) =
-// year 7500 in ethiopic-amete-alem calendar (5776 more than coptic)
-// (2007-2008 in gregorian calendar depending on month)
-static const CoptEthCalTestItem coptEthCalTestItems[] = {
-    { 1724, 12, 1, UCAL_MONTH, +1, 1725,  0, 1 },
-    { 1724, 12, 1, UCAL_MONTH, +9, 1725,  8, 1 },
-    { 1723, 12, 2, UCAL_MONTH, +1, 1724,  0, 2 }, // 1723 is a leap year
-    { 1723, 12, 2, UCAL_MONTH, +9, 1724,  8, 2 },
-    { 1725,  0, 1, UCAL_MONTH, -1, 1724, 12, 1 },
-    { 1725,  0, 1, UCAL_MONTH, -6, 1724,  7, 1 },
-    { 1724, 12, 1, UCAL_DATE,  +8, 1725,  0, 4 },
-    { 1723, 12, 1, UCAL_DATE,  +8, 1724,  0, 3 }, // 1723 is a leap year
-    { 1724,  0, 1, UCAL_DATE,  -1, 1723, 12, 6 }, // 1723 is a leap year
-    { 0, 0, 0, (UCalendarDateFields)0, 0, 0, 0, 0 } // terminator
-};
-
-typedef struct {
-    const char * locale;
-    int32_t      yearOffset;
-} CoptEthCalLocale;
-
-static const CoptEthCalLocale copEthCalLocales[] = {
-    { "en@calendar=coptic",   0    },
-    { "en@calendar=ethiopic", 276  },
-    { NULL,                   0    } // terminator
-};
-
-void CalendarRegressionTest::TestT6745()
-{
-    const CoptEthCalLocale * testLocalePtr;
-    for ( testLocalePtr = copEthCalLocales; testLocalePtr->locale != NULL; ++testLocalePtr) {
-        UErrorCode status = U_ZERO_ERROR;
-        Calendar *cal = Calendar::createInstance(Locale(testLocalePtr->locale), status);
-        if ( U_FAILURE(status) ) {
-            dataerrln((UnicodeString)"FAIL: Calendar::createInstance, locale " + testLocalePtr->locale + ", status " + u_errorName(status));
-            continue;
-        }
-        const CoptEthCalTestItem * testItemPtr;
-        for (testItemPtr = coptEthCalTestItems; testItemPtr->fieldDelta != 0; ++testItemPtr) {
-            status = U_ZERO_ERROR;
-            cal->set( testItemPtr->startYear + testLocalePtr->yearOffset, testItemPtr->startMonth, testItemPtr->startDay, 9, 0 );
-            cal->add( testItemPtr->fieldToChange, testItemPtr->fieldDelta, status );
-            if ( U_FAILURE(status) ) {
-                errln((UnicodeString)"FAIL: Calendar::add, locale " + testLocalePtr->locale + ", field/delta " +
-                        testItemPtr->fieldToChange + "/" + testItemPtr->fieldDelta + ", status " + u_errorName(status));
-                continue;
-            }
-            int32_t endYear = testItemPtr->endYear + testLocalePtr->yearOffset;
-            int32_t year  = cal->get(UCAL_YEAR, status);
-            int32_t month = cal->get(UCAL_MONTH, status);
-            int32_t day   = cal->get(UCAL_DATE, status);
-            if ( U_FAILURE(status) || year != endYear || month != testItemPtr->endMonth || day != testItemPtr->endDay ) {
-                errln((UnicodeString)"ERROR: Calendar::add, locale " + testLocalePtr->locale + ", field/delta " +
-                        testItemPtr->fieldToChange + "/" + testItemPtr->fieldDelta + ", status " + u_errorName(status) +
-                        ", expected " + endYear + "/" + testItemPtr->endMonth + "/" + testItemPtr->endDay +
-                        ", got " + year + "/" + month + "/" + day );
-            }
-        }
-        delete cal;
-    }
-}
-
 /**
  * Test behavior of fieldDifference around leap years.  Also test a large
  * field difference to check binary search.
@@ -2431,7 +2263,7 @@ void CalendarRegressionTest::TestLeapFieldDifference() {
     UErrorCode ec = U_ZERO_ERROR;
     Calendar* cal = Calendar::createInstance(ec);
     if (cal == NULL || U_FAILURE(ec)) {
-        dataerrln("FAIL: Calendar::createInstance(): %s", u_errorName(ec));
+        errln("FAIL: Calendar::createInstance()");
         delete cal;
         return;
     }
@@ -2504,7 +2336,7 @@ void CalendarRegressionTest::TestMalaysianInstance() {
     UErrorCode ec = U_ZERO_ERROR;
     Calendar* cal = Calendar::createInstance(loc, ec);
     if (U_FAILURE(ec)) {
-        dataerrln("FAIL: Can't construct calendar for ms_MY: %s", u_errorName(ec));
+        errln("FAIL: Can't construct calendar for ms_MY");
     }
     delete cal;
 }
@@ -2518,10 +2350,6 @@ void CalendarRegressionTest::TestWeekShift() {
     UErrorCode ec = U_ZERO_ERROR;
     GregorianCalendar cal(TimeZone::createTimeZone("America/Los_Angeles"),
                           Locale("en", "US"), ec);
-    if (U_FAILURE(ec)) {
-        dataerrln("Fail GregorianCalendar: %s", u_errorName(ec));
-        return;
-    }
     cal.setTime(UDate(997257600000.0), ec); // Wed Aug 08 01:00:00 PDT 2001
     // In pass one, change the first day of week so that the weeks
     // shift in August 2001.  In pass two, change the minimal days
@@ -2599,7 +2427,7 @@ void CalendarRegressionTest::TestTimeZoneTransitionAdd() {
 
     StringEnumeration *tz = TimeZone::createEnumeration();
     if (tz == NULL) {
-        dataerrln("FAIL: TimeZone::createEnumeration");
+        errln("FAIL: TimeZone::createEnumeration");
         return;
     }
 
@@ -2648,7 +2476,7 @@ void CalendarRegressionTest::TestTimeZoneTransitionAdd() {
     }
 
     if (U_FAILURE(ec)) {
-        dataerrln("FAIL: %s", u_errorName(ec));
+        errln("FAIL: %s", u_errorName(ec));
     }
 
     delete tz;
@@ -2686,7 +2514,7 @@ void CalendarRegressionTest::TestDeprecates(void)
     Calendar *c2 = Calendar::createInstance("ja_JP_TRADITIONAL",status);
 
     if(!c1 || !c2 || U_FAILURE(status)) {
-        dataerrln("Couldn't create calendars for roll of HOUR: %s", u_errorName(status));
+        errln("Couldn't create calendars for roll of HOUR");
         return;
     }
 
@@ -2772,43 +2600,7 @@ void CalendarRegressionTest::TestDeprecates(void)
 
     delete c1;
     delete c2;
-
 }
 
-void CalendarRegressionTest::TestT8057(void) {
-    // Set the calendar to the last day in a leap year
-    UErrorCode status = U_ZERO_ERROR;
-    GregorianCalendar *cal = (GregorianCalendar*)Calendar::createInstance(status);
-    if(U_FAILURE(status)) {
-        errln("Error creating Calendar: %s", u_errorName(status));
-        delete cal;
-        return;
-    }
-    cal->setLenient(FALSE);
-    cal->clear();
-    cal->set(2008, UCAL_DECEMBER, 31);
-
-    // Force calculating then fields once.
-    UDate t = cal->getTime(status);
-    if(U_FAILURE(status)) {
-        errln("Error while calculating the date");
-        delete cal;
-        return;
-    }
-
-    UDate expected = 1262246400000.0; // 2009-12-31 00:00 PST
-
-    cal->add(UCAL_YEAR, 1, status);
-    t = cal->getTime(status);
-    if (U_SUCCESS(status)) {
-        if (t != expected) {
-            dataerrln((UnicodeString)"FAIL: wrong date after add: expected=" + expected + " returned=" + t);
-        }
-    } else {
-        errln("FAIL: error while adding one year");
-    }
-
-    delete cal;
-}
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
