@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2003-2006, International Business Machines
+*   Copyright (C) 2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -36,7 +36,7 @@
 #include "unicode/ustring.h"
 #include "unicode/ures.h"
 #include "convtest.h"
-#include "unicode/tstdtmod.h"
+#include "tstdtmod.h"
 #include <string.h>
 #include <stdlib.h>
 
@@ -74,6 +74,7 @@ ConversionTest::TestToUnicode() {
     int32_t offsetsLength;
     UConverterToUCallback callback;
 
+    TestLog testLog;
     TestDataModule *dataModule;
     TestData *testData;
     const DataMap *testCase;
@@ -81,7 +82,7 @@ ConversionTest::TestToUnicode() {
     int32_t i;
 
     errorCode=U_ZERO_ERROR;
-    dataModule=TestDataModule::getTestDataModule("conversion", *this, errorCode);
+    dataModule=TestDataModule::getTestDataModule("conversion", testLog, errorCode);
     if(U_SUCCESS(errorCode)) {
         testData=dataModule->createTestData("toUnicode", errorCode);
         if(U_SUCCESS(errorCode)) {
@@ -183,9 +184,10 @@ ConversionTest::TestFromUnicode() {
     char charset[100], cbopt[4];
     const char *option;
     UnicodeString s, unicode, invalidUChars;
-    int32_t offsetsLength, index;
+    int32_t offsetsLength;
     UConverterFromUCallback callback;
 
+    TestLog testLog;
     TestDataModule *dataModule;
     TestData *testData;
     const DataMap *testCase;
@@ -194,7 +196,7 @@ ConversionTest::TestFromUnicode() {
     int32_t i, length;
 
     errorCode=U_ZERO_ERROR;
-    dataModule=TestDataModule::getTestDataModule("conversion", *this, errorCode);
+    dataModule=TestDataModule::getTestDataModule("conversion", testLog, errorCode);
     if(U_SUCCESS(errorCode)) {
         testData=dataModule->createTestData("fromUnicode", errorCode);
         if(U_SUCCESS(errorCode)) {
@@ -242,17 +244,15 @@ ConversionTest::TestFromUnicode() {
                 }
 
                 s=testCase->getString("callback", errorCode);
-                cc.setSub=0; // default: no subchar
 
-                if((index=s.indexOf((UChar)0))>0) {
-                    // read NUL-separated subchar first, if any
+                // read NUL-separated subchar first, if any
+                length=u_strlen(p=s.getTerminatedBuffer());
+                if(++length<s.length()) {
                     // copy the subchar from Latin-1 characters
                     // start after the NUL
-                    p=s.getTerminatedBuffer();
-                    length=index+1;
                     p+=length;
                     length=s.length()-length;
-                    if(length<=0 || length>=(int32_t)sizeof(cc.subchar)) {
+                    if(length>=(int32_t)sizeof(cc.subchar)) {
                         errorCode=U_ILLEGAL_ARGUMENT_ERROR;
                     } else {
                         int32_t j;
@@ -262,26 +262,13 @@ ConversionTest::TestFromUnicode() {
                         }
                         // NUL-terminate the subchar
                         cc.subchar[j]=0;
-                        cc.setSub=1;
                     }
 
                     // remove the NUL and subchar from s
-                    s.truncate(index);
-                } else if((index=s.indexOf((UChar)0x3d))>0) /* '=' */ {
-                    // read a substitution string, separated by an equal sign
-                    p=s.getBuffer()+index+1;
-                    length=s.length()-(index+1);
-                    if(length<0 || length>=LENGTHOF(cc.subString)) {
-                        errorCode=U_ILLEGAL_ARGUMENT_ERROR;
-                    } else {
-                        u_memcpy(cc.subString, p, length);
-                        // NUL-terminate the subString
-                        cc.subString[length]=0;
-                        cc.setSub=-1;
-                    }
-
-                    // remove the equal sign and subString from s
-                    s.truncate(index);
+                    s.truncate(u_strlen(s.getBuffer()));
+                } else {
+                    // no subchar
+                    cc.subchar[0]=0;
                 }
 
                 s.extract(0, 0x7fffffff, cbopt, sizeof(cbopt), "");
@@ -340,9 +327,9 @@ ConversionTest::TestGetUnicodeSet() {
 
     ParsePosition pos;
     UnicodeSet cnvSet, mapSet, mapnotSet, diffSet;
-    UnicodeSet *cnvSetPtr = &cnvSet;
     UConverter *cnv;
 
+    TestLog testLog;
     TestDataModule *dataModule;
     TestData *testData;
     const DataMap *testCase;
@@ -350,7 +337,7 @@ ConversionTest::TestGetUnicodeSet() {
     int32_t i;
 
     errorCode=U_ZERO_ERROR;
-    dataModule=TestDataModule::getTestDataModule("conversion", *this, errorCode);
+    dataModule=TestDataModule::getTestDataModule("conversion", testLog, errorCode);
     if(U_SUCCESS(errorCode)) {
         testData=dataModule->createTestData("getUnicodeSet", errorCode);
         if(U_SUCCESS(errorCode)) {
@@ -411,7 +398,7 @@ ConversionTest::TestGetUnicodeSet() {
                     continue;
                 }
 
-                ucnv_getUnicodeSet(cnv, (USet *)cnvSetPtr, (UConverterUnicodeSet)which, &errorCode);
+                ucnv_getUnicodeSet(cnv, (USet *)&cnvSet, (UConverterUnicodeSet)which, &errorCode);
                 ucnv_close(cnv);
 
                 if(U_FAILURE(errorCode)) {
@@ -1067,19 +1054,10 @@ ConversionTest::FromUnicodeCase(ConversionCase &cc, UConverterFromUCallback call
     // set the subchar
     int32_t length;
 
-    if(cc.setSub>0) {
-        length=(int32_t)strlen(cc.subchar);
+    if((length=strlen(cc.subchar))!=0) {
         ucnv_setSubstChars(cnv, cc.subchar, (int8_t)length, &errorCode);
         if(U_FAILURE(errorCode)) {
-            errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setSubstChars() failed - %s",
-                    cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
-            ucnv_close(cnv);
-            return FALSE;
-        }
-    } else if(cc.setSub<0) {
-        ucnv_setSubstString(cnv, cc.subString, -1, &errorCode);
-        if(U_FAILURE(errorCode)) {
-            errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setSubstString() failed - %s",
+            errln("fromUnicode[%d](%s cb=\"%s\" fb=%d flush=%d) ucnv_setSubChars() failed - %s",
                     cc.caseNr, cc.charset, cc.cbopt, cc.fallbacks, cc.finalFlush, u_errorName(errorCode));
             ucnv_close(cnv);
             return FALSE;
