@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2009, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -17,13 +17,15 @@
 
 #include "unicode/utypes.h"
 
-#if !UCONFIG_NO_COLLATION && !UCONFIG_NO_FILE_IO
+#if !UCONFIG_NO_COLLATION
 
 #include "unicode/uchar.h"
-#include "unicode/tstdtmod.h"
+
 #include "cstring.h"
 #include "ucol_tok.h"
+
 #include "tscoll.h"
+
 #include "dadrcoll.h"
 
 U_CDECL_BEGIN
@@ -37,15 +39,15 @@ DataDrivenCollatorTest::DataDrivenCollatorTest()
 status(U_ZERO_ERROR),
 sequences(status)
 {
-  driver = TestDataModule::getTestDataModule("DataDrivenCollationTest", *this, status);
+  TestLog testLog;
+
+  driver = TestDataModule::getTestDataModule("DataDrivenCollationTest", testLog, status);
   sequences.setDeleter(deleteSeqElement);
-  UCA = (RuleBasedCollator*)Collator::createInstance("root", status);
 }
 
 DataDrivenCollatorTest::~DataDrivenCollatorTest() 
 {
   delete driver;
-  delete UCA;
 }
 
 void DataDrivenCollatorTest::runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par */)
@@ -73,7 +75,7 @@ void DataDrivenCollatorTest::runIndexedTest( int32_t index, UBool exec, const ch
       name = "";
     }
   } else {
-    dataerrln("collate/DataDrivenTest data not initialized!");
+    errln("collate/DataDrivenTest data not initialized!");
     name = "";
   }
 
@@ -164,7 +166,7 @@ DataDrivenCollatorTest::processArguments(Collator *col, const UChar *start, int3
   }
 }
 
-void
+void 
 DataDrivenCollatorTest::processTest(TestData *testData) {
   Collator *col = NULL;
   const UChar *arguments = NULL;
@@ -203,21 +205,7 @@ DataDrivenCollatorTest::processTest(TestData *testData) {
         errln("No collator definition!");
       }
     }
-    
-    int32_t cloneSize = 0;
-    uint8_t* cloneBuf = NULL;
-    RuleBasedCollator* clone = NULL;
-    if(col != NULL){
-      RuleBasedCollator* rbc = (RuleBasedCollator*)col;
-      cloneSize = rbc->cloneBinary(NULL, 0, intStatus);
-      intStatus = U_ZERO_ERROR;
-      cloneBuf = (uint8_t*) malloc(cloneSize);
-      cloneSize = rbc->cloneBinary(cloneBuf, cloneSize, intStatus);
-      clone = new RuleBasedCollator(cloneBuf, cloneSize, UCA, intStatus);
-      if(U_FAILURE(intStatus)){
-          errln("Could not clone the RuleBasedCollator. Error: %s", u_errorName(intStatus));
-          intStatus= U_ZERO_ERROR;
-      }
+    if(col != NULL) {
       // get attributes
       testSetting = settings->getString("Arguments", intStatus);
       if(U_SUCCESS(intStatus)) {
@@ -225,9 +213,6 @@ DataDrivenCollatorTest::processTest(TestData *testData) {
         argLen = testSetting.length();
         arguments = testSetting.getBuffer();
         processArguments(col, arguments, argLen);
-        if(clone != NULL){
-            processArguments(clone, arguments, argLen);
-        }
         if(U_FAILURE(status)) {
           errln("Couldn't process arguments");
           break;
@@ -240,22 +225,16 @@ DataDrivenCollatorTest::processTest(TestData *testData) {
         UnicodeString sequence = currentCase->getString("sequence", status);
         if(U_SUCCESS(status)) {
             processSequence(col, sequence);
-            if(clone != NULL){
-                processSequence(clone, sequence);
-            }
         }
       }
     } else {
       errln("Couldn't instantiate a collator!");
     }
-    delete clone;
-    free(cloneBuf);
     delete col;
     col = NULL;
   }
 }
 
-
 void 
 DataDrivenCollatorTest::processSequence(Collator* col, const UnicodeString &sequence) {
   Collator::EComparisonResult relation = Collator::EQUAL;
