@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2011, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -54,7 +54,6 @@ DateFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const char*
         CASE(23,Test4210209)
         CASE(24,Test714)
         CASE(25,Test1684)
-        CASE(26,Test5554)
         default: name = ""; break;
     }
 }
@@ -70,15 +69,8 @@ void DateFormatRegressionTest::Test4029195(void)
     logln((UnicodeString) "today: " + today);
 
     SimpleDateFormat *sdf = (SimpleDateFormat*) DateFormat::createDateInstance();
-    if (failure(status, "SimpleDateFormat::createDateInstance")) {
-        return;
-    }
+    failure(status, "SimpleDateFormat::createDateInstance");
     UnicodeString pat;
-    if(sdf == NULL){
-        dataerrln("Error calling DateFormat::createDateTimeInstance");
-        return;
-    }
-
     pat = sdf->toPattern(pat);
     logln("pattern: " + pat);
     UnicodeString fmtd;
@@ -122,11 +114,6 @@ void DateFormatRegressionTest::Test4052408(void)
 
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::SHORT,
                                                 DateFormat::SHORT, Locale::getUS());
-    if (fmt == NULL) {
-        dataerrln("Error calling DateFormat::createDateTimeInstance");
-        return;
-    }
-
     UDate dt = date(97, UCAL_MAY, 3, 8, 55);
     UnicodeString str;
     str = fmt->format(dt, str);
@@ -224,10 +211,7 @@ void DateFormatRegressionTest::Test4056591(void)
 
     //try {
         SimpleDateFormat *fmt = new SimpleDateFormat(UnicodeString("yyMMdd"), Locale::getUS(), status);
-        if (failure(status, "new SimpleDateFormat", TRUE)) {
-            delete fmt;
-            return;
-        }
+        failure(status, "new SimpleDateFormat");
         UDate start = date(1809-1900, UCAL_DECEMBER, 25);
         fmt->set2DigitYearStart(start, status);
         failure(status, "fmt->setTwoDigitStartDate");
@@ -283,7 +267,7 @@ void DateFormatRegressionTest::Test4059917(void)
     UnicodeString myDate;
 
     fmt = new SimpleDateFormat( UnicodeString("yyyy/MM/dd"), status );
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
     myDate = "1997/01/01";
     aux917( fmt, myDate );
     
@@ -291,7 +275,7 @@ void DateFormatRegressionTest::Test4059917(void)
     fmt = NULL;
     
     fmt = new SimpleDateFormat( UnicodeString("yyyyMMdd"), status );
-    if(failure(status, "new SimpleDateFormat")) return;
+    failure(status, "new SimpleDateFormat");
     myDate = "19970101";
     aux917( fmt, myDate );
               
@@ -339,18 +323,12 @@ void DateFormatRegressionTest::Test4060212(void)
     logln("Using yyyy-DDD.hh:mm:ss");
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *formatter = new SimpleDateFormat(UnicodeString("yyyy-DDD.hh:mm:ss"), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
     ParsePosition pos(0);
     UDate myDate = formatter->parse( dateString, pos );
     UnicodeString myString;
     DateFormat *fmt = DateFormat::createDateTimeInstance( DateFormat::FULL,
                                                             DateFormat::LONG);
-    if (fmt == NULL) {
-        dataerrln("Error calling DateFormat::createDateTimeInstance");
-        delete formatter;
-        return;
-    }
-
     myString = fmt->format( myDate, myString);
     logln( myString );
 
@@ -362,13 +340,11 @@ void DateFormatRegressionTest::Test4060212(void)
         errln((UnicodeString) "Fail: Got " + cal->get(UCAL_DAY_OF_YEAR, status) +
                             " Want 40");
 
-    // this is an odd usage of "ddd" and it doesn't
-    // work now that date values are range checked per #3579.
     logln("Using yyyy-ddd.hh:mm:ss");
     delete formatter;
     formatter = NULL;
     formatter = new SimpleDateFormat(UnicodeString("yyyy-ddd.hh:mm:ss"), status);
-    if(failure(status, "new SimpleDateFormat")) return;
+    failure(status, "new SimpleDateFormat");
     pos.setIndex(0);
     myDate = formatter->parse( dateString, pos );
     myString = fmt->format( myDate, myString );
@@ -392,15 +368,15 @@ void DateFormatRegressionTest::Test4061287(void)
     UErrorCode status = U_ZERO_ERROR;
     
     SimpleDateFormat *df = new SimpleDateFormat(UnicodeString("dd/MM/yyyy"), status);
-    if (U_FAILURE(status)) {
-        dataerrln("Fail new SimpleDateFormat: %s", u_errorName(status));
-        delete df;
-        return;
+    if(U_FAILURE(status)) {
+      errln("Couldn't create SimpleDateFormat, error: %s", u_errorName(status));
+      delete df;
+      return;
     }
     failure(status, "new SimpleDateFormat");
     //try {
     logln(UnicodeString("") + df->parse("35/01/1971", status));  
-    failure(status, "df->parse(\"35/01/1971\")");
+    failure(status, "df->parse");
     //logln(df.parse("35/01/1971").toString());
     //}
     /*catch (ParseException e) {
@@ -441,16 +417,7 @@ void DateFormatRegressionTest::Test4065240(void)
         TimeZone::adoptDefault(TimeZone::createTimeZone("EST"));
         curDate = date(98, 0, 1);
         shortdate = DateFormat::createDateInstance(DateFormat::SHORT);
-        if (shortdate == NULL){
-            dataerrln("Error calling DateFormat::createDateInstance");
-            return;
-        }
-
         fulldate = DateFormat::createDateTimeInstance(DateFormat::LONG, DateFormat::LONG);
-        if (fulldate == NULL){
-            dataerrln("Error calling DateFormat::createDateTimeInstance");
-            return;
-        }
         strShortDate = "The current date (short form) is ";
         UnicodeString temp;
         temp = shortdate->format(curDate, temp);
@@ -467,7 +434,7 @@ void DateFormatRegressionTest::Test4065240(void)
 
         // Check to see if the resource is present; if not, we can't test
         ResourceBundle *bundle = new ResourceBundle(
-            NULL, *curLocale, status);
+            u_getDataDirectory(), *curLocale, status);
         failure(status, "new ResourceBundle");
             //(UnicodeString) "java.text.resources.DateFormatZoneData", curLocale);
 
@@ -487,6 +454,7 @@ void DateFormatRegressionTest::Test4065240(void)
     failure(status, "Locale::setDefault");
     TimeZone::setDefault(*saveZone);
     //}
+
     delete shortdate;
     delete fulldate;
     delete saveZone;
@@ -510,14 +478,7 @@ void DateFormatRegressionTest::Test4071441(void)
 {
     DateFormat *fmtA = DateFormat::createInstance();
     DateFormat *fmtB = DateFormat::createInstance();
-
-    if (fmtA == NULL || fmtB == NULL){
-        dataerrln("Error calling DateFormat::createInstance");
-        delete fmtA;
-        delete fmtB;
-        return;
-    }
-
+    
     // {sfb} Is it OK to cast away const here?
     Calendar *calA = (Calendar*) fmtA->getCalendar();
     Calendar *calB = (Calendar*) fmtB->getCalendar();
@@ -556,7 +517,7 @@ void DateFormatRegressionTest::Test4071441(void)
   US locale a string formatted according to mm/dd/yy and parses it
   correctly.
 
-  When given a string mm/dd/yyyy [sic] it only parses up to the first
+  When given a string mm/dd/yyyy it only parses up to the first
   two y's, typically resulting in a date in the year 1919.
   
   Please extend the parsing method(s) to handle strings with
@@ -569,9 +530,9 @@ void DateFormatRegressionTest::Test4073003(void)
 {
     //try {
     UErrorCode ec = U_ZERO_ERROR;
-    SimpleDateFormat fmt("MM/dd/yy", Locale::getUK(), ec);
+    SimpleDateFormat fmt("dd/MM/yy", Locale::getUK(), ec);
     if (U_FAILURE(ec)) {
-        dataerrln("FAIL: SimpleDateFormat constructor - %s", u_errorName(ec));
+        errln("FAIL: SimpleDateFormat constructor");
         return;
     }
         UnicodeString tests [] = { 
@@ -610,7 +571,7 @@ void DateFormatRegressionTest::Test4089106(void)
         UErrorCode status = U_ZERO_ERROR;
         SimpleDateFormat *f = new SimpleDateFormat(status);
         if(U_FAILURE(status)) {
-          dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+          errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
           delete f;
           delete def;
           delete z;
@@ -709,8 +670,8 @@ void DateFormatRegressionTest::Test4101483(void)
 {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *sdf = new SimpleDateFormat(UnicodeString("z"), Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
-    FieldPosition fp(UDAT_TIMEZONE_FIELD);
+    failure(status, "new SimpleDateFormat");
+    FieldPosition fp(DateFormat::TIMEZONE_FIELD);
     //Date d = date(9234567890L);
     UDate d = 9234567890.0;
     //StringBuffer buf = new StringBuffer("");
@@ -742,7 +703,7 @@ void DateFormatRegressionTest::Test4103340(void)
     // and some arbitrary time 
     UDate d = date(97, 3, 1, 1, 1, 1); 
     SimpleDateFormat *df = new SimpleDateFormat(UnicodeString("MMMM"), Locale::getUS(), status); 
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
 
     UnicodeString s;
     s = dateToString(d, s);
@@ -772,7 +733,7 @@ void DateFormatRegressionTest::Test4103341(void)
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *simple = new SimpleDateFormat(UnicodeString("MM/dd/yyyy HH:mm"), status);
     if(U_FAILURE(status)) {
-      dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
       delete simple;
       return;
     }
@@ -797,11 +758,11 @@ void DateFormatRegressionTest::Test4104136(void)
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *sdf = new SimpleDateFormat(status); 
     if(U_FAILURE(status)) {
-      dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
       delete sdf;
       return;
     }
-    if(failure(status, "new SimpleDateFormat")) return;
+    failure(status, "new SimpleDateFormat");
     UnicodeString pattern = "'time' hh:mm"; 
     sdf->applyPattern(pattern); 
     logln("pattern: \"" + pattern + "\""); 
@@ -862,7 +823,7 @@ void DateFormatRegressionTest::Test4104522(void)
     
     SimpleDateFormat *sdf = new SimpleDateFormat(status);
     if(U_FAILURE(status)) {
-      dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
       delete sdf;
       return;
     }
@@ -905,7 +866,7 @@ void DateFormatRegressionTest::Test4106807(void)
         new SimpleDateFormat(UnicodeString("yyyyMMddHHmmss %"), status)
     };
     if(U_FAILURE(status)) {
-      dataerrln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
+      errln("Couldn't create SimpleDateFormat, error %s", u_errorName(status));
       delete sdfs[0];
       delete sdfs[1];
       delete sdfs[2];
@@ -1008,7 +969,7 @@ void DateFormatRegressionTest::Test4134203(void)
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString dateFormat = "MM/dd/yy HH:mm:ss zzz";
     SimpleDateFormat *fmt = new SimpleDateFormat(dateFormat, status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
     ParsePosition p0(0);
     UDate d = fmt->parse("01/22/92 04:52:00 GMT", p0);
     logln(dateToString(d));
@@ -1030,7 +991,7 @@ void DateFormatRegressionTest::Test4151631(void)
     logln("pattern=" + pattern);
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *format = new SimpleDateFormat(pattern, Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
     UnicodeString result;
     FieldPosition pos(FieldPosition::DONT_CARE);
     result = format->format(date(1998-1900, UCAL_JUNE, 30, 13, 30, 0), result, pos);
@@ -1054,7 +1015,7 @@ void DateFormatRegressionTest::Test4151706(void)
     UnicodeString dateString("Thursday, 31-Dec-98 23:00:00 GMT");
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat fmt(UnicodeString("EEEE, dd-MMM-yy HH:mm:ss z"), Locale::getUS(), status);
-    if (failure(status, "new SimpleDateFormat", TRUE)) return;
+    failure(status, "new SimpleDateFormat");
     //try {
         UDate d = fmt.parse(dateString, status);
         failure(status, "fmt->parse");
@@ -1081,10 +1042,8 @@ DateFormatRegressionTest::Test4162071(void)
     UnicodeString format("EEE', 'dd-MMM-yyyy HH:mm:ss z"); // RFC 822/1123
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat df(format, Locale::getUS(), status);
-    if(U_FAILURE(status)) {
-        dataerrln("Couldn't create SimpleDateFormat - %s", u_errorName(status));
-        return;
-    }
+    if(U_FAILURE(status))
+        errln("Couldn't create SimpleDateFormat");
     
     //try {
         UDate x = df.parse(dateString, status);
@@ -1108,7 +1067,7 @@ void DateFormatRegressionTest::Test4182066(void) {
     SimpleDateFormat fmt("MM/dd/yy", Locale::getUS(), status);
     SimpleDateFormat dispFmt("MMM dd yyyy GG", Locale::getUS(), status);
     if (U_FAILURE(status)) {
-        dataerrln("Couldn't create SimpleDateFormat - %s", u_errorName(status));
+        errln("Couldn't create SimpleDateFormat");
         return;
     }
 
@@ -1193,7 +1152,7 @@ DateFormatRegressionTest::Test4210209(void) {
     DateFormat& fmt = *(DateFormat*)&sfmt; // Yuck: See j25
     DateFormat& disp = *(DateFormat*)&sdisp; // Yuck: See j25
     if (U_FAILURE(status)) {
-        dataerrln("Couldn't create SimpleDateFormat - %s", u_errorName(status));
+        errln("Couldn't create SimpleDateFormat");
         return;
     }
     Calendar* calx = (Calendar*)fmt.getCalendar(); // cast away const!
@@ -1238,11 +1197,6 @@ void DateFormatRegressionTest::Test714(void)
     DateFormat *fmt = DateFormat::createDateTimeInstance(DateFormat::NONE,
                                                          DateFormat::MEDIUM,
                                                          Locale::getUS());
-    if (fmt == NULL) {
-        dataerrln("Error calling DateFormat::createDateTimeInstance");
-        return;
-    }
-
     UnicodeString s;
         UnicodeString tests = 
           (UnicodeString) "7:25:43 AM" ;
@@ -1330,15 +1284,6 @@ void DateFormatRegressionTest::Test1684(void)
   UnicodeString pattern("yyyy MM WW EEE","");
   Calendar *cal = new GregorianCalendar(status);
   SimpleDateFormat *sdf = new SimpleDateFormat(pattern,status);
-  if (U_FAILURE(status)) {
-    dataerrln("Error constructing SimpleDateFormat");
-    for(i=0;i<kTest1684Count;i++) {
-        delete tests[i];
-    }
-    delete cal;
-    delete sdf;
-    return;
-  }
   cal->setFirstDayOfWeek(UCAL_SUNDAY);
   cal->setMinimalDaysInFirstWeek(1);
 
@@ -1423,7 +1368,7 @@ void DateFormatRegressionTest::Test1684(void)
   } else {
     logln(info);
   }
-
+  
   for(i=0;i<kTest1684Count;i++) {
     delete tests[i];
   }
@@ -1431,36 +1376,6 @@ void DateFormatRegressionTest::Test1684(void)
   delete sdf;
 }
 
-void DateFormatRegressionTest::Test5554(void)
-{
-  UErrorCode status = U_ZERO_ERROR;
-  UnicodeString pattern("Z","");
-  UnicodeString newfoundland("Canada/Newfoundland", "");
-  TimeZone *zone = TimeZone::createTimeZone(newfoundland);
-  Calendar *cal = new GregorianCalendar(zone, status);
-  SimpleDateFormat *sdf = new SimpleDateFormat(pattern,status);
-  if (U_FAILURE(status)) {
-    dataerrln("Error constructing SimpleDateFormat");
-    delete cal;
-    delete sdf;
-    return;
-  }
-  cal->set(2007, 1, 14);
-  UDate date = cal->getTime(status);
-  if (U_FAILURE(status)) {
-    errln("Error getting time to format");
-    return;
-  };
-  sdf->adoptCalendar(cal);
-  UnicodeString result;
-  UnicodeString correct("-0330", "");
-  sdf->format(date, result);
-  if (result != correct) {
-    errln("\nError: Newfoundland Z of Jan 14, 2007 gave '" + result + "', expected '" + correct + "'");
-  }
-  delete sdf;
-}
-
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 //eof
