@@ -1,16 +1,22 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
  
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "dtfmttst.h"
 #include "unicode/timezone.h"
 #include "unicode/gregocal.h"
 #include "unicode/smpdtfmt.h"
 #include "unicode/datefmt.h"
 #include "unicode/simpletz.h"
-
+#include "unicode/strenum.h"
+#include "cmemory.h"
+#include "caltest.h"  // for fieldName
 // *****************************************************************************
 // class DateFormatTest
 // *****************************************************************************
@@ -37,6 +43,10 @@ void DateFormatTest::runIndexedTest( int32_t index, UBool exec, const char* &nam
         TESTCASE(15,TestLocaleDateFormat);
         TESTCASE(16,TestWallyWedel);
         TESTCASE(17,TestDateFormatCalendar);
+        TESTCASE(18,TestSpaceParsing);
+        TESTCASE(19,TestExactCountFormat);
+        TESTCASE(20,TestWhiteSpaceParsing);
+        TESTCASE(21,TestInvalidPattern);
         default: name = ""; break;
     }
 }
@@ -62,7 +72,8 @@ void DateFormatTest::TestWallyWedel()
      * A String array for the time zone ids.
      */
     int32_t ids_length;
-    const UnicodeString **ids = TimeZone::createAvailableIDs(ids_length);
+    StringEnumeration* ids = TimeZone::createEnumeration();
+    ids_length = ids->count(status);
     /*
      * How many ids do we have?
      */
@@ -78,11 +89,12 @@ void DateFormatTest::TestWallyWedel()
     Calendar *cal = Calendar::createInstance(status);
     for (int32_t i = 0; i < ids_length; i++) {
         // logln(i + " " + ids[i]);
-        TimeZone *ttz = TimeZone::createTimeZone(*ids[i]);
+        const UnicodeString* id = ids->snext(status);
+        TimeZone *ttz = TimeZone::createTimeZone(*id);
         // offset = ttz.getRawOffset();
         cal->setTimeZone(*ttz);
         cal->setTime(today, status);
-        offset = cal->get(Calendar::ZONE_OFFSET, status) + cal->get(Calendar::DST_OFFSET, status);
+        offset = cal->get(UCAL_ZONE_OFFSET, status) + cal->get(UCAL_DST_OFFSET, status);
         // logln(i + " " + ids[i] + " offset " + offset);
         const char* sign = "+";
         if (offset < 0) {
@@ -117,19 +129,20 @@ void DateFormatTest::TestWallyWedel()
         UBool ok = fmtDstOffset == 0 || *fmtDstOffset == dstOffset;
         if (ok)
         {
-            logln(UnicodeString() + i + " " + *ids[i] + " " + dstOffset +
+            logln(UnicodeString() + i + " " + *id + " " + dstOffset +
                   " " + fmtOffset +
                   (fmtDstOffset != 0 ? " ok" : " ?"));
         }
         else
         {
-            errln(UnicodeString() + i + " " + *ids[i] + " " + dstOffset +
+            errln(UnicodeString() + i + " " + *id + " " + dstOffset +
                   " " + fmtOffset + " *** FAIL ***");
         }
         delete ttz;
         delete fmtDstOffset;
     }
     delete cal;
+    //  delete ids;   // TODO:  BAD API
     delete ids;
     delete sdf;
     delete tz;
@@ -204,13 +217,6 @@ DateFormatTest::escape(UnicodeString& s)
     return (s = buf);
 }
  
-const char* DateFormatTest::fieldNames[] = {
-        "ERA", "YEAR", "MONTH", "WEEK_OF_YEAR", "WEEK_OF_MONTH", "DAY_OF_MONTH", 
-        "DAY_OF_YEAR", "DAY_OF_WEEK", "DAY_OF_WEEK_IN_MONTH", "AM_PM", "HOUR", 
-        "HOUR_OF_DAY", "MINUTE", "SECOND", "MILLISECOND", "ZONE_OFFSET", //"DST_OFFSET", 
-        "YEAR_WOY", "DOW_LOCAL"
-};
- 
 // -------------------------------------
 
 // Map Calendar field number to to DateFormat field number
@@ -234,6 +240,9 @@ DateFormatTest::fgCalendarToDateFormatField[] = {
     DateFormat::kTimezoneField, 
     DateFormat::kYearWOYField,
     DateFormat::kDOWLocalField,
+    DateFormat::kExtendedYearField,
+    DateFormat::kJulianDayField,
+    DateFormat::kMillisecondsInDayField,
     (DateFormat::EField) -1
 };
 
@@ -257,20 +266,24 @@ DateFormatTest::TestFieldPosition(void)
     /* field values, in Calendar order */
 
     const char* expected[] = {
+    /* 0: US */
         "", "1997", "August", "", "", "13", "", "Wednesday", "", "PM", "2", "", 
         "34", "12", "", "PDT", "", 
-        /* Following two added by weiv for two new fields */ "", "", 
+        /* Following two added by weiv for two new fields */ "", "", "","","",
+    /* 1: France */
         "", "1997", "#",/* # is a marker for "ao\xfbt" == "aou^t" */  "", "", "13", "", "mercredi", 
         "", "", "", "14", "34", "", "", "GMT-07:00", "", 
-        /* Following two added by weiv for two new fields */ "", "", 
+        /* Following two added by weiv for two new fields */ "", "", "","","",
+    /* 2: (short fields) */
         "AD", "97", "8", "33", "3", "13", "225", "Wed", "2", "PM", "2", 
         "14", "34", "12", "5", "PDT", 
-        /* Following two added by weiv for two new fields */ "97", "4", "",
+        /* Following two added by weiv for two new fields */ "97", "4", "", "","","",
+    /* 3: (long fields) */
         "AD", "1997", "August", "0033", 
         "0003", "0013", "0225", "Wednesday", "0002", "PM", "0002", "0014", 
         "0034", "0012", "513", "Pacific Daylight Time", 
-        /* Following two added by weiv for two new fields */ "1997", "0004",
-        ""
+        /* Following two added by weiv for two new fields */ "1997", "0004", "","","", "",
+        NULL
 
     };
 
@@ -279,7 +292,7 @@ DateFormatTest::TestFieldPosition(void)
 
     dateFormats[0] = DateFormat::createDateTimeInstance(DateFormat::FULL, DateFormat::FULL, Locale::getUS());
     dateFormats[1] = DateFormat::createDateTimeInstance(DateFormat::FULL, DateFormat::FULL, Locale::getFrance());
-    dateFormats[2] = new SimpleDateFormat((UnicodeString)"G, y, M, d, k, H, m, s, S, E, D, F, w, W, a, h, K, z, Y, e", status);
+    dateFormats[2] = new SimpleDateFormat((UnicodeString)"G, yy, M, d, k, H, m, s, S, E, D, F, w, W, a, h, K, z, YY, e", status);
     dateFormats[3] = new SimpleDateFormat((UnicodeString)"GGGG, yyyy, MMMM, dddd, kkkk, HHHH, mmmm, ssss, SSSS, EEEE, DDDD, FFFF, wwww, WWWW, aaaa, hhhh, KKKK, zzzz, YYYY, eeee", status);
     for (j = 0, exp = 0; j < dateFormats_length;++j) {
         UnicodeString str;
@@ -287,10 +300,16 @@ DateFormatTest::TestFieldPosition(void)
         logln((UnicodeString)" Pattern = " + ((SimpleDateFormat*)df)->toPattern(str));
         str.truncate(0);
         logln((UnicodeString)"  Result = " + df->format(someDate, str));
-        for (int32_t i = 0; i < Calendar::FIELD_COUNT;++i) {
+        for (int32_t i = 0; i < UCAL_FIELD_COUNT;++i) {
             UnicodeString field;
             getFieldText(df, i, someDate, field);
             UnicodeString expStr;
+
+            if(expected[exp] == NULL) {
+              errln("FAIL: ran out of 'expected' strings (pattern %d, field %d - item %d) .. perhaps number of calendar fields has changed?\n", j, i, exp);
+              return; /* leak? This is a Fatal err */
+            }
+
             if(expected[exp][0]!='#') {
                 expStr=UnicodeString(expected[exp]);
             } else {
@@ -298,8 +317,8 @@ DateFormatTest::TestFieldPosition(void)
                 expStr.append((UChar)0x61).append((UChar)0x6f).append((UChar32)0xfb).append((UChar)0x74);
             }
             
-            if (!(field == expStr)) errln(UnicodeString("FAIL: field #") + i + " " +
-                fieldNames[i] + " = \"" + escape(field) + "\", expected \"" + escape(expStr) + "\"");
+            if (!(field == expStr)) errln(UnicodeString("FAIL: pattern #") + j + ", field #" + i + " " + 
+                                          CalendarTest::fieldName((UCalendarDateFields)i) + " = \"" + escape(field) + "\", expected \"" + escape(expStr) + "\"");
             ++exp;
         }
     }
@@ -467,8 +486,8 @@ DateFormatTest::TestCzechMonths459()
     DateFormat* fmt = DateFormat::createDateInstance(DateFormat::FULL, Locale("cs", "", ""));
     UnicodeString pattern;
     logln((UnicodeString)"Pattern " + ((SimpleDateFormat*) fmt)->toPattern(pattern));
-    UDate june = date(97, Calendar::JUNE, 15);
-    UDate july = date(97, Calendar::JULY, 15);
+    UDate june = date(97, UCAL_JUNE, 15);
+    UDate july = date(97, UCAL_JULY, 15);
     UnicodeString juneStr; fmt->format(june, juneStr);
     UnicodeString julyStr; fmt->format(july, julyStr);
     //try {
@@ -477,13 +496,13 @@ DateFormatTest::TestCzechMonths459()
         UnicodeString s; fmt->format(d, s);
         int32_t month,yr,day,hr,min,sec; dateToFields(d,yr,month,day,hr,min,sec);
         logln((UnicodeString)"  -> parse -> " + s + " (month = " + month + ")");
-        if (month != Calendar::JUNE) errln((UnicodeString)"FAIL: Month should be June");
+        if (month != UCAL_JUNE) errln((UnicodeString)"FAIL: Month should be June");
         logln((UnicodeString)"format(July 15 1997) = " + julyStr);
         d = fmt->parse(julyStr, status);
         fmt->format(d, s);
         dateToFields(d,yr,month,day,hr,min,sec);
         logln((UnicodeString)"  -> parse -> " + s + " (month = " + month + ")");
-        if (month != Calendar::JULY) errln((UnicodeString)"FAIL: Month should be July");
+        if (month != UCAL_JULY) errln((UnicodeString)"FAIL: Month should be July");
     //}
     //catch(ParseException e) {
     if (U_FAILURE(status))
@@ -578,7 +597,7 @@ DateFormatTest::TestQuotePattern161()
 {
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat* formatter = new SimpleDateFormat((UnicodeString)"MM/dd/yyyy 'at' hh:mm:ss a zzz", status);
-    UDate currentTime_1 = date(97, Calendar::AUGUST, 13, 10, 42, 28);
+    UDate currentTime_1 = date(97, UCAL_AUGUST, 13, 10, 42, 28);
     UnicodeString dateString; ((DateFormat*)formatter)->format(currentTime_1, dateString);
     UnicodeString exp("08/13/1997 at 10:42:28 AM ");
     logln((UnicodeString)"format(" + dateToString(currentTime_1) + ") = " + dateString);
@@ -683,6 +702,11 @@ DateFormatTest::TestBadInput135a()
 {
   UErrorCode status = U_ZERO_ERROR;
   SimpleDateFormat* dateParse = new SimpleDateFormat(status);
+  if(U_FAILURE(status)) {
+    errln("Failed creating SimpleDateFormat with %s. Quitting test", u_errorName(status));
+    delete dateParse;
+    return;
+  }
   const char* s;
   UDate date;
   const uint32_t PF_LENGTH = (int32_t)(sizeof(parseFormats)/sizeof(parseFormats[0]));
@@ -757,10 +781,14 @@ DateFormatTest::TestBadInput135a()
 void
 DateFormatTest::TestTwoDigitYear()
 {
-    DateFormat* fmt = DateFormat::createDateInstance(DateFormat::SHORT, Locale::getUK());
-    parse2DigitYear(*fmt, "5/6/17", date(117, Calendar::JUNE, 5));
-    parse2DigitYear(*fmt, "4/6/34", date(34, Calendar::JUNE, 4));
-    delete fmt;
+    UErrorCode ec = U_ZERO_ERROR;
+    SimpleDateFormat fmt("dd/MM/yy", Locale::getUK(), ec);
+    if (U_FAILURE(ec)) {
+        errln("FAIL: SimpleDateFormat constructor");
+        return;
+    }
+    parse2DigitYear(fmt, "5/6/17", date(117, UCAL_JUNE, 5));
+    parse2DigitYear(fmt, "4/6/34", date(34, UCAL_JUNE, 4));
 }
  
 // -------------------------------------
@@ -796,6 +824,11 @@ DateFormatTest::TestDateFormatZone061()
     date= 859248000000.0;
     logln((UnicodeString)"Date 1997/3/25 00:00 GMT: " + date);
     formatter = new SimpleDateFormat((UnicodeString)"dd-MMM-yyyyy HH:mm", Locale::getUK(), status);
+    if(U_FAILURE(status)) {
+      errln("Failed creating SimpleDateFormat with %s. Quitting test", u_errorName(status));
+      delete formatter;
+      return;
+    }
     formatter->adoptTimeZone(TimeZone::createTimeZone("GMT"));
     UnicodeString temp; formatter->format(date, temp);
     logln((UnicodeString)"Formatted in GMT to: " + temp);
@@ -843,7 +876,7 @@ DateFormatTest::TestDateFormatZone146()
         //*****************************greenwichcalendar.setTimeZone(TimeZone.getDefault());
         //greenwichcalendar.set(1997, 3, 4, 23, 0);
         // try anything to set hour to 23:00 !!!
-        greenwichcalendar->set(Calendar::HOUR_OF_DAY, 23);
+        greenwichcalendar->set(UCAL_HOUR_OF_DAY, 23);
         // get time
         UDate greenwichdate = greenwichcalendar->getTime(status);
         // format every way
@@ -891,7 +924,7 @@ DateFormatTest::TestDateFormatZone146()
 void
 DateFormatTest::TestLocaleDateFormat() // Bug 495
 {
-    UDate testDate = date(97, Calendar::SEPTEMBER, 15);
+    UDate testDate = date(97, UCAL_SEPTEMBER, 15);
     DateFormat *dfFrench = DateFormat::createDateTimeInstance(DateFormat::FULL, 
         DateFormat::FULL, Locale::getFrench());
     DateFormat *dfUS = DateFormat::createDateTimeInstance(DateFormat::FULL, 
@@ -998,4 +1031,151 @@ void DateFormatTest::TestDateFormatCalendar() {
     delete cal;
 }
 
+/**
+ * Test DateFormat's parsing of space characters.  See jitterbug 1916.
+ */
+void DateFormatTest::TestSpaceParsing() {
+    const char* DATA[] = {
+        "yyyy MM dd HH:mm:ss",
+
+        // pattern, input, expected parse or NULL if expect parse failure
+        "MMMM d yy", " 04 05 06",  NULL, // MMMM wants Apr/April
+        NULL,        "04 05 06",   NULL,
+        "MM d yy",   " 04 05 06",  "2006 04 05 00:00:00",
+        NULL,        "04 05 06",   "2006 04 05 00:00:00",
+        "MMMM d yy", " Apr 05 06", "2006 04 05 00:00:00",
+        NULL,        "Apr 05 06",  "2006 04 05 00:00:00",
+    };
+    const int32_t DATA_len = sizeof(DATA)/sizeof(DATA[0]);
+
+    expectParse(DATA, DATA_len, Locale("en"));
+}
+
+/**
+ * Test handling of "HHmmss" pattern.
+ */
+void DateFormatTest::TestExactCountFormat() {
+    const char* DATA[] = {
+        "yyyy MM dd HH:mm:ss",
+
+        // pattern, input, expected parse or NULL if expect parse failure
+        "HHmmss", "123456", "1970 01 01 12:34:56",
+        NULL,     "12345",  "1970 01 01 01:23:45",
+        NULL,     "1234",   NULL,
+        NULL,     "00-05",  NULL,
+        NULL,     "12-34",  NULL,
+        NULL,     "00+05",  NULL,
+        "ahhmm",  "PM730",  "1970 01 01 19:30:00",
+    };
+    const int32_t DATA_len = sizeof(DATA)/sizeof(DATA[0]);
+
+    expectParse(DATA, DATA_len, Locale("en"));
+}
+
+/**
+ * Test handling of white space.
+ */
+void DateFormatTest::TestWhiteSpaceParsing() {
+    const char* DATA[] = {
+        "yyyy MM dd",
+        
+        // pattern, input, expected parse or null if expect parse failure
+        
+        // Pattern space run should parse input text space run
+        "MM   d yy",   " 04 01 03",    "2003 04 01",
+        NULL,          " 04  01   03 ", "2003 04 01",
+    };
+    const int32_t DATA_len = sizeof(DATA)/sizeof(DATA[0]);
+    
+    expectParse(DATA, DATA_len, Locale("en"));
+}
+
+
+void DateFormatTest::TestInvalidPattern() {
+    UErrorCode ec = U_ZERO_ERROR;
+    SimpleDateFormat f(UnicodeString("Yesterday"), ec);
+    UnicodeString out;
+    FieldPosition pos;
+    f.format((UDate)0, out, pos);
+    logln(out);
+    // The bug is that the call to format() will crash.  By not
+    // crashing, the test passes.
+}
+
+/**
+ * Test parsing.  Input is an array that starts with the following
+ * header:
+ *
+ * [0]   = pattern string to parse [i+2] with
+ *
+ * followed by test cases, each of which is 3 array elements:
+ *
+ * [i]   = pattern, or NULL to reuse prior pattern
+ * [i+1] = input string
+ * [i+2] = expected parse result (parsed with pattern [0])
+ *
+ * If expect parse failure, then [i+2] should be NULL.
+ */
+void DateFormatTest::expectParse(const char** data, int32_t data_length,
+                                 const Locale& loc) {
+    const UDate FAIL = (UDate) -1;
+    const UnicodeString FAIL_STR("parse failure");
+    int32_t i = 0;
+
+    UErrorCode ec = U_ZERO_ERROR;
+    SimpleDateFormat fmt("", loc, ec);
+    SimpleDateFormat ref(data[i++], loc, ec);
+    SimpleDateFormat gotfmt("G yyyy MM dd HH:mm:ss z", loc, ec);
+    if (U_FAILURE(ec)) {
+        errln("FAIL: SimpleDateFormat constructor");
+        return;
+    }
+
+    const char* currentPat = NULL;
+    while (i<data_length) {
+        const char* pattern  = data[i++];
+        const char* input    = data[i++];
+        const char* expected = data[i++];
+
+        ec = U_ZERO_ERROR;
+        if (pattern != NULL) {
+            fmt.applyPattern(pattern);
+            currentPat = pattern;
+        }
+        UDate got = fmt.parse(input, ec);
+        UnicodeString gotstr(FAIL_STR);
+        if (U_FAILURE(ec)) {
+            got = FAIL;
+        } else {
+            gotstr.remove();
+            gotfmt.format(got, gotstr);
+        }
+
+        UErrorCode ec2 = U_ZERO_ERROR;
+        UDate exp = FAIL;
+        UnicodeString expstr(FAIL_STR);
+        if (expected != NULL) {
+            expstr = expected;
+            exp = ref.parse(expstr, ec2);
+            if (U_FAILURE(ec2)) {
+                // This only happens if expected is in wrong format --
+                // should never happen once test is debugged.
+                errln("FAIL: Internal test error");
+                return;
+            }
+        }
+
+        if (got == exp) {
+            logln((UnicodeString)"Ok: " + input + " x " +
+                  currentPat + " => " + gotstr);                
+        } else {
+            errln((UnicodeString)"FAIL: " + input + " x " +
+                  currentPat + " => " + gotstr + ", expected " +
+                  expstr);
+        }
+    }    
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 //eof
