@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2011, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -16,7 +16,7 @@
 #include "caltztst.h"
 
 /** 
- * Performs many different tests for DateFormat and SimpleDateFormat
+ * Performs many different tests for DataeFormat and SimpleDateFormat
  **/
 class DateFormatTest: public CalendarTimeZoneTest {
     // IntlTest override
@@ -44,10 +44,8 @@ public:
     /**
      * Verify that returned field position indices are correct.
      */
-    void TestFieldPosition(void);
+    virtual void TestFieldPosition(void);
  
-    void TestGeneral();
-
 public: // package
     // internal utility function
     static void getFieldText(DateFormat* df, int32_t field, UDate date, UnicodeString& str);
@@ -109,6 +107,12 @@ public:
      */
     virtual void TestBadInput135(void);
  
+private:
+    static const char* parseFormats[];
+    static const char* inputStrings[];
+ 
+    static const DateFormat::EField fgCalendarToDateFormatField [];
+
 public:
     /**
      * Verify the correct behavior when parsing an array of inputs against an
@@ -134,11 +138,7 @@ public:
      * Further test the formatting of time zones.
      */
     virtual void TestDateFormatZone146(void);
-
-    void TestTimeZoneStringsAPI(void);
-
-    void TestGMTParsing(void);
-
+ 
 public: // package
     /**
      * Test the formatting of dates in different locales.
@@ -155,83 +155,9 @@ public: // package
 
     void TestInvalidPattern(void);
 
-    void TestGreekMay(void);
-
-    void TestGenericTime(void);
-
-    void TestGenericTimeZoneOrder(void);
-
-    void Test6338(void);
-
-    void Test6726(void);
-
-    void Test6880(void);
-
-    void TestISOEra(void);
-
-    void TestFormalChineseDate(void);
-
-    void TestStandAloneGMTParse(void);
-
-public:
-    /**
-     * Test host-specific formatting.
-     */
-    void TestHost(void);
-
-public:
-    /**
-     * Test patterns added in CLDR 1.4
-     */
-    void TestEras(void);
-
-    void TestNarrowNames(void);
-
-    void TestStandAloneDays(void);
-
-    void TestStandAloneMonths(void);
-
-    void TestQuarters(void);
-    
-    void TestZTimeZoneParsing(void);
-
-    void TestRelativeClone(void);
-    
-    void TestHostClone(void);
-
-    void TestTimeZoneDisplayName(void);
-
-    void TestRoundtripWithCalendar(void);
-
-public:
-    /***
-     * Test Relative Dates
-     */
-     void TestRelative(void);
-/*   void TestRelativeError(void);
-     void TestRelativeOther(void);
-*/
-
-public:
-    /**
-     * Test parsing a number as a string
-     */
-    void TestNumberAsStringParsing(void);
-
- private:
-      void TestRelative(int daysdelta, 
-                                  const Locale& loc,
-                                  const char *expectChars);
-
  private:
     void expectParse(const char** data, int32_t data_length,
                      const Locale& locale);
-
-    void expect(const char** data, int32_t data_length,
-                const Locale& loc);
-
-    void expectFormat(const char **data, int32_t data_length,
-                      const Locale &locale);
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
