@@ -1,18 +1,18 @@
 /**
  *******************************************************************************
- * Copyright (C) 2001-2010, International Business Machines Corporation and
- * others. All Rights Reserved.
+ * Copyright (C) 2001-2003, International Business Machines Corporation and    *
+ * others. All Rights Reserved.                                                *
+ *******************************************************************************
+ *
  *******************************************************************************
  */
 
-#include <typeinfo>  // for 'typeid' to work
-
 #include "unicode/utypes.h"
 
 #if !UCONFIG_NO_SERVICE
 
 #include "icusvtst.h"
-#include "servloc.h"
+#include "iculserv.h"
 #include <stdio.h>
 
 
@@ -24,39 +24,30 @@ class WrongListener : public EventListener {
 
 class ICUNSubclass : public ICUNotifier {
     public:
-    UBool acceptsListener(const EventListener& /*l*/) const {
+    UBool acceptsListener(const EventListener& l) const {
         return TRUE;
         // return l instanceof MyListener;
     }
 
-    virtual void notifyListener(EventListener& /*l*/) const {
+    virtual void notifyListener(EventListener& l) const {
     }
 };
 
-// This factory does nothing
-class LKFSubclass0 : public LocaleKeyFactory {
-public:
-        LKFSubclass0()
-                : LocaleKeyFactory(VISIBLE, "LKFSubclass0")
-        {
-        }
-};
-
 class LKFSubclass : public LocaleKeyFactory {
-    Hashtable table;
+	Hashtable table;
 
     public:
     LKFSubclass(UBool visible) 
         : LocaleKeyFactory(visible ? VISIBLE : INVISIBLE, "LKFSubclass")
     {
-        UErrorCode status = U_ZERO_ERROR;
-        table.put("en_US", this, status);
+		UErrorCode status = U_ZERO_ERROR;
+		table.put("en_US", this, status);
     }
 
     protected:
-    virtual const Hashtable* getSupportedIDs(UErrorCode &/*status*/) const {
-        return &table;
-    }
+	virtual const Hashtable* getSupportedIDs(UErrorCode &/*status*/) const {
+		return &table;
+	}
 };
 
 class Integer : public UObject {
@@ -71,22 +62,22 @@ class Integer : public UObject {
     virtual ~Integer() {
     }
 
+    virtual UBool operator==(const UObject& other) const 
+    {
+        return other.getDynamicClassID() == getStaticClassID() &&
+            _val == ((Integer&)other)._val;
+    }
+
     public:
     /**
      * UObject boilerplate.
      */
-    static UClassID getStaticClassID() { 
-        return (UClassID)&fgClassID;
-    }
-
     virtual UClassID getDynamicClassID() const {
         return getStaticClassID();
     }
 
-    virtual UBool operator==(const UObject& other) const 
-    {
-        return typeid(*this) == typeid(other) &&
-            _val == ((Integer&)other)._val;
+    static UClassID getStaticClassID() { 
+        return (UClassID)&fgClassID;
     }
 
     public:
@@ -116,9 +107,8 @@ class TestIntegerService : public ICUService {
 
     virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& status) 
     {
-        Integer* i;
-        if (U_SUCCESS(status) && obj && (i = dynamic_cast<Integer*>(obj)) != NULL) {
-            return new SimpleFactory(i, id, visible);
+        if (U_SUCCESS(status) && obj && obj->getDynamicClassID() == Integer::getStaticClassID()) {
+            return new SimpleFactory((Integer*)obj, id, visible);
         }
         return NULL;
     }
@@ -157,15 +147,13 @@ UnicodeString append(UnicodeString& result, const UObject* obj)
     if (obj == NULL) {
         result.append("NULL");
     } else {
-        const UnicodeString* s;
-        const Locale* loc;
-        const Integer* i;
-        if ((s = dynamic_cast<const UnicodeString*>(obj)) != NULL) {
-            result.append(*s);
-        } else if ((loc = dynamic_cast<const Locale*>(obj)) != NULL) {
-            result.append(loc->getName());
-        } else if ((i = dynamic_cast<const Integer*>(obj)) != NULL) {
-            sprintf(buffer, "%d", (int)i->_val);
+        UClassID id = obj->getDynamicClassID();
+        if (id == UnicodeString::getStaticClassID()) {
+            result.append(*(UnicodeString*)obj);
+        } else if (id == Locale::getStaticClassID()) {
+            result.append(((Locale*)obj)->getName());
+        } else if (id == Integer::getStaticClassID()) {
+            sprintf(buffer, "%d", ((Integer*)obj)->_val);
             result.append(buffer);
         } else {
             sprintf(buffer, "%p", (const void*)obj);
@@ -281,7 +269,7 @@ ICUServiceTest::confirmStringsEqual(const UnicodeString& message, const UnicodeS
     if (equ) {
         logln(temp);
     } else {
-        dataerrln(temp);
+        errln(temp);
     }
 }
 
@@ -312,7 +300,7 @@ void
 ICUServiceTest::msgstr(const UnicodeString& message, UObject* obj, UBool err)
 {
     if (obj) {
-    UnicodeString* str = (UnicodeString*)obj;
+	UnicodeString* str = (UnicodeString*)obj;
         logln(message + *str);
         delete str;
     } else if (err) {
@@ -329,7 +317,7 @@ ICUServiceTest::testAPI_One()
     // register an object with one locale, 
     // search for an object with a more specific locale
     // should return the original object
-    UErrorCode status = U_ZERO_ERROR;
+	UErrorCode status = U_ZERO_ERROR;
     Integer* singleton0 = new Integer(0);
     service.registerInstance(singleton0, "en_US", status);
     {
@@ -454,7 +442,7 @@ ICUServiceTest::testAPI_One()
         confirmEqual("17) get invisible", result, singleton5);
         delete result;
     }
-    
+	
     // should not be able to locate invisible services
     {
         UErrorCode status = U_ZERO_ERROR;
@@ -472,25 +460,6 @@ ICUServiceTest::testAPI_One()
 /*
  ******************************************************************
  */
-class TestStringSimpleKeyService : public ICUService {
-public:
- 
-        virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& status) 
-    {
-                // We could put this type check into ICUService itself, but we'd still
-                // have to implement cloneInstance.  Otherwise we could just tell the service
-                // what the object type is when we create it, and the default implementation
-                // could handle everything for us.  Phooey.
-        if (obj && dynamic_cast<UnicodeString*>(obj) != NULL) {
-                        return ICUService::createSimpleFactory(obj, id, visible, status);
-        }
-        return NULL;
-    }
-
-    virtual UObject* cloneInstance(UObject* instance) const {
-        return instance ? new UnicodeString(*(UnicodeString*)instance) : NULL;
-    }
-};
 
 class TestStringService : public ICUService {
     public:
@@ -500,9 +469,8 @@ class TestStringService : public ICUService {
 
   virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& /* status */) 
     {
-        UnicodeString* s;
-        if (obj && (s = dynamic_cast<UnicodeString*>(obj)) != NULL) {
-            return new SimpleFactory(s, id, visible);
+        if (obj && obj->getDynamicClassID() == UnicodeString::getStaticClassID()) {
+            return new SimpleFactory((UnicodeString*)obj, id, visible);
         }
         return NULL;
     }
@@ -529,14 +497,14 @@ class AnonymousStringFactory : public ICUServiceFactory
         return result;
     }
 
-    static UClassID getStaticClassID() {
-        return (UClassID)&fgClassID;
-    }
-
     virtual UClassID getDynamicClassID() const {
         return getStaticClassID();
     }
 
+    static UClassID getStaticClassID() {
+        return (UClassID)&fgClassID;
+    }
+
     private:
     static const char fgClassID;
 };
@@ -564,17 +532,17 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
 
     UObject* create(const ICUServiceKey& key, const ICUService* /* service */, UErrorCode& status) const {
         if (U_FAILURE(status)) {
-        return NULL;
+	    return NULL;
         }
         UnicodeString temp;
         key.currentID(temp);
         if (U_SUCCESS(_status)) {
-        if (_ids.contains(&temp)) {
+	    if (_ids.contains(&temp)) {
                 return new UnicodeString(_factoryID + temp);
-        }
+	    }
         } else {
-        status = _status;
-    }
+	    status = _status;
+	}
         return NULL;
     }
 
@@ -605,14 +573,14 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
         return result;
     }
 
-    static UClassID getStaticClassID() {
-        return (UClassID)&fgClassID;
-    }
-
     virtual UClassID getDynamicClassID() const {
         return getStaticClassID();
     }
 
+    static UClassID getStaticClassID() {
+        return (UClassID)&fgClassID;
+    }
+
     private:
     static const char fgClassID;
 };
@@ -622,7 +590,7 @@ const char TestMultipleKeyStringFactory::fgClassID = '\0';
 void 
 ICUServiceTest::testAPI_Two()
 {
-    UErrorCode status = U_ZERO_ERROR;
+	UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerFactory(new AnonymousStringFactory(), status);
 
@@ -699,8 +667,8 @@ ICUServiceTest::testAPI_Two()
     // iterate over the display names
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(status);
-        service.getDisplayNames(names, status);
+        UVector names(userv_deleteStringPair, NULL, status);
+        service.getDisplayNames(names, Locale::getGerman(), status);
         for (int i = 0; i < names.size(); ++i) {
             const StringPair* pair = (const StringPair*)names[i];
             logln("  " + pair->displayName + " --> " + pair->id);
@@ -739,13 +707,13 @@ ICUServiceTest::testAPI_Two()
     // Rad dude's surfer gal 'replaces' Later's surfer gal
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(status);
+        UVector names(userv_deleteStringPair, NULL, status);
         service.getDisplayNames(names, Locale("es"), status);
         for (int i = 0; i < names.size(); ++i) {
             const StringPair* pair = (const StringPair*)names[i];
             logln("  " + pair->displayName + " --> " + pair->id);
         }
-        confirmIdentical("29) display names", names.size(), 7);
+        confirmIdentical("26) display names", names.size(), 7);
     }
 
     // we should get the display name corresponding to the actual id
@@ -890,7 +858,7 @@ void
 ICUServiceTest::testRBF()
 {
     // resource bundle factory.
-    UErrorCode status = U_ZERO_ERROR;
+	UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerFactory(new ICUResourceBundleFactory(), status);
 
@@ -910,7 +878,7 @@ ICUServiceTest::testRBF()
     // this should be fast since the display names were cached.
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(status);
+        UVector names(userv_deleteStringPair, NULL, status);
         service.getDisplayNames(names, Locale::getGermany(), status);
         logln("service display names for de_DE");
         for (int i = 0; i < names.size(); ++i) {
@@ -936,7 +904,7 @@ ICUServiceTest::testRBF()
             logln(UnicodeString("\n  --- ") + idNames[i] + " ---");
             {
                 UErrorCode status = U_ZERO_ERROR;
-                UVector names(status);
+                UVector names(userv_deleteStringPair, NULL, status);
                 service.getDisplayNames(names, idNames[i], status);
                 for (int i = 0; i < names.size(); ++i) {
                     const StringPair* pair = (const StringPair*)names[i];
@@ -1034,7 +1002,7 @@ class TestStringLocaleService : public ICULocaleService {
 };
 
 void ICUServiceTest::testLocale() {
-    UErrorCode status = U_ZERO_ERROR;
+	UErrorCode status = U_ZERO_ERROR;
     TestStringLocaleService service;
 
     UnicodeString* root = new UnicodeString("root");
@@ -1046,7 +1014,7 @@ void ICUServiceTest::testLocale() {
     service.registerInstance(root, "", status);
     service.registerInstance(german, "de", status);
     service.registerInstance(germany, Locale::getGermany(), status);
-    service.registerInstance(japanese, (UnicodeString)"ja", TRUE, status);
+    service.registerInstance(japanese, "ja", status);
     service.registerInstance(japan, Locale::getJapan(), status);
 
     {
@@ -1124,7 +1092,7 @@ void ICUServiceTest::testLocale() {
     }
 
     {
-        UErrorCode status = U_ZERO_ERROR;
+		UErrorCode status = U_ZERO_ERROR;
         UnicodeString english = "en";
         Locale localeResult;
         UnicodeString result;
@@ -1196,7 +1164,7 @@ void ICUServiceTest::testLocale() {
         UErrorCode status = U_ZERO_ERROR;
         StringEnumeration* locales = service.getAvailableLocales();
         if (locales) {
-            confirmIdentical("test available locales", locales->count(status), 6);
+            confirmIdentical("test available locales", locales->count(status), 5);
             logln("locales: ");
             {
                 const char* p;
@@ -1216,7 +1184,7 @@ class WrapFactory : public ICUServiceFactory {
     public:
     static const UnicodeString& getGreetingID() {
       if (greetingID == NULL) {
-    greetingID = new UnicodeString("greeting");
+	greetingID = new UnicodeString("greeting");
       }
       return *greetingID;
     }
@@ -1257,14 +1225,14 @@ class WrapFactory : public ICUServiceFactory {
     /**
      * UObject boilerplate.
      */
-    static UClassID getStaticClassID() { 
-        return (UClassID)&fgClassID;
-    }
-
     virtual UClassID getDynamicClassID() const {
         return getStaticClassID();
     }
 
+    static UClassID getStaticClassID() { 
+        return (UClassID)&fgClassID;
+    }
+
     private:
     static const char fgClassID;
     static UnicodeString* greetingID;
@@ -1278,7 +1246,7 @@ ICUServiceTest::testWrapFactory()
 {
     UnicodeString* greeting = new UnicodeString("Hello There");
     UnicodeString greetingID = "greeting";
-    UErrorCode status = U_ZERO_ERROR;
+	UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerInstance(greeting, greetingID, status);
 
@@ -1335,45 +1303,18 @@ void ICUServiceTest::testCoverage()
     }
 
     // ICUService
-        {
-                TestStringService service;
-                service.registerFactory(sf,     status);
+    TestStringService service;
+    service.registerFactory(sf, status);
 
-                {
-                        UnicodeString* result   = (UnicodeString*)service.get("object", status);
-                        if (result) {
-                                logln("object is: "     + *result);
-                                delete result;
-                        }       else {
-                                errln("could not get object");
-                        }
-                }
-        }
-  }
-  
-  // ICUServiceKey
-  {
-      UErrorCode status = U_ZERO_ERROR;
-          UnicodeString* howdy = new UnicodeString("Howdy");
-
-          TestStringSimpleKeyService service;
-          service.registerInstance(howdy, "Greetings", status);
-          {
-                  UnicodeString* result = (UnicodeString*)service.get("Greetings",      status);
-                  if (result) {
-                          logln("object is: "   + *result);
-                          delete result;
-                  }     else {
-                          errln("could not get object");
-                  }
-          }
-
-      UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, status);
-          // yuck, this is awkward to use.  All because we pass null in an overload.
-          // TODO: change this.
-          UnicodeString str("Greet");
-      service.getVisibleIDs(ids, &str, status);
-      confirmIdentical("no fallback of greet", ids.size(), 0);
+    {
+      UnicodeString* result = (UnicodeString*)service.get("object", status);
+      if (result) {
+	logln("object is: " + *result);
+	delete result;
+      } else {
+	errln("could not get object");
+      }
+    }
   }
 
   // ICULocaleService
@@ -1433,40 +1374,15 @@ void ICUServiceTest::testCoverage()
     }
     delete obj;
     delete key;
-
-        key = LocaleKey::createWithCanonicalFallback(&primary, &fallback, 123, status);
-        if (U_SUCCESS(status)) {
-                UnicodeString str;
-                key->currentDescriptor(str);
-                key->parsePrefix(str);
-                if (str != "123") {
-                        errln("did not get expected prefix");
-                }
-                delete key;
-        }
-
-        // coverage, getSupportedIDs is either overridden or the calling method is
-        LKFSubclass0 lkFactory;
-        Hashtable table0;
-        lkFactory.updateVisibleIDs(table0, status);
-        if (table0.count() != 0) {
-                errln("LKF returned non-empty hashtable");
-        }
-
-
-        // ResourceBundleFactory
-    key = LocaleKey::createWithCanonicalFallback(&primary, &fallback, status);
-        ICUResourceBundleFactory rbf;
-        UObject* icurb = rbf.create(*key, NULL, status);
-        if (icurb != NULL) {
-                logln("got resource bundle for key");
-                delete icurb;
-        }
-        delete key;
   }
 
- #if 0
- // ICUNotifier
+
+#if 0
+  // ResourceBundleFactory
+  ICUResourceBundleFactory rbf = new ICUResourceBundleFactory();
+  logln("RB: " + rbf.create(lkey, null));
+
+  // ICUNotifier
   ICUNotifier nf = new ICUNSubclass();
   try {
     nf.addListener(null);
