@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2009, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -23,21 +23,13 @@
 #include "unicode/ucnv.h"
 #include "unicode/uclean.h"
 #include "unicode/timezone.h"
-#include "unicode/curramt.h"
-#include "unicode/putil.h"
 
 #include "intltest.h"
 #include "caltztst.h"
 #include "itmajor.h"
-#include "cstring.h"
+
 #include "umutex.h"
 #include "uassert.h"
-#include "cmemory.h"
-#include "uoptions.h"
-
-#include "putilimp.h" // for uprv_getUTCtime()
-#include "unicode/locid.h"
-
 
 #ifdef XP_MAC_CONSOLE
 #include <console.h>
@@ -92,10 +84,10 @@ Int64ToUnicodeString(int64_t num)
     char buffer[64];    // nos changed from 10 to 64
     char danger = 'p';  // guard against overrunning the buffer (rtg)
 
-#ifdef U_WINDOWS
+#ifdef WIN32
     sprintf(buffer, "%I64d", num);
 #else
-    sprintf(buffer, "%lld", (long long)num);
+    sprintf(buffer, "%lld", num);
 #endif
     assert(danger == 'p');
 
@@ -110,11 +102,7 @@ operator+(const UnicodeString& left,
     char buffer[64];   // was 32, made it arbitrarily bigger (rtg)
     char danger = 'p'; // guard against overrunning the buffer (rtg)
 
-    // IEEE floating point has 52 bits of mantissa, plus one assumed bit
-    //  53*log(2)/log(10) = 15.95
-    // so there is no need to show more than 16 digits. [alan]
-
-    sprintf(buffer, "%.17g", num);
+    sprintf(buffer, "%.30g", num); // nos changed from 99 to 30
     assert(danger == 'p');
 
     return left + buffer;
@@ -123,9 +111,10 @@ operator+(const UnicodeString& left,
 #if !UCONFIG_NO_FORMATTING
 
 /**
- * Return a string display for for this, without surrounding braces.
+ * Originally coded this as operator+, but that makes the expression
+ * + char* ambiguous. - liu
  */
-UnicodeString _toString(const Formattable& f) {
+UnicodeString toString(const Formattable& f) {
     UnicodeString s;
     switch (f.getType()) {
     case Formattable::kDate:
@@ -135,75 +124,50 @@ UnicodeString _toString(const Formattable& f) {
             if (U_SUCCESS(status)) {
                 FieldPosition pos;
                 fmt.format(f.getDate(), s, pos);
-                s.insert(0, "Date:");
+                s.insert(0, "[Date:");
+                s.insert(s.length(), (UChar)0x005d);
             } else {
-                s = UnicodeString("Error creating date format]");
+                s = UnicodeString("[Error creating date format]");
             }
         }
         break;
     case Formattable::kDouble:
-        s = UnicodeString("double:") + f.getDouble();
+        s = UnicodeString("[Double:") + f.getDouble() + "]";
         break;
     case Formattable::kLong:
-        s = UnicodeString("long:") + f.getLong();
+        s = UnicodeString("[Long:") + f.getLong() + "]";
         break;
 
     case Formattable::kInt64:
-        s = UnicodeString("int64:") + Int64ToUnicodeString(f.getInt64());
+        s = UnicodeString("[Int64:") + Int64ToUnicodeString(f.getInt64()) + "]";
         break;
 
     case Formattable::kString:
         f.getString(s);
-        s.insert(0, "String:");
+        s.insert(0, "[String:");
+        s.insert(s.length(), (UChar)0x005d);
         break;
     case Formattable::kArray:
         {
             int32_t i, n;
             const Formattable* array = f.getArray(n);
-            s.insert(0, UnicodeString("Array:"));
+            s.insert(0, UnicodeString("[Array:"));
             UnicodeString delim(", ");
             for (i=0; i<n; ++i) {
                 if (i > 0) {
                     s.append(delim);
                 }
-                s = s + _toString(array[i]);
+                s = s + toString(array[i]);
             }
+            s.append(UChar(0x005d));
         }
         break;
-    case Formattable::kObject:
-        if (f.getObject()->getDynamicClassID() ==
-            CurrencyAmount::getStaticClassID()) {
-            const CurrencyAmount& c = (const CurrencyAmount&) *f.getObject();
-            s = _toString(c.getNumber()) + " " + UnicodeString(c.getISOCurrency());
-        } else {
-            s = UnicodeString("Unknown UObject");
-        }
-        break;
-    default:
-        s = UnicodeString("Unknown Formattable type=") + (int32_t)f.getType();
-        break;
     }
     return s;
 }
 
-/**
- * Originally coded this as operator+, but that makes the expression
- * + char* ambiguous. - liu
- */
-UnicodeString toString(const Formattable& f) {
-    UnicodeString s((UChar)91/*[*/);
-    s.append(_toString(f));
-    s.append((UChar)0x5d/*]*/);
-    return s;
-}
-
 #endif
 
-// useful when operator+ won't cooperate
-UnicodeString toString(int32_t n) {
-    return UnicodeString() + (long)n;
-}
-
 // stephen - cleaned up 05/05/99
 UnicodeString operator+(const UnicodeString& left, char num)
 { return left + (long)num; }
@@ -345,6 +309,68 @@ IntlTest::prettify(const UnicodeString &source, UBool parseBackslash)
     return target;
 }
 
+#if defined(_WIN32) || defined(WIN32) || defined(__OS2__) || defined(OS2)
+#define PREV_DIR ".."
+#else
+#define PREV_DIR "/../"
+#endif
+
+void
+IntlTest::pathnameInContext( char* fullname, int32_t maxsize, const char* relPath ) //nosmac
+{
+    const char* mainDir;
+    char  sepChar;
+    const char inpSepChar = '|';
+
+    // So what's going on is that ICU_DATA during tests points to:
+    //              ICU | source | data
+    //and we want   ICU | source |
+    //
+    // We'll add                 | test | testdata
+    //
+    // So, just add a .. here - back up one level
+
+    mainDir = u_getDataDirectory();
+    sepChar = U_FILE_SEP_CHAR;
+    char sepString[] = U_FILE_SEP_STRING;
+
+#if defined(XP_MAC)
+    Str255 volName;
+    int16_t volNum;
+    OSErr err = GetVol( volName, &volNum );
+    if (err != noErr)
+        volName[0] = 0;
+    mainDir = (char*) &(volName[1]);
+    mainDir[volName[0]] = 0;
+#else
+    char mainDirBuffer[255];
+    if(mainDir!=NULL) {
+        strcpy(mainDirBuffer, mainDir);
+        strcat(mainDirBuffer, PREV_DIR);
+    } else {
+        mainDirBuffer[0]='\0';
+    }
+    mainDir=mainDirBuffer;
+#endif
+
+    if (relPath[0] == '|')
+        relPath++;
+    int32_t lenMainDir = strlen(mainDir);
+    int32_t lenRelPath = strlen(relPath);
+    if (maxsize < lenMainDir + lenRelPath + 2) {
+        fullname[0] = 0;
+        return;
+    }
+    strcpy(fullname, mainDir);
+    strcat(fullname, sepString);
+    strcat(fullname, relPath);
+    char* tmp = strchr(fullname, inpSepChar);
+    while (tmp) {
+        *tmp = sepChar;
+        tmp = strchr(tmp+1, inpSepChar);
+    }
+}
+
 /*  IntlTest::setICU_DATA  - if the ICU_DATA environment variable is not already
  *                       set, try to deduce the directory in which ICU was built,
  *                       and set ICU_DATA to "icu/source/data" in that location.
@@ -484,49 +510,30 @@ void it_errln( UnicodeString message )
         IntlTest::gTest->errln( message );
 }
 
-void it_dataerr( UnicodeString message )
-{
-    if (IntlTest::gTest)
-        IntlTest::gTest->dataerr( message );
-}
-
-void it_dataerrln( UnicodeString message )
-{
-    if (IntlTest::gTest)
-        IntlTest::gTest->dataerrln( message );
-}
 
 IntlTest::IntlTest()
 {
     caller = NULL;
-    testPath = NULL;
+    path = NULL;
     LL_linestart = TRUE;
     errorCount = 0;
-    dataErrorCount = 0;
     verbose = FALSE;
     no_err_msg = FALSE;
-    warn_on_missing_data = FALSE;
     quick = FALSE;
     leaks = FALSE;
     testoutfp = stdout;
     LL_indentlevel = indentLevel_offset;
-    numProps = 0;
 }
 
 void IntlTest::setCaller( IntlTest* callingTest )
 {
     caller = callingTest;
     if (caller) {
-        warn_on_missing_data = caller->warn_on_missing_data;
         verbose = caller->verbose;
         no_err_msg = caller->no_err_msg;
         quick = caller->quick;
         testoutfp = caller->testoutfp;
         LL_indentlevel = caller->LL_indentlevel + indentLevel_offset;
-        numProps = caller->numProps;
-        for (int32_t i = 0; i < numProps; i++) {
-            proplines[i] = caller->proplines[i];
-        }
     }
 }
 
@@ -534,12 +541,12 @@ UBool IntlTest::callTest( IntlTest& testToBeCalled, char* par )
 {
     execCount--; // correct a previously assumed test-exec, as this only calls a subtest
     testToBeCalled.setCaller( this );
-    return testToBeCalled.runTest( testPath, par );
+    return testToBeCalled.runTest( path, par );
 }
 
 void IntlTest::setPath( char* pathVal )
 {
-    this->testPath = pathVal;
+    this->path = pathVal;
 }
 
 UBool IntlTest::setVerbose( UBool verboseVal )
@@ -549,13 +556,6 @@ UBool IntlTest::setVerbose( UBool verboseVal )
     return rval;
 }
 
-UBool IntlTest::setWarnOnMissingData( UBool warn_on_missing_dataVal )
-{
-    UBool rval = this->warn_on_missing_data;
-    this->warn_on_missing_data = warn_on_missing_dataVal;
-    return rval;
-}
-
 UBool IntlTest::setNoErrMsg( UBool no_err_msgVal )
 {
     UBool rval = this->no_err_msg;
@@ -582,11 +582,6 @@ int32_t IntlTest::getErrors( void )
     return errorCount;
 }
 
-int32_t IntlTest::getDataErrors( void )
-{
-    return dataErrorCount;
-}
-
 UBool IntlTest::runTest( char* name, char* par )
 {
     UBool rval;
@@ -595,10 +590,10 @@ UBool IntlTest::runTest( char* name, char* par )
     if (name)
         pos = strchr( name, delim ); // check if name contains path (by looking for '/')
     if (pos) {
-        testPath = pos+1;   // store subpath for calling subtest
+        path = pos+1;   // store subpath for calling subtest
         *pos = 0;       // split into two strings
     }else{
-        testPath = NULL;
+        path = NULL;
     }
 
     if (!name || (name[0] == 0) || (strcmp(name, "*") == 0)) {
@@ -747,15 +742,7 @@ int32_t IntlTest::IncErrorCount( void )
     return errorCount;
 }
 
-int32_t IntlTest::IncDataErrorCount( void )
-{
-    dataErrorCount++;
-    if (caller) caller->IncDataErrorCount();
-    return dataErrorCount;
-}
-
-void IntlTest::err()
-{
+void IntlTest::err() {
     IncErrorCount();
 }
 
@@ -771,39 +758,6 @@ void IntlTest::errln( const UnicodeString &message )
     if (!no_err_msg) LL_message( message, TRUE );
 }
 
-void IntlTest::dataerr( const UnicodeString &message )
-{
-    IncDataErrorCount();
-
-    if (!warn_on_missing_data) {
-        IncErrorCount();
-    }
-
-    if (!no_err_msg) LL_message( message, FALSE );
-}
-
-void IntlTest::dataerrln( const UnicodeString &message )
-{
-    IncDataErrorCount();
-    UnicodeString msg;
-    if (!warn_on_missing_data) {
-        IncErrorCount();
-        msg = message;
-    } else {
-        msg = UnicodeString("[DATA] " + message);
-    }
-
-    if (!no_err_msg) LL_message( msg + " - (Are you missing data?)", TRUE );
-}
-
-void IntlTest::errcheckln(UErrorCode status, const UnicodeString &message ) {
-    if (status == U_FILE_ACCESS_ERROR || status == U_MISSING_RESOURCE_ERROR) {
-        dataerrln(message);
-    } else {
-        errln(message);
-    }
-}
-
 /* convenience functions that include sprintf formatting */
 void IntlTest::log(const char *fmt, ...)
 {
@@ -880,33 +834,6 @@ void IntlTest::errln(const char *fmt, ...)
     errln(UnicodeString(buffer, ""));
 }
 
-void IntlTest::dataerrln(const char *fmt, ...)
-{
-    char buffer[4000];
-    va_list ap;
-
-    va_start(ap, fmt);
-    vsprintf(buffer, fmt, ap);
-    va_end(ap);
-    dataerrln(UnicodeString(buffer, ""));
-}
-
-void IntlTest::errcheckln(UErrorCode status, const char *fmt, ...)
-{
-    char buffer[4000];
-    va_list ap;
-
-    va_start(ap, fmt);
-    vsprintf(buffer, fmt, ap);
-    va_end(ap);
-    
-    if (status == U_FILE_ACCESS_ERROR || status == U_MISSING_RESOURCE_ERROR) {
-        dataerrln(UnicodeString(buffer, ""));
-    } else {
-        errln(UnicodeString(buffer, ""));
-    }
-}
-
 void IntlTest::printErrors()
 {
      IntlTest::LL_message(errorList, TRUE);
@@ -946,7 +873,6 @@ void IntlTest::LL_message( UnicodeString message, UBool newline )
     // stream out the message
     length = message.extract(0, message.length(), buffer, sizeof(buffer));
     if (length > 0) {
-        length = length > 10000 ? 10000 : length;
         fwrite(buffer, sizeof(*buffer), length, (FILE *)testoutfp);
     }
 
@@ -1011,18 +937,27 @@ main(int argc, char* argv[])
     UBool name = FALSE;
     UBool leaks = FALSE;
     UBool warnOnMissingData = FALSE;
-    UBool defaultDataFound = FALSE;
     UErrorCode errorCode = U_ZERO_ERROR;
     UConverter *cnv = NULL;
     const char *warnOrErr = "Failure";
-    UDate startTime, endTime;
-    int32_t diffTime;
-    const char *props[IntlTest::kMaxProps];
-    int32_t nProps = 0;
 
-    U_MAIN_INIT_ARGS(argc, argv);
+#ifdef XP_MAC_CONSOLE
+    argc = ccommand( &argv );
+#endif
+
+    /* Initialize ICU */
+	IntlTest::setICU_DATA();   // Must set data directory before u_init() is called.
+    u_init(&errorCode);
+    if (U_FAILURE(errorCode)) {
+        fprintf(stderr,
+                "#### %s: u_init() failed, error is \"%s\".\n"
+				"#### Most commonly indicates that the ICU data is not accesible.\n"
+                "#### Check setting of ICU_DATA, or check that ICU data library is available\n"
+				"#### ICU_DATA is currently set to \"%s\"\n", argv[0], u_errorName(errorCode), u_getDataDirectory());
+		u_cleanup();
+	    return 1;
+    }
 
-    startTime = uprv_getUTCtime();
 
     for (int i = 1; i < argc; ++i) {
         if (argv[i][0] == '-') {
@@ -1046,12 +981,6 @@ main(int argc, char* argv[])
               warnOnMissingData = TRUE;
               warnOrErr = "WARNING";
             }
-            else if (strncmp("prop:", str, 5) == 0) {
-                if (nProps < IntlTest::kMaxProps) {
-                    props[nProps] = str + 5;
-                }
-                nProps++;
-            }
             else {
                 syntax = TRUE;
             }
@@ -1071,7 +1000,7 @@ main(int argc, char* argv[])
                 "### Syntax:\n"
                 "### IntlTest [-option1 -option2 ...] [testname1 testname2 ...] \n"
                 "### where options are: verbose (v), all (a), noerrormsg (n), \n"
-                "### exhaustive (e), leaks (l), prop:<propery>=<value>, \n"
+                "### exhaustive (e), leaks (l)"
                 "### (Specify either -all (shortcut -a) or a test name). \n"
                 "### -all will run all of the tests.\n"
                 "### \n"
@@ -1087,85 +1016,37 @@ main(int argc, char* argv[])
         return 1;
     }
 
-    if (nProps > IntlTest::kMaxProps) {
-        fprintf(stdout, "### Too many properties.  Exiting.\n");
-    }
-
     UBool all_tests_exist = TRUE;
     MajorTestLevel major;
     major.setVerbose( verbose );
     major.setNoErrMsg( no_err_msg );
     major.setQuick( quick );
     major.setLeaks( leaks );
-    major.setWarnOnMissingData( warnOnMissingData );
-    for (int32_t i = 0; i < nProps; i++) {
-        major.setProperty(props[i]);
-    }
     fprintf(stdout, "-----------------------------------------------\n");
     fprintf(stdout, " IntlTest (C++) Test Suite for                 \n");
     fprintf(stdout, "   International Components for Unicode %s\n", U_ICU_VERSION);
-    {
-	const char *charsetFamily = "Unknown";
-        int32_t voidSize = (int32_t)sizeof(void*);
-        int32_t bits = voidSize * 8;
-        if(U_CHARSET_FAMILY==U_ASCII_FAMILY) {
-           charsetFamily="ASCII";
-        } else if(U_CHARSET_FAMILY==U_EBCDIC_FAMILY) {
-           charsetFamily="EBCDIC";
-        }
-        fprintf(stdout, 
-                    "   Bits: %d, Byte order: %s, Chars: %s\n",
-                     bits, U_IS_BIG_ENDIAN?"Big endian":"Little endian",
-                     charsetFamily);
-    }
     fprintf(stdout, "-----------------------------------------------\n");
     fprintf(stdout, " Options:                                       \n");
-    fprintf(stdout, "   all (a)                  : %s\n", (all?               "On" : "Off"));
-    fprintf(stdout, "   Verbose (v)              : %s\n", (verbose?           "On" : "Off"));
-    fprintf(stdout, "   No error messages (n)    : %s\n", (no_err_msg?        "On" : "Off"));
-    fprintf(stdout, "   Exhaustive (e)           : %s\n", (!quick?            "On" : "Off"));
-    fprintf(stdout, "   Leaks (l)                : %s\n", (leaks?             "On" : "Off"));
-    fprintf(stdout, "   Warn on missing data (w) : %s\n", (warnOnMissingData? "On" : "Off"));
-    for (int32_t i = 0; i < nProps; i++) {
-        fprintf(stdout, "   Custom property (prop:)  : %s\n", props[i]);
-    }
+    fprintf(stdout, "   all (a)               : %s\n", (all?        "On" : "Off"));
+    fprintf(stdout, "   Verbose (v)           : %s\n", (verbose?    "On" : "Off"));
+    fprintf(stdout, "   No error messages (n) : %s\n", (no_err_msg? "On" : "Off"));
+    fprintf(stdout, "   Exhaustive (e)        : %s\n", (!quick?     "On" : "Off"));
+    fprintf(stdout, "   Leaks (l)             : %s\n", (leaks?      "On" : "Off"));
     fprintf(stdout, "-----------------------------------------------\n");
 
-    /* Check whether ICU will initialize without forcing the build data directory into
-     *  the ICU_DATA path.  Success here means either the data dll contains data, or that
-     *  this test program was run with ICU_DATA set externally.  Failure of this check
-     *  is normal when ICU data is not packaged into a shared library.
-     *
-     *  Whether or not this test succeeds, we want to cleanup and reinitialize
-     *  with a data path so that data loading from individual files can be tested.
-     */
-    u_init(&errorCode);
-    if (U_FAILURE(errorCode)) {
-        fprintf(stderr,
-            "#### Note:  ICU Init without build-specific setDataDirectory() failed.\n");
-        defaultDataFound = FALSE;
-    }
-    else {
-        defaultDataFound = TRUE;
-    }
-    u_cleanup();
+    // Check that u_init() works
     errorCode = U_ZERO_ERROR;
-
-    /* Initialize ICU */
-    if (!defaultDataFound) {
-        IntlTest::setICU_DATA();   // Must set data directory before u_init() is called.
-    }
     u_init(&errorCode);
     if (U_FAILURE(errorCode)) {
-        fprintf(stderr,
-            "#### ERROR! %s: u_init() failed with status = \"%s\".\n"
-            "*** Check the ICU_DATA environment variable and \n"
-            "*** check that the data files are present.\n", argv[0], u_errorName(errorCode));
-            if(warnOnMissingData == 0) {
-                fprintf(stderr, "*** Exiting.  Use the '-w' option if data files were\n*** purposely removed, to continue test anyway.\n");
-                u_cleanup();
-                return 1;
-            }
+        fprintf(stdout,
+            "*** u_init() failed with error code = %s\n"
+                "*** Check the ICU_DATA environment variable and\n"
+                "*** check that the data files are present.\n",
+                u_errorName(errorCode));
+        if(!warnOnMissingData) {
+          fprintf(stdout, "*** Exiting.  Use the '-w' option if data files were\n*** purposely removed, to continue test anyway.\n");
+          return 1;
+        }
     }
 
 
@@ -1216,8 +1097,6 @@ main(int argc, char* argv[])
         }
     }
 
-    Locale originalLocale;  // Save the default locale for comparison later on.
-
     /* TODO: Add option to call u_cleanup and rerun tests. */
     if (all) {
         major.runTest();
@@ -1254,34 +1133,18 @@ main(int argc, char* argv[])
     free(_testDataPath);
     _testDataPath = 0;
 
-    Locale lastDefaultLocale;
-    if (originalLocale != lastDefaultLocale) {
-        major.errln("FAILURE: A test changed the default locale without resetting it.");
-    }
-
     fprintf(stdout, "\n--------------------------------------\n");
     if (major.getErrors() == 0) {
         /* Call it twice to make sure that the defaults were reset. */
         /* Call it before the OK message to verify proper cleanup. */
         u_cleanup();
-        u_cleanup();
+     u_cleanup();
 
         fprintf(stdout, "OK: All tests passed without error.\n");
-
-        if (major.getDataErrors() != 0) {
-            fprintf(stdout, "\t*WARNING* some data-loading errors were ignored by the -w option.\n");
-        }
     }else{
         fprintf(stdout, "Errors in total: %ld.\n", (long)major.getErrors());
         major.printErrors();
 
-
-        if (major.getDataErrors() != 0) {
-            fprintf(stdout, "\t*Note* some errors are data-loading related. If the data used is not the \n"
-                    "\tstock ICU data (i.e some have been added or removed), consider using\n"
-                    "\tthe '-w' option to turn these errors into warnings.\n");
-        }
-
         /* Call afterwards to display errors. */
         u_cleanup();
     }
@@ -1291,13 +1154,6 @@ main(int argc, char* argv[])
     if (execCount <= 0) {
         fprintf(stdout, "***** Not all called tests actually exist! *****\n");
     }
-    endTime = uprv_getUTCtime();
-    diffTime = (int32_t)(endTime - startTime);
-    printf("Elapsed Time: %02d:%02d:%02d.%03d\n",
-        (int)((diffTime%U_MILLIS_PER_DAY)/U_MILLIS_PER_HOUR),
-        (int)((diffTime%U_MILLIS_PER_HOUR)/U_MILLIS_PER_MINUTE),
-        (int)((diffTime%U_MILLIS_PER_MINUTE)/U_MILLIS_PER_SECOND),
-        (int)(diffTime%U_MILLIS_PER_SECOND));
     return major.getErrors();
 }
 
@@ -1322,7 +1178,7 @@ const char* IntlTest::loadTestData(UErrorCode& err){
         /* u_getDataDirectory shoul return \source\data ... set the
          * directory to ..\source\data\..\test\testdata\out\testdata
          */
-        strcpy(tdpath, directory);
+		strcpy(tdpath, directory);
         strcat(tdpath, tdrelativepath);
         strcat(tdpath,"testdata");
 
@@ -1330,7 +1186,7 @@ const char* IntlTest::loadTestData(UErrorCode& err){
 
         if(U_FAILURE(err)){
             err = U_FILE_ACCESS_ERROR;
-            it_dataerrln((UnicodeString)"Could not load testtypes.res in testdata bundle with path " + tdpath + (UnicodeString)" - " + u_errorName(err));
+            it_errln((UnicodeString)"Could not load testtypes.res in testdata bundle with path " + tdpath + (UnicodeString)" - " + u_errorName(err));
             return "";
         }
         ures_close(test);
@@ -1340,30 +1196,6 @@ const char* IntlTest::loadTestData(UErrorCode& err){
     return _testDataPath;
 }
 
-const char* IntlTest::getTestDataPath(UErrorCode& err) {
-    return loadTestData(err);
-}
-
-/* Returns the path to icu/source/test/testdata/ */
-const char *IntlTest::getSourceTestData(UErrorCode& /*err*/) {
-    const char *srcDataDir = NULL;
-#ifdef U_TOPSRCDIR
-    srcDataDir = U_TOPSRCDIR U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING;
-#else
-    srcDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING;
-    FILE *f = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING"rbbitst.txt", "r");
-    if (f) {
-        /* We're in icu/source/test/intltest/ */
-        fclose(f);
-    }
-    else {
-        /* We're in icu/source/test/intltest/Platform/(Debug|Release) */
-        srcDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING"test"U_FILE_SEP_STRING"testdata"U_FILE_SEP_STRING;
-    }
-#endif
-    return srcDataDir;
-}
-
 const char* IntlTest::fgDataDir = NULL;
 
 /* returns the path to icu/source/data */
@@ -1416,14 +1248,14 @@ const char *  IntlTest::pathToDataDirectory()
         }
         else {
             /* __FILE__ on MSVC7 does not contain the directory */
-            FILE *file = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "Makefile.in", "r");
-            if (file) {
-                fclose(file);
-                fgDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
-            }
-            else {
-                fgDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
-            }
+			FILE *file = fopen(".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING "Makefile.in", "r");
+			if (file) {
+				fclose(file);
+	            fgDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
+			}
+			else {
+	            fgDataDir = ".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING".."U_FILE_SEP_STRING "data" U_FILE_SEP_STRING;
+			}
         }
     }
 #endif
@@ -1443,10 +1275,6 @@ UnicodeString CharsToUnicodeString(const char* chars)
     return str.unescape();
 }
 
-UnicodeString ctou(const char* chars) {
-    return CharsToUnicodeString(chars);
-}
-
 #define RAND_M  (714025)
 #define RAND_IA (1366)
 #define RAND_IC (150889)
@@ -1491,197 +1319,6 @@ float IntlTest::random() {
     return random(&RAND_SEED);
 }
 
-static inline UChar toHex(int32_t i) {
-    return (UChar)(i + (i < 10 ? 0x30 : (0x41 - 10)));
-}
-
-static UnicodeString& escape(const UnicodeString& s, UnicodeString& result) {
-    for (int32_t i=0; i<s.length(); ++i) {
-        UChar c = s[i];
-        if (c <= (UChar)0x7F) {
-            result += c;
-        } else {
-            result += (UChar)0x5c;
-            result += (UChar)0x75;
-            result += toHex((c >> 12) & 0xF);
-            result += toHex((c >>  8) & 0xF);
-            result += toHex((c >>  4) & 0xF);
-            result += toHex( c        & 0xF);
-        }
-    }
-    return result;
-}
-
-#define VERBOSE_ASSERTIONS
-
-UBool IntlTest::assertTrue(const char* message, UBool condition, UBool quiet, UBool possibleDataError) {
-    if (!condition) {
-        if (possibleDataError) {
-            dataerrln("FAIL: assertTrue() failed: %s", message);
-        } else {
-            errln("FAIL: assertTrue() failed: %s", message);
-        }
-    } else if (!quiet) {
-        logln("Ok: %s", message);
-    }
-    return condition;
-}
-
-UBool IntlTest::assertFalse(const char* message, UBool condition, UBool quiet) {
-    if (condition) {
-        errln("FAIL: assertFalse() failed: %s", message);
-    } else if (!quiet) {
-        logln("Ok: %s", message);
-    }
-    return !condition;
-}
-
-UBool IntlTest::assertSuccess(const char* message, UErrorCode ec, UBool possibleDataError) {
-    if (U_FAILURE(ec)) {
-        if (possibleDataError) {
-            dataerrln("FAIL: %s (%s)", message, u_errorName(ec));
-        } else {
-            errcheckln(ec, "FAIL: %s (%s)", message, u_errorName(ec));
-        }
-        
-        return FALSE;
-    }
-    return TRUE;
-}
-
-UBool IntlTest::assertEquals(const char* message,
-                             const UnicodeString& expected,
-                             const UnicodeString& actual,
-                             UBool possibleDataError) {
-    if (expected != actual) {
-        if (possibleDataError) {
-            dataerrln((UnicodeString)"FAIL: " + message + "; got " +
-                  prettify(actual) +
-                  "; expected " + prettify(expected));
-        } else {
-            errln((UnicodeString)"FAIL: " + message + "; got " +
-                  prettify(actual) +
-                  "; expected " + prettify(expected));
-        }
-        return FALSE;
-    }
-#ifdef VERBOSE_ASSERTIONS
-    else {
-        logln((UnicodeString)"Ok: " + message + "; got " + prettify(actual));
-    }
-#endif
-    return TRUE;
-}
-
-UBool IntlTest::assertEquals(const char* message,
-                             const char* expected,
-                             const char* actual) {
-    if (uprv_strcmp(expected, actual) != 0) {
-        errln((UnicodeString)"FAIL: " + message + "; got \"" +
-              actual +
-              "\"; expected \"" + expected + "\"");
-        return FALSE;
-    }
-#ifdef VERBOSE_ASSERTIONS
-    else {
-        logln((UnicodeString)"Ok: " + message + "; got \"" + actual + "\"");
-    }
-#endif
-    return TRUE;
-}
-
-#if !UCONFIG_NO_FORMATTING
-UBool IntlTest::assertEquals(const char* message,
-                             const Formattable& expected,
-                             const Formattable& actual) {
-    if (expected != actual) {
-        errln((UnicodeString)"FAIL: " + message + "; got " +
-              toString(actual) +
-              "; expected " + toString(expected));
-        return FALSE;
-    }
-#ifdef VERBOSE_ASSERTIONS
-    else {
-        logln((UnicodeString)"Ok: " + message + "; got " + toString(actual));
-    }
-#endif
-    return TRUE;
-}
-#endif
-
-static char ASSERT_BUF[256];
-
-static const char* extractToAssertBuf(const UnicodeString& message) {
-    UnicodeString buf;
-    escape(message, buf);
-    buf.extract(0, 0x7FFFFFFF, ASSERT_BUF, sizeof(ASSERT_BUF)-1, 0);
-    ASSERT_BUF[sizeof(ASSERT_BUF)-1] = 0;
-    return ASSERT_BUF;
-}
-
-UBool IntlTest::assertTrue(const UnicodeString& message, UBool condition, UBool quiet) {
-    return assertTrue(extractToAssertBuf(message), condition, quiet);
-}
-
-UBool IntlTest::assertFalse(const UnicodeString& message, UBool condition, UBool quiet) {
-    return assertFalse(extractToAssertBuf(message), condition, quiet);
-}
-
-UBool IntlTest::assertSuccess(const UnicodeString& message, UErrorCode ec) {
-    return assertSuccess(extractToAssertBuf(message), ec);
-}
-
-UBool IntlTest::assertEquals(const UnicodeString& message,
-                             const UnicodeString& expected,
-                             const UnicodeString& actual) {
-    return assertEquals(extractToAssertBuf(message), expected, actual);
-}
-
-UBool IntlTest::assertEquals(const UnicodeString& message,
-                             const char* expected,
-                             const char* actual) {
-    return assertEquals(extractToAssertBuf(message), expected, actual);
-}
-//--------------------------------------------------------------------
-// Time bomb - allows temporary behavior that expires at a given
-//             release
-//--------------------------------------------------------------------
-
-UBool IntlTest::isICUVersionAtLeast(const UVersionInfo x) {
-    UVersionInfo v;
-    u_getVersion(v);
-    return (uprv_memcmp(v, x, U_MAX_VERSION_LENGTH) >= 0);
-}
-
-#if !UCONFIG_NO_FORMATTING
-UBool IntlTest::assertEquals(const UnicodeString& message,
-                             const Formattable& expected,
-                             const Formattable& actual) {
-    return assertEquals(extractToAssertBuf(message), expected, actual);
-}
-#endif
-
-void IntlTest::setProperty(const char* propline) {
-    if (numProps < kMaxProps) {
-        proplines[numProps] = propline;
-    }
-    numProps++;
-}
-
-const char* IntlTest::getProperty(const char* prop) {
-    const char* val = NULL;
-    for (int32_t i = 0; i < numProps; i++) {
-        int32_t plen = uprv_strlen(prop);
-        if ((int32_t)uprv_strlen(proplines[i]) > plen + 1
-                && proplines[i][plen] == '='
-                && uprv_strncmp(proplines[i], prop, plen) == 0) {
-            val = &(proplines[i][plen+1]);
-            break;
-        }
-    }
-    return val;
-}
-
 /*
  * Hey, Emacs, please set the following:
  *
