@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2010, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -13,7 +13,6 @@
 
 // The following includes utypes.h, uobject.h and unistr.h
 #include "unicode/fmtable.h"
-#include "unicode/testlog.h"
 
 U_NAMESPACE_USE
 
@@ -41,7 +40,6 @@ UnicodeString operator+(const UnicodeString& left, unsigned int num);
 UnicodeString operator+(const UnicodeString& left, float num);
 #if !UCONFIG_NO_FORMATTING
 UnicodeString toString(const Formattable& f); // liu
-UnicodeString toString(int32_t n);
 #endif
 //-----------------------------------------------------------------------------
 
@@ -62,58 +60,24 @@ UnicodeString toString(int32_t n);
         name = #test;                 \
         if (exec) {                   \
             logln(#test "---");       \
-            logln();                  \
+            logln((UnicodeString)""); \
             test();                   \
         }                             \
         break
 
-// More convenient macros. These allow easy reordering of the test cases.
-//
-//| void MyTest::runIndexedTest(int32_t index, UBool exec,
-//|                             const char* &name, char* /*par*/) {
-//|     TESTCASE_AUTO_BEGIN;
-//|     TESTCASE_AUTO(TestSomething);
-//|     TESTCASE_AUTO(TestSomethingElse);
-//|     TESTCASE_AUTO(TestAnotherThing);
-//|     TESTCASE_AUTO_END;
-//| }
-#define TESTCASE_AUTO_BEGIN \
-    for(;;) { \
-        int32_t testCaseAutoNumber = 0
-
-#define TESTCASE_AUTO(test) \
-        if (index == testCaseAutoNumber++) { \
-            name = #test; \
-            if (exec) { \
-                logln(#test "---"); \
-                logln(); \
-                test(); \
-            } \
-            break; \
-        }
-
-#define TESTCASE_AUTO_END \
-        name = ""; \
-        break; \
-    }
-
-class IntlTest : public TestLog {
+class IntlTest {
 public:
 
     IntlTest();
-    // TestLog has a virtual destructor.
 
-    virtual UBool runTest( char* name = NULL, char* par = NULL, char *baseName = NULL); // not to be overidden
+    virtual UBool runTest( char* name = NULL, char* par = NULL ); // not to be overidden
 
     virtual UBool setVerbose( UBool verbose = TRUE );
     virtual UBool setNoErrMsg( UBool no_err_msg = TRUE );
     virtual UBool setQuick( UBool quick = TRUE );
     virtual UBool setLeaks( UBool leaks = TRUE );
-    virtual UBool setWarnOnMissingData( UBool warn_on_missing_data = TRUE );
-    virtual int32_t setThreadCount( int32_t count = 1);
 
     virtual int32_t getErrors( void );
-    virtual int32_t getDataErrors (void );
 
     virtual void setCaller( IntlTest* callingTest ); // for internal use only
     virtual void setPath( char* path ); // for internal use only
@@ -136,12 +100,6 @@ public:
 
     virtual void errln( const UnicodeString &message );
 
-    virtual void dataerr( const UnicodeString &message );
-
-    virtual void dataerrln( const UnicodeString &message );
-    
-    void errcheckln(UErrorCode status, const UnicodeString &message );
-
     // convenience functions: sprintf() + errln() etc.
     void log(const char *fmt, ...);
     void logln(const char *fmt, ...);
@@ -149,9 +107,6 @@ public:
     void infoln(const char *fmt, ...);
     void err(const char *fmt, ...);
     void errln(const char *fmt, ...);
-    void dataerr(const char *fmt, ...);
-    void dataerrln(const char *fmt, ...);
-    void errcheckln(UErrorCode status, const char *fmt, ...);
 
     // Print ALL named errors encountered so far
     void printErrors(); 
@@ -174,75 +129,32 @@ public:
      */
     static float random();
 
-    /**
-     * Ascertain the version of ICU. Useful for 
-     * time bomb testing
-     */
-    UBool isICUVersionAtLeast(const UVersionInfo x);
-
-    enum { kMaxProps = 16 };
-
-    virtual void setProperty(const char* propline);
-    virtual const char* getProperty(const char* prop);
-
 protected:
-    /* JUnit-like assertions. Each returns TRUE if it succeeds. */
-    UBool assertTrue(const char* message, UBool condition, UBool quiet=FALSE, UBool possibleDataError=FALSE);
-    UBool assertFalse(const char* message, UBool condition, UBool quiet=FALSE);
-    UBool assertSuccess(const char* message, UErrorCode ec, UBool possibleDataError=FALSE);
-    UBool assertEquals(const char* message, const UnicodeString& expected,
-                       const UnicodeString& actual, UBool possibleDataError=FALSE);
-    UBool assertEquals(const char* message, const char* expected,
-                       const char* actual);
-#if !UCONFIG_NO_FORMATTING
-    UBool assertEquals(const char* message, const Formattable& expected,
-                       const Formattable& actual);
-    UBool assertEquals(const UnicodeString& message, const Formattable& expected,
-                       const Formattable& actual);
-#endif
-    UBool assertTrue(const UnicodeString& message, UBool condition, UBool quiet=FALSE);
-    UBool assertFalse(const UnicodeString& message, UBool condition, UBool quiet=FALSE);
-    UBool assertSuccess(const UnicodeString& message, UErrorCode ec);
-    UBool assertEquals(const UnicodeString& message, const UnicodeString& expected,
-                       const UnicodeString& actual);
-    UBool assertEquals(const UnicodeString& message, const char* expected,
-                       const char* actual);
-
     virtual void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL ); // overide !
 
-    virtual UBool runTestLoop( char* testname, char* par, char *baseName );
+    virtual UBool runTestLoop( char* testname, char* par );
 
     virtual int32_t IncErrorCount( void );
 
-    virtual int32_t IncDataErrorCount( void );
-
     virtual UBool callTest( IntlTest& testToBeCalled, char* par );
 
 
-    UBool       verbose;
-    UBool       no_err_msg;
-    UBool       quick;
-    UBool       leaks;
-    UBool       warn_on_missing_data;
-    int32_t     threadCount;
+    UBool      verbose;
+    UBool      no_err_msg;
+    UBool      quick;
+    UBool      leaks;
 
 private:
-    UBool       LL_linestart;
+    UBool      LL_linestart;
     int32_t     LL_indentlevel;
 
     int32_t     errorCount;
-    int32_t     dataErrorCount;
     IntlTest*   caller;
-    char*       testPath;           // specifies subtests
-    
-    char basePath[1024];
+    char*       path;           // specifies subtests
 
     //FILE *testoutfp;
     void *testoutfp;
 
-    const char* proplines[kMaxProps];
-    int32_t     numProps;
-
 protected:
 
     virtual void LL_message( UnicodeString message, UBool newline );
@@ -253,6 +165,10 @@ protected:
     static UnicodeString prettify(const UnicodeString &source, UBool parseBackslash=FALSE);
     static UnicodeString &appendHex(uint32_t number, int32_t digits, UnicodeString &target);
 
+    /* complete a relative path to a full pathname, and convert to platform-specific syntax. */
+    /* The character seperating directories for the relative path is '|'.                    */
+    static void pathnameInContext( char* fullname, int32_t maxsize, const char* relpath );
+
 public:
     static void setICU_DATA();       // Set up ICU_DATA if necessary.
 
@@ -261,8 +177,6 @@ public:
 public:
     UBool run_phase2( char* name, char* par ); // internally, supports reporting memory leaks
     static const char* loadTestData(UErrorCode& err);
-    virtual const char* getTestDataPath(UErrorCode& err);
-    static const char* getSourceTestData(UErrorCode& err);
 
 // static members
 public:
@@ -280,8 +194,6 @@ void it_infoln( void );
 void it_err(void);
 void it_err( UnicodeString message );
 void it_errln( UnicodeString message );
-void it_dataerr( UnicodeString message );
-void it_dataerrln( UnicodeString message );
 
 /**
  * This is a variant of cintltst/ccolltst.c:CharsToUChars().
@@ -290,7 +202,4 @@ void it_dataerrln( UnicodeString message );
  */
 extern UnicodeString CharsToUnicodeString(const char* chars);
 
-/* alias for CharsToUnicodeString */
-extern UnicodeString ctou(const char* chars);
-
 #endif // _INTLTEST
