@@ -1,6 +1,6 @@
 /********************************************************************
- * COPYRIGHT: 
- * Copyright (c) 1998-2001, International Business Machines Corporation and
+ * COPYRIGHT:
+ * Copyright (c) 1998-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -19,16 +19,18 @@
 
 #include "itutil.h"
 #include "tscoll.h"
-#include "ittxtbd.h"
 #include "itformat.h"
-#include "itconv.h"
 #include "ittrans.h"
 #include "itrbbi.h"
 #include "itrbnf.h"
 #include "itrbnfrt.h"
 #include "normconf.h"
+#include "regextst.h"
 #include "tstnorm.h"
 #include "canittst.h"
+#include "icusvtst.h"
+#include "testidna.h"
+#include "convtest.h"
 
 #define CASE_SUITE(id, suite) case id:                  \
                           name = #suite;                \
@@ -42,83 +44,119 @@
 void MajorTestLevel::runIndexedTest( int32_t index, UBool exec, const char* &name, char* par )
 {
     switch (index) {
-        case 0: name = "utility"; 
-                if (exec) { 
+        case 0: name = "utility";
+                if (exec) {
                     logln("TestSuite Utilities---"); logln();
                     IntlTestUtilities test;
                     callTest( test, par );
                 }
                 break;
 
-        case 1: name = "convert"; 
-                if (exec) {
-                    logln("TestSuite Convert---"); logln();
-                    IntlTestConvert test;
-                    callTest( test, par );
-                }
-                break;
-
-        case 2: name = "normalize"; 
+        case 1: name = "normalize";
+#if !UCONFIG_NO_NORMALIZATION
                 if (exec) {
                     logln("TestSuite Normalize---"); logln();
                     IntlTestNormalize test;
                     callTest( test, par );
                 }
+#endif
                 break;
 
-        case 3: name = "collate"; 
+        case 2: name = "collate";
+#if !UCONFIG_NO_COLLATION
                 if (exec) {
                     logln("TestSuite Collator---"); logln();
                     IntlTestCollator test;
                     callTest( test, par );
                 }
+#endif
                 break;
 
-        case 4: name = "textbounds"; 
+        case 3: name = "regex";
+#if !UCONFIG_NO_REGULAR_EXPRESSIONS
                 if (exec) {
-                    logln("TestSuite TextBoundary---"); logln();
-                    IntlTestTextBoundary test;
+                    logln("TestSuite Regex---"); logln();
+                    RegexTest test;
                     callTest( test, par );
                 }
+#endif
                 break;
 
-        case 5: name = "format"; 
+        case 4: name = "format";
+#if !UCONFIG_NO_FORMATTING
                 if (exec) {
                     logln("TestSuite Format---"); logln();
                     IntlTestFormat test;
                     callTest( test, par );
                 }
+#endif
                 break;
 
-        case 6: name = "translit"; 
+        case 5: name = "translit";
+#if !UCONFIG_NO_TRANSLITERATION
                 if (exec) {
                     logln("TestSuite Transliterator---"); logln();
                     IntlTestTransliterator test;
                     callTest( test, par );
                 }
+#endif
                 break;
 
-        case 7: name = "rbbi"; 
+        case 6: name = "rbbi";
+#if !UCONFIG_NO_BREAK_ITERATION
                 if (exec) {
                     logln("TestSuite RuleBasedBreakIterator---"); logln();
                     IntlTestRBBI test;
                     callTest( test, par );
                 }
+#endif
                 break;
-        case 8: name = "rbnf";
-            if (exec) {
+        case 7: name = "rbnf";
+#if !UCONFIG_NO_FORMATTING
+                if (exec) {
                     logln("TestSuite RuleBasedNumberFormat----"); logln();
                     IntlTestRBNF test;
                     callTest(test, par);
-            }
-            break;
-        case 9: name = "rbnfrt";
-            if (exec) {
+                }
+#endif
+                break;
+        case 8: name = "rbnfrt";
+#if !UCONFIG_NO_FORMATTING
+                if (exec) {
                     logln("TestSuite RuleBasedNumberFormat RT----"); logln();
                     RbnfRoundTripTest test;
                     callTest(test, par);
+                }
+#endif
+                break;
+
+        case 9: name = "icuserv";
+#if !UCONFIG_NO_SERVICE
+                if (exec) {
+                    logln("TestSuite ICUService---"); logln();
+                    ICUServiceTest test;
+                    callTest(test, par);
+                }
+#endif
+                break;
+        case 10: name = "idna";
+#if !UCONFIG_NO_IDNA  && !UCONFIG_NO_TRANSLITERATION
+            if(exec){
+                logln("TestSuite IDNA----"); logln();
+                TestIDNA test;
+                callTest(test,par);
             }
+#endif
             break;
+        case 11: name = "convert";
+#if !UCONFIG_NO_LEGACY_CONVERSION
+                if (exec) {
+                    logln("TestSuite Conversion---"); logln();
+                    ConversionTest test;
+                    callTest( test, par );
+                }
+#endif
+                break;
 
         default: name = ""; break;
     }
@@ -127,6 +165,9 @@ void MajorTestLevel::runIndexedTest( int32_t index, UBool exec, const char* &nam
 void IntlTestNormalize::runIndexedTest( int32_t index, UBool exec, const char* &name, char* par )
 {
     if(exec) logln("TestSuite Normalize:");
+#if UCONFIG_NO_NORMALIZATION
+    name="";
+#else
     switch (index) {
         CASE_SUITE(0, BasicNormalizerTest);
         CASE_SUITE(1, NormalizerConformanceTest); // this takes a long time
@@ -135,4 +176,5 @@ void IntlTestNormalize::runIndexedTest( int32_t index, UBool exec, const char* &
             name="";
             break;
     }
+#endif
 }
