@@ -1,11 +1,13 @@
 /********************************************************************
- * Copyright (c) 1999-2007, International Business Machines
- * Corporation and others. All Rights Reserved.
- ********************************************************************
- *   Date        Name        Description
- *   12/14/99    Madhu        Creation.
- *   01/12/2000  Madhu        updated for changed API
+ * COPYRIGHT: 
+ * Copyright (c) 1999-2003, International Business Machines Corporation and
+ * others. All Rights Reserved.
  ********************************************************************/
+/************************************************************************
+*   Date        Name        Description
+*   12/14/99    Madhu        Creation.
+*   01/12/2000  Madhu        updated for changed API
+************************************************************************/
 
 #include "unicode/utypes.h"
 
@@ -18,20 +20,13 @@
 #include "rbbiapts.h"
 #include "rbbidata.h"
 #include "cstring.h"
-#include "ubrkimpl.h"
 #include "unicode/ustring.h"
-#include "unicode/utext.h"
 
 /**
  * API Test the RuleBasedBreakIterator class
  */
 
 
-#define TEST_ASSERT_SUCCESS(status) {if (U_FAILURE(status)) {\
-errln("Failure at file %s, line %d, error = %s", __FILE__, __LINE__, u_errorName(status));}}
-
-#define TEST_ASSERT(expr) {if ((expr)==FALSE) { \
-errln("Test Failure at file %s, line %d", __FILE__, __LINE__);}}
 
 void RBBIAPITest::TestCloneEquals()
 {
@@ -68,14 +63,14 @@ void RBBIAPITest::TestCloneEquals()
         errln((UnicodeString)"ERROR:2 RBBI's == and != operator  failed.");
 
 
-    // Quick test of RulesBasedBreakIterator assignment -
+    // Quick test of RulesBasedBreakIterator assignment - 
     // Check that
     //    two different iterators are !=
     //    they are == after assignment
     //    source and dest iterator produce the same next() after assignment.
     //    deleting one doesn't disable the other.
     logln("Testing assignment");
-    RuleBasedBreakIterator *bix = (RuleBasedBreakIterator *)BreakIterator::createLineInstance(Locale::getDefault(), status);
+    RuleBasedBreakIterator *bix = (RuleBasedBreakIterator *)BreakIterator::createLineInstance(Locale::getEnglish(), status);
     if(U_FAILURE(status)){
         errln((UnicodeString)"FAIL : in construction");
         return;
@@ -122,16 +117,16 @@ void RBBIAPITest::TestCloneEquals()
     RuleBasedBreakIterator* bi1clone=(RuleBasedBreakIterator*)bi1->clone();
     RuleBasedBreakIterator* bi2clone=(RuleBasedBreakIterator*)bi2->clone();
 
-    if(*bi1clone != *bi1 || *bi1clone  != *biequal  ||
+    if(*bi1clone != *bi1 || *bi1clone  != *biequal  ||  
       *bi1clone == *bi3 || *bi1clone == *bi2)
         errln((UnicodeString)"ERROR:1 RBBI's clone() method failed");
 
-    if(*bi2clone == *bi1 || *bi2clone == *biequal ||
+    if(*bi2clone == *bi1 || *bi2clone == *biequal ||  
        *bi2clone == *bi3 || *bi2clone != *bi2)
         errln((UnicodeString)"ERROR:2 RBBI's clone() method failed");
 
     if(bi1->getText() != bi1clone->getText()   ||
-       bi2clone->getText() != bi2->getText()   ||
+       bi2clone->getText() != bi2->getText()   || 
        *bi2clone == *bi1clone )
         errln((UnicodeString)"ERROR: RBBI's clone() method failed");
 
@@ -146,22 +141,14 @@ void RBBIAPITest::TestCloneEquals()
 void RBBIAPITest::TestBoilerPlate()
 {
     UErrorCode status = U_ZERO_ERROR;
-    BreakIterator* a = BreakIterator::createWordInstance(Locale("hi"), status);
-    BreakIterator* b = BreakIterator::createWordInstance(Locale("hi_IN"),status);
-    if (U_FAILURE(status)) {
-        errln("Creation of break iterator failed %s", u_errorName(status));
-        return;
-    }
+    BreakIterator* a = BreakIterator::createLineInstance(Locale("hi"), status);
+    BreakIterator* b = BreakIterator::createLineInstance(Locale("hi_IN"),status);
     if(*a!=*b){
         errln("Failed: boilerplate method operator!= does not return correct results");
     }
-    BreakIterator* c = BreakIterator::createWordInstance(Locale("ja"),status);
-    if(a && c){
-        if(*c==*a){
-            errln("Failed: boilerplate method opertator== does not return correct results");
-        }
-    }else{
-        errln("creation of break iterator failed");
+    BreakIterator* c = BreakIterator::createLineInstance(Locale("th"),status);
+    if(*c==*a){
+        errln("Failed: boilerplate method opertator== does not return correct results");
     }
     delete a;
     delete b;
@@ -232,7 +219,7 @@ void RBBIAPITest::TestHashCode()
         errln((UnicodeString)"ERROR: different objects have same hashcodes");
 
     delete bi1clone;
-    delete bi2clone;
+    delete bi2clone; 
     delete bi1;
     delete bi2;
     delete bi3;
@@ -256,12 +243,10 @@ void RBBIAPITest::TestGetSetAdoptText()
     CharacterIterator* text1Clone = text1->clone();
     CharacterIterator* text2= new StringCharacterIterator(str2);
     CharacterIterator* text3= new StringCharacterIterator(str2, 3, 10, 3); //  "ond str"
-
+    
     wordIter1->setText(str1);
-    CharacterIterator *tci = &wordIter1->getText();
-    UnicodeString      tstr;
-    tci->getText(tstr);
-    TEST_ASSERT(tstr == str1);
+    if(wordIter1->getText() != *text1)
+       errln((UnicodeString)"ERROR:1 error in setText or getText ");
     if(wordIter1->current() != 0)
         errln((UnicodeString)"ERROR:1 setText did not set the iteration position to the beginning of the text, it is" + wordIter1->current() + (UnicodeString)"\n");
 
@@ -273,14 +258,9 @@ void RBBIAPITest::TestGetSetAdoptText()
 
 
     charIter1->adoptText(text1Clone);
-    TEST_ASSERT(wordIter1->getText() != charIter1->getText());
-    tci = &wordIter1->getText();
-    tci->getText(tstr);
-    TEST_ASSERT(tstr == str2);
-    tci = &charIter1->getText();
-    tci->getText(tstr);
-    TEST_ASSERT(tstr == str1);
-
+    if( wordIter1->getText() == charIter1->getText() || 
+        wordIter1->getText() != *text2 ||  charIter1->getText() != *text1 )
+        errln((UnicodeString)"ERROR:2 error is getText or setText()");
 
     RuleBasedBreakIterator* rb=(RuleBasedBreakIterator*)wordIter1->clone();
     rb->adoptText(text1);
@@ -291,84 +271,21 @@ void RBBIAPITest::TestGetSetAdoptText()
         errln((UnicodeString)"ERROR:2 error in adoptText ");
 
     // Adopt where iterator range is less than the entire orignal source string.
-    //   (With the change of the break engine to working with UText internally,
-    //    CharacterIterators starting at positions other than zero are not supported)
     rb->adoptText(text3);
-    TEST_ASSERT(rb->preceding(2) == 0);
-    TEST_ASSERT(rb->following(11) == BreakIterator::DONE);
-    //if(rb->preceding(2) != 3) {
-    //    errln((UnicodeString)"ERROR:3 error in adoptText ");
-    //}
-    //if(rb->following(11) != BreakIterator::DONE) {
-    //    errln((UnicodeString)"ERROR:4 error in adoptText ");
-    //}
-
-    // UText API
-    //
-    //   Quick test to see if UText is working at all.
-    //
-    const char *s1 = "\x68\x65\x6C\x6C\x6F\x20\x77\x6F\x72\x6C\x64"; /* "hello world" in UTF-8 */
-    const char *s2 = "\x73\x65\x65\x20\x79\x61"; /* "see ya" in UTF-8 */
-    //                012345678901
-
-    status = U_ZERO_ERROR;
-    UText *ut = utext_openUTF8(NULL, s1, -1, &status);
-    wordIter1->setText(ut, status);
-    TEST_ASSERT_SUCCESS(status);
-
-    int32_t pos;
-    pos = wordIter1->first();
-    TEST_ASSERT(pos==0);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==5);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==6);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==11);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==UBRK_DONE);
-
-    status = U_ZERO_ERROR;
-    UText *ut2 = utext_openUTF8(NULL, s2, -1, &status);
-    TEST_ASSERT_SUCCESS(status);
-    wordIter1->setText(ut2, status);
-    TEST_ASSERT_SUCCESS(status);
-
-    pos = wordIter1->first();
-    TEST_ASSERT(pos==0);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==3);
-    pos = wordIter1->next();
-    TEST_ASSERT(pos==4);
-
-    pos = wordIter1->last();
-    TEST_ASSERT(pos==6);
-    pos = wordIter1->previous();
-    TEST_ASSERT(pos==4);
-    pos = wordIter1->previous();
-    TEST_ASSERT(pos==3);
-    pos = wordIter1->previous();
-    TEST_ASSERT(pos==0);
-    pos = wordIter1->previous();
-    TEST_ASSERT(pos==UBRK_DONE);
-
-    status = U_ZERO_ERROR;
-    UnicodeString sEmpty;
-    UText *gut2 = utext_openUnicodeString(NULL, &sEmpty, &status);
-    wordIter1->getUText(gut2, status);
-    TEST_ASSERT_SUCCESS(status);
-    utext_close(gut2);
-
-    utext_close(ut);
-    utext_close(ut2);
+    if(rb->preceding(2) != 3) {
+        errln((UnicodeString)"ERROR:3 error in adoptText ");
+    }
+    if(rb->following(11) != BreakIterator::DONE) {
+        errln((UnicodeString)"ERROR:4 error in adoptText ");
+    }
 
     delete wordIter1;
     delete charIter1;
     delete rb;
 
- }
-
+ } 
 
+  
 void RBBIAPITest::TestIteration()
 {
     // This test just verifies that the API is present.
@@ -598,7 +515,7 @@ void RBBIAPITest::TestBuilder() {
      int32_t bounds1[] = {0, 3, 6, 7, 8, 11};
      UErrorCode status=U_ZERO_ERROR;
      UParseError    parseError;
-
+     
      RuleBasedBreakIterator *bi = new RuleBasedBreakIterator(rulesString1, parseError, status);
      if(U_FAILURE(status)) {
          errln("FAIL : in construction");
@@ -625,7 +542,7 @@ void RBBIAPITest::TestQuoteGrouping() {
      int32_t bounds1[] = {0, 6, 7, 10, 11, 12};
      UErrorCode status=U_ZERO_ERROR;
      UParseError    parseError;
-
+     
      RuleBasedBreakIterator *bi = new RuleBasedBreakIterator(rulesString1, parseError, status);
      if(U_FAILURE(status)) {
          errln("FAIL : in construction");
@@ -641,7 +558,7 @@ void RBBIAPITest::TestQuoteGrouping() {
 //      Test word break rule status constants.
 //
 void RBBIAPITest::TestRuleStatus() {
-     UChar str[30];
+     UChar str[30]; 
      u_unescape("plain word 123.45 \\u9160\\u9161 \\u30a1\\u30a2 \\u3041\\u3094",
               // 012345678901234567  8      9    0  1      2    3  4      5    6
               //                    Ideographic    Katakana       Hiragana
@@ -659,8 +576,8 @@ void RBBIAPITest::TestRuleStatus() {
                           UBRK_WORD_KANA_LIMIT, UBRK_WORD_NONE_LIMIT,   UBRK_WORD_KANA_LIMIT, UBRK_WORD_KANA_LIMIT};
 
      UErrorCode status=U_ZERO_ERROR;
-
-     RuleBasedBreakIterator *bi = (RuleBasedBreakIterator *)BreakIterator::createWordInstance(Locale::getEnglish(), status);
+     
+     RuleBasedBreakIterator *bi = (RuleBasedBreakIterator *)BreakIterator::createWordInstance(Locale::getDefault(), status);
      if(U_FAILURE(status)) {
          errln("FAIL : in construction");
      } else {
@@ -681,13 +598,6 @@ void RBBIAPITest::TestRuleStatus() {
                  errln("FAIL: incorrect tag value %d at position %d", tag, pos);
                  break;
              }
-
-             // Check that we get the same tag values from getRuleStatusVec()
-             int32_t vec[10];
-             int t = bi->getRuleStatusVec(vec, 10, status);
-             TEST_ASSERT_SUCCESS(status);
-             TEST_ASSERT(t==1);
-             TEST_ASSERT(vec[0] == tag);
          }
      }
      delete bi;
@@ -739,115 +649,6 @@ void RBBIAPITest::TestRuleStatus() {
 }
 
 
-//
-//  TestRuleStatusVec
-//      Test the vector form of  break rule status.
-//
-void RBBIAPITest::TestRuleStatusVec() {
-    UnicodeString rulesString  = "[A-N]{100}; \n"
-                                 "[a-w]{200}; \n"
-                                 "[\\p{L}]{300}; \n"
-                                 "[\\p{N}]{400}; \n"
-                                 "[0-5]{500}; \n"
-                                  "!.*;\n";
-     UnicodeString testString1  = "Aapz5?";
-     int32_t  statusVals[10];
-     int32_t  numStatuses;
-     int32_t  pos;
-
-     UErrorCode status=U_ZERO_ERROR;
-     UParseError    parseError;
-
-     RuleBasedBreakIterator *bi = new RuleBasedBreakIterator(rulesString, parseError, status);
-     TEST_ASSERT_SUCCESS(status);
-     if (U_SUCCESS(status)) {
-         bi->setText(testString1);
-
-         // A
-         pos = bi->next();
-         TEST_ASSERT(pos==1);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 100);
-         TEST_ASSERT(statusVals[1] == 300);
-
-         // a
-         pos = bi->next();
-         TEST_ASSERT(pos==2);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 200);
-         TEST_ASSERT(statusVals[1] == 300);
-
-         // p
-         pos = bi->next();
-         TEST_ASSERT(pos==3);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 200);
-         TEST_ASSERT(statusVals[1] == 300);
-
-         // z
-         pos = bi->next();
-         TEST_ASSERT(pos==4);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 1);
-         TEST_ASSERT(statusVals[0] == 300);
-
-         // 5
-         pos = bi->next();
-         TEST_ASSERT(pos==5);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 400);
-         TEST_ASSERT(statusVals[1] == 500);
-
-         // ?
-         pos = bi->next();
-         TEST_ASSERT(pos==6);
-         numStatuses = bi->getRuleStatusVec(statusVals, 10, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 1);
-         TEST_ASSERT(statusVals[0] == 0);
-
-         //
-         //  Check buffer overflow error handling.   Char == A
-         //
-         bi->first();
-         pos = bi->next();
-         TEST_ASSERT(pos==1);
-         memset(statusVals, -1, sizeof(statusVals));
-         numStatuses = bi->getRuleStatusVec(statusVals, 0, status);
-         TEST_ASSERT(status == U_BUFFER_OVERFLOW_ERROR);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == -1);
-
-         status = U_ZERO_ERROR;
-         memset(statusVals, -1, sizeof(statusVals));
-         numStatuses = bi->getRuleStatusVec(statusVals, 1, status);
-         TEST_ASSERT(status == U_BUFFER_OVERFLOW_ERROR);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 100);
-         TEST_ASSERT(statusVals[1] == -1);
-
-         status = U_ZERO_ERROR;
-         memset(statusVals, -1, sizeof(statusVals));
-         numStatuses = bi->getRuleStatusVec(statusVals, 2, status);
-         TEST_ASSERT_SUCCESS(status);
-         TEST_ASSERT(numStatuses == 2);
-         TEST_ASSERT(statusVals[0] == 100);
-         TEST_ASSERT(statusVals[1] == 300);
-         TEST_ASSERT(statusVals[2] == -1);
-     }
-     delete bi;
-
-}
-
 //
 //   Bug 2190 Regression test.   Builder crash on rule consisting of only a
 //                               $variable reference
@@ -860,7 +661,7 @@ void RBBIAPITest::TestBug2190() {
      int32_t bounds1[] = {0, 4, 8};
      UErrorCode status=U_ZERO_ERROR;
      UParseError    parseError;
-
+     
      RuleBasedBreakIterator *bi = new RuleBasedBreakIterator(rulesString1, parseError, status);
      if(U_FAILURE(status)) {
          errln("FAIL : in construction");
@@ -873,58 +674,48 @@ void RBBIAPITest::TestBug2190() {
 
 
 void RBBIAPITest::TestRegistration() {
-#if !UCONFIG_NO_SERVICE
     UErrorCode status = U_ZERO_ERROR;
-    BreakIterator* ja_word = BreakIterator::createWordInstance("ja_JP", status);
-
+    BreakIterator* thai_word = BreakIterator::createWordInstance("th_TH", status);
+    
     // ok to not delete these if we exit because of error?
-    BreakIterator* ja_char = BreakIterator::createCharacterInstance("ja_JP", status);
+    BreakIterator* thai_char = BreakIterator::createCharacterInstance("th_TH", status);
     BreakIterator* root_word = BreakIterator::createWordInstance("", status);
     BreakIterator* root_char = BreakIterator::createCharacterInstance("", status);
-
-    URegistryKey key = BreakIterator::registerInstance(ja_word, "xx", UBRK_WORD, status);
+    
+    URegistryKey key = BreakIterator::registerInstance(thai_word, "xx", UBRK_WORD, status);
     {
-        if (ja_word && *ja_word == *root_word) {
-            errln("japan not different from root");
+        if (*thai_word == *root_word) {
+            errln("thai not different from root");
         }
     }
-
+    
     {
         BreakIterator* result = BreakIterator::createWordInstance("xx_XX", status);
-        UBool fail = TRUE;
-        if(result){
-            fail = *result != *ja_word;
-        }
+        UBool fail = *result != *thai_word;
         delete result;
         if (fail) {
             errln("bad result for xx_XX/word");
         }
     }
-
+    
     {
-        BreakIterator* result = BreakIterator::createCharacterInstance("ja_JP", status);
-        UBool fail = TRUE;
-        if(result){
-            fail = *result != *ja_char;
-        }
+        BreakIterator* result = BreakIterator::createCharacterInstance("th_TH", status);
+        UBool fail = *result != *thai_char;
         delete result;
         if (fail) {
-            errln("bad result for ja_JP/char");
+            errln("bad result for th_TH/char");
         }
     }
-
+    
     {
         BreakIterator* result = BreakIterator::createCharacterInstance("xx_XX", status);
-        UBool fail = TRUE;
-        if(result){
-            fail = *result != *root_char;
-        }
+        UBool fail = *result != *root_char;
         delete result;
         if (fail) {
             errln("bad result for xx_XX/char");
         }
     }
-
+    
     {
         StringEnumeration* avail = BreakIterator::getAvailableLocales();
         UBool found = FALSE;
@@ -940,28 +731,25 @@ void RBBIAPITest::TestRegistration() {
             errln("did not find test locale");
         }
     }
-
+    
     {
         UBool unreg = BreakIterator::unregister(key, status);
         if (!unreg) {
             errln("unable to unregister");
         }
     }
-
+    
     {
-        BreakIterator* result = BreakIterator::createWordInstance("en_US", status);
+        BreakIterator* result = BreakIterator::createWordInstance("xx", status);
         BreakIterator* root = BreakIterator::createWordInstance("", status);
-        UBool fail = TRUE;
-        if(root){
-          fail = *root != *result;
-        }
+        UBool fail = *root != *result;
         delete root;
         delete result;
         if (fail) {
             errln("did not get root break");
         }
     }
-
+    
     {
         StringEnumeration* avail = BreakIterator::getAvailableLocales();
         UBool found = FALSE;
@@ -977,7 +765,7 @@ void RBBIAPITest::TestRegistration() {
             errln("found test locale");
         }
     }
-
+    
     {
         int32_t count;
         UBool   foundLocale = FALSE;
@@ -992,21 +780,20 @@ void RBBIAPITest::TestRegistration() {
             errln("BreakIterator::getAvailableLocales(&count), failed to find EN.");
         }
     }
-
-
-    // ja_word was adopted by factory
-    delete ja_char;
+    
+    
+    // that_word was adopted by factory
+    delete thai_char;
     delete root_word;
     delete root_char;
-#endif
 }
 
 void RBBIAPITest::RoundtripRule(const char *dataFile) {
     UErrorCode status = U_ZERO_ERROR;
     UParseError parseError;
-    parseError.line = 0;
-    parseError.offset = 0;
-    UDataMemory *data = udata_open(U_ICUDATA_BRKITR, "brk", dataFile, &status);
+	parseError.line = 0;
+	parseError.offset = 0;
+    UDataMemory *data = udata_open(NULL, "brk", dataFile, &status);
     uint32_t length;
     const UChar *builtSource;
     const uint8_t *rbbiRules;
@@ -1042,8 +829,8 @@ void RBBIAPITest::TestRoundtripRules() {
     RoundtripRule("line");
     RoundtripRule("char");
     if (!quick) {
-        RoundtripRule("word_ja");
-        RoundtripRule("word_POSIX");
+        RoundtripRule("word_th");
+        RoundtripRule("line_th");
     }
 }
 
@@ -1061,14 +848,15 @@ void RBBIAPITest::runIndexedTest( int32_t index, UBool exec, const char* &name,
         case  2: name = "TestHashCode"; if (exec) TestHashCode(); break;
         case  3: name = "TestGetSetAdoptText"; if (exec) TestGetSetAdoptText(); break;
         case  4: name = "TestIteration"; if (exec) TestIteration(); break;
-        case  5: name = "TestBuilder"; if (exec) TestBuilder(); break;
-        case  6: name = "TestQuoteGrouping"; if (exec) TestQuoteGrouping(); break;
-        case  7: name = "TestRuleStatus"; if (exec) TestRuleStatus(); break;
-        case  8: name = "TestRuleStatusVec"; if (exec) TestRuleStatusVec(); break;
-        case  9: name = "TestBug2190"; if (exec) TestBug2190(); break;
-        case 10: name = "TestRegistration"; if (exec) TestRegistration(); break;
-        case 11: name = "TestBoilerPlate"; if (exec) TestBoilerPlate(); break;
-        case 12: name = "TestRoundtripRules"; if (exec) TestRoundtripRules(); break;
+        case  5: name = "extra"; break;   // Extra
+        case  6: name = "extra"; break;   // Extra
+        case  7: name = "TestBuilder"; if (exec) TestBuilder(); break;
+        case  8: name = "TestQuoteGrouping"; if (exec) TestQuoteGrouping(); break;
+        case  9: name = "TestRuleStatus"; if (exec) TestRuleStatus(); break;
+        case 10: name = "TestBug2190"; if (exec) TestBug2190(); break;
+        case 11: name = "TestRegistration"; if (exec) TestRegistration(); break;
+        case 12: name = "TestBoilerPlate"; if (exec) TestBoilerPlate(); break;
+        case 13: name = "TestRoundtripRules"; if (exec) TestRoundtripRules(); break;
 
         default: name = ""; break; // needed to end loop
     }
@@ -1104,7 +892,7 @@ void RBBIAPITest::doTest(UnicodeString& testString, int32_t start, int32_t gotof
     if(gotoffset != expectedOffset)
          errln((UnicodeString)"ERROR:****returned #" + gotoffset + (UnicodeString)" instead of #" + expectedOffset);
     if(start <= gotoffset){
-        testString.extractBetween(start, gotoffset, selected);
+        testString.extractBetween(start, gotoffset, selected);  
     }
     else{
         testString.extractBetween(gotoffset, start, selected);
