@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2010, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /*   file name:  strtest.h
@@ -15,7 +15,6 @@
 /*
  * Test character- and string- related settings in utypes.h,
  * macros in putil.h, and constructors in unistr.h .
- * Also basic tests for std_string.h and charstr.h .
  */
 
 #ifndef __STRTEST_H__
@@ -31,22 +30,10 @@ public:
     void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=NULL);
 
 private:
-    void TestEndian();
-    void TestSizeofTypes();
-    void TestCharsetFamily();
-    void Test_U_STRING();
-    void Test_UNICODE_STRING();
-    void Test_UNICODE_STRING_SIMPLE();
-    void Test_UTF8_COUNT_TRAIL_BYTES();
-    void TestStdNamespaceQualifier();
-    void TestUsingStdNamespace();
-    void TestStringPiece();
-    void TestStringPieceComparisons();
-    void TestByteSink();
-    void TestCheckedArrayByteSink();
-    void TestStringByteSink();
-    void TestSTLCompatibility();
-    void TestCharString();
+    void TestEndian(void);
+    void TestSizeofWCharT(void);
+    void TestSizeofInt64(void);
+    void TestCharsetFamily(void);
 };
 
 #endif
