@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 2001, International Business Machines Corporation and
+ * Copyright (c) 1999-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -10,13 +10,19 @@
 # endif
 #endif
 
-#include <unicode/umachine.h>
+#include "unicode/utypes.h"
+#include "unicode/ustring.h"
+#include "umutex.h"
+#include "cmemory.h"
+#include "cstring.h"
+#include "uparse.h"
+
 
 #if !defined(WIN32) && !defined(XP_MAC) && !defined(U_RHAPSODY)
 #define POSIX 1
 #endif
 
-#if defined(POSIX) || defined(U_SOLARIS) || defined(AIX) || defined(HPUX)
+#if defined(POSIX) || defined(U_SOLARIS) || defined(U_AIX) || defined(U_HPUX)
 
 #define HAVE_IMP
 
@@ -60,17 +66,7 @@
 #undef sleep
 #endif
 
-#include "unicode/utypes.h"
 
-/* APP_NO_THREADS is an old symbol. We'll honour it if present. */
-#ifdef APP_NO_THREADS
-# define ICU_USE_THREADS 0
-#endif
-
-/* Default: use threads. */
-#ifndef ICU_USE_THREADS
-# define ICU_USE_THREADS 1
-#endif
 
 #include "tsmthred.h"
 
@@ -117,6 +113,8 @@ void MultithreadTest::runIndexedTest( int32_t index, UBool exec,
 #include "unicode/choicfmt.h"
 #include "unicode/msgfmt.h"
 #include "unicode/locid.h"
+#include "unicode/ucol.h"
+#include "ucaconf.h"
 
 #ifdef WIN32
 #define HAVE_IMP
@@ -129,17 +127,16 @@ void MultithreadTest::runIndexedTest( int32_t index, UBool exec,
 #   define NOIME
 #   define NOMCX
 #include <windows.h>
+#include <process.h>
 
 struct Win32ThreadImplementation
 {
-    HANDLE fHandle;
-    DWORD fThreadID;
+    unsigned long fHandle;
 };
 
-extern "C" unsigned long _stdcall SimpleThreadProc(void *arg)
+extern "C" void __cdecl SimpleThreadProc(void *arg)
 {
     ((SimpleThread*)arg)->run();
-    return 0;
 }
 
 SimpleThread::SimpleThread()
@@ -147,7 +144,6 @@ SimpleThread::SimpleThread()
 {
     Win32ThreadImplementation *imp = new Win32ThreadImplementation;
     imp->fHandle = 0;
-    imp->fThreadID = 0;
 
     fImplementation = imp;
 }
@@ -157,14 +153,25 @@ SimpleThread::~SimpleThread()
     delete (Win32ThreadImplementation*)fImplementation;
 }
 
-void SimpleThread::start()
+int32_t SimpleThread::start()
 {
     Win32ThreadImplementation *imp = (Win32ThreadImplementation*)fImplementation;
+    if(imp->fHandle != NULL) {
+        // The thread appears to have already been started.
+        //   This is probably an error on the part of our caller.
+        return -1;
+    }
 
-    if(imp->fHandle != NULL)
-        return;
-
-    imp->fHandle = CreateThread(NULL,0,SimpleThreadProc,(void*)this,0,&imp->fThreadID);
+    imp->fHandle = _beginthread( SimpleThreadProc, 0 /*stack size*/ , (void *)this );
+    if (imp->fHandle == -1) {
+        // An error occured
+        int err = errno;
+        if (err == 0) {
+            err = -1;
+        }
+        return err;
+    }
+    return 0;
 }
 
 void SimpleThread::sleep(int32_t millis)
@@ -186,9 +193,9 @@ SimpleThread::SimpleThread()
 SimpleThread::~SimpleThread()
 {}
 
-void 
+int32_t 
 SimpleThread::start()
-{}
+{ return 0; }
 
 void 
 SimpleThread::run()
@@ -200,7 +207,7 @@ SimpleThread::sleep(int32_t millis)
 #endif
 
 
-#if defined(POSIX)||defined(U_SOLARIS)||defined(AIX)||defined(HPUX)
+#if defined(POSIX)||defined(U_SOLARIS)||defined(U_AIX)||defined(U_HPUX)
 #define HAVE_IMP
 
 struct PosixThreadImplementation
@@ -225,7 +232,7 @@ SimpleThread::~SimpleThread()
     delete (PosixThreadImplementation*)fImplementation;
 }
 
-void SimpleThread::start()
+int32_t SimpleThread::start()
 {
     PosixThreadImplementation *imp = (PosixThreadImplementation*)fImplementation;
 
@@ -242,6 +249,7 @@ void SimpleThread::start()
     rc = pthread_create(&(imp->fThread),&attr,&SimpleThreadProc,(void*)this);
     pthread_attr_destroy(&attr);
 #endif
+    return rc;
 
 }
 
@@ -253,7 +261,7 @@ void SimpleThread::sleep(int32_t millis)
 
 #ifdef HPUX_CMA
     cma_sleep(millis/100);
-#elif defined(HPUX) || defined(OS390)
+#elif defined(U_HPUX) || defined(OS390)
     millis *= 1000;
     while(millis >= 1000000) {
         usleep(999999);
@@ -297,10 +305,18 @@ void MultithreadTest::runIndexedTest( int32_t index, UBool exec,
         break;
     case 2:
         name = "TestThreadedIntl";
+#if !UCONFIG_NO_FORMATTING
         if (exec)
             TestThreadedIntl();
+#endif
         break;
-
+    case 3:
+      name = "TestCollators";
+#if !UCONFIG_NO_COLLATION
+      if (exec)
+        TestCollators();
+#endif /* #if !UCONFIG_NO_COLLATION */
+      break;
     default:
         name = "";
         break; //needed to end loop
@@ -322,7 +338,10 @@ class TestThreadsThread : public SimpleThread
 {
 public:
     TestThreadsThread(char* whatToChange) { fWhatToChange = whatToChange; }
-    virtual void run() { SimpleThread::sleep(1000); *fWhatToChange = '*'; }
+    virtual void run() { SimpleThread::sleep(1000); 
+                         Mutex m;
+                         *fWhatToChange = '*'; 
+    }
 private:
     char *fWhatToChange;
 };
@@ -343,7 +362,9 @@ void MultithreadTest::TestThreads()
     logln("->" + UnicodeString(threadTestChars) + "<- Firing off threads.. ");
     for(i=0;i<THREADTEST_NRTHREADS;i++)
     {
-        threads[i]->start();
+        if (threads[i]->start() != 0) {
+            errln("Error starting thread %d", i);
+        }
         SimpleThread::sleep(200);
         logln(" Subthread started.");
     }
@@ -355,6 +376,7 @@ void MultithreadTest::TestThreads()
     while(patience--)
     {
         int32_t count = 0;
+        umtx_lock(NULL);
         for(i=0;i<THREADTEST_NRTHREADS;i++)
         {
             if(threadTestChars[i] == '*')
@@ -362,6 +384,7 @@ void MultithreadTest::TestThreads()
                 count++;
             }
         }
+        umtx_unlock(NULL);
         
         if(count == THREADTEST_NRTHREADS)
         {
@@ -449,11 +472,15 @@ void MultithreadTest::TestMutex()
 
     TestMutexThread1  thread1;
     TestMutexThread2  thread2(thread1);
-    thread2.start();
-    thread1.start();
+    if (thread2.start() != 0  || 
+        thread1.start() != 0 ) {
+        errln("Error starting threads.");
+    }
 
     for(int32_t patience = 12; patience > 0;patience--)
     {
+        // TODO:  Possible memory coherence issue in looking at fDone values
+        //        that are set in another thread without the mutex here.
         if(thread1.fDone && verbose)
             printf("Thread1 done\n");
 
@@ -569,12 +596,15 @@ private:
     UnicodeString fErrorString;
 };
 
-// ** FormatThreadTest - a thread that tests performing a number of numberformats.
-
-
 #define kFormatThreadIterations 20  // # of iterations per thread
 #define kFormatThreadThreads    10  // # of threads to spawn
 #define kFormatThreadPatience   60  // time in seconds to wait for all threads
+
+#if !UCONFIG_NO_FORMATTING
+
+// ** FormatThreadTest - a thread that tests performing a number of numberformats.
+
+
 struct FormatThreadTestData
 {
     double number;
@@ -625,6 +655,7 @@ void formatErrorMessage(UErrorCode &realStatus, const UnicodeString& pattern, co
     delete fmt;
 };
 
+static    UMTX ftMutex;
 
 class FormatThreadTest : public ThreadWithStatus
 {
@@ -639,6 +670,7 @@ public:
         fOffset = fgOffset;
     }
 
+
     virtual void run()
     {
         // Keep this data here to avoid static initialization.
@@ -667,25 +699,30 @@ public:
         int32_t iteration;
 
         UErrorCode status = U_ZERO_ERROR;
-        NumberFormat *formatter = NumberFormat::createInstance(Locale::ENGLISH,status);
+        NumberFormat *formatter = NumberFormat::createInstance(Locale::getEnglish(),status);
 
         if(U_FAILURE(status))
         {
+            Mutex m(&ftMutex);
             error("Error on NumberFormat::createInstance()");
             return;
         }
 
-        NumberFormat *percentFormatter = NumberFormat::createPercentInstance(Locale::FRENCH,status);
+        NumberFormat *percentFormatter = NumberFormat::createPercentInstance(Locale::getFrench(),status);
 
         if(U_FAILURE(status))
         {
-            error("Error on NumberFormat::createPercentInstance()");
+            {
+                Mutex m(&ftMutex);
+                error("Error on NumberFormat::createPercentInstance()");
+            }
             delete formatter;
             return;
         }
 
         for(iteration = 0;!getError() && iteration<kFormatThreadIterations;iteration++)
         {
+
             int32_t whichLine = (iteration + fOffset)%kNumberFormatTestDataLength;
 
             UnicodeString  output;
@@ -694,6 +731,7 @@ public:
 
             if(0 != output.compare(kNumberFormatTestData[whichLine].string))
             {
+                Mutex m(&ftMutex);
                 error("format().. expected " + kNumberFormatTestData[whichLine].string + " got " + output);
                 continue; // will break
             }
@@ -706,13 +744,14 @@ public:
 
             if(0 != output.compare(kPercentFormatTestData[whichLine].string))
             {
+                Mutex m(&ftMutex);
                 error("percent format().. \n" + showDifference(kPercentFormatTestData[whichLine].string,output));
                 continue;
             }
 
             // Test message error 
 #define kNumberOfMessageTests 3
-            UErrorCode       statusToCheck;
+            UErrorCode      statusToCheck;
             UnicodeString   patternToCheck;
             Locale          messageLocale;
             Locale          countryToCheck;
@@ -756,12 +795,14 @@ public:
             {
                UnicodeString tmp;
                errorToString(status,tmp);
+               Mutex m(&ftMutex);
                error("Failure on message format, pattern=" + patternToCheck +", error = " + tmp);
                continue;
             }
 
             if(result != expected)
             {
+                Mutex m(&ftMutex);
                 error("PatternFormat: \n" + showDifference(expected,result));
                 continue;
             }
@@ -769,6 +810,7 @@ public:
 
         delete formatter;
         delete percentFormatter;
+        Mutex m(&ftMutex);
         done();
     }
 
@@ -780,14 +822,21 @@ private:
 
 void MultithreadTest::TestThreadedIntl()
 {
+    umtx_init(&ftMutex);
 
     FormatThreadTest tests[kFormatThreadThreads];
  
     logln(UnicodeString("Spawning: ") + kFormatThreadThreads + " threads * " + kFormatThreadIterations + " iterations each.");
-    for(int32_t j = 0; j < kFormatThreadThreads; j++)
-        tests[j].start();
+    for(int32_t j = 0; j < kFormatThreadThreads; j++) {
+        int32_t threadStatus = tests[j].start();
+        if (threadStatus != 0) {
+            errln("System Error %d starting thread number %d.", threadStatus, j);
+            return;
+        }
+    }
 
-    for(int32_t patience = kFormatThreadPatience;patience > 0; patience --)
+    int32_t patience;
+    for(patience = kFormatThreadPatience;patience > 0; patience --)
     {
         logln("Waiting...");
 
@@ -795,13 +844,277 @@ void MultithreadTest::TestThreadedIntl()
         int32_t terrs = 0;
         int32_t completed =0;
 
-        for(i=0;i<kFormatThreadThreads;i++)
+        for(i=0;i<kFormatThreadThreads;i++) {
+            umtx_lock(&ftMutex);
+            UBool threadIsDone = tests[i].getDone();
+            umtx_unlock(&ftMutex);
+            if(threadIsDone)
+            {
+                completed++;
+                
+                logln(UnicodeString("Test #") + i + " is complete.. ");
+                
+                UnicodeString theErr;
+                if(tests[i].getError(theErr))
+                {
+                    terrs++;
+                    errln(UnicodeString("#") + i + ": " + theErr);
+                }
+                // print out the error, too, if any.
+            }
+        }
+        
+        if(completed == kFormatThreadThreads)
         {
-            if(tests[i].getDone())
+            logln("Done!");
+
+            if(terrs)
+            {
+                errln("There were errors.");
+            }
+
+            break;
+        }
+
+        SimpleThread::sleep(900);
+    }
+
+    if (patience <= 0) {
+        errln("patience exceeded. ");
+    }
+    umtx_destroy(&ftMutex);
+    return;
+
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
+#if !UCONFIG_NO_COLLATION
+
+#define kCollatorThreadThreads   10  // # of threads to spawn
+#define kCollatorThreadPatience kCollatorThreadThreads*100
+
+struct Line {
+  UChar buff[25];
+  int32_t buflen;
+} ;
+
+class CollatorThreadTest : public ThreadWithStatus
+{
+private: 
+  const UCollator *coll;
+  const Line *lines;
+  int32_t noLines;
+public:
+  CollatorThreadTest()  : ThreadWithStatus(),
+  coll(NULL),
+  lines(NULL),
+  noLines(0)
+ {
+  };
+  void setCollator(UCollator *c, Line *l, int32_t nl) 
+  {
+    coll = c;
+    lines = l;
+    noLines = nl;
+  }
+  virtual void run() {
+    //sleep(10000);
+  int32_t line = 0;
+
+  uint8_t sk1[1024], sk2[1024];
+  uint8_t *oldSk = NULL, *newSk = sk1;
+  int32_t resLen = 0, oldLen = 0;
+  int32_t i = 0;
+
+  for(i = 0; i < noLines; i++) {
+    resLen = ucol_getSortKey(coll, lines[i].buff, lines[i].buflen, newSk, 1024);
+
+    int32_t res = 0, cmpres = 0, cmpres2 = 0;
+
+    if(oldSk != NULL) {
+      res = strcmp((char *)oldSk, (char *)newSk);
+      cmpres = ucol_strcoll(coll, lines[i-1].buff, lines[i-1].buflen, lines[i].buff, lines[i].buflen);
+      cmpres2 = ucol_strcoll(coll, lines[i].buff, lines[i].buflen, lines[i-1].buff, lines[i-1].buflen);
+      //cmpres = res;
+      //cmpres2 = -cmpres;
+
+      if(cmpres != -cmpres2) {
+        error("Compare result not symmetrical on line "+ line);
+      }
+
+      if(((res&0x80000000) != (cmpres&0x80000000)) || (res == 0 && cmpres != 0) || (res != 0 && cmpres == 0)) {
+        error(UnicodeString("Difference between ucol_strcoll and sortkey compare on line ")+ UnicodeString(line));
+      }
+
+      if(res > 0) {
+        error(UnicodeString("Line %i is not greater or equal than previous line ")+ UnicodeString(i));
+        break;
+      } else if(res == 0) { /* equal */
+        res = u_strcmpCodePointOrder(lines[i-1].buff, lines[i].buff);
+        if (res == 0) {
+          error(UnicodeString("Probable error in test file on line %i (comparing identical strings)")+ UnicodeString(i));
+          break;
+        } else if (res > 0) {
+          error(UnicodeString("Sortkeys are identical, but code point comapare gives >0 on line ")+ UnicodeString(i));
+        }
+      }
+    }
+
+    oldSk = newSk;
+    oldLen = resLen;
+
+    newSk = (newSk == sk1)?sk2:sk1;
+  }
+
+    Mutex m;
+    done();
+  }
+};
+
+void MultithreadTest::TestCollators()
+{
+
+  UErrorCode status = U_ZERO_ERROR;
+  FILE *testFile = NULL;
+  char testDataPath[1024];
+  uprv_strcpy(testDataPath, IntlTest::loadTestData(status));
+  char* index = 0;
+  if (U_FAILURE(status)) {
+      errln("ERROR: could not open test data %s", u_errorName(status));
+	  return;
+  }
+  index=strrchr(testDataPath,(char)U_FILE_SEP_CHAR);
+
+  if((unsigned int)(index-testDataPath) != (strlen(testDataPath)-1)){
+          *(index+1)=0;
+  }
+  uprv_strcat(testDataPath,".."U_FILE_SEP_STRING);
+  uprv_strcat(testDataPath, "CollationTest_");
+
+  const char* type = "NON_IGNORABLE";
+
+  const char *ext = ".txt";
+  if(testFile) {
+    fclose(testFile);
+  }
+  char buffer[1024];
+  uprv_strcpy(buffer, testDataPath);
+  uprv_strcat(buffer, type);
+  int32_t bufLen = uprv_strlen(buffer);
+
+  // we try to open 3 files:
+  // path/CollationTest_type.txt
+  // path/CollationTest_type_SHORT.txt
+  // path/CollationTest_type_STUB.txt
+  // we are going to test with the first one that we manage to open.
+
+  uprv_strcpy(buffer+bufLen, ext);
+
+  testFile = fopen(buffer, "rb");
+
+  if(testFile == 0) {
+    uprv_strcpy(buffer+bufLen, "_SHORT");
+    uprv_strcat(buffer, ext);
+    testFile = fopen(buffer, "rb");
+
+    if(testFile == 0) {
+      uprv_strcpy(buffer+bufLen, "_STUB");
+      uprv_strcat(buffer, ext);
+      testFile = fopen(buffer, "rb");
+
+      if (testFile == 0) {
+        *(buffer+bufLen) = 0;
+        errln("ERROR: could not open any of the conformance test files, tried opening base %s", buffer);
+        return;        
+      } else {
+        infoln(
+          "INFO: Working with the stub file.\n"
+          "If you need the full conformance test, please\n"
+          "download the appropriate data files from:\n"
+          "http://oss.software.ibm.com/cvs/icu4j/unicodetools/com/ibm/text/data/");
+      }
+    }
+  }
+
+  Line *lines = new Line[200000];
+  uprv_memset(lines, 0, sizeof(Line)*200000);
+  int32_t lineNum = 0;
+
+  UChar bufferU[1024];
+  int32_t buflen = 0;
+  uint32_t first = 0;
+  uint32_t offset = 0;
+
+  while (fgets(buffer, 1024, testFile) != NULL) {
+    offset = 0;
+    if(*buffer == 0 || buffer[0] == '#') {
+      continue;
+    }
+    offset = u_parseString(buffer, bufferU, 1024, &first, &status);
+    buflen = offset;
+    bufferU[offset++] = 0;
+    lines[lineNum].buflen = buflen;
+    //lines[lineNum].buff = new UChar[buflen+1];
+    u_memcpy(lines[lineNum].buff, bufferU, buflen);
+    lineNum++;
+  }
+  fclose(testFile);
+
+
+  
+  UCollator *coll = ucol_open("root", &status);
+  if(U_FAILURE(status)) {
+    errln("Couldn't open UCA collator");
+    return;
+  }
+  ucol_setAttribute(coll, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
+  ucol_setAttribute(coll, UCOL_CASE_FIRST, UCOL_OFF, &status);
+  ucol_setAttribute(coll, UCOL_CASE_LEVEL, UCOL_OFF, &status);
+  ucol_setAttribute(coll, UCOL_STRENGTH, UCOL_TERTIARY, &status);
+  ucol_setAttribute(coll, UCOL_ALTERNATE_HANDLING, UCOL_NON_IGNORABLE, &status);
+
+  int32_t noSpawned = 0;
+  int32_t spawnResult = 0;
+    CollatorThreadTest *tests;
+    tests = new CollatorThreadTest[kCollatorThreadThreads];
+ 
+    logln(UnicodeString("Spawning: ") + kCollatorThreadThreads + " threads * " + kFormatThreadIterations + " iterations each.");
+    int32_t j = 0;
+    for(j = 0; j < kCollatorThreadThreads; j++) {
+      //logln("Setting collator %i", j);
+      tests[j].setCollator(coll, lines, lineNum);
+    }
+    for(j = 0; j < kCollatorThreadThreads; j++) {
+      log("%i ", j);
+      spawnResult = tests[j].start();
+      if(spawnResult != 0) {
+	infoln("THREAD INFO: Couldn't spawn more than %i threads", noSpawned);
+	break;
+      }
+      noSpawned++;
+    }
+    logln("Spawned all");
+
+    //for(int32_t patience = kCollatorThreadPatience;patience > 0; patience --)
+    for(;;)
+    {
+        logln("Waiting...");
+
+        int32_t i;
+        int32_t terrs = 0;
+        int32_t completed =0;
+
+        for(i=0;i<kCollatorThreadThreads;i++)
+        {
+            umtx_lock(NULL);
+            UBool threadIsDone = tests[i].getDone();
+            umtx_unlock(NULL);
+            if(threadIsDone)
             {
                 completed++;
 
-                logln(UnicodeString("Test #") + i + " is complete.. ");
+                //logln(UnicodeString("Test #") + i + " is complete.. ");
 
                 UnicodeString theErr;
                 if(tests[i].getError(theErr))
@@ -812,15 +1125,22 @@ void MultithreadTest::TestThreadedIntl()
                 // print out the error, too, if any.
             }
         }
+	logln("Completed %i tests", completed);
 
-        if(completed == kFormatThreadThreads)
+        if(completed == noSpawned)
         {
-            logln("Done!");
+            logln("Done! All %i tests are finished", noSpawned);
 
             if(terrs)
             {
                 errln("There were errors.");
             }
+            ucol_close(coll);
+            delete[] tests;
+            //for(i = 0; i < lineNum; i++) {
+              //delete[] lines[i].buff;
+            //}
+            delete[] lines;
 
             return;
         }
@@ -828,7 +1148,10 @@ void MultithreadTest::TestThreadedIntl()
         SimpleThread::sleep(900);
     }
     errln("patience exceeded. ");
+            ucol_close(coll);
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
+
 #endif // ICU_USE_THREADS
 
