@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2010, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -8,20 +8,14 @@
 #include "intltest.h"
 
 /** Helper function for TestUnicodeData */
-U_CFUNC void U_CALLCONV unicodeDataLineFn(void *context,
+U_CAPI void U_CALLCONV unicodeDataLineFn(void *context,
                               char *fields[][2], int32_t fieldCount,
                               UErrorCode *pErrorCode);
 
-U_CFUNC void U_CALLCONV
-derivedPropsLineFn(void *context,
-                   char *fields[][2], int32_t fieldCount,
-                   UErrorCode *pErrorCode);
-
-U_NAMESPACE_BEGIN
-
-class Hashtable;
-
-U_NAMESPACE_END
+U_CAPI void U_CALLCONV
+derivedCorePropsLineFn(void *context,
+                       char *fields[][2], int32_t fieldCount,
+                       UErrorCode *pErrorCode);
 
 /** 
  * Test API and functionality of class Unicode
@@ -34,7 +28,6 @@ public:
     void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
 
     void TestAdditionalProperties();
-    void TestBinaryValues();
 
 private:
 
@@ -43,11 +36,10 @@ private:
                               UErrorCode *pErrorCode);
 
     friend void U_CALLCONV
-    derivedPropsLineFn(void *context,
+    derivedCorePropsLineFn(void *context,
                            char *fields[][2], int32_t fieldCount,
                            UErrorCode *pErrorCode);
 
-    UnicodeSet derivedProps[30];
-    U_NAMESPACE_QUALIFIER Hashtable *unknownPropertyNames;
+    UnicodeSet derivedCoreProps[30];
 };
 
