@@ -1,7 +1,7 @@
 
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -11,14 +11,18 @@
 *   03/15/2000   Madhu        Creation.
 ************************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_TRANSLITERATION
+
 #include "ittrans.h"
 #include "unhxtrts.h"
 #include "unicode/utypes.h"
 #include "unicode/translit.h"
-#include "unicode/unitohex.h"
 #include "unicode/unifilt.h"
 #include "unicode/uchar.h"
 #include "intltest.h"
+#include "unitohex.h"
 /*converts a Unicodestring to integer*/
 static int32_t getInt(UnicodeString str)
 {
@@ -67,7 +71,9 @@ static void pseudoHandleTransliterate(const Transliterator* t,
 /**
  * Used by TestConstruction() and TestTransliterate.
  */
+int32_t gTestUniFilterClassID;
 class TestUniFilter : public UnicodeFilter {
+  virtual UClassID getDynamicClassID() const { return &gTestUniFilterClassID; }
     virtual UnicodeFunctor* clone() const {
         return new TestUniFilter(*this);
     }
@@ -77,6 +83,15 @@ class TestUniFilter : public UnicodeFilter {
        else
           return TRUE;
     }
+    // Stubs
+    virtual UnicodeString& toPattern(UnicodeString& result,
+                                     UBool /*escapeUnprintable*/) const {
+        return result;
+    }
+    virtual UBool matchesIndexValue(uint8_t /*v*/) const {
+        return FALSE;
+    }
+    virtual void addMatchSetTo(UnicodeSet& /*toUnionTo*/) const {}
 };
 void UniToHexTransliteratorTest::TestConstruction(){
     UErrorCode status=U_ZERO_ERROR;
@@ -497,4 +512,4 @@ void UniToHexTransliteratorTest::expectAux(const UnicodeString& tag,
     }
 }
 
-
+#endif /* #if !UCONFIG_NO_TRANSLITERATION */
