@@ -1,12 +1,12 @@
 /*
-**************************************************************************************
-*   Copyright (C) 1999-2006 International Business Machines Corporation and
+**********************************************************************
+*   Copyright (C) 1999-2003 Alan Liu ,International Business Machines Corporation and
 *   others. All Rights Reserved.
-**************************************************************************************
+**********************************************************************
 *   Date        Name        Description
 *   10/20/99    alan        Creation.
 *   03/22/2000  Madhu       Added additional tests
-**************************************************************************************
+**********************************************************************
 */
 
 #include "unicode/utypes.h"
@@ -17,17 +17,8 @@
 #include "unicode/ustring.h"
 #include "unicode/parsepos.h"
 #include "unicode/symtable.h"
-#include "unicode/uversion.h"
 #include "hash.h"
 
-
-#define TEST_ASSERT_SUCCESS(status) {if (U_FAILURE(status)) { \
-    errln("fail in file \"%s\", line %d: \"%s\"", __FILE__, __LINE__, \
-    u_errorName(status));}}
-
-#define TEST_ASSERT(expr) {if (!(expr)) { \
-    errln("fail in file \"%s\", line %d", __FILE__, __LINE__); }}
-
 UnicodeString operator+(const UnicodeString& left, const UnicodeSet& set) {
     UnicodeString pat;
     set.toPattern(pat);
@@ -61,20 +52,17 @@ UnicodeSetTest::runIndexedTest(int32_t index, UBool exec,
         CASE(10,TestToPattern);
         CASE(11,TestIndexOf);
         CASE(12,TestStrings);
-        CASE(13,Testj2268);
-        CASE(14,TestCloseOver);
-        CASE(15,TestEscapePattern);
-        CASE(16,TestInvalidCodePoint);
-        CASE(17,TestSymbolTable);
-        CASE(18,TestSurrogate);
-        CASE(19,TestPosixClasses);
-        CASE(20,TestIteration);
+        CASE(13,TestStringPatterns);
+        CASE(14,Testj2268);
+        CASE(15,TestCloseOver);
+        CASE(16,TestEscapePattern);
+        CASE(17,TestInvalidCodePoint);
+        CASE(18,TestSymbolTable);
+        CASE(19,TestSurrogate);
         default: name = ""; break;
     }
 }
 
-static const char NOT[] = "%%%%";
-
 /** 
  * UVector was improperly copying contents
  * This code will crash this is still true
@@ -88,97 +76,42 @@ void UnicodeSetTest::Testj2268() {
 }
 
 /**
- * Test toPattern().
+ * Test that toPattern() round trips with syntax characters and
+ * whitespace.
  */
 void UnicodeSetTest::TestToPattern() {
-    UErrorCode ec = U_ZERO_ERROR;
+    static const char* OTHER_TOPATTERN_TESTS[] = {
+        "[[:latin:]&[:greek:]]", 
+        "[[:latin:]-[:greek:]]",
+        "[:nonspacing mark:]",
+        NULL
+    };
 
-    // Test that toPattern() round trips with syntax characters and
-    // whitespace.
-    {
-        static const char* OTHER_TOPATTERN_TESTS[] = {
-            "[[:latin:]&[:greek:]]", 
-            "[[:latin:]-[:greek:]]",
-            "[:nonspacing mark:]",
-            NULL
-        };
+    for (int32_t j=0; OTHER_TOPATTERN_TESTS[j]!=NULL; ++j) {
+        UErrorCode ec = U_ZERO_ERROR;
+        UnicodeSet s(OTHER_TOPATTERN_TESTS[j], ec);
+        if (U_FAILURE(ec)) {
+            errln((UnicodeString)"FAIL: bad pattern " + OTHER_TOPATTERN_TESTS[j]);
+            continue;
+        }
+        checkPat(OTHER_TOPATTERN_TESTS[j], s);
+    }
+    
+    for (UChar32 i = 0; i <= 0x10FFFF; ++i) {
+        if ((i <= 0xFF && !u_isalpha(i)) || u_isspace(i)) {
 
-        for (int32_t j=0; OTHER_TOPATTERN_TESTS[j]!=NULL; ++j) {
-            ec = U_ZERO_ERROR;
-            UnicodeSet s(OTHER_TOPATTERN_TESTS[j], ec);
-            if (U_FAILURE(ec)) {
-                errln((UnicodeString)"FAIL: bad pattern " + OTHER_TOPATTERN_TESTS[j]);
+            // check various combinations to make sure they all work.
+            if (i != 0 && !toPatternAux(i, i)){
                 continue;
             }
-            checkPat(OTHER_TOPATTERN_TESTS[j], s);
-        }
-    
-        for (UChar32 i = 0; i <= 0x10FFFF; ++i) {
-            if ((i <= 0xFF && !u_isalpha(i)) || u_isspace(i)) {
-
-                // check various combinations to make sure they all work.
-                if (i != 0 && !toPatternAux(i, i)){
-                    continue;
-                }
-                if (!toPatternAux(0, i)){
-                    continue;
-                }
-                if (!toPatternAux(i, 0xFFFF)){
-                    continue;
-                }
+            if (!toPatternAux(0, i)){
+                continue;
+            }
+            if (!toPatternAux(i, 0xFFFF)){
+                continue;
             }
         }
     }
-
-    // Test pattern behavior of multicharacter strings.
-    {
-        ec = U_ZERO_ERROR;
-        UnicodeSet* s = new UnicodeSet("[a-z {aa} {ab}]", ec);
-
-        // This loop isn't a loop.  It's here to make the compiler happy.
-        // If you're curious, try removing it and changing the 'break'
-        // statements (except for the last) to goto's.
-        for (;;) {
-            if (U_FAILURE(ec)) break;
-            const char* exp1[] = {"aa", "ab", NOT, "ac", NULL};
-            expectToPattern(*s, "[a-z{aa}{ab}]", exp1);
-
-            s->add("ac");
-            const char* exp2[] = {"aa", "ab", "ac", NOT, "xy", NULL};
-            expectToPattern(*s, "[a-z{aa}{ab}{ac}]", exp2);
-
-            s->applyPattern("[a-z {\\{l} {r\\}}]", ec);
-            if (U_FAILURE(ec)) break;
-            const char* exp3[] = {"{l", "r}", NOT, "xy", NULL};
-            expectToPattern(*s, "[a-z{r\\}}{\\{l}]", exp3);
-
-            s->add("[]");
-            const char* exp4[] = {"{l", "r}", "[]", NOT, "xy", NULL};
-            expectToPattern(*s, "[a-z{\\[\\]}{r\\}}{\\{l}]", exp4);
-
-            s->applyPattern("[a-z {\\u4E01\\u4E02}{\\n\\r}]", ec);
-            if (U_FAILURE(ec)) break;
-            const char* exp5[] = {"\\u4E01\\u4E02", "\n\r", NULL};
-            expectToPattern(*s, "[a-z{\\u000A\\u000D}{\\u4E01\\u4E02}]", exp5);
-
-            // j2189
-            s->clear();
-            s->add(UnicodeString("abc", ""));
-            s->add(UnicodeString("abc", ""));
-            const char* exp6[] = {"abc", NOT, "ab", NULL};
-            expectToPattern(*s, "[{abc}]", exp6);
-
-            break;
-        }
-
-        if (U_FAILURE(ec)) errln("FAIL: pattern parse error");
-        delete s;
-    }
- 
-    // JB#3400: For 2 character ranges prefer [ab] to [a-b]
-    UnicodeSet s;
-    s.add((UChar)97, (UChar)98); // 'a', 'b'
-    expectToPattern(s, "[ab]", NULL);
 }
     
 UBool UnicodeSetTest::toPatternAux(UChar32 start, UChar32 end) {
@@ -365,9 +298,6 @@ UnicodeSetTest::TestAddRemove(void) {
     UnicodeSet set; // Construct empty set
     doAssert(set.isEmpty() == TRUE, "set should be empty");
     doAssert(set.size() == 0, "size should be 0");
-    set.complement();
-    doAssert(set.size() == 0x110000, "size should be 0x110000");
-    set.clear();
     set.add(0x0061, 0x007a);
     expectPairs(set, "az");
     doAssert(set.isEmpty() == FALSE, "set should not be empty");
@@ -675,87 +605,6 @@ void UnicodeSetTest::TestAPI() {
     }
 }
 
-void UnicodeSetTest::TestIteration() {
-    UErrorCode ec = U_ZERO_ERROR;
-    int i = 0;
-    int outerLoop;
-    
-    // 6 code points, 3 ranges, 2 strings, 8 total elements
-    //   Iteration will access them in sorted order -  a, b, c, y, z, U0001abcd, "str1", "str2"
-    UnicodeSet set("[zabyc\\U0001abcd{str1}{str2}]", ec);
-    TEST_ASSERT_SUCCESS(ec);
-    UnicodeSetIterator it(set);
-
-    for (outerLoop=0; outerLoop<3; outerLoop++) {
-        // Run the test multiple times, to check that iterator.reset() is working.
-        for (i=0; i<10; i++) {
-            UBool         nextv        = it.next();
-            UBool         isString     = it.isString();
-            int32_t       codePoint    = it.getCodepoint();
-            //int32_t       codePointEnd = it.getCodepointEnd();
-            UnicodeString s   = it.getString();
-            switch (i) {
-            case 0:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x61);
-                TEST_ASSERT(s == "a");
-                break;
-            case 1:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x62);
-                TEST_ASSERT(s == "b");
-                break;
-            case 2:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x63);
-                TEST_ASSERT(s == "c");
-                break;
-            case 3:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x79);
-                TEST_ASSERT(s == "y");
-                break;
-            case 4:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x7a);
-                TEST_ASSERT(s == "z");
-                break;
-            case 5:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == FALSE);
-                TEST_ASSERT(codePoint==0x1abcd);
-                TEST_ASSERT(s == UnicodeString((UChar32)0x1abcd));
-                break;
-            case 6:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == TRUE);
-                TEST_ASSERT(s == "str1");
-                break;
-            case 7:
-                TEST_ASSERT(nextv == TRUE);
-                TEST_ASSERT(isString == TRUE);
-                TEST_ASSERT(s == "str2");
-                break;
-            case 8:
-                TEST_ASSERT(nextv == FALSE);
-                break;
-            case 9:
-                TEST_ASSERT(nextv == FALSE);
-                break;
-            }
-        }
-        it.reset();  // prepare to run the iteration again.
-    }
-}
-                
-
-
-
 void UnicodeSetTest::TestStrings() {
     UErrorCode ec = U_ZERO_ERROR;
     
@@ -795,6 +644,55 @@ void UnicodeSetTest::TestStrings() {
     }        
 }
 
+static const char NOT[] = "%%%%";
+
+/**
+ * Test pattern behavior of multicharacter strings.
+ */
+void UnicodeSetTest::TestStringPatterns() {
+    UErrorCode ec = U_ZERO_ERROR;
+    UnicodeSet* s = new UnicodeSet("[a-z {aa} {ab}]", ec);
+
+    // This loop isn't a loop.  It's here to make the compiler happy.
+    // If you're curious, try removing it and changing the 'break'
+    // statements (except for the last) to goto's.
+    for (;;) {
+        if (U_FAILURE(ec)) break;
+        const char* exp1[] = {"aa", "ab", NOT, "ac", NULL};
+        expectToPattern(*s, "[a-z{aa}{ab}]", exp1);
+
+        s->add("ac");
+        const char* exp2[] = {"aa", "ab", "ac", NOT, "xy", NULL};
+        expectToPattern(*s, "[a-z{aa}{ab}{ac}]", exp2);
+
+        s->applyPattern("[a-z {\\{l} {r\\}}]", ec);
+        if (U_FAILURE(ec)) break;
+        const char* exp3[] = {"{l", "r}", NOT, "xy", NULL};
+        expectToPattern(*s, "[a-z{r\\}}{\\{l}]", exp3);
+
+        s->add("[]");
+        const char* exp4[] = {"{l", "r}", "[]", NOT, "xy", NULL};
+        expectToPattern(*s, "[a-z{\\[\\]}{r\\}}{\\{l}]", exp4);
+
+        s->applyPattern("[a-z {\\u4E01\\u4E02}{\\n\\r}]", ec);
+        if (U_FAILURE(ec)) break;
+        const char* exp5[] = {"\\u4E01\\u4E02", "\n\r", NULL};
+        expectToPattern(*s, "[a-z{\\u000A\\u000D}{\\u4E01\\u4E02}]", exp5);
+
+        // j2189
+        s->clear();
+        s->add(UnicodeString("abc", ""));
+        s->add(UnicodeString("abc", ""));
+        const char* exp6[] = {"abc", NOT, "ab", NULL};
+        expectToPattern(*s, "[{abc}]", exp6);
+
+        break;
+    }
+
+    if (U_FAILURE(ec)) errln("FAIL: pattern parse error");
+    delete s;
+}
+
 /**
  * Test the [:Latin:] syntax.
  */
@@ -852,10 +750,8 @@ void UnicodeSetTest::TestPropertySet() {
         "x",
 
         "[:math=false:]",
-        "q)*(",
-        // weiv: )(and * were removed from math in Unicode 4.0.1
-        //"(*+)",
-        "+<>^",
+        "q",
+        "(*+)",
 
         // JB#1767 \N{}, \p{ASCII}
         "[:Ascii:]",
@@ -949,31 +845,7 @@ void UnicodeSetTest::TestPropertySet() {
         
         "[ab\\uDC00cd]", // JB#2906: isolated trail in middle
         "abcd\\uDC00",
-        "ef\\uD800\\U00010000",
-
-        "[:^lccc=0:]", // Lead canonical class
-        "\\u0300\\u0301",
-        "abcd\\u00c0\\u00c5",
-
-        "[:^tccc=0:]", // Trail canonical class
-        "\\u0300\\u0301\\u00c0\\u00c5",
-        "abcd",
-
-        "[[:^lccc=0:][:^tccc=0:]]", // Lead and trail canonical class
-        "\\u0300\\u0301\\u00c0\\u00c5",
-        "abcd",
-
-        "[[:^lccc=0:]-[:^tccc=0:]]", // Stuff that starts with an accent but ends with a base (none right now)
-        "",
-        "abcd\\u0300\\u0301\\u00c0\\u00c5",
-        
-        "[[:ccc=0:]-[:lccc=0:]-[:tccc=0:]]", // Weirdos. Complete canonical class is zero, but both lead and trail are not
-        "\\u0F73\\u0F75\\u0F81",
-        "abcd\\u0300\\u0301\\u00c0\\u00c5",
-
-        "[:Assigned:]",
-        "A\\uE000\\uF8FF\\uFDC7\\U00010000\\U0010FFFD",
-        "\\u0888\\uFDD3\\uFFFE\\U00050005"
+        "ef\\uD800\\U00010000"
     };
 
     static const int32_t DATA_LEN = sizeof(DATA)/sizeof(DATA[0]);
@@ -984,101 +856,6 @@ void UnicodeSetTest::TestPropertySet() {
     }
 }
 
-/**
-  * Test that Posix style character classes [:digit:], etc.
-  *   have the Unicode definitions from TR 18.
-  */
-void UnicodeSetTest::TestPosixClasses() {
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:alpha:]", status);
-        UnicodeSet s2("\\p{Alphabetic}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:lower:]", status);
-        UnicodeSet s2("\\p{lowercase}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:upper:]", status);
-        UnicodeSet s2("\\p{Uppercase}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:punct:]", status);
-        UnicodeSet s2("\\p{gc=Punctuation}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:digit:]", status);
-        UnicodeSet s2("\\p{gc=DecimalNumber}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:xdigit:]", status);
-        UnicodeSet s2("[\\p{DecimalNumber}\\p{HexDigit}]", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:alnum:]", status);
-        UnicodeSet s2("[\\p{Alphabetic}\\p{DecimalNumber}]", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:space:]", status);
-        UnicodeSet s2("\\p{Whitespace}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:blank:]", status);
-        TEST_ASSERT_SUCCESS(status);
-        UnicodeSet s2("[\\p{Whitespace}-[\\u000a\\u000B\\u000c\\u000d\\u0085\\p{LineSeparator}\\p{ParagraphSeparator}]]",
-            status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:cntrl:]", status);
-        TEST_ASSERT_SUCCESS(status);
-        UnicodeSet s2("\\p{Control}", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:graph:]", status);
-        TEST_ASSERT_SUCCESS(status);
-        UnicodeSet s2("[^\\p{Whitespace}\\p{Control}\\p{Surrogate}\\p{Unassigned}]", status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-    {
-        UErrorCode status = U_ZERO_ERROR;
-        UnicodeSet s1("[:print:]", status);
-        TEST_ASSERT_SUCCESS(status);
-        UnicodeSet s2("[[:graph:][:blank:]-[\\p{Control}]]" ,status);
-        TEST_ASSERT_SUCCESS(status);
-        TEST_ASSERT(s1==s2);
-    }
-}
 /**
  * Test cloning of UnicodeSet.  For C++, we test the copy constructor.
  */
@@ -1122,8 +899,7 @@ void UnicodeSetTest::TestIndexOf() {
 void UnicodeSetTest::TestCloseOver() {
     UErrorCode ec = U_ZERO_ERROR;
 
-    char CASE[] = {(char)USET_CASE_INSENSITIVE};
-    char CASE_MAPPINGS[] = {(char)USET_ADD_CASE_MAPPINGS};
+    char CASE[] = {(char)USET_CASE};
     const char* DATA[] = {
         // selector, input, output
         CASE,
@@ -1153,50 +929,11 @@ void UnicodeSetTest::TestCloseOver() {
         CASE,
         "[ABC]","[A-Ca-c]",
 
-        CASE, "[i]", "[iI]",
-
-        CASE, "[\\u0130]",          "[\\u0130{i\\u0307}]", // dotted I
-        CASE, "[{i\\u0307}]",       "[\\u0130{i\\u0307}]", // i with dot
-
-        CASE, "[\\u0131]",          "[\\u0131]", // dotless i
-
-        CASE, "[\\u0390]",          "[\\u0390\\u1FD3{\\u03B9\\u0308\\u0301}]",
-
-        CASE, "[\\u03c2]",          "[\\u03a3\\u03c2\\u03c3]", // sigmas
-
-        CASE, "[\\u03f2]",          "[\\u03f2\\u03f9]", // lunate sigmas
-
-        CASE, "[\\u03f7]",          "[\\u03f7\\u03f8]",
-
-        CASE, "[\\u1fe3]",          "[\\u03b0\\u1fe3{\\u03c5\\u0308\\u0301}]",
-
-        CASE, "[\\ufb05]",          "[\\ufb05\\ufb06{st}]",
-        CASE, "[{st}]",             "[\\ufb05\\ufb06{st}]",
-
-        CASE, "[\\U0001044F]",      "[\\U00010427\\U0001044F]",
-
-        CASE, "[{a\\u02BE}]",       "[\\u1E9A{a\\u02BE}]", // first in sorted table
-
-        CASE, "[{\\u1f7c\\u03b9}]", "[\\u1ff2{\\u1f7c\\u03b9}]", // last in sorted table
-
-        CASE_MAPPINGS,
-        "[aq\\u00DF{Bc}{bC}{Fi}]",
-        "[aAqQ\\u00DF{ss}{Ss}{SS}{Bc}{BC}{bC}{bc}{FI}{Fi}{fi}]",
-
-        CASE_MAPPINGS,
-        "[\\u01F1]", // 'DZ'
-        "[\\u01F1\\u01F2\\u01F3]",
-        
-        CASE_MAPPINGS,
-        "[a-z]",
-        "[A-Za-z]",
-
         NULL
     };
 
     UnicodeSet s;
     UnicodeSet t;
-    UnicodeString buf;
     for (int32_t i=0; DATA[i]!=NULL; i+=3) {
         int32_t selector = DATA[i][0];
         UnicodeString pat(DATA[i+1]);
@@ -1211,72 +948,12 @@ void UnicodeSetTest::TestCloseOver() {
         if (s == t) {
             logln((UnicodeString)"Ok: " + pat + ".closeOver(" + selector + ") => " + exp);
         } else {
+            UnicodeString buf;
             errln((UnicodeString)"FAIL: " + pat + ".closeOver(" + selector + ") => " +
                   s.toPattern(buf, TRUE) + ", expected " + exp);
         }
     }
 
-#if 0
-    /*
-     * Unused test code.
-     * This was used to compare the old implementation (using USET_CASE)
-     * with the new one (using 0x100 temporarily)
-     * while transitioning from hardcoded case closure tables in uniset.cpp
-     * (moved to uniset_props.cpp) to building the data by gencase into ucase.icu.
-     * and using ucase.c functions for closure.
-     * See Jitterbug 3432 RFE: Move uniset.cpp data to a data file
-     *
-     * Note: The old and new implementation never fully matched because
-     * the old implementation turned out to not map U+0130 and U+0131 correctly
-     * (dotted I and dotless i) and because the old implementation's data tables
-     * were outdated compared to Unicode 4.0.1 at the time of the change to the
-     * new implementation. (So sigmas and some other characters were not handled
-     * according to the newer Unicode version.)
-     */
-    UnicodeSet sens("[:case_sensitive:]", ec), sens2, s2;
-    UnicodeSetIterator si(sens);
-    UnicodeString str, buf2;
-    const UnicodeString *pStr;
-    UChar32 c;
-    while(si.next()) {
-        if(!si.isString()) {
-            c=si.getCodepoint();
-            s.clear();
-            s.add(c);
-
-            str.setTo(c);
-            str.foldCase();
-            sens2.add(str);
-
-            t=s;
-            s.closeOver(USET_CASE);
-            t.closeOver(0x100);
-            if(s!=t) {
-                errln("FAIL: closeOver(U+%04x) differs: ", c);
-                errln((UnicodeString)"old "+s.toPattern(buf, TRUE)+" new: "+t.toPattern(buf2, TRUE));
-            }
-        }
-    }
-    // remove all code points
-    // should contain all full case folding mapping strings
-    sens2.remove(0, 0x10ffff);
-    si.reset(sens2);
-    while(si.next()) {
-        if(si.isString()) {
-            pStr=&si.getString();
-            s.clear();
-            s.add(*pStr);
-            t=s2=s;
-            s.closeOver(USET_CASE);
-            t.closeOver(0x100);
-            if(s!=t) {
-                errln((UnicodeString)"FAIL: closeOver("+s2.toPattern(buf, TRUE)+") differs: ");
-                errln((UnicodeString)"old "+s.toPattern(buf, TRUE)+" new: "+t.toPattern(buf2, TRUE));
-            }
-        }
-    }
-#endif
-
     // Test the pattern API
     s.applyPattern("[abc]", USET_CASE_INSENSITIVE, NULL, ec);
     if (U_FAILURE(ec)) {
@@ -1290,12 +967,6 @@ void UnicodeSetTest::TestCloseOver() {
     } else {
         expectContainment(v, "defDEF", "abcABC");
     }
-    UnicodeSet cm("[abck]", USET_ADD_CASE_MAPPINGS, NULL, ec);
-    if (U_FAILURE(ec)) {
-        errln("FAIL: construct w/case mappings failed");
-    } else {
-        expectContainment(cm, "abckABCK", CharsToUnicodeString("defDEF\\u212A"));
-    }
 }
 
 void UnicodeSetTest::TestEscapePattern() {
@@ -2002,9 +1673,6 @@ void UnicodeSetTest::expectToPattern(const UnicodeSet& set,
         errln((UnicodeString)"FAIL: toPattern() => \"" + pat + "\", expected \"" + expPat + "\"");
         return;
     }
-    if (expStrings == NULL) {
-        return;
-    }
     UBool in = TRUE;
     for (int32_t i=0; expStrings[i] != NULL; ++i) {
         if (expStrings[i] == NOT) { // sic; pointer comparison
