@@ -1,7 +1,7 @@
 
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2006, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************
 **********************************************************************
@@ -47,10 +47,10 @@ private:
 
     void TestAPI(void);
 
-    void TestIteration(void);
-
     void TestStrings(void);
 
+    void TestStringPatterns(void);
+
     void TestScriptSet(void);
 
     /**
@@ -74,8 +74,6 @@ private:
 
     void TestSurrogate();
 
-    void TestPosixClasses();
-
 private:
 
     UBool toPatternAux(UChar32 start, UChar32 end);
