@@ -1,7 +1,7 @@
 /*
 *****************************************************************************************
 *
-*   Copyright (C) 1996-2000, International Business Machines
+*   Copyright (C) 1996-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *****************************************************************************************
@@ -11,9 +11,13 @@
 #ifndef CTEST_H
 #define CTEST_H
 
+#include <stdarg.h>
+
+#include "unicode/utypes.h"
+#include "unicode/utrace.h"
 
 /*Deals with imports and exports of the dynamic library*/
-#ifdef _WIN32
+#if defined(_WIN32) || defined(U_CYGWIN)
     #define T_CTEST_EXPORT __declspec(dllexport)
     #define T_CTEST_IMPORT __declspec(dllimport)
 #else
@@ -86,6 +90,23 @@ T_CTEST_EXPORT_API extern int ERR_MSG;
  */
 T_CTEST_EXPORT_API extern int QUICK;
 
+/**
+ * Set this to nonzero to warn (not error) on missing data. 
+ * Otherwise, zero will cause an error to be propagated when data is not available.
+ * Affects the behavior of log_dataerr.
+ *
+ * @see log_data_err
+ * @internal Internal APIs for testing purpose only
+ */
+T_CTEST_EXPORT_API extern int WARN_ON_MISSING_DATA;
+
+/**
+ * ICU tracing level, is set by command line option
+ *
+ * @internal
+ */
+T_CTEST_EXPORT_API extern UTraceLevel ICU_TRACE;
+
 /**
  * Show the names of all nodes.
  *
@@ -146,6 +167,15 @@ T_CTEST_API void log_err(const char* pattern, ...);
  */
 T_CTEST_API void log_info(const char* pattern, ...);
 
+/**
+ * Log an informational message. (vprintf style)
+ * @param prefix a string that is output before the pattern and without formatting
+ * @param pattern printf-style format string
+ * @param ap variable-arguments list
+ * @internal Internal APIs for testing purpose only
+ */
+T_CTEST_API void vlog_info(const char *prefix, const char *pattern, va_list ap);
+
 /**
  * Log a verbose informational message. (printf style)
  * This message will only appear if the global VERBOSITY is nonzero
@@ -154,6 +184,15 @@ T_CTEST_API void log_info(const char* pattern, ...);
  */
 T_CTEST_API void log_verbose(const char* pattern, ...);
 
+/**
+ * Log an error message concerning missing data. (printf style)
+ * If WARN_ON_MISSING_DATA is nonzero, this will case a log_info (warning) to be
+ * printed, but if it is zero this will produce an error (log_err).
+ * @param pattern printf-style format string
+ * @internal Internal APIs for testing purpose only
+ */
+T_CTEST_API void log_data_err(const char *pattern, ...);
+
 /**
  * Processes the command line arguments.
  * This is a sample implementation
@@ -173,6 +212,9 @@ T_CTEST_API int processArgs(const TestNode* root,
                              const char* const argv[]);
 
 
+T_CTEST_API 
+const char* getTestName(void);
+
 
 
 #endif
