@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -28,7 +28,6 @@
 #include "unicode/uclean.h"
 #include "unewdata.h"
 #include "uoptions.h"
-#include "putilimp.h"
 
 #define STRING_STORE_SIZE 100000
 #define MAX_FILE_COUNT 2000
@@ -111,12 +110,6 @@ allocString(uint32_t length);
 static int
 compareFiles(const void *file1, const void *file2);
 
-static char * 
-pathToFullPath(const char *path);
-
-/* map non-tree separator (such as '\') to tree separator ('/') inplace. */
-static void
-fixDirToTreePath(char *s);
 /* -------------------------------------------------------------------------- */
 
 static UOption options[]={
@@ -129,8 +122,7 @@ static UOption options[]={
 /*6*/ UOPTION_DEF( "name", 'n', UOPT_REQUIRES_ARG),
 /*7*/ UOPTION_DEF( "type", 't', UOPT_REQUIRES_ARG),
 /*8*/ UOPTION_DEF( "source", 'S', UOPT_NO_ARG),
-/*9*/ UOPTION_DEF( "entrypoint", 'e', UOPT_REQUIRES_ARG),
-/*10*/UOPTION_SOURCEDIR,
+/*9*/ UOPTION_DEF( "entrypoint", 'e', UOPT_REQUIRES_ARG)
 };
 
 static char *symPrefix = NULL;
@@ -152,7 +144,6 @@ main(int argc, char* argv[]) {
     options[4].value=u_getDataDirectory();
     options[6].value=COMMON_DATA_NAME;
     options[7].value=DATA_TYPE;
-    options[10].value=".";
     argc=u_parseArgs(argc, argv, sizeof(options)/sizeof(options[0]), options);
 
     /* error handling, printing usage message */
@@ -163,7 +154,6 @@ main(int argc, char* argv[]) {
     } else if(argc<2) {
         argc=-1;
     }
-
     if(argc<0 || options[0].doesOccur || options[1].doesOccur) {
         FILE *where = argc < 0 ? stderr : stdout;
         
@@ -172,11 +162,11 @@ main(int argc, char* argv[]) {
          * required supported string length is 509 bytes.
          */
         fprintf(where,
-                "%csage: %s [ -h, -?, --help ] [ -v, --verbose ] [ -c, --copyright ] [ -C, --comment comment ] [ -d, --destdir dir ] [ -n, --name filename ] [ -t, --type filetype ] [ -S, --source tocfile ] [ -e, --entrypoint name ] maxsize listfile\n", argc < 0 ? 'u' : 'U', *argv);
+                "%csage: %s [ -h, -?, --help ] [ -v, --verbose ] [ -c, --copyright ] [ -C, --comment comment ] [ -d, --destdir dir ] [ -n, --name filename ] [ -t, --type filetype ] [ -S, --source tocfile ] [ -e, --entrypoint name ] [ maxsize ] [ [ -f ] filename ]\n", argc < 0 ? 'u' : 'U', *argv);
         if (options[0].doesOccur || options[1].doesOccur) {
             fprintf(where, "\n"
-                "Read the list file (default: standard input) and create a common data\n"
-                "file from specified files. Omit any files larger than maxsize, if maxsize > 0.\n");
+            "Read the list file (default: standard input) and create a common data\n"
+                    "file from specified files; omit any larger than maxsize.\n");
             fprintf(where, "\n"
             "Options:\n"
             "\t-h, -?, --help              this usage text\n"
@@ -240,14 +230,7 @@ main(int argc, char* argv[]) {
         }
 
         /* add the file */
-#if (U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR)
-        {
-          char *t;
-          while((t = uprv_strchr(line,U_FILE_ALT_SEP_CHAR))) {
-            *t = U_FILE_SEP_CHAR;
-          }
-        }
-#endif
+
         addFile(getLongPathname(line), sourceTOC, verbose);
     }
 
@@ -331,7 +314,7 @@ main(int argc, char* argv[]) {
             length=files[i].fileSize;
 
             if (nread != files[i].fileSize) {
-              fprintf(stderr, "gencmn: unable to read %s properly (got %ld/%ld byte%s)\n", files[i].pathname,  (long)nread, (long)files[i].fileSize, files[i].fileSize == 1 ? "" : "s");
+                fprintf(stderr, "gencmn: unable to read %s properly (got %ld/%ld byte%s)\n", files[i].pathname, (long)nread, (long)files[i].fileSize, files[i].fileSize == 1 ? "" : "s");
                 exit(U_FILE_ACCESS_ERROR);
             }
         }
@@ -384,6 +367,19 @@ main(int argc, char* argv[]) {
         }
 
 
+#if 0
+        symPrefix = (char *) uprv_malloc(uprv_strlen(entrypointName) + 2);
+
+        /* test for NULL */
+	if (symPrefix == NULL) {
+	    sprintf(buffer, "U_MEMORY_ALLOCATION_ERROR");
+	    exit(U_MEMORY_ALLOCATION_ERROR);
+	}
+
+        uprv_strcpy(symPrefix, entrypointName);
+        uprv_strcat(symPrefix, "_");
+#endif
+
         /* write the source file */
         sprintf(buffer,
             "/*\n"
@@ -457,7 +453,6 @@ static void
 addFile(const char *filename, UBool sourceTOC, UBool verbose) {
     char *s;
     uint32_t length;
-    char *fullPath = NULL;
 
     if(fileCount==MAX_FILE_COUNT) {
         fprintf(stderr, "gencmn: too many files, maximum is %d\n", MAX_FILE_COUNT);
@@ -467,48 +462,38 @@ addFile(const char *filename, UBool sourceTOC, UBool verbose) {
     if(!sourceTOC) {
         FileStream *file;
 
-        if(uprv_pathIsAbsolute(filename)) {
-            fprintf(stderr, "gencmn: Error: absolute path encountered. Old style paths are not supported. Use relative paths such as 'fur.res' or 'translit%cfur.res'.\n\tBad path: '%s'\n", U_FILE_SEP_CHAR, filename);
-            exit(U_ILLEGAL_ARGUMENT_ERROR);
-        }
-        fullPath = pathToFullPath(filename);
-
         /* store the pathname */
-        length = (uint32_t)(uprv_strlen(filename) + 1 + uprv_strlen(options[6].value) + 1);
+        length = (uint32_t)(uprv_strlen(filename) + 1);
         s=allocString(length);
-        uprv_strcpy(s, options[6].value);
-        uprv_strcat(s, U_TREE_ENTRY_SEP_STRING);
-        uprv_strcat(s, filename);
+        uprv_memcpy(s, filename, length);
+        files[fileCount].pathname=s;
 
         /* get the basename */
-        fixDirToTreePath(s);
+        s=(char *)findBasename(s);
         files[fileCount].basename=s;
+        length = (uint32_t)(uprv_strlen(s) + 1);
         files[fileCount].basenameLength=length;
-
-        files[fileCount].pathname=fullPath;
-
         basenameTotal+=length;
 
         /* try to open the file */
-        file=T_FileStream_open(fullPath, "rb");
+        file=T_FileStream_open(filename, "rb");
         if(file==NULL) {
-            fprintf(stderr, "gencmn: unable to open listed file %s\n", fullPath);
+            fprintf(stderr, "gencmn: unable to open listed file %s\n", filename);
             exit(U_FILE_ACCESS_ERROR);
         }
 
         /* get the file length */
         length=T_FileStream_size(file);
         if(T_FileStream_error(file) || length<=20) {
-            fprintf(stderr, "gencmn: unable to get length of listed file %s\n", fullPath);
+            fprintf(stderr, "gencmn: unable to get length of listed file %s\n", filename);
             exit(U_FILE_ACCESS_ERROR);
         }
-        
         T_FileStream_close(file);
 
         /* do not add files that are longer than maxSize */
         if(maxSize && length>maxSize) {
             if (verbose) {
-                printf("%s ignored (size %ld > %ld)\n", fullPath, (long)length, (long)maxSize);
+                printf("%s ignored (size %ld > %ld)\n", filename, (long)length, (long)maxSize);
             }
             return;
         }
@@ -517,20 +502,16 @@ addFile(const char *filename, UBool sourceTOC, UBool verbose) {
         char *t;
 
         /* get and store the basename */
-        /* need to include the package name */
-        length = (uint32_t)(uprv_strlen(filename) + 1 + uprv_strlen(options[6].value) + 1);
+        filename=findBasename(filename);
+        length = (uint32_t)(uprv_strlen(filename) + 1);
         s=allocString(length);
-        uprv_strcpy(s, options[6].value);
-        uprv_strcat(s, U_TREE_ENTRY_SEP_STRING);
-        uprv_strcat(s, filename);
-        fixDirToTreePath(s);
+        uprv_memcpy(s, filename, length);
         files[fileCount].basename=s;
 
-
         /* turn the basename into an entry point name and store in the pathname field */
         t=files[fileCount].pathname=allocString(length);
         while(--length>0) {
-            if(*s=='.' || *s=='-' || *s=='/') {
+            if(*s=='.' || *s=='-') {
                 *t='_';
             } else {
                 *t=*s;
@@ -540,6 +521,7 @@ addFile(const char *filename, UBool sourceTOC, UBool verbose) {
         }
         *t=0;
     }
+
     ++fileCount;
 }
 
@@ -557,69 +539,12 @@ allocString(uint32_t length) {
     return p;
 }
 
-static char * 
-pathToFullPath(const char *path) {
-    int32_t length;
-    int32_t newLength;
-    char *fullPath;
-    int32_t n;
-
-    length = (uint32_t)(uprv_strlen(path) + 1);
-    newLength = (length + 1 + (int32_t)uprv_strlen(options[10].value));
-    fullPath = uprv_malloc(newLength);
-    if(options[10].doesOccur) {
-        uprv_strcpy(fullPath, options[10].value);
-        uprv_strcat(fullPath, U_FILE_SEP_STRING);
-    } else {
-        fullPath[0] = 0;
-    }
-    n = (int32_t)uprv_strlen(fullPath);
-    uprv_strcat(fullPath, path);
-
-#if (U_FILE_ALT_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR)
-#if (U_FILE_ALT_SEP_CHAR != U_FILE_SEP_CHAR)
-    /* replace tree separator (such as '/') with file sep char (such as ':' or '\\') */
-    for(;fullPath[n];n++) {
-        if(fullPath[n] == U_FILE_ALT_SEP_CHAR) {
-            fullPath[n] = U_FILE_SEP_CHAR;
-        }
-    }
-#endif
-#endif
-#if (U_FILE_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR)
-    /* replace tree separator (such as '/') with file sep char (such as ':' or '\\') */
-    for(;fullPath[n];n++) {
-        if(fullPath[n] == U_TREE_ENTRY_SEP_CHAR) {
-            fullPath[n] = U_FILE_SEP_CHAR;
-        }
-    }
-#endif
-    return fullPath;
-}
-
 static int
 compareFiles(const void *file1, const void *file2) {
     /* sort by basename */
     return uprv_strcmp(((File *)file1)->basename, ((File *)file2)->basename);
 }
 
-static void
-fixDirToTreePath(char *s)
-{
-#if (U_FILE_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR) || ((U_FILE_ALT_SEP_CHAR != U_FILE_SEP_CHAR) && (U_FILE_ALT_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR))
-    char *t;
-#endif
-#if (U_FILE_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR)
-    for(t=s;t=uprv_strchr(t,U_FILE_SEP_CHAR);) {
-        *t = U_TREE_ENTRY_SEP_CHAR;
-    }
-#endif
-#if (U_FILE_ALT_SEP_CHAR != U_FILE_SEP_CHAR) && (U_FILE_ALT_SEP_CHAR != U_TREE_ENTRY_SEP_CHAR)
-    for(t=s;t=uprv_strchr(t,U_FILE_ALT_SEP_CHAR);) {
-        *t = U_TREE_ENTRY_SEP_CHAR;
-    }
-#endif
-}
 /*
  * Hey, Emacs, please set the following:
  *
