@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2011, International Business Machines
+*   Copyright (C) 1999-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -17,7 +17,6 @@
 
 #include "unicode/ucnv.h"
 #include "unicode/ustring.h"
-#include "unicode/putil.h"
 
 #include "uresimp.h"
 #include "cmemory.h"
@@ -30,20 +29,10 @@
 #include <stdio.h>
 #include <ctype.h>
 
-#if defined(U_WINDOWS) || defined(U_CYGWIN) || defined(U_MINGW)
+#ifdef WIN32
+#include <direct.h>
 #include <io.h>
 #include <fcntl.h>
-#define USE_FILENO_BINARY_MODE 1
-/* Windows likes to rename Unix-like functions */
-#ifndef fileno
-#define fileno _fileno
-#endif
-#ifndef setmode
-#define setmode _setmode
-#endif
-#ifndef O_BINARY
-#define O_BINARY _O_BINARY
-#endif
 #endif
 
 #define DERB_VERSION "1.0"
@@ -56,6 +45,8 @@ static const int32_t indentsize = 4;
 static int32_t truncsize = DERB_DEFAULT_TRUNC;
 static UBool trunc = FALSE;
 
+static const UChar baderror[] = { 0x0042, 0x0041, 0x0044, 0x0000 };
+
 static const char *getEncodingName(const char *encoding);
 static void reportError(const char *pname, UErrorCode *status, const char *when);
 static UChar *quotedString(const UChar *string);
@@ -103,9 +94,9 @@ main(int argc, char* argv[]) {
 
     /* Get the name of tool. */
     pname = uprv_strrchr(*argv, U_FILE_SEP_CHAR);
-#if U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR
+#ifdef WIN32
     if (!pname) {
-        pname = uprv_strrchr(*argv, U_FILE_ALT_SEP_CHAR);
+        pname = uprv_strrchr(*argv, '/');
     }
 #endif
     if (!pname) {
@@ -217,47 +208,36 @@ main(int argc, char* argv[]) {
         }
 
         p = uprv_strrchr(arg, U_FILE_SEP_CHAR);
-#if U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR
-        if (p == NULL) {
-            p = uprv_strrchr(arg, U_FILE_ALT_SEP_CHAR);
-        }
-#endif
         if (!p) {
             p = arg;
         } else {
-            p++;
+          p++;
         }
         q = uprv_strrchr(p, '.');
         if (!q) {
-            for (q = p; *q; ++q)
-                ;
+            for (q = p; *q; ++q);
         }
         uprv_strncpy(locale, p, q - p);
         locale[q - p] = 0;
 
         if (!(fromICUData = !uprv_strcmp(inputDir, "-"))) {
             UBool absfilename = *arg == U_FILE_SEP_CHAR;
-#ifdef U_WINDOWS
+#ifdef WIN32
             if (!absfilename) {
                 absfilename = (uprv_strlen(arg) > 2 && isalpha(arg[0])
-                    && arg[1] == ':' && arg[2] == U_FILE_SEP_CHAR);
+                  && arg[1] == ':' && arg[2] == U_FILE_SEP_CHAR);
             }
 #endif
             if (absfilename) {
                 thename = arg;
             } else {
-                q = uprv_strrchr(arg, U_FILE_SEP_CHAR);
-#if U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR
-                if (q == NULL) {
-                    q = uprv_strrchr(arg, U_FILE_ALT_SEP_CHAR);
-                }
-#endif
-                uprv_strcpy(infile, inputDir);
-                if(q != NULL) {
-                    uprv_strcat(infile, U_FILE_SEP_STRING);
-                    strncat(infile, arg, q-arg);
-                }
-                thename = infile;
+              q = uprv_strrchr(arg, U_FILE_SEP_CHAR);
+              uprv_strcpy(infile, inputDir);
+              if(q != NULL) {
+                uprv_strcat(infile, U_FILE_SEP_STRING),
+                strncat(infile, arg, q-arg);
+              }
+              thename = infile;
             }
         }
         status = U_ZERO_ERROR;
@@ -272,12 +252,12 @@ main(int argc, char* argv[]) {
             const char *filename = 0;
             const char *ext = 0;
 
-            if (!locale[0] || !tostdout) {
+            if (!locale || !tostdout) {
                 filename = uprv_strrchr(arg, U_FILE_SEP_CHAR);
 
-#if U_FILE_SEP_CHAR != U_FILE_ALT_SEP_CHAR
+#ifdef WIN32
                 if (!filename) {
-                    filename = uprv_strrchr(arg, U_FILE_ALT_SEP_CHAR);
+                    filename = uprv_strrchr(arg, '/');
                 }
 #endif
                 if (!filename) {
@@ -293,9 +273,9 @@ main(int argc, char* argv[]) {
 
             if (tostdout) {
                 out = stdout;
-#if defined(U_WINDOWS) || defined(U_CYGWIN) || defined(U_MINGW)
-                if (setmode(fileno(out), O_BINARY) == -1) {
-                    fprintf(stderr, "%s: couldn't set standard output to binary mode\n", pname);
+#ifdef WIN32
+                if (_setmode(_fileno(out), _O_BINARY) == -1) {
+                    fprintf(stderr, "%s: couldn't set standard output to binary mode\n, pname");
                     return 4;
                 }
 #endif
@@ -345,7 +325,7 @@ main(int argc, char* argv[]) {
 
             printCString(out, converter, "\n// derb(8) by Vladimir Weinstein and Yves Arrouye\n\n", -1);
 
-            if (locale[0]) {
+            if (locale) {
                 printCString(out, converter, locale, -1);
             } else {
                 printCString(out, converter, filename, (int32_t)(ext - filename));
@@ -356,8 +336,7 @@ main(int argc, char* argv[]) {
             if (out != stdout) {
                 fclose(out);
             }
-        }
-        else {
+        } else {
             reportError(pname, &status, "opening resource file");
         }
 
@@ -467,11 +446,58 @@ static void printHex(FILE *out, UConverter *converter, uint8_t what) {
 
     printString(out, converter, hex, (int32_t)(sizeof(hex)/sizeof(*hex)));
 }
+static const UChar *
+derb_getString(const ResourceData *pResData, const Resource res, int32_t *pLength) {
+    if(res!=RES_BOGUS) {
+        int32_t *p=(int32_t *)RES_GET_POINTER(pResData->pRoot, res);
+        if (pLength) {
+            *pLength=*p;
+        }
+        return (UChar *)++p;
+    } else {
+        if (pLength) {
+            *pLength=0;
+        }
+        return NULL;
+    }
+}
+
+static const char *
+derb_getTableKey(const Resource *pRoot, const Resource res, uint16_t indexS) {
+    uint16_t *p=(uint16_t *)RES_GET_POINTER(pRoot, res);
+    if(indexS<*p) {
+        return ((const char *)(pRoot)+(p[indexS+1])); /*RES_GET_KEY(pRoot, p[indexS+1]);*/
+    } else {
+        return NULL;    /* indexS>itemCount */
+    }
+}
+
+static Resource
+derb_getArrayItem(Resource *pRoot, Resource res, int32_t indexR) {
+    int32_t *p=(int32_t *)RES_GET_POINTER(pRoot, res);
+    if(indexR<*p) {
+        return ((Resource *)(p))[1+indexR];
+    } else {
+        return RES_BOGUS;   /* indexR>itemCount */
+    }
+}
+
+static Resource
+derb_getTableItem(const Resource *pRoot, const Resource res, uint16_t indexR) {
+    uint16_t *p=(uint16_t *)RES_GET_POINTER(pRoot, res);
+    uint16_t count=*p;
+    if(indexR<count) {
+        return ((Resource *)(p+1+count+(~count&1)))[indexR];
+    } else {
+        return RES_BOGUS;   /* indexR>itemCount */
+    }
+}
+
 
 static void printOutAlias(FILE *out,  UConverter *converter, UResourceBundle *parent, Resource r, const char *key, int32_t indent, const char *pname, UErrorCode *status) {
     static const UChar cr[] = { '\n' };
     int32_t len = 0;
-    const UChar* thestr = res_getAlias(&(parent->fResData), r, &len);
+    const UChar* thestr = derb_getString(&(parent->fResData), r, &len);
     UChar *string = quotedString(thestr);
     if(trunc && len > truncsize) {
         char msg[128];
@@ -482,15 +508,15 @@ static void printOutAlias(FILE *out,  UConverter *converter, UResourceBundle *pa
         len = truncsize;
     }
     if(U_SUCCESS(*status)) {
-        static const UChar openStr[] = { 0x003A, 0x0061, 0x006C, 0x0069, 0x0061, 0x0073, 0x0020, 0x007B, 0x0020, 0x0022 }; /* ":alias { \"" */
-        static const UChar closeStr[] = { 0x0022, 0x0020, 0x007D, 0x0020 }; /* "\" } " */
+        static const UChar open[] = { 0x003A, 0x0061, 0x006C, 0x0069, 0x0061, 0x0073, 0x0020, 0x007B, 0x0020, 0x0022 }; /* ":alias { \"" */
+        static const UChar close[] = { 0x0022, 0x0020, 0x007D, 0x0020 }; /* "\" } " */
         printIndent(out, converter, indent);
         if(key != NULL) {
             printCString(out, converter, key, -1);
         }
-        printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+        printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
         printString(out, converter, string, len);
-        printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+        printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
         if(verbose) {
             printCString(out, converter, " // ALIAS", -1);
         }
@@ -498,7 +524,6 @@ static void printOutAlias(FILE *out,  UConverter *converter, UResourceBundle *pa
     } else {
         reportError(pname, status, "getting binary value");
     }
-    uprv_free(string);
 }
 
 static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *resource, int32_t indent, const char *pname, UErrorCode *status)
@@ -510,11 +535,11 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
     const char *key = ures_getKey(resource);
 
     switch(ures_getType(resource)) {
-    case URES_STRING :
+    case RES_STRING :
         {
             int32_t len=0;
             const UChar* thestr = ures_getString(resource, &len, status);
-            UChar *string = quotedString(thestr);
+                        UChar *string = quotedString(thestr);
 
             /* TODO: String truncation */
             if(trunc && len > truncsize) {
@@ -527,19 +552,19 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             }
             printIndent(out, converter, indent);
             if(key != NULL) {
-                static const UChar openStr[] = { 0x0020, 0x007B, 0x0020, 0x0022 }; /* " { \"" */
-                static const UChar closeStr[] = { 0x0022, 0x0020, 0x007D }; /* "\" }" */
+                static const UChar open[] = { 0x0020, 0x007B, 0x0020, 0x0022 }; /* " { \"" */
+                static const UChar close[] = { 0x0022, 0x0020, 0x007D }; /* "\" }" */
                 printCString(out, converter, key, (int32_t)uprv_strlen(key));
-                printString(out, converter, openStr, (int32_t)(sizeof(openStr)/sizeof(*openStr)));
+                printString(out, converter, open, (int32_t)(sizeof(open)/sizeof(*open)));
                 printString(out, converter, string, len);
-                printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+                printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
             } else {
-                static const UChar openStr[] = { 0x0022 }; /* "\"" */
-                static const UChar closeStr[] = { 0x0022, 0x002C }; /* "\"," */
+                static const UChar open[] = { 0x0022 }; /* "\"" */
+                static const UChar close[] = { 0x0022, 0x002C }; /* "\"," */
 
-                printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+                printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
                 printString(out, converter, string, (int32_t)(u_strlen(string)));
-                printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+                printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
             }
 
             if(verbose) {
@@ -551,20 +576,20 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
         }
         break;
 
-    case URES_INT :
+    case RES_INT :
         {
-            static const UChar openStr[] = { 0x003A, 0x0069, 0x006E, 0x0074, 0x0020, 0x007B, 0x0020 }; /* ":int { " */
-            static const UChar closeStr[] = { 0x0020, 0x007D }; /* " }" */
+            static const UChar open[] = { 0x003A, 0x0069, 0x006E, 0x0074, 0x0020, 0x007B, 0x0020 }; /* ":int { " */
+            static const UChar close[] = { 0x0020, 0x007D }; /* " }" */
             UChar num[20];
 
             printIndent(out, converter, indent);
             if(key != NULL) {
                 printCString(out, converter, key, -1);
             }
-            printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+            printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
             uprv_itou(num, 20, ures_getInt(resource, status), 10, 0);
             printString(out, converter, num, u_strlen(num));
-            printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+            printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
 
             if(verbose) {
                 printCString(out, converter, "// INT", -1);
@@ -572,7 +597,7 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             printString(out, converter, cr, (int32_t)(sizeof(cr) / sizeof(*cr)));
             break;
         }
-    case URES_BINARY :
+    case RES_BINARY :
         {
             int32_t len = 0;
             const int8_t *data = (const int8_t *)ures_getBinary(resource, &len, status);
@@ -585,17 +610,17 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
                 len = truncsize;
             }
             if(U_SUCCESS(*status)) {
-                static const UChar openStr[] = { 0x003A, 0x0062, 0x0069, 0x006E, 0x0061, 0x0072, 0x0079, 0x0020, 0x007B, 0x0020 }; /* ":binary { " */
-                static const UChar closeStr[] = { 0x0020, 0x007D, 0x0020 }; /* " } " */
+                static const UChar open[] = { 0x003A, 0x0062, 0x0069, 0x006E, 0x0061, 0x0072, 0x0079, 0x0020, 0x007B, 0x0020 }; /* ":binary { " */
+                static const UChar close[] = { 0x0020, 0x007D, 0x0020 }; /* " } " */
                 printIndent(out, converter, indent);
                 if(key != NULL) {
                     printCString(out, converter, key, -1);
                 }
-                printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+                printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
                 for(i = 0; i<len; i++) {
                     printHex(out, converter, *data++);
                 }
-                printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+                printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
                 if(verbose) {
                     printCString(out, converter, " // BINARY", -1);
                 }
@@ -605,20 +630,20 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             }
         }
         break;
-    case URES_INT_VECTOR :
+    case RES_INT_VECTOR :
         {
             int32_t len = 0;
             const int32_t *data = ures_getIntVector(resource, &len, status);
             if(U_SUCCESS(*status)) {
-                static const UChar openStr[] = { 0x003A, 0x0069, 0x006E, 0x0074, 0x0076, 0x0065, 0x0063, 0x0074, 0x006F, 0x0072, 0x0020, 0x007B, 0x0020 }; /* ":intvector { " */
-                static const UChar closeStr[] = { 0x0020, 0x007D, 0x0020 }; /* " } " */
+                static const UChar open[] = { 0x003A, 0x0069, 0x006E, 0x0074, 0x0076, 0x0065, 0x0063, 0x0074, 0x006F, 0x0072, 0x0020, 0x007B, 0x0020 }; /* ":intvector { " */
+                static const UChar close[] = { 0x0020, 0x007D, 0x0020 }; /* " } " */
                 UChar num[20];
 
                 printIndent(out, converter, indent);
                 if(key != NULL) {
                     printCString(out, converter, key, -1);
                 }
-                printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+                printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
                 for(i = 0; i < len - 1; i++) {
                     int32_t numLen =  uprv_itou(num, 20, data[i], 10, 0);
                     num[numLen++] = 0x002C; /* ',' */
@@ -630,7 +655,7 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
                     uprv_itou(num, 20, data[len - 1], 10, 0);
                     printString(out, converter, num, u_strlen(num));
                 }
-                printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+                printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
                 if(verbose) {
                     printCString(out, converter, "// INTVECTOR", -1);
                 }
@@ -640,11 +665,11 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             }
       }
       break;
-    case URES_TABLE :
-    case URES_ARRAY :
+    case RES_TABLE :
+    case RES_ARRAY :
         {
-            static const UChar openStr[] = { 0x007B }; /* "{" */
-            static const UChar closeStr[] = { 0x007D, '\n' }; /* "}\n" */
+            static const UChar open[] = { 0x007B }; /* "{" */
+            static const UChar close[] = { 0x007D, '\n' }; /* "}\n" */
 
             UResourceBundle *t = NULL;
             ures_resetIterator(resource);
@@ -652,9 +677,9 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             if(key != NULL) {
                 printCString(out, converter, key, -1);
             }
-            printString(out, converter, openStr, (int32_t)(sizeof(openStr) / sizeof(*openStr)));
+            printString(out, converter, open, (int32_t)(sizeof(open) / sizeof(*open)));
             if(verbose) {
-                if(ures_getType(resource) == URES_TABLE) {
+                if(ures_getType(resource) == RES_TABLE) {
                     printCString(out, converter, "// TABLE", -1);
                 } else {
                     printCString(out, converter, "// ARRAY", -1);
@@ -673,18 +698,17 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
                   }
               }
             } else { /* we have to use low level access to do this */
-              Resource r;
-              int32_t resSize = ures_getSize(resource);
-              UBool isTable = (UBool)(ures_getType(resource) == URES_TABLE);
-              for(i = 0; i < resSize; i++) {
+              Resource r = RES_BOGUS;
+              for(i = 0; i < ures_getSize(resource); i++) {
                 /* need to know if it's an alias */
-                if(isTable) {
-                  r = res_getTableItemByIndex(&resource->fResData, resource->fRes, i, &key);
+                if(ures_getType(resource) == RES_TABLE) {
+                  r = derb_getTableItem(resource->fResData.pRoot, resource->fRes, (int16_t)i);
+                  key = derb_getTableKey(resource->fResData.pRoot, resource->fRes, (int16_t)i);
                 } else {
-                  r = res_getArrayItem(&resource->fResData, resource->fRes, i);
+                  r = derb_getArrayItem(resource->fResData.pRoot, resource->fRes, i);
                 }
                 if(U_SUCCESS(*status)) {
-                  if(res_getPublicType(r) == URES_ALIAS) {
+                  if(RES_GET_TYPE(r) == RES_ALIAS) {
                     printOutAlias(out, converter, resource, r, key, indent+indentsize, pname, status);
                   } else {
                     t = ures_getByIndex(resource, i, t, status);
@@ -698,7 +722,7 @@ static void printOutBundle(FILE *out, UConverter *converter, UResourceBundle *re
             }
 
             printIndent(out, converter, indent);
-            printString(out, converter, closeStr, (int32_t)(sizeof(closeStr) / sizeof(*closeStr)));
+            printString(out, converter, close, (int32_t)(sizeof(close) / sizeof(*close)));
             ures_close(t);
         }
         break;
