@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2000-2007, International Business Machines
+*   Copyright (C) 2000-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -23,8 +23,8 @@
 *   05/09/2001  weiv                    Case bits are now in the CEs, not in front
 */
 
+#include <stdio.h>
 #include "unicode/utypes.h"
-#include "unicode/putil.h"
 #include "unicode/udata.h"
 #include "unicode/uclean.h"
 #include "ucol_imp.h"
@@ -35,8 +35,6 @@
 #include "cstring.h"
 #include "cmemory.h"
 
-#include <stdio.h>
-
 /*
  * Global - verbosity
  */
@@ -63,42 +61,6 @@ static UDataInfo dummyDataInfo = {
 
 #else
 
-static const UDataInfo ucaDataInfo={
-    sizeof(UDataInfo),
-    0,
-
-    U_IS_BIG_ENDIAN,
-    U_CHARSET_FAMILY,
-    sizeof(UChar),
-    0,
-
-    {UCA_DATA_FORMAT_0, UCA_DATA_FORMAT_1, UCA_DATA_FORMAT_2, UCA_DATA_FORMAT_3},     /* dataFormat="UCol"            */
-    /* 03/26/2002 bumped up version since format has changed */
-    /* 09/16/2002 bumped up version since we went from UColAttributeValue */
-    /*            to int32_t in UColOptionSet */
-    /* 05/13/2003 This one also updated since we added UCA and UCD versions */
-    /*            to header */
-    /* 09/11/2003 Adding information required by data swapper */
-    {UCA_FORMAT_VERSION_0, UCA_FORMAT_VERSION_1, UCA_FORMAT_VERSION_2, UCA_FORMAT_VERSION_3},                 /* formatVersion                */
-    {0, 0, 0, 0}                  /* dataVersion = Unicode Version*/
-};
-
-static const UDataInfo invUcaDataInfo={
-    sizeof(UDataInfo),
-    0,
-
-    U_IS_BIG_ENDIAN,
-    U_CHARSET_FAMILY,
-    sizeof(UChar),
-    0,
-
-    {INVUCA_DATA_FORMAT_0, INVUCA_DATA_FORMAT_1, INVUCA_DATA_FORMAT_2, INVUCA_DATA_FORMAT_3},     /* dataFormat="InvC"            */
-    /* 03/26/2002 bumped up version since format has changed */
-    /* 04/29/2003 2.1 format - we have added UCA version to header */
-    {INVUCA_FORMAT_VERSION_0, INVUCA_FORMAT_VERSION_1, INVUCA_FORMAT_VERSION_2, INVUCA_FORMAT_VERSION_3},                 /* formatVersion                */
-    {0, 0, 0, 0}                  /* dataVersion = Unicode Version*/
-};
-
 UCAElements le;
 
 int32_t readElement(char **from, char *to, char separator, UErrorCode *status) {
@@ -317,7 +279,7 @@ static uint32_t addToInverse(UCAElements *element, UErrorCode *status) {
     addNewInverse(element, status);
   } else if(compareCEs(inverseTable[inversePos], element->CEs) > 0) {
     while((compResult = compareCEs(inverseTable[--position], element->CEs)) > 0);
-    if(VERBOSE) { fprintf(stdout, "p:%u ", (int)position); }
+    if(VERBOSE) { fprintf(stdout, "p:%i ", position); }
     if(compResult == 0) {
       addToExistingInverse(element, position, status);
     } else {
@@ -354,9 +316,9 @@ static InverseUCATableHeader *assembleInverseTable(UErrorCode *status)
 
     for(i = 2; i<inversePos; i++) {
       if(compareCEs(inverseTable[i-1], inverseTable[i]) > 0) { 
-        fprintf(stderr, "Error at %i: %08X & %08X\n", (int)i, (int)inverseTable[i-1][0], (int)inverseTable[i][0]);
+        fprintf(stderr, "Error at %i: %08X & %08X\n", i, inverseTable[i-1][0], inverseTable[i][0]);
       } else if(inverseTable[i-1][0] == inverseTable[i][0] && !(inverseTable[i-1][1] < inverseTable[i][1])) {
-        fprintf(stderr, "Continuation error at %i: %08X %08X & %08X %08X\n", (int)i, (int)inverseTable[i-1][0], (int)inverseTable[i-1][1], (int)inverseTable[i][0], (int)inverseTable[i][1]);
+        fprintf(stderr, "Continuation error at %i: %08X %08X & %08X %08X\n", i, inverseTable[i-1][0], inverseTable[i-1][1], inverseTable[i][0], inverseTable[i][1]);
       }
     }
 
@@ -391,18 +353,18 @@ static void writeOutInverseData(InverseUCATableHeader *data,
     uprv_memcpy(&invUcaInfo, &invUcaDataInfo, sizeof(UDataInfo));
     u_getUnicodeVersion(invUcaInfo.dataVersion);
 
-    pData=udata_create(outputDir, INVC_DATA_TYPE, INVC_DATA_NAME, &invUcaInfo,
+    pData=udata_create(outputDir, INVC_DATA_TYPE, U_ICUDATA_NAME "_" INVC_DATA_NAME, &invUcaInfo,
                        copyright, status);
 
     if(U_FAILURE(*status)) {
-        fprintf(stderr, "Error: unable to create %s"INVC_DATA_NAME", error %s\n", outputDir, u_errorName(*status));
+        fprintf(stderr, "Error: unable to create data memory, error %d\n", *status);
         return;
     }
 
     /* write the data to the file */
     if (VERBOSE) {
         fprintf(stdout, "Writing out inverse UCA table: %s%c%s.%s\n", outputDir, U_FILE_SEP_CHAR,
-                                                                INVC_DATA_NAME,
+                                                                U_ICUDATA_NAME "_" INVC_DATA_NAME,
                                                                 INVC_DATA_TYPE);
     }
     udata_writeBlock(pData, data, data->byteSize);
@@ -473,8 +435,8 @@ UCAElements *readAnElement(FILE *data, tempUCATable *t, UCAConstants *consts, UE
     // Directives.
     if(buffer[0] == '[') {
       uint32_t cnt = 0;
-      static const struct {
-        char name[128];
+      struct {
+        char name[256];
         uint32_t *what;
         ActionType what_to_do;
       } vt[]  = { {"[first tertiary ignorable",  consts->UCA_FIRST_TERTIARY_IGNORABLE,  READCE},
@@ -678,23 +640,17 @@ UCAElements *readAnElement(FILE *data, tempUCATable *t, UCAConstants *consts, UE
       i++;
     }
     element->noOfCEs = CEindex;
-#if 0
+
     element->isThai = UCOL_ISTHAIPREVOWEL(element->cPoints[0]);
-#endif
+
     // we don't want any strange stuff after useful data!
-    if (pointer == NULL) {
-        /* huh? Did we get ']' without the '['? Pair your brackets! */
-        *status=U_INVALID_FORMAT_ERROR;
-    }
-    else {
-        while(pointer < commentStart)  {
-            if(*pointer != ' ' && *pointer != '\t')
-            {
-                *status=U_INVALID_FORMAT_ERROR;
-                break;
-            }
-            pointer++;
+    while(pointer < commentStart)  {
+        if(*pointer != ' ' && *pointer != '\t')
+        {
+            *status=U_INVALID_FORMAT_ERROR;
+            break;
         }
+        pointer++;
     }
 
     if(U_FAILURE(*status)) {
@@ -744,11 +700,11 @@ void writeOutData(UCATableHeader *data,
     uprv_memcpy(&ucaInfo, &ucaDataInfo, sizeof(UDataInfo));
     u_getUnicodeVersion(ucaInfo.dataVersion);
 
-    pData=udata_create(outputDir, UCA_DATA_TYPE, UCA_DATA_NAME, &ucaInfo,
+    pData=udata_create(outputDir, UCA_DATA_TYPE, U_ICUDATA_NAME "_" UCA_DATA_NAME, &ucaInfo,
                        copyright, status);
 
     if(U_FAILURE(*status)) {
-        fprintf(stderr, "Error: unable to create %s"UCA_DATA_NAME", error %s\n", outputDir, u_errorName(*status));
+        fprintf(stderr, "Error: unable to create data memory, error %d\n", *status);
         return;
     }
 
@@ -784,35 +740,30 @@ write_uca_table(const char *filename,
                 UErrorCode *status)
 {
     FILE *data = fopen(filename, "r");
-    if(data == NULL) {
-        fprintf(stderr, "Couldn't open file: %s\n", filename);
-        return -1;
-    }
     uint32_t line = 0;
     UCAElements *element = NULL;
     UChar variableTopValue = 0;
     UCATableHeader *myD = (UCATableHeader *)uprv_malloc(sizeof(UCATableHeader));
     /* test for NULL */
-    if(myD == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        fclose(data);
-        return 0;
-    }
+	if(myD == NULL) {
+		*status = U_MEMORY_ALLOCATION_ERROR;
+		fclose(data);
+		return 0;
+	}
     uprv_memset(myD, 0, sizeof(UCATableHeader));
     UColOptionSet *opts = (UColOptionSet *)uprv_malloc(sizeof(UColOptionSet));
     /* test for NULL */
-    if(opts == NULL) {
-        *status = U_MEMORY_ALLOCATION_ERROR;
-        uprv_free(myD);
-        fclose(data);
-        return 0;
-    }
+	if(opts == NULL) {
+		*status = U_MEMORY_ALLOCATION_ERROR;
+		uprv_free(myD);
+		fclose(data);
+		return 0;
+	}
     uprv_memset(opts, 0, sizeof(UColOptionSet));
-    UChar contractionCEs[512][3];
-    uprv_memset(contractionCEs, 0, 512*3*sizeof(UChar));
+    UChar contractionCEs[256][3];
+    uprv_memset(contractionCEs, 0, 256*3*sizeof(UChar));
     uint32_t noOfContractions = 0;
     UCAConstants consts;
-    uprv_memset(&consts, 0, sizeof(consts));
 #if 0
     UCAConstants consts = {
       UCOL_RESET_TOP_VALUE,
@@ -846,6 +797,11 @@ write_uca_table(const char *filename,
 #endif
 
 
+    if(data == NULL) {
+        fprintf(stderr, "Couldn't open file: %s\n", filename);
+        return -1;
+    }
+
     uprv_memset(inverseTable, 0xDA, sizeof(int32_t)*3*0xFFFF);
 
     opts->variableTopValue = variableTopValue;
@@ -863,9 +819,6 @@ write_uca_table(const char *filename,
     if(U_FAILURE(*status))
     {
         fprintf(stderr, "Failed to init UCA temp table: %s\n", u_errorName(*status));
-        uprv_free(opts);
-        uprv_free(myD);
-        fclose(data);
         return -1;
     }
 
@@ -916,15 +869,15 @@ struct {
     int32_t surrogateCount = 0;
     while(!feof(data)) {
         if(U_FAILURE(*status)) {
-            fprintf(stderr, "Something returned an error %i (%s) while processing line %u of %s. Exiting...\n",
-                *status, u_errorName(*status), (int)line, filename);
+            fprintf(stderr, "Something returned an error %i (%s) while processing line %i of %s. Exiting...\n",
+                *status, u_errorName(*status), line, filename);
             exit(*status);
         }
 
         element = readAnElement(data, t, &consts, status);
         line++;
         if(VERBOSE) {
-          fprintf(stdout, "%u ", (int)line);
+          fprintf(stdout, "%i ", line);
         }
         if(element != NULL) {
             // we have read the line, now do something sensible with the read data!
@@ -960,25 +913,21 @@ struct {
     }
 
     if(UCAVersion[0] == 0 && UCAVersion[1] == 0 && UCAVersion[2] == 0 && UCAVersion[3] == 0) {
-        fprintf(stderr, "UCA version not specified. Cannot create data file!\n");
-        uprv_uca_closeTempTable(t);
-        uprv_free(opts);
-        uprv_free(myD);
-        fclose(data);
-        return -1;
+      fprintf(stderr, "UCA version not specified. Cannot create data file!\n");
+      return -1;
     }
 /*    {
         uint32_t trieWord = utrie_get32(t->mapping, 0xDC01, NULL);
     }*/
 
     if (VERBOSE) {
-        fprintf(stdout, "\nLines read: %u\n", (int)line);
-        fprintf(stdout, "Surrogate count: %i\n", (int)surrogateCount);
+        fprintf(stdout, "\nLines read: %i\n", line);
+        fprintf(stdout, "Surrogate count: %i\n", surrogateCount);
         fprintf(stdout, "Raw data breakdown:\n");
         /*fprintf(stdout, "Compact array stage1 top: %i, stage2 top: %i\n", t->mapping->stage1Top, t->mapping->stage2Top);*/
-        fprintf(stdout, "Number of contractions: %u\n", (int)noOfContractions);
-        fprintf(stdout, "Contraction image size: %u\n", (int)t->image->contractionSize);
-        fprintf(stdout, "Expansions size: %i\n", (int)t->expansions->position);
+        fprintf(stdout, "Number of contractions: %i\n", noOfContractions);
+        fprintf(stdout, "Contraction image size: %i\n", t->image->contractionSize);
+        fprintf(stdout, "Expansions size: %i\n", t->expansions->position);
     }
 
 
@@ -986,9 +935,9 @@ struct {
     /* first set up constants for implicit calculation */
     uprv_uca_initImplicitConstants(consts.UCA_PRIMARY_IMPLICIT_MIN, consts.UCA_PRIMARY_IMPLICIT_MAX, status);
     /* do the closure */
-    int32_t noOfClosures = uprv_uca_canonicalClosure(t, NULL, status);
+    int32_t noOfClosures = uprv_uca_canonicalClosure(t, status);
     if(noOfClosures != 0) {
-      fprintf(stderr, "Warning: %i canonical closures occured!\n", (int)noOfClosures);
+      fprintf(stderr, "Warning: %i canonical closures occured!\n", noOfClosures);
     }
 
     /* test */
@@ -997,17 +946,13 @@ struct {
     if (VERBOSE) {
         fprintf(stdout, "Compacted data breakdown:\n");
         /*fprintf(stdout, "Compact array stage1 top: %i, stage2 top: %i\n", t->mapping->stage1Top, t->mapping->stage2Top);*/
-        fprintf(stdout, "Number of contractions: %u\n", (int)noOfContractions);
-        fprintf(stdout, "Contraction image size: %u\n", (int)t->image->contractionSize);
-        fprintf(stdout, "Expansions size: %i\n", (int)t->expansions->position);
+        fprintf(stdout, "Number of contractions: %i\n", noOfContractions);
+        fprintf(stdout, "Contraction image size: %i\n", t->image->contractionSize);
+        fprintf(stdout, "Expansions size: %i\n", t->expansions->position);
     }
 
     if(U_FAILURE(*status)) {
         fprintf(stderr, "Error creating table: %s\n", u_errorName(*status));
-        uprv_uca_closeTempTable(t);
-        uprv_free(opts);
-        uprv_free(myD);
-        fclose(data);
         return -1;
     }
 
@@ -1029,7 +974,7 @@ struct {
     uprv_memcpy(inverse->UCAVersion, UCAVersion, sizeof(UVersionInfo));
     writeOutInverseData(inverse, outputDir, copyright, status);
 
-    uprv_uca_closeTempTable(t);
+    uprv_uca_closeTempTable(t);    
     uprv_free(myD);
     uprv_free(opts);
 
@@ -1096,14 +1041,15 @@ int main(int argc, char* argv[]) {
         return argc<0 ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
     if(options[3].doesOccur) {
-        fprintf(stdout, "genuca version %hu.%hu, ICU tool to read UCA text data and create UCA data tables for collation.\n",
+      fprintf(stdout, "genuca version %hu.%hu, ICU tool to read UCA text data and create UCA data tables for collation.\n",
 #if UCONFIG_NO_COLLATION
             0, 0
 #else
-            UCA_FORMAT_VERSION_0, UCA_FORMAT_VERSION_1
+            ucaDataInfo.formatVersion[0], ucaDataInfo.formatVersion[1]
 #endif
             );
-        fprintf(stdout, U_COPYRIGHT_STRING"\n");
+      fprintf(stdout, "Copyright (C) 2000-2001, International Business Machines\n");
+      fprintf(stdout, "Corporation and others.  All Rights Reserved.\n");
         exit(0);
     }
 
@@ -1157,16 +1103,16 @@ int main(int argc, char* argv[]) {
     UNewDataMemory *pData;
     const char *msg;
     
-    msg = "genuca writes dummy " UCA_DATA_NAME "." UCA_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
+    msg = "genuca writes dummy " U_ICUDATA_NAME "_" UCA_DATA_NAME "." UCA_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
     fprintf(stderr, "%s\n", msg);
-    pData = udata_create(destdir, UCA_DATA_TYPE, UCA_DATA_NAME, &dummyDataInfo,
+    pData = udata_create(destdir, UCA_DATA_TYPE, U_ICUDATA_NAME "_" UCA_DATA_NAME, &dummyDataInfo,
                          NULL, &status);
     udata_writeBlock(pData, msg, strlen(msg));
     udata_finish(pData, &status);
 
-    msg = "genuca writes dummy " INVC_DATA_NAME "." INVC_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
+    msg = "genuca writes dummy " U_ICUDATA_NAME "_" INVC_DATA_NAME "." INVC_DATA_TYPE " because of UCONFIG_NO_COLLATION, see uconfig.h";
     fprintf(stderr, "%s\n", msg);
-    pData = udata_create(destdir, INVC_DATA_TYPE, INVC_DATA_NAME, &dummyDataInfo,
+    pData = udata_create(destdir, INVC_DATA_TYPE, U_ICUDATA_NAME "_" INVC_DATA_NAME, &dummyDataInfo,
                          NULL, &status);
     udata_writeBlock(pData, msg, strlen(msg));
     udata_finish(pData, &status);
