@@ -1,6 +1,6 @@
 /******************************************************************************
 *
-*   Copyright (C) 2000-2002, International Business Machines
+*   Copyright (C) 2000-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -16,17 +16,29 @@
 *   (DLL, common data, etc.)
 */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include "unicode/utypes.h"
 #include "unicode/putil.h"
 #include "cmemory.h"
 #include "cstring.h"
 #include "filestrm.h"
 #include "toolutil.h"
+#include "unicode/uclean.h"
 #include "unewdata.h"
 #include "uoptions.h"
 
+#if U_HAVE_POPEN
+/*
+  We define __USE_POSIX2 so that we can get popen and pclose when
+  --enable-strict is used
+*/
+# ifndef __USE_POSIX2
+#  define __USE_POSIX2 1
+# endif
+# include <unistd.h>
+#endif
+#include <stdio.h>
+#include <stdlib.h>
+
 U_CDECL_BEGIN
 #include "pkgtypes.h"
 #include "makefile.h"
@@ -35,6 +47,9 @@ U_CDECL_END
 static int executeMakefile(const UPKGOptions *o);
 static void loadLists(UPKGOptions *o, UErrorCode *status);
 
+/* always have this fcn, just might not do anything */
+static void fillInMakefileFromICUConfig(UOption *option);
+
 /* This sets the modes that are available */
 static struct
 {
@@ -75,15 +90,17 @@ static UOption options[]={
     /*14*/    UOPTION_SOURCEDIR ,
     /*15*/    UOPTION_DEF( "entrypoint", 'e', UOPT_REQUIRES_ARG),
     /*16*/    UOPTION_DEF( "revision", 'r', UOPT_REQUIRES_ARG),
-    /*17*/    UOPTION_DEF( 0, 'M', UOPT_REQUIRES_ARG)
+    /*17*/    UOPTION_DEF( 0, 'M', UOPT_REQUIRES_ARG),
+    /*18*/    UOPTION_DEF( "force-prefix", 'f', UOPT_NO_ARG),
+    /*19*/    UOPTION_DEF( "numerictmp", 'N', UOPT_NO_ARG)
 };
 
 const char options_help[][160]={
     "Set the data name",
 #ifdef WIN32
-    "R:icupath for release version or D:icupath for debug version, where icupath is the directory where ICU is located",
+    "The directory where the ICU is located (e.g. <ICUROOT> which contains the bin directory)",
 #else
-    "Specify options for the builder",
+    "Specify options for the builder. (Autdetected if icu-config is available)",
 #endif
     "Specify the mode of building (see below; default: common)",
     "This usage text",
@@ -100,15 +117,18 @@ const char options_help[][160]={
     "Specify a custom source directory",
     "Specify a custom entrypoint name (default: short name)",
     "Specify a version when packaging in DLL or static mode",
-    "Pass the next argument to make(1)"
+    "Pass the next argument to make(1)",
+    "Add package to all file names if not present",
+    "Use short numeric temporary file names such as t1234.c"
 };
 
+const char  *progname = "PKGDATA";
+
 int
 main(int argc, char* argv[]) {
     FileStream  *out;
     UPKGOptions  o;
     CharList    *tail;
-    const char  *progname;
     UBool        needsHelp = FALSE;
     UErrorCode   status = U_ZERO_ERROR;
     char         tmp[1024];
@@ -140,8 +160,28 @@ main(int argc, char* argv[]) {
             fprintf(stderr, "Run '%s --help' for help.\n", progname);
             return 1;
         }
-        if(! (options[0].doesOccur && options[1].doesOccur) ) {
-            fprintf(stderr, " required parameters are missing: -p and -O are required \n");
+
+        if(!options[1].doesOccur) {
+            /* Try to fill in from icu-config or equivalent */
+            fillInMakefileFromICUConfig(&options[1]);
+        }
+#ifdef WIN32
+        else {
+            fprintf(stderr, "Warning: You are using the deprecated -O option\n"
+                            "\tYou can fix this warning by installing pkgdata, gencmn and genccode\n"
+                            "\tinto the same directory and not specifying the -O option to pkgdata.\n");
+        }
+#endif
+        
+        if(!options[1].doesOccur) {
+            fprintf(stderr, " required parameter is missing: -O is required \n");
+            fprintf(stderr, "Run '%s --help' for help.\n", progname);
+            return 1;
+        }
+        
+        if(!options[0].doesOccur) /* -O we already have - don't report it. */
+        {
+            fprintf(stderr, " required parameter -p is missing \n");
             fprintf(stderr, "Run '%s --help' for help.\n", progname);
             return 1;
         }
@@ -211,8 +251,8 @@ main(int argc, char* argv[]) {
     }
 
     o.shortName = options[0].value;
-    /**/ {
-        int len = uprv_strlen(o.shortName);
+    {
+        int32_t len = (int32_t)uprv_strlen(o.shortName);
         char *csname, *cp;
         const char *sp;
 
@@ -228,32 +268,36 @@ main(int argc, char* argv[]) {
         o.cShortName = csname;
     }
 
+    o.verbose   = options[5].doesOccur;
 #ifdef WIN32 /* format is R:pathtoICU or D:pathtoICU */
     {
         char *pathstuff = (char *)options[1].value;
         if(options[1].value[uprv_strlen(options[1].value)-1] == '\\') {
             pathstuff[uprv_strlen(options[1].value)-1] = '\0';
         }
-        if(*pathstuff == 'R' || *pathstuff == 'D') {
+        if(*pathstuff == PKGDATA_DERIVED_PATH || *pathstuff == 'R' || *pathstuff == 'D') {
             o.options = pathstuff;
             pathstuff++;
             if(*pathstuff == ':') {
                 *pathstuff = '\0';
                 pathstuff++;
-            } else {
-                fprintf(stderr, "Error: invalid windows build mode, should be R (release) or D (debug).\n", o.mode, progname);
+            }
+            else {
+                fprintf(stderr, "Error: invalid windows build mode, should be R (release) or D (debug).\n");
                 return 1;
             }
         } else {
-            fprintf(stderr, "Error: invalid windows build mode, should be R (release) or D (debug).\n", o.mode, progname);
+            fprintf(stderr, "Error: invalid windows build mode, should be R (release) or D (debug).\n");
             return 1;
         }
         o.icuroot = pathstuff;
+        if (o.verbose) {
+            fprintf(stdout, "# ICUROOT is %s\n", o.icuroot);
+        }
     }
 #else /* on UNIX, we'll just include the file... */
     o.options   = options[1].value;
 #endif
-    o.verbose   = options[5].doesOccur;
     if(options[6].doesOccur) {
         o.comment = U_COPYRIGHT_STRING;
     } else if (options[7].doesOccur) {
@@ -263,13 +307,19 @@ main(int argc, char* argv[]) {
     if( options[8].doesOccur ) {
         o.targetDir = options[8].value;
     } else {
-        o.targetDir = "";  /* cwd */
+        o.targetDir = ".";  /* cwd */
     }
 
     o.clean     = options[9].doesOccur;
     o.nooutput  = options[10].doesOccur;
     o.rebuild   = options[11].doesOccur;
 
+    o.numeric   = options[19].doesOccur;
+    if(o.numeric) { 
+      o.rebuild = TRUE; /* force rebuild if numeric */
+    }
+
+
     if( options[12].doesOccur ) {
         o.tmpDir    = options[12].value;
     } else {
@@ -413,27 +463,51 @@ static void loadLists(UPKGOptions *o, UErrorCode *status)
 {
     CharList   *l, *tail = NULL, *tail2 = NULL;
     FileStream *in;
-    char        line[2048];
-    char        tmp[1024];
-    const char* baseName;
+    char        line[16384];
+    char       *linePtr, *lineNext;
+    const uint32_t   lineMax = 16300;
+    char        tmp[1024], tmp2[1024];
+    char        pkgPrefix[1024];
+    int32_t     pkgPrefixLen;
+    const char *baseName;
     char       *s;
-
+    int32_t     ln;
+    UBool       fixPrefix;
+    
+    
+    fixPrefix = options[18].doesOccur;
+    
+    strcpy(pkgPrefix, o->shortName);
+    strcat(pkgPrefix, "_");
+    pkgPrefixLen=(int32_t)uprv_strlen(pkgPrefix);
     for(l = o->fileListFiles; l; l = l->next) {
         if(o->verbose) {
             fprintf(stdout, "# Reading %s..\n", l->str);
         }
         /* TODO: stdin */
         in = T_FileStream_open(l->str, "r");
-
+        
         if(!in) {
             fprintf(stderr, "Error opening <%s>.\n", l->str);
             *status = U_FILE_ACCESS_ERROR;
             return;
         }
-
+        
+        ln = 0;
+        
         while(T_FileStream_readLine(in, line, sizeof(line))!=NULL) {
+            ln++;
+            if(uprv_strlen(line)>lineMax) {
+                fprintf(stderr, "%s:%d - line too long (over %d chars)\n", l->str, ln, lineMax);
+                exit(1);
+            }
+            /* remove spaces at the beginning */
+            linePtr = line;
+            while(isspace(*linePtr)) {
+                linePtr++;
+            }
+            s=linePtr;
             /* remove trailing newline characters */
-            s=line;
             while(*s!=0) {
                 if(*s=='\r' || *s=='\n') {
                     *s=0;
@@ -441,29 +515,193 @@ static void loadLists(UPKGOptions *o, UErrorCode *status)
                 }
                 ++s;
             }
-            if((*line == 0) || (*line == '#')) {
+            if((*linePtr == 0) || (*linePtr == '#')) {
                 continue; /* comment or empty line */
             }
-
-            /* add the file */
-            s = (char*)getLongPathname(line);
-
-            baseName = findBasename(s);
-
-            o->files = pkg_appendToList(o->files, &tail, uprv_strdup(baseName));
-
-            if(s != baseName) { /* s was something long, so we leave it as it is */
-                o->filePaths = pkg_appendToList(o->filePaths, &tail2, uprv_strdup(s));
-            } else { /* s was just a basename, we want to prepend source dir*/
-                uprv_strcpy(tmp, o->srcDir);
-                uprv_strcat(tmp, o->srcDir[uprv_strlen(o->srcDir)-1]==U_FILE_SEP_CHAR?"":U_FILE_SEP_STRING);
-                uprv_strcat(tmp, s);
-                o->filePaths = pkg_appendToList(o->filePaths, &tail2, uprv_strdup(tmp));
+            
+            /* Now, process the line */
+            lineNext = NULL;
+            
+            while(linePtr && *linePtr) {
+                while(*linePtr == ' ') {
+                    linePtr++;
+                }
+                /* Find the next */
+                if(linePtr[0] == '"')
+                {
+                    lineNext = uprv_strchr(linePtr+1, '"');
+                    if(lineNext == NULL) {
+                        fprintf(stderr, "%s:%d - missing trailing double quote (\")\n",
+                            l->str, ln);
+                        exit(1);
+                    } else {
+                        lineNext++;
+                        if(*lineNext) {
+                            if(*lineNext != ' ') {
+                                fprintf(stderr, "%s:%d - malformed quoted line at position %d, expected ' ' got '%c'\n",
+                                    l->str, ln,  lineNext-line, (*lineNext)?*lineNext:'0');
+                                exit(1);
+                            }
+                            *lineNext = 0;
+                            lineNext++;
+                        }
+                    }
+                } else {
+                    lineNext = uprv_strchr(linePtr, ' ');
+                    if(lineNext) {
+                        *lineNext = 0; /* terminate at space */
+                        lineNext++;
+                    }
+                }
+                
+                /* add the file */
+                s = (char*)getLongPathname(linePtr);
+                
+                baseName = findBasename(s);
+                
+                if(s != baseName) {
+                    /* s was something 'long' with a path */
+                    if(fixPrefix && uprv_strncmp(pkgPrefix, baseName, pkgPrefixLen)) {
+                        /* path don't have the prefix, add package prefix to short and longname */
+                        uprv_strcpy(tmp, pkgPrefix);
+                        uprv_strcpy(tmp+pkgPrefixLen, baseName);
+                        
+                        uprv_strncpy(tmp2, s, uprv_strlen(s)-uprv_strlen(baseName));  /* should be:   dirpath only, ending in sep */
+                        tmp2[uprv_strlen(s)-uprv_strlen(baseName)]=0;
+                        uprv_strcat(tmp2, pkgPrefix);
+                        uprv_strcat(tmp2, baseName);
+                        
+                        o->files = pkg_appendToList(o->files, &tail, uprv_strdup(tmp));
+                        o->filePaths = pkg_appendToList(o->filePaths, &tail2, uprv_strdup(tmp2));
+                    } else {
+                        /* paths already have the prefix */
+                        o->files = pkg_appendToList(o->files, &tail, uprv_strdup(baseName));
+                        o->filePaths = pkg_appendToList(o->filePaths, &tail2, uprv_strdup(s));
+                    }
+                    
+                } else { /* s was just a basename, we want to prepend source dir*/
+                    /* check for prefix of package */
+                    uprv_strcpy(tmp, o->srcDir);
+                    uprv_strcat(tmp, o->srcDir[uprv_strlen(o->srcDir)-1]==U_FILE_SEP_CHAR?"":U_FILE_SEP_STRING);
+                    
+                    if(fixPrefix && strncmp(pkgPrefix,s, pkgPrefixLen)) {
+                        /* didn't have the prefix - add it */
+                        uprv_strcat(tmp, pkgPrefix);
+                        /* make up a new basename */
+                        uprv_strcpy(tmp2, pkgPrefix);
+                        uprv_strcat(tmp2, s);
+                        o->files = pkg_appendToList(o->files, &tail, uprv_strdup(tmp2));
+                    } else {
+                        o->files = pkg_appendToList(o->files, &tail, uprv_strdup(baseName));
+                    }
+                    uprv_strcat(tmp, s);
+                    o->filePaths = pkg_appendToList(o->filePaths, &tail2, uprv_strdup(tmp));
+                }
+                linePtr = lineNext;
             }
         }
-
         T_FileStream_close(in);
-    }
+  }
 }
 
+/* Try calling icu-config directly to get information */
+static void fillInMakefileFromICUConfig(UOption *option)
+{
+#if U_HAVE_POPEN
+    FILE *p;
+    size_t n;
+    static char buf[512] = "";
+    static const char cmd[] = "icu-config --incfile";
+    
+    if(options[5].doesOccur)
+    {
+        /* informational */
+        fprintf(stderr, "%s: No -O option found, trying '%s'.\n", progname, cmd);
+    }
+    
+    p = popen(cmd, "r");
+    
+    if(p == NULL)
+    {
+        fprintf(stderr, "%s: icu-config: No icu-config found. (fix PATH or use -O option)\n", progname);
+        return;
+    }
+    
+    n = fread(buf, 1, 511, p);
+    
+    pclose(p);
+    
+    if(n<=0)
+    {
+        fprintf(stderr,"%s: icu-config: Could not read from icu-config. (fix PATH or use -O option)\n", progname);
+        return;
+    }
+    
+    if(buf[strlen(buf)-1]=='\n')
+    {
+        buf[strlen(buf)-1]=0;
+    }
+    
+    if(buf[0] == 0)
+    {
+        fprintf(stderr, "%s: icu-config: invalid response from icu-config (fix PATH or use -O option)\n", progname);
+        return;
+    }
+    
+    if(options[5].doesOccur)
+    {
+        /* informational */
+        fprintf(stderr, "%s: icu-config: using '-O %s'\n", progname, buf);
+    }
+    option->value = buf;
+    option->doesOccur = TRUE;
+#else  /* ! U_HAVE_POPEN */
 
+#ifdef WIN32
+    char pathbuffer[_MAX_PATH] = {0};
+    char *fullEXEpath = NULL;
+    char *pathstuff = NULL;
+
+    if (strchr(progname, U_FILE_SEP_CHAR) != NULL || strchr(progname, U_FILE_ALT_SEP_CHAR) != NULL) {
+        /* pkgdata was executed with relative path */
+        fullEXEpath = _fullpath(pathbuffer, progname, sizeof(pathbuffer));
+        pathstuff = (char *)options[1].value;
+
+        if (fullEXEpath) {
+            pathstuff = strrchr(fullEXEpath, U_FILE_SEP_CHAR);
+            if (pathstuff) {
+                pathstuff[1] = 0;
+                uprv_memmove(fullEXEpath + 2, fullEXEpath, uprv_strlen(fullEXEpath)+1);
+                fullEXEpath[0] = PKGDATA_DERIVED_PATH;
+                fullEXEpath[1] = ':';
+                option->value = uprv_strdup(fullEXEpath);
+                option->doesOccur = TRUE;
+            }
+        }
+    }
+    else {
+        /* pkgdata was executed from the path */
+        /* Search for file in PATH environment variable: */
+        _searchenv("pkgdata.exe", "PATH", pathbuffer );
+        if( *pathbuffer != '\0' ) {
+            fullEXEpath = pathbuffer;
+            pathstuff = strrchr(pathbuffer, U_FILE_SEP_CHAR);
+            if (pathstuff) {
+                pathstuff[1] = 0;
+                uprv_memmove(fullEXEpath + 2, fullEXEpath, uprv_strlen(fullEXEpath)+1);
+                fullEXEpath[0] = PKGDATA_DERIVED_PATH;
+                fullEXEpath[1] = ':';
+                option->value = uprv_strdup(fullEXEpath);
+                option->doesOccur = TRUE;
+            }
+        }
+    }
+    /* else can't determine the path */
+#endif
+
+    /* no popen available */
+    /* Put other OS specific ways to search for the Makefile.inc type 
+       information or else fail.. */
+
+#endif
+}
