@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1998-2006, International Business Machines
+*   Copyright (C) 1998-2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -16,7 +16,6 @@
 */
 
 #include "unicode/utypes.h"
-#include "unicode/putil.h"
 #include "unicode/ucnv.h"
 #include "unicode/ucnv_err.h"
 #include "filestrm.h"
@@ -28,9 +27,6 @@
 #include "ucbuf.h"
 #include <stdio.h>
 
-#if !UCONFIG_NO_CONVERSION
-
-
 #define MAX_IN_BUF 1000
 #define MAX_U_BUF 1500
 #define CONTEXT_LEN 15
@@ -305,7 +301,6 @@ ucbuf_fillucbuf( UCHARBUF* buf,UErrorCode* error){
     }
     buf->currentPos = pTarget;
     buf->bufLimit=pTarget+outputWritten;
-    *buf->bufLimit=0; /*NUL terminate*/
     if(cbuf!=carr){
         uprv_free(cbuf);
     }
@@ -411,16 +406,16 @@ ucbuf_getcx32(UCHARBUF* buf,UErrorCode* error) {
      * to c32 or not
      */
     if(c32==0xFFFFFFFF){
-        if(buf->showWarning) {
-            char context[20];
-            int32_t len = 20;
-            if(length  < len) {
-                len = length; 
-            }
-            context[len]= 0 ; /* null terminate the buffer */
-            u_UCharsToChars( buf->currentPos, context, len);
-            fprintf(stderr,"Bad escape: [%c%s]...\n", (int)c1, context);
-        }
+		if(buf->showWarning) {
+			char context[20];
+			int32_t len = 20;
+			if(length  < len) {
+			  len = length; 
+			}
+			context[len]= 0 ; /* null terminate the buffer */
+			u_UCharsToChars( buf->currentPos, context, len);
+			fprintf(stderr,"Bad escape: [%c%s]...\n", c1,context);
+		}
         *error= U_ILLEGAL_ESCAPE_SEQUENCE;
         return c1;
     }else if(c32!=c2 || (c32==0x0075 && c2==0x0075 && c1==0x005C) /* for \u0075 c2=0x0075 and c32==0x0075*/){
@@ -461,61 +456,56 @@ ucbuf_open(const char* fileName,const char** cp,UBool showWarning, UBool buffere
     if(in!=NULL){
         UCHARBUF* buf =(UCHARBUF*) uprv_malloc(sizeof(UCHARBUF));
         fileSize = T_FileStream_size(in);
-        if(buf == NULL){
-            *error = U_MEMORY_ALLOCATION_ERROR;
-            T_FileStream_close(in);
-            return NULL;
-        }
-        buf->in=in;
-        buf->conv=NULL;
-        buf->showWarning = showWarning;
-        buf->isBuffered = buffered;
-        buf->signatureLength=0;
-        if(*cp==NULL || **cp=='\0'){
-            /* don't have code page name... try to autodetect */
-            ucbuf_autodetect_fs(in,cp,&buf->conv,&buf->signatureLength,error);
-        }else if(ucbuf_isCPKnown(*cp)){
-            /* discard BOM */
-            ucbuf_autodetect_fs(in,&knownCp,&buf->conv,&buf->signatureLength,error);
-        }
-        if(U_SUCCESS(*error) && buf->conv==NULL) {
-            buf->conv=ucnv_open(*cp,error);
-        }
-        if(U_FAILURE(*error)){
-            ucnv_close(buf->conv);
-            uprv_free(buf);
-            T_FileStream_close(in);
-            return NULL;
-        }
-        
-        if((buf->conv==NULL) && (buf->showWarning==TRUE)){
-            fprintf(stderr,"###WARNING: No converter defined. Using codepage of system.\n");
-        }
-        buf->remaining=fileSize-buf->signatureLength;
-        if(buf->isBuffered){
-            buf->bufCapacity=MAX_U_BUF;
+        if(buf){
+            buf->in=in;
+            buf->conv=NULL;
+            buf->showWarning = showWarning;
+            buf->isBuffered = buffered;
+            buf->signatureLength=0;
+            if(*cp==NULL || **cp=='\0'){
+                /* don't have code page name... try to autodetect */
+                ucbuf_autodetect_fs(in,cp,&buf->conv,&buf->signatureLength,error);
+            }else if(ucbuf_isCPKnown(*cp)){
+                /* discard BOM */
+                ucbuf_autodetect_fs(in,&knownCp,&buf->conv,&buf->signatureLength,error);
+            }
+            if(U_SUCCESS(*error) && buf->conv==NULL) {
+                buf->conv=ucnv_open(*cp,error);
+            }
+            if(U_FAILURE(*error)){
+                ucnv_close(buf->conv);
+                uprv_free(buf);
+                return NULL;
+            }
+            
+            if((buf->conv==NULL) && (buf->showWarning==TRUE)){
+                fprintf(stderr,"###WARNING: No converter defined. Using codepage of system.\n");
+            }
+            buf->remaining=fileSize-buf->signatureLength;
+            if(buf->isBuffered){
+                buf->buffer=(UChar*) uprv_malloc(U_SIZEOF_UCHAR* MAX_U_BUF);
+                buf->bufCapacity=MAX_U_BUF;
+            }else{
+                buf->buffer=(UChar*) uprv_malloc(U_SIZEOF_UCHAR * (buf->remaining+buf->signatureLength));
+                buf->bufCapacity=buf->remaining+buf->signatureLength;
+            }
+            if (buf->buffer == NULL) {
+                *error = U_MEMORY_ALLOCATION_ERROR;
+                return NULL;
+            }
+            buf->currentPos=buf->buffer;
+            buf->bufLimit=buf->buffer;
+            if(U_FAILURE(*error)){
+                fprintf(stderr, "Could not open codepage [%s]: %s\n", *cp, u_errorName(*error));
+                return NULL;
+            }
+            buf=ucbuf_fillucbuf(buf,error);
+            return buf;
         }else{
-            buf->bufCapacity=buf->remaining+buf->signatureLength+1/*for terminating nul*/;               
-        }
-        buf->buffer=(UChar*) uprv_malloc(U_SIZEOF_UCHAR * buf->bufCapacity );
-        if (buf->buffer == NULL) {
             *error = U_MEMORY_ALLOCATION_ERROR;
-            ucnv_close(buf->conv);
-            uprv_free(buf);
-            T_FileStream_close(in);
             return NULL;
         }
-        buf->currentPos=buf->buffer;
-        buf->bufLimit=buf->buffer;
-        if(U_FAILURE(*error)){
-            fprintf(stderr, "Could not open codepage [%s]: %s\n", *cp, u_errorName(*error));
-            ucnv_close(buf->conv);
-            uprv_free(buf);
-            T_FileStream_close(in);
-            return NULL;
-        }
-        buf=ucbuf_fillucbuf(buf,error);
-        return buf;
+
     }
     *error =U_FILE_ACCESS_ERROR;
     return NULL;
@@ -785,4 +775,3 @@ ucbuf_readline(UCHARBUF* buf,int32_t* len,UErrorCode* err){
     /* A compiler warning will appear if all paths don't contain a return statement. */
 /*    return NULL;*/
 }
-#endif
