@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2003-2010, International Business Machines
+*   Copyright (C) 2003, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -27,8 +27,6 @@
 #include "ucm.h"
 #include <stdio.h>
 
-#if !UCONFIG_NO_CONVERSION
-
 /* MBCS state handling ------------------------------------------------------ */
 
 /*
@@ -421,7 +419,7 @@ sumUpStates(UCMStates *states) {
 }
 
 U_CAPI void U_EXPORT2
-ucm_processStates(UCMStates *states, UBool ignoreSISOCheck) {
+ucm_processStates(UCMStates *states) {
     int32_t entry, state, cell, count;
 
     if(states->conversionType==UCNV_UNSUPPORTED_CONVERTER) {
@@ -531,16 +529,16 @@ ucm_processStates(UCMStates *states, UBool ignoreSISOCheck) {
             entry=states->stateTable[state][cell];
             if((uint8_t)MBCS_ENTRY_STATE(entry)>=states->countStates) {
                 fprintf(stderr, "ucm error: state table entry [%x][%x] has a next state of %x that is too high\n",
-                    (int)state, (int)cell, (int)MBCS_ENTRY_STATE(entry));
+                    state, cell, MBCS_ENTRY_STATE(entry));
                 exit(U_INVALID_TABLE_FORMAT);
             }
             if(MBCS_ENTRY_IS_FINAL(entry) && (states->stateFlags[MBCS_ENTRY_STATE(entry)]&0xf)!=MBCS_STATE_FLAG_DIRECT) {
                 fprintf(stderr, "ucm error: state table entry [%x][%x] is final but has a non-initial next state of %x\n",
-                    (int)state, (int)cell, (int)MBCS_ENTRY_STATE(entry));
+                    state, cell, MBCS_ENTRY_STATE(entry));
                 exit(U_INVALID_TABLE_FORMAT);
             } else if(MBCS_ENTRY_IS_TRANSITION(entry) && (states->stateFlags[MBCS_ENTRY_STATE(entry)]&0xf)==MBCS_STATE_FLAG_DIRECT) {
                 fprintf(stderr, "ucm error: state table entry [%x][%x] is not final but has an initial next state of %x\n",
-                    (int)state, (int)cell, (int)MBCS_ENTRY_STATE(entry));
+                    state, cell, MBCS_ENTRY_STATE(entry));
                 exit(U_INVALID_TABLE_FORMAT);
             }
         }
@@ -549,19 +547,18 @@ ucm_processStates(UCMStates *states, UBool ignoreSISOCheck) {
     /* is this an SI/SO (like EBCDIC-stateful) state table? */
     if(states->countStates>=2 && (states->stateFlags[1]&0xf)==MBCS_STATE_FLAG_DIRECT) {
         if(states->maxCharLength!=2) {
-            fprintf(stderr, "ucm error: SI/SO codepages must have max 2 bytes/char (not %x)\n", (int)states->maxCharLength);
+            fprintf(stderr, "ucm error: SI/SO codepages must have max 2 bytes/char (not %x)\n", states->maxCharLength);
             exit(U_INVALID_TABLE_FORMAT);
         }
         if(states->countStates<3) {
-            fprintf(stderr, "ucm error: SI/SO codepages must have at least 3 states (not %x)\n", (int)states->countStates);
+            fprintf(stderr, "ucm error: SI/SO codepages must have at least 3 states (not %x)\n", states->countStates);
             exit(U_INVALID_TABLE_FORMAT);
         }
         /* are the SI/SO all in the right places? */
-        if( ignoreSISOCheck ||
-           (states->stateTable[0][0xe]==MBCS_ENTRY_FINAL(1, MBCS_STATE_CHANGE_ONLY, 0) &&
+        if( states->stateTable[0][0xe]==MBCS_ENTRY_FINAL(1, MBCS_STATE_CHANGE_ONLY, 0) &&
             states->stateTable[0][0xf]==MBCS_ENTRY_FINAL(0, MBCS_STATE_CHANGE_ONLY, 0) &&
             states->stateTable[1][0xe]==MBCS_ENTRY_FINAL(1, MBCS_STATE_CHANGE_ONLY, 0) &&
-            states->stateTable[1][0xf]==MBCS_ENTRY_FINAL(0, MBCS_STATE_CHANGE_ONLY, 0))
+            states->stateTable[1][0xf]==MBCS_ENTRY_FINAL(0, MBCS_STATE_CHANGE_ONLY, 0)
         ) {
             states->outputType=MBCS_OUTPUT_2_SISO;
         } else {
@@ -576,7 +573,7 @@ ucm_processStates(UCMStates *states, UBool ignoreSISOCheck) {
     /* check that no unexpected state is a "direct" one */
     while(state<states->countStates) {
         if((states->stateFlags[state]&0xf)==MBCS_STATE_FLAG_DIRECT) {
-            fprintf(stderr, "ucm error: state %d is 'initial' - not supported except for SI/SO codepages\n", (int)state);
+            fprintf(stderr, "ucm error: state %d is 'initial' - not supported except for SI/SO codepages\n", state);
             exit(U_INVALID_TABLE_FORMAT);
         }
         ++state;
@@ -1037,11 +1034,9 @@ ucm_countChars(UCMStates *states,
      * must consist of only double-byte sequences
      */
     if(count>1 && states->outputType==MBCS_OUTPUT_2_SISO && length!=2*count) {
-        fprintf(stderr, "ucm error: SI/SO (like EBCDIC-stateful) result with %d characters does not contain all DBCS\n", (int)count);
+        fprintf(stderr, "ucm error: SI/SO (like EBCDIC-stateful) result with %d characters does not contain all DBCS\n", count);
         return -1;
     }
 
     return count;
 }
-#endif
-
